/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* $Id: rudolph.c 4290 2006-01-17 10:26:31Z jbader $ */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/time.h>
#include <sys/kd.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>

#include "ts-mem.h"
#include "realms.h"
#include "rudolph.h"
#include "canvas.h"
#include "Event.h"
#include "pixel.h"

static int fbfd = -1;
r_screen screen;
static char *fbp = 0;
static char *fb_page = 0;
static struct fb_var_screeninfo vinfo;
static struct fb_fix_screeninfo finfo;
static int fb_cnt = 0;
static int fast_blit = 0;
static long int screensize = 0;
static long int real_screensize = 0;
extern char* awt_args;
extern char* awt_splash;
int awt_zoom;

#ifdef HQ2X
void InitLUTs(void);
void hq2x_32(unsigned char * pIn, unsigned char * pOut, int Xres, int Yres, int BpL);
#endif

void show_splash(void) {
  int fd, n = 0;
  if(awt_splash && (fd = open(awt_splash, O_RDONLY)) > 0)
    while((n = read(fd, fbp, 1000000)) > 0);
  memcpy(screen->video, fbp, screensize);
}

#ifdef AWT_PIXELFORMAT_c332
/*
 * Set up our 8-bit palette.
 */
#define PALETTE_SIZE 256

static struct fb_cmap *oldCmap = NULL;

static int screen_set_palette(int fd) {
  struct fb_cmap cmap;
  int ret;
  unsigned int i;

  if (oldCmap == NULL) {
    // Save original palette setting.
    oldCmap = allocMem_system(sizeof(*oldCmap)
			      + 4 * sizeof(*(oldCmap->red)) * PALETTE_SIZE);
    oldCmap->start = 0;
    oldCmap->len = PALETTE_SIZE;

    // Our palette buffer starts after the end of the fb_cmap struct.
    oldCmap->red = (void *) (oldCmap + 1);
    oldCmap->green = oldCmap->red + PALETTE_SIZE;
    oldCmap->blue = oldCmap->green + PALETTE_SIZE;
    oldCmap->transp = oldCmap->blue + PALETTE_SIZE;

    if (ioctl(fd, FBIOGETCMAP, oldCmap) < 0) {
      perror("Failed to save system palette: FBIOGETCMAP");
      releaseMem(oldCmap);
      oldCmap = NULL;
    }
  }

  cmap.start = 0;
  cmap.len = PALETTE_SIZE;
  cmap.red = allocMem_system(3 * sizeof(*(cmap.red)) * PALETTE_SIZE);
  cmap.green = cmap.red + PALETTE_SIZE;
  cmap.blue = cmap.green + PALETTE_SIZE;
  cmap.transp = NULL;

  for (i = 0; i < cmap.len; i++) {
    cmap.red[i] = pixel2red(i) * 0xffff / 0x07;
    cmap.green[i] = pixel2green(i) * 0xffff / 0x07;
    cmap.blue[i] = pixel2blue(i) * 0xffff / 0x03;
  }

  ret = ioctl(fd, FBIOPUTCMAP, &cmap);

  if (ret < 0) {
    perror("Failed to set palette; colours will be weird: FBIOPUTCMAP");
  }

  releaseMem(cmap.red);
  return ret;
}

static void screen_restore_palette(int fd) {
  if (oldCmap) {
    if (ioctl(fd, FBIOPUTCMAP, oldCmap) < 0) {
      perror("Failed to restore system palette: FBIOPUTCMAP");
    }
  }
}
#endif


r_screen screen_init(void) {

  woempa(9, "called screen_init()\n");

  if(awt_args) {
    if(strcmp(awt_args, "zoom") == 0) awt_zoom = 1;
  }
  
  if (fbfd < 0) {

    /* 
    ** Open the file for reading and writing:
    */
    
    fbfd = open("/dev/fb0", O_RDWR);
    
    if (fbfd < 0) {
      wabort(ABORT_WONKA, "cannot open framebuffer device.\n");
    }
    
    woempa(9, "successfully opened framebuffer device ...\n");

    /*
    ** Get fixed screen information:
    */

    if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo)) {
      wabort(ABORT_WONKA, "error reading fixed information.\n");
    }
            
    /*
    ** Get variable screen information: 
    */
    
    if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo)) {
      wabort(ABORT_WONKA, "error reading variable information.\n");
    }                          
    
    /* 
    ** Figure out the size of the screen in bytes:
    */
    
    real_screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;

    if(finfo.line_length == (vinfo.xres * vinfo.bits_per_pixel / 8)) {
      fast_blit = 1;
    }
    
    /* 
    ** Map framebuffer device to memory:
    */
    
    fbp = (char *)mmap(0, finfo.smem_len, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);

    if ((int)fbp == -1) { 
      wabort(ABORT_WONKA, "failed to map framebuffer device to memory.\n");
    }

    if(finfo.ypanstep == 1) {
      fb_cnt = finfo.smem_len / real_screensize;
      if(fb_cnt < 2) {
        fb_cnt = 1;
      }
      fb_page = fbp + real_screensize;
    }
    else {
      fb_page = fbp;
    }

    ioctl(fbfd, KDSETMODE, KD_GRAPHICS);
    vinfo.xres_virtual = vinfo.xres;
    vinfo.yres_virtual = vinfo.yres;
	  ioctl(fbfd, FBIOPUT_VSCREENINFO, &vinfo);

    woempa(9, "succesfully mapped framebuffer device to memory ...\n");

    woempa(9, "--------------------------------------------------------\n");
    woempa(9, "framebuffer screen specifications:\n");
    woempa(9, "   x resolution: %3d, y resolution: %3d, bits per pixel: %3d\n", vinfo.xres, vinfo.yres, vinfo.bits_per_pixel);
    woempa(9, "   x offset    : %3d, y offset    : %3d, line length   : %3d\n", vinfo.xoffset, vinfo.yoffset, finfo.line_length);
    woempa(9, "   memory      : %p\n", fbp);
    woempa(9, "--------------------------------------------------------\n");

    /*
    ** Initialze screen structure:
    */

    if(awt_zoom) {
      screensize = real_screensize / 4;
#ifdef HQ2X
      InitLUTs();
#endif
    }
    else {
      screensize = real_screensize;
    }

    screen = allocMem_system(sizeof(r_Screen));

    if(swap_display) {
      screen->height = vinfo.xres;
      screen->width = vinfo.yres;
    } else {
      screen->height = vinfo.yres;
      screen->width = vinfo.xres;
    }

    if(awt_zoom) {
      screen->height /= 2;
      screen->width  /= 2;
    }

    screen->video = allocClearedMem_system(screensize);

    // Blank screen before setting palette to prevent flicker
    memcpy(fbp, screen->video, screensize);
    
#ifdef AWT_PIXELFORMAT_c332
    screen_set_palette(fbfd);
#endif

    show_splash();

    woempa(9, "finished screen_init()\n");
  }
  
  return screen;
}

void screen_shutdown(void) {
  if ((int) fbp > 0) {
    munmap(fbp, (unsigned)(vinfo.yres * finfo.line_length));
    fbp = NULL;
  }
  if (fbfd != -1) {
#ifdef AWT_PIXELFORMAT_c332
    if (oldCmap) {
      screen_restore_palette(fbfd);
      releaseMem(oldCmap);
      oldCmap = NULL;
    }
#endif
    close(fbfd);
    fbfd = -1;
  }
}

static void blitter(w_word * dst, w_word * src, w_size size) {

  w_int num_words = size / sizeof(w_word);
  w_int duffs = (num_words + 63) / 64;

  switch (num_words & 0x3f) {
    default:
    case  0: do { *dst++ = *src++;
    case 63:      *dst++ = *src++;
    case 62:      *dst++ = *src++;
    case 61:      *dst++ = *src++;
    case 60:      *dst++ = *src++;
    case 59:      *dst++ = *src++;
    case 58:      *dst++ = *src++;
    case 57:      *dst++ = *src++;
    case 56:      *dst++ = *src++;
    case 55:      *dst++ = *src++;
    case 54:      *dst++ = *src++;
    case 53:      *dst++ = *src++;
    case 52:      *dst++ = *src++;
    case 51:      *dst++ = *src++;
    case 50:      *dst++ = *src++;
    case 49:      *dst++ = *src++;
    case 48:      *dst++ = *src++;
    case 47:      *dst++ = *src++;
    case 46:      *dst++ = *src++;
    case 45:      *dst++ = *src++;
    case 44:      *dst++ = *src++;
    case 43:      *dst++ = *src++;
    case 42:      *dst++ = *src++;
    case 41:      *dst++ = *src++;
    case 40:      *dst++ = *src++;
    case 39:      *dst++ = *src++;
    case 38:      *dst++ = *src++;
    case 37:      *dst++ = *src++;
    case 36:      *dst++ = *src++;
    case 35:      *dst++ = *src++;
    case 34:      *dst++ = *src++;
    case 33:      *dst++ = *src++;
    case 32:      *dst++ = *src++;
    case 31:      *dst++ = *src++;
    case 30:      *dst++ = *src++;
    case 29:      *dst++ = *src++;
    case 28:      *dst++ = *src++;
    case 27:      *dst++ = *src++;
    case 26:      *dst++ = *src++;
    case 25:      *dst++ = *src++;
    case 24:      *dst++ = *src++;
    case 23:      *dst++ = *src++;
    case 22:      *dst++ = *src++;
    case 21:      *dst++ = *src++;
    case 20:      *dst++ = *src++;
    case 19:      *dst++ = *src++;
    case 18:      *dst++ = *src++;
    case 17:      *dst++ = *src++;
    case 16:      *dst++ = *src++;
    case 15:      *dst++ = *src++;
    case 14:      *dst++ = *src++;
    case 13:      *dst++ = *src++;
    case 12:      *dst++ = *src++;
    case 11:      *dst++ = *src++;
    case 10:      *dst++ = *src++;
    case  9:      *dst++ = *src++;
    case  8:      *dst++ = *src++;
    case  7:      *dst++ = *src++;
    case  6:      *dst++ = *src++;
    case  5:      *dst++ = *src++;
    case  4:      *dst++ = *src++;
    case  3:      *dst++ = *src++;
    case  2:      *dst++ = *src++;
    case  1:      *dst++ = *src++;
            } while (--duffs > 0);
  }

}

int awt_enabled = 1;

static inline void page_flip(void) {
  if(fb_cnt > 1) {
    vinfo.xoffset = 0;
    if(fbp == fb_page) {
      vinfo.yoffset = 0;
      fb_page = fbp + real_screensize;
    }
    else {
      vinfo.yoffset = vinfo.yres;
      fb_page = fbp;
    }
    ioctl(fbfd, FBIOPAN_DISPLAY, &vinfo);
  }
}

void screen_update(int x1, int y1, int x2, int y2) {

  w_ubyte       *fb = fb_page;
  w_int         x, y, size;
  w_ubyte       *scr;
  static w_word *buffer;
  w_ushort      *video = (w_ushort *)screen->video;

  if(awt_zoom) {
    if(!buffer) buffer = x_mem_alloc(screensize * 4 * 4);
  }
  else {
    buffer = (w_word *)screen->video;
  }

  if(awt_zoom) {
#ifdef HQ2X
    hq2x_32((unsigned char *)video, (unsigned char *)buffer, screen->width, screen->height, screen->width * 4 * 2);
   
    for(y = 0; y < screen->height * 2; y++) {
      for(x = 0; x < screen->width * 2; x++) {
        int pixel = buffer[y * screen->width * 2 + x];
        ((short *)buffer)[y * screen->width * 2 + x]  = rgb2pixel((pixel >> 16) & 0xff, (pixel >> 8) & 0xff, pixel & 0xff);
      }
    }
#else
    w_int offset = 0;
    w_int toggle = 0;
    for(y = 0; y < screen->height; y += toggle % 2) {
      w_int offset2 = screen->width * y;
      for(x = 0; x < screen->width; x++) {
        w_int pixel = video[offset2++];
        buffer[offset++] = (pixel) << 16 | pixel;
      }
      toggle++;
    }
#endif
  }
  
  if (fast_blit) {
    blitter((w_word*)fb, (w_word*)buffer, real_screensize);
  }
  else {
    size = vinfo.xres * vinfo.bits_per_pixel / 8;
    scr = (w_ubyte *)buffer;
    
    for (y = 0; y < (w_int)vinfo.yres; y++) {
      memcpy(fb, scr, size);
      fb += finfo.line_length;
      scr += size;
    }
  }

  page_flip();
}


