/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Dispatcher.java 4207 2005-01-11 08:04:23Z jbader $
*/

package com.acunia.wonka.rudolph;

import java.awt.Window;
import java.io.InputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;
import wonka.vm.Wonka;

public final class Dispatcher implements Runnable {

  static {
    String mouse_device = getMouseDevice();

    init(mouse_device);

    // Add a hook to restore keyboard settings, etc.
    Runtime.getRuntime().addShutdownHook(
      new Thread() {
        public void run() {
          shutdown();
        }
      });
  }

  /**
   * Set up the mouse or other pointing device.
   * The information describing the pointing device is found in file
   * 'device.config' in the system directory, in a line beginning
   * 'attach-mouse-device'. Then next two tokens on this line are
   * ignored, and the fourth token is used as a path to the pointing device.
   * <p>
   * If no such line is found in the 'device.config' file, system property
   * 'wonka.default.mouse.device' is used; if this is undefined then the 
   * path defaults to '/dev/ts'.
   */  
  private static String getMouseDevice() {
    InputStream s = ClassLoader.getSystemResourceAsStream("device.config");
    BufferedReader r = new BufferedReader(new InputStreamReader(s));
    String line;
    try {
      while ((line = r.readLine()) != null) {
        int start;
        int end;
        while ((start = line.indexOf("  ")) != -1) {
          line = line.substring(0, start) + line.substring(start + 1);
        }
        while ((start = line.indexOf("( ")) != -1) {
          end = line.indexOf(")", start + 2);
          if (end > start) {
            line = line.substring(0, start) + line.substring(end + 1);
          }
          else {
            line = line.substring(0, start);
          }
        }
        StringTokenizer t = new StringTokenizer(line);
        while (t.hasMoreTokens()) {
          String command = t.nextToken();
          if (command.toLowerCase().equals("attach-mouse-device")) {
            try {
              String family = t.nextToken(); // ignored
              String number = t.nextToken(); // ignored
              String path = t.nextToken();

              return path;

            }
            catch (NoSuchElementException e) {
              System.err.println("attach-mouse-device : syntax is attach-mouse-device <family> <number> <path>");
            }
            catch (NumberFormatException nfe) {
              System.err.println("attach-mouse-device : illegal device number");
            }
          }
        }
      }
    }
    catch (IOException ioe) {
      ioe.printStackTrace();
    }

    return Wonka.getProperty("wonka.default.mouse.device", "/dev/ts");
  }

  private static Dispatcher theDispatcher;

  private int sleep_millis = 1;
  private int priority = 5;
  private boolean keepOnGoing = false;
  private Thread thread = null;
  private Window root = null;

  public Dispatcher(Window root) {
    this.root = root;
  }

  public static synchronized Dispatcher getMainDispatcher() {
    if (theDispatcher == null) theDispatcher = new Dispatcher(null);
    return theDispatcher;
  }

  private native static void init(String mouse);
  private native static void shutdown();

  private native void poll(Window root); 

  public void run() {
    while(keepOnGoing) {
      try {
        Thread.sleep(sleep_millis);

        /*
        ** Check for events.
        */

        poll(root);
      }
      catch (InterruptedException e) {
      }
      catch (Throwable ex) {

        /*
        ** If it's not an InterruptedException, print a stacktrace
        ** but keep on going...
        */

        try {
          ex.printStackTrace();
        }
        catch(Throwable t){}
      }
    }
    thread = null;
  }

  public void start() {
    if(!keepOnGoing) {
      keepOnGoing = true;
      thread = new Thread(this, "Rudolph");
      thread.setPriority(priority);
      thread.setDaemon(false);
      thread.start();
    }
  }
  
  public void stop() {
    keepOnGoing = false;
  }

  public static boolean isDispatchThread(Thread t) {
    return "Rudolph".equals(t.getName());
  }
  
}

