/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package java.awt;

import java.util.*;
import java.awt.event.*;
import com.acunia.wonka.rudolph.peers.*;
import com.acunia.wonka.rudolph.*;

public class EventQueue {

  /**
   * variables
   */

  private AWTEvent nativeAWTEvent;
  private Vector queue = new Vector();
  private static Stack queueStack = new Stack();

  /**
   * Constructor
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public EventQueue() {
  }

  protected void dispatchEvent(AWTEvent event) {
    Object source = event.getSource();
    if (source instanceof Component) {
      ((Component)source).dispatchEvent(event);
    } 
  }

  /**
   * scan events from queue
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public synchronized AWTEvent getNextEvent() {
    if (queue.isEmpty()) return null;
    else {
      AWTEvent event = (AWTEvent) queue.elementAt(queue.size() - 1);
      queue.removeElementAt(queue.size() - 1);
      return event;
    }
  }
  
  public synchronized AWTEvent peekEvent() {
    if (queue.isEmpty()) return null;
    else return (AWTEvent) queue.elementAt(queue.size() - 1);
  }
  
  public synchronized AWTEvent peekEvent(int id) {
    System.out.println("EventQueue.peekEvent(id): TODO!");
    return null;
  }

  /**
   * post events
   * @status  Complient with java specs
   * @remark  Complient with java specs
   */
  public void postNativeEvent() {
    try {
      if (nativeAWTEvent != null) {
        postEvent(nativeAWTEvent);
        nativeAWTEvent = null;
      }
    }
    catch (Throwable t) {
      t.printStackTrace();
    }
  }
  
  // public synchronized void postEvent(AWTEvent event) {
  public void postEvent(AWTEvent event) {

    if(event.getID() != MouseEvent.MOUSE_PRESSED) {
      dispatchEvent(event);
      return;
    }

    if(DefaultMenu.menuOpened) {
      if(!((event.getSource() instanceof MenuItemComponent) ||
            (event.getSource() instanceof MenuWindow)) && DefaultMenu.lastMenuOpened != null) {
        ((DefaultMenu)DefaultMenu.lastMenuOpened.getPeer()).closeChildren();
        ((DefaultMenu)DefaultMenu.lastMenuOpened.getPeer()).closeParents();
        ((DefaultMenu)DefaultMenu.lastMenuOpened.getPeer()).close();
        DefaultMenu.menuOpened = false;
      }
    }

    dispatchEvent(event);  // temporary shortcut
  }

  public static boolean isDispatchThread() {
    return com.acunia.wonka.rudolph.Dispatcher.isDispatchThread(Thread.currentThread());
  }

  public void push(EventQueue newEventQueue) {
    Dispatcher dispatcher = new com.acunia.wonka.rudolph.Dispatcher(null);
    dispatcher.start();
    queueStack.push(dispatcher);
  }

  protected void pop() throws EmptyStackException {
    Dispatcher dispatcher = (Dispatcher)queueStack.pop();
    dispatcher.stop();
  }
}

