/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/* $Id: Dispatcher.c 3924 2003-05-14 08:08:29Z dumon $ */

#include <string.h>
#include <jni.h>
#include "rudolph.h"
#include "dispatch.h"

#include "hashtable.h"
#include "loading.h"
#include "oswald.h"

#include "registry.h"
#include "platform.h"
#include "awt-classes.h"

#include "Component.h"
#include "Event.h"
#include "Font.h"
#include "canvas.h"

#include "realms.h"

// TODO: put this in a header file "somewhere".
extern void mouse_set_path(char *s);

/*
** Positions for the splash screen. If you don't want it centered,
** #define SPLASH_POSITION_HORIZONTAL and/or SPLASH_POSITION_VERTICAL
** in your splash.xpm .
*/
#define LEFT  -1
#define TOP   -1
#define RIGHT  1
#define BOTTOM 1
#define MIDDLE 0

/*
** This is where we define the images to be used for the clear pattern
** and the splash screen. If CLEAR_IMAGE/SPLASH_IMAGE is not defined,
** no code will be generated to clear the screen / draw the "splash" image.
** For example, you could include this code in DEBUG versions only.
*/
//#ifdef DEBUG
#define CLEAR_IMAGE  "clear.xpm"
#define SPLASH_IMAGE "splash.xpm"
//#endif

#if defined(CLEAR_IMAGE) || defined(SPLASH_IMAGE)
#include "palette.h"
#endif

#ifdef  CLEAR_IMAGE
#include CLEAR_IMAGE
#endif

#ifdef  SPLASH_IMAGE
#include SPLASH_IMAGE

#ifndef SPLASH_POSITION_HORIZONTAL
#define SPLASH_POSITION_HORIZONTAL MIDDLE
#endif

#ifndef SPLASH_POSITION_VERTICAL
#define SPLASH_POSITION_VERTICAL MIDDLE
#endif
#endif

x_monitor tree_lock;

extern char* awt_splash;

/*
** Prototype.
*/

static void do_clearscreen(w_ubyte *scr, int w, int h);
static void do_splashscreen(w_ubyte *scr, int w, int h);

void Dispatcher_init(JNIEnv *env, jobject thisObj, w_instance mouseString) {
  static int init = 1;
  w_instance tlock = NULL;
  w_string mouse_string;
  char *mouse_chars;
  w_int count;
  
  if (init == 1) {
    
    /*
    ** Disable static initializer: 
    */
    
    init = 0;

    /*
    ** Initialize the lock:
    */
    
    tlock = (w_instance)getStaticReferenceField(clazzComponent, F_Component_lock);
    tree_lock = allocMonitor(tlock);
    
    /*
    ** Make sure we have a default EventQueue:
    */
  
    classMustBeInitialized(clazzEventQueue);
    defaultEventQueue = allocInstance(W_Thread_system, clazzEventQueue);
    newGlobalReference(defaultEventQueue);

    /*
    ** Make sure we have a default Toolkit:
    */
    
    classMustBeInitialized(clazzToolkit);
    
    /*
    ** Make sure we have system colors:
    */ 
    
    classMustBeInitialized(clazzSystemColor);

    /*
    ** Initialize screen:
    */
    
    woempa(9, "initialize video driver ...\n");
    screen = screen_init();
    if(!awt_splash) {
      do_clearscreen(screen->video, screen->width, screen->height);
      do_splashscreen(screen->video, screen->width, screen->height);
      screen_update(0, 0, 0, 0);
    }

    /*
    ** Initialize mouse:
    */

    mouse_string = String2string(mouseString);
    mouse_chars = string2UTF8(mouse_string, &count);
    mouse_set_path(mouse_chars + 2);

    woempa(9, "initialize mouse driver ...\n");
    mouse_init();

    /*
    ** Init keyboard.
    */
    woempa(9, "initialize keyboard driver ...\n");
    keyboard_init();

    /*
    ** Initialize fonts:
    */

    // woempa(9, "initialize font engine ...\n");
    // Font_init();

    /*
    ** Initialize rootCanvas:
    */

    woempa(9, "initialize component registry ...\n");
    rootCanvas = registry_constructor(screen->width, screen->height);
  }    
}


/*
** Runs keyboard shutdown routine (regardless of whether the keyboard was
** initialized successfully). Could run other device shutdown routines if
** we had to, but at the moment it doesn't look like we do.
*/
void Dispatcher_shutdown(JNIEnv *env, jobject thisobj) {
  screen_shutdown();
  keyboard_shutdown();
}

void Dispatcher_poll(JNIEnv *env, jobject thisObj, jobject root) {
  
  static r_event event = NULL;
  static int down = 0;
  static int ox = 0;
  static int oy = 0;
  
  int state = 0;
  int x = 0;
  int y = 0;

  int     VK = 0;
  w_char  keychar = 0;
  int     mod = 0;
  int     pressed = 0;

  r_canvas canvas;

  /*
  ** Reset our native priority.
  */

  if(x_thread_priority_get(JNIEnv2w_thread(env)->kthread) != 5) x_thread_priority_set(JNIEnv2w_thread(env)->kthread, 5);

  /*
  ** Initialize the event:
  */
  
  if(!event) event = x_mem_calloc(sizeof(r_Event));

  /*
  ** Check the root.
  */

  if(!root) {
    canvas = rootCanvas;
  }
  else {
    r_component component = getWotsitField(root, F_Component_wotsit);
    canvas = (r_canvas)component->object;
  }
  
  while(keyboard_poll(&VK, &keychar, &mod, &pressed)) {
    /*
    ** Received a keyboard event.
    */

    if(pressed) {
      Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_PRESSED);
      if(!keyboard_isMod(VK)) Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_TYPED);
    }
    else {
      Event_addKeyEvent(VK, (w_int)keychar, mod, R_EVENT_KEY_RELEASED);
    }
        
  }

  if (mouse_poll(&state, &x, &y)) {
      
    woempa(5, "received touchscreen event at (%i, %i): %i [P:%d, R:%d, D:%d]\n", 
           x, y, state, R_EVENT_MOUSE_PRESSED, R_EVENT_MOUSE_RELEASED, R_EVENT_MOUSE_DRAGGED);
      
    if (state == R_EVENT_MOUSE_RELEASED) {

      if (down == 1) {

        down = 0;

        event->tag = R_EVENT_MOUSE_RELEASED;
        event->x = x;
        event->y = y;
        event->modifiers = mod;
        Event_dispatchEvent(canvas, event);
      }
    }
    else if (state == R_EVENT_MOUSE_PRESSED) {

      if (down == 0) {

        down = 1;

        event->tag = R_EVENT_MOUSE_PRESSED;
        event->x = x;
        event->y = y;
        event->modifiers = mod;

        Event_dispatchEvent(canvas, event);
      }
    }
    else {

      if (abs(ox - x) > 5 || abs(oy - y) > 5) {

        if (down) {
          event->tag = R_EVENT_MOUSE_DRAGGED;

          event->x = x;
          event->y = y;
          event->modifiers = mod;

          Event_dispatchEvent(canvas, event);
        }
        else {
          event->tag = R_EVENT_MOUSE_MOVED;

          event->x = x;
          event->y = y;
          event->modifiers = mod;

          // Event_dispatchEvent(canvas, event);
        }

        ox = event->x;
        oy = event->y;
      }
    }
  }
  else {
    x_thread_sleep(1);
  }

  mouse_flush();
}

static void do_clearscreen(w_ubyte *scr, int w, int h) {
#ifdef CLEAR_IMAGE
  int x, y;
  int *clear_palette;
  int *clear_image;
  w_int    i;
  char   **p;
  w_ubyte *col;

  /* Draw a test pattern rather than just blanking the screen; this can
  ** give us some easy clues as to why our new display type isn't working
  ** as it should.
  */

  /*
  ** First we look up the colour names and make an array of colours.
  */
  clear_palette = allocMem_system(clear_ncolors * sizeof(int));
  for (i = 0; i < clear_ncolors; ++i) {
    col = clear_colors[i+i+1];
    p = palette;
    while (*p) {
      if (strncmp(col, *p, 32) ==0) {
        w_ubyte *rgb = *(p+1);
        clear_palette[i] = rgb2pixel(rgb[0], rgb[1], rgb[2]);
        break;
      }
      p += 2;
    }
    woempa(1, "clear palette[%d] = %16x\n", i, clear_palette[i]);
  }

  /*
  ** Now use this to build a memory image of the clear pattern.
  */
  clear_image = allocMem_system(clear_width * clear_height * sizeof(int));
  for (x = 0; x < clear_width; ++x) {
    for (y = 0; y < clear_height; ++y) {
      w_ubyte  c = clear_pixels[y][x];
      for (i = 0; i < clear_ncolors; ++i) {
        if (*clear_colors[i+i] == c) {
          clear_image[x + clear_width * y] = clear_palette[i];
          break;
        }
      }
      woempa(1, "clear image[%d, %d] = %16x\n", x, y, clear_image[x + clear_width * y]);
    }
  }

  /*
  ** Iterate over the whole screen, tiling the clear_image.
  */
  for (y = 0; y < h; y++) {
    for (x = 0; x < w; x++) {
      pixelset(clear_image[(x % clear_width) + clear_width * (y % clear_height)], scr, screen->width, x, y);
    }
  }

  releaseMem(clear_palette);
  releaseMem(clear_image);
#endif 
}

static void do_splashscreen(w_ubyte *scr, int w, int h) {
#ifdef SPLASH_IMAGE
  int x, y, xoff, yoff;
  int *splash_palette;
  w_int    i;
  char   **p;
  w_ubyte *col;

  if (SPLASH_POSITION_HORIZONTAL == LEFT) {
    xoff = 0;
  }
  else if (SPLASH_POSITION_HORIZONTAL == RIGHT) {
    xoff = w - splash_width;
  }
  else {
    xoff = (w / 2) - (splash_width / 2);
  }
  if (SPLASH_POSITION_VERTICAL == TOP) {
   yoff = 0;
  }
  else if (SPLASH_POSITION_VERTICAL == BOTTOM) {
   yoff = h - splash_height;
  }
  else {
    yoff = (h / 2) - (splash_height / 2);
  }

  /*
  ** As for the clear pattern, first look up the colours in our palette.
  */
  splash_palette = allocMem_system(splash_ncolors * sizeof(int));
  for (i = 0; i < splash_ncolors; ++i) {
    col = splash_colors[i+i+1];
    p = palette;
    while (*p) {
      if (strncmp(col, *p, 32) ==0) {
        w_ubyte *rgb = *(p+1);
        splash_palette[i] = rgb2pixel(rgb[0], rgb[1], rgb[2]);
        break;
      }
      p += 2;
    }
    woempa(1, "splash palette[%d] = %16x\n", i, splash_palette[i]);
  }

  /*
  ** We don't tile this image, so no need for a temporary copy in memory.
  */
  for (y = 0; y < splash_height; y++) {
    for (x = 0; x < splash_width; x++) {
      w_ubyte  c = splash_pixels[y][x];
      for (i = 0; i < splash_ncolors; ++i) {
        if (*splash_colors[i+i] == c) {
          pixelset(splash_palette[i], scr, w, x + xoff, y + yoff);
        }
      }
    }
  }
  
  releaseMem(splash_palette);
#endif
}

void init_awt(void) {

  /*
  ** Wonka Native Interface:
  */
  
  collectAWTFixups();
  loadAWTClasses();
}

void init_module(void) {
  woempa(9, "initializing AWT module\n");
  init_awt();
}

void clean_module(void) {
  woempa(9, "cleaning AWT module\n");
}

