
header {
  #include "oswald.h"
  #include "core-classes.h"
  #include "threads.h"
  #include "j-spot.h"
  #include "node.h"
  #include "rt-hooks.h"
  #include "mimic-stack.h"
  #include "register-allocator.h"
  #include "code-generator.h"
  #include "assembler.h"
  #include "liveness.h"
  #include "ir.h"
  #include "is.h"

  void cg_patch_jump(j_patch patch);
  void cg_patch_exit(j_patch patch);
  void cg_patch_constant(j_patch patch);

  static void count_stack_map_entries(j_node instruction, j_node operand, va_list list) {

    int id = va_arg(list, int) - 1;    
    int *size = (int *)va_arg(list, int);
    j_node call = (j_node)id;
    j_node node;

    if (operand) {
      node = operand;
    }
    else {
      node = instruction;
    }

    if (node != call && node->visited != id && node_is_collectable(node)) {
      // Compute the required size:
      *size += 4;

      // Mark the node as being "saved":
      node->visited = id;
    }
  }

  static void emit_stack_map_entries(j_node instruction, j_node operand, va_list list) {

    int id = va_arg(list, int) + 1;
    int *offset = (int *)va_arg(list, int);
    j_node call = (j_node)id;
    j_node node;

    if (operand) {
      node = operand;
    }
    else {
      node = instruction;
    }

    if (node != call && node->visited != id && node_is_collectable(node)) {
      // Load the node into register EAX:
      cg_node2register(node, R0);
      
      // Push the parameter onto the stack:
      asm_emit(assembler->body, arm_str_rm12(R0, SP, *offset));

      // Invalidate EAX:
      jm->registers[R0] = NULL;
    
      // Decrease the offset:  
      *offset -= 4;

      // Mark the node as being "saved":
      node->visited = id;
    }
  }
}

terminal j_assign, j_store, j_neg, j_iseq, j_isne, j_sub, j_add, j_mul, j_and, j_or, j_xor, j_native_call, j_java_call, j_null_pointer, j_array_bounds, j_jump, j_lt, j_gt, j_le, j_ge, j_eq, j_ne, j_return, j_constant, j_variable;

statement: register;

register : j_assign(register, register)
  code {
    ra_allocate(node->node1);
    ra_assign(node->node2, node->node1->reg);
    
    // TODO: if the use counter is 2 and if the target node is the source operand of a phi-node, move the value directly to memory?
  }

register : j_store(register, register)
  code {
    int offset;
    
    if (node->node2->node2->operator == j_constant) {
      // compute the constant offset:
      offset = node->node2->node2->data.constant.value * 4;

      if (offset > -2048 && offset < 2047) {
        // load the value in a register:
        ra_allocate(node->node1);

        // load the base address in a register:
        ra_allocate(node->node2->node1);

        /*
        ** Emit a move instruction that uses a 12-bit displacement.
        */

        asm_emit(assembler->body, arm_str_rm12(node->node1->reg, node->node2->node1->reg, offset));
      }
      else {
        jabort("invalid stack offset '%d'\n", offset);
      }
    }
    else {
      jabort("instruction not supported yet");
    }
  }

register : j_neg(register)
  code {
    ra_allocate(node->node1);
    ra_assign(node, node->node1->reg);

    asm_emit(assembler->body, arm_rsb_ri8(node->reg, node->reg, 0));
  }

register : j_iseq(register)
  code {
    ra_allocate(node->node1);
    ra_assign(node, node->node1->reg);
    
    asm_emit(assembler->body, arm_cmp_ri8(node->reg, 0));
    asm_emit(assembler->body, arm_mov_ri8_ne(node->reg, 0));
    asm_emit(assembler->body, arm_mov_ri8_eq(node->reg, 1));
  }

register : j_isne(register)
  code {
    ra_allocate(node->node1);
    ra_assign(node, node->node1->reg);
 
    asm_emit(assembler->body, arm_cmp_ri8(node->reg, 0));
    asm_emit(assembler->body, arm_mov_ri8_ne(node->reg, 1));
    asm_emit(assembler->body, arm_mov_ri8_eq(node->reg, 0));
  }

register: j_sub(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);

    asm_emit(assembler->body, arm_sub_rr(node->reg, node->reg, node->node2->reg));
  }
  cost {
    return 2;
  }

register: j_sub(register, constant)
  code {
    if (node->node2->data.constant.value >= 0 && node->node2->data.constant.value <= 255) {  // we can only use 8-bit unsigned immediate values
      ra_allocate(node->node1);
      ra_consume(node->node2);
      ra_assign(node, node->node1->reg);

      asm_emit(assembler->body, arm_sub_ri8(node->reg, node->reg, node->node2->data.constant.value));
    }
    else {
      ra_allocate(node->node1);
      ra_allocate(node->node2);
      ra_assign(node, node->node1->reg);

      asm_emit(assembler->body, arm_sub_rr(node->reg, node->reg, node->node2->reg));
    }
  }

register: j_add(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);

    asm_emit(assembler->body, arm_add_rr(node->reg, node->reg, node->node2->reg));
  }
  cost {
    return 2;
  }

register: j_add(register, constant)
  code {
    if (node->node2->data.constant.value >= 0 && node->node2->data.constant.value <= 255) {  // we can only use 8-bit unsigned immediate values
      ra_allocate(node->node1);
      ra_consume(node->node2);
      ra_assign(node, node->node1->reg);

      asm_emit(assembler->body, arm_add_ri8(node->reg, node->reg, node->node2->data.constant.value));
    }
    else {
      ra_allocate(node->node1);
      ra_allocate(node->node2);
      ra_assign(node, node->node1->reg);

      asm_emit(assembler->body, arm_add_rr(node->reg, node->reg, node->node2->reg));
    }
  }

register: j_mul(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);
    
    asm_emit(assembler->body, arm_mul_rr(node->reg, node->reg, node->node2->reg));
  }

register: j_and(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);
    
    asm_emit(assembler->body, arm_and_rr(node->reg, node->reg, node->node2->reg));
  }

register: j_or(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);
    
    asm_emit(assembler->body, arm_orr_rr(node->reg, node->reg, node->node2->reg));
  }

register: j_xor(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    ra_assign(node, node->node1->reg);
    
    asm_emit(assembler->body, arm_eor_rr(node->reg, node->reg, node->node2->reg));
  }

register: j_native_call
  code {
    j_node temp;
    int i;

    if (!ra_is_free(R0)) {
      ra_spill(node, jm->registers[R0]);
    }

    if (!ra_is_free(R1)) {
      ra_spill(node, jm->registers[R1]);
    }

    if (!ra_is_free(R2)) {
      ra_spill(node, jm->registers[R2]);
    }

    if (!ra_is_free(R3)) {
      ra_spill(node, jm->registers[R3]);
    }

    i = node->data.call.argc;
    
    while (i > 0) {
      i = i - 1;

      temp = *(j_node *)(node->data.call.args + i);

      if (i < 4) {
        /*
        ** The first four parameters are loaded into registers 1 to
        ** 4, resp. r0 to r3.
        */
      
        cg_node2register(temp, i);

        jm->registers[i] = temp;
      }
      else {
        /*
        ** The remaining parameters are pushed onto the stack in 
        ** reverse order.
        */
       
        jfail("too many arguments ...\n");
      }
 
      ra_consume(temp);
    }
    
    /*
    ** Call far.  The BL (branch and link) instruction has only a 32Mb
    ** range which doesn't always suffice when branch from the data to 
    ** the code segment.  As such, we change the PC and LR "manually".
    */
   
    // Load the branch address into R8:
    cg_constant2register(assembler->body, node->data.call.value, R8);
   
    // Make the next PC the return address:
    asm_emit(assembler->body, arm_mov_rr(LR, PC)); 

    // Branch by overwriting the PC:
    asm_emit(assembler->body, arm_mov_rr(PC, R8)); 
    
    /*
    ** Setup the registers for the return value (if any).
    */
  
    if (node->type == j_void) {
      jm->registers[R0] = NULL;
      jm->registers[R1] = NULL;
      jm->registers[R2] = NULL;
      jm->registers[R3] = NULL;
      node->reg = 0;
    }
    else {
      jm->registers[R0] = node;
      jm->registers[R1] = NULL;
      jm->registers[R2] = NULL;
      jm->registers[R3] = NULL;
      node->reg = R0;
    }
  }


register: j_java_call
  code {
    j_node temp;
    int offset;
    int slots;
    int size;
    int i;
   
    woempa(9, "going to emit code for java call %q (%p)\n", node, node);
 
    slots = method_size_locals() + method_size_stack();
    
    if (!ra_is_free(R0)) {
      ra_spill(node, jm->registers[R0]);
    }

    if (!ra_is_free(R1)) {
      ra_spill(node, jm->registers[R1]);
    }

    if (!ra_is_free(R2)) {
      ra_spill(node, jm->registers[R2]);
    }

    if (!ra_is_free(R3)) {
      ra_spill(node, jm->registers[R3]);
    }

    /*
    ** Make sure we have constants "0" (no stack trace) and "1" (stack
    ** trace) handy in registers.  These registers must be cleared for
    ** cg_node2register() to work correctly.
    */
    
    cg_constant2register(assembler->body, 0, R2);
    cg_constant2register(assembler->body, stack_trace, R3);
    jm->registers[R2] = NULL;
    jm->registers[R3] = NULL;

    /*
    ** Reserve place for the stack frame:
    */

    size = sizeof(x_Frame) + node->data.call.argc * 8;
    
    // Compute the size of the stack map: 
    for (i = 0; i < slots; i++) {
      temp = ms_node_state(node->pc, j_post_execution, i);
      if (temp) {
        node_operands(temp, count_stack_map_entries, node, &size);
      }
    }

    size += 4;   // stack map terminator

    if (size > 250) {
      jfail("stack frame to large\n");
    }
    
    offset = size;
    asm_emit(assembler->body, arm_sub_ri8(SP, SP, size));

    /*
    ** Protect the live references by padding them to the stack frame,
    ** terminated by '42'.
    */

    offset -= 4; // - 4 because initially, SP points to the last used slot

    // push the terminator:
    cg_constant2register(assembler->body, 42, R0);
    asm_emit(assembler->body, arm_str_rm12(R0, SP, offset));
    jm->registers[R0] = NULL;
    offset -= 4;

    // push the references:
    for (i = 0; i < slots; i++) {
      temp = ms_node_state(node->pc, j_post_execution, i);
      if (temp) {
        node_operands(temp, emit_stack_map_entries, node, &offset);
      }
    }

    /*
    ** Push the slots for the return value of the callee(s): 
    */
   
    asm_emit(assembler->body, arm_str_rm12(R2, SP, offset));
    offset -= 4;
    asm_emit(assembler->body, arm_str_rm12(R2, SP, offset));
    offset -= 4;

    /*
    asm_emit(assembler->body, arm_str_rm12(R2, SP, offset));   // disabled for now; we don't support 64-bit values yet
    offset -= 4;
    asm_emit(assembler->body, arm_str_rm12(R2, SP, offset));   // disabled for now; we don't support 64-bit values yet
    offset -= 4;
    */

    /*
    ** Push the arguments:
    */
 
    i = node->data.call.argc;
    
    while (i > 1) {
      i = i - 1;

      temp = *(j_node *)(node->data.call.args + i);

      /*
      ** Push the slot's identifier:
      */
      
      if (temp->type == j_unknown) {
        jabort("we can't protect parameters of type j_unknown\n");
      }
      else if (temp->type == j_address) {
        asm_emit(assembler->body, arm_str_rm12(R3, SP, offset));
        offset -= 4;
      }
      else {
        asm_emit(assembler->body, arm_str_rm12(R2, SP, offset));
        offset -= 4;
      }
 
      /*
      ** Push the slot's value:
      */
      
      cg_node2register(temp, R0);
      asm_emit(assembler->body, arm_str_rm12(R0, SP, offset));
      jm->registers[R0] = NULL;
    
      offset -= 4;
      
      ra_consume(temp);
    }

    /*
    ** Move a pointer to the frame and the method in R0 and R1:
    */
 
    temp = *(j_node *)(node->data.call.args);

    asm_emit(assembler->body, arm_mov_rr(R1, SP));
    jm->registers[R1] = NULL;     // make sure we clear 'R1' or cg_node2register() gets confused
 
    cg_node2register(temp, R0);
    jm->registers[R0] = NULL;     // make sure we clear 'R0' or cg_node2register() gets confused
    
    /*
    ** Call far.  The BL (branch and link) instruction has only a 32Mb
    ** range which doesn't always suffice when branch from the data to 
    ** the code segment.  As such, we change the PC and LR "manually".
    */
   
    // Load the branch address into R8:
    cg_constant2register(assembler->body, node->data.call.value, R8);
   
    // Make the next PC the return address:
    asm_emit(assembler->body, arm_mov_rr(LR, PC)); 

    // Branch by overwriting the PC:
    asm_emit(assembler->body, arm_mov_rr(PC, R8)); 
     
    /*
    ** Clean up the arguments and the stack frame:
    */

    asm_emit(assembler->body, arm_add_ri8(SP, SP, size));

    /*
    ** Setup the registers for the return value (if any).
    */
 
    if (node->type == j_void) {
      jm->registers[R0] = NULL;
      jm->registers[R1] = NULL;
      jm->registers[R2] = NULL;
      jm->registers[R3] = NULL;
      node->reg = 0;
    }
    else {
      jm->registers[R0] = node;
      jm->registers[R1] = NULL;
      jm->registers[R2] = NULL;
      jm->registers[R3] = NULL;
      node->reg = R0;
    }
  }

register: j_null_pointer(register)
  code {
    // Compare the instance with NULL:
    ra_allocate(node->node1);
    asm_emit(assembler->body, arm_cmp_ri8(node->node1->reg, 0));
   
    // If the instance is non-zero, branch to the end of the IR instruction:
    asm_emit(assembler->body, arm_bne(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);

    // Load the exception calzz in R0:
    cg_constant2register(assembler->body, (j_word)clazzNullPointerException, R0);
   
    // Make the next PC the return address:
    asm_emit(assembler->body, arm_mov_rr(LR, PC));
 
    // Branch by overwriting the PC:
    cg_constant2register(assembler->body, (j_word)rt_exception_throw, PC);
  }

register: j_array_bounds(register, register)
  code {
    /*
    ** Save the caller saved registers on stack:
    */

    if (!ra_is_free(R0)) {
      ra_spill(node, jm->registers[R0]);
    }

    if (!ra_is_free(R1)) {
      ra_spill(node, jm->registers[R1]);
    }

    if (!ra_is_free(R2)) {
      ra_spill(node, jm->registers[R2]);
    }

    if (!ra_is_free(R3)) {
      ra_spill(node, jm->registers[R3]);
    }

    /*
    ** Prepare the function arguments:
    */

    cg_node2register(node->node1, R0);
    cg_node2register(node->node2, R1);

    // Make the next PC the return address:
    asm_emit(assembler->body, arm_mov_rr(LR, PC));
 
    // Branch by overwriting the PC:
    cg_constant2register(assembler->body, (j_word)rt_check_array_bounds, PC);

    /*
    ** Setup the registers to return control.
    */

    jm->registers[R0] = NULL;
    jm->registers[R1] = NULL;
    jm->registers[R2] = NULL;
    jm->registers[R3] = NULL;
  }

register: j_jump
  code {
    asm_emit(assembler->body, arm_b(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_lt(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);

    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_blt(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_gt(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);

    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_bgt(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_le(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);

    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_ble(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_ge(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);

    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_bge(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_eq(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);

    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_beq(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_ne(register, register)
  code {
    ra_allocate(node->node1);
    ra_allocate(node->node2);
    
    asm_emit(assembler->body, arm_cmp_rr(node->node1->reg, node->node2->reg));
    asm_emit(assembler->body, arm_bne(0));
    asm_add_patch(asm_size_body(), 0, ir_get_branch(node), &cg_patch_jump);
  }

register: j_return(register)
  code {
    /*
    ** On x86, %eax is used to hold the result, or %edx:%eax for 64-bit 
    ** results.
    */
    
    cg_node2register(node->node1, R0);
     
    /*
    ** If the given node is not the last node in the linearized DAG, we 
    ** are compiling a method with multiple returns.  If that is the case,
    ** we make all nodes but the last node jump to a single exit point.
    */
 
    if (node->next) {
      asm_emit(assembler->body, arm_b(0));
      asm_add_patch(asm_size_body(), 0, NULL, &cg_patch_exit);
    }
  }

register: j_return
  code {
    /*
    ** If the given node is not the last node in the linearized DAG, we 
    ** are compiling a method with multiple returns.  If that is the case,
    ** we make all nodes but the last node jump to a single exit point.
    */
 
    if (node->next) {
      asm_emit(assembler->body, arm_b(0));
      asm_add_patch(asm_size_body(), 0, NULL, &cg_patch_exit);
    }
  }

constant : j_constant
  code {
    /*
    ** Do nothing - we encountered a literal, not an instruction.  Every
    ** instruction is held responsible for "preparing" its own operands.
    ** This is required because (a) some x86 instructions can keep one or
    ** more operands in memory and (b) to get the addresses of the join 
    ** points (forward and backward references) right.
    */
  }

register : j_constant
  code {
    /*
    ** Do nothing - we encountered a literal, not an instruction.  Every
    ** instruction is held responsible for "preparing" its own operands.
    ** This is required because (a) some x86 instructions can keep one or
    ** more operands in memory and (b) to get the addresses of the join 
    ** points (forward and backward references) right.
    */
  }

register : j_variable
  code {
    /*
    ** Do nothing - we encountered a literal, not an instruction.  Every
    ** instruction is held responsible for "preparing" its own operands.
    ** This is required because (a) some x86 instructions can keep one or
    ** more operands in memory and (b) to get the addresses of the join 
    ** points (forward and backward references) right.
    */
  }

register : j_variable(register, register)
  code {
    /*
    ** Do nothing - we encountered a literal, not an instruction.  Every
    ** instruction is held responsible for "preparing" its own operands.
    ** This is required because (a) some x86 instructions can keep one or
    ** more operands in memory and (b) to get the addresses of the join 
    ** points (forward and backward references) right.
    */
  }


footer {
  void burg_walk(j_node node, int goal, int depth) {

    j_node kids[3];
    short *nts;
    int rule;

    if (node->emitted == 0) {
      node->offset = asm_size_body();

      rule = burg_rule(node->state, goal);
      nts = burg_nts[rule];

      // woempa(9, "%snode %s (%p), goal %d, rule '%s' (%d), nts %p, kids[0] = %d, kids[1] = %d\n", burg_spaces(depth), node_get_operator(node), node, goal, burg_rule_name[rule], rule, nts, (kids[0] ? kids[0]->operator : 0), (kids[1] ? kids[1]->operator : 0));

      if (rule && nts) {
        burg_kids(node, rule, kids);

        if (kids[0] && (depth == 0 || !node_is_call(kids[0]))) {
          burg_walk(kids[0], nts[0], depth + 1);
        }

        if (kids[1] && (depth == 0 || !node_is_call(kids[1]))) {
          burg_walk(kids[1], nts[1], depth + 1);
        }

      }
      else {
        woempa(9, "%snode '%s', goal %d, rule '%s' (%d), nts %p\n", burg_spaces(depth), node_get_operator(node), goal, burg_rule_name[rule], rule, nts);
        fprintf(stderr, "failing operator '%s'\n", node_get_operator(node));
        exit(0);
      }

      burg_code(rule, node);
      node->emitted = 1;
    }
  }
}

