/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: code-generator.c 4031 2003-07-10 15:19:28Z dbuytaert $
*/

#include "oswald.h"

#include "methods.h"
#include "j-spot.h"
#include "ir.h"
#include "register-allocator.h"
#include "code-generator.h"
#include "assembler.h"
#include "is.h"

static void cg_register2stack(j_buffer buffer, int reg, int offset) {
  
  #ifdef DEBUG
    if (offset == 0) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else if (offset < - jm->slots * 4 - 16) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else 
  #endif

  if (offset > -2048 && offset < 2047) {
    /*
    ** Emit a move instruction that uses a 12-bit displacement.
    */

    asm_emit(buffer, arm_str_rm12(reg, R11, offset));
  }
  
  #ifdef DEBUG
    else {
      jabort("invalid stack offset '%d'\n", offset);
    }
  #endif
}

static void cg_stack2register(j_buffer buffer, int offset, int reg) {
  
  #ifdef DEBUG
    if (offset == 0) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else if (offset < - jm->slots * 4 - 16) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else 
  #endif
 
  if (offset > -2048 && offset < 2047) {
    /*
    ** Emit a move instruction that uses a 12-bit displacement.
    */

    asm_emit(buffer, arm_ldr_m12r(reg, R11, offset));
  }

  #ifdef DEBUG
    else {
      jabort("invalid stack offset '%d'\n", offset);
    }
  #endif
}

void cg_patch_jump(j_patch patch) {

  /*
  ** This function patches the generated machine code to insert
  ** the correct 24-bit jump offset to the target basic block.
  */

  int offset = patch->dst->offset - patch->offset - 4;

  if (offset > -83886608 && offset < 83886607) {
    // the offset need sign extension, see A4-10:
    *((int *)(assembler->body->base + patch->offset - 4)) |= (int)(0x00ffffff & (offset >> 2));
      // TODO: it would be nice if this code snippet would rely less on "assembler.c" internals.
  }
  else {
    jfail("patch's jump offset out of range: %d\n", offset);
  }
}

void cg_patch_exit(j_patch patch) {

  /*
  ** This function patches the generated machine code to insert
  ** the correct 24-bit jump offset to the function's epilogue.
  */
 
  int offset = assembler->body->step - assembler->body->base - patch->offset - 4;
    
  if (offset > -83886608 && offset < 83886607) {
    // the offset need sign extension, see A4-10:
    *((int *)(assembler->body->base + patch->offset - 4)) |= (int)(0x00ffffff & (offset >> 2));
      // TODO: it would be nice if this code snippet would rely less on "assembler.c" internals.
  }
  else {
    jfail("patch's exit offset out of range: %d\n", offset);
  }
}

void cg_patch_constant(j_patch patch) {

  /*
  ** This function patches the generated machine code to insert
  ** the correct 12-bit offset to the function's constant pool.
  */
  
  int offset = assembler->body->step - assembler->body->base - patch->offset + patch->value - 4;

  if ((offset & 0xfffff000) == 0) {   
    // the offset is always positive, see A5-28:
    *((int *)(assembler->body->base + patch->offset - 4)) |= offset;
      // TODO: it would be nice if this code snippet would rely less on "assembler.c" internals.
  }
  else {
    jfail("patch's constant offset out of range: %d\n", offset);
  }
}

void cg_constant2register(j_buffer buffer, int value, int reg) {

  // TODO: use the MVN instruction for small 8-bit negative numbers.
  
  if (value >= 0 && value <= 255) {
    /*
    ** The MOV instruction allows an 8-bit constant to be loaded into
    ** a registers.
    */
    
    asm_emit(buffer, arm_mov_ri8(reg, value));
  }
  else {
    
    /*
    ** Because ARM instructions are pricesely 32 bits there aren't any
    ** instructions that will directly load 32-bit immediate constants
    ** into a register.  As such, we have to use a constant or literal
    ** pool (typically) placed after the function's epilogue.
    */
    
    asm_emit(assembler->constant, arm_i32(value));
    asm_emit(buffer, arm_ldr_m12r(reg, PC, 0));
    asm_add_patch(asm_size_body(), asm_size_constant(), NULL, &cg_patch_constant);
  }  
}


void cg_spill(j_node instruction, j_node variable) {
 
  /*
  ** Allocate and initialize the node's stack location.  Stack slots 
  ** are assigned lazily - they are assigned when nodes are actually
  ** being spilled.
  */

  if (variable->stack == 0) {
    variable->stack = - 4 * ra_stack_slot(instruction, variable) - 16;
  } 

  /*
  ** Emit spill code:
  */
  
  cg_register2stack(assembler->body, variable->reg, variable->stack);

  woempa(9, "spill node %q in register %d to stack offset %x\n", variable, variable->reg, variable->stack);
}

void cg_init(void) {

  j_node node;
  int i;
 
  woempa(9, "initializing register/stack layout\n");

  i = jm->method->exec.nargs;

  if (jm->method->flags & ACC_STATIC) {

    while (i >= 0) {
      node = jm->parameters[i];

      if (node) { 
        if (i < 2) {
          /*
          ** The first four parameters are loaded into registers 1 to 4, resp. r0
          ** to r3.
          */
          jm->registers[i + 2] = node;
          node->reg = i + 2;
        }
        else {
          /*
          ** The remaining parameters are pushed onto the stack in reverse order.
          */

          node->stack = 4 * (i - 1);
          node->spill = 1;
        }
  
        woempa(9, "assigned %q to register %d, stack slot %d, spill %d\n", node, node->reg, node->stack, node->spill);  
      }  
     
      i = i - 1;
    }
  }
  else {

    while (i >= 0) {
      node = jm->parameters[i];

      if (node) { 
        if (i < 3) {
          /*
          ** The first four parameters are loaded into registers 1 to 4, resp. r0
          ** to r3.
          */
            
          jm->registers[i + 1] = node;
          node->reg = i + 1;
        }
        else {
          /*
          ** The remaining parameters are pushed onto the stack in reverse order.
          */

          node->stack = 4 * (i - 2);
          node->spill = 1;
        }
        
        woempa(9, "assigned %q to register %d, stack slot %d, spill %d\n", node, node->reg, node->stack, node->spill);  
      }  
     
      i = i - 1;
    }
  }
}

void cg_prologue(void) {

  woempa(9, "called prologue\n");

  /*
  ** Move the stack pointer into R12.
  */
  
  asm_emit(assembler->prologue, arm_mov_rr(R12, SP));
  
  /*
  ** Push R18 (call address), the frame pointer, the stack pointer,
  ** the link register and the program counter on the stack.
  */
 
  asm_emit(assembler->prologue, arm_stmdb(SP, (0xfffffff0 & jm->usage) | BIT_R8 | BIT_R11 | BIT_R12 | BIT_LR | BIT_PC));   

  /*
  ** Update the frame pointer.
  */
  
  asm_emit(assembler->prologue, arm_sub_ri8(R11, R12, 4));
  
  /*
  ** Update the stack pointer.  Note that the SUB instruction takes an
  ** unsigned 8-bit constant (0x00 - 0xff) so we can only decrement 62
  ** words using a single instruction.
  */
  
  if (jm->slots > 60) {
    jfail("too many local variables\n");
  }
 
  if (jm->slots > 0) {
    asm_emit(assembler->prologue, arm_sub_ri8(SP, SP, 4 * jm->slots));
  }
}

void cg_epilogue(void) {

  woempa(9, "called epilogue\n");

  /*
  ** Restore the temporary callee-saved registers by popping them from
  ** the stack.
  */

  asm_emit(assembler->epilogue, arm_ldmdb(R11, (0xfffffff0 & jm->usage) | BIT_R8 | BIT_R11 | BIT_SP | BIT_PC));
}

void cg_node2register(j_node node, int r) {

  int offset;

  // NOTE: the order of blocks in the if-clauses below is important.

  if (node) {

    woempa(9, "move '%q' (register = %d, spill = %d, stack = %d) to register %d\n", node, (ra_in_register(node) ? node->reg : -1), node->spill, node->stack, r);

    /*
    ** A literal with a constant value.
    */

    if (node->operator == j_constant) {
      cg_constant2register(assembler->body, (j_word)node->data.constant.value, r);
    } 

    /*
    ** A variable that currently resides in a register.
    */
      
    else if (ra_in_register(node)) {
      if (node->reg != r) {
        asm_emit(assembler->body, arm_mov_rr(r, node->reg));
      }
    }

    /*
    ** A parameter on the stack.
    */
  
    else if (node_is_parameter(node)) {
      cg_stack2register(assembler->body, node->stack, r);
    }
 
    /*
    ** A variable that is kept in memory.
    */
 
    else if (node_is_memory_variable(node)) {
      if (node->node2->operator == j_constant) {
        // extract the constant offset:
        offset = node->node2->data.constant.value * 4;
     
        if (offset > -2048 && offset < 2047) {
          // move the start address to the specified register:
          cg_node2register(node->node1, r);

          /*
          ** Emit a move instruction that uses a 12-bit displacement.
          */

          asm_emit(assembler->body, arm_ldr_m12r(r, r, offset));
        }
        else {
          jabort("invalid stack offset '%d'\n", offset);
        }
      }
      else {
        jabort("instruction not supported yet!\n");
      }
    }
 
    /*
    ** A variable that is spilled on the stack.
    */
  
    else if (ra_on_stack(node)) {
      cg_stack2register(assembler->body, node->stack, r);
    }
  
    /*
    ** Default.
    */
  
    else {
      jfail("don't know what to do with %q (address: %p, stack: %d, spill: %d, register: %d)\n", node, node, node->stack, node->spill, node->reg);
    }
  }
}

void cg_clear_cache(char * begin, char * end) {

  register char * _beg __asm ("a1") = begin;
  register char * _end __asm ("a2") = end;
  register int _flg __asm ("a3") = 3;

  /*
  ** Options for Linux's "sys_cacheflush" system call:
  **   a3 == 3: flush both data and instruction cache.
  **   a3 == 2: write-back and flush data cache.
  **   a3 == 1: flush instruction cache.
  ** See include/asm-mips/cachectl.h for details.
  */

  asm volatile (
      "swi 0x9f0002 @ sys_cacheflush"
      : /* outputs */
      : /* inputs */
      : "a1"
  );
}

// Prototype required for disassembler in hal/cpu/arm/src/disassemble.c:
unsigned long db_disasm(unsigned long, int, int);

j_word *cg_disassemble(j_word * address) {
  return NULL;
}


