#ifndef _J_X86_IS_H
#define _J_X86_IS_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: is.h 3863 2003-04-25 09:23:25Z dbuytaert $ 
*/

#ifndef J_NUM_REGISTERS
 #define J_NUM_REGISTERS 4
#endif

inline static j_word endian_32(j_word w) {
  return (w >> 24) | ((w >> 8) & 0x0000ff00) | ((w << 8) & 0x00ff0000) | (w << 24);
}

/*
** Register field:
*/

#define AL     0x0
#define CL     0x1
#define DL     0x2
#define BL     0x3
#define AH     0x4
#define CH     0x5
#define DH     0x6
#define BH     0x7

#define AX     0x0
#define CX     0x1
#define DX     0x2
#define BX     0x3
//#define SP     0x4
#define BP     0x5
#define SI     0x6
#define DI     0x7

#define EAX    0x0
#define ECX    0x1
#define EDX    0x2
#define EBX    0x3
#define ESP    0x4
#define EBP    0x5
#define ESI    0x6
#define EDI    0x7

/*
** Bits
*/

#define BIT_EAX   (1 << EAX)
#define BIT_ECX   (1 << ECX)
#define BIT_EDX   (1 << EDX)
#define BIT_EBX   (1 << EBX)
#define BIT_ESP   (1 << ESP)
#define BIT_EBP   (1 << EBP)
#define BIT_ESI   (1 << ESI)
#define BIT_EDI   (1 << EDI)

/*
** Condition codes:
*/

#define COND_EQ    0x4
#define COND_NE    0x5
#define COND_LT    0xc
#define COND_GE    0xd
#define COND_LE    0xe
#define COND_GT    0xf

/*
** Binary instruction encoding:
**
**    rr = register to register
**    mr = memory to register
**    ir = immediate to register
**    im = immediate to memory
** The destination register is specified first.
*/

#define x86_add_rr(rd, rs) \
  0, 16, 8, 0x01, 2, 0x03, 3, rs, 3, rd

#define x86_add_m8r(rd, offset, rs) \
  0, 24, 8, 0x03, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_add_rm8(rd, offset, rs) \
  0, 24, 8, 0x01, 2, 0x1, 3, rs, 3, rd, 8, offset
  
#define x86_add_ri8(rd, ad) \
  0, 24, 8, 0x83, 5, 0x18, 3, rd, 8, (char)ad
  
#define x86_add_ri32(rd, ad) \
  0, 48, 8, 0x81, 5, 0x18, 3, rd, 32, endian_32(ad)

#define x86_add_i8m8(rd, offset, data) \
  0, 32, 8, 0x83, 2, 0x01, 3, 0x000, 3, rd, 8, offset, 8, (char)data 

#define x86_sub_rr(rd, rs) \
  0, 16, 8, 0x29, 2, 0x03, 3, rs, 3, rd

#define x86_sub_m8r(rd, offset, rs) \
  0, 24, 8, 0x2b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_sub_rm8(rd, offset, rs) \
  0, 24, 8, 0x29, 2, 0x1, 3, rs, 3, rd, 8, offset  

#define x86_sub_ri8(rd, ad) \
  0, 24, 8, 0x83, 5, 0x1d, 3, rd, 8, (char)ad

#define x86_sub_ri32(rd, ad) \
  0, 48, 8, 0x81, 5, 0x1d, 3, rd, 32, endian_32(ad)

#define x86_sub_i8m8(rd, offset, data) \
  0, 32, 8, 0x83, 2, 0x01, 3, 0x5, 3, rd, 8, offset, 8, (char)data

#define x86_mul_rr(rd, rs) \
  0, 24, 8, 0x0f, 8, 0xaf, 2, 0x03, 3, rd, 3, rs

#define x86_mul_m8r(rd, rs, offset) \
  0, 32, 8, 0x0f, 8, 0xaf, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_mul_ri8r(rd, data, rs) \
  0, 24, 8, 0x6b, 2, 0x3, 3, rd, 3, rs, 8, (char)data

#define x86_neg_r(rd) \
  0, 16, 8, 0xf7, 8, 0xd8 | rd

#define x86_neg_m8(rd, offset) \
  0, 24, 8, 0xf7, 2, 0x1, 3, 0x3, 3, rd, 8, offset 

#define x86_mov_rr(rd, rs) \
  0, 16, 8, 0x89, 2, 0x3, 3, rs, 3, rd

#define x86_mov_ri32(rd, ad) \
  0, 40, 8, 0xb8 | rd, 32, endian_32(ad)

#define x86_mov_ri32(rd, ad) \
  0, 40, 8, 0xb8 | rd, 32, endian_32(ad)
  
#define x86_mov_m8r(rd, rs, offset) \
  0, 24, 8, 0x8b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_mov_m32r(rd, rs, offset) \
  0, 48, 8, 0x8b, 2, 0x2, 3, rd, 3, rs, 32, endian_32(offset)

#define x86_mov_rm8(rd, offset, rs) \
  0, 24, 8, 0x89, 2, 0x1, 3, rs, 3, rd, 8, offset
 
#define x86_mov_rm32(rd, offset, rs) \
  0, 48, 8, 0x89, 2, 0x2, 3, rs, 3, rd, 32, endian_32(offset)
  
#define x86_lea(rd, offset, rs) \
  0, 24, 8, 0x8d, 2, 0x1, 3, rd, 3, rs, 8, offset
    
#define x86_push_r(rd) \
  0, 8, 8, 0x50 | rd

#define x86_push_i32(imm) \
  0, 40, 8, 0x68, 32, endian_32(imm)

#define x86_pop(rd) \
  0, 8, 8, 0x58 | rd

#define x86_call_r(r) \
  0, 16, 8, 0xff, 8, 0xd0 | r

#define x86_call_i32(offset) \
  0, 40, 8, 0xe8, 32, endian_32(offset)

#define x86_gt_8(offset) \
  0, 16, 4, 0x7, 4, COND_GT, 8, offset 

#define x86_ge_8(offset) \
  0, 16, 4, 0x7, 4, COND_GE, 8, offset
  
#define x86_lt_8(offset) \
  0, 16, 4, 0x7, 4, COND_LT, 8, offset
  
#define x86_le_8(offset) \
  0, 16, 4, 0x7, 4, COND_LE, 8, offset
  
#define x86_eq_8(offset) \
  0, 16, 4, 0x7, 4, COND_EQ, 8, offset
  
#define x86_ne_8(offset) \
  0, 16, 4, 0x7, 4, COND_NE, 8, offset

#define x86_gt_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_GT, 32, endian_32(offset) 

#define x86_ge_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_GE, 32, endian_32(offset)

#define x86_lt_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_LT, 32, endian_32(offset)

#define x86_le_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_LE, 32, endian_32(offset)

#define x86_eq_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_EQ, 32, endian_32(offset)

#define x86_ne_32(offset) \
  0, 48, 8, 0x0f, 4, 0x8, 4, COND_NE, 32, endian_32(offset)

#define x86_jmp_8(offset) \
  0, 16, 8, 0xeb, 8, offset

#define x86_jmp_32(offset) \
  0, 40, 8, 0xe9, 32, endian_32(offset)

#define x86_jmp_r(reg) \
  0, 16, 8, 0xff, 5, 0x1c, 3, reg

#define x86_cmp_rr(ra, rb) \
  0, 16, 8, 0x39, 2, 0x3, 3, rb, 3, ra

#define x86_cmp_rm8(rd, offset, rs) \
  0, 24, 8, 0x3b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_cmp_m8r(offset, rd, rs) \
  0, 24, 8, 0x39, 2, 0x1, 3, rs, 3, rd, 8, offset

#define x86_cmp_rm32(rd, offset, rs) \
  0, 48, 8, 0x3b, 2, 0x2, 3, rd, 3, rs, 32, endian_32(offset)
  
#define x86_cmp_m32r(offset, rd, rs) \
  0, 48, 8, 0x39, 2, 0x2, 3, rs, 3, rd, 32, endian_32(offset)
 
#define x86_cmp_i32r(reg, imm) \
  0, 48, 8, 0x81, 5, 0x1f, 3, reg, 32, endian_32(imm)

#define x86_cmp_i8m8(rs, offset, data) \
  0, 32, 8, 0x83, 2, 0x1, 3, 0x7, 3, rs, 8, offset, 8, (char)data 

#define x86_and_rr(rd, rs) \
  0, 16, 8, 0x21, 2, 0x3, 3, rd, 3, rs

#define x86_and_i8r(rd, data) \
  0, 24, 8, 0x83, 5, 0x1c, 3, rd, 8, (char)data 

#define x86_and_i32r(rd, data) \
  0, 48, 8, 0x81, 5, 0x1c, 3, rd, 32, endian_32(data)

#define x86_and_m8r(rd, offset, rs) \
  0, 24, 8, 0x23, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_or_rr(rd, rs) \
  0, 16, 8, 0x09, 2, 0x3, 3, rd, 3, rs

#define x86_or_m8r(rd, offset, rs) \
  0, 24, 8, 0x0b, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_or_rm8(rd, offset, rs) \
  0, 24, 8, 0x09, 2, 0x1, 3, rs, 3, rd, 8, offset

#define x86_or_i8r(rd, data) \
  0, 24, 8, 0x83, 5, 0x19, 3, rd, 8, (char)data
  
#define x86_xor_rr(rd, rs) \
  0, 16, 8, 0x31, 2, 0x3, 3, rd, 3, rs

#define x86_xor_m8r(rd, offset, rs) \
  0, 24, 8, 0x33, 2, 0x1, 3, rd, 3, rs, 8, offset

#define x86_xor_rm8(rd, offset, rs) \
  0, 24, 8, 0x31, 2, 0x1, 3, rs, 3, rd, 8, offset

#define x86_xor_i8r(rd, data) \
  0, 24, 8, 0x83, 5, 0x1e, 3, rd, 8, (char)data 

#define x86_shl(rd) \
  0, 16, 8, 0xd1, 5, 0x1c, 3, rd

#define x86_shr_cl(rd) \
  0, 16, 8, 0xd3, 5, 0x1d, 3, rd

#define x86_shr(rd) \
  0, 16, 8, 0xd1, 5, 0x1d, 3, rd  

#define x86_shl_cl(rd) \
  0, 16, 8, 0xd3, 5, 0x1c, 3, rd

#define x86_inc_r(rd) \
  0, 16, 8, 0xff, 5, 0x18, 3, rd

#define x86_inc_m8(rd, offset) \
  0, 24, 8, 0xff, 2, 0x01, 3, 0x0, 3, rd, 8, offset

#define x86_inc_m32(rd, offset) \
  0, 48, 8, 0xff, 2, 0x02, 3, 0x0, 3, rd, 32, endian_32(offset)

#define x86_dec_r(rd) \
  0, 16, 8, 0xff, 5, 0x19, 3, rd

#define x86_dec_m8(rd, offset) \
  0, 24, 8, 0xff, 2, 0x01, 3, 0x1, 3, rd, 8, offset

#define x86_dec_m32(rd, offset) \
  0, 48, 8, 0xff, 2, 0x02, 3, 0x1, 3, rd, 32, endian_32(offset)

#define x86_sete(rs) \
  0, 24, 8, 0x0f, 4, 0x9, 4, 0x4, 5, 0x18, 3, rs
  
#define x86_setne(rs) \
  0, 24, 8, 0x0f, 4, 0x9, 4, 0x5, 5, 0x18, 3, rs
  
#define x86_nop() \
  0, 8, 8, 0x90

#endif
