/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: code-generator.c 4026 2003-07-10 09:27:02Z dbuytaert $
*/

#include "oswald.h"

#include "methods.h"
#include "j-spot.h"
#include "ir.h"
#include "register-allocator.h"
#include "code-generator.h"
#include "assembler.h"
#include "is.h"


/*
** Generate machine code for moving register content to the stack at
** the specified offset.
*/ 
static void cg_register2stack(j_buffer buffer, int reg, int offset) {

  #ifdef DEBUG
    if (offset == 0) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else if (offset < - jm->slots * 4 - 16) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else 
  #endif

  if (offset > -128 && offset < 127) {
    /*
    ** Emit a move instruction that uses a 8-bit displacement.
    */

    asm_emit(buffer, x86_mov_rm8(EBP, offset, reg));
  }
  else {
    /*
    ** Emit a move instruction that uses a 32-bit displacement.
    */

    asm_emit(buffer, x86_mov_rm32(EBP, offset, reg));
  }
}


/*
** Generate machine code for moving stack contents at a specified offset
** to the specified register.
*/ 
static void cg_stack2register(j_buffer buffer, int offset, int reg) {
  
  #ifdef DEBUG
    if (offset == 0) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else if (offset < - jm->slots * 4 - 16) {
      jabort("invalid stack offset '%d'\n", offset);
    } 
    else 
  #endif
  
  if (offset > -128 && offset < 127) {
    /*
    ** Emit a move instruction that uses a 8-bit displacement.
    */

    asm_emit(buffer, x86_mov_m8r(reg, EBP, offset));
  }
  else {
    /*
    ** Emit a move instruction that uses a 32-bit displacement.
    */

    asm_emit(buffer, x86_mov_m32r(reg, EBP, offset));
  }
}

void cg_patch_jump(j_patch patch) {

  /*
  ** This function patches the generated machine code to insert
  ** the correct 32-bit jump offset to the target basic block.
  */

  *((int *)(assembler->body->base + patch->offset - 4)) = (int)(patch->dst->offset - patch->offset);
    // TODO: it would be nice if this code snippet would rely less on "assembler.c" internals.
}

void cg_patch_exit(j_patch patch) {

  /*
  ** This function patches the generated machine code to insert
  ** the correct 32-bit jump offset to the function's epilogue.
  */

  *((int *)(assembler->body->base + patch->offset - 4)) = (int)(assembler->body->step - assembler->body->base - patch->offset);
    // TODO: it would be nice if this code snippet would rely less on "assembler.c" internals.
}

void cg_spill(j_node instruction, j_node variable) {

  /*
  ** Allocate and initialize the node's stack location.  Stack slots 
  ** are assigned lazily - they are assigned when nodes are actually
  ** being spilled.
  */

  if (variable->stack == 0) {
    variable->stack = - 4 * ra_stack_slot(instruction, variable);
  } 

  /*
  ** Emit spill code:
  */

  cg_register2stack(assembler->body, variable->reg, variable->stack);

  woempa(9, "spill node %q in register %d to stack offset %x\n", variable, variable->reg, variable->stack);
}

void cg_init(void) {

  j_node node;
  int i;
 
  woempa(9, "initializing register/stack layout\n");

  i = jm->method->exec.nargs;

  if (jm->method->flags & ACC_STATIC) {
    while (i >= 0) {
      node = jm->parameters[i];

      if (node) {
        node->stack = 4 * node->data.variable.offset + 16;
        node->spill = 1;
          // + 16: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
      }
      i = i - 1;
    }
  }
  else {
    while (i >= 0) {
      node = jm->parameters[i];
      
      if (node) {
        node->stack = 4 * node->data.variable.offset + 12;
        node->spill = 1;
          // + 12: skip the "meta information", see "wonka/hal/cpu/x86/scr/calls.s".
      }
      i = i - 1;
    }
  }
}

void cg_prologue(void) {

  woempa(9, "called prologue\n");

  /*
  ** Save the old base pointer.
  */
  
  asm_emit(assembler->prologue, x86_push_r(EBP));           // push %ebp

  /*
  ** Set base pointer as frame pointer.
  */
  
  asm_emit(assembler->prologue, 0, 16, 8, 0x89, 8, 0xe5);   // mov %esp, %ebp
  
  /*
  ** Reserve stack space for the local variables by decrementing ESP.
  */
  
  asm_emit(assembler->prologue, x86_sub_ri32(ESP, jm->slots * 4));
   
  /*
  ** Push temporary callee-saved registers onto the stack so we can restore them later on.
  */

  if (jm->usage & (1 << EBX)) {
    asm_emit(assembler->prologue, x86_push_r(EBX));
  }
  
  if (jm->usage & (1 << ESI)) {
    asm_emit(assembler->prologue, x86_push_r(ESI));
  }
  
  if (jm->usage & (1 << EDI)) {
    asm_emit(assembler->prologue, x86_push_r(EDI));
  }
}

void cg_epilogue(void) {

  woempa(9, "called epilogue\n");

  /*
  ** Restore the temporary callee-saved registers by popping them from the stack.
  */
  
  if (jm->usage & (1 << EBX)) {
    cg_stack2register(assembler->epilogue, - (jm->slots + 1) * 4, EBX);  
  }

  if (jm->usage & (1 << ESI)) {
    cg_stack2register(assembler->epilogue, - (jm->slots + 2) * 4, ESI);  
  }

  if (jm->usage & (1 << EDI)) {
    cg_stack2register(assembler->epilogue, - (jm->slots + 3) * 4, EDI);  
  }

  /*
  ** "leave": prepare the stack for returning.
  */
  
  asm_emit(assembler->epilogue, 0, 8, 8, 0xc9);

  /*
  ** "ret": pop the return address off the stack and jump to this location.
  */
  
  asm_emit(assembler->epilogue, 0, 8, 8, 0xc3);
}


/*
** Generate machine code for moving the contents of the node to the
** specified register.  Depending on the type of node, other machinecode
** will being generated.
*/

void cg_node2register(j_node node, int r) {

  int offset;

  // NOTE: the order of blocks in the if-clauses below is important.

  if (node) {

    woempa(9, "move '%q' (register = %d, spill = %d, stack = %d) to register %d\n", node, (ra_in_register(node) ? node->reg : -1), node->spill, node->stack, r);

    /*
    ** A literal with a constant value.
    */

    if (node->operator == j_constant) {
      if (node->data.constant.value == 0) {
        /*
        ** The preferred instruction to clear a register is "xor reg, reg";
        ** the condition codes are not preserved though.
        */
        asm_emit(assembler->body, x86_xor_rr(r, r));
      }
      else {
        asm_emit(assembler->body, x86_mov_ri32(r, (j_word)node->data.constant.value));
      }
    } 

    /*
    ** A variable that currently resides in a register.  Do not generate a
    ** move if source and destination are the same.
    */
      
    else if (ra_in_register(node)) {
      if (node->reg != r) {
        asm_emit(assembler->body, x86_mov_rr(r, node->reg));
      }
    }

    /*
    ** A parameter on the stack.
    */
  
    else if (node_is_parameter(node)) {
      cg_stack2register(assembler->body, node->stack, r);
    }
 
    /*
    ** A variable that is spilled on the stack.
    */
  
    else if (ra_on_stack(node)) {
      cg_stack2register(assembler->body, node->stack, r);
    }

    /*
    ** A variable that is kept in memory.
    */
 
    else if (node_is_memory_variable(node)) {
      if (node->node2->operator == j_constant) {
        // extract the constant offset:
        offset = node->node2->data.constant.value * 4;
     
        // move the start address to the specified register:
        cg_node2register(node->node1, r);

        if (offset > -128 && offset < 127) {
          /*
          ** Emit a move instruction that uses a 8-bit displacement.
          */

          asm_emit(assembler->body, x86_mov_m8r(r, r, offset));
        }
        else {
          /*
          ** Emit a move instruction that uses a 32-bit displacement.
          */

          asm_emit(assembler->body, x86_mov_m32r(r, r, offset));
        }
      }
      else {
        jabort("instruction not supported yet!\n");
      }
    }
 
    /*
    ** Default.
    */
  
    else {
      jfail("don't know what to do with %q (address: %p, stack: %d, spill: %d, register: %d)\n", node, node, node->stack, node->spill, node->reg);
    }
  }
}

void cg_clear_cache(char * begin, char * end) {
  // do nothing
}

// Prototype required for disassembler in hal/cpu/x86/src/disassemble.c:
unsigned long db_disasm(unsigned long, int, int);

j_word *cg_disassemble(j_word * address) {
  return (j_word *)db_disasm((unsigned long)address, 0, 0);
}

