#ifndef _J_ASSEMBLER_H
#define _J_ASSEMBLER_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: assembler.h 3863 2003-04-25 09:23:25Z dbuytaert $ 
*/

extern j_assembler assembler;

typedef struct j_Assembler {
  j_buffer prologue;
  j_buffer epilogue;
  j_buffer constant;
  j_buffer body;
} j_Assembler;

typedef struct j_Buffer {
  j_patch patches;
  char *base;
  char *step;
  int size;
} j_Buffer;

typedef struct j_Patch {
  void (*patcher)(j_patch patch);
  j_node dst;
  int offset;
  int value;
  j_patch next;
} j_Patch;

void asm_init(void);
  /* Initialize or reset the assembler. */

int asm_emit(j_buffer buffer, int endian, int count, ...);
  /* Emit (append) one or more instructions to a code buffer, resp. the prologue, body or epilogue buffer. */

void asm_dump(void);
  /* Print the emitted code to screen. */

char *asm_insn(j_word inst);
  /* Return a string with a bit-wise representation of the specified word/instruction. */

char * asm_export(void);
  /* Export the emitted code as a single memory chunk filled with binary machine code. */

int asm_size_body(void);
  /* Get the size of the body expressed in bytes. */

int asm_size_prologue(void);
  /* Get the size of the prologue expressed in bytes. */

int asm_size_epilogue(void);
  /* Get the size of the epilogue expressed in bytes. */

int asm_size_constant(void);
  /* Get the size of the constant or literal pool expressed in bytes. */

int asm_size_total(void);
  /* Get the total size of generated machine code expressed in bytes. */

void asm_add_patch(int offset, int value, j_node dst, void (*patcher)(j_patch patch));
  /* Register a patch to the assembler. */

#endif
