#ifndef _J_METHOD_H
#define _J_METHOD_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: method.h 3863 2003-04-25 09:23:25Z dbuytaert $ 
*/

#include "j-spot.h"

typedef struct j_Method {
  // General:
  char *name;
  w_method method;              // wonka method
  int failed;                   // status flag
  
  // Translation:
  unsigned char * bytecode;     // a data block with bytecodes
  unsigned char * work;         // a list with work inforamation
  unsigned char * join;         // a list of join points
  unsigned char * code;         // a list of bytecodes
  unsigned char * data;         // a list of meta data
  unsigned int * depth;         // a list of stack depths

  j_node * * states;            // an array of states

  int instructions;             // the number of instructions
  int curr_pc;                  // current program counter
  int inst_pc;                  // current instruction pointer
  int length;                   // length of the bytecode
  int sp;

  
  j_node * parameters;          // an array of parameters
  j_node * locals;              // an array of local variables
  j_node * stack;               // an array of stack variables
  
  j_node start;                 // the start node
  
  // Depth first spanning tree, dominators:
  j_node * dfs;
  int * ancestor;               // ancestor array
  int * fbucket;                // bucket with first nodes
  int * nbucket;                // bucket with next nodes
  int * parent;                 // parent array
  int * sdom;                   // semi-dominators
  int * idom;                   // immediate dominators
  
  int todo[100][3];
  int next;
    
  // Register allocation:
  j_node * registers;           // an array of all the allocated registers
  int usage;                    // bitmask indicating which registers
                                // are used by this method
  
  // Code generation:
  int prots;                    // the number of references to protect
  int slots;                    // the number of required slots on the native stack

} j_Method;

j_method method_create(w_method method);
  /* Create a new method. */

int method_size_stack(void);
  /* Get the size of the stack variables. */

int method_size_locals(void);
  /* Get the size of the local variables. */

int method_size_parameters(void);
  /* Get the size of the function parameters. */

int method_num_parameters(void);
  /* Get the number of function parameters. */

int method_is_static(void);
  /* Returns true when the method is static. */

void method_dump(void);
  /* Dump the method to screen for debuging purpose. */

#endif /* _J_METHOD_H */
