#ifndef _J_MIMIC_STACK_H
#define _J_MIMIC_STACK_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: mimic-stack.h 3863 2003-04-25 09:23:25Z dbuytaert $ 
*/

#include "j-spot.h"

typedef enum {
  j_pre_execution = 0,
  j_post_execution = 1
} j_timestamp;

void ms_init(void);
  /* Initializes the mimic stack. */
  
void ms_load_state(int pc, j_timestamp timestamp);
  /* Load an old stack context. */
  
void ms_save_state(int pc, j_timestamp timestamp);
  /* Save the current stack context. */

j_node ms_node_state(int pc, j_timestamp timestamp, int slot);
  /* Return the the node at the specified slot of the given state. */
  
void ms_uses(void);
  /* Increment the use counter of the live variables. */
 
void ms_clone(int inherit);
  /* Clone the current state information. */
 
void ms_push(j_node node);
  /* Push a new node on the mimic stack. */

j_node ms_pop(j_type type);
  /* Pop a node off the mimic stack. */
  
j_node ms_top(j_type type);
  /* Get the top node off the mimic stack. */
  
j_node ms_next(j_type type);
  /* Get the next node on the stack. */

void ms_dup1(void);
  /* Duplicate the top of the stack. */

void ms_dup1_x1(void);
  /* Duplicate the top of the stack and insert a new node beneath the second node. */

void ms_dup1_x2(void);
  /* Duplicate the top of the stack and insert a new node beneath the third node. */

void ms_dup2(void);
  /* Duplicate the top two nodes on the stack. */

void ms_dup2_x1(void);
  /* Duplicate the top two nodes on the stack and insert a new node beneath the third node. */

void ms_dup2_x2(void);
  /* Duplicate the top two nodes on the stack and insert a new node beneath the forth node. */

void ms_swap(void);
  /* Swaps the top two nodes on the stack. */

void ms_dump_stack(void);
  /* Dump the mimic stack to screen. */

void ms_dump_state(int pc, j_timestamp timestamp);
  /* Dump the state information at the specified PC to screen. */

j_node ms_set_local(j_type type, int offset);
  /* Set a local variable. */

j_node ms_get_local(j_type type, int offset);
  /* Get a local variable. */

#endif
