#ifndef _J_RT_HOOKS_H
#define _J_RT_HOOKS_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: rt-hooks.h 3988 2003-06-05 13:07:52Z dbuytaert $ 
*/

#include "j-spot.h"
#include "ieee754.h"

/*
** Run-time intefaces:
*/

void rt_exception_throw(w_clazz clazz);

#define rt_float_add float32_add
  /* Add two floats. */

#define rt_float_sub float32_sub
  /* Subtract two floats. */

#define rt_float_mul float32_mul
  /* Multiply two floats. */

#define rt_float_neg float32_negate
  /* Negate a float. */

#define rt_float_div float32_div
  /* Divide two floats. */

w_float rt_float_rem(w_float value1, w_float value2);
  /* Calculate the remainder of a float division. */

int rt_integer_div(int value1, int value2);
  /* Divide two integers. */
   
int rt_integer_rem(int value1, int value2);
  /* Calculate the remainder of an integer division. */

int rt_integer2byte(int value);
   /* Convert an integer to a byte. */

int rt_integer2character(int value);
   /* Convert an integer to a character. */

#define rt_integer2float int32_to_float32
   /* Convert an integer to a float. */

int rt_integer2short(int value);
   /* Convert an integer to a short. */

int rt_float2integer(w_float value);
   /* Convert a float to an integer. */

int rt_monitorenter(j_method method, j_node symbol);
  /* Enter monitor for the object reference in symbol. */
  
int rt_monitorexit(j_method method, j_node symbol);
  /* Exit monitor for the object reference in symbol. */

void rt_athrow(w_instance throwable);
  /* Throw an error or exception. */

void rt_putstatic(int * fields, int slot, j_word value);
  /* Set a static field in the object reference. */

void rt_putfield_reference(w_instance instance, int slot, j_word value);
  /* Set a instance field in the object reference. */

w_instance rt_getfield_reference(w_instance instance, int slot);
  /* Retrieve a reference field. */

w_instance rt_new(w_clazz clazz);
  /* Create a new instance of the specified class. */

int rt_instanceof(w_instance instance, w_clazz clazz);
  /* Determine whether an instance is of a given type. */

void rt_checkcast(w_instance instance, w_clazz clazz);
  /* Ensure that an instance is of the given type. */

int rt_invokeinterface(w_method method, x_frame frame, ...);
  /* Invoke an interface method. */

int rt_invokestatic(w_method method, x_frame frame, ...);
  /* Invoke a static class method. */

int rt_array_create(w_clazz clazz, int dimensions, ...);
  /* Create a new array of primitive type components. */

int rt_float_compare_g(w_float, w_float);
  /* Compare 2 floats with NaN check */

int rt_float_compare_l(w_float, w_float);
  /* Compare 2 floats without NaN check */

void rt_check_array_bounds(w_instance instance, int index);
  /* Check the array bounds of the specified array instance. */

extern void * array_load_dispatcher[];
extern void * array_store_dispatcher[];
extern void * invokenonvirtual_dispatcher[];
extern void * invokevirtual_dispatcher[];
extern void * invokespecial_dispatcher[];
 
#endif
