/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

public class ArithmeticOperationsTest  {

  /*
  ** The "int_" functions will never be compiled.
  */

  int int_iadd(int a, int b) {
    return a + b;
  }

  int int_isub(int a, int b) {
    return a - b;
  }

  int int_imul(int a, int b) {
    return a * b;
  }

  int int_irem(int a, int b) {
    return a % b;
  }
  
  int int_idiv(int a, int b) {
    return a / b;
  }

  int int_ineg(int a) {
    return - a;
  }

  float int_fadd(float a, float b) {
    return a + b;
  }

  float int_fsub(float a, float b) {
    return a - b;
  }

  float int_fmul(float a, float b) {
    return a * b;
  }

  float int_frem(float a, float b) {
    return a % b;
  }
  
  float int_fdiv(float a, float b) {
    return a / b;
  }

  float int_fneg(float a) {
    return - a;
  }

  /*
  ** The "com_" functions will awlays be compiled.
  */

  int com_iadd(int a, int b) {
    return a + b;
  }

  int com_isub(int a, int b) {
    return a - b;
  }

  int com_imul(int a, int b) {
    return a * b;
  }

  int com_irem(int a, int b) {
    return a % b;
  }

  int com_idiv(int a, int b) {
    return a / b;
  }

  int com_ineg(int a) {
    return - a;
  }

  float com_fadd(float a, float b) {
    return a + b;
  }

  float com_fsub(float a, float b) {
    return a - b;
  }

  float com_fmul(float a, float b) {
    return a * b;
  }

  float com_frem(float a, float b) {
    return a % b;
  }
  
  float com_fdiv(float a, float b) {
    return a / b;
  }

  float com_fneg(float a) {
    return - a;
  }
  
  public void run(JTest jt) {

    /*
    ** To test the arithmetic operations both using positive and negative
    ** numbers, we let 'i' and 'j' itererate over negative to a positive
    ** values.
    */

    // Integers:

    int k[] = { 0, 5, -5, 25, -25, Character.MAX_VALUE, Character.MIN_VALUE, Short.MAX_VALUE, Short.MIN_VALUE, Integer.MIN_VALUE, Integer.MAX_VALUE };
 
    for (int i = 0; i < 10; i++) {
      for (int j = 0; j < 10; j++) {
        jt.check(int_iadd(k[i], k[j]), com_iadd(k[i], k[j]), "ArithemeticOperationsTest.com_iadd("+ k[i] +", "+ k[j] +")");
        jt.check(int_isub(k[i], k[j]), com_isub(k[i], k[j]), "ArithemeticOperationsTest.com_isub("+ k[i] +", "+ k[j] +")");
        jt.check(int_imul(k[i], k[j]), com_imul(k[i], k[j]), "ArithemeticOperationsTest.com_imul("+ k[i] +", "+ k[j] +")");
        jt.check(int_ineg(k[i]), com_ineg(k[i]), "ArithemeticOperationsTest.com_ineg("+ k[i] +")");
        
        if (k[j] != 0) {   // skip over the ArithmeticExceptions
          jt.check(int_irem(k[i], k[j]), com_irem(k[i], k[j]), "ArithemeticOperationsTest.com_irem("+ k[i] +", "+ k[j] +")");
          jt.check(int_idiv(k[i], k[j]), com_idiv(k[i], k[j]), "ArithemeticOperationsTest.com_idiv("+ k[i] +", "+ k[j] +")");
        }
      }
    }

    // Floats:

    float f[] = { 0.00f, -1.11f, 2.22f, -3.33f, 4.44f, -5.55f, 6.66f, -7.77f, 8.88f, -9.99f };

    for (int i = 0; i < 10; i++) {
      for (int j = 0; j < 10; j++) {
        jt.check(int_fadd(f[i], f[j]), com_fadd(f[i], f[j]), "ArithemeticOperationsTest.com_fadd("+ f[i] +", "+ f[j] +")");
        jt.check(int_fsub(f[i], f[j]), com_fsub(f[i], f[j]), "ArithemeticOperationsTest.com_fsub("+ f[i] +", "+ f[j] +")");
        jt.check(int_fmul(f[i], f[j]), com_fmul(f[i], f[j]), "ArithemeticOperationsTest.com_fmul("+ f[i] +", "+ f[j] +")");
        jt.check(int_fneg(i), com_fneg(i), "ArithemeticOperationsTest.com_fneg("+ f[i] +")");
        
        if (f[j] != 0) {   // skip over the ArithmeticExceptions
          jt.check(int_frem(f[i], f[j]), com_frem(f[i], f[j]), "ArithemeticOperationsTest.com_frem("+ f[i] +", "+ f[j] +")");
          jt.check(int_fdiv(f[i], f[j]), com_fdiv(f[i], f[j]), "ArithemeticOperationsTest.com_fdiv("+ f[i] +", "+ f[j] +")");
        }
      }
    }
  }
}

