/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

public class ExceptionTest  {

  /*
  ** The "int_" functions will never be compiled.
  */

  public int int_throw_implicit(Integer i) {
    return i.intValue();
  }

  public void int_throw_explicit() {
    throw(new NullPointerException());
  }

  public int int_propagate_3(Integer i) {
    try {
      return int_throw_implicit(i);
    }
    catch (NullPointerException e) {
      return 3;
    }
  }

  public void int_propagate_2(int x) {
    int_throw_explicit();
  }

  public void int_propagate_1(int x) {
    int_propagate_2(x);
  }
  
  public int int_try_catch(int x) {
    try {
      int_propagate_1(x);
      return 0;
    }
    catch (Exception e) {
      return x;
    } 
  }

  public int int_try_catch_finally(int x) {

    int y = x;

    try {
      int_propagate_1(x);
      y = y + 1;
    }
    catch (Exception e) {
      y = y + 2;
    } 
    finally {
      y = y + 4;
      return y;
    }
  }

  /*
  ** The "com_" functions will always be compiled.
  */

  public int com_throw_implicit(Integer i) {
    return i.intValue();
  }

  public void com_throw_explicit() {
    throw(new NullPointerException());
  }

  public int com_propagate_3(Integer i) {
    try {
      return com_throw_implicit(i);
    }
    catch (NullPointerException e) {
      return 3;
    }
  }

  public void com_propagate_2(int x) {
    if (x <= 6) {             // to switch between Java and native stack frames
      int_throw_explicit();
    }
    else {
      com_throw_explicit();
    }
  }

  public void com_propagate_1(int x) {
    if (x <= 3) {             // to switch between Java and native stack frames
      int_propagate_2(x);
    }
    else {
      com_propagate_2(x);
    }
  }

  public int com_try_catch(int x) {
    try {
      com_propagate_1(x);
      return 0;
    }
    catch (Exception e) {
      return x;
    } 
  }

  public int com_try_catch_finally(int x) {

    int y = x;

    try {
      com_propagate_1(x);
      y = y + 1;
    }
    catch (Exception e) {
      y = y + 2;
    } 
    finally {
      y = y + 4;
      return y;
    }
  }

  public void run(JTest jt) {

    /*
    ** Test throwing of explicit exceptions, exception handling and 
    ** propagation of exceptions through both Java and native stack 
    ** frames.
    */

    for (int i = 0; i < 10; i++) {
      jt.check(int_try_catch(i), com_try_catch(i), "ExceptionTest.com_try_catch("+ i +")");
      jt.check(int_try_catch_finally(i), com_try_catch_finally(i), "ExceptionTest.com_try_catch_finally("+ i +")");
    }

    /*
    ** Test throwing of implicit exceptions.
    */

    Integer d[] = { new Integer(1), null, new Integer(3), null, new Integer(4) };

    for (int i = 0; i < 5; i++) {
      jt.check(int_propagate_3(d[i]), com_propagate_3(d[i]), "ExceptionTest.com_propagate_3("+ d[i] +")");
    }
  }
}
