/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

import java.util.Vector;

public class MethodTest extends Dummy {
 
  private Vector v = new Vector();
 
  public static int max(int x, int y) {
    return (x > y) ? x : y;
  }

  public int compute_1(int a) {
    return a + 1;
  } 

  public int compute_2(int a, int b) {
    return (a << 1) + b + 1;
  } 

  public int compute_3(int a, int b, int c) {
    return (a << 2) + (b << 1) + c + 1;
  } 

  private int privateMethod(int x) {
    return - x;
  } 

  public int publicMethod(int x) {
    return - x;
  } 

  /*
  ** The "int_" functions will never be compiled.
  */

  public static int int_invokestatic_1(int x, int y) {
    return max(x, y);
  }

  public static int int_invokestatic_2(int x, int y) {
    return int_invokestatic_1(x, y);
  }

  public static int int_invokestatic_3(int x, int y) {
    return int_invokestatic_1(x, y);
  }

  public Object int_invokespecial_1(int x) {
    // "invokespecial" is used to invoke (1) instance initialization methods.
    return new Integer(x);
  }

  public int int_invokespecial_2(int x) {
    // "invokespecial" is used to invoke (2) superclass methods.
    super.invokespecial(x);
    return this.x;
  }
 
  public int int_invokespecial_3(int x) {
    // "invokespecial" is used to invoke (3) private methods.
    return privateMethod(x);
  }

  public int int_invokevirtual_1(int x) {
    v.add(new Integer(x));
    return v.size();
  }

  private int int_propagate_1(MethodTest mt, int x) {
    return mt.publicMethod(x);
  }

  private int int_propagate_2(MethodTest mt, int x) {
    return mt.privateMethod(x);
  }

  public int int_exception_1(MethodTest mt, int x) {
    try {
      return int_propagate_1(mt, x);
    }
    catch (Exception e) {
      return -1;
    }
  }

  public int int_exception_2(MethodTest mt, int x) {
    try {
      return int_propagate_2(mt, x);
    }
    catch (Exception e) {
      return -1;
    }
  }

  public int int_parameter_1(int x, int y) {
    return compute_1(x > y ? x : y);
      // NOTE: "compute_1" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_2(int x, int y) {
    return compute_1(x > y ? x - y : y - x);
     // NOTE: "compute_1" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_3(int x, int y) {
    return compute_2((x > y ? x : y), (y > x ? x : y));
     // NOTE: "compute_2" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_4(int x, int y) {
    return compute_2((x > y ? x - y : y - x), (y > x ? y - x : x - y));
      // NOTE: "compute_2" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_5(int x, int y) {
    return compute_3((x > y ? x : y), (y > x ? x : y), (x == y ? x : y));
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_6(int x, int y) {
    return compute_3((x > y ? x - y : y - x), (y > x ? y - x : x - y), (x == y ? x + y : x * y));
     // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_7(int x, int y) {
    return compute_3(x - y, y - x, x * y);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_8(int x, int y) {
    return compute_3(x - y + x, y - x + y, x * y * x);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int int_parameter_9(int x, int y) {
    return compute_3(x++, y++, x--);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int int_recursive_1(int x) {
    if (x < 0) {
      x = 0;
    }
    else {   
      x = x + int_recursive_1(x - 1);
    }

    return x;
  }

  public int int_recursive_2(int x) {
    if (x < 0) {
      x = 0;
    }
    else if (x < 5) {   
      x = x + int_recursive_2(x - 1);
    }
    else {
      x = x + int_recursive_2(x - 2);
    }

    return x;   
  }

  public int int_recursive_3(int x) {

    /*
    ** Fibonacci - simple to write, horrible to compute.
    */

    if (x < 2) {
      x = 1;
    }
    else {
      x = int_recursive_3(x - 1) + int_recursive_3(x - 2);
    }

    return x;   
  }

  /*
  ** The "com_" functions will always be compiled.
  */

  public static int com_invokestatic_1(int x, int y) {
    return max(x, y);
  }

  public static int com_invokestatic_2(int x, int y) {
    // Call a static Java method.
    return int_invokestatic_1(x, y);
     // ---^^^
  }

  public static int com_invokestatic_3(int x, int y) {
    // Call a static native method.
    return com_invokestatic_1(x, y);
     // ---^^^
  }

  public Object com_invokespecial_1(int x) {
    // "invokespecial" is used to invoke (1) instance initialization methods.
    return new Integer(x);
  }
  
  public int com_invokespecial_2(int x) {
    // "invokespecial" is used to invoke (2) superclass methods.
    super.invokespecial(x);
    return this.x;
  }

  public int com_invokespecial_3(int x) {
    // "invokespecial" is used to invoke (3) private methods.
    return privateMethod(x);
  }

  public int com_invokevirtual_1(int x) {
    v.add(new Integer(x));
    return v.size();
  }

  private int com_propagate_1(MethodTest mt, int x) {
    return mt.publicMethod(x);
  }

  private int com_propagate_2(MethodTest mt, int x) {
    return mt.privateMethod(x);
  }

  public int com_exception_1(MethodTest mt, int x) {
    try {
      return com_propagate_1(mt, x);
    }
    catch (Exception e) {
      return -1;
    }
  }

  public int com_exception_2(MethodTest mt, int x) {
    try {
      return com_propagate_2(mt, x);
    }
    catch (Exception e) {
      return -1;
    }
  }
 
  public int com_parameter_1(int x, int y) {
    return compute_1(x > y ? x : y);
	// NOTE: "compute_1" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_2(int x, int y) {
    return compute_1(x > y ? x - y : y - x);
     // NOTE: "compute_1" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_3(int x, int y) {
    return compute_2((x > y ? x : y), (y > x ? x : y));
      // NOTE: "compute_2" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_4(int x, int y) {
    return compute_2((x > y ? x - y : y - x), (y > x ? y - x : x - y));
      // NOTE: "compute_2" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_5(int x, int y) {
    return compute_3((x > y ? x : y), (y > x ? x : y), (x == y ? x : y));
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_6(int x, int y) {
    return compute_3((x > y ? x - y : y - x), (y > x ? y - x : x - y), (x == y ? x + y : x * y));
     // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_7(int x, int y) {
    return compute_3(x - y, y - x, x * y);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_8(int x, int y) {
    return compute_3(x - y + x, y - x + y, x * y * x);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }

  public int com_parameter_9(int x, int y) {
    return compute_3(x++, y++, x--);
      // NOTE: "compute_3" is both a call and join point that needs phi-nodes.
  }
 
  public int com_recursive_1(int x) {
    if (x < 0) {
      x = 0;
    }
    else {   
      x = x + com_recursive_1(x - 1);
    }

    return x; 
  } 

  public int com_recursive_2(int x) {
    if (x < 0) {
      x = 0;
    }
    else if (x < 5) {   
      x = x + com_recursive_2(x - 1);
    }
    else {
      x = x + com_recursive_2(x - 2);
    }

    return x;   
  }

  public int com_recursive_3(int x) {

    /*
    ** Fibonacci - simple to write, horrible to compute.
    */

    if (x < 2) {
      x = 1;
    }
    else {
      x = com_recursive_3(x - 1) + com_recursive_3(x - 2);
    }

    return x;   
  }

  public void run(JTest jt) {

    /*
    ** Bytecode "invokestatic".  We test both compilation of static 
    ** methods and JIT compiled methods calling static methods - both
    ** native and Java methods.
    */

    for (int i = 0; i < 5; i++) {
      jt.check(int_invokestatic_1(2, i), com_invokestatic_1(2, i), "MethodTest.com_invokestatic_1(2, "+ i +")");
      jt.check(int_invokestatic_2(2, i), com_invokestatic_2(2, i), "MethodTest.com_invokestatic_2(2, "+ i +")");
      jt.check(int_invokestatic_3(2, i), com_invokestatic_3(2, i), "MethodTest.com_invokestatic_3(2, "+ i +")");
    }

    /*
    ** Bytecode "invokespecial".
    */

    for (int i = 0; i < 5; i++) {
      jt.check(int_invokespecial_1(i), com_invokespecial_1(i), "MethodTest.com_invokespecial_1("+ i +")");
      jt.check(int_invokespecial_2(i), com_invokespecial_2(i), "MethodTest.com_invokespecial_2("+ i +")");
      jt.check(int_invokespecial_3(i), com_invokespecial_3(i), "MethodTest.com_invokespecial_3("+ i +")");
    }

    /*
    ** Bytecode "invokevirtual".
    */

    for (int i = 0; i < 5; i++) {
      jt.check(int_invokevirtual_1(i), com_invokevirtual_1(i) - 1, "MethodTest.com_invokevirtual_1("+ i +")");
    }

    /*
    ** Exceptions.
    */

    MethodTest d[] = { this, null, this, null, this };

    for (int i = 0; i < 5; i++) {
      jt.check(int_exception_1(d[i], i), com_exception_1(d[i], i), "MethodTest.com_exception_1("+ d[i] +", "+ i +")");
      jt.check(int_exception_2(d[i], i), com_exception_2(d[i], i), "MethodTest.com_exception_2("+ d[i] +", "+ i +")");
    }

    /*
    ** Complex parameter passing.
    */

    for (int i = 0; i < 5; i++) {
      jt.check(int_parameter_1(i, 5 - i), com_parameter_1(i, 5 - i), "MethodTest.com_parameter_1("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_2(i, 5 - i), com_parameter_2(i, 5 - i), "MethodTest.com_parameter_2("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_3(i, 5 - i), com_parameter_3(i, 5 - i), "MethodTest.com_parameter_3("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_4(i, 5 - i), com_parameter_4(i, 5 - i), "MethodTest.com_parameter_4("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_5(i, 5 - i), com_parameter_5(i, 5 - i), "MethodTest.com_parameter_5("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_6(i, 5 - i), com_parameter_6(i, 5 - i), "MethodTest.com_parameter_6("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_7(i, 5 - i), com_parameter_7(i, 5 - i), "MethodTest.com_parameter_7("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_8(i, 5 - i), com_parameter_8(i, 5 - i), "MethodTest.com_parameter_8("+ i +", "+ (5 - i) +")");
      jt.check(int_parameter_9(i, 5 - i), com_parameter_9(i, 5 - i), "MethodTest.com_parameter_9("+ i +", "+ (5 - i) +")");
    }

    /*
    ** Recursive methods.
    */

    for (int i = 0; i < 10; i++) {
      jt.check(int_recursive_1(i), com_recursive_1(i), "MethodTest.com_recursive_1("+ i +")");
      jt.check(int_recursive_2(i), com_recursive_2(i), "MethodTest.com_recursive_2("+ i +")");
      jt.check(int_recursive_3(i), com_recursive_3(i), "MethodTest.com_recursive_3("+ i +")");
    }
  }
}

