/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

package com.acunia.wonka.test.jspot;

public class ObjectTest  {
  
  private static Dummy a;
  private Dummy b;
  private static int x;
  private int y;

  public Object getObject(int x) {
    return (Object)(new Integer(x));
  }

  /*
  ** The "int_" functions will never be compiled.
  */

  public Integer int_new_1(int x) {
    return new Integer(x);
  }
 
  public int int_getstatic_1() {
    return x;
  }

  public boolean int_getstatic_2() {
    return (x < 3);
  }
 
  public int int_getstatic_3(Dummy d) {
    return d.m;
  }

  public int int_getstatic_exception(Dummy d) {
    try {
      return int_getstatic_3(d);
    }
    catch (NullPointerException e) {
      return 0;
    }
  }

  public int int_getfield_1() {
    return y;
  }
 
  public boolean int_getfield_2() {
    return (y < 3);
  }

  public int int_getfield_3(Dummy d) {
    return d.z;
  }

  public int int_getfield_exception(Dummy d) {
    try {
      return int_getfield_3(d);
    }
    catch (NullPointerException e) {
      return 0;
    }
  }

  public int int_putstatic_1(int x) {
    this.x = x;
    return this.x;
  }

  public int int_putstatic_2(Dummy d) {
    this.a = d;
    return this.a.z;
  }

  public int int_putfield_1(int y) {
    this.y = y;
    return this.y;
  }

  public int int_putfield_2(Dummy d) {
    this.b = d;
    return this.b.z;
  }

  public boolean int_instanceof_1(Object object) {
    return (object instanceof Integer);
  }

  public boolean int_checkcast_1(Object object) {
    Integer i = (Integer)object;
    return true;
  }

  public Integer int_checkcast_2(int x) {
    return (Integer)getObject(x);
  }

  public int int_monitorenter() {
    synchronized(new Object()) {
	return 1;
    }
  }

  /*
  ** The "com_" functions will always be compiled.
  */

  public String com_new_1() {
    return new String("foo");
  }

  public Integer com_new_2(int x) {
    return new Integer(x);
  }

  public int com_getstatic_1() {
    return x;
  }

  public boolean com_getstatic_2() {
    return (x < 3);
  }
    
  public int com_getstatic_3(Dummy d) {
    return d.m;
  }

  public int com_getstatic_exception(Dummy d) {
    try {
      return com_getstatic_3(d);
    }
    catch (NullPointerException e) {
      return 0;
    }
  }

  public int com_getfield_1() {
    return y;
  }

  public boolean com_getfield_2() {
    return (y < 3);
  }

  public int com_getfield_3(Dummy d) {
    return d.z;
  }

  public int com_getfield_exception(Dummy d) {
    try {
      return com_getfield_3(d);
    }
    catch (NullPointerException e) {
      return 0;
    }
  }

  public int com_putstatic_1(int x) {
    this.x = x;
    return this.x;
  }

  public int com_putstatic_2(Dummy d) {
    this.a = d;
    return this.a.z;
  }

  public int com_putfield_1(int y) {
    this.y = y;
    return this.y;
  }
 
  public int com_putfield_2(Dummy d) {
    this.b = d;
    return this.b.z;
  }

 
  public boolean com_instanceof_1(Object object) {
    return (object instanceof Integer);
  }
    
  public boolean com_checkcast_1(Object object) {
    Integer i = (Integer)object;
    return true;
  }

  public Integer com_checkcast_2(int x) {
    return (Integer)getObject(x);
  }

  public int com_monitorenter() {
    synchronized(new Object()) {
	return 1;
    }
  }

  public int com_locals_1(int x) {
    /*
    ** Used to test the way local references are being dealt with in
    ** generated code.
    */

    int i;

    for (i = 0; i < x; i++) {
      Object o = new Object();
    }

    return i;
  }

  public int com_locals_2(int x) {
    /*
    ** Used to test the way local references are being dealt with in
    ** generated code.
    */

    int i;

    for (i = 0; i < x; i++) {
      Object o = new Object();
      Dummy.invokestatic(o);
    }

    return i;
  }


  public void run(JTest jt) {

    Dummy d[] = { new Dummy(), new Dummy(), null, null, new Dummy() };

    /*
    ** Test the "new" bytecode;
    */

    jt.check((com_new_1()), com_new_1(), "ObjectTest.com_new_1()");
    jt.check((com_new_2(3)).intValue(), (com_new_2(3)).intValue(), "ObjectTest.com_new_2(3)");

    /*
    ** Test the "getstatic" bytecode:
    */

    for (x = 0; x < 5; x++) {
      jt.check(int_getstatic_1(), com_getstatic_1(), "ObjectTest.com_getstatic_1()");
      jt.check(int_getstatic_2(), com_getstatic_2(), "ObjectTest.com_getstatic_2()");
    }
 
    // Exceptions:
    for (int i = 0; i < 5; i++) {
      jt.check(int_getstatic_exception(d[i]), com_getstatic_exception(d[i]), "ObjectTest.com_getstatic_exception("+ d[i] +")");
    }

    /*
    ** Test the "getfield" bytecode:
    */
    
    for (y = 0; y < 5; y++) {
      jt.check(int_getfield_1(), com_getfield_1(), "ObjectTest.com_getfield_1()");
      jt.check(int_getfield_2(), com_getfield_2(), "ObjectTest.com_getfield_2()");
    }

    // Exceptions:
    for (int i = 0; i < 5; i++) {
      jt.check(int_getfield_exception(d[i]), com_getfield_exception(d[i]), "ObjectTest.com_getfield_exception("+ d[i] +")");
    }

    /*
    ** Test the "putstatic" bytecode:
    */
 
    for (int i = 0; i < 5; i++) {
      jt.check(int_putstatic_1(i), com_putstatic_1(i), "ObjectTest.com_putstatic_1("+ i +")");
      jt.check(int_putstatic_2(new Dummy(i)), com_putstatic_2(new Dummy(i)), "ObjectTest.com_putstatic_2("+ new Dummy(i) +")");
    }

    /*
    ** Test the "putfield" bytecode:
    */
 
    for (int i = 0; i < 5; i++) {
      jt.check(int_putfield_1(i), com_putfield_1(i), "ObjectTest.com_putfield_1("+ i +")");
      jt.check(int_putfield_2(new Dummy(i)), com_putfield_2(new Dummy(i)), "ObjectTest.com_putfield_2("+ new Dummy(i) +")");
    }

    /*
    ** Test the "instanceof" bytecode:
    */
 
    jt.check((int_instanceof_1(new Integer(5))), com_instanceof_1(new Integer(5)), "ObjectTest.com_instanceof_1()");
    jt.check((int_instanceof_1(new Long(5))), com_instanceof_1(new Long(5)), "ObjectTest.com_instanceof_1()");
    jt.check((int_instanceof_1(new Float(5))), com_instanceof_1(new Float(5)), "ObjectTest.com_instanceof_1()");
    jt.check((int_instanceof_1(new Double(5))), com_instanceof_1(new Double(5)), "ObjectTest.com_instanceof_1()");
    jt.check((int_instanceof_1(new Integer(5))), com_instanceof_1(new Integer(5)), "ObjectTest.com_instanceof_1()");

    /*
    ** Test the "checkcast" bytecode:
    */

    for (int i = 0; i < 5; i++) {
      jt.check(int_checkcast_2(i), com_checkcast_2(i), "ObjectTest.com_checkcast_2("+ i +")");
    }

    /*
    // NOTE: disabled for now because they throw exceptions ...
    jt.check((int_checkcast_1(new Integer(5))), com_checkcast_1(new Integer(5)), "ObjectTest.com_checkcast_1()");
    jt.check((int_checkcast_1(new Long(5))), com_checkcast_1(new Long(5)), "ObjectTest.com_checkcast_1()");
    jt.check((int_checkcast_1(new Float(5))), com_checkcast_1(new Float(5)), "ObjectTest.com_checkcast_1()");
    jt.check((int_checkcast_1(new Double(5))), com_checkcast_1(new Double(5)), "ObjectTest.com_checkcast_1()");
    jt.check((int_checkcast_1(new Integer(5))), com_checkcast_1(new Integer(5)), "ObjectTest.com_checkcast_1()");
    */
 
    /*
    ** Test the "monitorenter" and "monitorexit" bytecode:
    */

    jt.check(com_monitorenter(), com_monitorenter(), "ObjectTest.com_monitorenter()");
   
    /*
    ** Test the way local references are being dealt with: 
    */

    jt.check(com_locals_1(2500), com_locals_1(2500), "ProblemTest.com_locals("+ 2500 +")");
    jt.check(com_locals_2(5000), com_locals_2(5000), "ProblemTest.com_locals("+ 5000 +")");
  }
}

