/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: bytecode.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include <string.h>

#include "methods.h"
#include "constant.h"
#include "bytecode.h"
#include "arena.h"

j_Bytecode bytecodes[] = {
  { "nop",				0x00, 0, 0, 0, (OPCODE_FLAGS) },
  { "aconst_null",			0x01, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_m1",			0x02, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_0",				0x03, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_1",				0x04, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_2",				0x05, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_3",				0x06, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_4",				0x07, 0, 0, 1, (OPCODE_FLAGS) },
  { "iconst_5",				0x08, 0, 0, 1, (OPCODE_FLAGS) },
  { "lconst_0",				0x09, 0, 0, 1, (OPCODE_FLAGS) },
  { "lconst_1",				0x0a, 0, 0, 1, (OPCODE_FLAGS) },
  { "fconst_0",				0x0b, 0, 0, 1, (OPCODE_FLAGS) },
  { "fconst_1",				0x0c, 0, 0, 1, (OPCODE_FLAGS) },
  { "fconst_2",				0x0d, 0, 0, 1, (OPCODE_FLAGS) },
  { "dconst_0",				0x0e, 0, 0, 1, (OPCODE_FLAGS) },
  { "dconst_1",				0x0f, 0, 0, 1, (OPCODE_FLAGS) },
  { "bipush",				0x10, 1, 0, 1, (OPCODE_FLAGS) },
  { "sipush",				0x11, 2, 0, 1, (OPCODE_FLAGS) },
  { "ldc",				0x12, 1, 0, 1, (OPCODE_FLAGS) },
  { "ldc_w",				0x13, 2, 0, 1, (OPCODE_FLAGS) },
  { "ldc2_w",				0x14, 2, 0, 1, (OPCODE_FLAGS) },
  { "iload",				0x15, 1, 0, 1, (OPCODE_FLAGS) },
  { "lload",				0x16, 1, 0, 1, (OPCODE_FLAGS) },
  { "fload",				0x17, 1, 0, 1, (OPCODE_FLAGS) },
  { "dload",				0x18, 1, 0, 1, (OPCODE_FLAGS) },
  { "aload",				0x19, 1, 0, 1, (OPCODE_FLAGS) },
  { "iload_0",				0x1a, 0, 0, 1, (OPCODE_FLAGS) },
  { "iload_1",				0x1b, 0, 0, 1, (OPCODE_FLAGS) },
  { "iload_2",				0x1c, 0, 0, 1, (OPCODE_FLAGS) },
  { "iload_3",				0x1d, 0, 0, 1, (OPCODE_FLAGS) },
  { "lload_0",				0x1e, 0, 0, 1, (OPCODE_FLAGS) },
  { "lload_1",				0x1f, 0, 0, 1, (OPCODE_FLAGS) },
  { "lload_2",				0x20, 0, 0, 1, (OPCODE_FLAGS) },
  { "lload_3",				0x21, 0, 0, 1, (OPCODE_FLAGS) },
  { "fload_0",				0x22, 0, 0, 1, (OPCODE_FLAGS) },
  { "fload_1",				0x23, 0, 0, 1, (OPCODE_FLAGS) },
  { "fload_2",				0x24, 0, 0, 1, (OPCODE_FLAGS) },
  { "fload_3",				0x25, 0, 0, 1, (OPCODE_FLAGS) },
  { "dload_0",				0x26, 0, 0, 1, (OPCODE_FLAGS) },
  { "dload_1",				0x27, 0, 0, 1, (OPCODE_FLAGS) },
  { "dload_2",				0x28, 0, 0, 1, (OPCODE_FLAGS) },
  { "dload_3",				0x29, 0, 0, 1, (OPCODE_FLAGS) },
  { "aload_0",				0x2a, 0, 0, 1, (OPCODE_FLAGS) },
  { "aload_1",				0x2b, 0, 0, 1, (OPCODE_FLAGS) },
  { "aload_2",				0x2c, 0, 0, 1, (OPCODE_FLAGS) },
  { "aload_3",				0x2d, 0, 0, 1, (OPCODE_FLAGS) },
  { "iaload",				0x2e, 0, 2, 1, (OPCODE_FLAGS) },
  { "laload",				0x2f, 0, 2, 1, (OPCODE_FLAGS) },
  { "faload",				0x30, 0, 2, 1, (OPCODE_FLAGS) },
  { "daload",				0x31, 0, 2, 1, (OPCODE_FLAGS) },
  { "aaload",				0x32, 0, 2, 1, (OPCODE_FLAGS) },
  { "baload",				0x33, 0, 2, 1, (OPCODE_FLAGS) },
  { "caload",				0x34, 0, 2, 1, (OPCODE_FLAGS) },
  { "saload",				0x35, 0, 2, 1, (OPCODE_FLAGS) },
  { "istore",				0x36, 1, 1, 0, (OPCODE_FLAGS) },
  { "lstore",				0x37, 1, 1, 0, (OPCODE_FLAGS) },
  { "fstore",				0x38, 1, 1, 0, (OPCODE_FLAGS) },
  { "dstore",				0x39, 1, 1, 0, (OPCODE_FLAGS) },
  { "astore",				0x3a, 1, 1, 0, (OPCODE_FLAGS) },
  { "istore_0",				0x3b, 0, 1, 0, (OPCODE_FLAGS) },
  { "istore_1",				0x3c, 0, 1, 0, (OPCODE_FLAGS) },
  { "istore_2",				0x3d, 0, 1, 0, (OPCODE_FLAGS) },
  { "istore_3",				0x3e, 0, 1, 0, (OPCODE_FLAGS) },
  { "lstore_0",				0x3f, 0, 1, 0, (OPCODE_FLAGS) },
  { "lstore_1",				0x40, 0, 1, 0, (OPCODE_FLAGS) },
  { "lstore_2",				0x41, 0, 1, 0, (OPCODE_FLAGS) },
  { "lstore_3",				0x42, 0, 1, 0, (OPCODE_FLAGS) },
  { "fstore_0",				0x43, 0, 1, 0, (OPCODE_FLAGS) },
  { "fstore_1",				0x44, 0, 1, 0, (OPCODE_FLAGS) },
  { "fstore_2",				0x45, 0, 1, 0, (OPCODE_FLAGS) },
  { "fstore_3",				0x46, 0, 1, 0, (OPCODE_FLAGS) },
  { "dstore_0",				0x47, 0, 1, 0, (OPCODE_FLAGS) },
  { "dstore_1",				0x48, 0, 1, 0, (OPCODE_FLAGS) },
  { "dstore_2",				0x49, 0, 1, 0, (OPCODE_FLAGS) },
  { "dstore_3",				0x4a, 0, 1, 0, (OPCODE_FLAGS) },
  { "astore_0",				0x4b, 0, 1, 0, (OPCODE_FLAGS) },
  { "astore_1",				0x4c, 0, 1, 0, (OPCODE_FLAGS) },
  { "astore_2",				0x4d, 0, 1, 0, (OPCODE_FLAGS) },
  { "astore_3",				0x4e, 0, 1, 0, (OPCODE_FLAGS) },
  { "iastore",				0x4f, 0, 3, 0, (OPCODE_FLAGS) },
  { "lastore",				0x50, 0, 3, 0, (OPCODE_FLAGS) },
  { "fastore",				0x51, 0, 3, 0, (OPCODE_FLAGS) },
  { "dastore",				0x52, 0, 3, 0, (OPCODE_FLAGS) },
  { "aastore",				0x53, 0, 3, 0, (OPCODE_FLAGS) },
  { "bastore",				0x54, 0, 3, 0, (OPCODE_FLAGS) },
  { "castore",				0x55, 0, 3, 0, (OPCODE_FLAGS) },
  { "sastore",				0x56, 0, 3, 0, (OPCODE_FLAGS) },
  { "pop",				0x57, 0, 1, 0, (OPCODE_FLAGS) },
  { "pop2",				0x58, 0, 2, 0, (OPCODE_FLAGS) },
  { "dup",				0x59, 0, 1, 2, (OPCODE_FLAGS) },
  { "dup_x1",				0x5a, 0, 2, 3, (OPCODE_FLAGS) },
  { "dup_x2",				0x5b, 0, 3, 4, (OPCODE_FLAGS) },
  { "dup2",				0x5c, 0, 2, 4, (OPCODE_FLAGS) },
  { "dup2_x1",				0x5d, 0, 3, 4, (OPCODE_FLAGS) },
  { "dup2_x2",				0x5e, 0, 4, 6, (OPCODE_FLAGS) },
  { "swap",				0x5f, 0, 2, 2, (OPCODE_FLAGS) },
  { "iadd",				0x60, 0, 2, 1, (OPCODE_FLAGS) },
  { "ladd",				0x61, 0, 2, 1, (OPCODE_FLAGS) },
  { "fadd",				0x62, 0, 2, 1, (OPCODE_FLAGS) },
  { "dadd",				0x63, 0, 2, 1, (OPCODE_FLAGS) },
  { "isub",				0x64, 0, 2, 1, (OPCODE_FLAGS) },
  { "lsub",				0x65, 0, 2, 1, (OPCODE_FLAGS) },
  { "fsub",				0x66, 0, 2, 1, (OPCODE_FLAGS) },
  { "dsub",				0x67, 0, 2, 1, (OPCODE_FLAGS) },
  { "imul",				0x68, 0, 2, 1, (OPCODE_FLAGS) },
  { "lmul",				0x69, 0, 2, 1, (OPCODE_FLAGS) },
  { "fmul",				0x6a, 0, 2, 1, (OPCODE_FLAGS) },
  { "dmul",				0x6b, 0, 2, 1, (OPCODE_FLAGS) },
  { "idiv",				0x6c, 0, 2, 1, (OPCODE_FLAGS) },
  { "ldiv",				0x6d, 0, 2, 1, (OPCODE_FLAGS) },
  { "fdiv",				0x6e, 0, 2, 1, (OPCODE_FLAGS) },
  { "ddiv",				0x6f, 0, 2, 1, (OPCODE_FLAGS) },
  { "irem",				0x70, 0, 2, 1, (OPCODE_FLAGS) },
  { "lrem",				0x71, 0, 2, 1, (OPCODE_FLAGS) },
  { "frem",				0x72, 0, 2, 1, (OPCODE_FLAGS) },
  { "drem",				0x73, 0, 2, 1, (OPCODE_FLAGS) },
  { "ineg",				0x74, 0, 1, 1, (OPCODE_FLAGS) },
  { "lneg",				0x75, 0, 1, 1, (OPCODE_FLAGS) },
  { "fneg",				0x76, 0, 1, 1, (OPCODE_FLAGS) },
  { "dneg",				0x77, 0, 1, 1, (OPCODE_FLAGS) },
  { "ishl",				0x78, 0, 2, 1, (OPCODE_FLAGS) },
  { "lshl",				0x79, 0, 2, 1, (OPCODE_FLAGS) },
  { "ishr",				0x7a, 0, 2, 1, (OPCODE_FLAGS) },
  { "lshr",				0x7b, 0, 2, 1, (OPCODE_FLAGS) },
  { "iushr",				0x7c, 0, 2, 1, (OPCODE_FLAGS) },
  { "lushr",				0x7d, 0, 2, 1, (OPCODE_FLAGS) },
  { "iand",				0x7e, 0, 2, 1, (OPCODE_FLAGS) },
  { "land",				0x7f, 0, 2, 1, (OPCODE_FLAGS) },
  { "ior",				0x80, 0, 2, 1, (OPCODE_FLAGS) },
  { "lor",				0x81, 0, 2, 1, (OPCODE_FLAGS) },
  { "ixor",				0x82, 0, 2, 1, (OPCODE_FLAGS) },
  { "lxor",				0x83, 0, 2, 1, (OPCODE_FLAGS) },
  { "iinc",				0x84, 2, 0, 0, (OPCODE_FLAGS) },
  { "i2l",				0x85, 0, 1, 1, (OPCODE_FLAGS) },
  { "i2f",				0x86, 0, 1, 1, (OPCODE_FLAGS) },
  { "i2d",				0x87, 0, 1, 1, (OPCODE_FLAGS) },
  { "l2i",				0x88, 0, 1, 1, (OPCODE_FLAGS) },
  { "l2f",				0x89, 0, 1, 1, (OPCODE_FLAGS) },
  { "l2d",				0x8a, 0, 1, 1, (OPCODE_FLAGS) },
  { "f2i",				0x8b, 0, 1, 1, (OPCODE_FLAGS) },
  { "f2l",				0x8c, 0, 1, 1, (OPCODE_FLAGS) },
  { "f2d",				0x8d, 0, 1, 1, (OPCODE_FLAGS) },
  { "d2i",				0x8e, 0, 1, 1, (OPCODE_FLAGS) },
  { "d2l",				0x8f, 0, 1, 1, (OPCODE_FLAGS) },
  { "d2f",				0x90, 0, 1, 1, (OPCODE_FLAGS) },
  { "i2b",				0x91, 0, 1, 1, (OPCODE_FLAGS) },
  { "i2c",				0x92, 0, 1, 1, (OPCODE_FLAGS) },
  { "i2s",				0x93, 0, 1, 1, (OPCODE_FLAGS) },
  { "lcmp",				0x94, 0, 2, 1, (OPCODE_FLAGS) },
  { "fcmpl",				0x95, 0, 2, 1, (OPCODE_FLAGS) },
  { "fcmpg",				0x96, 0, 2, 1, (OPCODE_FLAGS) },
  { "dcmpl",				0x97, 0, 2, 1, (OPCODE_FLAGS) },
  { "dcmpg",				0x98, 0, 2, 1, (OPCODE_FLAGS) },
  { "ifeq",				0x99, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ifne",				0x9a, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "iflt",				0x9b, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ifge",				0x9c, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ifgt",				0x9d, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ifle",				0x9e, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmpeq",			0x9f, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmpne",			0xa0, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmplt",			0xa1, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmpge",			0xa2, 2, 0, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmpgt",			0xa3, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_icmple",			0xa4, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_acmpeq",			0xa5, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "if_acmpne",			0xa6, 2, 2, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "goto",				0xa7, 2, 0, 0, (OPCODE_FLAGS | OPCODE_JUMP) },
  { "jsr",				0xa8, 2, 0, 1, (OPCODE_FLAGS | OPCODE_JUMP) },
  { "ret",				0xa9, 1, 0, 0, (OPCODE_FLAGS | OPCODE_JUMP) },
  { "tableswitch",			0xaa, 0, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "lookupswitch",			0xab, 0, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ireturn",				0xac, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "lreturn",				0xad, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "freturn",				0xae, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "dreturn",				0xaf, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "areturn",				0xb0, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "vreturn",				0xb1, 0, 0, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "getstatic",			0xb2, 2, 0, 1, (OPCODE_FLAGS) },
  { "putstatic",			0xb3, 2, 1, 0, (OPCODE_FLAGS) },
  { "getfield",				0xb4, 2, 1, 1, (OPCODE_FLAGS) },
  { "putfield",				0xb5, 2, 2, 0, (OPCODE_FLAGS) },
  { "invokevirtual",			0xb6, 2, 0, 0, (OPCODE_FLAGS) },
  { "invokespecial",			0xb7, 2, 0, 0, (OPCODE_FLAGS) },
  { "invokestatic",			0xb8, 2, 0, 0, (OPCODE_FLAGS) },
  { "invokeinterface",			0xb9, 4, 0, 0, (OPCODE_FLAGS) },
  { "in_new",   			0xba, 2, 0, 1, (OPCODE_FLAGS) },
  { "new",				0xbb, 2, 0, 1, (OPCODE_FLAGS) },
  { "newarray",				0xbc, 1, 1, 1, (OPCODE_FLAGS) },
  { "anewarray",			0xbd, 2, 1, 1, (OPCODE_FLAGS) },
  { "arraylength",			0xbe, 0, 1, 1, (OPCODE_FLAGS) },
  { "athrow",				0xbf, 0, 1, 0, (OPCODE_FLAGS | OPCODE_EXIT) },
  { "checkcast",			0xc0, 2, 0, 0, (OPCODE_FLAGS) },
  { "instanceof",			0xc1, 2, 1, 1, (OPCODE_FLAGS) },
  { "monitorenter",			0xc2, 0, 1, 0, (OPCODE_FLAGS) },
  { "monitorexit",			0xc3, 0, 1, 0, (OPCODE_FLAGS) },
  { "wide",				0xc4, 0, 0, 0, (OPCODE_FLAGS) },
  { "multianewarray",			0xc5, 3, 1, 1, (OPCODE_FLAGS) },
  { "ifnull",				0xc6, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "ifnonnull",			0xc7, 2, 1, 0, (OPCODE_FLAGS | OPCODE_JUMP | OPCODE_SPLIT) },
  { "goto_w",				0xc8, 4, 0, 0, (OPCODE_FLAGS | OPCODE_JUMP) },
  { "jsr_w",				0xc9, 4, 0, 0, (OPCODE_FLAGS | OPCODE_JUMP) },
  { "breakpoint",			0xca, 0, 0, 0, (OPCODE_FLAGS) },
  { "in_getstatic",			0xcb, 2, 0, 1, (OPCODE_FLAGS) },
  { "in_putstatic",			0xcc, 2, 1, 0, (OPCODE_FLAGS) },
  { "in_getfield",			0xcd, 2, 1, 1, (OPCODE_FLAGS) },
  { "in_putfield",			0xce, 2, 2, 0, (OPCODE_FLAGS) },
  { "in_invokestatic",			0xcf, 2, 0, 0, (OPCODE_FLAGS) },
  { "in_invokenonvirtual",	        0xd0, 2, 0, 0, (OPCODE_FLAGS) },
  { "in_invokesuper",			0xd1, 2, 0, 0, (OPCODE_FLAGS) },
  { "in_invokevirtual",			0xd2, 2, 0, 0, (OPCODE_FLAGS) },
  { "putstatic_quick",			0xd3, 0, 0, 0, (OPCODE_FLAGS) },
  { "getstatic2_quick",			0xd4, 0, 0, 0, (OPCODE_FLAGS) },
  { "putstatic2_quick",			0xd5, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokevirtual_quick",		0xd6, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokenonvirtual_quick",		0xd7, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokesuper_quick",		0xd8, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokestatic_quick",		0xd9, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokeinterface_quick",		0xda, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokevirtualobject_quick",	0xdb, 0, 0, 0, (OPCODE_FLAGS) },
  { "none",	                        0xdc, 0, 0, 0, (OPCODE_FLAGS) },
  { "new_quick",			0xdd, 0, 0, 0, (OPCODE_FLAGS) },
  { "anewarray_quick",			0xde, 0, 0, 0, (OPCODE_FLAGS) },
  { "multianewarray_quick",		0xdf, 0, 0, 0, (OPCODE_FLAGS) },
  { "checkcast_quick",			0xe0, 0, 0, 0, (OPCODE_FLAGS) },
  { "instanceof_quick",			0xe1, 0, 0, 0, (OPCODE_FLAGS) },
  { "invokevirtual_quick_w",		0xe2, 0, 0, 0, (OPCODE_FLAGS) },
  { "getfield_quick_w",			0xe3, 0, 0, 0, (OPCODE_FLAGS) },
  { "putfield_quick_w",			0xe4, 0, 0, 0, (OPCODE_FLAGS) },
  { "iseq",		        	0xe5, 7, 1, 1, (OPCODE_FLAGS) },
  { "isne",			        0xe6, 7, 1, 1, (OPCODE_FLAGS) },
  { "no_code03",			0xe7, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code04",			0xe8, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code05",			0xe9, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code06",			0xea, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code07",			0xeb, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code08",			0xec, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code09",			0xed, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code10",			0xee, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code11",			0xef, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code12",			0xf0, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code13",			0xf1, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code14",			0xf2, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code15",			0xf3, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code16",			0xf4, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code17",			0xf5, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code18",			0xf6, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code19",			0xf7, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code20",			0xf8, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code21",			0xf9, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code22",			0xfa, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code23",			0xfb, 0, 0, 0, (OPCODE_FLAGS) },
  { "no_code24",			0xfc, 0, 0, 0, (OPCODE_FLAGS) },
  { "fail",		        	0xfd, 0, 0, 0, (OPCODE_FLAGS) },    
  { "impdep1",				0xfe, 0, 0, 0, (OPCODE_FLAGS) },
  { "impdep2",				0xff, 0, 0, 0, (OPCODE_FLAGS) }
};

void replace_idioms(int pc) {
  
  unsigned char idiom_iseq[] = { 8, ifeq, 0x0, 0x7, iconst_0, goto_n, 0x0, 0x4, iconst_1 };
  unsigned char idiom_isne[] = { 8, ifne, 0x0, 0x7, iconst_0, goto_n, 0x0, 0x4, iconst_1 };
 
  if (memcmp(idiom_iseq + 1, jm->bytecode + pc, idiom_iseq[0]) == 0) {
    jm->bytecode[pc + 0] = iseq;
    jm->bytecode[pc + 1] = fail;
    jm->bytecode[pc + 2] = fail;
    jm->bytecode[pc + 3] = fail;
    jm->bytecode[pc + 4] = fail;
    jm->bytecode[pc + 5] = fail;
    jm->bytecode[pc + 6] = fail;
    jm->bytecode[pc + 7] = fail;
      // NOTE: we don't clear the other bytecodes in the sequence because we might jump to it from elsewhere (double join point)
  }
  else if (memcmp(idiom_isne + 1, jm->bytecode + pc, idiom_isne[0]) == 0) {
    jm->bytecode[pc + 0] = isne;
    jm->bytecode[pc + 1] = fail;
    jm->bytecode[pc + 2] = fail;
    jm->bytecode[pc + 3] = fail;
    jm->bytecode[pc + 4] = fail;
    jm->bytecode[pc + 5] = fail;
    jm->bytecode[pc + 6] = fail;
    jm->bytecode[pc + 7] = fail;
      // NOTE: we don't clear the other bytecodes in the sequence because we might jump to it from elsewhere (double join point)
  }
}

static void bytecode_add_target(int pc) {
  
  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }

  /*
  ** A counter is associated with each bytecode instruction and for
  ** each bytecode instruction that represents a branch target, the 
  ** counter of each target instruction is incremented by one except
  ** for the subsequent fall-through instruction (pc == jm->curr_pc). 
  */
  
  if (pc != jm->curr_pc) {
    jm->join[pc] += 1;
  }

  /*
  ** Mark the bytecode instruction as a basic block entry.
  */

  jm->data[pc] |= OPCODE_ENTRY;
}

static void bytecode_del_target(int pc) {
  
  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }

  /*
  ** When a bytecode instruction is encountered that is not a 
  ** fall-through instruction, such as an exit point (athrow,
  ** ireturn, etc) or unconditional branch (goto), the counter
  ** of the subsequent instruction is decremented by one by
  ** calling "bytecode_del_target()".
  */
  
  jm->join[pc] -= 1;
}

int bytecode_is_entry(int pc) {

  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }
  
  if (jm->data[pc] & OPCODE_ENTRY) {
    return 1;
  }
  else {
    return 0;
  }
}

int bytecode_is_split(int pc) {

  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }
 
  if (bytecodes[jm->code[pc]].flags & OPCODE_SPLIT) {
    return 1;
  }
  else {
    return 0;
  }
}

int bytecode_is_jump(int pc) {

  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }
 
  if (bytecodes[jm->code[pc]].flags & OPCODE_JUMP) {
    return 1;
  }
  else {
    return 0;
  }
}
int bytecode_is_join(int pc) {

  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }
  
  if (jm->join[pc] > 0) {
    return 1;
  }
  else {
    return 0;
  }
}

int bytecode_is_exit(int pc) {

  if (pc < 0 || pc > jm->length) {
    jabort("invalid program counter\n");
  }
  
  if (bytecodes[jm->code[pc]].flags & OPCODE_EXIT) {
    return 1;
  }
  else {
    return 0;
  }
}


int bytecode_tos_consumed(int pc) {

  int i = pc + 1;

  while (i < jm->length) {
    if (jm->code[i]) {
      /*
      ** If the current bytecode represents a join point:
      */

      if (jm->join[i]) {
        return 0;
      }

      /*
      ** If the current bytecode has a lower stack depth: that is,
      ** when depth[pc]_max is larger than depth[i]_min, where "max"
      ** is the state after both pops and pushes and "min" the state
      ** after all pops (but before all pushes).
      */

      if (jm->depth[pc] > jm->depth[i] - bytecodes[jm->code[i]].p_stack) {
          // TODO: the above is not 100% accurate because some opcodes have a variable number of pushes.
        return 1;
      }
      
      /*
      ** If the current bytecode represents an exit point:
      */

      if (bytecodes[jm->code[i]].flags & OPCODE_EXIT) {
        return 1;
      }

      /*
      ** If the current bytecode represents change in program flow
      ** (a jump but not necessarily a branch):
      */

      if (bytecodes[jm->code[i]].flags & OPCODE_JUMP) {
        return 0;        
      }
    }
 
    i++;       
  }

  return 0;
}

void bytecode(void) {
  
  unsigned char code;
  w_method method;
  int depth;
  int num;
  int def;
  int low;
  int hgh;
  int i;

  /*
  ** This function will loop over the bytecodes to see if they are all
  ** supported.
  */
  
  jm->inst_pc = 0;
  jm->curr_pc = 0;
  depth = 0;

  // allocate memory for the join points:
  jm->join = arena_allocate(jm->length * sizeof(char));
  jm->code = arena_allocate(jm->length * sizeof(char));
  jm->data = arena_allocate(jm->length * sizeof(char));
  jm->depth = arena_allocate(jm->length * sizeof(int));
  
  while (jm->failed == 0 && jm->inst_pc < jm->length) {

    /*
    ** Check whether there are common bytecode sequences or idioms that
    ** can be replaaced by shorter/faster alternatives.
    */

    replace_idioms(jm->curr_pc);

    /*
    ** Get the current instruction.
    */
    
    code = jm->bytecode[jm->curr_pc++];
    depth = depth - bytecodes[code].c_stack + bytecodes[code].p_stack;
   
    j_flarp(green, "pc = %d, bytecode = %s, stack depth = %d\n", jm->inst_pc, bytecode_get_name(code), depth);

    /*
    ** Update the code buffer:
    */
 
    jm->code[jm->inst_pc] = code;

    /*
    ** Update the size buffer:
    */
  
    jm->depth[jm->inst_pc] = depth;

    /*
    ** Update the join buffer:
    */

    switch (code) {
      
      case (iseq): // NOTE: we don't jump over the "iseq" bytecode sequence because we might jump to it from elsewhere (double join point)
      case (isne): // NOTE: we don't jump over the "isne" bytecode sequence because we might jump to it from elsewhere (double join point)
        jm->curr_pc += 7;
        break;
        
      /*
      ** Zero byte argument:
      */
      
      case (athrow):
      case (ireturn):
      case (freturn):
      case (areturn):
      case (vreturn):
        bytecode_del_target(jm->curr_pc);
        break;
        
      /*
      ** One byte argument:
      */ 
        
      case (ldc):
      case (bipush):
      case (iload):
      case (fload):
      case (aload):
      case (istore):
      case (fstore):
      case (astore):
      case (newarray):
        jm->curr_pc += 1;
        break;
      
      /*
      ** Two byte argument:
      */
     
      case (checkcast):
      case (instanceof):   
      case (sipush):
      case (ldc_w):
      case (getstatic):
      case (in_getstatic):
      case (putstatic):
      case (in_putstatic):
      case (getfield):
      case (in_getfield):
      case (putfield):
      case (in_putfield):
      case (anewarray):
      case (iinc):
      case (in_new):
      case (new):
        jm->curr_pc += 2;
        break;
  
      case (invokevirtual):
      case (in_invokevirtual):
      case (invokespecial):
      case (in_invokenonvirtual):
      case (in_invokesuper):
      case (invokestatic):
      case (in_invokestatic):
        method = getMethodConstant(jm->method->spec.declaring_clazz, bytecode_get_short());
        depth = depth - method->exec.arg_i + method->exec.return_i; 
        break;
        
      case (invokeinterface):
        method = getIMethodConstant(jm->method->spec.declaring_clazz, bytecode_get_short());
        depth = depth - method->exec.arg_i + method->exec.return_i; 
        jm->curr_pc += 2; // skip over the useless <nargs> and zero byte
        break;
      
      case (if_icmpeq):        
      case (if_icmpne):
      case (if_icmplt):
      case (if_icmpge):
      case (if_icmpgt):
      case (if_icmple):
      case (if_acmpeq):        
      case (if_acmpne):
      case (ifnonnull):       
      case (ifnull):
      case (ifeq):
      case (ifne):
      case (iflt):
      case (ifge):
      case (ifgt):
      case (ifle):
        bytecode_add_target(jm->inst_pc + bytecode_get_short());
        bytecode_add_target(jm->curr_pc);
          // NOTE: branch targets are not necessarily join points, yet every join join is a branch target.
        break;
 
      case (goto_n):
        bytecode_add_target(jm->inst_pc + bytecode_get_short());
        bytecode_del_target(jm->curr_pc);
          // NOTE: branch targets are not necessarily join points, yet every join join is a branch target.
        break;
        
      /*
      ** Three byte argument:
      */

      case (multianewarray):
        jm->curr_pc += 3;
        break;

      /*
      ** Four byte argument:
      */
 
      case (goto_w):
        bytecode_add_target(jm->inst_pc + bytecode_get_word());
        bytecode_del_target(jm->curr_pc);
          // NOTE: branch targets are not necessarily join points, yet every join join is a branch target.
        break;
        
      /*
      ** Unsupported bytecodes:
      */
     
      case (i2d):
      case (i2l):
      case (l2d):
      case (l2i):
      case (l2f):
      case (f2l):
      case (f2d):
      case (d2i):
      case (d2l):
      case (d2f):
      case (lor):
      case (lxor):
      case (land):
      case (ishl):
      case (ishr):
      case (lshl):
      case (lshr):
      case (ladd):
      case (dadd):
      case (lsub):
      case (dsub):
      case (lmul):
      case (dmul):
      case (ldiv):
      case (ddiv):
      case (lrem):
      case (drem):
      case (lneg):
      case (dneg):
      case (iushr):
      case (lushr):
      case (lcmp):
      case (dcmpl):
      case (dcmpg):
      case (lload):
      case (dload):
      case (dload_0):
      case (dload_1):
      case (dload_2):
      case (dload_3):
      case (lload_0):
      case (lload_1):
      case (lload_2):
      case (lload_3):
      case (dstore_0):
      case (dstore_1):
      case (dstore_2):
      case (dstore_3):
      case (lstore_0):
      case (lstore_1):
      case (lstore_2):
      case (lstore_3):
      case (baload):
      case (laload):
      case (daload):
      case (lstore):
      case (dstore):
      case (bastore): 
      case (lastore):
      case (dastore):
      case (dconst_1): 
      case (lconst_1): 
      case (ret):
      case (jsr):
      case (jsr_w):
      case (ldc2_w):
      case (monitorenter):
      case (monitorexit):
      case (dreturn):
      case (lreturn):
      case (dup_x1):
      case (dup_x2):
      case (dup2_x1):
      case (dup2_x2):
        jfail("bytecode '%s' is not supported yet\n", bytecode_get_name(code));
        break;
      
      case (tableswitch):
        jfail("bytecode '%s' is not supported yet\n", bytecode_get_name(code));
          // NOTE: temporary disabled
          // TODO: use bytecode_add_target()
        
        // word align the operands:
        jm->curr_pc = (jm->inst_pc + 4) & -4;
        
        // retrieve the default branch offset, the low endpoint and the high endpoint:
        def = bytecode_get_word();
        low = bytecode_get_word();
        hgh = bytecode_get_word();

        // skip each branch:
        for (i = 0; i < hgh - low + 1; i++) {
          jm->curr_pc += 4;
        }

        break;
       
      case (lookupswitch):
        jfail("bytecode '%s' is not supported yet\n", bytecode_get_name(code));
          // NOTE: temporary disabled
          // TODO: use bytecode_add_target()
        
        // word align the operands:
        jm->curr_pc = (jm->inst_pc + 4) & -4;

        // retrieve the default branch offset and the number of branch/offset pairs:
        def = bytecode_get_word();
        num = bytecode_get_word();

        // skip each branch/offset pair:
        for (i = 0; i < num; i++) {
          jm->curr_pc += 8;
        }
        
        break;

      case (wide):
        code = jm->bytecode[jm->curr_pc++];

        switch (code) {

          /*
          ** Two byte argument:
          */
          
          case (iload):
          case (fload):
          case (aload):
          case (istore):
          case (fstore):
          case (astore):
            jm->curr_pc += 2;
            break;
            
          case (iinc):
            jm->curr_pc += 4;
            break;

          /*
          ** Unsupported bytecode:
          */
            
          case (dload):
          case (lload):
          case (dstore):
          case (lstore):
          case (ret):
            jfail("bytecode '%s' is not supported yet\n", bytecode_get_name(code));
            break;
        }
    }
    
    jm->inst_pc = jm->curr_pc;
  }
}

char * bytecode_get_name(unsigned int opcode) {
  return (char *)bytecodes[opcode].name;
}

