/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: compiler.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include "j-spot.h"
#include "arena.h"
#include "code-generator.h"
#include "register-allocator.h"
#include "statistics.h"
#include "assembler.h"
#include "bytecode.h"
#include "ir.h"
#include "is.h"

// Prototypes of functions exported by the emitted code generator:
j_state burg_label(j_node node);
void burg_walk(j_node node, int goal, int depth);
 
void compile(void) {

  j_state state;
  j_node node;
    
  /*
  ** Initialize the register allocator:
  */
  
  // allocate memory for the registers:
  jm->registers = arena_allocate(J_NUM_REGISTERS * sizeof(j_node));

  /*
  ** Reset the code emitter.
  */

  asm_init();

  /*
  ** Initialize the register allocator.
  */
 
  cg_init();

  /*
  ** Iterate over the specified method's DAGs and invoke the code
  ** generator to labeli, reduce and emit code for each DAG.
  */
  
  node = jm->start;
  
  while (node && !jm->failed) {

    j_flarp(yellow, "*  %q (%p) %q %q  [%q, %q]\n", node, node, node->node1, node->node2, node->outgoing[0], node->outgoing[1]);

    if (node->critical) {
      state = (j_state)burg_label(node);

      if (state) {
        woempa(9, "instruction %q with operands %q %q is live\n", node, node->node1, node->node2);

        if (ir_is_basic_block_entry(node) == j_explicit) {
          ra_basic_block_entry(node);
        }

        if (ir_is_basic_block_exit(node) == j_explicit) {
          /*
           ** If the node represents an "explicit basic block exit" such
           ** as a jump instruction for example, we spill life variables
           ** prior to emitting the jump instruction.
           */

          ra_basic_block_exit(node);
        }

        burg_walk(node, 1, 0);

        if (ir_is_basic_block_exit(node) == j_implicit) {
          /*
           ** If the node represents an "implicit basic block exit" such
           ** as a regular fall-through instruction followed by a join
           ** point, we spill life variables after having emitted the 
           ** instruction.
           */

          ra_basic_block_exit(node);
        }
      }
      else {
        jfail("the subject tree is unmatched: the tree may be corrupt or inconsistent with the grammar\n");
      }
    }
    else {
      woempa(9, "instruction %q with operands %q %q is dead\n", node, node->node1, node->node2);
    }

    node = node->next;
  }
  
  if (!jm->failed) {
    cg_prologue();
    cg_epilogue();
  }
}

