/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: constant-propagation.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include "j-spot.h"
#include "method.h"
#include "node.h"
#include "ir.h"

static void cp_make_constant(j_node node, int value) {

  int i;
  
  /*
  ** Nodes that are the target of different destinations (join points such as
  ** loop headers) are ignored for now.
  */

  if (node->src[1] == NULL)  {
   
    // Constant nodes have a lattice value of 1:
    node->lattice = 1;
    node->operator = j_constant;
    node->data.constant.value = value;

    if (node->node1) {
      node->node1->use -= 1;
      node->node1 = NULL;
    }
 
    if (node->node2) {
      node->node2->use -= 1;
      node->node2 = NULL;
    }
    
    for (i = 0; i < j_max_edges; i++) {
      if (node->dst[i]) { 
        cp_make_constant(node->dst[i], value);
      }
    } 
  }
}

static void cp_propagate(j_node instruction, j_node operand, va_list args) {

  j_node node;
  
  if (operand) {
    node = operand;
  }
  else {
    node = instruction;
  }
   
  if (node && !node->visited) {
    node->visited = 1;

    node_operands(node, cp_propagate);

    switch (node->operator) {
      case j_constant:
        cp_make_constant(node, node->data.constant.value);
        break;
      case j_assign:
        if (node->node1->operator == j_constant) {
          cp_make_constant(node->node2, node->node1->data.constant.value);
        }
        break;
      case j_sub:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value - node->node2->data.constant.value);
        }
        break;
      case j_add:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value + node->node2->data.constant.value);
        }
        break;
      case j_mul:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value * node->node2->data.constant.value);
        }
        break;
      case j_and:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value & node->node2->data.constant.value);
        }
        break;
      case j_xor:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value ^ node->node2->data.constant.value);
        }
        break;
      case j_or:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          cp_make_constant(node, node->node1->data.constant.value | node->node2->data.constant.value);
        }
        break;
      case j_lsh:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          jabort("lsh: perform constant propagation\n");
        }
        break;
      case j_rsh:
        if (node->node1->operator == j_constant && node->node2->operator == j_constant) {
          jabort("rsh: perform constant propagation\n");
        }
        break;
      case j_neg:
        if (node->node1->operator == j_constant) {
          cp_make_constant(node, - node->node1->data.constant.value);
        }
        break;
      case j_iseq:
        if (node->node1->operator == j_constant) {
          if (node->node1->data.constant.value == 0) {
            cp_make_constant(node, 1);
          }
          else {
            cp_make_constant(node, 0);
          }
        }
        break;
      case j_isne:
        if (node->node1->operator == j_constant) {
          if (node->node1->data.constant.value == 0) {
            cp_make_constant(node, 0);
          }
          else {
            cp_make_constant(node, 1);
          }
        }
        break;
      default:
        break;
    }
  }
}

void constant_propagation(void) {

  /*
  ** Demand-driven, recursive constant propagations.
  */
 
  j_node node = jm->start;

  while (node) {
    cp_propagate(node, NULL, NULL);
    node = node->next;
  }
}

