/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ct-hooks.c 3988 2003-06-05 13:07:52Z dbuytaert $
*/

// stdlib headers:
#include <sys/time.h>
#include <unistd.h>

// wonka headers:
#include "clazz.h"
#include "arrays.h"
#include "constant.h"
#include "interpreter.h"
#include "exception.h"
#include "loading.h"
#include "methods.h"
#include "wstrings.h"
#include "checks.h"
#include "fields.h"
#include "heap.h"

// j-spot headers:
#include "ir.h"
#include "dot.h"
#include "stabs.h"
#include "arena.h"
#include "statistics.h"
#include "translate.h"
#include "dominance.h"
#include "assembler.h"
#include "compiler.h"
#include "ct-hooks.h"
#include "bytecode.h"
#include "liveness.h"
#include "dead-code-elimination.h"
#include "check-elimination.h"
#include "constant-propagation.h"

extern char * jspot_call_range;
extern char * jspot_size_range;
extern char * jspot_name_range;
extern int jspot_compile;
extern int jspot_verbose;
extern int jspot_stabs;
extern int jspot_stats;
extern int jspot_asm;
extern int jspot_dot;

void ct_compile(w_method method) {

  w_thread thread = currentWonkaThread;
  static int count = 0;
  static int calls = 0;
  j_method m = NULL;
  struct timeval t1;
  struct timeval t2;
  int compile_time;
  int i;

  if (exceptionThrown(thread)) {
    return;
  }
  
  if (method->exec.dispatcher > 20) {  
    /*
    ** If we get here, then this is the first time we try compiling this
    ** method.  To avoid that we would try to compile it again we update
    ** the method's dispatcher table.  To understand how this works, see
    ** "interpreter.c" and "dispatcher.c".
    */

    method->exec.dispatcher = method->exec.dispatcher - 20;

    calls++;
  }
  else {
    /*
    ** The above if-clause occasionally triggers when two or more threads
    ** simultaneously return from the same profiled Java method.  No harm
    ** is done as all code path to this function are synchronized.
    */

    return;
  }
  
  /*
  ** Call ranges: compile methods in the specified call range(s) only.
  */
  
  if (jspot_call_range && !jspot_check_nrange(jspot_call_range, calls)) {
    return;
  }

  /*
  ** Size ranges: compile methods in the specified size range(s) only.
  */
  
  if (jspot_size_range && !jspot_check_nrange(jspot_size_range, method->exec.code_length)) {
    return;
  }
 
  /*
  ** Name ranges: compile methods in the specified name range(s) only.
  */
  
  if (jspot_name_range && !jspot_check_arange(jspot_name_range, method->spec.name) && !jspot_check_arange(jspot_name_range, method->spec.declaring_clazz->dotified)) {
    return;
  }
   
  /*
  ** Never compile methods with exceptions handlers for we don't have 
  ** native exception handlers yet.
  */

  if (method->exec.numExceptions) {
    joempa("failed compiling '%M': exception handlers are not supported yet\n", method);
    return;
  }

  /*
  ** Never compile methods whose bytecode size exceeds 2500 bytes (or
  ** some internal datastructures might overflow).
  */
 
  if (method->exec.code_length > 2000) {
    joempa("failed compiling '%M': bytecode size exceeds 2000 bytes\n", method);
    return;
  }
 
  /*
  ** Never compile methods that take longs or doubles as its
  ** arguments: these primitive types are not supported yet.
  */

  // check the parameters:
  for (i = 0; i < method->exec.nargs; i++) {
    if (method->spec.arg_types[i] == clazzLong || method->spec.arg_types[i] == clazzDouble) {
      joempa("failed compiling '%M': argument type not supported yet\n", method);
      return;
    }
  }
        
  // check the return type:
  if (method->spec.return_type == clazzLong || method->spec.return_type == clazzDouble) {
    joempa("failed compiling '%M': return type not supported yet\n", method);
    return;
  }

  /*
  ** Never compile methods that start with "int_".  This requirement is
  ** imposed by our test framework such that we can compare interpreted
  ** output with compiled output.
  */

  if (string_char(method->spec.name, 0) == 'i' && string_char(method->spec.name, 1) == 'n' && string_char(method->spec.name, 2) == 't' && string_char(method->spec.name, 3) == '_') { 
    return;
  }

  /*
  ** We can start compiling the method:
  */
  
  j_flarp(red, "compiling method '%M' (%p) ...\n", method, method);
  gettimeofday(&t1, 0);
  
  /*
  ** Create and initialize the method.
  */
   
  m = method_create(method);

  /*
  ** Never compile methods with unsupported bytecodes.
  */
 
  if (m->failed == 0) {
    bytecode(); 
  }
 
  /*
  ** Translate: convert bytecode to intermediate representation.
  */

  if (m->failed == 0) {
    translate();
  }
  
  /*
  ** Intermediate representation: build and optimize the intermediate
  ** representation.
  */
    
  if (m->failed == 0) {
    ir();
  }
 
  /*
  ** Calculate the DFS tree, domance frontier, SSA form, etc.
  */
  
  /*
  if (m->failed == 0) {
    dominance();
  }
  */
 
  /*
  ** Constant propagation.
  */
 
  if (m->failed == 0) {
    constant_propagation();
  }
 
  /*
  ** Constant propagation.
  */
 
  if (m->failed == 0) {
    check_elimination();
  }
 
  /*
  ** Dead code elimination.
  */
 
  if (m->failed == 0) {
    dead_code_elimination();
  }
 
  /*
  ** Liveness: compute liveness information.
  */
 
  if (m->failed == 0) {
    liveness();
  }
 
  /*
  ** Compile: convert the intermediate representation to machine code.
  */
    
  if (m->failed == 0) {
    compile();
  }   
  
  /*
  ** Register: export the generated machine code.
  */
 
  if (m->failed == 0) { 
    method->exec.function.void_fun = (w_void_fun)asm_export();
  }

  /*
  ** Register: hook the generated machine code into the VM.
  */
  
  if (m->failed == 0) { 
    method->flags |= METHOD_IS_COMPILED;
    method->return_address = (w_word)((int)method->exec.function.void_fun + asm_size_prologue() + asm_size_body());
    
    if (isSet(method->flags, ACC_STATIC)) {
      method->exec.dispatcher = 13;
    }
    else {
      method->exec.dispatcher = 5;
    }

    if (isNotSet(method->flags, ACC_SYNCHRONIZED)) {
      method->exec.dispatcher += 4;
    }

    if (method->spec.return_type == clazz_void) {
      method->exec.dispatcher += 3;
    }
    else if (method->exec.return_i == 2) {
      method->exec.dispatcher += 2;
    }
    else if (isSet(method->spec.return_type->flags, CLAZZ_IS_PRIMITIVE)) {
      method->exec.dispatcher += 1;
    }
  }
  
  
  /*
  ** Debug magic.
  */
 
  gettimeofday(&t2, 0);
  compile_time = t2.tv_sec * 1000000 + t2.tv_usec - t1.tv_sec * 1000000 - t1.tv_usec;
    
  if (m->failed == 0) {

    /*
    ** Create a C string with the method's name that can be used as a
    ** filename (dot) and as a symbol for use with a debugger (stabs).
    */
    
    // Write the intermediate representation to file for use with "dot".
    if (jspot_dot) {
      dot();
    }
   
    // Write the stabs debug information to file for use with "gdb".
    if (jspot_stabs) {
      stabs();
    }  
    
    // Write the generated assembly code to file.
    if (jspot_asm) {
      asm_dump();
    }
 
    // Write some debug information to screen.
    if (jspot_verbose) { 
      count = count + 1;
      joempa("%d/%d: '%M' (bytecode size: %d, machine code size: %d, compile time: %d usec)\n", count, calls, method, method->exec.code_length, asm_size_total(), compile_time);
    }
  }

  /*
  ** Statistics accounting
  */
  
  if (jspot_stats) {
    if (jm->failed == 0) {
      statistics_method(method->exec.code_length, asm_size_total(), arena_size(), compile_time);
    }
    else {
      statistics_method(method->exec.code_length, 0, arena_size(), compile_time);
    }
  }
 
  /*
  ** Free the allocated resources.
  */
  
  arena_release();
}

/*
** Compile-time interfaces:
*/

static j_type clazz2jtype(w_clazz clazz) {

  j_type type;

  if (clazz == clazz_boolean) {
    type = j_boolean;
  }
  else if (clazz == clazz_byte) {
    type = j_byte;
  }
  else if (clazz == clazz_char) {
    type = j_character;
  }
  else if (clazz == clazz_double) {
    type = j_double;
  }
  else if (clazz == clazz_float) {
    type = j_float;
  } 
  else if (clazz == clazz_int) {
    type = j_integer;
  }
  else if (clazz == clazz_long) {
    type = j_long;
  }
  else if (clazz == clazz_short) {
    type = j_short;
  }
  else if (clazz == clazz_void) {
    type = j_void;
  }
  else {
    type = j_address;
  }
  
  return type;
}

char * ct_method_name(void) {
  
  static char * name = NULL;

  if (name == NULL) {
    name = (char *)x_mem_alloc(1024);
  }

  x_snprintf(name, 1024, "%k/%m", jm->method->spec.declaring_clazz, jm->method);

  return name;
}

j_node ct_load_static(int index) {
 
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  w_field field = getFieldConstant(jm->method->spec.declaring_clazz, index);
 
  /* 
  ** Make sure the field's class is properly initialized as the generated 
  ** machine will omit this test.
  */

  classMustBeInitialized(field->declaring_clazz);
  
  return node_create_constant(clazz2jtype(field->value_clazz), (int)(field->declaring_clazz->staticFields + field->size_and_slot));
}

j_node ct_load_static_fields(int index) {

  /*
  ** We can't pre-calculate the exact address of the required item in 
  ** the constant pool at compile time, but we can pre-resolve the
  ** field. 
  */
  
  w_field field = getFieldConstant(jm->method->spec.declaring_clazz, index);

  /* 
  ** Make sure the field's class is properly initialized as the generated 
  ** machine will omit this test.
  */
  
  classMustBeInitialized(field->declaring_clazz);

  return node_create_constant(j_address, (int)field->declaring_clazz->staticFields);
}

j_node ct_load_slot(int index) {

  /*
  ** We can't pre-calculate the exact address of the required item in 
  ** the constant pool at compile time, but we can pre-resolve the
  ** field. 
  */
  
  w_field field = getFieldConstant(jm->method->spec.declaring_clazz, index);

  /* 
  ** Make sure the field's class is properly initialized as the generated 
  ** machine will omit this test.
  */
  
  classMustBeInitialized(field->declaring_clazz);

  return node_create_constant(clazz2jtype(field->value_clazz), field->size_and_slot);
}

j_node ct_load_type(int index) {
 
  /*
  ** Map the given index onto a primitive type.
  */
  
  w_clazz clazz = atype2clazz[index];
  
  /* 
  ** Make sure the class is properly initialized as the generated machine
  ** will omit this test.
  */
 
  classMustBeInitialized(clazz);

  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);

  return node_create_constant(j_address, (int)clazz);
}

j_node ct_load_class(int index) {

  w_clazz clazz;

  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */

  clazz = getClassConstant(jm->method->spec.declaring_clazz, index);
 
  /* 
  ** Make sure the class is properly initialized as the generated machine
  ** will omit this test.
  */

  classMustBeInitialized(clazz);
  
  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);
  
  return node_create_constant(j_address, (int)clazz);
}

j_node ct_load_array(int index) {
 
  w_clazz clazz;
  
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  clazz = getClassConstant(jm->method->spec.declaring_clazz, index);

  /* 
  ** Make sure the class is properly initialized as the generated machine
  ** will omit this test.
  */

  classMustBeInitialized(clazz);

  clazz = getNextDimension(clazz);
 
  woempa(9, "returning pointer to clazz '%k' (%p)\n", clazz, clazz);
  
  return node_create_constant(j_address, (int)clazz);
}

j_node ct_load_constant(int index) {

  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time.
  */
  
  j_node node = NULL;
  w_cnt constant = jm->method->spec.declaring_clazz->constants + index;

  if (constant->cnt_tag == CONSTANT_STRING || constant->cnt_tag == RESOLVED_STRING) {
    node = node_create_constant(j_address, (int)getStringConstant(jm->method->spec.declaring_clazz, index));
  }
  else if (constant->cnt_tag == CONSTANT_INTEGER) {
    node = node_create_constant(j_integer, (int)getIntegerConstant(jm->method->spec.declaring_clazz, index));
  }
  else if (constant->cnt_tag == CONSTANT_FLOAT) {
    node = node_create_constant(j_float, (int)getFloatConstant(jm->method->spec.declaring_clazz, index));
  }
  else if (constant->cnt_tag == CONSTANT_LONG) {
    jfail("oops");
    // node = node_create_constant(j_long, (int)getLongConstant(jm->method->spec.declaring_clazz, index));
  } 
  else if (constant->cnt_tag == CONSTANT_DOUBLE) {
    jfail("oops");
    // node = node_create_constant(j_double, (int)getDoubleConstant(jm->method->spec.declaring_clazz, index));
  }
  else {
    jabort("bad constant type %d\n", constant->cnt_tag);
  }

  return node;
}

j_node ct_load_method(int index, int * argc) {

  w_method m;
  int i;
  
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */

  m = getMethodConstant(jm->method->spec.declaring_clazz, index);
  
  /*
  ** Check whether we have a method that is being supported:
  */
  
  // check the parameter list:
  for (i = 0; i < m->exec.nargs; i++) {
    if (m->spec.arg_types[i] == clazz_long || m->spec.arg_types[i] == clazz_double) {
      jfail("the primitive types 'long' and 'double' are not supported yet\n");
    }
  }

  // check the return type:
  if (m->spec.return_type == clazz_long || m->spec.return_type == clazz_double) {
    jfail("the primitive types 'long' and 'double' are not supported yet\n");
  }
        
  /*
  ** Set the number of arguments that are passed onto the method
  ** upon calling the method.
  */
  
  *argc = m->exec.nargs;
  
  /*
  ** Return the method:
  */

  return node_create_constant(clazz2jtype(m->spec.return_type), (int)m);
}

j_node ct_load_interface(int index, int * argc) {

  w_method m;
  int i;
  
  /*
  ** We can pre-calculate the exact address of the required item in
  ** the constant pool at compile time. 
  */
  
  m = getIMethodConstant(jm->method->spec.declaring_clazz, index);
 
  /*
  ** Check whether we have a method that is being supported:
  */
  
  // check the parameter list:
  for (i = 0; i < m->exec.nargs; i++) {
    if (m->spec.arg_types[i] == clazz_long || m->spec.arg_types[i] == clazz_double) {
      jfail("the primitive types 'long' and 'double' are not supported yet\n");
    }
  }

  // check the return type:
  if (m->spec.return_type == clazz_long || m->spec.return_type == clazz_double) {
    jfail("the primitive types 'long' and 'double' are not supported yet\n");
  }
        
  /*
  ** Set the number of arguments that are passed onto the method
  ** upon calling the method.
  */
  
  *argc = m->exec.nargs;
  
  /*
  ** Return the method:
  */

  return node_create_constant(clazz2jtype(m->spec.return_type), (int)m);
}


j_node ct_array_length_offset(void) {
  return node_create_constant(j_integer, F_Array_length);
}

