/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: dead-code-elimination.c 3970 2003-05-26 14:59:19Z dbuytaert $
*/

#include "j-spot.h"
#include "method.h"
#include "node.h"
#include "ir.h"

static j_node list;

static inline void worklist_add(j_node node) {

  if (node->critical == 0) {
    node->critical = 1;
    node->list = list;
    list = node;
  }
}

static inline j_node worklist_del(void) {

  j_node node = list;
  list = node->list;
  
  return node;
}

static void dce_mark_literal(j_node literal) {
 
  int i;
  
  if (literal->definition) {
    worklist_add(literal->definition);
  }
  
  for (i = 0; i < j_max_edges; i++) {
    if (literal->src[i]) {  
      worklist_add(literal->src[i]);
    }
    if (literal->dst[i]) {  
      worklist_add(literal->dst[i]);
    }
  } 
}

static void dce_mark_instruction(j_node instruction, j_node operand, va_list args) {

  /*
  ** If an instruction is critical, its operands (possibly instructions)
  ** are critical too.  So if def(operand) is not marked as critical, 
  ** mark it and add it to the worklist.
  */

  if (operand) {
    worklist_add(operand);

    dce_mark_literal(operand);
  }
}

void dead_code_elimination(void) {

  j_node node;
  j_node temp;

  /*
  ** Compile a worklist of critical operations such as return values,
  ** escape points (function calls), etc.
  */
  
  list = NULL;
  node = jm->start;
  
  while (node) {
    if (ir_is_basic_block_entry(node) || ir_is_basic_block_exit(node) || node_is_escapable(node)) {
      worklist_add(node);
    }

    node = node->next;
  }

  /*
  ** Work back up the SSA edges to find each critical instruction's
  ** antecedent.
  */

  while (list) {
    temp = worklist_del();

    /*
    ** Mark all operands and parameters as being critical.
    */
    
    node_operands(temp, dce_mark_instruction); 

    /*
    ** Mark all clones and definitions as being critical.
    */

    dce_mark_literal(temp);
  }
}

