/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: dot.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>

#include "j-spot.h"
#include "wstrings.h"
#include "ct-hooks.h"
#include "ir.h"

FILE *df;

void dot_emit_node(j_node node) {
  
  j_node temp;
  int i;
 
  if (node->visited != 9) {
    node->visited = 9;

    /*
    ** Print the node's labels.  Try to do some basic formatting to
    ** high-light some of the meta data and to make the graph a bit
    ** more human-readable.
    */

    if (node->operator == j_constant && node->type == j_address) { 
      fprintf(df, "  n%p [ label = \"constant\\n%p\" ]\n", node, (j_word *)node->data.constant.value);
    }
    else if (node->operator == j_constant && node->type != j_address) { 
      fprintf(df, "  n%p [ label = \"constant\\n%d\" ]\n", node, node->data.constant.value);
    }
    else if (node->operator == j_variable && node->data.variable.location == j_parameter) {
      fprintf(df, "  n%p [ label = \"parameter %d\" ]\n", node, node->data.variable.offset);
    }
    else if (node->operator == j_variable && node->data.variable.offset < method_size_locals()) {
      fprintf(df, "  n%p [ label = \"temporary %d\\ntype = local\" ]\n", node, node->data.variable.offset);
    }
    else if (node->operator == j_variable && node->data.variable.offset >= method_size_locals()) {
      fprintf(df, "  n%p [ label = \"temporary %d\\ntype = stack\" ]\n", node, node->data.variable.offset);
    }
    else if (node_is_call(node)) { 
      if (node->name) {
        fprintf(df, "  n%p [ label = \"call '%s'\\n %p\\npc = %d\", shape = box ]\n", node, node->name, (j_word *)node->data.call.value, node->pc);
      }
      else {
        fprintf(df, "  n%p [ label = \"call\\n%p\\npc = %d\", shape = box ]\n", node, (j_word *)node->data.call.value, node->pc);
      }

      /*
      ** If the current node is a function call, iterate through all the
      ** parameter nodes:
      */

      for (i = 0; i < node->data.call.argc; i++) {
        temp = *(j_node *)(node->data.call.args + i);

        if (temp != NULL) {
          dot_emit_node(temp);
          fprintf(df, "  n%p -> n%p [ style = dotted ]\n", node, temp);
        }
      }
    }
    else {
      if (node->name) {
        fprintf(df, "  n%p [ label = \"%s\\n'%s'\\npc = %d\", shape = box ]\n", node, node_get_operator(node), node->name, node->pc);
      }
      else {
        fprintf(df, "  n%p [ label = \"%s\\npc = %d\", shape = box ]\n", node, node_get_operator(node), node->pc);
      }
    }

    /*
    ** Link the different basic blocks (DAGs):
    */

    if (node->outgoing[0] != NULL) {
      if (node->outgoing[1] != NULL) {
        fprintf(df, "  n%p -> n%p [ label = true ]\n", node, node->outgoing[0]);
      }
      else {
        fprintf(df, "  n%p -> n%p\n", node, node->outgoing[0]);
      }
    }

    if (node->outgoing[1] != NULL) {
      if (node->outgoing[0] != NULL) {
        fprintf(df, "  n%p -> n%p [ label = false ]\n", node, node->outgoing[1]);
      }
      else {
        fprintf(df, "  n%p -> n%p\n", node, node->outgoing[1]);
      }
    }

    /*
    ** If we have a generic node with one or more child nodes, connect 
    ** the the childs to their parent node:
    */

    if (node->node1) {
      dot_emit_node(node->node1);
      fprintf(df, "  n%p -> n%p [ style = dotted ]\n", node->node1, node);
    }
    if (node->node2) {
      dot_emit_node(node->node2);
      fprintf(df, "  n%p -> n%p [ style = dotted ]\n", node, node->node2);
    }

    /*
    ** If the node has been cloned, print the clones and connect them
    ** using a solid grey edge.
    */
    
    for (i = 0; i < j_max_edges; i++) {
      if (node->dst[i]) {
        dot_emit_node(node->dst[i]);
        fprintf(df, "  n%p -> n%p [ style = dotted, color = grey ]\n", node, node->dst[i]);
      }
      if (node->src[i]) {
        dot_emit_node(node->src[i]);
        fprintf(df, "  n%p -> n%p [ style = dotted, color = grey ]\n", node, node->src[i]);
      }
    }
  }
}

void dot(void) {
  
  static char * name = NULL;
  static int number = 1;
  j_node node;

  if (name == NULL) {
    /*
    ** Initialize the name buffer.
    */
    
    name = (char *)x_mem_alloc(64);
    
    /*
    ** Create the target directory.
    */
  
    mkdir("user", S_IRWXU | S_IRWXG | S_IRWXO);
    mkdir("user/dot", S_IRWXU | S_IRWXG | S_IRWXO);
  }
  
  node = jm->start;

  sprintf(name, "user/dot/%d.dot", number++); 
  
  df = fopen(name, "w");
 
  if (df != NULL) {
    
    fprintf(df, "digraph G {\n");
    fprintf(df, "  page  = 11;\n");
    fprintf(df, "  start [ label = \"%s(...)\", shape = box, fontsize = 16 ]\n", ct_method_name());
    fprintf(df, "  start -> n%p\n", node);

    while (node) {
      dot_emit_node(node);
      node = node->next;
    }  
  
    fprintf(df, "}\n");
 
    fclose(df);
  }
}

