/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: hotspot.c 3863 2003-04-25 09:23:25Z dbuytaert $ 
*/

#include "j-spot.h"
#include "hotspot.h"
#include "ct-hooks.h"

#include "methods.h"

/*
** Profiling and hot-spot detection settings
*/

#define J_INVOCATION_THRESHOLD   32
#define J_RESET_THRESHOLD        96
#define J_CRITICAL_INIT          32
#define J_CRITICAL_D             1
#define J_CRITICAL_I             2 

extern x_mutex jspot_mutex;
extern int jspot_compile;
extern int jspot_profile;

void hs_method_invoke(w_method method) {
}

void hs_method_return(w_method method) {
 
  w_thread thread;

  /*
  ** Only proceed if the compiler is enabled.
  */
 
  if (jspot_compile == 0) {
    return;
  }
  
  /*
  ** When profiling is disabled, we compile every method upon returning
  ** control.  However, when profiling is enabled, we try compiling the
  ** frequently executed methods that are part of the current hot-spot.
  */

  if (jspot_profile) {

    thread = currentWonkaThread;

    /*
    ** Each method's execution frequency is tracked using an invocation
    ** counter.
    */
    
    method->counter = method->counter + 1;
    
    // fprintf(stderr, "thread %p with counter %3d - method %p with counter %3d\n", thread, thread->counter, method, method->counter);

    /*
    ** We use a per-thread saturating counter to measure the dynamic 
    ** execution percentage of critical methods.  Initialized at 
    ** J_CRITICAL_INIT, the counter is decremented by J_CRITICAL_D each 
    ** time a critical method is invoked and incremented by J_CRITICAL_I
    ** each time a  non-critical method is invoked.   
    */
    
    if (method->counter >= J_INVOCATION_THRESHOLD) {

      if (thread->counter - J_CRITICAL_D >= 0) {
        thread->counter = thread->counter - J_CRITICAL_D;
      }
      
      /*
      ** If the thread's saturatig counter reached 0, a critical path
      ** has been identified and the compiler is invoked.
      */
      
      if (thread->counter < J_CRITICAL_I * 5) {
        if (x_mutex_lock(jspot_mutex, 0) == xs_success) {
          ct_compile(method);
          x_mutex_unlock(jspot_mutex);
        }
      }
      else {
        /*
        ** Periodically, the invocation counters are reset to ensure
        ** that only frequently executing methods get marked as 
        ** critical.
        */
        
        if (method->counter >= J_RESET_THRESHOLD) {
          method->counter = 0;
        }
      }
    }
    else {

      if (thread->counter + J_CRITICAL_I <= J_CRITICAL_INIT) {
        thread->counter = thread->counter + J_CRITICAL_I;
      }
    }
  }    
  else {
    if (x_mutex_lock(jspot_mutex, 0) == xs_success) { 
      ct_compile(method);
      x_mutex_unlock(jspot_mutex);
    }
  }
}

