/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: ir.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include <string.h>

#include "oswald.h"
#include "j-spot.h"
#include "bytecode.h"
#include "mimic-stack.h"
#include "ir.h"

extern int jspot_propagate;

int ir_is_jump(j_node node) {

  switch (node->operator) {
    case j_jump:
    case j_le:
    case j_gt:
    case j_eq:
    case j_ne:
    case j_lt:
    case j_ge:
      return 1;
    default:
      return 0;
  }
}

int ir_is_split(j_node node) {
  
  if (node->outgoing[0] != NULL && node->outgoing[1] != NULL) {
    return 1;
  }
  else {
    return 0;
  }
}

int ir_is_join(j_node node) {
  
  if (node->incoming[0] != NULL && node->incoming[1] != NULL) {
    return 1;
  }
  else {
    return 0;
  }
}

int ir_is_exit(j_node node) {

  if (node->outgoing[0] == NULL && node->outgoing[1] == NULL) {
    return 1;
  } 
  else {
    return 0;
  }
}

j_boundary ir_is_basic_block_entry(j_node node) {

  if (ir_is_join(node)) {
    return j_explicit;
  }
  else if (node->incoming[0] != NULL && ir_is_jump(node->incoming[0])) {
    return j_explicit;
  }
  else if (node->incoming[1] != NULL && ir_is_jump(node->incoming[1])) {
    return j_explicit;
  }
  else {
    return j_no_boundary;
  }
}

j_boundary ir_is_basic_block_exit(j_node node) {

  if (ir_is_jump(node)) {
    return j_explicit;
  }
  else if (ir_is_exit(node)) {
    return j_explicit;
  }
  else if (node->outgoing[0] && ir_is_join(node->outgoing[0])) {
    return j_implicit;
  }
  else {
    return j_no_boundary;
  }
}


void ir_add_node(j_node node, int tedge, int fedge) {

  j_node prev;
  j_node next;

  /*
  ** Add the node to the ordered list of nodes.  The list is ordered
  ** based on the nodes' program counter.  That is, based upon their
  ** their occurence in the Java bytecode and independent of control
  ** flow.
  */
  
  next = jm->start;
  prev = NULL;

  while (next && next != node && next->pc < node->pc + 1) {
    prev = next;
    next = next->next;
  }
   
  if (node != next) {     // Don't insert duplicate nodes.

    woempa(9, "add node %q with pc %d\n", node, node->pc);
    
    if (prev == NULL) {
      node->next = jm->start;
      jm->start = node;
    }
    else if (next != NULL) {
      prev->next = node;
      node->next = next;
    }
    else {
      prev->next = node;
    }

    /*
    ** Set the start and the exit nodes.
    */
    
    if (jm->start == NULL) {
      jm->start = node;
    }
        
    /*
    ** Take care of the outgoing edges.
    */
    
    node->outgoing[0] = (j_node)tedge;
    node->outgoing[1] = (j_node)fedge;
  
    /*
    ** Update the method's instruction counter:
    */

    jm->instructions += 1;
  }
}

j_node ir_get_node_next(int pc) {

  j_node node = jm->start;

  /*
  ** Get the node at the specified program counter or the next one.
  */

  while (node && node->pc < pc) {
    node = node->next;
  }

  return node;
}

j_node ir_get_node_prev(int pc) {

  j_node node = jm->start;

  /*
  ** Get the node at the specified program counter or the previous one.
  */

  while (node && node->next && node->next->pc <= pc) {
    node = node->next;
  }

  return node;
}

j_node ir_get_branch(j_node node) {
  return node->outgoing[0];
}

void ir_build_dags(void) {

  j_node node = NULL;
  j_node next = NULL;
  j_node temp = NULL;
  j_node src = NULL;
  j_node dst = NULL;
  int i;
  int j;

  node = jm->start;

  while (node && !jm->failed) {
  
    /*
    ** At the points where control /flows/ to one or more other nodes,
    ** we connect or link all the nodes involved.
    */
    
    if ((int)node->outgoing[0] == j_link_none) {
      node->outgoing[0] = NULL;
    }
    else if ((int)node->outgoing[0] == j_link_next) {
      node_add_link(node, node->next, 1);
    }
    else {
      next = ir_get_node_next((int)node->outgoing[0]);
      node_add_link(node, next, 1);
    }
         
    if ((int)node->outgoing[1] == j_link_none) {
      node->outgoing[1] = NULL;
    } 
    else if ((int)node->outgoing[1] == j_link_next) {
      node_add_link(node, node->next, 0);
    }
    else {
      next = ir_get_node_next((int)node->outgoing[1]);
      node_add_link(node, next, 0);
    }

    node = node->next;
  }
  
  node = jm->start;

  while (node && !jm->failed) {
    /*
    ** Insert phi-nodes at control join points.
    */

    if (ir_is_jump(node)) {
      for (i = 0; i < jm->slots; i++) {
        src = ms_node_state(node->pc, j_post_execution, i);
        if (src && !node_is_parameter(src)) {
          for (j = 0; j < j_max_edges; j++) {
            temp = node->outgoing[j];
            if (temp) {
              dst = ms_node_state(temp->pc, j_pre_execution, i);
              if (dst) {
                node_add_clone(src, dst);
             
                // TODO: move this logic to the compiler backend 
                #ifdef X86 
                  src->stack = - 4 * (i + 1);
                  dst->stack = - 4 * (i + 1);
                  dst->spill = 1;
                #else 
                  src->stack = - 4 * (i + 1) - 16;
                  dst->stack = - 4 * (i + 1) - 16;
                  dst->spill = 1;
                #endif
              }
            }
          }
        }
      }
    }
    
    if (ir_is_join(node)) {
      for (i = 0; i < jm->slots; i++) {
        dst = ms_node_state(node->pc, j_pre_execution, i);
        if (dst && !node_is_parameter(dst)) {
          for (j = 0; j < j_max_edges; j++) {
            temp = node->incoming[j];
            if (temp) {
              src = ms_node_state(temp->pc, j_post_execution, i);
              if (src) {
                node_add_clone(src, dst);
             
                // TODO: move this logic to the compiler backend 
                #ifdef X86 
                  src->stack = - 4 * (i + 1);
                  dst->stack = - 4 * (i + 1);
                  dst->spill = 1;
                #else 
                  src->stack = - 4 * (i + 1) - 16;
                  dst->stack = - 4 * (i + 1) - 16;
                  dst->spill = 1;
                #endif
              }
            }
          }
        }
      }
    }

    node = node->next;
  }
}

void ir(void) {
  
  ir_build_dags();
}

