/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: j-spot.c 3682 2003-03-19 14:12:41Z gray $
*/

#include <string.h>

#include "oswald.h"
#include "wstrings.h"
#include "j-spot.h"
#include "arena.h"
#include "method.h"
#include "node.h"

extern char * jspot_args;

#ifdef DEBUG
  char * jspot_call_range = NULL;
  char * jspot_size_range = NULL;
  char * jspot_name_range = NULL;
  int jspot_compile = 1;
  int jspot_profile = 0;
  int jspot_verbose = 1;
  int jspot_stabs = 1;
  int jspot_stats = 1;
  int jspot_asm = 0;
  int jspot_dot = 0;
#else
  char * jspot_call_range = NULL;
  char * jspot_size_range = NULL;
  char * jspot_name_range = NULL;
  int jspot_compile = 1;
  int jspot_profile = 0;
  int jspot_verbose = 0;
  int jspot_stabs = 0;
  int jspot_stats = 0;
  int jspot_asm = 0;
  int jspot_dot = 0;
#endif

x_mutex jspot_mutex;

static void joempa_va_list(char * format, va_list args) {
  
  static char * buffer = NULL; 

  if (jspot_verbose) {
    
    if (buffer == NULL) {
      buffer = x_mem_calloc(1024);
    }

    x_vsnprintf(buffer, 1024, format, args);

    fprintf(stderr, "%s", buffer);
  }
}

void joempa(char * format, ...) {

  va_list args;
  
  va_start(args, format);
  joempa_va_list(format, args);
  va_end(args);
}

void jabort(char * format, ...) {
 
  va_list args;
 
  joempa("internal compiler error when compiling '%M': ", jm->method);

  va_start(args, format);
  joempa_va_list(format, args);
  va_end(args);

  abort();  
}

void jfail(char * format, ...) {
 
  va_list args;
 
  joempa("failed compiling '%M': ", jm->method);

  va_start(args, format);
  joempa_va_list(format, args);
  va_end(args);

  jm->failed = 1;
}

void jspot_init(w_void) {
  
  jspot_parse_arguments();

  x_formatter('q', node_print);

  jspot_mutex = x_mem_calloc(sizeof(x_Mutex));
  x_mutex_create(jspot_mutex);
}

void jspot_help(void) {

  printf("Usage:\n");
  printf("  -Wcompiler:<option>=<value>, ...\n");
  printf("\n");
  printf("Options:\n");
  printf("  compile=y|n             enable or disable the runtime compiler (default: %s)\n", (jspot_compile ? "y" : "n"));
  printf("  profile=y|n             enable or disable the runtime profiler (default: %s)\n", (jspot_profile ? "y" : "n"));
  printf("  verbose=y|n             enable or disable verbose debug output (default: %s)\n", (jspot_verbose ? "y" : "n"));
  printf("  call-range=f-t/m/f-t    compile methods in the specified call range(s) only (default: <all>)\n");
  printf("  size-range=f-t/m/f-t    compile methods in the specified size range(s) only (default: <all>)\n");
  printf("  name-range=string       compile methods in the specified name range(s) only (default: <all>)\n");
  printf("  stabs=y|n               write stabs debug information to file (default: %s)\n", (jspot_stabs ? "y" : "n"));
  printf("  stats=y|n               write statistics information to file (default: %s)\n", (jspot_stabs ? "y" : "n"));
  printf("  asm=y|n                 write the generated assembly code to file (default: %s)\n", (jspot_asm ? "y" : "n"));
  printf("  dot=y|n                 write the intermediate representations to file (default: %s)\n", (jspot_dot ? "y" : "n"));
  printf("  help                    print this help message\n");

  exit(1);
}

void jspot_parse_arguments() {

  char * args = jspot_args;
  char * argument;
  char * key;
  char * value;

  /*
  ** Go over the arguments one by one. They are seperated with comma's.
  */

  while ((argument = strsep((char **)&args, ",")) != NULL) {

    /*
    ** Every parameter is a key-value pair seperated by ='s.
    */
    
    key = argument;
    value = index(argument, '=');
    
    if (value != NULL) {
      value[0] = '\0';
      value++;
    }

    /*
    ** Check the key/value pairs.
    */
    
    if (strcmp(key, "compile") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_compile = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_compile = 0;
      } 
      else {
        printf("compile error: 'compile' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }
      
    if (strcmp(key, "profile") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_profile = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_profile = 0;
      } 
      else {
        printf("compile error: 'profile' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }
    
    if (strcmp(key, "verbose") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_verbose = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_verbose = 0;
      } 
      else {
        printf("compile error: 'verbose' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }

    if (strcmp(key, "stabs") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_stabs = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_stabs = 0;
      } 
      else {
        printf("compiler error: 'stabs' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }
 
    if (strcmp(key, "stats") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_stats = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_stats = 0;
      } 
      else {
        printf("compiler error: 'stats' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }
    
    if (strcmp(key, "call-range") == 0) {
      jspot_call_range = value;
    }
    
    if (strcmp(key, "size-range") == 0) {
      jspot_size_range = value;
    }
       
    if (strcmp(key, "name-range") == 0) {
      jspot_name_range = value;
    }
       
    if (strcmp(key, "asm") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_asm = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_asm = 0;
      } 
      else {
        printf("compiler error: 'asm' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }

    if (strcmp(key, "dot") == 0) {
      if (strcmp(value, "y") == 0) {
        jspot_dot = 1;
      }
      else if (strcmp(value, "n") == 0) {
        jspot_dot = 0;
      } 
      else {
        printf("compiler error: 'dot' should be set to 'y' or 'n', but not to '%s'!\n", value);
        exit(1);
      }
    }
  }
}

int jspot_check_nrange(char * ranges, int value) {

  char * string;
  char * range;
  int check;
  int f;
  int t;

  string = x_mem_calloc(strlen(ranges) + 1);
  strcpy(string, ranges);
   // NOTE: calling "strsep" changes the specified string so we have to work with a backup copy.
  
  check = 0;

  while ((range = strsep((char **)&string, "/")) != NULL) {
    
    if (index(range, '-')) {
      sscanf(range, "%d-%d", &f, &t);
      
      if (value >= f && value <= t) {
        check = 1;
      }
    }
    else {
      sscanf(range, "%d", &f);

      if (value == f) {
        check = 1;
      }
    }
  }
 
  x_mem_free(string);

  return check;
}

int jspot_check_arange(char * ranges, w_string string) {

  int length;
  int check;
  int i;

  if (string_length(string) < strlen(ranges)) {
    length = string_length(string);
  }
  else {
    length = strlen(ranges);
  }
    
  check = 1;
  
  for (i = 0; i < length; i++) {
    if (ranges[i] != string_char(string, i)) {
      check = 0;
    }
  }
  
  return check;
}

