/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: method.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include <string.h>

#include "methods.h"

#include "j-spot.h"
#include "arena.h"
#include "bytecode.h"
#include "register-allocator.h"
#include "ir.h"

static inline char *method_spaces(const int depth) { 
  const char *spaces = "                                          ";  // 40 spaces
  return (char *)(spaces + strlen(spaces) - (2 * depth));
}

j_method method_create(w_method wmethod) {

  j_method method = arena_allocate(sizeof(j_Method));
  /*
  ** Allocate memory for the bytecode:
  */
 
  method->bytecode = arena_allocate(wmethod->exec.code_length * sizeof(char));
  memcpy(method->bytecode, wmethod->exec.code, wmethod->exec.code_length);
  
  /*
  ** Initialize internal fields:
  */

  method->length = wmethod->exec.code_length;
  method->method = wmethod;
  method->slots = wmethod->exec.local_i + wmethod->exec.stack_i + 1; 
  
  /*
  ** Set the global method "jm":
  */
 
  jm = method;

  return method;
}

int method_size_stack(void) {
  return jm->method->exec.stack_i;
}

int method_size_locals(void) {
  return jm->method->exec.local_i;
}

int method_size_parameters(void) {
  return jm->method->exec.arg_i;
}

int method_num_parameters(void) {
  if (method_is_static()) {
    return jm->method->exec.nargs - 1;
  }
  else {
    return jm->method->exec.nargs;
  }
}

int method_is_static(void) {
  return jm->method->flags & ACC_STATIC;
}

void method_dump_node(j_node node, int depth) {

  woempa(9, "%s%q\n", method_spaces(depth), node);
      
  if (node->node1) {
    method_dump_node(node->node1, depth + 1);
  }

  if (node->node2) {
    method_dump_node(node->node2, depth + 1);
  }
}

void method_dump(void) {

  j_node node = jm->start;
  
  woempa(9, "------------------------------------------------------------------\n");
  while (node) {
    method_dump_node(node, 0);
    node = node->next;
  }
  woempa(9, "------------------------------------------------------------------\n");
}
  
