/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: rt-hooks.c 4025 2003-07-09 08:04:20Z dbuytaert $
*/

// System headers:
#include <assert.h>

// Wonka headers:
#include "interpreter.h"
#include "methods.h"
#include "wstrings.h"
#include "loading.h"
#include "threads.h"
#include "checks.h"
#include "clazz.h"
#include "calls.h"
#include "fields.h"
#include "arrays.h"
#include "exception.h"
#include "ieee754.h"
#include "Math.h"

// j-spot headers:
#include "ir.h"
#include "dot.h"
#include "compiler.h"
#include "translate.h"
#include "assembler.h"
#include "rt-hooks.h"
#include "call-stack.h"

#ifdef DEBUG
 #define verify_gc_state \
  assert(frame->thread->flags & WT_THREAD_NOT_GC_SAFE);
 #define verify_frame(f, m) \
   _verify_frame(f, m)
#else
 #define verify_gc_state
 #define verify_frame(f, m) 
#endif

#define invoke_method(f, m)                                                                               \
  verify_frame(f, m);                                                                                     \
  thread->numframes--;                                                                                    \
  if (m->flags & METHOD_IS_COMPILED) {                                                                    \
    if (m->flags & (ACC_STATIC | ACC_SYNCHRONIZED)) {                                                     \
      f->method = NULL;                                                                                   \
      f->clazz = NULL;                                                                                    \
      thread->top = f;                                                                                    \
      dispatchers[m->exec.dispatcher](f, m);                                                              \
    }                                                                                                     \
    else {                                                                                                \
      f->method = m;                                                                                      \
      f->clazz = m->spec.declaring_clazz;                                                                 \
      thread->top = f;                                                                                    \
      f->top[-1].c = _call_instance(w_thread2JNIEnv(thread), f->top, &m->exec);                           \
    }                                                                                                     \
  }                                                                                                       \
  else {                                                                                                  \
    f->method = NULL;                                                                                     \
    f->clazz = NULL;                                                                                      \
    thread->top = f;                                                                                      \
    enterSafeRegion(thread);                                                                              \
    dispatchers[m->exec.dispatcher](f, m);                                                                \
    enterUnsafeRegion(thread);                                                                            \
  }                                                                                                       \
  thread->numframes++;                                                                                    \

  /*
  ** NOTE 1: "verify_frame()" should be called _before_ linking the frame in the stack;
  **         it temporary modifies some frame-pointers which would confuse or crash the
  **         GC, for example.
  ** NOTE 2: we can't shortcut in case of a JNI method as these can and should be run
  **         in safe mode.
  */

#define null_pointer_check(instance)                                                     \
  if (instance == NULL) {                                                                \
    exception_throw(clazzNullPointerException);                                          \
    return 0;                                                                            \
  }

#define setup_frame()                                                                    \
  thread = (w_thread)thread_current->xref;                                               \
  frame->previous = thread->top;                                                         \
  frame->thread = thread;                                                                \
  frame->flags |= FRAME_STACKMAP;                                                        \
  frame->auxs = thread->top->auxs;                                                       \
  frame->top = frame->Slots + method->exec.arg_i;                                        \
  frame->map = (w_instance *)(frame->top + 1);                                           \
  verify_gc_state; 

#define return_void()                                                                    \
  verify_gc_state;                                                                       \
  if (thread->exception) {                                                               \
    thread->top = frame->previous;                                                       \
    exception_propagate(thread->exception);                                              \
  }                                                                                      \
  else {                                                                                 \
    thread->top = frame->previous;                                                       \
  }                                                                                      \
  return 0;

#define return_word()                                                                    \
  if (thread->exception) {                                                               \
    thread->top = frame->previous;                                                       \
    exception_propagate(thread->exception);                                              \
  }                                                                                      \
  else {                                                                                 \
    result.i = frame->top[-1].c;                                                         \
    thread->top = frame->previous;                                                       \
  }                                                                                      \
  return result.i;

/*
** No lookup needs to be performed.
*/
  
#define invokenonvirtual_lookup()                                                        \
  woempa(9, "'%M' calls non-virtual instance method '%M' on instance %p (%k)\n", thread->top->method, method, instance, instance2clazz(instance));

/*
** Instance methods are always being invoked using an object.  Due to
** inheritance, the type of an instance - and hence the actual method
** to call - is not always known at compile-time so we have to perform 
** a lookup to bind the method dynamically.
*/

#define invokevirtual_lookup()                                                          \
  clazz = instance2clazz(instance);                                                     \
                                                                                        \
  if (method->spec.declaring_clazz == clazz) {                                          \
    actual_method = method;                                                             \
  }                                                                                     \
  else {                                                                                \
    actual_method = clazz->vmlt[method->slot];                                          \
  }                                                                                     \
  woempa(9, "'%M' calls virtual instance method '%M' on instance %p (%k)\n", thread->top->method, actual_method, instance, instance2clazz(instance));
 
/*
** Instance methods are always being invoked using an object.  Due to
** inheritance, the type of an instance - and hence the actual method
** to call - is not always known at compile-time so we have to perform 
** a lookup to bind the method dynamically.
*/

#define invokespecial_lookup()                                                         \
  super_clazz = getSuper(thread->top->clazz);                                          \
  if (isSuperClass(method->spec.declaring_clazz, super_clazz)) {                       \
    method = virtualLookup(method, super_clazz);                                       \
  }                                                                                    \
  woempa(9, "'%M' calls virtual instance method '%M' on instance %p (%k)\n", thread->top->method, method, instance, instance2clazz(instance));

void exception_propagate(w_instance exception) {
  
  w_thread thread;
  w_method method;
  
  thread = (w_thread)thread_current->xref;

  if (thread->top->method) {
    method = thread->top->method;
  }
  else {
    method = thread->top->previous->method;
  }
  
  woempa(9, "thread = %p, top = %p, method = %M, exception = %p, return address = %p\n", thread, thread->top, method, thread->exception, method->return_address);
 
  /*
  ** Skip the rest of the JIT-ed function by overwriting our caller's
  ** return address.  Note that we can't overwrite our own return 
  ** address because some intermediate function might have used callee
  ** saved registers. 
  */
 
  cs_skip_remainder((j_word)method->return_address);
}

void exception_throw(w_clazz clazz) {

  w_thread thread = (w_thread)thread_current->xref;
 
  /*
  ** Skip the rest of the JIT-ed function by overwriting our caller's
  ** return address.  Note that we can't overwrite our own return 
  ** address because some intermediate function might have used callee
  ** saved registers. 
  */

  woempa(9, "JITed method %m of thread %p throws a %k\n", thread->top->method, thread, clazz);
  
  throwException(thread, clazz, NULL);
 
  cs_skip_remainder((j_word)thread->top->method->return_address);
}

void rt_exception_throw(w_clazz clazz) {

  exception_throw(clazz);
    // NOTE: this level of indirection is required to make cs_skip_remainder() work as expected.
}

static void _verify_frame(x_frame frame, w_method method) {

  w_instance instance;
  w_clazz  * clazz;
  int i = 0;
  int n = 0;

  /*
  ** Verify that we are in unsafe GC state.
  */

  
  frame->top = frame->top - method->exec.nargs;

  /*
  ** Verify the slots for the parameters.
  */
  
  for (n = 0; n < method->exec.nargs; n++) {
    clazz = &method->spec.arg_types[n];
    mustBeLoaded(clazz);

    woempa(9, "argument %d of %M (%p) is %k (%p)\n", i, method, method, method->spec.arg_types[i], method->spec.arg_types[i]);
      // NOTE: the above line is quite redundant but will cause Wonka to crash when invalid.
     
    if (clazzIsPrimitive(*clazz)) {
      woempa(9, "call: push '%d' of primitive type %w\n", frame->top[i].c, *clazz);
      
      if (isSet((*clazz)->type, VM_TYPE_TWO_CELL)) {
        assert(frame->top[i++].s == 0);
        assert(frame->top[i++].s == 0);
      }
      else {
        /*
        ** NOTE: are floats always being passed as "w_float"s (32-bit 
        ** stack slot) or do they occasionally get passed as "float"s 
        ** (64-bit stack slot)?
        */
        assert(frame->top[i++].s == 0);
      }
    }
    else if ((*clazz)->dims == 0) {
      instance = (w_instance)frame->top[i].c;

      if (instance && !isAssignmentCompatible(instance2clazz(instance), *clazz)) {
        jabort("parameter '%p' is not assignment compatible with %k\n", instance, *clazz);
      }
      
      woempa(9, "call: push object reference '%p' of type %k\n", instance, instance2clazz(instance));
        // NOTE: we use "instance2clazz(instance)" rather then "clazz" because the former will cause Wonka to crash when invalid. 
      
      assert(frame->top[i++].s == stack_trace);
    } 
    else {
      instance = (w_instance)frame->top[i].c;
      
      if (instance && !isAssignmentCompatible(instance2clazz(instance), *clazz)) {
        jabort("parameter '%p' is not assignment compatible with %k\n", instance, *clazz);
      }
       
      woempa(9, "call: push array reference '%p' of type %k\n", instance, instance2clazz(instance));
        // NOTE: we use "instance2clazz(instance)" rather then "clazz" because the former will cause Wonka to crash when invalid. 
    
      assert(frame->top[i++].s == stack_trace);
    }
  }

  /*
  ** Verify the slost for the return value of the callee(s).  They should be
  ** null in order not to upset the GC).
  */

  assert(frame->top[i++].s == 0);
  // assert(frame->top[i++].s == 0);  // disabled for now; we don't support 64-bit values yet
  
  frame->top = frame->top + method->exec.nargs;
}

int rt_integer_div(int value1, int value2) {

  int result = 0;
  
  if (value2 == 0) {
    /*
    ** If the second value is zero, throw an ArithmeticException.
    */
    
    exception_throw(clazzArithmeticException);
  }
  else {
    if (value1 >= 0 && value2 > 0) {
      result = value1 / value2;
    }
    else if (value1 == (int)0x80000000) {
      if (value2 == -1) {
        result = 0x80000000;
      }
      else {
        result = value1 / value2;      
      }
    }
    else if (value1 < 0 && value2 < 0) {
      value1 = -value1;
      value2 = -value2;
      result = value1 / value2;
    }
    else if (value1 < 0) { 
      value1 = -value1;
      result = -(value1 / value2);
    }
    else {
      value2 = - value2;
      result = -(value1 / value2);
    }
  }
  
  return result;
}

int rt_integer_rem(int value1, int value2) {

  int remainder = 0;
  
  if (value2 == 0) {
    /*
    ** If the second value is zero, throw an ArithmeticException.
    */
    
    exception_throw(clazzArithmeticException);
  }
  else {
    if (value1 != (int)0x80000000 || value2 != -1) {
      remainder = value1 - value2 * (value1 / value2);
    }
  }
  
  return remainder;
}

w_float rt_float_rem(w_float value1, w_float value2) {

  w_float remainder;

  // NOTE: we can't use "float32_rem()" as this function should not the same as the remainder operation defined by IEEE 754. 

  if (float32_is_NaN(value1) || float32_is_NaN(value2) || float32_eq(value2, F_ZERO) || float32_is_Infinite(value1)) {
    remainder = (w_float) F_NAN;
  }
  else {
    if (float32_eq(value1, F_ZERO) || float32_is_Infinite(value2)) {
      remainder = value1;
    }
    else   {
      remainder = float32_mul(float32_abs(value2), F_FLOAT_MAX_VALUE);
      if (float32_lt(remainder, float32_abs(value1))) {
        if (float32_is_negative(value1)) {
          remainder = F_MINUS_ZERO;
        }
        else {
          remainder = F_ZERO;
        }
      }
      else {
        remainder = float32_div(value1, value2);
        remainder = int32_to_float32(float32_to_int32_round_to_zero(remainder));
        remainder = float32_mul(value2,remainder);
        remainder = float32_sub(value1,remainder);
      }
    }
  }

  return remainder;
}

int rt_integer2byte(int value) {

  // NOTE: the "i2b" bytecode does sign-extension.

  value = value & 0x000000ff;
  value = ((value & 0x00000080) ? (value | 0xffffff00) : (value & 0x0000007f));

  return value;
}

int rt_integer2character(int value) {
  
  // NOTE: the "i2c" bytecode does zero-extension.

  value = value & 0xffff;

  return value;
}

int rt_integer2short(int value) {

  // NOTE: the "i2s" bytecode does sign-extension.

  value = value & 0xffff;
  value = ((value & 0x8000) ? (w_int)(value | 0xffff0000) : (w_int)(value & 0x00007fff));

  return value;
}

int rt_float2integer(w_float value) {

  int result;

  if (float32_is_NaN(value)) {
    result = 0;
  }
  else {
    result = float32_to_int32_round_to_zero(value);
  }

  return result;
}

int rt_monitorenter(j_method method, j_node node) {
  jabort("called rt_monitorenter\n");
  return 1;
}

int rt_monitorexit(j_method method, j_node node) {
  jabort("called rt_monitorexit\n");
  return 1;
}

void rt_athrow(w_instance throwable) {

  w_thread thread = (w_thread)thread_current->xref;

  woempa(9, "throwing exception instance %p of type %k\n", throwable, instance2clazz(throwable));

  throwExceptionInstance(thread, throwable);
  
  exception_propagate(throwable);
}

int rt_getstatic(j_word * data) {
  
  woempa(9, "get static %d at memory location %p\n", *data, data);
  
  return *data;
}

void rt_putstatic(int * fields, int slot, j_word value) {

  woempa(9, "put %d in the static variable at slot %d of fields %p\n", value, slot, fields);
  
  fields[slot] = value;
}

int rt_getfield_primitive(w_instance instance, int slot) {

  woempa(9, "get field %d of instance %p (%k): %d\n", slot, instance, instance2clazz(instance), instance[slot]);
  
  return (int)instance[slot];
}

w_instance rt_getfield_reference(w_instance instance, int slot) {
 
  w_instance field = (w_instance)instance[instance2clazz(instance)->instanceSize + slot];

  woempa(9, "get field %d of instance %p (%k): %p (%k)\n", slot, instance, instance2clazz(instance), field, instance2clazz(field));

  return field;
}

void rt_putfield_primitive(w_instance instance, int slot, j_word value) {

  woempa(9, "put %d in the local variable at slot %d of instance %p (%k)\n", value, slot, instance, instance2clazz(instance));
  
  instance[slot] = value;
}

void rt_putfield_reference(w_instance instance, int slot, j_word value) {

  woempa(9, "put instance %p (%k) in the local variable at slot %d of instance %p (%k)\n", value, instance2clazz((w_instance)value), slot, instance, instance2clazz(instance));

  setReferenceFieldUnsafe(instance, (w_instance)value, slot);
}

w_instance rt_new(w_clazz clazz) {

  w_instance instance;
  w_thread thread;
  
  thread = (w_thread)thread_current->xref;
  instance = allocInstance(thread, clazz);

  woempa(9, "created new instance %p of type '%k' (%p)\n", instance, clazz, clazz);
  
  return instance;
}

int rt_instanceof(w_instance instance, w_clazz clazz) {

  woempa(9, "instance %p of type %k, clazz %k\n", instance, instance2clazz(instance), clazz);
  
  if (instance == NULL) {
    
    /*
    ** If the object reference is 'null', the JVM is supposed to push 
    ** '0' on the operand stack.
    */
    
    return 0;
  }
  else {
    return isAssignmentCompatible(instance2clazz(instance), clazz);
  }
}

void rt_checkcast(w_instance instance, w_clazz clazz) {

  woempa(9, "instance %p of type %k, clazz %k\n", instance, instance2clazz(instance), clazz);

  if (instance && !isAssignmentCompatible(instance2clazz(instance), clazz)) {
    /*
    ** Throw a "ClassCastException" when the object reference is non-null 
    ** and if it can not be casted to the resolved type.
    */
    
    exception_throw(clazzClassCastException);
  }
}

int rt_invokenonvirtual_void(w_method method, x_frame frame, ...) {

  w_instance instance;
  w_thread thread;
 
  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokenonvirtual_lookup();
  invoke_method(frame, method);
  return_void();
}

int rt_invokenonvirtual_word(w_method method, x_frame frame, ...) {

  w_instance instance;
  w_thread thread;
  jvalue result;
 
  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokenonvirtual_lookup();
  invoke_method(frame, method);
  return_word();
}

int rt_invokevirtual_void(w_method method, x_frame frame, ...) {

  w_method actual_method;
  w_instance instance;
  w_thread thread;
  w_clazz clazz;

  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokevirtual_lookup();
  invoke_method(frame, actual_method);
  return_void();
}

int rt_invokevirtual_word(w_method method, x_frame frame, ...) {

  w_method actual_method;
  w_instance instance;
  w_thread thread;
  w_clazz clazz;
  jvalue result;

  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokevirtual_lookup();
  invoke_method(frame, actual_method);
  return_word();
}

int rt_invokespecial_void(w_method method, x_frame frame, ...) {
  
  w_instance instance;
  w_clazz super_clazz;  
  w_thread thread;
 
  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokespecial_lookup();
  invoke_method(frame, method);
  return_void();
}

int rt_invokespecial_word(w_method method, x_frame frame, ...) {
  
  w_instance instance;
  w_clazz super_clazz;  
  w_thread thread;
  jvalue result;
 
  setup_frame();
  instance = (w_instance)frame->Slots[0].c;
  invokespecial_lookup();
  invoke_method(frame, method);
  return_word();
}

int rt_invokestatic(w_method method, x_frame frame, ...) {
  
  w_thread thread;
  jvalue result;

  setup_frame();
  woempa(9, "'%M' calls static method '%M'\n", thread->top->method, method);
  invoke_method(frame, method);
  return_word();
}

int rt_invokeinterface(w_method method, x_frame frame, ...) {

  w_method actual_method;
  w_clazz actual_clazz;
  w_instance instance;
  w_thread thread;
  jvalue result;

  setup_frame();

  instance = (w_instance)frame->Slots[0].c;
 
  null_pointer_check(instance);
  
  /*
  ** Instance methods are always being invoked using an object.  Due to
  ** inheritance, the type of an instance - and hence the actual method
  ** to call - is known only at run-time.  However, all private methods,
  ** final methods and methods of final classes can be bound statically
  ** as they can not be inherited, and no look-up (dynamic binding) has
  ** to be performed.
  */

  actual_clazz = instance2clazz(instance);
  actual_method = interfaceLookup(method, actual_clazz);
  
  /*
  ** Throw an IcnompatibleClassChangeError if the resolved method is not 
  ** implemented by the specified instance.
  */
  
  if (actual_method == NULL) {
    exception_throw(clazzIncompatibleClassChangeError);
    return 0;
  }

  woempa(9, "'%M' calls interface method '%M' on instance %p (%k)\n", thread->top->method, actual_method, instance, instance2clazz(instance));

  invoke_method(frame, actual_method);
  return_word();
}

int rt_array_create(w_clazz clazz, int dimensions, ...) {
  
  w_instance instance;
  w_thread thread;
  int * dimension;
  va_list ap;
  int i;
    
  woempa(9, "create a %d-dimensional array of type '%k'\n", dimensions, clazz);
      
  thread = (w_thread)thread_current->xref;
 
  /*
  ** Retrieve the number of elements in each dimension of the array
  ** and load them in memory.  To make this thread-safe, we have to
  ** dynamically allocate the dimension-array.
  */
  
  dimension = x_mem_alloc(dimensions * sizeof(int));
    // TODO: this obviously adds too much overhead in case of one-dimensional arrays.

  va_start(ap, dimensions);
  for (i = 0; i < dimensions; i++) {
    dimension[i] = va_arg(ap, int);

    woempa(9, "dimension %d has size %d\n", i, dimension[i]);

    /*
    ** If any of the given dimensions is less than zero, throw a 
    ** NegativeArraySizeException.
    */
   
    if (dimension[i] < 0) { 
      x_mem_free(dimension);
      exception_throw(clazzNegativeArraySizeException);
      return 0;
    }
  }
  va_end(ap);

  instance = allocArrayInstance(thread, clazz, dimensions, dimension);

  x_mem_free(dimension);

  return (int)instance;
}

w_instance rt_array_load_reference(w_instance array, unsigned int index) {

  w_instance instance = (w_instance)instance2Array_instance(array)[index];
  
  woempa(9, "load value at slot %d from array %p of type '%k' and size %d\n", index, array, instance2clazz(array), instance2Array_length(array));
  
  return instance;
}

int rt_array_load_integer(w_instance array, unsigned int index) {

  woempa(9, "load value at slot %d from array %p of type '%k' and size %d\n", index, array, instance2clazz(array), instance2Array_length(array));
  
  return (int)instance2Array_int(array)[index];
}

int rt_array_load_short(w_instance array, unsigned int index) {

  woempa(9, "load value at slot %d from array %p of type '%k' and size %d\n", index, array, instance2clazz(array), instance2Array_length(array));
  
  return (int)instance2Array_short(array)[index];
}

int rt_array_load_character(w_instance array, unsigned int index) {

  woempa(9, "load value at slot %d from array %p of type '%k' and size %d\n", index, array, instance2clazz(array), instance2Array_length(array));
  
  return (int)instance2Array_char(array)[index];
}

int rt_array_load_float(w_instance array, unsigned int index) {

  woempa(9, "load value at slot %d from array %p of type '%k' and size %d\n", index, array, instance2clazz(array), instance2Array_length(array));
  
  return (int)instance2Array_float(array)[index];
}

void rt_array_store_reference(w_instance array, unsigned int index, w_instance value) {
 
  woempa(9, "store value %d into slot %d of array %p of type '%k' (%p) and size %d\n", value, index, array, instance2clazz(array), instance2clazz(array), instance2Array_length(array));

  /*
  ** If the type of the given instance is not assignment compatible
  ** with the type of the elements in the array, throw a 
  ** clazzArrayStoreException.
  */
  
  if (value && !isAssignmentCompatible(instance2clazz(value), instance2clazz(array)->previousDimension)) {
    exception_throw(clazzArrayStoreException);
    return;
  }

  woempa(9, "store value %d into slot %d of array %p of type '%k' and size %d\n", value, index, array, instance2clazz(array), instance2Array_length(array));
  
  instance2Array_int(array)[index] = (w_int)value;
    // NOTE: we can't use 'setArrayReferenceField()' as is can possibly enter a safe state.
}

void rt_array_store_integer(w_instance array, unsigned int index, int value) {
 
  woempa(9, "store value %d into slot %d of array %p of type '%k' and size %d\n", value, index, array, instance2clazz(array), instance2Array_length(array));
      
  instance2Array_int(array)[index] = (w_int)value;
}

void rt_array_store_short(w_instance array, unsigned int index, int value) {
  
  woempa(9, "store value %d into slot %d of array %p of type '%k' and size %d\n", value, index, array, instance2clazz(array), instance2Array_length(array));
      
  instance2Array_short(array)[index] = (w_short)value;
}

void rt_array_store_character(w_instance array, unsigned int index, int value) {
  
  woempa(9, "store value %d into slot %d of array %p of type '%k' and size %d\n", value, index, array, instance2clazz(array), instance2Array_length(array));
      
  instance2Array_char(array)[index] = (w_char)value;
}

void rt_array_store_float(w_instance array, unsigned int index, int value) {
 
  woempa(9, "store value %d into slot %d of array %p of type '%k' and size %d\n", value, index, array, instance2clazz(array), instance2Array_length(array));
      
  instance2Array_float(array)[index] = (w_float)value;
}

int rt_array_length(w_instance array) {
 
  woempa(9, "return length of array %p of type '%k'\n", array, instance2clazz(array));
  
  return instance2Array_length(array);
}

void rt_check_array_bounds(w_instance array, int index) {

  woempa(9, "check index %d of array %p (%k) with size %d\n", index, array, instance2clazz(array), instance2Array_length(array));

  if (index < 0 || index >= (unsigned int)instance2Array_length(array)) { 
    exception_throw(clazzArrayIndexOutOfBoundsException);
  } 
}

int rt_float_compare_g(w_float value1, w_float value2) {

  if (float32_is_NaN(value1) || float32_is_NaN(value2) ) {
    return 1;
  }
  else if (float32_eq(value1, value2)) {
    return 0;
  }
  else if (float32_lt(value1, value2)) {
    return -1;
  }
  else {
    return 1;
  }
}


int rt_float_compare_l(w_float value1, w_float value2) {

  if (float32_is_NaN(value1) || float32_is_NaN(value2) ) {
    return -1;
  }
  else if (float32_eq(value1, value2)) {
    return 0;
  }
  else if (float32_lt(value1, value2)) {
    return -1;
  }
  else {
    return 1;
  }
}

void * putfield_dispatcher[] = {
  rt_putfield_reference,       // 0
  rt_putfield_primitive,       // 1
};

void * array_load_dispatcher[] = {
  /*
  ** Perform no null pointer check and no array bound.
  */
 
  rt_array_load_reference,     // 0 
  rt_array_load_integer,       // 1
  rt_array_load_short,         // 2
  rt_array_load_character,     // 3
  rt_array_load_float,         // 4
  NULL,                        // 5 (long, to be implemented)
  NULL,                        // 6 (double, to be implemented)
  NULL                         // 7 (byte or boolean, to be implemented)
};

void * array_store_dispatcher[] = {
  /*
  ** Perform no null pointer check and no array bound.
  */
  
  rt_array_store_reference,    // 0 
  rt_array_store_integer,      // 1
  rt_array_store_short,        // 2
  rt_array_store_character,    // 3
  rt_array_store_float,        // 4
  NULL,                        // 5 (long, to be implemented)
  NULL,                        // 6 (double, to be implemented)
  NULL                         // 7 (byte or boolean, to be implemented)
};

void * invokenonvirtual_dispatcher[] = {

  rt_invokenonvirtual_void,         // 0 (return nothing)
  rt_invokenonvirtual_word,         // 1 (return word)
};

void * invokevirtual_dispatcher[] = {

  rt_invokevirtual_void,            // 0 (return nothing)
  rt_invokevirtual_word,            // 1 (return word)
};

void * invokespecial_dispatcher[] = {

  rt_invokespecial_void,            // 0 (return nothing)
  rt_invokespecial_word,            // 1 (return word)
};


