/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: statistics.c 3907 2003-05-07 14:32:15Z dbuytaert $
*/

#include "j-spot.h"
#include "methods.h"
#include "threads.h"

#ifdef DEBUG

// Prototypes of functions exported by the emitted code generator:
void burg_statistics(void);

int total_machine_code_size = 0;
int total_bytecode_size = 0;
int total_memory_usage = 0;
int total_compiled_methods = 0;
int total_null_pointer_checks = 0;
int total_array_bounds_checks = 0;
int eliminated_null_pointer_checks = 0;
int eliminated_array_bounds_checks = 0;
int total_spills = 0;
long long total_compile_time = 0;

void statistics_method(int bytecode_size, int machine_code_size, int memory_usage, int compile_time) {

  static FILE *fd;
  
  if (fd == NULL) {
    fd = fopen("statistics.txt", "w");
    fprintf(fd, "# bytecode size  |  machine code size  |  memory usage  |  compile time\n");
  }
  
  if (machine_code_size) {
    total_compiled_methods += 1;

    total_bytecode_size += bytecode_size; 
    total_machine_code_size += machine_code_size;
    total_memory_usage += memory_usage;
    total_compile_time += compile_time;
    /*
    ** Log the data to file:
    */
    
    fprintf(fd, "%15d %15d %15d %15d\n", bytecode_size, machine_code_size, memory_usage, compile_time);
    fflush(fd);
  }
}

void statistics_null_pointer(int eliminated) {
  eliminated_null_pointer_checks += eliminated;
  total_null_pointer_checks += 1;
}

void statistics_array_bounds(int eliminated) {
  eliminated_array_bounds_checks += eliminated;
  total_array_bounds_checks += 1;
}

void statistics_spill(void) {
  total_spills += 1;
}

void statistics_dump(void) {
  
  if (total_compiled_methods) {
    fprintf(stderr, "--- compiler statistics ------------------------------------------------------------\n");
    fprintf(stderr, "* number of compiled methods.....: %d\n", total_compiled_methods);
    fprintf(stderr, "* accumulated bytecode size......: %d bytes in total, %d bytes on average\n", total_bytecode_size, total_bytecode_size / total_compiled_methods);
    fprintf(stderr, "* accumulated machine code size..: %d bytes in total, %d bytes on average\n", total_machine_code_size, total_machine_code_size / total_compiled_methods);
    fprintf(stderr, "* accumulated memory usage.......: %d bytes in total, %d bytes on average\n", total_memory_usage, total_memory_usage / total_compiled_methods);
    fprintf(stderr, "* accumulated compile time.......: %lld usec in total, %lld usec on average\n", total_compile_time, total_compile_time / total_compiled_methods);
    fprintf(stderr, "* total number of spills.........: %d\n", total_spills);
    fprintf(stderr, "* null pointer checks............: eliminated %d checks out of %d\n", eliminated_null_pointer_checks, total_null_pointer_checks);
    fprintf(stderr, "* array bounds checks............: eliminated %d checks out of %d\n", eliminated_array_bounds_checks, total_array_bounds_checks);
    fprintf(stderr, "* code generator statistics......:\n");
    burg_statistics();
    fprintf(stderr, "------------------------------------------------------------------------------------\n");
  }
}

#else 

void statistics_method(int bytecode_size, int machine_code_size, int memory_usage, int compile_time) {
}

void statistics_spill(void) {
}

void statistics_dump(void) {
}

void statistics_null_pointer(int eliminated) {
}

void statistics_array_bounds(int eliminated) {
}

#endif

