/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: translate.c 3991 2003-06-06 09:13:49Z dbuytaert $
*/

#include "methods.h"

#include "j-spot.h"
#include "arena.h"
#include "ir.h"
#include "mimic-stack.h"
#include "ct-hooks.h"
#include "rt-hooks.h"
#include "bytecode.h"

static void save_pc(j_node node, int pc) {

  jm->todo[jm->next][0] = pc;
  jm->todo[jm->next][1] = jm->sp;
  jm->todo[jm->next][2] = jm->inst_pc;
  jm->next += 1;
}
      
static int next_pc(void) {

  int pp = 0;
  int pc = jm->length;

  if (jm->next > 0) {
    jm->next -= 1;
    
    pc = jm->todo[jm->next][0];
    jm->sp = jm->todo[jm->next][1];
    pp = jm->todo[jm->next][2];

    ms_load_state(pp, j_post_execution); 
  }

  return pc;
}

static inline void load_wrapper(j_type type, int index) {

  j_node node;
    
  /*
  ** Push or load the local variable 'number' of the current frame onto
  ** the operand stack.
  */

  // get the node from the local variables:
  node = ms_get_local(type, index);
  
  // push the node on the operand stack:
  ms_push(node);
  
  // debug output: 
  woempa(9, "load_wrapper: %q\n", node);
}

static inline void store_wrapper(j_type type, int index) {

  j_node node1;
  j_node node2;
  j_node node3;
  
  /*
  ** Store the local variable at the top of the operand stack to local
  ** variable 'number'.
  */

  // pop the node from the operand stack:
  node1 = ms_pop(type);

  // get the node from the local variables:
  node2 = ms_set_local(type, index);
  
  // create a new node:
  node3 = node_create_insn(j_assign, node1, node2);

  // mark the node as a top-level node:
  ir_add_node(node3, j_link_next, j_link_none);

  // debug output:
  woempa(9, "store_wrapper: %q\n", node3);
}

static inline void array_create_wrapper(int dimensions, j_node type) {

  int i;
  j_node node;
  j_node temp;
  
  /*
  ** Allocate a new one or multi-dimensional array of the specified type.
  */

  // we create a function node that will call 'rt_array_create':
  node = node_create_call("array_create", j_native_call, j_address, &rt_array_create, 2 + dimensions);
  
  // Initialize the size at 1:  
  node->array_size = 1;

  for (i = 0; i < dimensions; i++) {
    // pop the dimensions from the operand stack:
    temp = ms_pop(j_integer);

    // try to determine whether the array has a constant size:
    if (temp->operator == j_constant) {
      node->array_size = node->array_size * temp->data.constant.value;
    }
    else {
      node->array_size = 0;
    }

    // pass the size of each dimensions to the 'rt_array_create' function:
    node_add_param(node, temp);
  }
 
  // add the number of dimensions to the parameter list:
  node_add_param(node, node_create_constant(j_integer, dimensions));

  // add the array type to the parameter list:
  node_add_param(node, type);

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "array_create_wrapper: %q\n", node);
}

static inline void array_load_wrapper(j_type type, int offset) {
  
  void * function;
  
  j_node node;
  j_node index;
  j_node array;
 
  /*
  ** Load a value from an array.
  */

  index = ms_pop(j_integer);
  array = ms_pop(j_address);

  // determine what function to call:
  function = array_load_dispatcher[offset];

  // we create a function node that will call the resolved function:
  node = node_create_call("array_load", j_native_call, type, function, 2);
  
  // add the index or location to store the value into:
  node_add_param(node, index);

  // add the array reference to perform the operation on:
  node_add_param(node, array);
 
  // add a null pointer check:
  if (node_check_null_pointer(array)) {
    ir_add_node(node_create_insn(j_null_pointer, array, NULL), j_link_next, j_link_none);
  }

  // add an array bounds check:
  if (node_check_array_bounds(array, index)) {
    ir_add_node(node_create_insn(j_array_bounds, array, index), j_link_next, j_link_none);
  }

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "array_load_wrapper: %q\n", node);
}

static inline void array_store_wrapper(j_type type, int offset) {
  
  void * function;
  j_node node;
  j_node value;
  j_node index;
  j_node array;
  
  /*
  ** Store a value into an array.
  */

  value = ms_pop(type);
  index = ms_pop(j_integer);
  array = ms_pop(j_address);
  
  // determine what function to call:
  function = array_store_dispatcher[offset];
  
  // we create a function node that will call the resolved function:
  node = node_create_call("array_store", j_native_call, type, function, 3);
  
  // add the value to store:
  node_add_param(node, value);
  
  // add the index or location to store the value into:
  node_add_param(node, index);

  // add the array reference to perform the operation on:
  node_add_param(node, array);

  // add a null pointer check:
  if (node_check_null_pointer(array)) {
    ir_add_node(node_create_insn(j_null_pointer, array, NULL), j_link_next, j_link_none);
  }

  // add an array bounds check:
  if (node_check_array_bounds(array, index)) {
    ir_add_node(node_create_insn(j_array_bounds, array, index), j_link_next, j_link_none);
  }


  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
  
  // debug output: 
  woempa(9, "array_store_wrapper: %q\n", node);
}

static inline void push_wrapper(j_type type, int value) {

  j_node node;
   
  /*
  ** Push a constant onto the operand stack.
  */

  // create a new constant:
  node = node_create_constant(type, value);

  // push the node on the mimic operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "push_wrapper: %q\n", node);
}

static inline void ldc_wrapper(int value) {

  j_node node;
    
  /*
  ** Push an item from the constant pool.
  */

  // create a new constant:
  node = ct_load_constant(value);
  
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "ldc_wrapper: %q\n", node);
}

static inline void una_wrapper(j_operator operator, j_type type) {

  j_node node1;
  j_node node2;
    
  // pop the node from the operand stack:
  node1 = ms_pop(type);

  if (node1->operator == j_constant) {
    node2 = NULL;
    jabort("perform constant propagation\n");
  }
  else {
    // create a new node:
    node2 = node_create_insn(operator, node1, 0);
  }
  // push the node on the operand stack:
  ms_push(node2);

  // debug output:
  woempa(9, "una_wrapper: %q\n", node2);
}

static inline void bin_wrapper(j_operator operator, j_type type) {

  j_node node1;
  j_node node2;
  j_node node3;
  
  node3 = NULL;

  // pop the nodes from the operand stack:
  node2 = ms_pop(type);
  node1 = ms_pop(type);

  if (node1->operator == j_constant && node2->operator == j_constant) {
    switch (operator) {
      case j_add:
        node3 = node_create_constant(type, node1->data.constant.value + node2->data.constant.value);
        break;
      case j_sub:
        node3 = node_create_constant(type, node1->data.constant.value - node2->data.constant.value);
        break;
      case j_mul:
        node3 = node_create_constant(type, node1->data.constant.value * node2->data.constant.value);
        break;
      case j_and:
        node3 = node_create_constant(type, node1->data.constant.value & node2->data.constant.value);
        break;
      case j_xor:
        node3 = node_create_constant(type, node1->data.constant.value ^ node2->data.constant.value);
        break;
      case j_or:
        node3 = node_create_constant(type, node1->data.constant.value | node2->data.constant.value);
        break;
      default:
        jabort("perform constant propagation\n");
    }
  }
  else { 
    // create a new node:
    node3 = node_create_insn(operator, node1, node2);
  }
  
  // push the node on the operand stack:
  ms_push(node3);

  // debug output:
  woempa(9, "bin_wrapper: %q\n", node3);
}

static inline void shift_wrapper(j_operator operator, j_type type) {

  j_node node1;
  j_node node2;
  j_node node3;
 
  // pop the nodes from the operand stack:
  node2 = ms_pop(type);
  node1 = ms_pop(type);

  if (node1->operator == j_constant && node2->operator == j_constant) {
    node3 = NULL;
    jabort("perform constant propagation\n");
  }
  else {
    // create a new node:
    node3 = node_create_insn(operator, node1, node2);
  }
  
  // push the node on the operand stack:
  ms_push(node3);

  // debug output:
  woempa(9, "shift_wrapper: %q\n", node3);
}

static inline void x2y_wrapper(j_type x, j_type y) {

  j_node node = NULL;

  if (x == j_integer && y == j_byte) {
    // we create a function node that will call 'rt_integer2byte':
    node = node_create_call("insteger2byte", j_native_call, j_byte, &rt_integer2byte, 1);
  }
  else if (x == j_integer && y == j_character) {
    // we create a function node that will call 'rt_integer2character':
    node = node_create_call("insteger2character", j_native_call, j_character, &rt_integer2character, 1);
  }
  else if (x == j_integer && y == j_float) {
    // we create a function node that will call 'rt_integer2float':
    node = node_create_call("integer2float", j_native_call, j_float, &rt_integer2float, 1);
  }
  else if (x == j_integer && y == j_short) {
    // we create a function node that will call 'rt_integer2short':
    node = node_create_call("integer2short", j_native_call, j_short, &rt_integer2short, 1);
  }
  else if (x == j_float && y == j_integer) {
    // we create a function node that will call 'rt_float2integer':
    node = node_create_call("float2integer", j_native_call, j_integer, &rt_float2integer, 1);
  }
  else {
    jabort("the specified type conversion \"%d to %d\" is not supported yet\n", jm->method, x, y);
  }

  // add the operand to the parameter list:
  node_add_param(node, ms_pop(x));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);   
  
  // push the node on the operand stack:
  ms_push(node);

  // debug output:
  woempa(9, "x2y_wrapper: %q\n", node);
}

static inline void cmp_wrapper(j_type type, int number) {
  jabort("we should not get here\n");
}

static inline void ifx_wrapper(j_operator operator, j_type type) {

  int offset;
  j_node node1;
  j_node node2;
  j_node node3;
  
  /*
  ** Pops two nodes from the operand stack for comparison.  If
  ** the comparision succeeds, an offset is calculated from the
  ** two bytes that follow the bytecode.
  */
  
  // get the branch offset:
  offset = bytecode_get_short();

  // pop the nodes from the operand stack:
  node1 = ms_pop(type);
  node2 = ms_pop(type);

  // create a new node:
  node3 = node_create_insn(operator, node2, node1);

  // mark the node as a top-level node:
  ir_add_node(node3, jm->inst_pc + offset, j_link_next);

  // add the second branch to our internal 'todo'-list:
  save_pc(node3, jm->inst_pc + offset);

  // debug output: 
  woempa(9, "ifx_wrapper: %q\n", node3);
}

static inline void if0_wrapper(j_operator operator, j_type type) {

  int offset;
  j_node node1;
  j_node node2;
  j_node node3;
 
  /*
  ** The if0_wrapper compares the top of stack against zero and 
  ** branches if the comparison succeeds.
  */

  // get the branch offset:
  offset = bytecode_get_short();

  // create a new constant:
  node1 = node_create_constant(type, 0);
  
  // pop the nodes from the operand stack:
  node2 = ms_pop(type);
  
  // create a new node:
  node3 = node_create_insn(operator, node2, node1);

  // mark the node as a top-level node:
  ir_add_node(node3, jm->inst_pc + offset, j_link_next);

  // add the second branch to our internal 'todo'-list:
  save_pc(node3, jm->inst_pc + offset);

  /*
  ** Save the active state information (post state):
  */

  ms_save_state(jm->inst_pc, j_post_execution);

  // debug output: 
  woempa(9, "if0_wrapper: %q\n", node3);
}

static inline void iinc_wrapper(int value, int index) {

  j_node node1;
  j_node node2;
  j_node node3;
  j_node node4;
  j_node node5;
  
  /*
  ** Increment the local variable at the specified by "index" by the given
  ** value.
  */
  
  // get the local variable from the local variable pool:  
  node1 = ms_get_local(j_integer, index);

  // create a new constant:
  node2 = node_create_constant(j_integer, value);

  // create a new node:
  node3 = node_create_insn(j_add, node1, node2);
 
  // set the local variable in the local variable pool:
  node4 = ms_set_local(j_integer, index);
  
  // create a new node:
  node5 = node_create_insn(j_assign, node3, node4);

  // mark the node as a top-level node:
  ir_add_node(node5, j_link_next, j_link_none);
  
  // debug output:
  woempa(9, "iinc_wrapper: %q\n", node4);
}

static inline void goto_wrapper(int offset) {

  j_node node;
 
  // create a new node:
  node = node_create_insn(j_jump, 0, 0);

  // mark the node as a top-level node:
  ir_add_node(node, jm->inst_pc + offset, j_link_none);

  // jump to the target instruction:
  jm->curr_pc = jm->inst_pc + offset;

  // debug output:
  woempa(9, "goto: %q\n", node);
}

static inline void return_wrapper(j_type type) {

  j_node node;

  // pop the node from the operand stack:
  node = ms_pop(type);
  
  // create a new node:
  node = node_create_insn(j_return, node, 0);

  // mark the node as a top-level node:
  ir_add_node(node, j_link_none, j_link_none);
}

static inline void 
translate_aconst_null(void) {
  push_wrapper(j_address, 0);
}

static inline void 
translate_iconst_m1(void) {
  push_wrapper(j_integer, -1);
}

static inline void 
translate_iconst_0(void) {
  push_wrapper(j_integer, 0);
}

static inline void 
translate_iconst_1(void) {
  push_wrapper(j_integer, 1);
}

static inline void 
translate_iconst_2(void) {
  push_wrapper(j_integer, 2);
}

static inline void 
translate_iconst_3(void) {
  push_wrapper(j_integer, 3);
}

static inline void 
translate_iconst_4(void) {
  push_wrapper(j_integer, 4);
}

static inline void 
translate_iconst_5(void) {
  push_wrapper(j_integer, 5);
}

static inline void translate_lconst_0(void) {
  push_wrapper(j_long, 0);
}

static inline void translate_lconst_1(void) {
  push_wrapper(j_long, 1);
}

static inline void translate_fconst_0(void) {
  push_wrapper(j_float, F_ZERO);  // F_ZERO is defined in "ieee754.h"
}

static inline void translate_fconst_1(void) {
  push_wrapper(j_float, F_ONE);   // F_ONE is defined in "ieee754.h"
}

static inline void translate_fconst_2(void) {
  push_wrapper(j_float, F_TWO);   // F_TWO is defined in "ieee754.h"
}

static inline void translate_dconst_0(void) {
  push_wrapper(j_double, 0);
}

static inline void translate_dconst_1(void) {
  push_wrapper(j_double, 1);
}

static inline void translate_bipush(void) {
  push_wrapper(j_integer, (signed int)(signed char)bytecode_get_byte());
    // NOTE: we have to sign-extend the byte operand before casting it to a signed integer.
}

static inline void translate_sipush(void) {
  push_wrapper(j_integer, (signed int)(signed short)bytecode_get_short());
    // NOTE: we have to sign-extend the short operand before casting it to a signed integer.
}

static inline void translate_ldc(void) {
  ldc_wrapper(bytecode_get_byte());
}

static inline void translate_ldc_w(void) {
  ldc_wrapper(bytecode_get_short());
}

static inline void translate_ldc2_w (void) {
  ldc_wrapper(bytecode_get_short());
}

static inline void translate_iload(void) {
  load_wrapper(j_integer, bytecode_get_byte());
}

static inline void translate_wide_iload(void) {
  load_wrapper(j_integer, bytecode_get_short());
}

static inline void translate_lload(void) {
  load_wrapper(j_long, bytecode_get_byte());
}

static inline void translate_wide_lload(void) {
  load_wrapper(j_long, bytecode_get_short());
}

static inline void translate_fload(void) {
  load_wrapper(j_float, bytecode_get_byte());
}

static inline void translate_wide_fload(void) {
  load_wrapper(j_float, bytecode_get_short());
}

static inline void translate_dload(void) {
  load_wrapper(j_double, bytecode_get_byte());  
}

static inline void translate_wide_dload(void) {
  load_wrapper(j_double, bytecode_get_short());
}

static inline void translate_aload(void) {
  load_wrapper(j_address, bytecode_get_byte());
}

static inline void translate_wide_aload(void) {
  load_wrapper(j_address, bytecode_get_short());
}

static inline void translate_iload_0(void) {
  load_wrapper(j_integer, 0);
}

static inline void translate_iload_1(void) {
  load_wrapper(j_integer, 1);
}

static inline void translate_iload_2(void) {
  load_wrapper(j_integer, 2);
}

static inline void translate_iload_3(void) {
  load_wrapper(j_integer, 3);
}

static inline void translate_lload_0(void) {
  load_wrapper(j_long, 0);
}

static inline void translate_lload_1(void) {
  load_wrapper(j_long, 1);
}

static inline void translate_lload_2(void) {
  load_wrapper(j_long, 2);
}

static inline void translate_lload_3(void) {
  load_wrapper(j_long, 3);
}

static inline void translate_fload_0(void) {
  load_wrapper(j_float, 0);
}

static inline void translate_fload_1(void) {
  load_wrapper(j_float, 1);
}

static inline void translate_fload_2(void) {
  load_wrapper(j_float, 2);
}

static inline void translate_fload_3(void) {
  load_wrapper(j_float, 3);
}

static inline void translate_dload_0(void) {
  load_wrapper(j_double, 0);
}

static inline void translate_dload_1(void) {
  load_wrapper(j_double, 1);
}

static inline void translate_dload_2(void) {
  load_wrapper(j_double, 2);
}

static inline void translate_dload_3(void) {
  load_wrapper(j_double, 3);
}

static inline void translate_aload_0(void) {
  load_wrapper(j_address, 0);
}

static inline void translate_aload_1(void) {
  load_wrapper(j_address, 1);
}

static inline void translate_aload_2(void) {
  load_wrapper(j_address, 2);
}

static inline void translate_aload_3(void) {
  load_wrapper(j_address, 3);
}

static inline void translate_aaload(void) {
  array_load_wrapper(j_address, 0);
}

static inline void translate_iaload(void) {
  array_load_wrapper(j_integer, 1);
}

static inline void translate_saload(void) {
  array_load_wrapper(j_short, 2);
}

static inline void translate_caload(void) {
  array_load_wrapper(j_character, 3);
}

static inline void translate_faload(void) {
  array_load_wrapper(j_float, 4);
}

static inline void translate_laload(void) {
  array_load_wrapper(j_long, 5);
}

static inline void translate_daload(void) {
  array_load_wrapper(j_double, 6); 
}

static inline void translate_baload(void) {
  array_load_wrapper(j_byte, 7);  // or j_boolean depending on type of array
}

static inline void translate_istore(void) {
  store_wrapper(j_integer, bytecode_get_byte());
}

static inline void translate_wide_istore(void) {
  store_wrapper(j_integer, bytecode_get_short());
}

static inline void translate_lstore(void) {
  store_wrapper(j_long, bytecode_get_byte());
}

static inline void translate_wide_lstore(void) {
  store_wrapper(j_long, bytecode_get_short());
}

static inline void translate_fstore(void) {
  store_wrapper(j_float, bytecode_get_byte());
}

static inline void translate_wide_fstore(void) {
  store_wrapper(j_float, bytecode_get_short());
}

static inline void translate_dstore(void) {
  store_wrapper(j_double, bytecode_get_byte());
}

static inline void translate_wide_dstore(void) {
  store_wrapper(j_double, bytecode_get_short());
}

static inline void translate_astore(void) {
  store_wrapper(j_address, bytecode_get_byte());
}

static inline void translate_wide_astore(void) {
  store_wrapper(j_address, bytecode_get_short());
}

static inline void translate_istore_0(void) {
  store_wrapper(j_integer, 0);
}

static inline void translate_istore_1(void) {
  store_wrapper(j_integer, 1);
}

static inline void translate_istore_2(void) {
  store_wrapper(j_integer, 2);
}

static inline void translate_istore_3(void) {
  store_wrapper(j_integer, 3);
}

static inline void translate_lstore_0(void) {
  store_wrapper(j_long, 0);
}

static inline void translate_lstore_1(void) {
  store_wrapper(j_long, 1);
}

static inline void translate_lstore_2(void) {
  store_wrapper(j_long, 2);
}

static inline void translate_lstore_3(void) {
  store_wrapper(j_long, 3);
}

static inline void translate_fstore_0(void) {
  store_wrapper(j_float, 0);
}

static inline void translate_fstore_1(void) {
  store_wrapper(j_float, 1);
}

static inline void translate_fstore_2(void) {
  store_wrapper(j_float, 2); 
}

static inline void translate_fstore_3(void) {
  store_wrapper(j_float, 3);
}

static inline void translate_dstore_0(void) {
  store_wrapper(j_double, 0);
}

static inline void translate_dstore_1(void) {
  store_wrapper(j_double, 1);
}

static inline void translate_dstore_2(void) {
  store_wrapper(j_double, 2);
}

static inline void translate_dstore_3(void) {
  store_wrapper(j_double, 3);
}

static inline void translate_astore_0(void) {
  store_wrapper(j_address, 0);
}

static inline void translate_astore_1(void) {
  store_wrapper(j_address, 1);
}

static inline void translate_astore_2(void) {
  store_wrapper(j_address, 2);
}

static inline void translate_astore_3(void) {
  store_wrapper(j_address, 3);
}

static inline void translate_aastore(void) {
  array_store_wrapper(j_address, 0);
}

static inline void translate_iastore(void) {
  array_store_wrapper(j_integer, 1);
}

static inline void translate_sastore(void) {
  array_store_wrapper(j_short, 2);
}

static inline void translate_castore(void) {
  array_store_wrapper(j_character, 3);
}

static inline void translate_fastore(void) {
  array_store_wrapper(j_float, 4);
}
 
static inline void translate_lastore(void) {
  array_store_wrapper(j_long, 5);
}

static inline void translate_dastore(void) {
  array_store_wrapper(j_double, 6);
}

static inline void translate_bastore(void) {
  array_store_wrapper(j_byte, 7);  // or j_boolean depending on type of array
}

static inline void translate_pop(void) {
  ms_pop(j_unknown);
}

static inline void translate_pop2(void) {
  ms_pop(j_unknown);
  ms_pop(j_unknown);
}

static inline void translate_dup(void) {
  ms_dup1();
}

static inline void translate_dup_x1(void) {
  ms_dup1_x1();
}

static inline void translate_dup_x2(void) {
  ms_dup1_x2();
}

static inline void translate_dup2(void) {
  ms_dup2();
}

static inline void translate_dup2_x1(void) {
  ms_dup1_x1();
}

static inline void translate_dup2_x2(void) {
  ms_dup1_x2();
}

static inline void translate_swap(void) {
  ms_swap();
}

static inline void translate_iadd(void) {
  bin_wrapper(j_add, j_integer);
}

static inline void translate_ladd(void) {
  bin_wrapper(j_add, j_long);
}

static inline void translate_fadd(void) {

  j_node node;
    
  /*
  ** Add two floats.
  */

  // we create a function node that will call 'rt_float_add':
  node = node_create_call("float_add", j_native_call, j_float, &rt_float_add, 2);
  
  // add the operands to the parameter list:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_fadd: %q\n", node);
}

static inline void translate_dadd(void) {
  bin_wrapper(j_add, j_double);
}

static inline void translate_isub(void) {
  bin_wrapper(j_sub, j_integer);
}

static inline void translate_lsub(void) {
  bin_wrapper(j_sub, j_long);
}

static inline void translate_fsub(void) {

  j_node node;
    
  /*
  ** Subtract two floats.
  */

  // we create a function node that will call 'rt_float_sub':
  node = node_create_call("float_sub", j_native_call, j_float, &rt_float_sub, 2);
  
  // add the operands to the parameter list:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_fsub: %q\n", node);
}

static inline void translate_dsub(void) {
  bin_wrapper(j_sub, j_double);
}

static inline void translate_imul(void) {
  bin_wrapper(j_mul, j_integer);
}

static inline void translate_lmul(void) {
  bin_wrapper(j_mul, j_long);
}

static inline void translate_fmul(void) {

  j_node node;
    
  /*
  ** Multiply two floats.
  */

  // we create a function node that will call 'rt_float_mul':
  node = node_create_call("float_mul", j_native_call, j_float, &rt_float_mul, 2);
  
  // add the operands to the parameter list:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_fmul: %q\n", node);
}

static inline void translate_dmul(void) {
  bin_wrapper(j_mul, j_double);
}

static inline void translate_idiv(void) {
  
  j_node node;
    
  /*
  ** Divide two integers.
  */

  // we create a function node that will call 'rt_integer_div':
  node = node_create_call("integer_div", j_native_call, j_integer, &rt_integer_div, 2);
  
  // add the values to devide:
  node_add_param(node, ms_pop(j_integer));
  node_add_param(node, ms_pop(j_integer));
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
  
  // push the node on the operand stack:
  ms_push(node);
 
  // debug output: 
  woempa(9, "translate_idiv: %q\n", node);
}

static inline void translate_ldiv(void) {
}

static inline void translate_fdiv(void) {

  j_node node;
    
  /*
  ** Divide two floats.
  */

  // we create a function node that will call 'rt_float_div':
  node = node_create_call("float_div", j_native_call, j_float, &rt_float_div, 2);
  
  // add the operands to the parameter list:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_fdiv: %q\n", node);
}
   
static inline void translate_ddiv(void) {
}

static inline void translate_irem(void) {

  j_node node;
    
  /*
  ** Calculate the remainder of an integer division.
  */

  // we create a function node that will call 'rt_integer_rem':
  node = node_create_call("integer_rem", j_native_call, j_integer, &rt_integer_rem, 2);
  
  // add the values to devide:
  node_add_param(node, ms_pop(j_integer));
  node_add_param(node, ms_pop(j_integer));
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
  
  // push the node on the operand stack:
  ms_push(node);
 
  // debug output: 
  woempa(9, "translate_irem: %q\n", node);
}

static inline void translate_lrem(void) {
  bin_wrapper(j_rem, j_long);
}

static inline void translate_frem(void) {

  j_node node;
    
  /*
  ** Calculate the remainder of a floats division.
  */

  // we create a function node that will call 'rt_float_rem':
  node = node_create_call("float_rem", j_native_call, j_float, &rt_float_rem, 2);
  
  // add the operands to the parameter list:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_frem: %q\n", node);
}

static inline void translate_iseq(void) {
  una_wrapper(j_iseq, j_integer);

  /*
  ** Advance the current program counter by 7; the "iseq" instruction
  ** is a replacement for a common bytecode sequence or idiom.
  */

  jm->curr_pc += 7;
}

static inline void translate_isne(void) {
  una_wrapper(j_isne, j_integer);

  /*
  ** Advance the current program counter by 7; the "isne" instruction
  ** is a replacement for a common bytecode sequence or idiom.
  */

  jm->curr_pc += 7;
}


static inline void translate_drem(void) {
  bin_wrapper(j_rem, j_double);
}

static inline void translate_ineg(void) {
  una_wrapper(j_neg, j_integer);
}

static inline void translate_lneg(void) {
  una_wrapper(j_neg, j_long);
}

static inline void translate_fneg(void) {

  j_node node;
    
  /*
  ** Negate a float.
  */

  // we create a function node that will call 'rt_float_neg':
  node = node_create_call("float_reg", j_native_call, j_float, &rt_float_neg, 1);
  
  // add the operand to the parameter list:
  node_add_param(node, ms_pop(j_float));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "translate_fneg: %q\n", node);
}

static inline void translate_dneg(void) {
  una_wrapper(j_neg, j_double);
}

static inline void translate_ishl(void) {
  shift_wrapper(j_lsh, j_integer);
}

static inline void translate_lshl(void) {
  shift_wrapper(j_lsh, j_long);
}

static inline void translate_ishr(void) {
  shift_wrapper(j_rsh, j_integer);
}

static inline void translate_lshr(void) {
  shift_wrapper(j_rsh, j_long);
}

static inline void translate_iushr(void) {
  shift_wrapper(j_rsh, j_integer);
}

static inline void translate_lushr(void) {
  shift_wrapper(j_rsh, j_long);
}

static inline void translate_iand(void) {
  bin_wrapper(j_and, j_integer);
}

static inline void translate_land(void) {
  bin_wrapper(j_and, j_long);
}

static inline void translate_ior(void) {
  bin_wrapper(j_or, j_integer);
}

static inline void translate_lor(void) {
  bin_wrapper(j_or, j_long);
}

static inline void translate_ixor(void) {
  bin_wrapper(j_xor, j_integer);
}

static inline void translate_lxor(void) {
  bin_wrapper(j_xor, j_long);
}

static inline void translate_iinc(void) {
  
  int index = bytecode_get_byte();
  int value = (signed int)(signed char)bytecode_get_byte();
    // NOTE: we have to sign-extend the byte operand before casting it to a signed integer.
  
  iinc_wrapper(value, index);
}

static inline void translate_wide_iinc(void) {

  int index = bytecode_get_short();
  int value = (signed int)(signed short)bytecode_get_short();
    // NOTE: we have to sign-extend the short operand before casting it to a signed integer.

  iinc_wrapper(value, index);
}

static inline void translate_i2l(void) {
  x2y_wrapper(j_integer, j_long);
}

static inline void translate_i2f(void) {
  x2y_wrapper(j_integer, j_float);
}

static inline void translate_i2d(void) {
  x2y_wrapper(j_integer, j_double);
}

static inline void translate_l2i(void) {
  x2y_wrapper(j_long, j_integer);
}

static inline void translate_l2f(void) {
  x2y_wrapper(j_long, j_float);
}

static inline void translate_l2d(void) {
  x2y_wrapper(j_long, j_double);
}

static inline void translate_f2i(void) {
  x2y_wrapper(j_float, j_integer);
}

static inline void translate_f2l(void) {
  x2y_wrapper(j_float, j_long);
}

static inline void translate_f2d(void) {
  x2y_wrapper(j_float, j_double);
}

static inline void translate_d2i(void) {
  x2y_wrapper(j_double, j_integer);
}

static inline void translate_d2l(void) {
  x2y_wrapper(j_double, j_long);
}
 
static inline void translate_d2f(void) {
  x2y_wrapper(j_double, j_float);
}
 
static inline void translate_i2b(void) {
  x2y_wrapper(j_integer, j_byte);
}
 
static inline void translate_i2c(void) {
  x2y_wrapper(j_integer, j_character);
}
 
static inline void translate_i2s(void) {
  x2y_wrapper(j_integer, j_short);
}
 
static inline void translate_lcmp(void) {
  cmp_wrapper(j_long, j_long);
}
 
static inline void translate_fcmpg(void) {
  
  j_node node;
    
  /*
  ** Compare two floats with NaN check included
  */

  // we create a function node that will call 'rt_float_compare':
  node = node_create_call("fcmpg", j_native_call, j_integer, &rt_float_compare_g, 2);
  
  // add the values to devide:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
  
  // push the node on the operand stack:
  ms_push(node);
 
  // debug output: 
  woempa(9, "translate_fcmpg: %q\n", node);
}

static inline void translate_fcmpl(void) {
  
  j_node node;
    
  /*
  ** Compare two floats with NaN check included
  */

  // we create a function node that will call 'rt_float_compare':
  node = node_create_call("fcmpl", j_native_call, j_integer, &rt_float_compare_l, 2);
  
  // add the values to devide:
  node_add_param(node, ms_pop(j_float));
  node_add_param(node, ms_pop(j_float));
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
  
  // push the node on the operand stack:
  ms_push(node);
 
  // debug output: 
  woempa(9, "translate_fcmpg: %q\n", node);
}

static inline void translate_dcmpg(void) {
  cmp_wrapper(j_double, j_double);
}

static inline void translate_dcmpl(void) {
  cmp_wrapper(j_double, j_long);
}

static inline void translate_ifeq(void) {
  if0_wrapper(j_eq, j_integer);
}

static inline void translate_ifne(void) {
  if0_wrapper(j_ne, j_integer);
}

static inline void translate_iflt(void) {
  if0_wrapper(j_lt, j_integer);
}

static inline void translate_ifge(void) {
  if0_wrapper(j_ge, j_integer);
}

static inline void translate_ifgt(void) {
  if0_wrapper(j_gt, j_integer);
}

static inline void translate_ifle(void) {
  if0_wrapper(j_le, j_integer);
}

static inline void translate_if_icmpeq(void) {
  ifx_wrapper(j_eq, j_integer);
}

static inline void translate_if_icmpne(void) {
  ifx_wrapper(j_ne, j_integer);
}

static inline void translate_if_icmplt(void) {
  ifx_wrapper(j_lt, j_integer);
}

static inline void translate_if_icmpge(void) {
  ifx_wrapper(j_ge, j_integer);
}

static inline void translate_if_icmpgt(void) {
  ifx_wrapper(j_gt, j_integer);
}

static inline void translate_if_icmple(void) {
  ifx_wrapper(j_le, j_integer);
}

static inline void translate_if_acmpeq(void) {
  ifx_wrapper(j_eq, j_address);
}

static inline void translate_if_acmpne(void) {
  ifx_wrapper(j_ne, j_address);
}

static inline void translate_goto(void) {
  goto_wrapper(bytecode_get_short());
}

static inline void translate_goto_w(void) {
  goto_wrapper(bytecode_get_word());
}

static inline void translate_jsr(void) {
}

static inline void translate_jsr_w(void) {
}

static inline void translate_ret(void) {
}

static inline void translate_wide_ret(void) {
}

static inline void translate_tableswitch(void) {

  j_node node1;
  j_node node2;
  j_node node3;
  j_node temp;
  int offset;
  int def;
  int low;
  int hgh;
  int i;
 
  /*
  ** Translating this bytecode instruction requires an extra temporary
  ** to be used.
  */
  
  temp = ms_next(j_integer);
  
  /*
  ** Pop the index from the stack.
  */
  
  node1 = ms_pop(j_integer);
  
  /*
  ** Skip over the zero to three bytes of padding to get at the first
  ** word aligned address.
  */
          
  jm->curr_pc = (jm->inst_pc + 4) & -4;

  /*
  ** Retrieve the 32-bit default branch offset, the 32-bit low endpoint
  ** and the 32-bit high endpoint.
  */
  
  def = bytecode_get_word();
  low = bytecode_get_word();
  hgh = bytecode_get_word();
 
  woempa(9, "default branch offset = %d, low endpoint = %d, high endpoint = %d\n", def, low, hgh);
  
  /*
  ** Retrieve the different branch offsets: one branch offset for high,
  ** one branch offset for low, and one for each case value in between
  ** high and low.
  */
  
  for (i = 0; i < hgh - low + 1; i++) {
    // get the branch offset:
    offset = bytecode_get_word();

    // create a new constant:
    node2 = node_create_constant(j_integer, low + i);

    // assign the node to a temporary:
    node3 = node_create_insn(j_assign, node2, temp);
 
    // mark the node as a top-level node:
    ir_add_node(node3, j_link_next, j_link_none);

    // create an new comparison node: 
    node3 = node_create_insn(j_eq, node1, temp);

    // mark the node as a top-level node:
    ir_add_node(node3, jm->inst_pc + offset, j_link_next);
    
    // add the branch to our internal 'todo'-list:
    save_pc(node3, jm->inst_pc + offset);
    
    woempa(9, "- %d. branch with offset %d, target pc %d\n", i, offset, jm->inst_pc + offset);
  }
  
  // create an new jump node: 
  node3 = node_create_insn(j_jump, 0, 0);
  
  // mark the node as a top-level node:
  ir_add_node(node3, jm->inst_pc + def, j_link_none);
  
  woempa(9, "setting pc to default branch at pc %d\n", jm->inst_pc + def);

  // jump to the target instruction:
  jm->curr_pc = jm->inst_pc + def;
  
  // debug output:
  woempa(9, "tableswitch\n");
}

static inline void translate_lookupswitch(void) {

  j_node node1;
  j_node node2;
  j_node node3;
  j_node temp;
  int branch;
  int offset;
  int def;
  int num;
  int i;
  
  /*
  ** Translating this bytecode instruction requires an extra temporary
  ** to be used.
  */
  
  temp = ms_next(j_integer);
    
  /*
  ** Pop the index from the stack.
  */
  
  node1 = ms_pop(j_integer);
  
  /*
  ** Skip over the zero to three bytes of padding to get at the first
  ** word aligned address.
  */
          
  jm->curr_pc = (jm->inst_pc + 4) & -4;

  /*
  ** Retrieve the 32-bit default branch offset, the 32-bit low endpoint
  ** and the 32-bit high endpoint.
  */
  
  def = bytecode_get_word();
  num = bytecode_get_word();
 
  woempa(9, "default branch offset = %d, number of branch/offset pairs = %d\n", def, num);
 
  /*
  ** Retrieve the different branch offsets: one branch offset for high,
  ** one branch offset for low, and one for each case value in between
  ** high and low.
  */
  
  for (i = 0; i < num; i++) {
    // get the branch/offset pair:
    branch = bytecode_get_word();
    offset = bytecode_get_word();

    // create a new constant:
    node2 = node_create_constant(j_integer, branch);

    // assign the node to a temporary:
    node3 = node_create_insn(j_assign, node2, temp);
 
    // mark the node as a top-level node:
    ir_add_node(node3, j_link_next, j_link_none);

    // create an new comparison node: 
    node3 = node_create_insn(j_eq, node1, temp);

    // mark the node as a top-level node:
    ir_add_node(node3, jm->inst_pc + offset, j_link_next);
    
    // add the branch to our internal 'todo'-list:
    save_pc(node3, jm->inst_pc + offset);
    
    woempa(9, "- %d. branch with offset %d, target pc %d\n", i, offset, jm->inst_pc + offset);
  }
 
  // create an new jump node: 
  node3 = node_create_insn(j_jump, 0, 0);
  
  // mark the node as a top-level node:
  ir_add_node(node3, jm->inst_pc + def, j_link_none);
  
  woempa(9, "setting pc to default branch at pc %d\n", jm->inst_pc + def);

  // jump to the target instruction:
  jm->curr_pc = jm->inst_pc + def;
  
  // debug output:
  woempa(9, "lookupswitch\n");
}

static inline void translate_ireturn(void) {
  return_wrapper(j_integer);
}

static inline void translate_freturn(void) {
  return_wrapper(j_float);
}

static inline void translate_areturn(void) {
  return_wrapper(j_address);
}

static inline void translate_lreturn(void) {
  return_wrapper(j_long);
}

static inline void translate_dreturn(void) {
  return_wrapper(j_double);
}

static inline void translate_vreturn(void) {

  j_node node;

  // create a new node:
  node = node_create_insn(j_return, 0, 0);

  // mark the node as a top-level node:
  ir_add_node(node, j_link_none, j_link_none);

}

static inline void translate_ifnull(void) {
  if0_wrapper(j_eq, j_address);
}

static inline void translate_ifnonnull(void) {
  if0_wrapper(j_ne, j_address);
}

static inline void translate_monitorenter(void) { 

  j_node node;
  
  // create a new node to save the local variable:
  node = node_create_call("monitorenter", j_native_call, j_void, &rt_monitorenter, 1);
  node_add_param(node, ms_pop(j_address));
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);

  // debug output:
  woempa(9, "monitorenter: %q\n", node);
}

static inline void translate_monitorexit(void) {

  j_node node;
  
  // create a new node to save the local variable:
  node = node_create_call("monitorexit", j_native_call, j_void, &rt_monitorexit, 1);
  node_add_param(node, ms_pop(j_address));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);
  
  // debug output:
  woempa(9, "monitorexit: %q\n", node);
}

static inline void translate_athrow(void) {

  j_node node;

  // create a new symbol to save the local variable:
  node = node_create_call("athrow", j_native_call, j_void, &rt_athrow, 1);
  node_add_param(node, ms_pop(j_address));

  // mark the node as a top-level node:
  ir_add_node(node, j_link_none, j_link_none);  
    // NOTE: "j_link_none" because no bytecodes are being excecuted after an "athrow" bytecode.
  
  // debug output:
  woempa(9, "athrow: %q\n", node);
}

static inline void translate_getstatic(void) {

  j_node field;
  j_node node;

  /*
  ** Get a static field of a class.
  */

  // create a new constant:
  field = ct_load_static(bytecode_get_short());

  // create a memory variable:
  node = node_create_memory_variable(field->type, field, node_create_constant(j_integer, 0));

  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "getstatic: %q\n", node);
}

static inline void translate_putstatic(void) {

  j_node fields;
  j_node slot;
  j_node value;
  j_node node;
  int offset;
 
  /*
  ** Set a field in an object.
  */

  offset = bytecode_get_short();
  
  fields = ct_load_static_fields(offset);
  slot = ct_load_slot(offset);
  value = ms_pop(slot->type);

  // create a memory variable:
  node = node_create_memory_variable(slot->type, fields, slot);

  // create an instruction for the assigment:
  node = node_create_insn(j_store, value, node);

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);
 
  // debug output: 
  woempa(9, "putstatic: %q\n", node);
}

static inline void translate_getfield(void) {

  j_node instance;
  j_node slot;
  j_node node;
   
  /*
  ** Set a field in an object.
  */

  slot = ct_load_slot(bytecode_get_short());
  instance = ms_pop(j_address);

  // add a null pointer check:
  if (node_check_null_pointer(instance)) {
    ir_add_node(node_create_insn(j_null_pointer, instance, NULL), j_link_next, j_link_none);
  }

  if (slot->type == j_address) {
    // we create a function node that will call the resolved function:
    node = node_create_call("getfield", j_native_call, slot->type, rt_getfield_reference, 2);
    node_add_param(node, slot);
    node_add_param(node, instance);

    // mark the node as a top-level node:
    ir_add_node(node, j_link_next, j_link_none);
  }
  else {
    // create a memory variable:
    node = node_create_memory_variable(slot->type, instance, slot);

    // mark the node as a top-level node:
    ir_add_node(node, j_link_next, j_link_none);
      // FIXME: the above statement should _not_ be required.  However removing it results in a segfault in nextToken() of java.io.StreamTokenizer.  To be investigated.
  }

  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "getfield: %q\n", node);
}

static inline void translate_putfield(void) {

  j_node instance;
  j_node slot;
  j_node node;
  j_node value;
  
  /*
  ** Set a field in the specified object.
  */

  slot = ct_load_slot(bytecode_get_short());
  value = ms_pop(slot->type);
  instance = ms_pop(j_address);

  // add a null pointer check:
  if (node_check_null_pointer(instance)) {
    ir_add_node(node_create_insn(j_null_pointer, instance, NULL), j_link_next, j_link_none);
  }

  if (slot->type == j_address) {
    // we create a function node that will call the resolved function:
    node = node_create_call("putfield", j_native_call, j_void, rt_putfield_reference, 3);
  
    node_add_param(node, value);
    node_add_param(node, slot);
    node_add_param(node, instance);

    // mark the node as a top-level node:
    ir_add_node(node, j_link_next, j_link_none);
  }
  else {
    // create a memory variable:
    node = node_create_memory_variable(slot->type, instance, slot);

    // create an instruction for the assigment:
    node = node_create_insn(j_store, value, node);

    // mark the node as a top-level node:
    ir_add_node(node, j_link_next, j_link_none);
  }
  
  // debug output: 
  woempa(9, "putfield: %q\n", node);
}

static inline void translate_invokenonvirtual(void) {

  j_node instance;
  j_node function;
  j_node node;
  int argc;
  int i;
  
  /*
  ** Invoke an instance method, dispatch based on the class.
  */
 
  // resolve the method based on a 2-byte index:
  function = ct_load_method((int)bytecode_get_short(), &argc);
 
  // create a function node: 
  node = node_create_call("invokenonvirtual", j_java_call, function->type, NULL, argc + 2);
  
  // pop the arguments from the stack:
  for (i = 0; i < argc; i++) {
    node_add_param(node, ms_pop(j_unknown));
  }

  // pop the object reference from the stack:
  instance = ms_pop(j_address);
  node_add_param(node, instance);
  
  // pass on the resolved method:
  node_add_param(node, function);
 
  // update the function node: 
  if (function->type == j_void) {
    node->data.call.value = invokenonvirtual_dispatcher[0];
  }
  else {
    node->data.call.value = invokenonvirtual_dispatcher[1];
  }
 
  // add a null pointer check:
  if (node_check_null_pointer(instance)) {
    ir_add_node(node_create_insn(j_null_pointer, instance, NULL), j_link_next, j_link_none);
  }

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  if (function->type != j_void) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokenonvirtual: %q\n", node);
}


static inline void translate_invokevirtual(void) {
 
  j_node instance;
  j_node function;
  j_node node;
  int argc;
  int i;
  
  /*
  ** Invoke an instance method, dispatch based on the class.
  */
  
  // resolve the method based on a 2-byte index:
  function = ct_load_method((int)bytecode_get_short(), &argc);
  
  // create a function node:
  node = node_create_call("invokevirtual", j_java_call, function->type, NULL, argc + 2);
 
  // pop the arguments from the stack:
  for (i = 0; i < argc; i++) {
    node_add_param(node, ms_pop(j_unknown));
  }
 
  // pop the object reference from the stack:
  instance = ms_pop(j_address);
  node_add_param(node, instance);
  
  // pass on the resolved method:
  node_add_param(node, function);

  // update the function node: 
  if (function->type == j_void) {
    node->data.call.value = invokevirtual_dispatcher[0];
  }
  else {
    node->data.call.value = invokevirtual_dispatcher[1];
  }
 
  // add a null pointer check:
  if (node_check_null_pointer(instance)) {
    ir_add_node(node_create_insn(j_null_pointer, instance, NULL), j_link_next, j_link_none);
  }

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  if (function->type != j_void) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokevirtual: %q\n", node);
}

static inline void translate_invokespecial(void) {

  j_node instance;
  j_node function;
  j_node node;
  int argc;
  int i;
  
  /*
  ** Invoke an instance method, dispatch based on the class.
  */
 
  // resolve the method based on a 2-byte index:
  function = ct_load_method((int)bytecode_get_short(), &argc);
 
  // create a function node: 
  node = node_create_call("invokespecial", j_java_call, function->type, NULL, argc + 2);
  
  // pop the arguments from the stack:
  for (i = 0; i < argc; i++) {
    node_add_param(node, ms_pop(j_unknown));
  }

  // pop the object reference from the stack:
  instance = ms_pop(j_address);
  node_add_param(node, instance);
  
  // pass on the resolved method:
  node_add_param(node, function);
 
  // update the function node: 
  if (function->type == j_void) {
    node->data.call.value = invokespecial_dispatcher[0];
  }
  else {
    node->data.call.value = invokespecial_dispatcher[1];
  }
  
  // add a null pointer check:
  if (node_check_null_pointer(instance)) {
    ir_add_node(node_create_insn(j_null_pointer, instance, NULL), j_link_next, j_link_none);
  }

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  if (function->type != j_void) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokespecial: %q\n", node);
}

static inline void translate_invokestatic(void) {

  j_node function;
  j_node node;
  int argc;
  int i;

  /*
  ** Invoke a static class method.
  */
  
  // resolve the method based on a 2-byte index:
  function = ct_load_method((int)bytecode_get_short(), &argc);

  // we create a function node to call 'rt_invokestatic':
  node = node_create_call("invokestatic", j_java_call, function->type, &rt_invokestatic, argc + 1);

  // pop the arguments from the stack:
  for (i = 0; i < argc; i++) {
    node_add_param(node, ms_pop(j_unknown));
  }
    // NOTE: in case of "invokestatic" there is no object reference on 
    //       the stack; instead there is an object reference in the 
    //       constant pool of the current class.
  
  // pass on the resolved method:
  node_add_param(node, function);
 
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  if (function->type != j_void) {
    ms_push(node);
  }
  
  // debug output: 
  woempa(9, "invokestatic: %q\n", node);
}

static inline void translate_invokeinterface(void) {

  j_node function;
  j_node node;
  int argc;
  int i;
  
  /*
  ** Invoke an interface methos.
  */
  
  // resolve the method based on a 2-byte index:
  function = ct_load_interface((int)bytecode_get_short(), &argc);

  // we create a function node to call 'rt_invokeinterface':
  node = node_create_call("invokeinterface", j_java_call, function->type, &rt_invokeinterface, argc + 2);
  
  // skip over the useless <nargs> and zero byte:
  bytecode_get_short();
  
  // pop the arguments and the object reference from the stack:
  for (i = 0; i < argc + 1; i++) {
    node_add_param(node, ms_pop(j_unknown));
  }
  
  // pass on the resolved method:
  node_add_param(node, function);
 
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  if (function->type != j_void) {
    ms_push(node);
  }
  
  // debug output:
  woempa(9, "invokeinterface: %q\n", node);
}

static inline void translate_new(void) {

  j_node field;
  j_node node;
    
  /*
  ** Push or load local the variable 'number' of the current frame onto
  ** the operand stack.
  */

  field = ct_load_class(bytecode_get_short());

  // we create a function node that will call 'rt_new':
  node = node_create_call("new", j_native_call, j_address, &rt_new, 1);
  node_add_param(node, field);
 
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);  
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "new: %q\n", node);
}

static inline void translate_newarray(void) {
  array_create_wrapper(1, ct_load_type(bytecode_get_byte())); 
}

static inline void translate_multianewarray(void) {
  
  int index = bytecode_get_short();
  int dimensions = bytecode_get_byte();
  
  array_create_wrapper(dimensions, ct_load_class(index)); 
}

static inline void translate_anewarray(void) {
  array_create_wrapper(1, ct_load_array(bytecode_get_short())); 
}

static inline void translate_arraylength(void) {

  j_node array; 
  j_node node;
    
  /*
  ** Get the length of an array.
  */

  array = ms_pop(j_address);

  // add a null pointer check:
  if (node_check_null_pointer(array)) {
    ir_add_node(node_create_insn(j_null_pointer, array, NULL), j_link_next, j_link_none);
  }

  // create a memory variable:
  node = node_create_memory_variable(j_integer, array, ct_array_length_offset());

  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);
    // FIXME: the above statement should _not_ be required.  However removing it results in a segfault in com_test_20() of com.acunia.wonka.test.jspot.ProblemTest.  To be investigated.

  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "arraylength: %q\n", node);
}

static inline void translate_checkcast(void) {

  j_node instance;
  j_node node;
   
  /*
  ** Ensure that an instance is of the given type.
  */

  /*
  ** Note that the "checkcast" bytecode does not pop the object reference
  ** from the operand stack.  
  */ 
  
  // get the top node from the operand stack:
  instance = ms_top(j_address);
  
  // we create a function node that will call 'rt_checkcast':
  node = node_create_call("checkcast", j_native_call, j_void, &rt_checkcast, 2);
  node_add_param(node, ct_load_class(bytecode_get_short()));
  node_add_param(node, instance);   
  
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);
 
  // debug output: 
  woempa(9, "checkcast: %q\n", node);
}

static inline void translate_instanceof(void) {

  j_node node;
   
  /*
  ** Determine whether an instance is of a given type.
  */

  // we create a function node that will call 'rt_instanceof':
  node = node_create_call("instanceof", j_native_call, j_integer, &rt_instanceof, 2);
  node_add_param(node, ct_load_class(bytecode_get_short()));
  node_add_param(node, ms_pop(j_address));
   
  // mark the node as a top-level node:
  ir_add_node(node, j_link_next, j_link_none);
 
  // push the node on the operand stack:
  ms_push(node);

  // debug output: 
  woempa(9, "instanceof: %q\n", node);
}

void translate(void) {

  unsigned char code;
  
  woempa(9, "[begin of translate()]\n");

  /*
  ** Initialize our method:
  */

  jm->curr_pc = 0;
  jm->inst_pc = 0;
  
  // allocate memory for the local variables:
  jm->work = arena_allocate(jm->length * sizeof(char));
      
  /*
  ** Initialize our minic stack:
  */
 
  ms_init();
  
  /*
  ** Translate each bytecode:
  */
  
  while (jm->failed == 0 && jm->curr_pc < jm->length) {

    if (jm->work[jm->curr_pc] != 0xff) {
   
      code = jm->bytecode[jm->curr_pc++];
     
      j_flarp(white, "pc = %d, bytecode = %d (%s)\n", jm->inst_pc, code, bytecode_get_name(code));

      /*
      ** If the current instruction is a join point, create new nodes
      ** and save the stack state.
      */
     
      if (bytecode_is_entry(jm->inst_pc)) {
        // clone the current state information:
        ms_clone(!bytecode_is_join(jm->inst_pc));
      }
      
      // save the active state information (pre state):
      ms_save_state(jm->inst_pc, j_pre_execution);
      
      switch (code) {
        case (nop):
          break;

        case (aconst_null):
          translate_aconst_null();
          break;

        case (iconst_m1):
          translate_iconst_m1();
          break;

        case (iconst_0):
          translate_iconst_0();
          break;

        case (iconst_1):
          translate_iconst_1();
          break;

        case (iconst_2):
          translate_iconst_2();
          break;

        case (iconst_3):
          translate_iconst_3();
          break;

        case (iconst_4):
          translate_iconst_4();
          break;

        case (iconst_5):
          translate_iconst_5();
          break;

        case (lconst_0):
          translate_lconst_0();
          break;

        case (lconst_1):
          translate_lconst_1();
          break;

        case (fconst_0):
          translate_fconst_0();
          break;

        case (fconst_1):
          translate_fconst_1();
          break;

        case (fconst_2):
          translate_fconst_2();
          break;

        case (dconst_0):
          translate_dconst_0();
          break;

        case (dconst_1):
          translate_dconst_1();
          break;

        case (bipush):
          translate_bipush();
          break;

        case (sipush):
          translate_sipush();
          break;

        case (ldc):
          translate_ldc();
          break;

        case (ldc_w):
          translate_ldc_w();
          break;

        case (ldc2_w):
          translate_ldc2_w();
          break;

        case (iload):
          translate_iload();
          break;

        case (lload):
          translate_lload();
          break;

        case (fload):
          translate_fload();
          break;

        case (dload):
          translate_dload();
          break;

        case (aload):
          translate_aload();
          break;

        case (iload_0):
          translate_iload_0();
          break;

        case (iload_1):
          translate_iload_1();
          break;

        case (iload_2):
          translate_iload_2();
          break;

        case (iload_3):
          translate_iload_3();
          break;

        case (lload_0):
          translate_lload_0();
          break;

        case (lload_1):
          translate_lload_1();
          break;

        case (lload_2):
          translate_lload_2();
          break;

        case (lload_3):
          translate_lload_3();
          break;

        case (fload_0):
          translate_fload_0();
          break;

        case (fload_1):
          translate_fload_1();
          break;

        case (fload_2):
          translate_fload_2();
          break;

        case (fload_3):
          translate_fload_3();
          break;

        case (dload_0):
          translate_dload_0();
          break;

        case (dload_1):
          translate_dload_1();
          break;

        case (dload_2):
          translate_dload_2();
          break;

        case (dload_3):
          translate_dload_3();
          break;

        case (aload_0):
          translate_aload_0();
          break;

        case (aload_1):
          translate_aload_1();
          break;

        case (aload_2):
          translate_aload_2();
          break;

        case (aload_3):
          translate_aload_3();
          break;

        case (iaload):
          translate_iaload();
          break;

        case (laload):
          translate_laload();
          break;

        case (faload):
          translate_faload();
          break;

        case (daload):
          translate_daload();
          break;

        case (aaload):
          translate_aaload();
          break;

        case (baload):
          translate_baload();
          break;

        case (caload):
          translate_caload();
          break;

        case (saload):
          translate_saload();
          break;

        case (istore):
          translate_istore();
          break;

        case (lstore):
          translate_lstore();
          break;

        case (fstore):
          translate_fstore();
          break;

        case (dstore):
          translate_dstore();
          break;

        case (astore):
          translate_astore();
          break;

        case (istore_0):
          translate_istore_0();
          break;

        case (istore_1):
          translate_istore_1();
          break;

        case (istore_2):
          translate_istore_2();
          break;

        case (istore_3):
          translate_istore_3();
          break;

        case (lstore_0):
          translate_lstore_0();
          break;

        case (lstore_1):
          translate_lstore_1();
          break;

        case (lstore_2):
          translate_lstore_2();
          break;

        case (lstore_3):
          translate_lstore_3();
          break;

        case (fstore_0):
          translate_fstore_0();
          break;

        case (fstore_1):
          translate_fstore_1();
          break;

        case (fstore_2):
          translate_fstore_2();
          break;

        case (fstore_3):
          translate_fstore_3();
          break;

        case (dstore_0):
          translate_dstore_0();
          break;

        case (dstore_1):
          translate_dstore_1();
          break;

        case (dstore_2):
          translate_dstore_2();
          break;

        case (dstore_3):
          translate_dstore_3();
          break;

        case (astore_0):
          translate_astore_0();
          break;

        case (astore_1):
          translate_astore_1();
          break;

        case (astore_2):
          translate_astore_2();
          break;

        case (astore_3):
          translate_astore_3();
          break;

        case (iastore):
          translate_iastore();
          break;

        case (lastore):
          translate_lastore();
          break;

        case (fastore):
          translate_fastore();
          break;

        case (dastore):
          translate_dastore();
          break;

        case (aastore):
          translate_aastore();
          break;

        case (bastore):
          translate_bastore();
          break;

        case (castore):
          translate_castore();
          break;

        case (sastore):
          translate_sastore();
          break;

        case (pop):
          translate_pop();
          break;

        case (pop2):
          translate_pop2();
          break;

        case (dup):
          translate_dup();
          break;

        case (dup_x1):
          translate_dup_x1();
          break;

        case (dup_x2):
          translate_dup_x2();
          break;

        case (dup2):
          translate_dup2();
          break;

        case (dup2_x1):        
          translate_dup2_x1();
          break;

        case (dup2_x2):
          translate_dup2_x2();
          break;

        case (swap):
          translate_swap();
          break;

        case (iadd):
          translate_iadd();
          break;

        case (ladd):
          translate_ladd();
          break;

        case (fadd):
          translate_fadd();
          break;

        case (dadd):
          translate_dadd();
          break;

        case (isub):
          translate_isub();
          break;

        case (lsub):
          translate_lsub();
          break;

        case (fsub):
          translate_fsub();
          break;

        case (dsub):
          translate_dsub();
          break;

        case (imul):
          translate_imul();
          break;

        case (lmul):
          translate_lmul();
          break;

        case (fmul):
          translate_fmul();
          break;

        case (dmul):
          translate_dmul();
          break;

        case (idiv):
          translate_idiv();
          break;

        case (ldiv):
          translate_ldiv();

        case (fdiv):
          translate_fdiv();
          break;

        case (ddiv):
          translate_ddiv();
          break;

        case (irem):
          translate_irem();
          break;

        case (lrem):
          translate_lrem();
          break;

        case (frem):
          translate_frem();
          break;

        case (drem):
          translate_drem();
          break;

        case (ineg):
          translate_ineg();
          break;

        case (iseq):
          translate_iseq();
          break;
          
        case (isne):
          translate_isne();
          break;
          
        case (lneg):
          translate_lneg();
          break;

        case (fneg):
          translate_fneg();
          break;

        case (dneg):
          translate_dneg();
          break;

        case (ishl):
          translate_ishl();
          break;

        case (lshl):
          translate_lshl();
          break;

        case (ishr):
          translate_ishr();
          break;

        case (lshr):
          translate_lshr();
          break;

        case (iushr):
          translate_iushr();
          break;

        case (lushr):
          translate_lushr();
          break;

        case (iand):
          translate_iand();
          break;

        case (land):
          translate_land();
          break;

        case (ior):
          translate_ior();
          break;

        case (lor):
          translate_lor();
          break;

        case (ixor):
          translate_ixor();
          break;

        case (lxor):
          translate_lxor();
          break;

        case (iinc):
          translate_iinc();
          break;

        case (i2l):
          translate_i2l();
          break;

        case (i2f):
          translate_i2f();
          break;

        case (i2d):
          translate_i2d();
          break;

        case (l2i):
          translate_l2i();
          break;

        case (l2f):
          translate_l2f();
          break;

        case (l2d):
          translate_l2d();
          break;

        case (f2i):
          translate_f2i();
          break;

        case (f2l):
          translate_f2l();
          break;

        case (f2d):
          translate_f2d();
          break;

        case (d2i):
          translate_d2i();
          break;

        case (d2l):
          translate_d2i();
          break;

        case (d2f):
          translate_d2f();
          break;

        case (i2b):
          translate_i2b();
          break;

        case (i2c):
          translate_i2c();
          break;

        case (i2s):
          translate_i2s();
          break;

        case (lcmp):
          translate_lcmp();
          break;

        case (fcmpl):
          translate_fcmpl();
          break;

        case (fcmpg):
          translate_fcmpg();
          break;

        case (dcmpl):
          translate_dcmpl();
          break;

        case (dcmpg):
          translate_dcmpg();
          break;

        case (ifeq):
          translate_ifeq();
          break;

        case (ifne):
          translate_ifne();
          break;

        case (iflt):
          translate_iflt();
          break;

        case (ifge):
          translate_ifge();
          break;

        case (ifgt):
          translate_ifgt();
          break;

        case (ifle):
          translate_ifle();
          break;

        case (if_icmpeq):        
          translate_if_icmpeq();
          break;

        case (if_icmpne):
          translate_if_icmpne();
          break;

        case (if_icmplt):
          translate_if_icmplt();
          break;

        case (if_icmpge):
          translate_if_icmpge();
          break;

        case (if_icmpgt):
          translate_if_icmpgt();
          break;

        case (if_icmple):
          translate_if_icmple();
          break;

        case (if_acmpeq):        
          translate_if_acmpeq();
          break;

        case (if_acmpne):
          translate_if_acmpne();
          break;

        case (goto_n):
          translate_goto();
          break;

        case (goto_w):
          translate_goto_w();
          break;

        case (jsr):
          translate_jsr();
          break;

        case (jsr_w):
          translate_jsr_w();
          break;

        case (ret):
          translate_ret();
          break;

        case (tableswitch): 
          translate_tableswitch();
          break;

        case (lookupswitch):
          translate_lookupswitch();
          break;

        case (ireturn):
          translate_ireturn();
          break;

        case (freturn):
          translate_freturn();
          break;

        case (areturn):
          translate_areturn();
          break;

        case (lreturn):
          translate_lreturn();
          break;

        case (dreturn):
          translate_dreturn();
          break;

        case (vreturn):
          translate_vreturn();
          break;

        case (getstatic):
        case (in_getstatic):
          translate_getstatic();
          break;

        case (putstatic):
        case (in_putstatic):
          translate_putstatic();
          break;

        case (getfield):
        case (in_getfield):
          translate_getfield();
          break;

        case (putfield):
        case (in_putfield):
          translate_putfield();
          break;

        case (in_invokenonvirtual):
          translate_invokenonvirtual();
          break;
          
        case (invokevirtual):
        case (in_invokevirtual):
          translate_invokevirtual();
          break;

        case (invokespecial):
        case (in_invokesuper):
          translate_invokespecial();
          break;

        case (invokestatic):
        case (in_invokestatic):
          translate_invokestatic();
          break;

        case (invokeinterface):
          translate_invokeinterface();
          break;

        case (new):
        case (in_new):
          translate_new();
          break;

        case (newarray):
          translate_newarray();
          break;

        case (anewarray):
          translate_anewarray();
          break;

        case (arraylength):
          translate_arraylength();
          break;

        case (athrow):
          translate_athrow();
          break;

        case (checkcast):
          translate_checkcast();
          break;

        case (instanceof):
          translate_instanceof();
          break;

        case (monitorenter):
          translate_monitorenter();
          break;

        case (monitorexit):
          translate_monitorexit();
          break;

        case (multianewarray):
          translate_multianewarray();
          break;

        case (ifnull):
          translate_ifnull();
          break;

        case (ifnonnull):
          translate_ifnonnull();
          break;

        case (wide):
          code = jm->bytecode[jm->curr_pc++];

          switch (code) {
            case (iload):
              translate_wide_iload();
              break;

            case (lload):
              translate_wide_lload();
              break;

            case (fload):
              translate_wide_fload();
              break;

            case (dload):
              translate_wide_dload();
              break;

            case (aload):
              translate_wide_aload();
              break;

            case (istore):
              translate_wide_istore();
              break;

            case (lstore):
              translate_wide_lstore();
              break;

            case (fstore):
              translate_wide_fstore();
              break;

            case (dstore):
              translate_wide_dstore();
              break;

            case (astore):
              translate_wide_astore();
              break;

            case (iinc):
              translate_wide_iinc();
              break;

            case (ret):
              translate_wide_ret();
              break;

            case (fail):
              jfail("attempt to access bytecode that has been substituted\n");
              break;
              
            case (breakpoint):
              jabort("ERROR: bytecode `breakpoint' not implemented yet!\n");

            default:
              jabort("ERROR: encounterd unknown bytecode `%d'.\n", code);
          }

        case (fail):
          jfail("attempt to access bytecode that has been substituted\n");
          break;

        case (breakpoint):
          jabort("bytecode `breakpoint' not implemented yet!\n");

        default:
          jabort("encounterd unknown bytecode `%d'.\n", code);
      }    

      // save the active state information (post state):
      ms_save_state(jm->inst_pc, j_post_execution);
      
      /*
      ** Indicate that we have already been here.
      */
        
      jm->work[jm->inst_pc] = 0xff;
    }
    else {
      code = 0x00;
    }
 
    /*
    ** Check if there is anything left to be done.  The order of the if
    ** clauses is important.
    */

    if (jm->curr_pc >= jm->length) {
      jm->curr_pc = next_pc();
      woempa(9, "hey, we are at the end of our method - did we traverse all paths ... ? (old pc: %d, new pc: %d)\n", jm->inst_pc, jm->curr_pc);
    }
    else if (bytecode_is_exit(jm->inst_pc)) {
      jm->curr_pc = next_pc();
      woempa(9, "hey, we are at an exit point - did we traverse all paths ... ? (old pc: %d, new pc: %d)\n", jm->inst_pc, jm->curr_pc);
        // NOTE: both return instructions and athrow instructions are exit points.
    }

    if (jm->work[jm->curr_pc] == 0xff) {
      jm->curr_pc = next_pc();
      woempa(9, "hey, we have been here already - did we traverse all paths ... ? (old pc: %d, new pc: %d)\n", jm->inst_pc, jm->curr_pc);
    }
    
    jm->inst_pc = jm->curr_pc;
  }
 
  woempa(9, "[end of translate()]\n");
}

