/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** Design based on the iburg, mburg and monoburg code generator generators.
** For more information, read "Burg - Fast optimal instruction selection and
** tree parsing", written by Fraser, Henry, and Proebsting.
*/

#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include "burg.h"

extern FILE * yyin; 
int yyparse(void);
int yyerror(char * fmt, ...);

j_nonterminal nonterminal_list = NULL;
j_terminal terminal_list = NULL;
j_rule rule_list = NULL;
char * header = NULL;
char * footer = NULL;

void header_create(char * data) {
  header = strdup(data);
}

void footer_create(char * data) {
  footer = strdup(data);
}

j_terminal terminal_create(char *name) {

  j_terminal terminal;

  /*
  ** Check whether the given terminal already exists.
  */
  
  terminal = terminal_list;

  while (terminal) {
    if (strcmp(name, terminal->name) == 0) {
      break;
    }

    terminal = terminal->next;
  }

  /*
  ** Create a new terminal if it does not exist yet.
  */

  if (terminal == NULL) {
    terminal = (j_terminal)calloc(1, sizeof(j_Terminal));
    terminal->name = strdup(name);
  
    if (terminal_list) {
      terminal->number = terminal_list->number + 1;
      terminal->next = terminal_list;
      terminal_list = terminal;
    }
    else {
      terminal->number = 1;
      terminal_list = terminal;
    }
  }

  return terminal;
}

j_terminal terminal_get(char * name) {

  j_terminal terminal = terminal_list;

  while (terminal) {
    if (strcmp(name, terminal->name) == 0) {
      break;
    }

    terminal = terminal->next;
  }

  return terminal;
}

void terminal_print(void) {
  
  j_terminal terminal = terminal_list;
  j_list item;
  
  fprintf(stderr, "\nterminals\n---------\n");
  
  while (terminal) {
    fprintf(stderr, "terminal '%s' (%d)\n", terminal->name, terminal->number);
    
    item = terminal->rules;
    fprintf(stderr, "  + rules with this terminal on the right-hand side:\n");
    while (item) {
      fprintf(stderr, "    - rule '%s : %p'\n", item->rule->name, item->rule->tree);
      item = item->next;
    }
    
    terminal = terminal->next;
  }
}

j_nonterminal nonterminal_get(char * name) {

  j_nonterminal nonterminal = nonterminal_list;

  while (nonterminal) {
    if (strcmp(name, nonterminal->name) == 0) {
      break;
    }

    nonterminal = nonterminal->next;
  }

  return nonterminal;
}

int nonterminal_num(void) {

  j_nonterminal nonterminal = nonterminal_list;
  int number = 0;
  
  while (nonterminal) {
    number += 1;
    nonterminal = nonterminal->next;
  }

  return number;
}

int rule_num(void) {

  j_rule rule = rule_list;
  int number = 0;
  
  while (rule) {
    number += 1;
    rule = rule->next;
  }

  return number;
}

j_nonterminal nonterminal_create(char * name) {

  j_nonterminal nonterminal = nonterminal_get(name);

  /*
  ** Create a new non-terminal if it does not exist yet.
  */

  if (nonterminal == NULL) {
    fprintf(stderr, "nonterminal_create: %s\n", name);
    
    nonterminal = (j_nonterminal)calloc(1, sizeof(j_Nonterminal));
    nonterminal->name = strdup(name);
  
    if (nonterminal_list) {
      nonterminal->number = nonterminal_list->number + 1;
      nonterminal->next = nonterminal_list;
      nonterminal_list = nonterminal;
    }
    else {
      nonterminal->number = 1;
      nonterminal_list = nonterminal;
    }
  }

  return nonterminal;
}

void nonterminal_print(void) {

  j_nonterminal nonterminal = nonterminal_list;
  j_list item;

  fprintf(stderr, "\nnon-terminals\n-------------\n");
  
  while (nonterminal) {
    fprintf(stderr, "non-terminal '%s' (%d)\n", nonterminal->name, nonterminal->number);

    item = nonterminal->rules;
    fprintf(stderr, "  + rules with this non-terminal on the left-hand side:\n");
    while (item) {
      fprintf(stderr, "    - rule '%s : %p'\n", item->rule->name, item->rule->tree);
      item = item->next;
    }
    
    item = nonterminal->chain;
    fprintf(stderr, "  + rules with this non-terminal on the right-hand side:\n");
    while (item) {
      fprintf(stderr, "    - rule '%s : %p'\n", item->rule->name, item->rule->tree);
      item = item->next;
    }

    nonterminal = nonterminal->next;
  }
}

j_tree tree_create(char *name, j_tree ltree, j_tree rtree) {

  j_terminal terminal;
  j_tree tree;

  fprintf(stderr, "tree_create: %s %p %p\n", name, ltree, rtree);
  
  tree = (j_tree)calloc(1, sizeof(j_Tree));
  tree->name = strdup(name);
  tree->ltree = ltree;
  tree->rtree = rtree;
  
  terminal = terminal_get(name);

  if (terminal) {
    tree->terminal = terminal;
  }
  else if (ltree != NULL || rtree != NULL) {
    yyerror("terminal '%s' used but not declared", name);
  }
  else {  
    tree->nonterminal = nonterminal_create(name);
  }
 
  return tree;
}

void rule_add(j_list * list, j_rule rule) {

  j_list curr = *list;
  j_list item = NULL; 

  while (curr && curr->rule != rule) {
    curr = curr->next;
  }

  if (curr == NULL) {    // Don't insert duplicate rules.

    /*
    ** Create a new list item.
    */
    
    item = (j_list)calloc(1, sizeof(j_List));
    item->rule = rule;
    
    if (list) {
      item->next = *list;
      *list = item;
    }
    else {
      *list = item;
    }
  } 
}

j_rule rule_create(char *name, j_tree tree, char * code, char * cost) {

  j_rule rule;
  j_rule curr;
 
  rule = (j_rule)calloc(1, sizeof(j_Rule));
  rule->name = strdup(name);
  rule->code = strdup(code);
  rule->cost = strdup(cost);
  rule->tree = tree;
  
  fprintf(stderr, "rule_create: %s %p\n", name, tree);

  if (rule_list) {
    /*
    ** Add the new rule at the end of the list to keep them sorted.
    */
 
    curr = rule_list;
    while (curr->next) {
      curr = curr->next;
    }    

    rule->number = curr->number + 1;
    curr->next = rule;
  }
  else {
    rule->number = 1;
    rule_list = rule;
  }
 
  rule->nonterminal = nonterminal_get(name);
  rule_add(&rule->nonterminal->rules, rule);

  if (tree->terminal) {
    rule_add(&tree->terminal->rules, rule);
  }
  
  if (tree->nonterminal) {
    rule_add(&tree->nonterminal->chain, rule);
  }
  
  return rule;
}

void rule_print(void) {
  
  j_rule rule = rule_list;
 
  fprintf(stderr, "\nrules\n-----\n");
  
  while (rule) {
    fprintf(stderr, "rule #%d '%s : %p'\n", rule->number, rule->name, rule->tree);
    rule = rule->next;
  }
}

void emit_tree(FILE * file, j_tree tree) {

  assert(tree->terminal || tree->nonterminal); 
  
  if (tree->nonterminal) {
    fprintf(file, "%s", tree->nonterminal->name);
  }
  else {
    fprintf(file, "%s", tree->terminal->name);
  }

  if (tree->ltree || tree->rtree) {  
    fprintf(file, "(");
  }
  
  if (tree->ltree) {
    emit_tree(file, tree->ltree);
  }
    
  if (tree->ltree && tree->rtree) {
    fprintf(file, ", ");
  }
 
  if (tree->rtree) {
    emit_tree(file, tree->rtree);
  }

  if (tree->ltree || tree->rtree) {  
    fprintf(file, ")");
  }
}

void emit_header(FILE * file) {
  
  j_nonterminal nonterminal;
  j_rule rule;

  /*
  ** Emit the required include statements.
  */

  fprintf(file, 
    "#include <stdio.h>\n"
    "#include <stdlib.h>\n"
    "#include <assert.h>\n"
    "\n"
    "#include \"j-spot.h\"\n"
    "#include \"arena.h\"\n"
    "\n");
  
  /*
  ** Emit integer codes for the non-terminals to index the cost and rule
  ** vectors in the state record structure.
  */
  
  nonterminal = nonterminal_list;

  while (nonterminal) {
    fprintf(file, "#define nt_%s %d\n", nonterminal->name, nonterminal->number);
    nonterminal = nonterminal->next;
  }
  fprintf(file, "\n");

  /*
  ** Emit the structure for the state records.
  */
 
  fprintf(file, 
    "typedef struct j_State {\n"
    "  int operator;\n"
    "  j_state lstate;\n"
    "  j_state rstate;\n"
    "  unsigned short cost[%d];\n"
    "  unsigned short rule[%d];\n"
    "} j_State;"
    "\n\n", nonterminal_num() + 1, nonterminal_num() + 1);

  /*
  ** Emit an array that maps rule numbers to rule names.
  */
  
  fprintf(file, 
    "static char * burg_rule_name[] = {\n"
    "  NULL,  // add a leading NULL as by convention, rule numbers start at 1\n");
  
  rule = rule_list;
  while (rule) {
    fprintf(file, "  \"%s : ", rule->name);
    emit_tree(file, rule->tree);
    fprintf(file, "\",\n");
    
    rule = rule->next;
  }
  
  fprintf(file,
    "  NULL   // add a trailing NULL to denote the end of the rule name array\n"
    "};\n"
    "\n");
 
  /*
  ** Emit an array that counts how often a rule has been applied.
  */
  
  fprintf(file, 
    "static int burg_rule_count[] = {\n"
    "  0,  // add a leading NULL as by convention, rule numbers start at 1\n");
 
  rule = rule_list;
  while (rule) {
    fprintf(file, "  0,\n");
    
    rule = rule->next;
  }
  
  fprintf(file,
    "  0    // add a trailing 0 to denote the end of the rule name array\n"
    "};\n"
    "\n");
 
  /*
  ** Emit the custom, user-supplied header.
  */
 
  if (header) {
    fprintf(file, 
      "/*\n"
      "** Custom header as defined in the user supplied machine description.\n"
      "*/\n"
      "%s\n// end of custom header\n\n", header);
  }
}

void emit_footer(FILE * file) {

  /*
  ** Emit the custom, user-supplied footer.
  */
 
  if (footer) {
    fprintf(file, 
      "/*\n"
      "** Custom footer as defined in the user supplied machine description.\n"
      "*/\n"
      "%s\n// end of custom footer\n", footer);
  }
}

void emit_state_test(FILE * file, j_tree tree, char * suffix) {

  int length;
  
  assert(tree->terminal || tree->nonterminal); 
  
  if (tree->nonterminal) {
    fprintf(file, "%s && %s->rule[nt_%s]", suffix, suffix, tree->nonterminal->name);
  }
  else {
    fprintf(file, "%s && %s->operator == %s && ", suffix, suffix, tree->terminal->name);

    if (tree->ltree == NULL) {
      fprintf(file, "state->lstate == NULL && ");  
    }
 
    if (tree->rtree == NULL) {
      fprintf(file, "state->rstate == NULL && ");  
    }
  }

  if (tree->terminal) {
    if (tree->ltree) {
      length = strlen(suffix);
      emit_state_test(file, tree->ltree, strcat(suffix, "->lstate"));
      *(suffix + length) = '\0'; 
    }
    
    if (tree->ltree && tree->rtree) {
      fprintf(file, " && ");
    }
 
    if (tree->rtree) {
      length = strlen(suffix);
      emit_state_test(file, tree->rtree, strcat(suffix, "->rstate"));
      *(suffix + length) = '\0';  
    }
    
    if (!tree->ltree && !tree->rtree) {
      fprintf(file, "1");
    }
  }
}

void emit_state_cost(FILE * file, j_tree tree, char * suffix) {

  int length;

  assert(tree->terminal || tree->nonterminal); 
  
  if (tree->nonterminal) {
    fprintf(file, "%s->cost[nt_%s]", suffix, tree->nonterminal->name);
  }
  else {
    if (tree->ltree) {
      length = strlen(suffix);
      emit_state_cost(file, tree->ltree, strcat(suffix, "->lstate"));
      *(suffix + length) = '\0';  
    }
    
    if (tree->ltree && tree->rtree) {
      fprintf(file, " + ");
    }
 
    if (tree->rtree) {
      length = strlen(suffix);
      emit_state_cost(file, tree->rtree, strcat(suffix, "->rstate"));
      *(suffix + length) = '\0';  
    }
    
    if (!tree->ltree && !tree->rtree) {
      fprintf(file, "0");
    }
  }
}

void emit_record(FILE * file, j_rule rule) {
  
  fprintf(file,
    "    if (cost < state->cost[nt_%s]) {\n"
    "      // fprintf(stderr, \"set cost[nt_%s] of rule '%%s' to %%d\\n\", burs_rule_name[%d], cost);\n"
    "\n"
    "      state->cost[nt_%s] = cost;\n"
    "      state->rule[nt_%s] = %d;\n", rule->nonterminal->name, rule->nonterminal->name, rule->number, rule->nonterminal->name, rule->nonterminal->name, rule->number);
 
  if (rule->nonterminal->chain) {
    fprintf(file,
      "\n"
      "      burg_closure_%s(state, cost);\n", rule->nonterminal->name);
  } 
 
  fprintf(file,
    "    }\n");
}

void emit_rule_cost(FILE * file, j_rule rule) {
  
  fprintf(file, 
    "static inline unsigned short burg_cost_%d(void) {\n"
    "\n"
    "  // fprintf(stderr, \"burg_cost_%d(): called cost function of rule '%%s'\\n\", burg_rule_name[%d]);\n"
    "\n"
    "  %s\n"
    "}\n\n", rule->number, rule->number, rule->number, rule->cost);
}

void emit_rule_code(FILE * file, j_rule rule) {
  
  fprintf(file, 
    "static inline void burg_code_%d(j_node node) {\n"
    "\n"
    "  // fprintf(stderr, \"burg_code_%d(): called code function of rule '%%s'\\n\", burg_rule_name[%d]);\n"
    "\n"
    "  %s\n"
    "}\n\n", rule->number, rule->number, rule->number, rule->code);
}

void emit_closure(FILE * file, j_nonterminal nonterminal) {

  j_list item = nonterminal->chain;
  
  if (item) {
  
    fprintf(file, 
      "void burg_closure_%s(j_state state, int cost) {\n"
      "\n"
      "  // fprintf(stderr, \"burg_closure_%s(%%p, %%d)\\n\", state, cost);\n"
      "\n", nonterminal->name, nonterminal->name);

    while (item) {
      emit_record(file, item->rule);
      item = item->next;
    }

    fprintf(file, 
      "}\n\n");

    nonterminal = nonterminal->next;
  }
}

void emit_terminal_state(FILE * file, j_terminal terminal) {

  char * string = (char *)malloc(1024 * sizeof(char));
  j_list item;

  fprintf(file, 
    "void burg_%s_state(j_state state) {\n"
    "\n"
    "  int cost = 0;\n"
    "\n", terminal->name);

  item = terminal->rules;
  
  while (item) {
    fprintf(file, 
      "  if (");

    emit_state_test(file, item->rule->tree, strcpy(string, "state"));
      
    fprintf(file, 
      ") {\n"
      "    cost = ");
      
    emit_state_cost(file, item->rule->tree, strcpy(string, "state"));
    
    fprintf(file, 
      " + burg_cost_%d();\n"
      "\n", item->rule->number);

    emit_record(file, item->rule);
    
    fprintf(file, 
      "  }\n"
      "\n");

    item = item->next;
  }

  fprintf(file, "}\n\n");

  free(string);
}

void emit_labeller(FILE * file) {

  j_nonterminal nonterminal;
  j_terminal terminal;
  j_rule rule;
  int i;
  
  /*
  ** Emit the code functions.
  */
  
  rule = rule_list;
  while (rule) {
    emit_rule_code(file, rule);
    rule = rule->next;
  }
  
  /*
  ** Emit the cost functions.
  */
  
  rule = rule_list;
  while (rule) {
    emit_rule_cost(file, rule);
    rule = rule->next;
  }
  
  /*
  ** Emit an array that maps rule numbers to code functions.
  */
  
  fprintf(file,
    "typedef void (*burg_function)(j_node node);\n"
    "\n");
  
  fprintf(file,
    "static burg_function burg_rule_code[] = {\n"
    "  NULL,  // add a leading NULL as by convention, rule numbers start at 1\n");
  
  rule = rule_list;
  while (rule) {
    fprintf(file, "  burg_code_%d,\n", rule->number);
    rule = rule->next;
  }
  
  fprintf(file,
    "  NULL   // add a trailing NULL to denote the end of the rule name array\n"
    "};\n"
    "\n");
 
  /*
  ** Emit the code function.
  */
    
  fprintf(file, 
    "static inline void burg_code(int rule, j_node node) {\n"
    "\n"
    "  // fprintf(stderr, \"burg_code(): called code function of rule '%%s'\\n\", burg_rule_name[rule]);\n"
    "\n"
    "  burg_rule_code[rule](node);\n"
    "  burg_rule_count[rule] += 1;\n"
    "\n"
    "}\n\n");
  
  /*
  ** Emit the closure functions.
  */

  nonterminal = nonterminal_list;
  while (nonterminal) {
    emit_closure(file, nonterminal);
    nonterminal = nonterminal->next;
  }
 
  /*
  ** Emit the state functions.
  */
  
  terminal = terminal_list;
  while (terminal) {
    emit_terminal_state(file, terminal);
    terminal = terminal->next;
  }
 
  /*
  ** Emit the main state function.
  */
  
  fprintf(file,
    "j_state burg_state(j_operator operator, j_state l, j_state r) {\n"
    "\n"
    "  j_state state;\n"
    "\n"
    "  state = (j_state)arena_allocate(sizeof(j_State));\n"
    "  state->operator = operator;\n"
    "  state->lstate = l;\n"
    "  state->rstate = r;\n");
    
  nonterminal = nonterminal_list;
  i = 0;

  while (nonterminal) {
    fprintf(file, "  state->cost[%d] = (unsigned short)(0 - 1);   // infinite cost\n", i++);
    nonterminal = nonterminal->next;
  }
  fprintf(file, "  state->cost[%d] = (unsigned short)(0 - 1);   // infinite cost\n", i++);

  fprintf(file,
    "\n"
    "  switch (operator) {\n");

  terminal = terminal_list;
  while (terminal) {
    fprintf(file, 
      "    case %s:\n" 
      "      burg_%s_state(state);\n"
      "      break;\n", terminal->name, terminal->name);
    terminal = terminal->next;
  }
  
  fprintf(file,
    "    default:\n"
    "      fprintf(stderr, \"error in burg_state(): unrecognized operator '%%d'\\n\", operator);\n" 
    "      abort();\n"
    "  }\n"
    "\n"
    "  return state;\n"
    "}\n\n");
    
  /*
  ** Emit the recursive label function.
  */

  fprintf(file,
    "j_state burg_label(j_node node) {\n"
    "\n"
    "  j_state l = NULL;\n"
    "  j_state r = NULL;\n"
    "\n"
    "  if (node == NULL) {\n"
    "    fprintf(stderr, \"error in burg_label(): node is NULL\\n\");\n"
    "    abort();\n"
    "  }\n"
    "\n"
    "  // label the left tree:\n"
    "  if (node->node1) {\n"
    "    l = burg_label(node->node1);\n"
    "  }\n"
    "\n"
    "  // label the right tree:\n"
    "  if (node->node2) {\n"
    "    r = burg_label(node->node2);\n"
    "  }\n"
    "\n"
    "  node->state = burg_state(node->operator, l, r);\n"
    "\n"
    "  return node->state;\n"
    "}\n\n");
}

void emit_nonterminals(FILE * file, j_tree tree) {
  
  if (tree->nonterminal) {
    fprintf(file, "  nt_%s,\n", tree->nonterminal->name);
  }

  if (tree->ltree) {
    emit_nonterminals(file, tree->ltree);
  }
  
  if (tree->rtree) {
    emit_nonterminals(file, tree->rtree);
  }
}

void emit_burg_dump_state_rule(FILE * file) {
  
  j_nonterminal nonterminal;

  fprintf(file, 
    "void burg_dump_state_rule(j_state state, int depth) {\n"
    "\n"
    "  fprintf(stderr, \"%%s * state->rule = { ");

  nonterminal = nonterminal_list;
  while (nonterminal) {
    if (nonterminal->next) {
      fprintf(file, "nt_%s = %%d, ", nonterminal->name); 
    }
    else {
      fprintf(file, "nt_%s = %%d", nonterminal->name); 
    }
    
    nonterminal = nonterminal->next;
  }
  
  fprintf(file, 
    " }\\n\", burg_spaces(depth)");

  nonterminal = nonterminal_list;
  while (nonterminal) {
    fprintf(file, ", state->rule[nt_%s]", nonterminal->name); 
    nonterminal = nonterminal->next;
  }
  
  fprintf(file,
    ");\n"
    "}\n\n");
}

void emit_burg_dump_state_cost(FILE * file) {
  
  j_nonterminal nonterminal;

  fprintf(file, 
    "void burg_dump_state_cost(j_state state, int depth) {\n"
    "\n"
    "  fprintf(stderr, \"%%s * state->cost = { ");

  nonterminal = nonterminal_list;
  while (nonterminal) {
    if (nonterminal->next) {
      fprintf(file, "nt_%s = %%d, ", nonterminal->name); 
    }
    else {
      fprintf(file, "nt_%s = %%d", nonterminal->name); 
    }
    
    nonterminal = nonterminal->next;
  }
  
  fprintf(file, 
    " }\\n\", burg_spaces(depth)");

  nonterminal = nonterminal_list;
  while (nonterminal) {
    fprintf(file, ", state->cost[nt_%s]", nonterminal->name); 
    nonterminal = nonterminal->next;
  }
  
  fprintf(file,
    ");\n"
    "}\n\n");
}

void emit_burg_dump(FILE * file) {
  
  fprintf(file,
    "char *burg_spaces(const int depth) {\n"
    "\n"
    "  const char *spaces = \"                                          \";  // 40 spaces\n"
    "  return (char *)(spaces + strlen(spaces) - (2 * depth));\n"
    "}\n\n");
   
  emit_burg_dump_state_rule(file);
  emit_burg_dump_state_cost(file);
  
  fprintf(file,
    "void burg_dump_rule_table(void) {\n"
    "\n"
    "  int i;"
    "\n"
    "  for (i = 1; burg_rule_name[i]; i++) {\n"
    "    fprintf(stderr, \"  %%d: '%%s'\\n\", i, burg_rule_name[i]);\n"
    "  }\n"
    "}\n\n"
  );
  
  fprintf(file,
    "void burg_dump_tree(j_node node, int depth) {\n"
    "\n"
    "  if (depth == 0) {\n"
    "    fprintf(stderr, \"\\nrule table:\\n\");\n"
    "    burg_dump_rule_table();\n"
    "    fprintf(stderr, \"\\nsubject tree:\\n\");\n"
    "  }\n"
    "\n"
    "  fprintf(stderr, \"%%snode %%d (%%p)\\n\", burg_spaces(depth), node->operator, node);\n"
    "\n"
    "  burg_dump_state_rule(node->state, depth);\n"
    "  burg_dump_state_cost(node->state, depth);\n"
    "\n"
    "  if (node->node1) {\n"
    "    burg_dump_tree(node->node1, depth + 1);\n"
    "  }\n"
    "\n"
    "  if (node->node2) {\n"
    "    burg_dump_tree(node->node2, depth + 1);\n"
    "  }\n"
    "}\n\n");
}  

void emit_kids(FILE * file, j_tree tree) {

  if (tree->ltree) {
    fprintf(file, "->node1");
    emit_kids(file, tree->ltree);
  }
    
  if (tree->rtree) {
    fprintf(file, "->node2");
    emit_kids(file, tree->rtree);
  }
}

void emit_reducer(FILE * file) {

  j_nonterminal nonterminal;
  j_rule rule;
 
  /*
  ** Emit array index by rule numbers where each element points to a
  ** zero-terminated vector of short integers that encode the goal
  ** non-terminals for that rule's pattern, left-to-right.
  */
 
  rule = rule_list;
  while (rule) {
    fprintf(file, 
      "static short burg_nts_%d[] = {\n", rule->number);
    
    emit_nonterminals(file, rule->tree);

    fprintf(file,
      "  0\n"
      "};\n"
      "\n"); 
    
    rule = rule->next;
  }
   
  fprintf(file, 
    "short * burg_nts[] = {\n"
    "  0,\n");
    
  rule = rule_list;
  while (rule) {
    fprintf(file, "  burg_nts_%d,\n", rule->number);
 
    rule = rule->next;
  }
    
  fprintf(file,
    "};\n"
    "\n"); 

  /*
  ** Emit a function to dump the subject tree.
  */
  
  emit_burg_dump(file);
  
  /*
  ** Emit the rule function.
  */

  fprintf(file, 
    "int burg_rule(j_state state, int goal) {\n"
    "\n"
    "  int rule = 0;\n"
    "\n"
    "  assert(state && goal);\n"
    "\n"
    "  switch(goal) {\n");
    
  nonterminal = nonterminal_list;
  while (nonterminal) {
     fprintf(file, 
       "    case nt_%s:\n"
       "      rule = state->rule[nt_%s];\n"
       "      // fprintf(stderr, \"burg_rule(): state->rule[nt_%s] is %%d, nt_%s is %%d\\n\", state->rule[nt_%s], nt_%s);\n"
       "      break;\n", nonterminal->name, nonterminal->name, nonterminal->name, nonterminal->name, nonterminal->name, nonterminal->name);
    
    nonterminal = nonterminal->next;
  }
  
  fprintf(file,
    "    default:\n"
    "      fprintf(stderr, \"error in burg_rule(): unrecognized non-terminal '%%d'\\n\", goal);\n" 
    "      abort();\n"
    "  }\n"
    "\n"
    "  return rule;\n"
    "}\n"
    "\n");
  
  /*
  ** Emit the kids function.
  */

  fprintf(file, 
    "void burg_kids(j_node node, int rule, j_node kids[]) {\n"
    "\n"
    "  switch(rule) {\n");

  rule = rule_list;
  while (rule) {
   
    fprintf(file, "    case %d:\n", rule->number);
  
    if (rule->tree->terminal) { 
  
      if (rule->tree->ltree) { 
        fprintf(file, "      kids[0] = node->node1");
        emit_kids(file, rule->tree->ltree); 
        fprintf(file, ";\n");
      }
      else {
        fprintf(file, "      kids[0] = NULL;\n");
      }
    
      if (rule->tree->rtree) {
        fprintf(file, "      kids[1] = node->node2");
        emit_kids(file, rule->tree->rtree);
        fprintf(file, ";\n");
      }
      else {
        fprintf(file, "      kids[1] = NULL;\n");
      }
    }
    else {
      fprintf(file, "      kids[0] = node;\n");
      fprintf(file, "      kids[1] = NULL;\n");
    }
    
    fprintf(file, "      break;\n");

    rule = rule->next; 
  }
  
  fprintf(file,
    "    default:\n"
    "      fprintf(stderr, \"error in burg_kids(): invalid rule number '%%d'\\n\", rule);\n" 
    "      abort();\n"
    "  }\n"
    "\n"
    "}\n"
    "\n");
  /*
  ** Emit the statistics function.
  */

  fprintf(file, 
    "void burg_statistics(void) {\n"
    "\n"
    "  int i;\n"
    "\n"
    "  for (i = 1; i < %d; i++) {\n"
    "    fprintf(stderr, \"     %%6d: %%s\\n\", burg_rule_count[i], burg_rule_name[i]);\n"
    "  }\n"
    "}\n"
    "\n", rule_num() + 1);
 
}

int main(int argc, char * argv[]) {

  char * ifile = argv[2];
  char * ofile = argv[1];
  FILE * file = NULL;

  /*
  ** Open the input file.
  */
  
  if (ifile) {
    yyin = fopen(ifile, "r");
  }
  else {
    yyin = stdin;
  }
  
  /*
  ** Open the output file.
  */

  if (ofile) {
    file = fopen(ofile, "w");
  }
  else {
    file = stdout;
  }
  
  /*
  ** Parse the input file.
  */
  
  yyparse();

  /*
  ** Emit some debug output.
  */
  
  terminal_print();
  nonterminal_print();
  rule_print();

  /*
  ** Emit the output file.
  */

  emit_header(file);
  emit_labeller(file);
  emit_reducer(file);
  emit_footer(file);

  return 0;
} 

