/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: assembler-test.c 3863 2003-04-25 09:23:25Z dbuytaert $
*/

#include <unistd.h>
#include <stdio.h>

#include "j-spot.h"
#include "debug.h"
#include "j-test.h"
#include "../hal/cpu/arm/include/is.h"
#include "../hal/cpu/x86/include/is.h"

#include "assembler.h"

int tests;

#define test(code, l, r)             \
    tests++;                         \
    if (assembler_cmp(code, l, r) != 0) {    \
      fprintf(stderr, "- '%s == %s' invalid:\n     0x%x\t: %s\n     %s\t: %s\n", #l, #r, assembler_cmp(code, l, r), asm_insn(assembler_cmp(code, l, r)), #r, asm_insn(r)); \
      abort(); \
    }    

j_word assembler_cmp(j_buffer code, int size, j_word right) {

  j_word left;
  
  size = size < 4 ? size : 4;
  left = *(j_word *)(code->step - size);
 
  fprintf(stderr, "left = %p, right = %p, size = %d\n", (char *)left, (char *)right, size);
  
  return (left != right) ? left : 0;
}

void assembler_test(void) {  
  
  j_buffer code;
  
  /*
  ** Create a code buffer to emit the code to.
  */
  
  code = (j_buffer)malloc(sizeof(j_Buffer));
  code->size = 50 * 1024;
  code->base = (void *)malloc(code->size);
  code->step = code->base;
  
  /*
  ** Code generator test:
  */

  test(code, asm_emit(code, 1, 32, 8, 0xaa, 8, 0xbb, 8, 0xcc, 8, 0xdd), 0xaabbccdd);
  test(code, asm_emit(code, 0, 32, 8, 0xaa, 8, 0xbb, 8, 0xcc, 8, 0xdd), 0xddccbbaa);

  /*
  ** ARM instruction set:
  */ 

  test(code, asm_emit(code, arm_mov_rr(R0, R1)), 0xe1a00001);
  test(code, asm_emit(code, arm_mov_rr(R4, R3)), 0xe1a04003);
  test(code, asm_emit(code, arm_mov_rr(SP, R1)), 0xe1a0d001);
  test(code, asm_emit(code, arm_mov_rr(R1, LR)), 0xe1a0100e);
  test(code, asm_emit(code, arm_mov_rr(PC, LR)), 0xe1a0f00e);
  test(code, asm_emit(code, arm_mov_rr(LR, PC)), 0xe1a0e00f);
  
  test(code, asm_emit(code, arm_mov_ri8(R0, 50)), 0xe3a00032);
  test(code, asm_emit(code, arm_mov_ri8(R1, 20)), 0xe3a01014);
  test(code, asm_emit(code, arm_mov_ri8(R2, 10)), 0xe3a0200a);
  test(code, asm_emit(code, arm_mov_ri8(R3,  1)), 0xe3a03001);

  test(code, asm_emit(code, arm_add_rr(R1, R1, R2)), 0xe0811002);
  test(code, asm_emit(code, arm_add_rr(R2, R2, R1)), 0xe0822001);
  test(code, asm_emit(code, arm_add_rr(R3, R3, R2)), 0xe0833002);
  test(code, asm_emit(code, arm_add_ri8(R1, R1, 4)), 0xe2811004);
  test(code, asm_emit(code, arm_add_ri8(R2, R2, 4)), 0xe2822004);

  test(code, asm_emit(code, arm_sub_rr(R1, R1, R2)), 0xe0411002);
  test(code, asm_emit(code, arm_sub_ri8(R1, R3, 4)), 0xe2431004);     
  test(code, asm_emit(code, arm_sub_ri8(R5, R8, 8)), 0xe2485008);
  test(code, asm_emit(code, arm_sub_ri8(SP, SP, 8)), 0xe24dd008);

  test(code, asm_emit(code, arm_mul_rr(R3, R5, R0)), 0xe0030095);
  test(code, asm_emit(code, arm_mul_rr(R3, R0, R1)), 0xe0030190);
  test(code, asm_emit(code, arm_mul_rr(R12, R2, R3)), 0xe00c0392);
  
  test(code, asm_emit(code, arm_rsb_rr(R3, R5, R2)), 0xe0653002);
  test(code, asm_emit(code, arm_rsb_rr(R3, R7, R4)), 0xe0673004);
  test(code, asm_emit(code, arm_rsb_rr(SP, R12, SP)), 0xe06cd00d);
  
  test(code, asm_emit(code, arm_rsb_ri8(R1, R1, 0)), 0xe2611000);
  test(code, asm_emit(code, arm_rsb_ri8(R3, R2, 0)), 0xe2623000);
  test(code, asm_emit(code, arm_rsb_ri8(R2, R10, 32)), 0xe26a2020);
  
  test(code, asm_emit(code, arm_and_rr(R3, R5, R2)), 0xe0053002);
  test(code, asm_emit(code, arm_and_rr(R2, R2, R3)), 0xe0022003);
  test(code, asm_emit(code, arm_and_rr(R7, R7, R3)), 0xe0077003);
  
  test(code, asm_emit(code, arm_orr_rr(R6, R3, R1)), 0xe1836001);
  test(code, asm_emit(code, arm_orr_rr(R4, R3, R2)), 0xe1834002);
  test(code, asm_emit(code, arm_orr_rr(R0, R0, R3)), 0xe1800003);
  
  test(code, asm_emit(code, arm_eor_rr(R3, R8, R7)), 0xe0283007);
  test(code, asm_emit(code, arm_eor_rr(R10, R6, R7)), 0xe026a007);
  test(code, asm_emit(code, arm_eor_rr(R3, R0, R12)), 0xe020300c);
  
  test(code, asm_emit(code, arm_cmp_rr(R1, R2)), 0xe1510002);
  test(code, asm_emit(code, arm_cmp_rr(R3, R6)), 0xe1530006);
  test(code, asm_emit(code, arm_cmp_rr(R4, R2)), 0xe1540002);

  test(code, asm_emit(code, arm_stmdb(SP, BIT_R2 | BIT_R3)), 0xe92d000c);
  test(code, asm_emit(code, arm_stmdb(SP, BIT_R11 | BIT_R12 | BIT_LR | BIT_PC)), 0xe92dd800);
  test(code, asm_emit(code, arm_stmdb(SP, BIT_R4 | BIT_R5 | BIT_R11 | BIT_R12 | BIT_LR | BIT_PC)), 0xe92dd830);

  test(code, asm_emit(code, arm_ldmdb(R11, BIT_R4 | BIT_R5 | BIT_R11 | BIT_SP | BIT_PC)), 0xe91ba830);
  test(code, asm_emit(code, arm_ldmdb(R11, BIT_R11 | BIT_SP | BIT_PC)), 0xe91ba800);

  test(code, asm_emit(code, arm_ldr_m12r(R3, R4, 4)), 0xe5943004);
  test(code, asm_emit(code, arm_ldr_m12r(R5, PC, 60)), 0xe59f503c);
  test(code, asm_emit(code, arm_ldr_m12r(R1, PC, 312)), 0xe59f1138);
    // TODO: I had problems with testing negative offsets.

  test(code, asm_emit(code, arm_str_rm12(R1, SP, 8)), 0xe58d1008);
  test(code, asm_emit(code, arm_str_rm12(R3, SP, 12)), 0xe58d300c);
  test(code, asm_emit(code, arm_str_rm12(R4, SP, 4)), 0xe58d4004);
  test(code, asm_emit(code, arm_str_rm12(R0, R11, -16)), 0xe50b0010);
  test(code, asm_emit(code, arm_str_rm12(R2, R11, -28)), 0xe50b201c);
  test(code, asm_emit(code, arm_str_rm12(R3, R11, -20)), 0xe50b3014);

  test(code, asm_emit(code, arm_str_rm0(R1, SP)), 0xe58d1000);
  test(code, asm_emit(code, arm_str_rm0(R1, R2)), 0xe5821000);
  test(code, asm_emit(code, arm_str_rm0(R2, R3)), 0xe5832000);

  /*
  ** x86 instruction set:
  */

  test(code, asm_emit(code, x86_pop(EBX)), 0x5b);
  test(code, asm_emit(code, x86_pop(EDI)), 0x5f);
  test(code, asm_emit(code, x86_pop(ESI)), 0x5e);
  
  test(code, asm_emit(code, x86_mov_rr(EAX, EDX)), 0xd089);
  test(code, asm_emit(code, x86_mov_rr(ESI, ESI)), 0xf689);
  test(code, asm_emit(code, x86_mov_rr(EBP, ESP)), 0xe589);

  /*
    // TODO: change test framework so it also checks 32-bit immediates!
  test(code, asm_emit(code, x86_mov_ri32(EAX, 0x81a0fd0)), 0x081a0fd0a1);
  
  test(code, asm_emit(code, x86_mov_rm8(EDX, EBP, + 8)), 0x08558b);
  test(code, asm_emit(code, x86_mov_rm8(EBX, EBP, - 4)), 0xfc5d8b);
  test(code, asm_emit(code, x86_mov_rm8(ECX, EBP, + 12)), 0x0c4d8b);
  test(code, asm_emit(code, x86_mov_rm8(EDX, ECX, + 4)), 0x04518b);
  
  test(code, asm_emit(code, x86_mov_m8r(EBP, 0x8, EAX)), 0x084589);
  test(code, asm_emit(code, x86_mov_m8r(EBP, 0x4, EBX)), 0x045d89);
  test(code, asm_emit(code, x86_mov_m8r(EAX, 0x4, EBX)), 0x045889);
  test(code, asm_emit(code, x86_mov_m8r(EAX, 0x4, EAX)), 0x044089);
  */
  test(code, asm_emit(code, x86_add_rr(EDX, EAX)), 0xc201);
  test(code, asm_emit(code, x86_add_rr(EAX, EDX)), 0xd001);
  test(code, asm_emit(code, x86_add_rr(EDX, EBX)), 0xda01);
  test(code, asm_emit(code, x86_add_rr(EAX, EBX)), 0xd801);
  test(code, asm_emit(code, x86_add_rr(EAX, EDI)), 0xf801);
  test(code, asm_emit(code, x86_add_rr(EBX, EDI)), 0xfb01);

  test(code, asm_emit(code, x86_add_m8r(EDI, 0x8, ESI)), 0x087e03);
  test(code, asm_emit(code, x86_add_m8r(EAX, 0x14, EBP)), 0x144503);
  test(code, asm_emit(code, x86_add_m8r(EAX, 0x8, EBP)), 0x084503);
  test(code, asm_emit(code, x86_add_m8r(ECX, 0x10, EDX)), 0x104a03);

  test(code, asm_emit(code, x86_add_rm8(EDX, 0x14, EAX)), 0x144201);
  test(code, asm_emit(code, x86_add_rm8(EBP, 0xffffffe8, EAX)), 0xe84501);
  test(code, asm_emit(code, x86_add_rm8(EBP, 0xffffffe4, EAX)), 0xe44501);

  test(code, asm_emit(code, x86_add_ri8(EBX, 0x04)), 0x04c383);
  test(code, asm_emit(code, x86_add_ri8(ESP, 0x08)), 0x08c483);
  test(code, asm_emit(code, x86_add_ri8(ESP, 0x0c)), 0x0cc483);
  test(code, asm_emit(code, x86_add_ri8(ECX, 0x0ffffffd0)), 0xd0c183);
  test(code, asm_emit(code, x86_add_ri8(EBX, 0x02)), 0x02c383);

  /*
    // TODO: our test framework is not capable of testing those
  test(code, asm_emit(code, x86_add_ri32(EBX, 0xb4f3)), 0x00b4f3c383);
  */

  test(code, asm_emit(code, x86_add_i8m8(EAX, 0x8, 0x4)), 0x04084083);
  test(code, asm_emit(code, x86_add_i8m8(EBX, 0x14, 0x4)), 0x04144383);
  test(code, asm_emit(code, x86_add_i8m8(ECX, 0x8, 0x2)), 0x02084183);
  test(code, asm_emit(code, x86_add_i8m8(EDX, 0x8, 0x4)), 0x04084283);

  test(code, asm_emit(code, x86_sub_rr(EDX, EAX)), 0xc229);
  test(code, asm_emit(code, x86_sub_rr(ESI, EAX)), 0xc629);
  test(code, asm_emit(code, x86_sub_rr(EDX, EBX)), 0xda29);
  test(code, asm_emit(code, x86_sub_rr(EAX, ECX)), 0xc829);
  test(code, asm_emit(code, x86_sub_rr(EAX, EDX)), 0xd029);
  test(code, asm_emit(code, x86_sub_rr(ECX, EDX)), 0xd129);

  test(code, asm_emit(code, x86_sub_m8r(EAX, 0xc, EBP)), 0x0c452b);
  test(code, asm_emit(code, x86_sub_m8r(EBX, 0xffffffec, EBP)), 0xec5d2b);
  test(code, asm_emit(code, x86_sub_m8r(ECX, 0x10, EBP)), 0x104d2b);
  test(code, asm_emit(code, x86_sub_m8r(EDX, 0x3c, ECX)), 0x3c512b);

  test(code, asm_emit(code, x86_sub_rm8(EBP, 0xffffffb8, EDI)), 0xb87d29);
  test(code, asm_emit(code, x86_sub_rm8(EBP, 0xffffffac, ESI)), 0xac7529);
  test(code, asm_emit(code, x86_sub_rm8(EBP, 0xffffffc8, ECX)), 0xc84d29);
  test(code, asm_emit(code, x86_sub_rm8(EBP, 0xffffffe0, ECX)), 0xe04d29);
  test(code, asm_emit(code, x86_sub_rm8(EBP, 0xffffffc8, EAX)), 0xc84529);
 
  test(code, asm_emit(code, x86_sub_ri8(ESP, 0x4)), 0x04ec83);
  test(code, asm_emit(code, x86_sub_ri8(ESP, 0x8)), 0x08ec83);
  test(code, asm_emit(code, x86_sub_ri8(EBX, 0x1)), 0x01eb83);
  test(code, asm_emit(code, x86_sub_ri8(EAX, 0x1)), 0x01e883);

  /*
    // TODO: change test framework so it also checks 32-bit immediates!
  test(code, asm_emit(code, x86_sub_ri32(ESP, 0x184)), 0x00000184ec81);
  test(code, asm_emit(code, x86_sub_ri32(EBX, 0x813a2a0)), 0x0813a2a0eb81);
  */
  
  test(code, asm_emit(code, x86_sub_i8m8(EBP, 0xfffffff0, 0x2)), 0x02f06d83);
  test(code, asm_emit(code, x86_sub_i8m8(EBP, 0xffffffc0, 0x2)), 0x02c06d83);
  test(code, asm_emit(code, x86_sub_i8m8(EBP, 0xffffffb8, 0x2)), 0x02b86d83);
  test(code, asm_emit(code, x86_sub_i8m8(EBP, 0xffffffc8, 0x40)), 0x40c86d83);
  
  test(code, asm_emit(code, x86_mul_rr(EAX, EDX)), 0xc2af0f);
  test(code, asm_emit(code, x86_mul_rr(ECX, EDI)), 0xcfaf0f);
  test(code, asm_emit(code, x86_mul_rr(ECX, EBX)), 0xcbaf0f);
  test(code, asm_emit(code, x86_mul_rr(EDX, ESI)), 0xd6af0f);
  test(code, asm_emit(code, x86_mul_rr(EAX, EDX)), 0xc2af0f);
  test(code, asm_emit(code, x86_mul_rr(EDX, ECX)), 0xd1af0f);

  test(code, asm_emit(code, x86_mul_ri8r(ECX, 0x4, EAX)), 0x04c86b);
  test(code, asm_emit(code, x86_mul_ri8r(EDX, 0x4, EAX)), 0x04d06b);

  test(code, asm_emit(code, x86_mul_m8r(EAX, EDI, 0x10)), 0x1047af0f);
  test(code, asm_emit(code, x86_mul_m8r(EAX, EDX, 0xfffffff8)), 0xf842af0f);
  test(code, asm_emit(code, x86_mul_m8r(EDX, EBP, 0xffffff8c)), 0x8c55af0f);
  test(code, asm_emit(code, x86_mul_m8r(EAX, EBP, 0xffffff88)), 0x8845af0f);
  test(code, asm_emit(code, x86_mul_m8r(EDX, EBP, 0xffffffa0)), 0xa055af0f);
  test(code, asm_emit(code, x86_mul_m8r(ECX, EBP, 0xffffffa4)), 0xa44daf0f);
 
  test(code, asm_emit(code, x86_neg_r(EDI)), 0xdff7);
  test(code, asm_emit(code, x86_neg_r(EBX)), 0xdbf7);
  test(code, asm_emit(code, x86_neg_r(ESI)), 0xdef7);
  test(code, asm_emit(code, x86_neg_r(EAX)), 0xd8f7);
  test(code, asm_emit(code, x86_neg_r(ECX)), 0xd9f7);
  test(code, asm_emit(code, x86_neg_r(EDX)), 0xdaf7);

  test(code, asm_emit(code, x86_neg_m8(EAX, 0xfffffff8)), 0xf858f7);
  test(code, asm_emit(code, x86_neg_m8(EBP, 0xffffffc4)), 0xc45df7);
  test(code, asm_emit(code, x86_neg_m8(ESI, 0x4)), 0x045ef7);
 
  test(code, asm_emit(code, x86_lea(ESI, 0x0, ESI)), 0x00768d);
  test(code, asm_emit(code, x86_lea(EDX, 0x4, EAX)), 0x04508d);
  test(code, asm_emit(code, x86_lea(EAX, 0x8, EBX)), 0x08438d);
  test(code, asm_emit(code, x86_lea(EBX, 0xffffffe0, ESI)), 0xe05e8d);
  test(code, asm_emit(code, x86_lea(ESP, 0xfffffff4, EBP)), 0xf4658d);
  test(code, asm_emit(code, x86_lea(ECX, 0xffffffff, EBP)), 0xff4d8d);

  test(code, asm_emit(code, x86_gt_8(0xf2)), 0xf27f);
  test(code, asm_emit(code, x86_gt_8(0xf2)), 0xf27f);
  
  test(code, asm_emit(code, x86_ge_8(0x23)), 0x237d);
  test(code, asm_emit(code, x86_ge_8(0xf2)), 0xf27d);
  
  test(code, asm_emit(code, x86_lt_8(0x23)), 0x237c);
  test(code, asm_emit(code, x86_lt_8(0xf2)), 0xf27c);
  
  test(code, asm_emit(code, x86_le_8(0xf2)), 0xf27e);
  test(code, asm_emit(code, x86_le_8(0x23)), 0x237e);
  
  test(code, asm_emit(code, x86_eq_8(0xf2)), 0xf274);
  test(code, asm_emit(code, x86_eq_8(0x23)), 0x2374);
  
  test(code, asm_emit(code, x86_ne_8(0xf2)), 0xf275);
  test(code, asm_emit(code, x86_ne_8(0x23)), 0x2375);
 
   // NOTE: the conditional jump with full displacement (32-bit memory
   //       address) can't be tested due to some limitions of the test
   //       framework.
  
  test(code, asm_emit(code, x86_jmp_8(0x10)), 0x10eb);
  test(code, asm_emit(code, x86_jmp_8(0xef)), 0xefeb);
 
   // NOTE: the unconditional jump with full displacement (32-bit memory
   //       address) can't be tested due to some limitions of the test
   //       framework.
  
  test(code, asm_emit(code, x86_cmp_rr(EAX, EDX)), 0xd039);
  test(code, asm_emit(code, x86_cmp_rr(EAX, EBX)), 0xd839);
  test(code, asm_emit(code, x86_cmp_rr(EBX, EDI)), 0xfb39);
  test(code, asm_emit(code, x86_cmp_rr(EBX, ESI)), 0xf339);

  test(code, asm_emit(code, x86_cmp_rm8(ESI, 0x14, EBP)), 0x14753b);
  test(code, asm_emit(code, x86_cmp_rm8(EBX, 0x18, ESI)), 0x185e3b);
  test(code, asm_emit(code, x86_cmp_rm8(ESI, 0x0c, EBP)), 0x0c753b);
 
  test(code, asm_emit(code, x86_cmp_m8r(0x0c, ECX, EAX)), 0x0c4139);
  test(code, asm_emit(code, x86_cmp_m8r(0xe4, EBP, EDI)), 0xe47d39);
  test(code, asm_emit(code, x86_cmp_m8r(0xb0, EBP, ECX)), 0xb04d39);

  test(code, asm_emit(code, x86_cmp_i8m8(EBP, 0xfffffff0, 0x1)), 0x01f07d83);
  test(code, asm_emit(code, x86_cmp_i8m8(EBP, 0xffffffe8, 0x1)), 0x01e87d83);  
  test(code, asm_emit(code, x86_cmp_i8m8(EBX, 0x18, 0xffffffff)), 0xff187b83);   
  test(code, asm_emit(code, x86_cmp_i8m8(EDX, 0xfffffffc, 0x1)), 0x01fc7a83);   

  /*
    // NOTE: can not be tested yet due to some limitations of our test framework.
  test(code, asm_emit(code, x86_cmp_i32r(EAX, 0xfd42)), 0xfd42053b);
  test(code, asm_emit(code, x86_cmp_i32r(EDX, 0xfcf8)), 0xfcf8153b);
  test(code, asm_emit(code, x86_cmp_i32r(EDX, 0x1234)), 0x12341d3b);
  */
  
  test(code, asm_emit(code, x86_and_rr(EAX, ESI)), 0xc621);
  test(code, asm_emit(code, x86_and_rr(EAX, EBX)), 0xc321);
  test(code, asm_emit(code, x86_and_rr(ESI, EAX)), 0xf021);

  test(code, asm_emit(code, x86_and_i8r(EAX, 0x3f)), 0x3fe083);
  test(code, asm_emit(code, x86_and_i8r(EBX, 0xfffffffc)), 0xfce383);
  test(code, asm_emit(code, x86_and_i8r(ECX, 0x8)), 0x08e183);
  test(code, asm_emit(code, x86_and_i8r(EDX, 0x1)), 0x01e283);

  test(code, asm_emit(code, x86_and_m8r(EAX, 0xffffffe8, EBP)), 0xe84523);
  test(code, asm_emit(code, x86_and_m8r(EBX, 0xfffffff0, EBP)), 0xf05d23);
  test(code, asm_emit(code, x86_and_m8r(EDX, 0xffffffd4, EBP)), 0xd45523);
  
  test(code, asm_emit(code, x86_or_rr(ECX, EDX)), 0xca09);
  test(code, asm_emit(code, x86_or_rr(EBX, EAX)), 0xd809);
  test(code, asm_emit(code, x86_or_rr(EAX, EBX)), 0xc309);

  test(code, asm_emit(code, x86_or_m8r(EAX, 0xffffffd4, EBP)), 0xd4450b);
  test(code, asm_emit(code, x86_or_m8r(ECX, 0xffffffc4, EBP)), 0xc44d0b);
  test(code, asm_emit(code, x86_or_m8r(EDX, 0xffffffec, EBP)), 0xec550b);

  test(code, asm_emit(code, x86_or_rm8(EAX, 0xfffffff8, EDX)), 0xf85009);
  test(code, asm_emit(code, x86_or_rm8(EBX, 0xfffffffc, EAX)), 0xfc4309);
  test(code, asm_emit(code, x86_or_rm8(EDX, 0xfffffff0, EAX)), 0xf04209);

  test(code, asm_emit(code, x86_or_i8r(EAX, 0x10)), 0x10c883);
  test(code, asm_emit(code, x86_or_i8r(EAX, 0x04)), 0x04c883);
  test(code, asm_emit(code, x86_or_i8r(EDX, 0x01)), 0x01ca83);
  test(code, asm_emit(code, x86_or_i8r(EDX, 0x20)), 0x20ca83);
 
  test(code, asm_emit(code, x86_xor_rr(EDX, EDX)), 0xd231);
  test(code, asm_emit(code, x86_xor_rr(EBX, EBX)), 0xdb31);
  test(code, asm_emit(code, x86_xor_rr(EAX, EAX)), 0xc031);

  test(code, asm_emit(code, x86_xor_m8r(EAX, 0xffffffd4, EBP)), 0xd44533);
  test(code, asm_emit(code, x86_xor_m8r(EAX, 0xffffffcc, EBP)), 0xcc4533);

  test(code, asm_emit(code, x86_xor_rm8(EBP, 0xffffffe8, EAX)), 0xe84531);
  test(code, asm_emit(code, x86_xor_rm8(EBP, 0xffffffe8, ECX)), 0xe84d31);
  test(code, asm_emit(code, x86_xor_rm8(EBP, 0xffffffd8, EDX)), 0xd85531);

  test(code, asm_emit(code, x86_xor_i8r(EAX, 0x1)), 0x01f083);
  test(code, asm_emit(code, x86_xor_i8r(EAX, 0x1f)), 0x1ff083);

  test(code, asm_emit(code, x86_shr(EAX)), 0xe8d1);
  test(code, asm_emit(code, x86_shr(EBX)), 0xebd1);
  test(code, asm_emit(code, x86_shr(ECX)), 0xe9d1);  
  test(code, asm_emit(code, x86_shr(EDX)), 0xead1);
 
  test(code, asm_emit(code, x86_shr_cl(EAX)), 0xe8d3);
  test(code, asm_emit(code, x86_shr_cl(ESI)), 0xeed3);
  test(code, asm_emit(code, x86_shr_cl(EDI)), 0xefd3);
 
  test(code, asm_emit(code, x86_shl(EAX)), 0xe0d1);
  test(code, asm_emit(code, x86_shl(ECX)), 0xe1d1);
  test(code, asm_emit(code, x86_shl(EDX)), 0xe2d1);
  
  test(code, asm_emit(code, x86_shl_cl(EAX)), 0xe0d3);
  test(code, asm_emit(code, x86_shl_cl(ESI)), 0xe6d3);
  test(code, asm_emit(code, x86_shl_cl(EDX)), 0xe2d3);

  /*
   // NOTE: no longer works, we use a 16-bit variant now.
  test(code, asm_emit(code, x86_dec_r(EAX)), 0x48);
  test(code, asm_emit(code, x86_dec_r(EBX)), 0x4b);
  test(code, asm_emit(code, x86_dec_r(ECX)), 0x49);
  test(code, asm_emit(code, x86_dec_r(EDX)), 0x4a);
  */
  
  test(code, asm_emit(code, x86_dec_m8(EBP, 0xffffffe8)), 0xe84dff);
  test(code, asm_emit(code, x86_dec_m8(EBP, 0xffffff9c)), 0x9c4dff);
  test(code, asm_emit(code, x86_dec_m8(EAX, 0x20)), 0x2048ff);

  /*
   // NOTE: no longer works, we use a 16-bit variant now.
  test(code, asm_emit(code, x86_inc_r(EAX)), 0x40);
  test(code, asm_emit(code, x86_inc_r(EBX)), 0x43);
  test(code, asm_emit(code, x86_inc_r(ECX)), 0x41);
  test(code, asm_emit(code, x86_inc_r(EDX)), 0x42);
  */

  test(code, asm_emit(code, x86_inc_m8(EBP, 0xffffffc8)), 0xc845ff);
  test(code, asm_emit(code, x86_inc_m8(EBP, 0xffffffc0)), 0xc045ff);
  test(code, asm_emit(code, x86_inc_m8(EAX, 0x20)), 0x2040ff);
  test(code, asm_emit(code, x86_inc_m8(EBX, 0x14)), 0x1443ff);
  
  test(code, asm_emit(code, x86_sete(AL)), 0xc0940f);
  
  test(code, asm_emit(code, x86_nop()), 0x90);
  
  /*
  ** Debug output:
  */        

  fprintf(stderr, "--> passed %d tests\n", tests);

}

