/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package javax.crypto;

import java.security.Key;
import java.security.InvalidKeyException;
import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.Provider;
import java.security.SecureRandom;
import java.security.spec.AlgorithmParameterSpec;

public class KeyAgreement {

  public final static KeyAgreement getInstance(String algorithm)throws NoSuchAlgorithmException{
    SecurityAction action = new SecurityAction(algorithm, "KeyAgreement.");
    return new KeyAgreement((KeyAgreementSpi) action.spi, action.provider, algorithm);
  }

  public final static KeyAgreement getInstance(String algorithm, Provider provider) throws NoSuchAlgorithmException{
    SecurityAction action = new SecurityAction(algorithm, provider, "KeyAgreement.");
    return new KeyAgreement((KeyAgreementSpi) action.spi, action.provider, algorithm);
  }

  public final static KeyAgreement getInstance(String algorithm, String provider)
    throws NoSuchAlgorithmException, NoSuchProviderException {

    SecurityAction action = new SecurityAction(algorithm, provider, "KeyAgreement.");
    return new KeyAgreement((KeyAgreementSpi) action.spi, action.provider, algorithm);
  }

  private Provider provider;
  private KeyAgreementSpi keyAgrSpi;
  private String algorithm;
  private boolean invalidState = true;

  protected KeyAgreement(KeyAgreementSpi keyAgrSpi, Provider provider, String algorithm){
    if (keyAgrSpi == null || provider == null || algorithm == null){
            throw new NullPointerException();
    }
    this.keyAgrSpi = keyAgrSpi;
    this.provider = provider;
    this.algorithm = algorithm;
  }

  public final String getAlgorithm(){
    return algorithm;
  }

  public final Provider getProvider(){
    return provider;
  }

  public final Key doPhase(Key key, boolean last) throws InvalidKeyException, IllegalStateException {
    if(invalidState){
      throw new IllegalStateException("KeyAgreement was not yet initialized");
    }
    return keyAgrSpi.engineDoPhase(key, last);
  }

  public final byte[] generateSecret() throws IllegalStateException {
    if(invalidState){
      throw new IllegalStateException("KeyAgreement was not yet initialized");
    }
    return keyAgrSpi.engineGenerateSecret();
  }

  public final int generateSecret(byte[] bytes, int offset) throws IllegalStateException, ShortBufferException {
    if(invalidState){
      throw new IllegalStateException("KeyAgreement was not yet initialized");
    }
    return keyAgrSpi.engineGenerateSecret(bytes,offset);
  }

  public final SecretKey generateSecret(String algorithm)
    throws IllegalStateException, NoSuchAlgorithmException, InvalidKeyException {

    if(invalidState){
      throw new IllegalStateException("KeyAgreement was not yet initialized");
    }
    return keyAgrSpi.engineGenerateSecret(algorithm);
  }

  public final void init(Key key) throws InvalidKeyException {
    keyAgrSpi.engineInit(key, new SecureRandom());
    invalidState = false;
  }

  public final void init(Key key, SecureRandom srandom) throws InvalidKeyException {
    keyAgrSpi.engineInit(key, srandom);
    invalidState = false;
  }

  public final void init(Key key, AlgorithmParameterSpec params) throws InvalidKeyException, InvalidAlgorithmParameterException {
    keyAgrSpi.engineInit(key, params, new SecureRandom());
    invalidState = false;
  }

  public final void init(Key key, AlgorithmParameterSpec params,  SecureRandom srandom)
    throws InvalidKeyException, InvalidAlgorithmParameterException {

    keyAgrSpi.engineInit(key, params, srandom);
    invalidState = false;
  }
}
