/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "wonka.h"   /* the w_ types */
#include "oswald.h"
#include "jdwp.h"
#include "arrays.h"
#include "clazz.h"
#include "heap.h"
#include "checks.h"
#include "descriptor.h"

/*
** Get the number of elements in a given array.
*/

w_void jdwp_array_length(jdwp_command_packet cmd) {
  w_instance  instance;
  w_int       length;

  /*
  ** Get the instance from the command packet.
  */

  memcpy(&instance, &cmd->data[0], 4);

  /*
  ** Get the length of the array.
  */
  
  length = instance2Array_length(instance);
  length = swap_int(length);

  /*
  ** Send this result to the debugger.
  */
  
  jdwp_send_reply(cmd->id, &length, 4, jdwp_err_none);

  return;
}


/*
** Get the elements from given array. The given range should be within the bounds of
** the array.
*/

w_void jdwp_array_get_values(jdwp_command_packet cmd) {
  w_instance  instance;
  w_clazz     clazz;
  w_int       first;
  w_int       length;
  w_ubyte     *reply;
  w_int       i;
  w_int       position = 0;
  w_byte      abyte;
  w_short     ashort;
  w_int       anint;
  w_long      along;
  w_desc      elementDesc;
  w_clazz     elementClazz;
  w_ubyte     *array;

  /*
  ** Get the instance, the number of the first element and the number of elements
  ** from the command packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  memcpy(&first,  &cmd->data[4], 4);
  memcpy(&length, &cmd->data[8], 4);
  
  length = swap_int(length);
  first  = swap_int(first);
  
  woempa(9, "Get array values.  Start: %d,  Length: %d\n", first, length);

  /*
  ** Check the range against the bounds of the array.
  */

  if((w_word)(first + length) > instance2Array_length(instance)) {

    /*
    ** Naughty naught, the range doesn't match the boundaries of the array.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_length);

  } else {

    /*
    ** The range is OK.
    ** Allocate some memory for the reply packet.
    */

    reply = x_mem_alloc((w_word)(1 + 4 + length * 10));

    /*
    ** Get the type of elements in the array.
    */

    array = instance2Array_byte(instance);
    clazz = instance2clazz(instance);

    elementDesc = clazz->desc->body.array.component;
    elementClazz = desc2Class(clazz->loader, elementDesc);

    /*
    ** Check the type of the elements and add a type tag to the reply.
    */

    if(isAssignmentCompatible(elementClazz, clazzThread)) {            /* Threads */
      reply[0] = jdwp_tag_thread;
    }
    else if(isAssignmentCompatible(elementClazz, clazzThreadGroup)) {  /* ThreadGroups */
      reply[0] = jdwp_tag_thread_group;
    }
    else if(isAssignmentCompatible(elementClazz, clazzClassLoader)) {  /* ClassLoaders */
      reply[0] = jdwp_tag_class_loader;
    }
    else if(isAssignmentCompatible(elementClazz, clazzString)) {       /* Strings */
      reply[0] = jdwp_tag_string;
    }
    else if(elementDesc->tag == D_Primitive) {                         /* primitive types */
      switch(elementDesc->body.primitive.type) {
        case P_boolean:   reply[0] = jdwp_tag_boolean;   break;        /* booleans */
        case P_char:      reply[0] = jdwp_tag_char;      break;        /* chars */
        case P_float:     reply[0] = jdwp_tag_float;     break;        /* floats */
        case P_double:    reply[0] = jdwp_tag_double;    break;        /* doubles */
        case P_byte:      reply[0] = jdwp_tag_byte;      break;        /* bytes */
        case P_short:     reply[0] = jdwp_tag_short;     break;        /* shorts */
        case P_int:       reply[0] = jdwp_tag_int;       break;        /* ints */
        case P_long:      reply[0] = jdwp_tag_long;      break;        /* longs */
        case P_void:      reply[0] = jdwp_tag_void;      break;        /* voids (?) */
        case P_Object:    reply[0] = jdwp_tag_object;    break;        /* Objects */
      }
    }
    else if(elementDesc->tag == D_Class) {                             /* Classes */
      reply[0] = jdwp_tag_object; 
    }
    else if(elementDesc->tag == D_Array) {                             /* arrays */
      reply[0] = jdwp_tag_array;
    }

    /*
    ** Store the number of elements in the reply packet.
    */

    length = swap_int(length);
    memcpy(&reply[1], &length, 4);
    length = swap_int(length);
    position = 5;
    
    /*
    ** Go over all the elements in the array and add them to the reply.
    */
    
    for(i = first; i < first + length; i++) {

      switch(reply[0]) {
        case jdwp_tag_void:
          break;
        case jdwp_tag_byte:                     /* byte & boolean take 1 byte */
        case jdwp_tag_boolean:
          memcpy(&abyte, &array[i], 1);
          memcpy(&reply[position], &abyte, 1);
          position += 1;
          break;
        case jdwp_tag_char:                     /* char & short take 2 bytes */
        case jdwp_tag_short:
          memcpy(&ashort, &array[i * 2], 2);
          ashort = swap_short(ashort);
          memcpy(&reply[position], &ashort, 2);
          position += 2;
          break;
        case jdwp_tag_int:                      /* int & float take 4 bytes */
        case jdwp_tag_float:
          memcpy(&anint, &array[i * 4], 4);
          anint = swap_int(anint);
          memcpy(&reply[position], &anint, 4);
          position += 4;
          break;
        case jdwp_tag_double:                   /* double & long take 8 bytes */
        case jdwp_tag_long:         
          memcpy(&along, &array[i * 8], 8);
          along = swap_long(along);
          memcpy(&reply[position], &along, 8);
          position += 8;
          break;
        case jdwp_tag_array:                    /* All the others take 4 bytes    */
        case jdwp_tag_object:                   /* plus an additional type tag    */
        case jdwp_tag_string:                   /* which gives 5 bytes in total   */
        case jdwp_tag_thread:                   /* Another reason why they are    */
        case jdwp_tag_thread_group:             /* handled seperately is because  */
        case jdwp_tag_class_loader:             /* they are sent as ID's and do   */ 
        case jdwp_tag_class_object:             /* not require to be byte swapped */
          reply[position] = reply[0];
          position += 1;
          memcpy(&anint, &array[i * 4], 4);
          memcpy(&reply[position], &anint, 4);
          position += 4;
          break;
      }

    }

    /*
    ** Send the reply to the debugger.
    */ 
  
    jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);

    /*
    ** Clean up.
    */

    x_mem_free(reply);
  } 
    
  return;
}


/*
** The dispatcher for the 'Array reference' command set.
*/

w_void dispatch_arrayref(jdwp_command_packet cmd) {

  switch((jdwp_arrayref_cmd)cmd->command) {
    case jdwp_arrayref_length:      jdwp_array_length(cmd);      break;
    case jdwp_arrayref_getValues:   jdwp_array_get_values(cmd);  break;
    case jdwp_arrayref_setValues:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
    default:
  }
  
}

