/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"      /* x_thread_... */
#include "wonka.h"       /* the w_ types */
#include "ts-mem.h"      /* allocMem_system */
#include "realms.h"      /* allocMem_system */
#include "jdwp.h"
#include "clazz.h"       /* w_clazz struct */
#include "methods.h"     /* w_method struct */
#include "descriptor.h"  /* w_desc struct */
#include "checks.h"      /* isAssignmentCompatible */


/*
** Returns the JNI signature of the given referenceID.
*/

w_void jdwp_ref_signature(jdwp_command_packet cmd) {
  w_clazz  clazz;
  w_ubyte  *name;
  w_ubyte  *sig;
  w_int    length;
 
  /*
  ** Get the class from the command packet.
  */
 
  memcpy(&clazz, &cmd->data[0], 4);

  /*
  ** Get the signature of the class and convert it to UTF8. 
  */

  sig = x_mem_alloc(255);
  name = jdwp_string2UTF8(clazz->desc->name, &length);

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, name, length + 4, jdwp_err_none);

  /*
  ** Clean up.
  */

  x_mem_free(name);
  x_mem_free(sig);
  
  return;
}


/*
** Returns a ClassLoaderID for the the given reference type. 
*/

w_void jdwp_ref_classloader(jdwp_command_packet cmd) {
  w_clazz     clazz;
  w_instance  classloader;
 
  /*
  ** Get the class from the command packet.
  */
 
  memcpy(&clazz, &cmd->data[0], 4);

  /*
  ** Get the classloader.
  */ 

  classloader = clazz->loader;

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, &classloader, 4, jdwp_err_none);

  return;
}


/*
** Get information about each field (except the inhereted fields) of the given reference type. 
** Note: Fields will be returned in the same order as they are in the .class file.
*/

w_void jdwp_ref_fields(jdwp_command_packet cmd) {
  w_clazz    clazz;
  w_field   field;
  w_int      i;
  w_ubyte    *reply;
  w_ubyte    *string;
  w_int      position = 0;
  w_int      count;
  w_int      length;
  w_flags    flags;
  
  /*
  ** Get the class and the number of fields.
  */
  
  memcpy(&clazz, &cmd->data[0], 4);
  count = clazz->numFields;

  /*
  ** Allocate enough memory.
  ** It's not that easy to determine the exact amount required so we take what 
  ** should be enough... (Reminds me of the '640K should be enough for everyone'
  ** quote, not a good sign).
  */
  
  reply = x_mem_alloc((w_word)(4 + (count * (4 + 50 + 50 + 4))));

  /*
  ** Add the number of fields to the packet.
  */
  
  count = swap_int(count);
  memcpy(&reply[position], &count, 4);
  position += 4;
  count = swap_int(count);

  /*
  ** Go over all the fields and add them to the reply.
  */
  
  for(i = 0; i < count; i++) {
    field = &clazz->fields[i];

    /*
    ** Store the fieldID.
    */

    length = (w_int)field;
    memcpy(&reply[position], &length, 4);
    position += 4;

    /*
    ** Store the field name.
    */

    string = jdwp_string2UTF8(field->name, &length);
    memcpy(&reply[position], string, (w_word)(length + 4));
    position += length + 4;
    x_mem_free(string);

    /*
    ** Store the field signature.
    */
    
    string = jdwp_string2UTF8(field->desc->name, &length);
    memcpy(&reply[position], string, (w_word)(length + 4));
    position += length + 4;
    x_mem_free(string);

    /*
    ** Store the field flags.
    */
   
    flags = swap_int(field->flags);
    memcpy(&reply[position], &flags, 4);
    position += 4;
  }
 
  /*
  ** Send the reply to the debugger.
  */
  
  jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);
  
  /*
  ** Cleanup.
  */
  
  x_mem_free(reply);
  
  return;
}


/*
** Returns all (but not the inhereted) methods of a given referenceID. 
** Also constructors (<init>) and initializers (<clinit>) are included.
** Methods should be returned in the same order they occur in the classfile.
** [CG 20020321] ... but they won't be, can't be, sorry.
*/

w_void jdwp_ref_methods(jdwp_command_packet cmd) {
  w_clazz    clazz;
  w_method   method;
  w_int      i;
  w_ubyte    *reply;
  w_ubyte    *string;
  w_int      position = 0;
  w_int      oldpos = 0;
  w_int      oldlen = 0;
  w_int      count;
  w_int      length;
  w_flags    flags;
  w_instance exception = NULL;
  
  /*
  ** Get the class and the number of methods.
  */
  
  memcpy(&clazz, &cmd->data[0], 4);
  count = clazz->numMethods;

  /*
  ** Allocate enough memory.
  ** It's not that easy to determine the exact amount required so we take what 
  ** should be enough... (Reminds me of the '640K should be enough for everyone'
  ** quote, not a good sign).
  */
  
  reply = x_mem_calloc((w_word)(4 + (count * (4 + 100 + 50 + 4))));

  /*
  ** Store the number of methods to the reply.
  */
  
  count = swap_int(count);
  memcpy(&reply[position], &count, 4);
  position += 4;
  count = swap_int(count);

  /*
  ** Go over each of the methods and add them to the reply.
  */
  
  for(i = 0; i < count; i++) {
    method = clazz->methods[i];

    /*
    ** Store the methodID.
    */

    length = (w_int)method;
    memcpy(&reply[position], &length, 4);
    position += 4;

    /*
    ** Store the method name.
    */

    string = jdwp_string2UTF8(method->name, &length);
    memcpy(&reply[position], string, (w_word)(length + 4));
    position += length + 4;
    x_mem_free(string);

    /*
    ** Store the method signature.
    */
   
    oldpos = position;
    string = jdwp_string2UTF8(method->param_list, &length);
    memcpy(&reply[position], string, (w_word)(length + 4));
    position += length + 4;
    x_mem_free(string);
    oldlen = length;

    method->return_type = mustBeLoaded(method->return_type, &exception);
    string = jdwp_string2UTF8(method->return_type->desc->name, &length);
    memcpy(&reply[position], ((w_ubyte*)string + 4), (w_word)(length));
    position += length;
    x_mem_free(string);
    oldlen += length;
    oldlen = swap_int(oldlen);
    memcpy(&reply[oldpos], &oldlen, 4);

    /*
    ** Store the method flags.
    */
   
    flags = swap_int(method->flags);
    memcpy(&reply[position], &flags, 4);
    position += 4;
  }
 
  /*
  ** Send the reply to the debugger.
  */
  
  jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);
  
  /*
  ** Cleanup.
  */
  
  x_mem_free(reply);
  
  return;
}


/*
** TODO: fill in this space...
*/

w_void jdwp_ref_get_values(jdwp_command_packet cmd) {
  w_clazz      clazz;
  w_clazz      fieldclazz;
  w_field      field;
  w_int        length;
  w_int        pos1 = 0;
  w_int        pos2 = 0;
  w_int        i;
  w_ubyte      *reply;
  w_ubyte      abyte;
  w_int        anint;
  w_short      ashort;
  w_long       along;

  /*
  ** Get the class out of the command packet.
  */
  
  memcpy(&clazz, &cmd->data[pos1], 4);
  pos1 += 4;

  /*
  ** Get the length out of the command packet.
  ** This is the number of fields of which the value is requested.
  */ 

  memcpy(&length, &cmd->data[pos1], 4);
  pos1 += 4;
  length = swap_int(length);

  /*
  ** Allocate enough memory for the reply.
  */

  reply = x_mem_alloc((w_word)(4 + length * 10));

  /*
  ** Store the number of fields in the reply. Since we return a value for each 
  ** requested field, this is the same number as the number of fields requested.
  */
  
  length = swap_int(length);
  memcpy(&reply[pos2], &length, 4);
  pos2 += 4;
  length = swap_int(length);

  woempa(9, "Get static field values (%d) for reftype %p\n", length, clazz);

  /*
  ** Go over the fields and add their value to the reply.
  */

  for(i = 0; i < length; i++) {

    /*
    ** Get the fieldID from the command packet.
    */
    
    memcpy(&field, &cmd->data[pos1], 4);
    pos1 += 4;

    woempa(9, "  -> field %w (%p) = %d (%w)\n", field->name, field, field->clazz->staticFields[field->slot], field->desc->name);

    /*
    ** Search for the class this field holds.
    */

    fieldclazz = desc2Class(clazz->loader, field->desc);

    /*
    ** Check what type of class this is and add the value to the reply. 
    ** Note that different types don't have the same length. A byte is only 1 byte while a
    ** long takes 8 bytes. Everything instance-alike takes 4 bytes, ...
    */
    
    if(isAssignmentCompatible(fieldclazz, clazzThread)) {            /* a Thread */
      woempa(9, "       Thread field\n");
      reply[pos2++] = jdwp_tag_thread;
      anint = (field->clazz->staticFields[field->slot]);
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(fieldclazz, clazzThreadGroup)) {  /* a ThreadGroup */
      woempa(9, "       Threadgroup field\n");
      reply[pos2++] = jdwp_tag_thread_group;
      anint = (field->clazz->staticFields[field->slot]);
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(fieldclazz, clazzClassLoader)) {  /* a ClassLoader */
      woempa(9, "       Classloader field\n");
      reply[pos2++] = jdwp_tag_class_loader;
      anint = (field->clazz->staticFields[field->slot]);
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(isAssignmentCompatible(fieldclazz, clazzString)) {       /* a String */
      woempa(9, "       String field\n");
      reply[pos2++] = jdwp_tag_string;
      anint = (field->clazz->staticFields[field->slot]);
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(field->desc->tag == D_Primitive) {                       /* a primitive type */
      woempa(9, "primitive:  %w\n", field->desc->name);
      switch(field->desc->body.primitive.type) {
        case P_boolean:   reply[pos2++] = jdwp_tag_boolean;          /* boolean */
                          abyte = (w_ubyte)field->clazz->staticFields[field->slot];
                          memcpy(&reply[pos2++], &abyte, 1);
                          break;
        case P_char:      reply[pos2++] = jdwp_tag_char;             /* char */
                          ashort = swap_short(field->clazz->staticFields[field->slot]);
                          memcpy(&reply[pos2], &ashort, 2);
                          pos2 += 2;
                          break;
        case P_float:     reply[pos2++] = jdwp_tag_float;            /* float */
                          anint = swap_int(field->clazz->staticFields[field->slot]);
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
        case P_double:    reply[pos2++] = jdwp_tag_double;           /* double */
                          along = ((w_long)field->clazz->staticFields[field->slot + 1] << 32) + (w_long)field->clazz->staticFields[field->slot];
                          along = swap_long(along);
                          memcpy(&reply[pos2], &along, 8);
                          pos2 += 8;
                          break;
        case P_byte:      reply[pos2++] = jdwp_tag_byte;             /* byte */
                          abyte = (w_ubyte)field->clazz->staticFields[field->slot];
                          memcpy(&reply[pos2++], &abyte, 1);
                          break;
        case P_short:     reply[pos2++] = jdwp_tag_short;            /* short */
                          ashort = swap_short(field->clazz->staticFields[field->slot]);
                          memcpy(&reply[pos2], &ashort, 2);
                          pos2 += 2;
                          break;
        case P_int:       reply[pos2++] = jdwp_tag_int;              /* int */
                          anint = swap_int(field->clazz->staticFields[field->slot]);
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
        case P_long:      reply[pos2++] = jdwp_tag_long;             /* long */
                          along = ((w_long)field->clazz->staticFields[field->slot + 1] << 32) + (w_long)field->clazz->staticFields[field->slot];
                          along = swap_long(along);
                          memcpy(&reply[pos2], &along, 8);
                          pos2 += 8;
                          break;
        case P_void:      reply[pos2++] = jdwp_tag_void;             /* void */
                          break;
        case P_Object:    reply[pos2++] = jdwp_tag_object;           /* Object */
                          anint = field->clazz->staticFields[field->slot];
                          memcpy(&reply[pos2], &anint, 4);
                          pos2 += 4;
                          break;
      }
    }
    else if(field->desc->tag == D_Class) {                           /* a Class of some sort */
      woempa(9, "It's a class...\n");
      reply[pos2++] = jdwp_tag_object;
      anint = field->clazz->staticFields[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
    else if(field->desc->tag == D_Array) {                           /* an array */
      woempa(9, "Array\n");
      reply[pos2++] = jdwp_tag_array;
      anint = field->clazz->staticFields[field->slot];
      memcpy(&reply[pos2], &anint, 4);
      pos2 += 4;
    }
  }

  /*
  ** Send the reply back to the debugger.
  */
  
  jdwp_send_reply(cmd->id, reply, pos2, jdwp_err_none);

  /*
  ** Clean up.
  */

  x_mem_free(reply);

  return;
  
}


/*
** Returns the filename in which a given referenceID is defined.
** The path to the filename is not included.
*/

w_void jdwp_ref_source(jdwp_command_packet cmd) {
  w_clazz  clazz;
  w_ubyte  *file;
  w_ubyte  *unknown = "<unknown>";
  w_int    length;

  /*
  ** Get the classID from the command packet.
  */
 
  memcpy(&clazz, &cmd->data[0], 4);

  /*
  ** Check if the class has a source file set.
  */

  if(clazz->source) {

    /*
    ** The source file is known, convert it to UTF8 and send it to the debugger.
    */

    file = jdwp_string2UTF8(clazz->source, &length);
    jdwp_send_reply(cmd->id, file, length + 4, jdwp_err_none);
    
  } else {

    /*
    ** The source file is unknown, take the default response ("<unknown>") and 
    ** send that one to the debugger.
    */

    file = jdwp_cstring2UTF8(unknown, &length);
    jdwp_send_reply(cmd->id, file, length + 4, jdwp_err_none);
    
  }

  /*
  ** Clean up.
  */
  
  x_mem_free(file);
  
  return;
}


/*
** Get all the interfaces that the given class implements.
*/

w_void jdwp_ref_interfaces(jdwp_command_packet cmd) {
  w_clazz  clazz;
  w_clazz  interface;
  w_int    length;
  w_int    i;
  w_int    ID;
  w_ubyte  *reply;
  w_int    position = 0;

  /*
  ** Get the classID from the command packet.
  */ 
 
  memcpy(&clazz, &cmd->data[0], 4);

  /*
  ** Get the number of interfaces.
  */
   
  length = clazz->numInterfaces;

  /*
  ** Allocate memory for the reply packet.
  */

  reply = x_mem_alloc((w_word)(4 + (length * 4)));

  /*
  ** Store the number of interfaces in the reply.
  */

  length = swap_int(length);
  memcpy(&reply[position], &length, 4);
  position += 4;
  length = swap_int(length);

  /* 
  ** Go over all the interfaces and add them to the reply.
  */
  
  for(i = 0; i < length; i++) {
    interface = clazz->interfaces[i];
    ID = (w_int)interface;
    memcpy(&reply[position], &ID, 4);
    position += 4;
  }

  /*
  ** Send the reply to the debugger.
  */
  
  jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);

  /*
  ** Clean up.
  */

  x_mem_free(reply);

  return;
}


/*
** The dispatcher for the 'Reference Type' command set.
*/

w_void dispatch_reftype(jdwp_command_packet cmd) {

  switch((jdwp_reftype_cmd)cmd->command) {
    case jdwp_reftype_signature:     jdwp_ref_signature(cmd);   break;
    case jdwp_reftype_classLoader:   jdwp_ref_classloader(cmd); break; 
    case jdwp_reftype_fields:        jdwp_ref_fields(cmd);      break;
    case jdwp_reftype_methods:       jdwp_ref_methods(cmd);     break;
    case jdwp_reftype_getValues:     jdwp_ref_get_values(cmd);  break;
    case jdwp_reftype_sourceFile:    jdwp_ref_source(cmd);      break;
    case jdwp_reftype_interfaces:    jdwp_ref_interfaces(cmd);  break;
    case jdwp_reftype_modifiers:
    case jdwp_reftype_nestedTypes:
    case jdwp_reftype_status:
    case jdwp_reftype_classObject:
    case jdwp_reftype_sourceDebugExtension:
    default:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
  }

}

