/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"  /* x_thread_... */
#include "wonka.h"   /* the w_ types */
#include "ts-mem.h"  /* allocMem_system */
#include "realms.h"  /* allocMem_system */
#include "jdwp.h"
#include <string.h>  /* strncmp */
#include "core-classes.h" /* F_... stuff */
#include "stack.h"   /* w_frame */
#include "descriptor.h" 
#include "methods.h" 
#include "checks.h" 
#include "clazz.h" 

/*
** Get the values of local variables in a given stackframe. 
*/

w_void jdwp_stack_get_values(jdwp_command_packet cmd) {
  w_instance  instance;
  w_frame     frame;
  w_int       slots;
  w_int       slot;
  w_int       i;
  w_ubyte     sig;
  w_ubyte     *reply;
  w_byte      abyte;
  w_short     ashort;
  w_int       anint;
  w_long      along;
  w_int       pos1 = 0;
  w_int       pos2 = 0;

  /*
  ** Get the instace from the command packet.
  */

  memcpy(&instance, &cmd->data[pos1], 4);
  pos1 += 4;

  /*
  ** Get the frame from the command packet.
  */
  
  memcpy(&frame, &cmd->data[pos1], 4);
  pos1 += 4;

  /*
  ** Get the number of slots to be retrieved from the command packet.
  */
  
  memcpy(&slots, &cmd->data[pos1], 4);
  pos1 += 4;
  slots = swap_int(slots);

  /*
  ** Allocate memory for the reply.
  */

  reply = x_mem_alloc((w_word)(4 + slots * 10));

  /*
  ** Store the number of slots in the reply. 
  */
  
  slots = swap_int(slots);
  memcpy(&reply[pos2], &slots, 4);
  pos2 += 4;
  slots = swap_int(slots);

  /*
  ** Go over all the requested slots and add their value to the reply packet.
  */
  
  for(i = 0; i < slots; i++) {

    /*
    ** Get the slot number from the command packet.
    */

    memcpy(&slot, &cmd->data[pos1], 4);
    pos1 += 4;
    slot = swap_int(slot);

    /*
    ** Get the type tag from the command packet.
    */

    memcpy(&sig, &cmd->data[pos1], 1);
    pos1 += 1;

    /*
    ** Store the retrieved type tag in the reply packet.
    */

    memcpy(&reply[pos2], &sig, 1);
    pos2 += 1;
    
    /* 
    ** Check the type tag, because different types take a different
    ** amount of space in the reply.
    ** Also add the value to the reply.
    */

    switch(sig) {
      case 'V': break;  /* noppes */
      case 'B':         /* a byte */
      case 'Z': abyte = (w_byte)frame->locals[slot];
                memcpy(&reply[pos2], &abyte, 1);
                pos2 += 1;
                break;  
      case 'C':         /* a short */
      case 'S': ashort = swap_short(( (w_short)(frame->locals[slot]) ));
                memcpy(&reply[pos2], &ashort, 2);
                pos2 += 2;
                break;   
      case 'J':         /* a long */
      case 'D': along = ((w_long)frame->locals[slot + 1] << 32) + (w_long)frame->locals[slot];
                along = swap_long(along);
                memcpy(&reply[pos2], &along, 8);
                pos2 += 8;
                break;
      case 'F':         /* a word */
      case 'I': anint = swap_int(frame->locals[slot]);
                memcpy(&reply[pos2], &anint, 4);
                pos2 += 4;
                break;
      default:  anint = frame->locals[slot];
                memcpy(&reply[pos2], &anint, 4);
                pos2 += 4;
    }
  }

  /*
  ** Send the reply to the debugger.
  */

  jdwp_send_reply(cmd->id, reply, pos2, jdwp_err_none);

  /*
  ** Clean up.
  */
  
  x_mem_free(reply);
  
  return;
}


/*
** Get the 'this' reference for a given stackframe. If the frame's method is native or static, 
** the reply will contain the null reference.
*/

w_void jdwp_stack_this(jdwp_command_packet cmd) {
  w_thread   thread;
  w_frame    frame;
  w_ubyte    *reply;
  w_int      instanceID;
  w_instance instance;
  w_clazz    clazz;
  
  /*
  ** Get the address of the thread out of the packet.
  ** Also get the frame ID.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  memcpy(&frame, &cmd->data[4], 4);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Oki, it's a thread.
    */

    reply = x_mem_calloc(5);

    /*
    ** Check if the frame's method is native or static.
    */
    
    if (!isSet(frame->flags, FRAME_NATIVE) && !isSet(frame->method->flags, ACC_STATIC)) {

      /*
      ** It's a plain old method. Get the 'this' instance from the top of the 
      ** frame and add it to the reply. Then get the class type of the instance.
      */
     
      instanceID = frame->locals[0];
      memcpy(&reply[1], &instanceID, 4);
      instance = (w_instance)instanceID;
      clazz = instance2clazz(instance);

      /*
      ** Check the type of the class and add a type tag to the reply.
      */

      if(isAssignmentCompatible(clazz, clazzThread)) {
        reply[0] = jdwp_tag_thread;
      }
      else if(isAssignmentCompatible(clazz, clazzThreadGroup)) {
        reply[0] = jdwp_tag_thread_group;
      }
      else if(isAssignmentCompatible(clazz, clazzClassLoader)) {
        reply[0] = jdwp_tag_class_loader;
      }
      else if(isAssignmentCompatible(clazz, clazzString)) {
        reply[0] = jdwp_tag_string;
      }
      else if(clazz->desc->tag == D_Primitive) {
        switch(clazz->desc->body.primitive.type) {
          case P_boolean:   reply[0] = jdwp_tag_boolean;   break;
          case P_char:      reply[0] = jdwp_tag_char;      break;
          case P_float:     reply[0] = jdwp_tag_float;     break;
          case P_double:    reply[0] = jdwp_tag_double;    break;
          case P_byte:      reply[0] = jdwp_tag_byte;      break;
          case P_short:     reply[0] = jdwp_tag_short;     break;
          case P_int:       reply[0] = jdwp_tag_int;       break;
          case P_long:      reply[0] = jdwp_tag_long;      break;
          case P_void:      reply[0] = jdwp_tag_void;      break;
          case P_Object:    reply[0] = jdwp_tag_object;    break;
        }
      }
      else if(clazz->desc->tag == D_Class) {
        reply[0] = jdwp_tag_object;
      }
      else if(clazz->desc->tag == D_Array) {
        reply[0] = jdwp_tag_array;
      }

    } else { 

      /*
      ** The method is native or static.
      */

    }

    /*
    ** Send the reply to the debugger.
    */

    jdwp_send_reply(cmd->id, reply, 5, jdwp_err_none);

    /*
    ** Clean up.
    */

    x_mem_free(reply);
     
  } else {

    /*
    ** This is not a valid thread, complain about it.
    */
  
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_thread);

  }

  return;
}

/*
** The dispatcher for the 'Stack frame' command set.
*/

w_void dispatch_stack(jdwp_command_packet cmd) {

  switch((jdwp_stack_cmd)cmd->command) {
    case jdwp_stack_getValues:   jdwp_stack_get_values(cmd);  break;
    case jdwp_stack_thisObject:  jdwp_stack_this(cmd);        break;
    case jdwp_stack_setValues:
    case jdwp_stack_popFrames:
    default:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
  }

}

