/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"  /* x_thread_... */
#include "wonka.h"   /* the w_ types */
#include "ts-mem.h"  /* allocMem_system */
#include "realms.h"  /* allocMem_system */
#include "stack.h"   /* w_Frame */
#include "jdwp.h"
#include "wstrings.h" /* string2UTF8 */
#include <string.h>  /* strncmp */
#include "core-classes.h" /* F_... stuff */
#include "methods.h"

/*
** Returns the name of the thread with given ID.
*/

w_void jdwp_thr_name(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_ubyte    *name;
  w_int      length;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if(thread == NULL || thread == (w_thread)0xaaaaaaaa) {
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);
    return;
  }
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread. 
    ** Now all that's left is converting the name from a wonka string in a 
    ** UTF8 string, pack it and send it back to the debugger.
    */
    
    name = jdwp_string2UTF8(thread->name, &length);
    
    jdwp_send_reply(cmd->id, name, length + 4, jdwp_err_none);

    x_mem_free(name);
    
  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Suspend a thread. Suspending and resuming threads are counted. This means that when 
** a thread is suspended x times, it should be resumed x times before it will actually 
** run again.
**
** TODO: A thread suspended with this function should still appear to be running to the
**       other threads in the VM. Also Thread.resume should have no effect on threads 
**       suspended this way.
*/

w_void jdwp_thr_suspend(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread.
    */
    
    x_thread_suspend(thread->kthread);
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Resume a suspended thread.
*/

w_void jdwp_thr_resume(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread.
    */
    
    x_thread_resume(thread->kthread);
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_none);

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Returns the status of the thread with given ID.
*/

w_void jdwp_thr_status(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_int      status;
  w_int      suspend;
  w_ubyte    reply[8];
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread. 
    */
    
    switch(thread->state) {
      case 1:   /* Wonka says the thread is ready, now check with Oswald. */
                switch(thread->kthread->state) {
                  case xt_ready:        status = swap_int(jdwp_ts_running);
                                        suspend = swap_int(0);
                                        break;
                  case xt_mutex:
                  case xt_rescheduled:
                  case xt_queue:
                  case xt_mailbox:
                  case xt_semaphore:
                  case xt_signals:
                  case xt_monitor:
                  case xt_block:
                  case xt_map:
                  case xt_joining:      status = swap_int(jdwp_ts_monitor);
                                        suspend = swap_int(0);
                                        break;
                  case xt_waiting:      status = swap_int(jdwp_ts_wait);
                                        suspend = swap_int(0);
                                        break;
                  case xt_suspended:    status = swap_int(4);
                                        suspend = swap_int(1);
                                        break;
                  case xt_sleeping:     status = swap_int(jdwp_ts_sleeping);
                                        suspend = swap_int(0);
                                        break;
                  case xt_ended:
                  case xt_unknown:      status = swap_int(jdwp_ts_zombie);
                                        suspend = swap_int(0);
                                        break;
                }
                break;
      case 3:   status = swap_int(jdwp_ts_sleeping);  /* 3: Sleeping       */
                suspend = swap_int(0);
                break;
      case 5:   status = swap_int(4);                 /* 5: Suspended      */
                suspend = swap_int(1);
                break;
      case 6:   status = swap_int(jdwp_ts_monitor);   /* 6: Locking        */
                suspend = swap_int(0);
                break;
      case 7:   status = swap_int(jdwp_ts_wait);      /* 7: Waiting        */
                suspend = swap_int(0);
                break;
      case 0:                                         /* 0: Unstarted      */
      case 2:                                         /* 2: Dying          */
      case 4:                                         /* 4: Completed      */
      case 8:                                         /* 8: Stopped early  */
      case 9:                                         /* 9: It's dead, Jim */
      default:  status = swap_int(jdwp_ts_zombie);  
                suspend = swap_int(0);
                break;
    }

    memcpy(&reply[0], &status, 4);
    memcpy(&reply[4], &suspend, 4);
    
    jdwp_send_reply(cmd->id, reply, 8, jdwp_err_none);

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Returns the threadgroup the thread with given ID belongs to.
*/

w_void jdwp_thr_group(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_group    group;
  w_int      result;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread. 
    ** Get the instance of the parent threadgroup and send it to the debugger.
    */
    
    group = thread->group;
    result = (w_int)(group->ThreadGroup);
    
    jdwp_send_reply(cmd->id, &result, 4, jdwp_err_none);

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
**  Returns the call stack of a thread. The thread has to be suspended, and the frameID is only 
**  valid as long as the thread is suspended. The first frame is the current one, the next is 
**  its caller, etc...
*/

w_void jdwp_thr_frames(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_int      start;
  w_int      length;
  w_frame    cursor;
  w_frame    top;
  w_int      count = 0;
  w_int      position = 0;
  w_ubyte    *reply;
  w_ubyte    location[17];  /* tag - classID - methodID - index(pc) */
  w_int      frame;
  w_clazz    clazz;
  w_method   method;
  w_long     pc;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is a thread. 
    */

    memcpy(&start, &cmd->data[4], 4);
    memcpy(&length, &cmd->data[8], 4);

    start = swap_int(start);
    length = swap_int(length);
    
    if(thread->kthread->state == xt_suspended) {  /* Should be '== 5' for suspended. */

      /*
      ** Jolly, it's is a suspended thread. Time for some action.
      ** Skip the first 'start' frames.
      */
     
      cursor = thread->top;
      
      while(cursor && count < start) {
        count++;
        cursor = cursor->previous;
      }
      
      top = cursor;

      /* 
      ** Count the remaining frames.
      */
      
      count = 0;
 
      while(cursor && (length == -1 ? 1 : (count < length))) {
        count++;
        cursor = cursor->previous;
      }

      reply = x_mem_alloc((w_word)(4 + (count * (4 + 17))));
      count = swap_int(count);
      memcpy(&reply[position], &count, 4);
      position += 4;
      
      cursor = top;
      count = 0;

      while(cursor && (length == -1 ? 1 : (count < length))) {
        
        /*
        ** Add this frame to the reply.
        */

        frame = (w_int)cursor;
        
        memcpy(&reply[position], &frame, 4);
        position += 4;

        /*
        ** Get the method
        */
    
        if (isSet(cursor->flags, FRAME_NATIVE)) {
          method = cursor->native;
        } else {
          method = cursor->method;
        }
        
        /*
        ** Get the class.
        */

        clazz = method->clazz;

        /*
        ** Put everything in the location.
        */

        pc = cursor->pc;
        pc = swap_long(pc);

        memset(&location, 0, 17);
        
        location[0] = jdwp_tt_class;     /* TODO: Not everything is a class.... */
        
        memcpy(&location[1], &clazz, 4);
        memcpy(&location[5], &method, 4);
        memcpy(&location[9], &pc, 8);

        /*
        ** Store the location in the reply.
        */ 

        memcpy(&reply[position], &location, 17);
        position += 17;

        /*
        ** On to the next frame.
        */
        
        count++;
        cursor = cursor->previous;
      }

      jdwp_send_reply(cmd->id, reply, position, jdwp_err_none);

      x_mem_free(reply);
      
    } else {

      /* 
      ** This thread is not suspended, therefore this command is not allowed.
      */

      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_thread);
      
    }

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Count the number of frames on this thread's stack, but only if it's suspended.
** The returned number of frames is valid as long as the thread is suspended.
*/

w_void jdwp_thr_countframes(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_frame    cursor;
  w_int      count = 0;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread. 
    */

    if(thread->kthread->state == xt_suspended) {

      /*
      ** Jolly, this is a suspended thread (state == 5). Time for some action.
      ** Begin at the top frame.
      */

      cursor = thread->top;

      /* 
      ** Go through all the previous frames and increment a counter. 
      */
      
      while(cursor) {
        count++;
        cursor = cursor->previous;
      }
    
      /*
      ** Swap the result (if needed) and send it to the debugger.
      */

      count = swap_int(count);

      jdwp_send_reply(cmd->id, &count, 4, jdwp_err_none);
      
    } else {

      /* 
      ** This thread is not suspended, therefore this command is not allowed.
      */

      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_thread);
      
    }

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** Return the number of times a given thread has been suspended. In other words:
** The number of times it should be resumed before it starts running again.
*/

w_void jdwp_thr_suspend_count(jdwp_command_packet cmd) {
  w_thread   thread;
  w_instance instance;
  w_int      count;
  
  /*
  ** Get the address of the thread out of the packet.
  */
  
  memcpy(&instance, &cmd->data[0], 4);
  thread = getWotsitField(instance, F_Thread_wotsit);
  
  /* 
  ** sanity check : See if this is indeed a thread.
  */
  
  if (strncmp(thread->label, "thread", 6) == 0) {

    /*
    ** Everything checks out. This is indeed a thread.
    */

    count = (thread->kthread->flags & TF_COUNTER_MASK) >> TF_COUNTER_SHIFT;
    count = swap_int(count);

    jdwp_send_reply(cmd->id, &count, 4, jdwp_err_none);

  } else {

    /*
    ** This is not (or no longer) a thread... Let the debugger know about 
    ** this little fact.
    */
    
    jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_invalid_object);

  }
  
  return;
}


/*
** The dispatcher for the 'Thread reference' command set.
*/

w_void dispatch_threadref(jdwp_command_packet cmd) {

  switch((jdwp_threadref_cmd)cmd->command) {
    case jdwp_threadref_name:                     jdwp_thr_name(cmd);           break;
    case jdwp_threadref_suspend:                  jdwp_thr_suspend(cmd);        break;
    case jdwp_threadref_resume:                   jdwp_thr_resume(cmd);         break;
    case jdwp_threadref_status:                   jdwp_thr_status(cmd);         break;
    case jdwp_threadref_threadGroup:              jdwp_thr_group(cmd);          break;
    case jdwp_threadref_frames:                   jdwp_thr_frames(cmd);         break;
    case jdwp_threadref_frameCount:               jdwp_thr_countframes(cmd);    break;
    case jdwp_threadref_suspendCount:             jdwp_thr_suspend_count(cmd);  break; 
    case jdwp_threadref_ownedMonitors:
    case jdwp_threadref_currentContendedMonitor:
    case jdwp_threadref_stop:
    case jdwp_threadref_interrupt:
    case jdwp_threadref_popTopFrame:
    default:
      jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
  }
  
}

