/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#include "oswald.h"  /* x_thread_... */
#include "wonka.h"   /* the w_ types */
#include "ts-mem.h"  /* allocMem_system */
#include "realms.h"  /* allocMem_system */
#include "jdwp.h"
#include "wstrings.h" /* string2UTF8 */

#include "network.h"
#include <sys/socket.h>

#include <stdio.h>   /* printf, ... */

#define JDWP_THREAD_STACK_SIZE 65536
#define JDWP_THREAD_PRIORITY 10

x_thread jdwp_thread;

/*
** Command line arguments from argument.c
*/

extern char  *jdwp_args;
extern int   jdwp_enabled;

/*
** Internal configuration parameters.
*/

static char   *jdwp_address_host  = "127.0.0.1";
static char   *jdwp_address_port  = "5555";
static w_int  jdwp_config_server  = 0; 
static w_int  jdwp_config_suspend = 0; 

w_int jdwp_connected = 0;

/*
** The sockets which will be used for transport.
*/

static w_int sock;
static w_int serversock;


/*
** The mutex used for locking.
*/

x_mutex  jdwp_mutex;

/*
** When jdwp_events_enabled equals 0, sending events will be supressed.
*/

w_int    jdwp_events_enabled = 0;


/*
** Show a little explanation about the JDWP parameters.
*/

void print_jdwp_help(void) {
  
  printf("  Usage of -Xjdwp:   -Xjdwp:<option>=<value>, ...\n");
  printf("\n");
  printf("Options :\n");
  printf("  suspend=y|n         Suspend the vm after startup (default: n) (not yet implemeted)\n");
  printf("  transport=<name>    Transport type. dt_socket is the only supported transport.\n");
  printf("                      (default: dt_socket)\n");
  printf("  address=<address>   Address to connect/listen to. (default: 127.0.0.1:5555)\n");
  printf("  server=y|n          Listen for the debugger or connect to the debugger. \n");
  printf("                      (default: n -> connect to the debugger)\n");
  printf("\n");
  exit(1);

}

/*
** Connect to the debugger. Only TCP/IP socket connections are possible for now. In the future, 
** this will most likely be extended to connections over serial ports and others.
** PS: These functions should eventually move to a seperate file/layer, which makes it easier
**     to implement different transports for JDWP.
*/

w_void jdwp_connect() {
  
  w_int              res = 1;
  struct sockaddr_in sa;
  w_ubyte            *buffer;
  w_int              count = 0;
  struct hostent     *host;
  w_int              port_nr;
  w_int              timeout = 0;

  /*
  ** Get the host address and port.
  */

  woempa(9, "JDWP Host : %s\n", jdwp_address_host);
  woempa(9, "JDWP Port : %s\n", jdwp_address_port);
  
  host = w_gethostbyname(jdwp_address_host);
  sscanf(jdwp_address_port, "%d", &port_nr);

  /*
  ** Get a socket and fill in the address structure.
  */

  sock = w_socket(PF_INET, SOCK_STREAM, 0);
  memset(&sa,0,sizeof(sa)); 
  sa.sin_addr.s_addr = *((unsigned long*)host->h_addr_list[0]);
  sa.sin_family = host->h_addrtype;
  sa.sin_port = swap_short(port_nr);
 
  if(jdwp_config_server) {

    /*
    ** We should behave as a server and wait for jdwp connections.
    */
   
    w_int socksize = sizeof(struct sockaddr_in);

    if(serversock == 0) {
    
      serversock = sock;
      sa.sin_addr.s_addr = INADDR_ANY;
      res = w_bind(serversock, (struct sockaddr*)&sa, sizeof(struct sockaddr_in));
      res = w_listen(serversock, 1);  
    }

    sock = w_accept(serversock, (struct sockaddr *)&sa, &socksize, 0);
    printf("sock: %d\n", sock);
    if(sock < 0) res = -1; else res = 0;
    fcntl(sock, F_SETFL, O_NONBLOCK);
  } else {

    /*
    ** We should behave as a client and connect to the debugger at startup.
    */
    
    res = w_connect(sock, (struct sockaddr*)&sa, sizeof(struct sockaddr_in));
    
  }

  if(res != 0) {
    woempa(9, "-------------------------------------------------------------------------------------------------\n");
    woempa(9, "         J D W P :     C O U L D    N O T    C O N N E C T    TO    T H E    D E B U G G E R\n");
    woempa(9, "-------------------------------------------------------------------------------------------------\n");
    x_thread_sleep(x_eternal);
  }

  /*
  ** Wait for the handshake. Keep receiving bytes until we have received the full 14 bytes.
  */

  buffer = x_mem_calloc(15);
  
  while(count < 14) {
    count += w_recv(sock, (buffer + count), (w_word)(14 - count), 0, (w_int *)&timeout);
  }
  
  if(strcmp(buffer, "JDWP-Handshake") != 0) {
    wabort(ABORT_WONKA, "Incredible bad handshake: %s\n", buffer);
  }
  
  x_mem_free(buffer);

  /*
  ** Respond to the handshake.
  */

  w_send(sock, "JDWP-Handshake", 14, 0);

  woempa(9, "JDWP connection established.\n");

  jdwp_connected = 1;

  /*
  ** From now on, it's safe to send events.
  */ 

  jdwp_events_enabled = 1;
  
//  x_mutex_unlock(jdwp_mutex);

  return;
}


/*
** Disconnect the debugger.
*/

w_void jdwp_disconnect() {
  if(sock) w_socketclose(sock);
  return;
}


/*
** Receive a packet from the debugger. This function will block the thread that called it if
** there aren't any packets available.
*/

w_void *jdwp_recv_packet() {
  
  jdwp_command_packet cmd;
  w_int peek;
  w_int count = 0;
  w_int timeout = 0;
  
  /*
  ** First peek to get the length of this packet
  */
    
  w_recv(sock, &peek, sizeof(w_int), MSG_PEEK, (w_int *)&timeout);
  peek = swap_int(peek);

  /*
  ** We now know the length of the entire command so we can allocate a
  ** buffer and retrieve the whole packet.
  */
    
  cmd = allocMem_system((w_word)peek);
  while(count < peek) {
    count += w_recv(sock, ((w_ubyte *)cmd + count), (w_word)(peek - count), 0, (w_int *)&timeout);
  }
  
  return cmd;
}


/*
** Send a packet to the debugger.
*/

w_void jdwp_send_packet(w_void *packet) {
  w_send(sock, packet, swap_int(((jdwp_reply_packet)packet)->length), 0);
  return;
}


/*
** Send a reply to the debugger. This makes it easier to send replies because it will take
** care of setting the id, calculating the length and actually sending the packet.
*/

w_void jdwp_send_reply(w_int id, w_void* data, w_int length, w_int error) {
  jdwp_reply_packet reply;

  /*
  ** Allocate memory for the packet. 
  */

  reply = x_mem_alloc((w_word)(offsetof(jdwp_Reply_Packet, data) + length));

  /*
  ** Fill in the header of the packet with the received parameters.
  */

  reply->length = swap_int((offsetof(jdwp_Reply_Packet, data) + length));
  reply->id = id;
  reply->flags = 0x80;
  reply->err1 = (error >> 8) & 0x00FF;
  reply->err2 = error & 0x00FF;

  /*
  ** If we send back a not_implemented error, make some noise !
  */

  if(error == jdwp_err_not_implemented) {
    woempa(9, "-------------------------------------------------------------------------------------------------\n");
    woempa(9, "                    J D W P :     N O T    Y E T    I M P L E M E N T E D\n");
    woempa(9, "-------------------------------------------------------------------------------------------------\n");
  }

  /*
  ** Check if there actually is data to be sent with this packet. If there is, add it to the packet.
  */

  if(length) memcpy(reply->data, data, (w_word)length);

  /*
  ** Send the packet to the debugger.
  */

  jdwp_send_packet(reply);

  /*
  ** Clean up.
  */

  x_mem_free(reply);
}


static w_int current_cmdID = 0;


/*
** Send a command to the debugger. 
*/

w_void jdwp_send_command(w_void* data, w_int length, w_int cmd_set, w_int cmd) {
  jdwp_command_packet command;

  /*
  ** Allocate memory for the packet.
  */

  command = x_mem_alloc((w_word)(offsetof(jdwp_Command_Packet, data) + length));

  /*
  ** Fill in the header of the packet.
  */

  command->length = swap_int((offsetof(jdwp_Reply_Packet, data) + length));
  command->id = current_cmdID++;
  command->flags = 0;
  command->command_set = cmd_set;
  command->command = cmd;

  /*
  ** If there's data to be added to the packet, do so.
  */
  
  if(length) memcpy(command->data, data, (w_word)length);

  /*
  ** Send the packet to the debugger.
  */

  jdwp_send_packet(command);

  /*
  ** Clean up.
  */

  x_mem_free(command);
}


/*
** Parse jdwp arguments.
*/

w_void jdwp_parse_arguments(char *args) {
  char *argument;
  char *key;
  char *value;

  /*
  ** Go over the arguments one by one. They are seperated with comma's.
  */

  while((argument = strsep((char **)&args, ",")) != NULL) {

    /*
    ** Every parameter has a key and value, they are seperated with ='s.
    */
    
    key = argument;
    value = index(argument, '=');
    if(value != NULL) {
      value[0] = '\0';
      value++;
    }

    /*
    ** Check the key/value pairs.
    */

    /*
    ** server: if server is set to 'y', we have to listen for connections,
    **         if it's set to 'n', we have to connect to the debugger.
    */

    if(strcmp(key, "server") == 0) {
      if(strcmp(value, "y") == 0) {
        jdwp_config_server = 1;
      }
      else if(strcmp(value, "n") == 0) {
        jdwp_config_server = 0;
      } 
      else {
        printf("jdwp error: server should be set to 'y' or 'n', but not to '%s' !\n", value);
        exit(1);
      }
    } 

    /*
    ** suspend: if suspend is set to 'y', the VM should start up in a suspended 
    **          state, if it's set to 'n', everything stays the way it was.
    */
    
    else if(strcmp(key, "suspend") == 0) {
      if(strcmp(value, "y") == 0) {
        jdwp_config_suspend = 1;
      } 
      else if(strcmp(value, "n") != 0) {
        jdwp_config_suspend = 0;
      }
      else {
        printf("jdwp error: suspend should be set to 'y' or 'n', but not to '%s' !\n", value);
        exit(1);
      }
    } 

    /*
    ** transport: the method of transport for jdwp. Only dt_socket is supported for 
    **            the moment, but support for serial connections is pending...
    */
    
    else if(strcmp(key, "transport") == 0) {
      if(strcmp(value, "dt_socket") != 0) {
        printf("jdwp error: transport can only be set to 'dt_socket', so '%s' is out of the question !\n", value);
        exit(1);
      }
    } 

    /*
    ** address: the address and/or port to listen/connect to.
    */
    
    else if(strcmp(key, "address") == 0) {
      jdwp_address_host = value;
      jdwp_address_port = index(value, ':');
      if(jdwp_address_port != NULL) {
        jdwp_address_port[0] = '\0';
        jdwp_address_port++;
      } else {
        jdwp_address_port = value;
        jdwp_address_host = "127.0.0.1";
      }
    }
  }
  
  return;
}


/*
** Initialize JDWP. 
*/

w_void jdwp_init(w_void) {
  w_ubyte  *stack;

  /*
  ** Check if jdwp is enabled with the -Xdebug commandline parameter.
  */

  if(!jdwp_enabled) {
    
    return;

  }

  /*
  ** Check if there are jdwp specific arguments on the commandline.
  */

  if(jdwp_args) {
    jdwp_parse_arguments(jdwp_args);
  }

  /*
  ** Allocate memory for the thread and its stack.
  */
  
  jdwp_thread = allocClearedMem_system(sizeof(x_Thread));
  stack  = allocMem_system(JDWP_THREAD_STACK_SIZE);

  /*
  ** Create the jdwp mutex.
  */

  jdwp_mutex = x_mem_alloc(sizeof(x_Mutex));
  x_mutex_create(jdwp_mutex);

  /*
  ** Create the dispatcher thread.
  */
  
  x_thread_create(jdwp_thread, jdwp_dispatcher, 0, stack, JDWP_THREAD_STACK_SIZE, JDWP_THREAD_PRIORITY, TF_START);

  /*
  ** Lock the mutex and do this twice. The first lock will be unlocked from within the jdwp thread. This causes
  ** this function to wait until the thread is ready with it's initializations. 
  */ 

//  x_mutex_lock(jdwp_mutex, x_eternal);
//  x_mutex_lock(jdwp_mutex, x_eternal);
//  x_mutex_unlock(jdwp_mutex);
  
}


/*
** This is the main loop of JDWP and it's running in its own thread. First it will try to connect 
** to the debugger, then it goes in to a loop where it waits for commands from the debugger. 
** When a command packet is received, it will be parsed and dispatched to the different commandset
** dispatchers. This will go on and on forever.
*/

w_void jdwp_dispatcher(w_void *argument) {
  
  jdwp_command_packet cmd;
  
  Wonka_AttachArgs   waa;
  JavaVM             *allCreatedJavaVMs[1];
  JavaVM             *mrWonka;
  JNIEnv             *environment;
  jint               nVMs;
  jint               rc;

  /*
  ** Attach JDWP thread to wonka:
  */
  
  rc = JNI_GetCreatedJavaVMs(allCreatedJavaVMs,1,&nVMs);

  if (rc == 0 && nVMs > 0) {
    waa.name = (char*)"JDWP";
    mrWonka = allCreatedJavaVMs[0];
    rc = ((*mrWonka)->AttachCurrentThread)(mrWonka, &environment, &waa);
    if(rc == 0) {
      woempa(9, "attached the JDWP thread to wonka\n");
    } else {
      wabort(ABORT_WONKA, "failed to attach the JDWP thread to wonka\n");
    }
  }
  else {
    wabort(ABORT_WONKA, "failed to attach the JDWP thread to wonka\n");
  }                                         
  
  while(1) {

    /*
    ** Try to connect to the debugger.
    */

    jdwp_connect();

    /*
    ** Now we are connected.
    ** Receive packets and respond to them.
    */

    while(jdwp_connected) {

      cmd = jdwp_recv_packet();

      if(cmd->flags == 0) {

        /*
        ** It's a command packet.
        */

        woempa(9, "command set: %d,  command: %d,  length: %d\n", cmd->command_set, cmd->command, swap_int(cmd->length));

        switch((jdwp_cmdset)cmd->command_set) {
          case jdwp_cmdset_virtualMachine:        dispatch_vm(cmd);              break;
          case jdwp_cmdset_referenceType:         dispatch_reftype(cmd);         break;
          case jdwp_cmdset_classType:             dispatch_classtype(cmd);       break;
          case jdwp_cmdset_arrayType:             dispatch_arraytype(cmd);       break;
          case jdwp_cmdset_interfaceType:         dispatch_inttype(cmd);         break;
          case jdwp_cmdset_method:                dispatch_method(cmd);          break;
          case jdwp_cmdset_field:                 dispatch_field(cmd);           break;
          case jdwp_cmdset_objectReference:       dispatch_objref(cmd);          break;
          case jdwp_cmdset_stringReference:       dispatch_strref(cmd);          break;
          case jdwp_cmdset_threadReference:       dispatch_threadref(cmd);       break;
          case jdwp_cmdset_threadGroupReference:  dispatch_threadgrpref(cmd);    break;
          case jdwp_cmdset_arrayReference:        dispatch_arrayref(cmd);        break;
          case jdwp_cmdset_classLoaderReference:  dispatch_classloaderref(cmd);  break;
          case jdwp_cmdset_eventRequest:          dispatch_eventreq(cmd);        break;
          case jdwp_cmdset_stackFrame:            dispatch_stack(cmd);           break;
          case jdwp_cmdset_classObjectReference:  dispatch_classobjref(cmd);     break;
          case jdwp_cmdset_event:                 dispatch_event(cmd);           break;
          case jdwp_cmdset_wonka:                 dispatch_wonka(cmd);           break;
          default:

            /*
            ** None of the above -> Seems there's a command set we don't know about 
            ** so send the 'not implemented' error.
            */

            jdwp_send_reply(cmd->id, NULL, 0, jdwp_err_not_implemented);
        }

      } else {

        /*
        ** It's a reply packet.
        */

        woempa(9, "Reply packet received, but we don't know how to handle this.. (so we just forget about it)\n");

      }

    }

    jdwp_disconnect();

  }

  releaseMem(cmd);
  
  return;
}


/*
** Convert a Wonka String to a JDWP UTF8 string. This UTF8 is different from those used in
** Wonka because the length at the beginning of the string is 4 bytes instead of 2 bytes.
*/

w_ubyte *jdwp_string2UTF8(w_string string, w_int *length) {
  w_ubyte *UTF8string;
  w_ubyte *result;
  
  UTF8string = string2UTF8(W_Thread_system, string, length);
  result = x_mem_alloc((w_word)(*length + 2));
  memcpy(&result[2], UTF8string, (w_word)*length);
  *length = swap_int((*length - 2));
  memcpy(&result[0], length, 4);
  *length = swap_int(*length);
  releaseMem(UTF8string);

  return result;
}


/*
** Convert a cstring to a JDWP UTF8 string.
*/

w_ubyte *jdwp_cstring2UTF8(w_ubyte *string, w_int *length) {
  w_ubyte *result;
  
  *length = strlen(string);
  result = x_mem_calloc((w_word)(*length + 4 + 1));
  memcpy(&result[4], string, (w_word)*length);
  *length = swap_int(*length);
  memcpy(&result[0], length, 4);
  *length = swap_int(*length);

  return result;
}


/*
** Convert a JDWP UTF8 string to a cstring.
*/

w_ubyte *jdwp_UTF82cstring(w_ubyte *string, w_int *length) {
  w_ubyte *result;
  
  memcpy(length, &string[0], 4);
  *length = swap_int(*length);
  
  result = x_mem_calloc((w_word)(*length + 1));
  memcpy(result, &string[4], (w_word)*length);

  return result;
}


/*
** Convert a Wonka string to a JNI mangled string.
*/

w_ubyte *jdwp_string2JNI(w_string string) {
  w_int    length = string->length;
  w_ubyte  *jni = x_mem_calloc((w_word)(length + 50));
  w_ubyte  *iter = jni;
  w_int    i;
  w_char   c;
  w_ubyte  hex[5];

  for(i = 0; i < length; i++) {
    c = string->chars[i];
    if(c & 0xFF80) { /* unicode */
      sprintf((w_ubyte *)&hex, "%x", c);
      *(iter++) = '_';
      *(iter++) = '0';
      strcpy(iter, (w_ubyte *)&hex);
      iter += 4;
    } else {
      switch(c & 0x00FF) {
        case '/' : *(iter++) = '_'; break ;
        case '[' : *(iter++) = '_'; *(iter++) = '3'; break;
        case ';' : *(iter++) = '_'; *(iter++) = '2'; break;
        case '_' : *(iter++) = '_'; *(iter++) = '1'; break;
        case '(' : *(iter++) = '_'; *(iter++) = '_'; break;
        case ')' : break;
        default  : *(iter++) = c & 0x00FF;
      }
    }
  }
  
  return jni;
}


/*
** Convert a cstring to a JNI mangled string.
*/

w_ubyte *jdwp_cstring2JNI(w_ubyte *string) {
  w_int    length = strlen(string);
  w_ubyte  *jni = x_mem_calloc((w_word)(length + 50));
  w_ubyte  *iter = jni;
  w_int    i;

  for(i = 0; i < length; i++) {
    switch(string[i]) {
      case '/' : *(iter++) = '_'; break ;
      case '[' : *(iter++) = '_'; *(iter++) = '3'; break;
      case ';' : *(iter++) = '_'; *(iter++) = '2'; break;
      case '_' : *(iter++) = '_'; *(iter++) = '1'; break;
      case '(' : *(iter++) = '_'; *(iter++) = '_'; break;
      case ')' : break;
      default  : *(iter++) = string[i];
    }
  }
  
  return jni;
}

