#ifndef _OSWALD_H
#define _OSWALD_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: oswald.h 4294 2006-05-10 11:35:19Z chris $
*/

#include <pthread.h>
#include "xmisc.h"

#define NUM_PRIORITIES     64

#ifdef __BIT_TYPES_DEFINED__
typedef int32_t      x_boolean;
typedef u_int8_t     x_ubyte;
typedef u_int32_t    x_time;
typedef u_int32_t    x_size;
typedef u_int32_t    x_sleep;
typedef u_int32_t    x_word;
typedef u_int32_t    x_uword;
typedef int32_t      x_int;
typedef int64_t      x_long;
#else 
typedef int           x_boolean;
typedef unsigned char x_ubyte;
typedef unsigned int  x_time;
typedef unsigned int  x_size;
typedef unsigned long x_sleep;
typedef unsigned int  x_word;
typedef unsigned int  x_uword;
typedef signed int    x_int;
typedef signed long long x_long;
#endif
typedef x_word                  x_flags;
typedef struct x_Queue *        x_queue;
typedef struct x_Monitor *      x_monitor;
typedef struct x_Thread *       x_thread;
typedef struct x_Sem *          x_sem;
typedef struct x_Mutex *        x_mutex;

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

typedef const char * (*x_report)(x_thread thread);  /* report generator */

/*
** When getting a sem or locking a mutex, these are two extreme values for the
** timeout parameter: x_no_wait means always return immediately (perhaps with
** a return code of xs_no_instance), x_eternal means hang on in there for
** however long it takes.
*/
#define x_no_wait 0
#define x_eternal 0xffffffff

/*
** We're using POSIX threads for oswald like functions and types
*/

#include "o4p.h"
#include "vsprintf.h"

/*
** Possible values for the `flags' argument of x_thread_create
*/
#define TF_START              0x00000000 /* start immediately */
#define TF_SUSPENDED          0x00000001 /* initially thread is suspended */
/*
 * Other possible values of x_thread flags word
 */
#define TF_TIMEOUT            0x00010000 /* most recent operation timed out */
#define TF_WAIT_INT           0x00020000 /* Thread was waiting on monitor, but interrupted by another thread. */
#define TF_RECEIVING          0x00040000 /* Thread blocked receiving on a queue */
#define TF_SENDING            0x00080000 /* Thread blocked sending on a queue */

#define isSet(x, flag)            ((x) & (flag))
#define isNotSet(x, flag)         (!isSet((x), (flag)))
#define setFlag(x, flag)          ((x) |= (flag))
#define unsetFlag(x, flag)        ((x) &= ~(flag))
#define maskFlags(m, f)           ((m) & (f))

/*
** In the real Oswald these calls disable/reenable the scheduler.  
** What we do is to lock a global mutex, which should be close enough.
*/
void x_scheduler_disable(void);
void x_scheduler_enable(void);

/*
** pthreads based x_Monitor and the x_monitor_... primitives.
*/

x_status x_monitor_create(x_monitor);
x_status x_monitor_delete(x_monitor);
x_status x_monitor_enter(x_monitor, x_sleep);
#define  x_monitor_eternal(m) x_monitor_enter((m), x_eternal)
x_status x_monitor_exit(x_monitor);
x_status x_monitor_wait(x_monitor, x_sleep);
x_status x_monitor_notify(x_monitor);
x_status x_monitor_notify_all(x_monitor);
x_status x_monitor_stop_waiting(x_monitor monitor, x_thread thread);

/*
** pthreads based x_Queue and the x_queue_... primitives.
*/
x_status x_queue_create(x_queue queue, void *queue_start, w_size queue_size);
x_status x_queue_delete(x_queue queue);
x_status x_queue_receive(x_queue queue, void **dest_msg, x_sleep waittime);
x_status x_queue_send(x_queue queue, void *src_msg, x_sleep waittime);
x_status x_queue_flush(x_queue queue, void(*do_this)(void *data));

/*
** pthreads based x_Sem and the x_sem_... primitives.
*/
x_status x_sem_create(x_sem semaphore, w_size initial_count);
x_status x_sem_delete(x_sem semaphore);
x_status x_sem_get(x_sem semaphore, x_sleep wait_option);
x_status x_sem_put(x_sem semaphore);
x_status x_sem_wait(x_sem semaphore);
x_status x_sem_post(x_sem semaphore);

/*
** pthreads based x_Mutex and the x_mutex_... primitives.
*/
x_status x_mutex_create(x_mutex mutex);
x_status x_mutex_delete(x_mutex mutex);
x_status x_mutex_lock(x_mutex mutex, x_sleep timeout);
x_status x_mutex_unlock(x_mutex mutex);
  
/*
** And again for threads.
*/

x_status x_thread_create(x_thread thread, x_entry entry_function, void* entry_input, void *stack_start, w_size stack_size, w_size priority, w_word flags);
x_status x_thread_delete(x_thread thread);
w_size x_thread_priority_get(x_thread thread);
w_int x_thread_priority_set(x_thread thread, w_size new_priority);
x_status x_thread_resume(x_thread thread);
x_status x_thread_sleep(x_sleep timer_ticks);
x_status x_thread_suspend(x_thread thread);
x_status x_thread_join(x_thread thread, void **result, x_sleep timeout);
x_status x_thread_stop_waiting(x_thread);
x_status x_thread_wakeup(x_thread);

inline static x_state x_thread_state(x_thread thread) {
  return thread->state;
}


x_thread x_thread_current(void);
void x_thread_yield(void);

/*
** The Oswald time API (part of).
*/
x_sleep x_time_get(void);

x_size x_millis2ticks(x_size millis);
x_size x_seconds2ticks(x_size seconds);
x_size x_ticks2usecs(x_size ticks);
x_size x_usecs2ticks(x_size usecs);


/*
** The Oswald memory API.
*/
x_status x_mem_lock(x_sleep timeout);
x_status x_mem_unlock(void);

void x_mem_init(void);
#ifdef DEBUG
void *_x_mem_alloc(w_size bytes, const char *file, int line);
void *_x_mem_calloc(w_size bytes, const char *file, int line);
#define x_mem_alloc(bytes) _x_mem_alloc((bytes), __FILE__,__LINE__)
#define x_mem_calloc(bytes) _x_mem_calloc((bytes), __FILE__,__LINE__)
#else
void *_x_mem_alloc(w_size bytes);
void *_x_mem_calloc(w_size bytes);
#define x_mem_alloc(bytes) _x_mem_alloc((bytes))
#define x_mem_calloc(bytes) _x_mem_calloc((bytes))
#endif
void x_mem_free(void*);
void *x_mem_realloc(void *old, w_size size);

/*
** Unconditionally walk all blocks in memory, calling a callback on each one.
*/
x_status x_mem_walk(x_sleep timeout, x_boolean (*callback)(void * mem, void * arg), void *arg);

/*
** Walk memory and for each block which has a given tag, invoke the callback
** and if it returns flase (0) abort the walk.
*/
x_status x_mem_scan(x_sleep timeout, x_word tag, x_boolean (*callback)(void * mem, void * arg), void * arg);

w_size x_mem_total(void);
#define x_mem_gettotal x_mem_total

w_size x_mem_avail(void);

x_status x_mem_tag_set(void * mem, w_word tag);
w_word x_mem_tag_get(void * mem);
w_size x_mem_size(void * mem);
void x_mem_discard(void * block);
x_status x_mem_collect(w_size * bytes, w_size * num);
x_boolean x_mem_is_block(void * mem);

void * x_alloc_static_mem(void * memory, w_size size);


/*
** Don't use the memory monitor directly: always call x_memory_lock/unlock.
** We make the monitor visible here for debugging purposes only.
*/
extern x_monitor memory_monitor;

/*
** This is the entry point which the OS will call when it has completed
** its own internal initialisation.
*/
w_ubyte* x_os_main(int argc, char** argv, w_ubyte *first_unused_memory);

/*
** Call this from main() to start up oswald.
** Parameters: max heap in bytes, tick interval in milliseconds.
*/
x_status x_oswald_init(x_size max_heap, x_size millis);

#define x_preemption_disable x_scheduler_disable()
#define x_preemption_enable  x_scheduler_enable()

/*
** Debugging related stuff...
*/

void x_dump_threads(void);
void x_dump_mutex(char *, x_mutex);
void x_dump_monitor(char *, x_monitor);
void x_dump_monitor_if_locked(char *, x_monitor);

#ifdef DEBUG

void _loempa(const char *function, const int line, const int level, const char *fmt, ...);

static const int loempa_trigger = DEBUG_LEVEL;

#define loempa(level, format, a...) {                          \
  if (level >= loempa_trigger) {                               \
    _loempa(__FUNCTION__, __LINE__, level, format, ##a);       \
  }                                                            \
}

static const x_boolean debug = TRUE;

#else /* No DEBUG */

#define loempa(level, format, a...)
static const x_boolean debug = FALSE;

#endif /* DEBUG */

/*
** Runtime checks stuff...
*/

#ifdef RUNTIME_CHECKS

static const x_boolean runtime_checks = TRUE;

/*
** Return true when the event type doesn't match.

inline static x_boolean x_event_type_bad(void * event, x_type type) {
  return ((((x_event)event)->flags_type & EVENT_TYPE_MASK) != (x_ushort)type);
}
*/

void _assert(const char * message);

#define x_assert_string(x) x_assert_val(x)
#define x_assert_val(x)    #x
static inline void _x_fail(const char *function, const char *file, int line, const char *message) {
  printf("Failed assertion in %s at %s:%d : %s\n", function, file, line, message);
}

#define x_assert(test) ((test) ? (void) 0 : _x_fail(__FUNCTION__, __FILE__, __LINE__, x_assert_string(test)))

/*
** Return true when we are in a context sensitive environment, i.e. interrupt handler
** or timer handler. When runtime checks is not defined, we return false so that the
** compiler optimizes away the checks.
*/

inline static x_boolean x_in_context_critical(x_sleep timeout) {
  return FALSE;
}

#else /* No RUNTIME_CHECKS */

static const x_boolean runtime_checks = FALSE;

/*
inline static x_boolean x_event_type_bad(void * event, x_type type) {
  return FALSE;
}
*/

#define x_assert(test)

inline static x_boolean x_in_context_critical(x_sleep timeout) {
  return FALSE;
}

#endif /* RUNTIME_CHECKS */

#endif /* _OSWALD_H */

