/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: mailbox.c 2532 2002-08-14 11:09:08Z vandeneede $
*/

#include <oswald.h>

x_status x_mailbox_create(x_mailbox mailbox) {

  return x_event_init(&mailbox->Event, xe_mailbox);
  
}

inline static x_status x_mailbox_try_send(x_mailbox mailbox, void * message, const x_boolean decrement_competing) {

  if (x_event_is_deleted(mailbox)) {
    if (decrement_competing) {
      mailbox->Event.n_competing--;
    }
    return xs_deleted;
  }

  if (x_event_type_bad(mailbox, xe_mailbox)) {
    return xs_bad_element;
  }

  if (mailbox->message == NULL) {
    mailbox->message = message;
    x_event_signal_all(&mailbox->Event);
    return xs_success;
  }

  return xs_no_instance;
    
}

x_status x_mailbox_send(x_mailbox mailbox, void * message, x_sleep timeout) {

  x_status status;
  x_thread thread = thread_current;

  if (x_in_context_critical(timeout)) {
    return xs_bad_context;
  }
  
  x_preemption_disable;

  /*
  ** See if the message is NULL so that we can put our new message in. If that is the
  ** case, we signal the event to indicate that there is a message waiting...
  **
  ** If the mailbox is occupied, we can only compete untill we find the message slot to be
  ** NULL or timeout...
  */

  status = x_mailbox_try_send(mailbox, message, false);
  if (status == xs_no_instance) {
    if (timeout) {
      while (timeout && status == xs_no_instance) {
        timeout = x_event_compete_for(thread, &mailbox->Event, timeout);
        status = x_mailbox_try_send(mailbox, message, true);
      }
    }
  }

  x_preemption_enable;

  return status;
    
}

inline static x_status x_mailbox_try_receive(x_mailbox mailbox, void ** message, const x_boolean decrement_competing) {

  if (x_event_is_deleted(mailbox)) {
    if (decrement_competing) {
      mailbox->Event.n_competing--;
    }
    return xs_deleted;
  }

  if (x_event_type_bad(mailbox, xe_mailbox)) {
    return xs_bad_element;
  }

  if (mailbox->message != NULL) {
    *message = (void *)mailbox->message;
    mailbox->message = NULL;
    x_event_signal_all(&mailbox->Event);
    return xs_success;
  }

  return xs_no_instance;
  
}

x_status x_mailbox_receive(x_mailbox mailbox, void ** message, x_sleep timeout) {

  x_status status;
  x_thread thread = thread_current;

  if (x_in_context_critical(timeout)) {
    return xs_bad_context;
  }
  
  x_preemption_disable;

  status = x_mailbox_try_receive(mailbox, message, false);
  if (status == xs_no_instance) {
    if (timeout) {
      x_event_compete_for(thread, &mailbox->Event, timeout);
      status = x_mailbox_try_receive(mailbox, message, true);
    }
  }

  x_preemption_enable;
  
  return status;
  
}

x_status x_mailbox_delete(x_mailbox mailbox) {

  x_status status;
  
  x_preemption_disable;
  status = xi_event_destroy(&mailbox->Event);
  x_preemption_enable;
  
  return status;
  
}
