#include "tests.h"
#include <stdio.h>
#include <string.h>

typedef struct w_Desc * w_desc;
typedef unsigned short w_ushort;
typedef unsigned char w_ubyte;
typedef void * w_thread;
typedef struct w_String * w_string;
typedef unsigned short w_char;
typedef unsigned int w_size;
typedef unsigned int w_uword;
typedef signed int w_int;
typedef void * w_clazz;

#define reallocMem(x, s) x_mem_realloc(x, s)
#define releaseMem(x) x_mem_free(x)
#define woempa loempa

typedef struct w_String {
  w_string next;
  w_uword hashcode;
  w_ushort length;
  w_char chars[0];
} w_String;

w_string allstrings = NULL;

w_string newstring(char * str) {

  w_string string;
  int i;
  w_string existing;
  w_string * update;
  
  update = &allstrings;
  for (existing = allstrings; existing; existing = existing->next) {
    if (existing->length == strlen(str)) {
      for (i = 0; i < existing->length; i++) {
        if (existing->chars[i] != str[i]) {
          break;
        }
      }
      if (i == existing->length) {
        break;
      }
    }
    update = &existing->next;
  }

  if (existing) {
    woempa(9, "Found existing '%s'...\n", str);
    return existing;
  }
  
  string = x_mem_alloc(sizeof(w_String) + (strlen(str) + 1) * sizeof(w_char));
  string->length = strlen(str);
  for (i = 0; i < string->length; i++) {
    string->chars[i] = str[i];
  }
  string->chars[i] = '\0';
  
  string->hashcode = 0;
  for (i = 0; i < string->length; i++) {
    string->hashcode = string->chars[i] * i * i * i;
  }

  string->next = NULL;
  *update = string;
  
  woempa(9, "New string '%w', hashcode = 0x%08x\n", string, string->hashcode);
    
  return string;
  
}

w_string unicode2String(w_thread thread, w_char *chars, w_size length) {

  char buffer[200];
  w_size i;
  
  memset(buffer, 0x00, 200);
  
  for (i = 0; i < length; i++) {
    buffer[i] = chars[i];
  }
  
  return newstring(buffer);
  
}

void touchString(w_string string) {
  woempa(9, "Touching '%w'\n", string);
}

// real work starts here

static const x_ubyte tag_boolean   =  4;
static const x_ubyte tag_character =  5;
static const x_ubyte tag_float     =  6;
static const x_ubyte tag_double    =  7;
static const x_ubyte tag_byte      =  8;
static const x_ubyte tag_short     =  9;
static const x_ubyte tag_integer   = 10;
static const x_ubyte tag_long      = 11;
static const x_ubyte tag_void      = 12;
static const x_ubyte tag_object    = 13;
static const x_ubyte tag_method    = 14;
static const x_ubyte tag_array     = 15;

typedef struct w_Desc {
  w_desc   next;                  /* Next in hash chain or next free descriptor. */
  w_ushort hashcode;              /* Hash code of the descriptor. */
  w_ubyte  num;                   /* Number of arguments or array depth or size in bits. */
  w_ubyte  flagstag;              /* Upper 4 bits, flags, lower 4 bits tag type. */
} w_Desc;

#define TAG_MASK       0x0f       /* To mask out the tag bits of the descriptor. */
#define FLAGS_MASK     0xf0       /* To mask out the flags bits of the descriptor. */

#define DESC_TOUCHED   0x10       /* Flag to indicate that the tag is directly referred to. */
#define DESC_ALIVE     0x20       /* Flag to indicate that the tag is indirectly referred to. */
#define DESC_UNUSED_1  0x40
#define DESC_UNUSED_2  0x80

/*
** A helper structure to parse descriptor strings.
*/

typedef struct w_Idesc {
  w_string string;                /* Original descriptor string. */
  w_char * current;               /* The current unicode pointer we are processing. */
  w_char * falloff;               /* This pointer points to one position beyond the last character. */
  w_desc * arguments;             /* In case of a method descriptor, an array holding the arguments. */
  w_thread thread;                /* The thread on whoes behalf we can allocate memory. */
} w_Idesc;

typedef struct w_Idesc * w_idesc;

/*
** Stack helper structure for garbage collector. When tracing descriptors
** that have children references, the component descriptor or argument and
** return descriptors for array or method types, these are pushed on a growing
** stack and popped afterwards for further processing.
*/

typedef struct w_Sdesc {
  w_desc * slots;                 /* Array with desc slots. */
  w_size top;                     /* The stack top index, points to free slot, i.e. 'empty top stack'. */
  w_size capacity;                /* The number of free slots available. */
} w_Sdesc;

typedef struct w_Sdesc * w_sdesc;

/*
** A memory page for allocating descriptors from and the first page
** pointer for memory pages. The constant defines the size of the total page
** in words.
*/

typedef struct w_Mdesc * w_mdesc;

typedef struct w_Mdesc {
  w_mdesc page;                   /* Pointer to the next memory page for descriptors. */
  w_size avail;                   /* How many free slots there are still available. */
  w_uword * free;                 /* The pointer to the free slots available. */
  w_uword slots[0];               /* The slots allocated with the page. */
} w_Mdesc;

static const w_size mdesc_size = 1024;

static w_mdesc first_page;        /* The first memory page allocated. */
static w_mdesc cache_page;        /* Cache for the page we last allocated from. */
static w_mdesc last_page;         /* The last allocated page. */
static w_desc  list_avail;        /* A linked list of available free descriptors; num contains the number of component slots. */
static w_size  num_avail;         /* The number of free descriptors on list_avail. */

static w_Desc Desc_byte      = { NULL, 0x0811,  8, DESC_TOUCHED | DESC_ALIVE |  8 };
static w_Desc Desc_character = { NULL, 0x1970, 16, DESC_TOUCHED | DESC_ALIVE |  5 };
static w_Desc Desc_double    = { NULL, 0x0304, 64, DESC_TOUCHED | DESC_ALIVE |  7 };
static w_Desc Desc_float     = { NULL, 0x1965, 32, DESC_TOUCHED | DESC_ALIVE |  6 };
static w_Desc Desc_integer   = { NULL, 0xcafe, 32, DESC_TOUCHED | DESC_ALIVE | 10 };
static w_Desc Desc_long      = { NULL, 0xbabe, 64, DESC_TOUCHED | DESC_ALIVE | 11 };
static w_Desc Desc_short     = { NULL, 0xdead, 16, DESC_TOUCHED | DESC_ALIVE |  9 };
static w_Desc Desc_boolean   = { NULL, 0xbeaf,  1, DESC_TOUCHED | DESC_ALIVE |  4 };
static w_Desc Desc_void      = { NULL, 0xf00f,  0, DESC_TOUCHED | DESC_ALIVE | 12 };

w_desc desc_byte = &Desc_byte;
w_desc desc_character = &Desc_character;
w_desc desc_double = &Desc_double;
w_desc desc_float = &Desc_float;
w_desc desc_integer = &Desc_integer;
w_desc desc_long = &Desc_long;
w_desc desc_short = &Desc_short;
w_desc desc_boolean = &Desc_boolean;
w_desc desc_void = &Desc_void;

/*
** The number of hashtable leader buckets we have and the hashtable itself;
** plus the monitor lock we have for descriptor handling.
*/

static const w_size buckets = 255;
static w_desc * hashtable;
static x_monitor monitor;

/*
** Allocate and release a descriptor with the given number of component slots.
** The search function first walks over the free descriptors to see if there is
** one with a matching number of component slots.
*/

static w_desc searchAvailable(w_size components) {

  w_desc desc;
  w_desc * update = & list_avail;
  
  for (desc = list_avail; desc; desc = desc->next) {
    woempa(9, "SEARCHING %d <-> %d\n", desc->num, components);
    if (desc->num == components) {
      *update = desc->next;
      woempa(9, "Yippie, found a matching free descriptor with %d slots.\n", desc->num);
      num_avail -= 1;
      return desc;
    }
    update = & desc->next;
  }

  return NULL;
    
}

static w_desc allocDescriptor(w_size components) {

  w_desc desc;
  w_mdesc mdesc;
  w_size slots;

  desc = searchAvailable(components);

  if (! desc) {
    slots = components + sizeof(w_Desc) / sizeof(w_uword);
    woempa(9, "Allocating %d slots, %d available.\n", slots, cache_page->avail);
    if (slots > cache_page->avail) {

      /*
      ** Put the remaining slots as a free descriptor in available list.
      */
    
      desc = (w_desc) cache_page->free;
      desc->num = cache_page->avail - sizeof(w_Desc) / sizeof(w_uword);
      desc->next = list_avail;
      list_avail = desc;
      num_avail += 1;
      woempa(9, "Put remaining slots as descriptor with %d components.\n", desc->num);

      /*
      ** Now allocate a new page.
      */    

      woempa(9, "Allocating new page.\n");
      mdesc = x_mem_alloc(sizeof(w_desc) * mdesc_size);
      mdesc->page = NULL;
      mdesc->free = mdesc->slots;
      mdesc->avail = (mdesc_size - sizeof(w_Mdesc)) / sizeof(w_uword);
      last_page->page = mdesc;
      last_page = mdesc;
      cache_page = mdesc;
    }

    desc = (w_desc) cache_page->free;
    cache_page->free += slots;
    cache_page->avail -= slots;

    woempa(9, "Currently %d available slots.\n", cache_page->avail);

  }

  return desc;
  
}

static void releaseDescriptor(w_desc desc) {

  w_ubyte tag = desc->flagstag & TAG_MASK;

  woempa(9, "Releasing %t\n", desc);
    
  if (tag == tag_object || tag == tag_array) {
    desc->num = 1;
  }
  else {
    desc->num += 1;
  }
  
  desc->next = list_avail;
  list_avail = desc;
  num_avail += 1;

  woempa(9, "Put descriptor with %d slots on available list.\n", desc->num);

}

/*
** Touch a descriptor to indicate that it is alive.
*/

void touchDescriptor(w_desc desc) {
  desc->flagstag |= DESC_TOUCHED;
}

/*
** Get and set the descriptor from a certain offset in a method descriptor.
*/

static w_desc getDescriptor(w_desc desc, w_size i) {
  return * (w_desc *) ((unsigned char *)desc + sizeof(w_Desc) + sizeof(w_desc) * i);
}

static void setDescriptor(w_desc desc, w_size i, w_desc slot) {
  * (w_desc *) ((unsigned char *)desc + sizeof(w_Desc) + sizeof(w_desc) * i) = slot;
}

/*
** Get and set the name string of an object type descriptor.
*/

static void setName(w_desc desc, w_string string) {
  * (w_string *) ((unsigned char *)desc + sizeof(w_Desc)) = string;
}

static w_string getName(w_desc desc) {
  return * (w_string *) ((unsigned char *)desc + sizeof(w_Desc));
}

/*
** Get and set the component descriptor of an array descriptor.
*/

static w_desc getComponent(w_desc desc) {
  return * (w_desc *) ((unsigned char *)desc + sizeof(w_Desc));
}

static void setComponent(w_desc desc, w_desc component) {
  * (w_desc *) ((unsigned char *)desc + sizeof(w_Desc)) = component;
}

/*
** Return true when a given method descriptor matches the elements
** in the intermediate descriptor structure. The given 'desc' must
** point to a method descriptor and the given idesc must have been
** parsed correctly. I.e., no checking is done here.
*/

static x_boolean compareMethodType(w_idesc idesc, w_size num, w_desc returns, w_desc desc) {

  w_size i;
  
  if ((w_ushort) idesc->string->hashcode == desc->hashcode && num == desc->num) {
    
    /*
    ** Check the argument descriptors.
    */
      
    for (i = 0; i < desc->num; i++) {
      if (idesc->arguments[i] != getDescriptor(desc, i)) {
        return false;
      }
    }

    /*
    ** Check the return descriptor.
    */
      
    if (returns != getDescriptor(desc, i)) {
      return false;
    }

    /*
    ** Once we arrive here, we have a match...
    */
      
    return true;

  }
  
  return false;
  
}

static x_boolean compareArrayType(w_int depth, w_desc component, w_desc desc) {

  if (desc->num == depth) {
    if (component == getComponent(desc)) {
      return true;
    }
  }
  
  return false;
  
}

static x_boolean compareObjectType(w_string name, w_desc desc) {

  if ((w_ushort) name->hashcode == desc->hashcode) {
    if (name == getName(desc)) {
      return true;
    }
  }
  
  return false;
  
}

/*
** Returns NULL or the descriptor for the base type if the 
** character pointed to by idesc->current is a base type identifier character.
** If it returns a valid descriptor, the idesc->current pointer has been advanced by
** 1 position to point to the next character.
*/

static w_desc isBaseType(w_idesc idesc) {

  w_desc desc;

  switch (*idesc->current) {
    case 'B': desc = desc_byte; break;
    case 'C': desc = desc_character; break;
    case 'D': desc = desc_double; break;
    case 'F': desc = desc_float; break;
    case 'I': desc = desc_integer; break;
    case 'J': desc = desc_long; break;
    case 'S': desc = desc_short; break;
    case 'Z': desc = desc_boolean; break;
    default : desc = NULL;
  }
  
  if (desc) {
    idesc->current += 1;
  }
  
  return desc;
  
}


/*
** Check if an object descriptor with the given name allready
** exists in the hashtable. If it does not exist, create one and put it in
** the hashtable. Otherwise, return the existing object descriptor.
*/

w_desc findObjectDescriptor(w_string name) {

  w_ushort hashcode = (w_ushort) name->hashcode;
  w_size slot = hashcode % buckets;
  w_desc desc;

  for (desc = hashtable[slot]; desc; desc = desc->next) {
    if ((desc->flagstag & TAG_MASK) == tag_object) {
      if (compareObjectType(name, desc)) {
        return desc;
      }
    }
  }

  desc = allocDescriptor(1);
  desc->flagstag = DESC_TOUCHED | tag_object;
  desc->num = 32;
  desc->hashcode = (w_ushort) name->hashcode;
  setName(desc, name);
  desc->next = hashtable[slot];
  hashtable[slot] = desc;

  return desc;

}

/*
** Returns NULL or a valid descriptor wether the character pointed to by idesc->current
** is the object type identifier character 'L'. When it returns not NULL, the
** idesc->current pointer has been advanced to the next character, not part of the
** object descriptor and the returned descriptor is an object type descriptor.
*/

static w_desc isObjectType(w_idesc idesc) {

  w_desc desc = NULL;
  w_char * start;
  w_string name;

  if (*idesc->current == 'L') {
    idesc->current += 1;
    start = idesc->current;
    while (*idesc->current != ';') {
      if (idesc->current == idesc->falloff) {
        x_Throw("no matching semicolon found.");
      }
      idesc->current += 1;
    }
    name = unicode2String(idesc->thread, start, (w_size) (idesc->current - start));
    idesc->current += 1;
    desc = findObjectDescriptor(name);
  }
  
  return desc;
  
}

/*
** Check if an array descriptor with the given depth and component allready
** exists in the hashtable. If it does not exist, create one and put it in
** the hashtable. Otherwise, return the existing array descriptor.
*/

static w_desc findArrayDescriptor(w_int depth, w_desc component) {

  w_ushort hashcode = component->hashcode + depth;
  w_size slot = hashcode % buckets;
  w_desc desc;

  for (desc = hashtable[slot]; desc; desc = desc->next) {
    if ((desc->flagstag & TAG_MASK) == tag_array) {
      if (compareArrayType(depth, component, desc)) {
        return desc;
      }
    }
  }

  desc = allocDescriptor(1);
  desc->flagstag = DESC_TOUCHED | tag_array;
  desc->num = depth;
  desc->hashcode = component->hashcode + depth;
  setComponent(desc, component);
  desc->next = hashtable[slot];
  hashtable[slot] = desc;

  return desc;

}

static w_desc findMethodDescriptor(w_idesc idesc, w_size num, w_desc returns) {

  w_ushort hashcode = (w_ushort)idesc->string->hashcode;
  w_size slot = hashcode % buckets;
  w_desc desc;
  w_size i;

  woempa(9, "Looking for method with %d arguments\n", num);
  
  for (desc = hashtable[slot]; desc; desc = desc->next) {
    if ((desc->flagstag & TAG_MASK) == tag_method) {
      if (compareMethodType(idesc, num, returns, desc)) {
        return desc;
      }
    }
  }

  desc = allocDescriptor(num + 1);
  desc->flagstag = DESC_TOUCHED | tag_method;
  desc->num = num;
  desc->hashcode = (w_ushort) idesc->string->hashcode;
  for (i = 0; i < num; i++) {
    setDescriptor(desc, i, idesc->arguments[i]);
  }
  setDescriptor(desc, i, returns);
  desc->next = hashtable[slot];
  hashtable[slot] = desc;
  
  return desc;

}

/*
** Returns NULL or a descriptor wether the character pointer to by idesc->current
** is the array identifier character '['. When it returns not NULL, the idesc->current
** pointer has been advanced to point to the next character not part of the 
** array type and the returned type is an array descriptor.
*/

static w_desc isArrayType(w_idesc idesc) {

  w_int depth = 0;
  w_desc desc = NULL;
  
  while (*idesc->current == '[') {
    depth += 1;
    if (idesc->current == idesc->falloff) {
      x_Throw("badly formed array descriptor.");
    }
    idesc->current += 1;
  }
  
  if (depth) {
    desc = isBaseType(idesc);
    if (desc == NULL) {
      desc = isObjectType(idesc);
    }
    if (desc == NULL) {
      x_Throw("Bad array component descriptor.");
    }
    desc = findArrayDescriptor(depth, desc);
  }

  return desc;
  
}

/*
** Returns NULL or a valid field type descriptor,
** wether the character pointed to by idesc->current is a valid field type
** descriptor. After return, idesc->current points to the next character
** that was not part of the field type.
*/

static w_desc isFieldType(w_idesc idesc) {

  w_desc desc = NULL;

  if ((desc = isBaseType(idesc))) {
    return desc;
  }
  else if ((desc = isArrayType(idesc))) {
    return desc;
  }
  else if ((desc = isObjectType(idesc))) {
    return desc;
  }

  return desc;

}

static w_desc isMethodType(w_idesc idesc) {

  static const w_size increment = 10;
  w_desc desc = NULL;
  w_desc * arguments = NULL;
  w_size i;

  if (*idesc->current == '(') {
    idesc->current += 1;

    /*
    ** Parse all argument characters into argument descriptors.
    */
    
    for (i = 0; *idesc->current != ')'; i += 1) {
      if (i == 257) {
        x_Throw("Badly formed method descriptor.");
      }
      if (i % increment == 0) {
        arguments = reallocMem(idesc->arguments, sizeof(w_desc) * (i + increment));
        if (arguments == NULL) {
          x_Throw("no more memory for arguments.");
        }
        idesc->arguments = arguments;
      }
      arguments[i] = isFieldType(idesc);
      if (arguments[i] == NULL) {
        x_Throw("Badly formed method argument.");
      }
    }
    idesc->current += 1;

    /*
    ** Find the return argument descriptor.
    */

    desc = isFieldType(idesc);
    if (desc == NULL) {
      if (*idesc->current == 'V') {
        desc = desc_void;
        idesc->current += 1;
      }
    }

    if (desc) {
      desc = findMethodDescriptor(idesc, i, desc);
    }
    else {
      x_Throw("Badly formed method return argument.");
    }

    /*
    ** Check for trailing garbage.
    */
    
    if (idesc->current != idesc->falloff) {
      x_Throw("Trailing garbage characters in method descriptor.");
    }
  }

  return desc;

}

/*
** Convert a string into a desciptor. When all went successfully, it returns a unique
** descriptor for the given string, otherwise NULL is returned.
*/

w_desc string2Descriptor(w_thread thread, w_string string) {

  volatile void * exception = NULL;
  w_desc desc = NULL;
  w_Idesc Idesc;

  Idesc.string = string;
  Idesc.current = string->chars;
  Idesc.falloff = string->chars + string->length;
  Idesc.arguments = NULL;

  x_monitor_eternal(monitor);

  x_Try {
    desc = isMethodType(&Idesc);
    if (desc == NULL) {
      desc = isFieldType(&Idesc);
    }
  }
  x_Catch (exception) {
    woempa(9, "Something went wrong '%s'\n", exception);
    desc = NULL;
  }

  x_monitor_exit(monitor);

  /*
  ** If we have allocated memory during the parsing, free it now.
  */
  
  if (Idesc.arguments) {
    releaseMem(Idesc.arguments);
  }
  
  return desc;

}

/*
** Swizzle a given constant slot that contains a descriptor string into a 
** slot containing a descriptor. Returns true when all went successfully or false
** otherwise. When it returns false, the slot will still contain a string reference.
*/

x_boolean swizzle2Descriptor(w_clazz clazz, w_size idx) {
  return false;
}

static w_desc pop(w_sdesc stack) {

  if (stack->top) {
    return stack->slots[--stack->top];
  }
  
  return NULL;
  
}

static void push(w_sdesc stack, w_desc desc) {

  static const w_size increment = 2;
  w_desc * slots;

  if (stack->top == stack->capacity) {
    stack->capacity += increment;
    slots = x_mem_realloc(stack->slots, sizeof(w_desc) * stack->capacity);
    if (slots == NULL) {
      x_Throw("no memory for descriptor stack.\n");
    }
    stack->slots = slots;
  }
  
  stack->slots[stack->top++] = desc;
  
}

static void traceChildren(w_sdesc stack, w_desc desc) {

  w_size i;
  w_ubyte tag;

  /*
  ** Only trace non base type descriptors (array, method or object). The base
  ** types will never be collected anyhow; method, array or object types are identified
  ** by a tag number that is larger than tag_void; and only trace descriptors that are
  ** not tagged as alive yet. For any type that contains children, we push the children
  ** on our small stack to process them after we have processed the parent descriptor.
  */

  do {
    tag = desc->flagstag & TAG_MASK;
    if (! (desc->flagstag & DESC_ALIVE) && (tag > tag_void)) {
      desc->flagstag |= DESC_ALIVE;
      if (tag == tag_object) {
        touchString(getName(desc));
      }
      else if (tag == tag_array) {
        push(stack, getComponent(desc));
      }
      else if (tag == tag_method) {
        for (i = 0; i <= desc->num; i++) {
          push(stack, getDescriptor(desc, i));
        }
      }
    }
    desc = pop(stack);
  } while (desc);

}

void collectDescriptors(void) {

  w_desc desc;
  w_size i;
  w_Sdesc Stack;
  w_desc * update;
  w_desc victim;
  volatile void * exception;

  Stack.slots = NULL;
  Stack.top = 0;
  Stack.capacity = 0;

  x_Try {  
    for (i = 0; i < buckets; i++) {
      for (desc = hashtable[i]; desc; desc = desc->next) {
        if (desc->flagstag & DESC_TOUCHED) {
          traceChildren(&Stack, desc);
        }
      }
    }

    /*
    ** Collect all descriptors that haven't been touched and reset
    ** the touched ones. Don't forget to unlink descriptors from the
    ** hashtable when they are being freed.
    */

    for (i = 0; i < buckets; i++) {
      update = & hashtable[i];
      desc = hashtable[i];
      while (desc) {
        if (desc->flagstag & (DESC_TOUCHED | DESC_ALIVE)) {
          desc->flagstag &= ~(DESC_TOUCHED | DESC_ALIVE);
          update = & desc->next;
          desc = desc->next;
        }
        else {
          victim = desc;
          *update = victim->next;
          desc = victim->next;
          releaseDescriptor(victim);
        }
      }
    }
  }

  /*
  ** The only exception that could occur is that we could not allocate memory for the
  ** stack. No big deal, we collect stuff next time around...
  */
    
  x_Catch (exception) {
    woempa(9, "error: %s\n", exception);
  }

  if (Stack.slots) {
    x_mem_free(Stack.slots);
  }

}

x_int oswald_snprintf(char * buf, int bufsize, const char *fmt, ...);

static char tag2char[] = { 'x', 'x', 'x', 'x', 'Z', 'C', 'F', 'D', 'B', 'S', 'I', 'J', 'V' };

static char * formatDescriptor(char * buffer, int * remain, void * d) {

  w_desc desc = d;
  w_ubyte tag = desc->flagstag & TAG_MASK;
  w_string name;
  w_int left = *remain;
  w_size i = 0;
  w_size j;

  if (tag == tag_object) {
    name = getName(desc);
    i = oswald_snprintf(buffer, left, "L%w;", name);
    buffer += i;
    left -= i;
  }
  else if (tag == tag_method) {
    *buffer++ = '(';
    left -= 1;
    for (j = 0; j < desc->num; j++) {
      i = oswald_snprintf(buffer, left, "%t", getDescriptor(desc, j));
      buffer += i;
      left -= i;
    }
    i = oswald_snprintf(buffer, left, ")%t", getDescriptor(desc, j));
    buffer += i;
    left -= i;
  }
  else if (tag == tag_array) {
    for (j = 0; j < desc->num; j++) {
      *buffer++ = '[';
      left -= 1;
    }
    i = oswald_snprintf(buffer, left, "%t", getComponent(desc));
    buffer += i;
    left -= i;
  }
  else if (tag < tag_object) {
    *buffer++ = tag2char[tag];
    left -= 1;
  }

  *remain = left;
    
  return buffer;

}

void initDescriptors(void) {

  /*
  ** Register the formatter function and the format character for printing
  ** descriptors through our vsnprintf function.
  */
  
  register_customer('t', formatDescriptor);

  /*
  ** Allocate memory for hashtable, monitor and create the monitor.
  */
  
  hashtable = x_mem_alloc(sizeof(w_desc) * buckets);
  monitor = x_mem_alloc(sizeof(x_Monitor));
  x_monitor_create(monitor);

  /*
  ** Allocate and set up the first memory page for descriptors.
  */
  
  first_page = x_mem_alloc(sizeof(w_desc) * mdesc_size);
  first_page->page = NULL;
  first_page->free = first_page->slots;
  first_page->avail = (mdesc_size - sizeof(w_Mdesc)) / sizeof(w_uword);
  cache_page = first_page;
  last_page = first_page;

  woempa(9, "%d free slots in first page.\n", first_page->avail);
  
}

void dumpDescriptors(void) {

  w_size i;
  w_desc desc;
  w_size total = 0;
  
  woempa(9, "--- Dumping descriptors -----------------------------------------------------\n");
  for (i = 0; i < buckets; i++) {
    for (desc = hashtable[i]; desc; desc = desc->next) {
      woempa(9, "%3d %t\n", i, desc);
      total += 1;
    }
  }
  woempa(9, "--- Done dumping %4d descriptors --------------------------------------------\n", total);

}

void d_test(void *arg) {

  w_string s;
  w_size i = 0;
  w_size j;
  w_desc * descs = x_mem_alloc(sizeof(w_desc) * 1024);

  string();
  exit(0);

  initDescriptors();

  s = newstring("(II)F");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);
    
  s = newstring("(JJDFIZLjava/lang/Object;ZJJDZZ[[[I)[[[[Ljava/lang/String;");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("(JJDFIZLjava/lang/Object;ZJJDZZ[[[I)V");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("[[[[Ljava/lang/String;");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("[[[[Ljava/lang/Object;");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("(II)S");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("Ljava/lang/ref/Reference;");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  s = newstring("(II)V");
  descs[i++] = string2Descriptor(NULL, s);
  loempa(9, "'%w' -> %p\n", s, descs[i - 1]);

  collectDescriptors();
  dumpDescriptors();
  woempa(9, "---------------- %d descriptors\n", i);
  for (j = 1; j < i - 1; j++) {
    if (j != 5)
    touchDescriptor(descs[j]);
  }
  collectDescriptors();
  dumpDescriptors();
    
  exit(0);
      
}

char * print_string(char * buffer, int * remain, void * s) {

  w_string string = s;
  w_char * chars = string->chars;
  w_char * end = string->chars + string->length;
  
  while (*remain > 0 && chars < end) {
    *buffer++ = *chars++;
  }

  return buffer;

}


static x_thread thread;

x_ubyte * descriptor_test(x_ubyte * memory) {

  register_customer('w', print_string);

  thread = x_alloc_static_mem(memory, sizeof(x_Thread));
  x_thread_create(thread, d_test, thread, x_alloc_static_mem(memory, 1024 * 16), 1024 * 16, 4, TF_START);
  return memory;

}
