/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#include <stdio.h>
#include <string.h>

void write_opening(FILE * o) {
  fprintf(o, "\n");
  fprintf(o, "/*\n");
  fprintf(o, "** This file is generated automatically. Do not edit this file.\n");
  fprintf(o, "** Use the 'credtab' tool to generate it.\n");
  fprintf(o, "**\n");
  fprintf(o, "*/\n");
  fprintf(o, "\n");
}

void write_includes(FILE * o, char * includes) {
  FILE * file;
  const unsigned int size = 256;
  char buffer[size];
  
  file = fopen(includes, "r");
  if (! file) {
    printf("Could not open file '%s'.\n", includes);
    exit(1);
  }

  fprintf(o, "#include \"oswald.h\"\n");
  fprintf(o, "#include \"modules.h\"\n");
  fprintf(o, "#include \"elf.h\"\n");
  fprintf(o, "#include \"symbols.h\"\n");

  fprintf(o, "\n");
  
  while (fgets(buffer, (signed int)size, file)) {
    fprintf(o, "%s", buffer);
  }
}

void write_insert_function(FILE * o) {
  fprintf(o, "\n");
  fprintf(o, "x_status x_symtab_insert(x_symtab symtab, const char * label, const void * address, x_flags flags) {\n");
  fprintf(o, "\n");
  fprintf(o, "  x_ident ident;\n");
  fprintf(o, "\n");
  fprintf(o, "  if (symtab->num_symbols < symtab->capacity) {\n");
  fprintf(o, "    ident = x_ident_search(label, true);\n");
  fprintf(o, "    symtab->symbols[ symtab->num_symbols ].next = NULL;\n");
  fprintf(o, "    symtab->symbols[ symtab->num_symbols ].flags = (SYM_EXPORTED | SYM_RESOLVED | flags);\n");
  fprintf(o, "    symtab->symbols[ symtab->num_symbols ].value.address = (x_address)address;\n");
  fprintf(o, "    symtab->symbols[ symtab->num_symbols ].ident = ident;\n");
  fprintf(o, "    ident->symbols = & symtab->symbols[ symtab->num_symbols ];\n");
  fprintf(o, "\n");
  fprintf(o, "    symtab->num_symbols += 1;\n");
  fprintf(o, "\n");
  fprintf(o, "    return xs_success;\n");
  fprintf(o, "\n");
  fprintf(o, "  }\n");
  fprintf(o, "\n");
  fprintf(o, "  return xs_no_instance;\n");
  fprintf(o, "\n"); 
  fprintf(o, "}\n");
  fprintf(o, "\n");
}

void write_func_head(FILE * o) {
  fprintf(o, "static x_Module Static_module;\n");
  fprintf(o, "static x_module static_module = &Static_module;\n");
  fprintf(o, "\n");
  fprintf(o, "extern x_symtab x_symtab_static(void);\n");
  fprintf(o, "x_symtab symtab;\n");
  fprintf(o, "\n");
  fprintf(o, "void x_symtab_kernel(void) {\n");
  fprintf(o, "  symtab = x_symtab_static();\n");
  fprintf(o, "\n");
}

void write_func_tail(FILE * o) {
  fprintf(o, "\n");
  fprintf(o, "  static_module->ident = x_ident_search(\"kernel\", true);\n");
  fprintf(o, "  static_module->symtab = symtab;\n");
  fprintf(o, "  symtab->module = static_module;\n");
  fprintf(o, "\n");
  fprintf(o, "}\n");
  fprintf(o, "\n");
}

void write_func_symbol(FILE * o, char * name, char * type) {
  fprintf(o, "  x_symtab_insert(symtab, \"%s\", %s, %s);\n", name, name, 
          (strncmp(type, "f", 1) == 0 ? "SYM_FUNCTION" : "SYM_VARIABLE"));
}

#define skip_whitespace(cursor) while (*cursor == ' ' || *cursor == '\t') { cursor++; }

int main(int argc, char * argv[]) {

  FILE * file;
  FILE * f_output;
  const unsigned int size = 256;
  char buffer[size];
  char name[size];
  char comment[size];
  char arguments[size];
  char * cursor;
  char * writte;
  char * newline;

  if (argc < 4) {
    fprintf(stderr, "usage %s <table> <includes> <output>\n", argv[0]);
    exit(0);
  }
  
  /*
  ** Open the table file.
  */
  
  file = fopen(argv[1], "r");
  if (! file) {
    printf("Could not open file '%s'.\n", argv[1]);
    exit(1);
  }

  /*
  ** Open the output file.
  */
  
  f_output = fopen(argv[3], "w");
  if (! f_output) {
    fprintf(stderr, "Could not open output file '%s'.\n", argv[3]);
    exit(1);
  }
  
  write_opening(f_output);
  write_includes(f_output, argv[2]);
  write_insert_function(f_output);
  write_func_head(f_output);

  /*
  ** Read in the symbols
  */
  
  while (fgets(buffer, (signed int)size, file)) {
    cursor = buffer;
    newline = buffer + strlen(buffer) - 1;

    /*
    ** Skip empty lines.
    */
    
    if (strlen(buffer) < 3) {
      continue;
    }
    
    skip_whitespace(cursor);
    
    /*
    ** Skip comment lines.
    */
    
    if (*cursor == '#') {
      continue;
    }

    /*
    ** Parse the name of the symbol out of the buffer up to the newline or the next white space.
    */
    
    memset(name, 0x00, size);
    writte = name;
    while (*cursor != '\n' && *cursor != ' ') {
      *writte++ = *cursor;
      cursor++;
    }
    
    skip_whitespace(cursor);
    
    memset(arguments, 0x00, size);
    if (cursor < newline) {
      writte = arguments;
      while (*cursor != '#' && *cursor != '\n') {
        *writte++ = *cursor;
        cursor++;
      }
    }

    skip_whitespace(cursor);
    
    memset(comment, 0x00, size);
    if (cursor < newline) {
      cursor += 1;
      skip_whitespace(cursor);
      writte = comment;
      while (*cursor != '\n' && *cursor != '\0') {
        *writte++ = *cursor;
        cursor++;
      }
    }
    
    /* printf("label = '%s' arguments = '%s' comment = '%s'\n", name, arguments, comment); */

    write_func_symbol(f_output, name, arguments);
    
  }
  
  write_func_tail(f_output);
  
  fclose(file);
  fclose(f_output);

  exit(0);

}

