/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package gnu.testlet.wonka.lang.reflect.Array;
import gnu.testlet.Testlet;
import gnu.testlet.TestHarness;
import java.lang.reflect.Array;
import java.lang.*;

/**
* Tests for java.lang.reflect.Array
* <br>
* the set functions are not implemented in wonka --> test not written yet	
*
*/
public class SMArrayTest implements Testlet
{
	protected TestHarness th;

  	public void test (TestHarness harness)
  	{
		th = harness;    
      		th.setclass("java.lang.reflect.Array");
		test_newInstance();
		test_getLength();
		test_get();
		test_getBoolean();
		test_getByte();
		test_getChar();
		test_getDouble();
		test_getFloat();
		test_getInt();
		test_getLong();
		test_getShort();
		test_set();
		test_setBoolean();
		test_setByte();
		test_setChar();
		test_setDouble();
		test_setFloat();
		test_setInt();
		test_setLong();
		test_setShort();


	}
/**
* implemented.	<br>
* This test should be altered if the max dimensionsize of arrays is NOT 255 <br>
*
*/
	protected void test_newInstance()
	{
	    th.checkPoint("newInstance(java.lang.Class,int)java.lang.Object");

		Object oa = Array.newInstance(Integer.TYPE, 10);
		th.check(oa.getClass().isArray(), "oa should be an Array");

		int [] ia = (int[])Array.newInstance(Integer.TYPE, 10);
		th.check (ia.length == 10, "ia.length = "+ia.length+", should be 10");
		try 	{ ia[9] = 10; th.check(true); 
			  th.check(ia[9] == 10, "array is not correctly implemented");	
			}
		catch (Exception e) { th.fail("no exception expected"); }
		try   { ia[10] = 10; 
			th.fail("should throw an ArrayIndexOutOfBoundsException"); 
		      }
		catch (Exception e) { th.check(true); }

		try 	{
			oa = Array.newInstance(Float.TYPE, -1);
			th.fail("should throw an NegativeArraySizeException");
			}
		catch	(NegativeArraySizeException ne)	{ th.check(true); }
		try 	{
			oa = Array.newInstance(Float.TYPE, 0);
		        th.check(true);
			th.check(oa != null);
			}
		catch	(NegativeArraySizeException ne)	{th.fail("should throw not NegativeArraySizeException"); }



	    th.checkPoint("newInstance(java.lang.Class,int[])java.lang.Object");

		ia = new int [10];
		oa = Array.newInstance(Integer.TYPE, ia);
		th.check(oa.getClass().isArray(), "oa should be an Array");
// test on max dimensionsize
		ia = (int[])Array.newInstance(Integer.TYPE, 256);
		for (int i=0 ; i < 256 ; i++) { ia[i] = 10; }
		try 	{ 
			oa = Array.newInstance(Integer.TYPE, ia);
			th.fail("should throw an IllegalArgumentException"); 
		        }
		catch (IllegalArgumentException e) { th.check(true); }
		ia = new int [0];		
		try 	{ 
			oa = Array.newInstance(Integer.TYPE, ia);
			th.fail("should throw an IllegalArgumentException"); 
		        }
		catch (IllegalArgumentException e) { th.check(true); }
		
	}
/**
* implemented.
*
*/
	protected void test_getLength()
	{
		th.checkPoint("getLength(java.lang.Object)int");
		
		int [] ia = null;
		try  {	Array.getLength(ia);
			th.fail("should throw an NullPointerException");
		     }
		catch (NullPointerException ne)	{ th.check(true); }

		Object oa = Array.newInstance(Integer.TYPE, 2);
		try  {	th.check(Array.getLength(oa), 2 ,"length should be 2");
			th.check(true);
		     }
		catch (IllegalArgumentException ie)	{ th.fail("should not throw IllegalArgumentException, got:"+ie); }

		oa = new Object();
		try  {	Array.getLength(oa);
			th.fail("should throw IllegalArgumentException");
		     }
		catch (IllegalArgumentException ie)	{ th.check(true); }

		ia = (int[])Array.newInstance(Integer.TYPE, 25);
		for (int i=0 ; i < 25 ; i++) { ia[i] = 10; }
		th.check(Array.getLength(ia)== 25, "length is the number of dimensions");
		th.check(ia.length == 25, "test covers also newIstance()");
	}
/**
* implemented.
*
*/
	protected void test_get()
	{
		th.checkPoint("get(java.lang.Object,int)java.lang.Object");

		boolean []ba = new boolean[2];
		ba[1] = true; ba[0] =false;
		Object ro = Array.get(ba, 1);
		th.check(ro instanceof Boolean ,"get() failed  -- 1 ");
		th.check(((Boolean)ro).booleanValue(), "wrong boolean value returned -- 2");

		byte []bta = new byte[2];
		bta[1] = 13; bta[0] =23;
		ro =Array.get(bta, 1);
		th.check(ro instanceof Byte ,"get() failed  -- 3 ");
		th.check(((Byte)ro).byteValue()==13, "wrong byte value returned -- 4 ,got:"+ro);

		char [] ca = new char[2];
		ca[1] = 't'; ca[0] ='f';
		ro =Array.get(ca, 0);
		th.check(ro instanceof Character ,"get() failed  -- 5 ");
		th.check(((Character)ro).charValue()== 'f', "wrong char value returned -- 6 ,got:"+ro);

		double []da = new double[2];
		da[1] = 3.0; da[0] =2.0;
		ro =Array.get(da, 1);
		th.check(ro instanceof Double ,"get() failed  -- 7 ");
		th.check(((Double)ro).doubleValue()== 3.0, "wrong double value returned -- 8  ,got:"+ro);
                th.check(da[1] == 3.0 ,"basic check");

		float []fa = new float[2];
		fa[1] = 3.6f; fa[0] =2.5f;
		ro =Array.get(fa, 0);
		th.check(ro instanceof Float ,"get() failed  -- 9 ");
		th.check(((Float)ro).floatValue()== 2.5f, "wrong float value returned -- 10  ,got:"+ro);

		int []ia = new int[2];
		ia[1] = 23; ia[0] = 45;
		ro =Array.get(ia, 1);
		th.check(ro instanceof Integer ,"get() failed  -- 11 ");
		th.check(((Integer)ro).intValue()== 23, "wrong int value returned -- 12  ,got:"+ro);

		long []la = new long[2];
		la[1] = 123L; la[0] =213L;
		ro =Array.get(la, 1);
		th.check(ro instanceof Long ,"get() failed  -- 13 ");
		th.check(((Long)ro).longValue()== 123, "wrong long value returned -- 14  ,got:"+ro);

		short []sa = new short[2];
		sa[1] = 12; sa[0] =21;
		ro =Array.get(sa, 1);
		th.check(ro instanceof Short ,"get() failed  -- 15 ");
		th.check(((Short)ro).shortValue()== 12, "wrong short value returned --16 ,got:"+ro);

		Object []oa = new Object[2];
		oa[0] = this ; oa[1] = null; 
		ro = Array.get(oa , 0 );
		th.check( ro.equals(this),"get returned wrong Object");		
		ro = Array.get(oa , 1 );
		th.check( ro == null , "null objects are allowed!");
		try   {	Array.get(oa , -1);
			th.fail("ArrayIndexOutOfBoundsException should be thrown");
		      }		
		catch ( ArrayIndexOutOfBoundsException ae ) { th.check(true); }
		try   {	Array.get(oa , 10);
			th.fail("ArrayIndexOutOfBoundsException should be thrown");
		      }		
		catch ( ArrayIndexOutOfBoundsException ae ) { th.check(true); }
		oa = null;
		try   {	Array.get(oa , 0);
			th.fail("NullPointerException should be thrown");
		      }		
		catch ( NullPointerException ne ) { th.check(true); }




	}
/**
* implemented.
*
*/
	protected void test_getBoolean()
	{
		th.checkPoint("getBoolean(java.lang.Object,int)boolean");

		boolean [] ba = new boolean [10];
		for (int i=0; i < 10 ; i++) { ba[i] = true; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getBoolean(ba , j), "retrieved wrong value -- "+j); }
		ba[0] = false;
		th.check(!Array.getBoolean(ba,0) , "retrieved wrong value -- 11");
		
		try {	Array.getBoolean(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getBoolean(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getBoolean(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getBoolean(new Object() , 1);
			th.fail("should throw an IllegalArgumentException");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		short []sa = new short[2];
		sa[1] = 12; sa[0] =21;
		try {	Array.getBoolean(sa , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }		
		Object[] ob = (Object [])Array.newInstance(new Boolean(false).getClass() , 2);
		ob[1] = ob[0] =new Boolean(false);
		try {	Array.getBoolean(ob , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }		
	}
	
/**
* implemented.
*
*/
	protected void test_getByte()
	{
		th.checkPoint("getByte(java.lang.Object,int)byte");

		byte [] ba = new byte [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (byte)i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getByte(ba , j)== (byte)j, "retrieved wrong value -- "+j); }
	
		try {	Array.getByte(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getByte(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getByte(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getByte(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		short []sa = new short[2];
		sa[1] = 12; sa[0] =21;
		try {	Array.getByte(sa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }
	}
/**
* implemented.
*
*/
	protected void test_getChar()
	{
		th.checkPoint("getChar(java.lang.Object,int)char");

		char [] ba = new char [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (char)i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getChar(ba , j)== (char)j, "retrieved wrong value -- "+j); }
	
		try {	Array.getChar(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getChar(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getChar(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getChar(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		short []sa = new short[2];
		sa[1] = 12; sa[0] =21;
		try {	Array.getChar(sa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }
		
		byte []bta = new byte[2];
		bta[1] = (byte)12; bta[0] =(byte)21;
		try {	Array.getChar(bta , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }
	}
/**
* implemented.
*
*/
	protected void test_getDouble()
	{
		th.checkPoint("getDouble(java.lang.Object,int)double");

		double d;
		double [] ba = new double [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (double)i; }
		Array.get(ba , 1);
		for (int j=0; j < 10 ; j++)
			{
			th.check(((Double)Array.get(ba,j)).doubleValue() == (double)j, "retrieved wrong value -- a "+j);
			th.check( Array.getDouble(ba , j)== (double)j, "retrieved wrong value -- b "+j);
		}
	
		try {	Array.getDouble(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getDouble(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getDouble(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getDouble(new Object() , 1);
			th.fail("should throw an IllegalArgumentException --  1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		boolean []wa = new boolean[2];
		wa[1] = false; wa[0] =true;
		try {	Array.getDouble(wa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		float []fa = new float[2];
		fa[1] = 1.2f; fa[0] =2.3f;
		try {	d = Array.getDouble(fa , 1); th.check(d == (double) 1.2f,"checking value of float"); }
		catch (IllegalArgumentException ne) { th.fail("shouldn't throw an IllegalArgumentException -- 1"); }
		long []la = new long[2];
		la[1] = 1L; la[0] =2L;
		try {	d = Array.getDouble(la , 1); th.check(d == 1.0);  }
		catch (IllegalArgumentException ne) { th.fail("shouldn't throw an IllegalArgumentException -- 2"); }
		int []ia = new int[2];
		ia[1] = 1; ia[0] =2;
		try {	Array.getDouble(ia , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 3"); }
		char []ca = new char[2];
		ca[1] = 'f'; ca[0] ='a';
		try {	Array.getDouble(ca , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 4"); }
		byte []bta = new byte[2];
		bta[1] = 1; bta[0] =1;
		try {	Array.getDouble(bta , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 5"); }
		short []sa = new short[2];
		sa[1] = 1; sa[0] =1;
		try {	Array.getDouble(sa , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 6"); }

	}
/**
* implemented.
*
*/
	protected void test_getFloat()
	{
		th.checkPoint("getFloat(java.lang.Object,int)float");

		float [] ba = new float [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (float)i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getFloat(ba , j)== (float)j, "retrieved wrong value -- "+j); }
	
		try {	Array.getFloat(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 1");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getFloat(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 2");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getFloat(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getFloat(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		boolean []wa = new boolean[2];
		wa[1] = false; wa[0] =true;
		try {	Array.getFloat(wa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		double []da = new double[2];
		da[1] = 1.2; da[0] =2.3;
		try {	Array.getFloat(da , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
	
		long []la = new long[2];
		la[1] = 1L; la[0] =2L;
		try {	Array.getFloat(la , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 1"); }
		int []ia = new int[2];
		ia[1] = 1; ia[0] =2;
		try {	Array.getFloat(ia , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 2"); }
		char []ca = new char[2];
		ca[1] = 'f'; ca[0] ='a';
		try {	Array.getFloat(ca , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 3"); }
		byte []bta = new byte[2];
		bta[1] = 1; bta[0] =1;
		try {	Array.getFloat(bta , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 4"); }
		short []sa = new short[2];
		sa[1] = 1; sa[0] =1;
		try {	Array.getFloat(sa , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 5"); }


	}
/**
* implemented.
*
*/
	protected void test_getInt()
	{
		th.checkPoint("getInt(java.lang.Object,int)int");

		int [] ba = new int [10];
		for (int i=0; i < 10 ; i++) { ba[i] = i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getInt(ba , j)== j, "retrieved wrong value -- "+j); }
	
		try {	Array.getInt(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 1");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getInt(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 2");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getInt(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getInt(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		boolean []wa = new boolean[2];
		wa[1] = false; wa[0] =true;
		try {	Array.getInt(wa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		double []da = new double[2];
		da[1] = 1.2; da[0] =2.3;
		try {	Array.getInt(da , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		float []fa = new float[2];
		fa[1] = 1.2f; fa[0] =2.3f;
		try {	Array.getInt(fa , 1);
			th.fail("should throw an IllegalArgumentException -- 4");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		long []la = new long[2];
		la[1] = 12L; la[0] =343443L;
		try {	Array.getInt(la , 1);
			th.fail("should throw an IllegalArgumentException -- 5");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		char []ca = new char[2];
		ca[1] = 'f'; ca[0] ='a';
		try {	Array.getInt(ca , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 1"); }
		byte []bta = new byte[2];
		bta[1] = 1; bta[0] =1;
		try {	Array.getInt(bta , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 2"); }
		short []sa = new short[2];
		sa[1] = 1; sa[0] =1;
		try {	Array.getInt(sa , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 3"); }


	}
/**
* implemented.
*
*/
	protected void test_getLong()
	{
		th.checkPoint("getLong(java.lang.Object,int)long");

		long [] ba = new long [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (long)i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getLong(ba , j)== (long)j, "retrieved wrong value -- "+j); }
	
		try {	Array.getLong(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 1");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getLong(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 2");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getLong(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getLong(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		boolean []wa = new boolean[2];
		wa[1] = false; wa[0] =true;
		try {	Array.getLong(wa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		double []da = new double[2];
		da[1] = 1.2; da[0] =2.3;
		try {	Array.getLong(da , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		float []fa = new float[2];
		fa[1] = 1.2f; fa[0] =2.3f;
		try {	Array.getLong(fa , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
	
		int []ia = new int[2];
		ia[1] = 1; ia[0] =2;
		try {	Array.getLong(ia , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 1"); }
		char []ca = new char[2];
		ca[1] = 'f'; ca[0] ='a';
		try {	Array.getLong(ca , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 2"); }
		byte []bta = new byte[2];
		bta[1] = 1; bta[0] =1;
		try {	Array.getLong(bta , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 3"); }
		short []sa = new short[2];
		sa[1] = 1; sa[0] =1;
		try {	Array.getLong(sa , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException -- 4"); }


	}
/**
* implemented.
*
*/
	protected void test_getShort()
	{
		th.checkPoint("getShort(java.lang.Object,int)short");

		short [] ba = new short [10];
		for (int i=0; i < 10 ; i++) { ba[i] = (short)i; }
		for (int j=0; j < 10 ; j++)
			{ th.check( Array.getShort(ba , j)==(short) j, "retrieved wrong value -- "+j); }
	
		try {	Array.getShort(ba , -1);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 1");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		try {	Array.getShort(ba , 10);
			th.fail("should throw an ArrayIndexOutOfBoundsException -- 2");
		    }
		catch (ArrayIndexOutOfBoundsException ae) { th.check(true); }
		ba = null ;
		try {	Array.getShort(ba , 1);
			th.fail("should throw a NullPointerException");
		    }
		catch (NullPointerException ne) { th.check(true); }

		try {	Array.getShort(new Object() , 1);
			th.fail("should throw an IllegalArgumentException -- 1");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		boolean []wa = new boolean[2];
		wa[1] = false; wa[0] =true;
		try {	Array.getShort(wa , 1);
			th.fail("should throw an IllegalArgumentException -- 2");
		    }
		catch (IllegalArgumentException ne) { th.check(true); }

		double []da = new double[2];
		da[1] = 1.2; da[0] =2.3;
		try {	Array.getShort(da , 1);
			th.fail("should throw an IllegalArgumentException -- 3");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		float []fa = new float[2];
		fa[1] = 1.2f; fa[0] =2.3f;
		try {	Array.getShort(fa , 1);
			th.fail("should throw an IllegalArgumentException -- 4");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		long []la = new long[2];
		la[1] = 12L; la[0] =343443L;
		try {	Array.getShort(la , 1);
			th.fail("should throw an IllegalArgumentException -- 5");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		char []ca = new char[2];
		ca[1] = 'L'; ca[0] ='L';
		try {	Array.getShort(ca , 1);
			th.fail("should throw an IllegalArgumentException -- 6");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }
		int []ia = new int[2];
		ia[1] = 12; ia[0] =343;
		try {	Array.getShort(ia , 1);
			th.fail("should throw an IllegalArgumentException -- 7");
		    }
		catch (IllegalArgumentException ne) {  th.check(true); }

		byte []bta = new byte[2];
		bta[1] = 1; bta[0] =1;
		try {	Array.getShort(bta , 1); th.check(true);  }
		catch (IllegalArgumentException ne) {  th.fail("shouldn't throw an IllegalArgumentException"); }


	}
/**
* not implemented.
*
*/
	protected void test_set()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setBoolean()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setByte()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setChar()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setDouble()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setFloat()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setInt()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setLong()
	{


	}
/**
* not implemented.
*
*/
	protected void test_setShort()
	{


	}

}
