/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


*/

/******************************************************************************************************************************************
 This file contains the helper classes needed to run TCP/IP tests in a mauve framework.
 In brief, it contains the classes:
 ThreadServer: runs a server in a separate thread. The server awaits ONE socket call and handles the connection
 ReplyHandler: the socket handler for the thread server
 MultithreadServer: as above, but this server runns multiple connections in separate threads, so it can handle multiple calls at one time
 ThreadReplayHandler: the socket handler class for the multi-threadserver
*/

package gnu.testlet.wonka.net;

import java.io.*;
import java.net.*;

import gnu.testlet.Testlet;
import gnu.testlet.TestHarness;

/**
****************************************************************************************************************************************
*
* PortAdress TCP/IP and datagram adress& port classes
* with many of the socket build, send- and receive functions using an 'extended' adress, consisting of both an InetAdress
* and a connection port, this class helps in accessing both parts in one class
*
****************************************************************************************************************************************
**/
public class DatagramAddress
{
  /** Null member **/
  final static DatagramAddress NullAddress = new DatagramAddress(null, -1);

  private InetAddress targetAddress;
  private int        targetPort;

  /** constructors **/
  public DatagramAddress(InetAddress address, int port)
  {
    targetAddress = address;
    targetPort    = port;
  }
  static DatagramAddress getDatagramAddress (InetAddress address, int port) /** static constructor **/
  {return new DatagramAddress(address, port);}

  public DatagramAddress(String address, int port, TestHarness harness)
  {
    try
    {
      targetAddress = InetAddress.getByName(address);
      targetPort    = port;
    }
    catch (UnknownHostException e)
    {
      targetAddress = null;
      targetPort    = -1;
      if(harness != null)
        harness.fail(e.toString());
    }
  }

  static DatagramAddress getDatagramAddress(String address, int port, TestHarness harness) /** static constructor **/
  {return new DatagramAddress(address, port, harness);}

  public DatagramAddress(Socket s)
  {
    targetAddress = s.getInetAddress();
    targetPort    = s.getPort();
  }
  static DatagramAddress getDatagramAddress(Socket s) /** static constructor **/ {return new DatagramAddress(s);}


  public DatagramAddress(DatagramPacket d)
  {
    targetAddress = d.getAddress();
    targetPort    = d.getPort();
  }
  static DatagramAddress getDatagramAddress(DatagramPacket d) /** static constructor **/ {return new DatagramAddress(d);}

  public DatagramAddress(DatagramSocket s)
  {
    targetAddress = s.getLocalAddress();
    targetPort    = s.getLocalPort();
  }
  static DatagramAddress getDatagramAddress(DatagramSocket s) /** static constructor **/ {return new DatagramAddress(s);}

  public DatagramAddress(ExternalDatagramHandler handler)
  {
    this(handler.getSocket() );
  }
  static DatagramAddress getDatagramAddress(ExternalDatagramHandler handler) /** static constructor **/
    {return new DatagramAddress(handler.getSocket());}

  static DatagramAddress getDefaultTarget  (ExternalDatagramHandler handler) /** Handler default target **/
    {return handler.getDefaultTarget();}



  public DatagramAddress(DatagramAddress tocopy)
  {
    targetAddress = tocopy.getAddress();
    targetPort    = tocopy.getPort();
  }

  //return the data
  /** get target address **/public InetAddress getTargetAddress() {return targetAddress;}
  /** get target address **/public InetAddress getTarget() {return targetAddress;}
  /** get target address **/public InetAddress getAddress(){return targetAddress;}

  /** get target port **/public int getTargetPort(){return targetPort;}
  /** get target port **/public int getPort()      {return targetPort;}

  /** a DatagramAddress is considered null when it is unusable : when either its address or host name are invallid **/
  public boolean isNullAddress()                      {return(targetAddress == null || targetPort <0); }
  public boolean isNull()                             {return isNullAddress(); }
  static boolean isNullAddress(DatagramAddress target){return(target==null)?true:target.isNullAddress();}
  static boolean isNull(DatagramAddress target)       {return isNullAddress(target);}


  /** is multicast address. checks if the target's ip-address is a multicast address **/
  public boolean isMulticastAddress()     {return(targetAddress == null)? false: targetAddress.isMulticastAddress();}
  static boolean isMulticastAddress(DatagramAddress target) {return(target==null)?false:target.isMulticastAddress();}





  /** String representation: overrides Object.toString ()**/
  public String toString()
  {
    String tostring;
    if( isNullAddress())
      tostring = "NullAddress";
    else
      tostring = "<"+targetAddress.toString()+","+targetPort+">";
    return tostring;
  }

  /** comparison function: overrides Object.equals ()**/
  public boolean equals(Object tocheck)
  {
    boolean result = false;
    if(tocheck instanceof DatagramAddress)
    {
      DatagramAddress other = (DatagramAddress)tocheck;
      if (targetPort == other.getPort())
      {
        InetAddress otheraddress = other.getAddress();
        if((targetAddress.getHostAddress()).equals(otheraddress.getHostAddress()) )
          result = true;
      }
    }
    return result;
  }




//end of class DatagramAddress
}
