/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/******************************************************************************************************************************************
 This file contains the default setup framework for running the TCP/IP server and client help classes. It will be used to test the
 server, replysocket and listenersocket threads.
 This framework will be extended into a series of test classes to deal with the Socket and ServerSocket classes. At present,use it
 as a quick and dirty reference to the testing classes
*/

package gnu.testlet.wonka.net;

import java.io.*;
import java.net.*;

import gnu.testlet.Testlet;
import gnu.testlet.TestHarness;

/// default test framework
public class LocalSocketTest implements Testlet
{
  protected static TestHarness harness;


  //static in-class listener socket, same host server in different thread. Port number = 4000
  private void testLocalDatagramHandler()
  {
    int onboardport = 40001;
    DatagramSocket onboardsocket;
    int remoteport= 40002;
    LocalDatagramHandler remotehandler;

    DatagramPacket inpacket;
    byte[] inbuffer;
    DatagramPacket outpacket;
    byte[] outbuffer;

    String expected;
    String received;

    try
    {
      harness.checkPoint("Constructing LocalDatagramHandler");
      harness.verbose("    ==> building on-board socket");
      onboardsocket = new DatagramSocket(onboardport);
      harness.check(onboardport, onboardsocket.getLocalPort(),"constructed DatagramSocket on specified port" );

      harness.verbose("    ==> building external socket handler");
      remotehandler = new LocalDatagramHandler(remoteport, "local datagram simulation handler", harness);
      harness.check(remoteport, remotehandler.getSocketPort(),"constructed DatagramSocket on specified port" );
      remotehandler.setTargetPort(onboardport);
      harness.check(onboardport, remotehandler.getTargetPort(),"Specified target port" );


      /**********************************************/

      // sending from socket handler to host
      harness.checkPoint("testing ExternalDatagramHandler send functions: connection data and quit commands");
      // building receiving bufer and datagram packet
      expected = remotehandler.getConnectionData();
      inbuffer = new byte[255];
      inpacket = new DatagramPacket(inbuffer, inbuffer.length);

      //sending data and receiving packet
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+NETDEFS.toString(inpacket.getData(),inpacket.getLength())+">, length "+inpacket.getLength());
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength());
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected);

      //with cropped packet
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+NETDEFS.toString(inpacket.getData(),inpacket.getLength()+">, length "+inpacket.getLength());
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength());
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected.substring(0,received.length()));

      // quit-message
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendQuitMessage();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+NETDEFS.toString(inpacket.getData(),inpacket.getLength()+">, length "+inpacket.getLength());
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength());
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, NETDEFS.QUITSTRING );




      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:delayed send-messages");
      //sending immediately and with delay
      remotehandler.sendMessage("message1: directly");
      remotehandler.sendMessageDelayed("message2: delay 1 second" ,1000L);
      remotehandler.sendMessageDelayed("message3: delay 3 seconds",3000L);
      remotehandler.sendMessageDelayed("message4: delay 2 seconds",2000L);
      remotehandler.sendMessage("message5: directly");
      harness.verbose(" 5 mesages sent with delay up to 3 seconds");

      //message 1
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength());
      harness.verbose("Received message (1st) : "+received);
      harness.check(received, remotehandler.buildSendMessage("message1: directly") );

      //message5
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength());
      harness.verbose("Received message (2nd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message5: directly") );

      //message2
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData() , inpacket.getLength() );
      harness.verbose("Received message (3rd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message2: delay 1 second") );

      //message4
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received message (4th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message4: delay 2 seconds") );

      //message3
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received message (5th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message3: delay 3 seconds") );



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:mirror-messages");
      //mirror once
      remotehandler.replyMessage(true); //verbose on
      outbuffer = NETDEFS.stringToBytes("mirrortest1");
      onboardsocket.send(new DatagramPacket(outbuffer, outbuffer.length, InetAddress.getLocalHost(), remoteport ) );
      //expected = "mirrortest1";
      //onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), InetAddress.getLocalHost(), remoteport ) );
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest1"));

      outbuffer = NETDEFS.stringToBytes("mirrortest2");
      onboardsocket.send(new DatagramPacket(outbuffer, outbuffer.length, InetAddress.getLocalHost(), remoteport ) );
      remotehandler.replyMessagesContinuously(true); //verbose on
      outbuffer = NETDEFS.stringToBytes("mirrortest3");
      onboardsocket.send(new DatagramPacket(outbuffer, outbuffer.length, InetAddress.getLocalHost(), remoteport ) );
      outbuffer = NETDEFS.stringToBytes("mirrortest4");
      onboardsocket.send(new DatagramPacket(outbuffer, outbuffer.length, InetAddress.getLocalHost(), remoteport ) );

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest2"));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest3"));

      outbuffer = NETDEFS.QUITSTRINGBYTES;
      onboardsocket.send(new DatagramPacket(outbuffer, outbuffer.length, InetAddress.getLocalHost(), remoteport ) );

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest4"));

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[200], 200) );
      received = NETDEFS.bytesToString(inpacket.getData(), inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("Quit"));



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:awaitMessage");
      // wrap host socket into handler, so we can order messages to be sent x seconds into the future
      LocalDatagramHandler onboardhandler = new LocalDatagramHandler(onboardsocket,"on-board handler",harness);
      onboardhandler.setTargetPort(remoteport);
      harness.verbose("constructed onboard socket handler ");
      //test 1: direct call on receiver without timeout
      onboardhandler.sendMessage("receivetest1");
      harness.verbose("waiting for message already sent");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest1"));
      //test 2 : delayed call on receiver without timeout
      onboardhandler.sendMessageDelayed("receivetest2", 1000L); //wait one second before sending
      harness.verbose("waiting for message to be sent 1 second from now");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest2"));

      harness.checkPoint("testing ExternalSocketHandler:receiveMessage(timeout)");
      //timeout test 3 : calling timeout with imediate reply;
      onboardhandler.sendMessage("receivetest3");
      harness.verbose("waiting for message already sent with timeout listener");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest3"));
     //timeout test 4; calling timeout, letting expire
      harness.verbose("timeout listener waiting for message non sent => will timeout after 1 second");//for 1000 millis ");
      received = remotehandler.receiveMessage(1000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessageDelayed("receivetest5",2000L);
      harness.verbose("waiting for message to be sent in future, but after listener timeout");
      received = remotehandler.receiveMessage(1000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //catching message sent
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest5"));
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessageDelayed("receivetest6",500L);
      harness.verbose("waiting for message to be sent in future, but within timeout limits");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE);
      harness.check(received, onboardhandler.buildSendMessage("receivetest6"));



      /**********************************************/

      //closing down
      harness.verbose("Closing down handler and socket");
      //handler
      remotehandler.close();
      //socket
      onboardsocket.close();

    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }
  }


      /**********************************************/
      /**********************************************/
      /**********************************************/
  private void testStringToBytes(String text)
  {
      harness.verbose("testing : <"+text+">");
      text = NETDEFS.unicodeString(text); //sm<euh>rebr<euh>d
      harness.verbose("to unicode:"+text);
      byte[] testbuffer = NETDEFS.stringToBytes(text);
      harness.check(testbuffer.length,text.length()*2,"Testbuffer length");
      harness.verbose("to bytes : "+NETDEFS.toString(testbuffer) );
      harness.check(NETDEFS.bytesToString(testbuffer), text, "Decoding bytes to String");
  }
      /**********************************************/
      /**********************************************/
      /**********************************************/


  public void test (TestHarness newharness)
	{
		harness = newharness;
		harness.setclass("Testing the Net external handler classes");
		
		testLocalDatagramHandler();
/*
		testStringToBytes("1.2..3... test");
		testStringToBytes("sm/u00f6rebr/u00f6d");
		testStringToBytes("/u0395/u039B/u039B/u0391/u03A3");//E L L A S
		testStringToBytes("/u3070/u3093/u3056/u3044");//<ba><n><za><i>
*/
	}

}
