/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



/******************************************************************************************************************************************
 This file contains the helper classes needed to run TCP/IP tests in a mauve framework.
 In brief, it contains the classes:
 ThreadServer: runs a server in a separate thread. The server awaits ONE socket call and handles the connection
 ReplyHandler: the socket handler for the thread server
 MultithreadServer: as above, but this server runns multiple connections in separate threads, so it can handle multiple calls at one time
 ThreadReplayHandler: the socket handler class for the multi-threadserver
*/

package gnu.testlet.wonka.net;

import java.io.*;
import java.net.*;

import gnu.testlet.Testlet;
import gnu.testlet.TestHarness;

/**
****************************************************************************************************************************************
*
* NETDEFS net definitions
* Not a real class, but provides a number of static definitions that can be used in other classes to make the behavior
* of a function more clear. For instance you can command handler.reply(timeout, NETDEFS.SOCKET_LEAVEOPEN, NETDEFS.VERBOSE);
* instead of reply(timeout, false, true)
*
****************************************************************************************************************************************
**/

public class NETDEFS
{
/**
  final variables for close-socket-on-exit questions: CLOSE_ON_EXIT and LEAVE_OPEN
**/
final static boolean SOCKET_CLOSE     = true;
final static boolean SOCKET_LEAVEOPEN = false;

final static boolean VERBOSE    = true;
final static boolean NOVERBOSE  = false;

// the local host
final static String LOCALHOST =  "localhost";
//the quit message for breaking out of a continuous reply loop
final static String QUITSTRING =  "Quit";
//the quit message for breaking out of a continuous reply loop in double bytes format
final static byte[] QUITSTRINGBYTES =  stringToBytes(QUITSTRING);

  static byte[] stringToBytes(String text)
  {
    byte[] buffer = new byte[text.length() *2];
    char c;
    int j=0;
    for(int i=0;i<text.length(); i++)
    {
      c=text.charAt(i);
      buffer[j++]= (byte)(c>>8);
      buffer[j++] =(byte)(c&0x00ff);
    }
    return buffer;
  }

  static String bytesToString(byte[] buffer) {return bytesToString(buffer, buffer.length); }
  static String bytesToString(byte[] buffer, int bufferlen)
  {
    StringBuffer textbuf = new StringBuffer();
    textbuf.setLength(bufferlen/2);
    int cint;
    int i=0;
    for(int j=0;j<bufferlen; j+=2)
    {
      cint = (buffer[j]<0)?0x100+buffer[j]:buffer[j];
      cint <<= 8; //*= 0x100;
      cint += (buffer[j+1]<0)?0x100+buffer[j+1]:buffer[j+1];
//System.out.println("adding character <"+buffer[j]+","+buffer[j+1]+"> ("+Integer.toHexString(cint)+'/'+(char)cint+") to pos "+i);
      textbuf.setCharAt(i++,(char)cint);
    }
    return textbuf.toString();
  }

/**
 Special help function: transform a substring "/u1234" of a given string into a /u1234 unicode char. Like this we can
 initialise strings with unicode characters from a western standard keyboard
*/
  static String unicodeString(String source)
  {
    String dest = "";
    int letter = 0;
    String unicodestring;
    char c;
    while(letter <source.length() )
    {
      c=source.charAt(letter);
      if(c != '/')
        letter++;
      else
      {
        unicodestring = source.substring(letter+2,letter+6);
        c= (char)(Integer.parseInt(unicodestring,16));
        letter +=6;
      }

      dest += c;
    }

    return dest;
  }

  static String toString(byte[] buffer) {return toString(buffer, buffer.length); }
  static String toString(byte[] buffer, int bufferlen)
  {
    String bytestring = new String();
    for(int i=0; i<bufferlen; i++)
    {
      if(buffer[i] == 0)
        bytestring = bytestring + ",  " ;
      else if(buffer[i] < 0)
        bytestring = bytestring +", 0x"+Integer.toHexString(0x100+buffer[i]);
      else if(buffer[i] < 0x1f ) //controll character
        bytestring = bytestring + ", 0x" + Integer.toHexString(buffer[i]);
      else
        bytestring = bytestring + ", <" + (char)buffer[i]+">";
    }
    bytestring = "["+bytestring.substring(2) +"]";
    return bytestring;
  }
}
