/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/



/******************************************************************************************************************************************
 This file contains the default setup framework for running the TCP/IP server and client help classes. It will be used to test the
 server, replysocket and listenersocket threads.
 This framework will be extended into a series of test classes to deal with the Socket and ServerSocket classes. At present,use it
 as a quick and dirty reference to the testing classes
*/

package gnu.testlet.wonka.net;

import java.io.*;
import java.net.*;

import gnu.testlet.Testlet;
import gnu.testlet.TestHarness;

/// default test framework
public class TCPIPSockets implements Testlet
{
  protected static TestHarness harness;

  //static in-class listener socket, same host server in different thread. Port number = 4000
  private void clientHandlerTest()
  {
      String localhost = "localhost";
      int serverport = 40000;
      ServerSocket listener = null;

      ExternalSocketHandler clienthandler = null;
      Socket hostsocket = null;
      PrintWriter hostwriter = null;
      BufferedReader hostreader = null;

      String receiveddata;

    try
    {
      harness.verbose("building local server");
      listener = new ServerSocket(serverport);

      // huilding client ExternalSocketHandler
      harness.checkPoint("testing ExternalSocketHandler:construction from name and port");
      clienthandler = new ExternalSocketHandler(localhost,serverport,"test client handler", harness);
      harness.verbose("Constructed : "+clienthandler);//(clienthandler.toString() );
      // awaiting connection
      hostsocket = listener.accept();
      harness.verbose("Dispatched connection to host : "+hostsocket);//(hostsocket.toString() );
      hostwriter = new PrintWriter(hostsocket.getOutputStream(), true);
      hostreader = new BufferedReader( new InputStreamReader(hostsocket.getInputStream() ) );
      harness.verbose("host readers opened");//(hostsocket.toString() );
      // ( ALWAYS open the writer before the reader, otherwise the socket can deadlock )

      // sending from socket handler to host
      harness.checkPoint("testing ExternalSocketHandler:send-messages");
      //connection data : testing more then one send-command
      clienthandler.sendConnectionData();
      receiveddata = hostreader.readLine();
      harness.verbose("Received connection data : "+receiveddata);
      harness.check(receiveddata,clienthandler.getConnectionData() );
      //connection data (2): testing more then one send-command
      clienthandler.sendConnectionData();
      receiveddata = hostreader.readLine();
      harness.verbose("Received connection data (2): "+receiveddata);
      harness.check(receiveddata,clienthandler.getConnectionData() );
      // quit-message
      clienthandler.sendQuitMessage();
      receiveddata = hostreader.readLine();
      harness.verbose("Received message : "+receiveddata);
      harness.check(receiveddata,"Quit" );


      harness.checkPoint("testing ExternalSocketHandler:send-messages");
      //sending immediately and with delay
      clienthandler.sendMessage("message1: directly");
      clienthandler.sendMessage("message2: delay 1 second" ,1000);
      clienthandler.sendMessage("message3: delay 3 seconds",3000);
      clienthandler.sendMessage("message4: delay 2 seconds",2000);
      clienthandler.sendMessage("message5: directly");
      harness.verbose(" 5 mesages sent with delay up to 3 seconds");

      //message 1
      receiveddata = hostreader.readLine();
      harness.verbose("Received message (1st) : "+receiveddata);
      harness.check(receiveddata,clienthandler.buildSendMessage("message1: directly") );
      //message5
      receiveddata = hostreader.readLine();
      harness.verbose("Received message (2nd) : "+receiveddata);
      harness.check(receiveddata,clienthandler.buildSendMessage("message5: directly") );
      //message2
      receiveddata = hostreader.readLine();
      harness.verbose("Received message (3rd) : "+receiveddata);
      harness.check(receiveddata,clienthandler.buildSendMessage("message2: delay 1 second") );
      //message4
      receiveddata = hostreader.readLine();
      harness.verbose("Received message (4th) : "+receiveddata);
      harness.check(receiveddata,clienthandler.buildSendMessage("message4: delay 2 seconds") );
      //message3
      receiveddata = hostreader.readLine();
      harness.verbose("Received message (5th) : "+receiveddata);
      harness.check(receiveddata,clienthandler.buildSendMessage("message3: delay 3 seconds") );


      harness.checkPoint("testing ExternalSocketHandler:mirror-messages");
      //sending immediately and with delay
      clienthandler.replyMessage(true); //verbose on
      hostwriter.println("mirrortest1");
      receiveddata = hostreader.readLine();
      harness.verbose("Received message "+receiveddata);
      harness.check(receiveddata, clienthandler.buildReplyMessage("mirrortest1"));

      hostwriter.println("mirrortest2");
      clienthandler.replyMessagesContinuously(true); //verbose on
      hostwriter.println("mirrortest3");
      hostwriter.println("mirrortest4");
      receiveddata = hostreader.readLine();
      harness.check(receiveddata, clienthandler.buildReplyMessage("mirrortest2"));
      receiveddata = hostreader.readLine();
      harness.check(receiveddata, clienthandler.buildReplyMessage("mirrortest3"));
      receiveddata = hostreader.readLine();
      harness.check(receiveddata, clienthandler.buildReplyMessage("mirrortest4"));
      hostwriter.println("Quit");
      receiveddata = hostreader.readLine();
      harness.check(receiveddata, clienthandler.buildReplyMessage("Quit") );


      harness.checkPoint("testing ExternalSocketHandler:receiveMessage");
      // wrap host socket into handler, so we can order messages to be sent x seconds into the future
      ExternalSocketHandler hosthandler = new ExternalSocketHandler(hostsocket,"host handler",harness);
      //test 1: direct call on receiver without timeout
      hostwriter.println("receivetest1");
      harness.verbose("waiting for message already sent");
      receiveddata = clienthandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(receiveddata, "receivetest1");
      //test 2 : delayed call on receiver without timeout
      hosthandler.sendMessage("receivetest2",1000); //wait one second before sending
      harness.verbose("waiting for message to be sent 1 second from now");
      receiveddata = clienthandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(receiveddata, hosthandler.buildSendMessage("receivetest2"));

      harness.checkPoint("testing ExternalSocketHandler:receiveMessage(timeout)");
      //timeout test 3 : calling timeout with imediate reply;
      hostwriter.println("receivetest3");
      harness.verbose("waiting for message already sent with timeout listener");
      receiveddata = clienthandler.receiveMessage(3000,NETDEFS.VERBOSE); //verbose on
      harness.check(receiveddata, "receivetest3");
     //timeout test 4; calling timeout, letting expire
      harness.verbose("timeout listener waiting for message non sent => will timeout after 3 seconds");//for 1000 millis ");
      receiveddata = clienthandler.receiveMessage(3000,NETDEFS.VERBOSE);
      harness.check(receiveddata, null);
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      hosthandler.sendMessage("receivetest5",3000);
      harness.verbose("waiting for message to be sent in future, but after listener timeout");
      receiveddata = clienthandler.receiveMessage(2000,NETDEFS.VERBOSE);
      harness.check(receiveddata, null);
      //catching message sent
      receiveddata = clienthandler.awaitMessage(); //verbose on
      harness.check(receiveddata, hosthandler.buildSendMessage("receivetest5"));
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      hosthandler.sendMessage("receivetest6",500);
      harness.verbose("waiting for message to be sent in future, but within timeout limits");
      receiveddata = clienthandler.receiveMessage(3000,NETDEFS.VERBOSE);
      harness.check(receiveddata, hosthandler.buildSendMessage("receivetest6"));


      //
      //closing down
      harness.verbose("Closing down streams, handler, socket, server");
      //streams
      hostreader.close();
      hostwriter.close();
      //handler
      clienthandler.close();
      //socket
      hostsocket.close();
      //server
      listener.close();

    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }
  }


  private void threadServerTest()
  {
      String localhost = "localhost";
      int serverport = 40001;
      ExternalSocketServer externalserver = null;

      Socket clientsocket1 = null;
      ExternalSocketHandler hosthandler1 = null;
      PrintWriter clientwriter1 = null;
      BufferedReader clientreader1 = null;

      String receiveddata;

    try
    {
      //build the testing thread server
      harness.verbose("building thread server");
      externalserver = new ExternalSocketServer(serverport, "External thread server 1", harness, true); // send
      harness.verbose("thread server built");
      externalserver.start();
      harness.verbose("thread server started");
      harness.check(externalserver.getNumberOfHandlers(),0); //no handlers yet

      harness.checkPoint("testing ExternalThreadServer:client accept with message");
      clientsocket1 = new Socket(localhost, serverport);
      clientwriter1 = new PrintWriter(clientsocket1.getOutputStream(), true);
      clientreader1 = new BufferedReader( new InputStreamReader(clientsocket1.getInputStream() ) );
      harness.verbose("client socket and readers opened");//(hostsocket.toString() );
      // ( ALWAYS open the writer before the reader, otherwise the socket can deadlock )
      //checking autosent connection data

      harness.verbose(">> waiting for connection data");
      receiveddata = clientreader1.readLine();
      harness.verbose(">> Received connection data : "+receiveddata);

      //now check if the threadserver accepted the client and sent a message
      harness.verbose(">> checking assigned host handler");
      harness.check(externalserver.getNumberOfHandlers(),1); //one client handler
      hosthandler1 = externalserver.getLastHandler();

      harness.verbose(">> checking received data against server");
      harness.check(receiveddata, hosthandler1.getConnectionData() );

    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }

    try
    {
      //closing down
      harness.verbose(">> closing down streams, socket and server");
      clientreader1.close();
      clientwriter1.close();
      clientsocket1.close();
      externalserver.close();
    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }
  }

  //static in-class listener socket, same host server in different thread. Port number = 4000
  private void datagramHandlerTest()
  {
    DatagramAddress onboardaddress;
    DatagramSocket onboardsocket;
    DatagramAddress remoteaddress;
    ExternalDatagramHandler remotehandler;

    DatagramPacket inpacket;
    byte[] inbuffer;
    DatagramPacket outpacket;
    byte[] outbuffer;

    String expected;
    String received;

    try
    {
      harness.verbose("    ==> building on-board socket");
      onboardsocket = new DatagramSocket(40001);
      onboardaddress = new DatagramAddress(onboardsocket);
      harness.verbose("constructed DatagramSocket address : " + onboardaddress );

      harness.verbose("    ==> building external socket handler");
      remotehandler = new ExternalDatagramHandler(40002, "test handler", harness, onboardsocket);
      harness.verbose("constructed "+remotehandler);
      remoteaddress = new DatagramAddress(remotehandler);
      harness.verbose("address : " + remoteaddress );
      harness.verbose("default connection : " +remotehandler.getDefaultTarget() );

      harness.check(onboardaddress, remotehandler.getDefaultTarget() );


      /**********************************************/

      // sending from socket handler to host
      harness.checkPoint("testing ExternalDatagramHandler send functions: connection data and quit commands");
      // building receiving bufer and datagram packet
      expected = remotehandler.getConnectionData();
      inbuffer = new byte[255];
      inpacket = new DatagramPacket(inbuffer, inbuffer.length);

      //sending data and receiving packet
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected);

      //with cropped packet
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected.substring(0,received.length()));

      // quit-message
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendQuitMessage();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, NETDEFS.QUITSTRING );


      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler send functions: string to byte");

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);

      String testsingles = "12345ABCED"; //10 chars
      harness.verbose("    =>  sending unicode one-byte string "+testsingles);
      remotehandler.sendMessage(testsingles);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testsingles) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testdoubles = unicodeString("sm/u00f6rebr/u00f6d"); //9 chars
      // 2-byte unicode but representation as wide 255 char ascii possibge
      harness.verbose("    =>  sending unicode two-byte string "+testdoubles);
      remotehandler.sendMessage(testdoubles);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testdoubles) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testdoubles2 = unicodeString("/u0395/u039B/u039B/u0391/u03A3 - hellas"); //14 chars
      harness.verbose("    =>  sending unicode two-byte string "+testdoubles2);
      remotehandler.sendMessage(testdoubles2);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testdoubles2) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testtriples = unicodeString("/u3070-BA /u3093-N /u3056-ZA /u3044-I"); //16 chars
      harness.verbose("    =>  sending unicode three-byte string "+testtriples);
      harness.verbose("    =>  byte representation "+byteString(testtriples.getBytes()));
      remotehandler.sendMessage(testtriples);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testtriples) );


      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:delayed send-messages");
      //sending immediately and with delay
      remotehandler.sendMessage("message1: directly");
      remotehandler.sendMessage("message2: delay 1 second" ,1000);
      remotehandler.sendMessage("message3: delay 3 seconds",3000);
      remotehandler.sendMessage("message4: delay 2 seconds",2000);
      remotehandler.sendMessage("message5: directly");
      harness.verbose(" 5 mesages sent with delay up to 3 seconds");

      //message 1
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (1st) : "+received);
      harness.check(received, remotehandler.buildSendMessage("message1: directly") );
      //message5
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (2nd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message5: directly") );
      //message2
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (3rd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message2: delay 1 second") );
      //message4
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (4th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message4: delay 2 seconds") );
      //message3
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (5th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message3: delay 3 seconds") );



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:mirror-messages");
      //mirror once
      remotehandler.replyMessage(true); //verbose on
      expected = "mirrortest1";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage(expected));

      expected="mirrortest2";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      remotehandler.replyMessagesContinuously(true); //verbose on
      expected="mirrortest3";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      expected="mirrortest4";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest2"));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest3"));

      expected="Quit";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest4"));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("Quit"));



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:awaitMessage");
      // wrap host socket into handler, so we can order messages to be sent x seconds into the future
      ExternalDatagramHandler onboardhandler = new ExternalDatagramHandler(onboardsocket,"on-board handler",harness, remoteaddress);
      harness.verbose("constructed onboard socket handler ");
      //test 1: direct call on receiver without timeout
      onboardhandler.sendMessage("receivetest1");
      harness.verbose("waiting for message already sent");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest1"));
      //test 2 : delayed call on receiver without timeout
      onboardhandler.sendMessage("receivetest2", 1000); //wait one second before sending
      harness.verbose("waiting for message to be sent 1 second from now");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest2"));

      harness.checkPoint("testing ExternalSocketHandler:receiveMessage(timeout)");
      //timeout test 3 : calling timeout with imediate reply;
      onboardhandler.sendMessage("receivetest3", 1000); //wait one second before sending
      harness.verbose("waiting for message already sent with timeout listener");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest3"));
     //timeout test 4; calling timeout, letting expire
      harness.verbose("timeout listener waiting for message non sent => will timeout after 3 seconds");//for 1000 millis ");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessage("receivetest5",3000);
      harness.verbose("waiting for message to be sent in future, but after listener timeout");
      received = remotehandler.receiveMessage(2000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //catching message sent
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest5"));
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessage("receivetest6",500);
      harness.verbose("waiting for message to be sent in future, but within timeout limits");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE);
      harness.check(received, onboardhandler.buildSendMessage("receivetest6"));



      /**********************************************/

      //closing down
      harness.verbose("Closing down handler and socket");
      //handler
      remotehandler.close();
      //socket
      onboardsocket.close();

    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }
  }

  //static in-class listener socket, same host server in different thread. Port number = 4000
  private void basicMulticastHandlerTest()
  {
    DatagramAddress onboardaddress;
    MulticastSocket onboardsocket;
    DatagramAddress remoteaddress;
    ExternalMulticastHandler remotehandler;

    DatagramPacket inpacket;
    byte[] inbuffer;
    DatagramPacket outpacket;
    byte[] outbuffer;

    String expected;
    String received;

    try
    {
      harness.verbose("    ==> building on-board socket");
      onboardsocket = new MulticastSocket(40001);
      onboardaddress = new DatagramAddress(onboardsocket);
      harness.verbose("constructed DatagramSocket address : " + onboardaddress );

      harness.verbose("    ==> building external socket handler");
      remotehandler = new ExternalMulticastHandler(40002, "test handler", harness, onboardsocket);
      harness.verbose("constructed multicastHandler");
      harness.verbose("string representation "+remotehandler.toString());
      remoteaddress = new DatagramAddress(remotehandler);
      harness.verbose("address : " + remoteaddress );
      harness.verbose("default connection : " +remotehandler.getDefaultTarget() );

      harness.check(onboardaddress, remotehandler.getDefaultTarget() );


      /**********************************************/

      // sending from socket handler to host
      harness.checkPoint("testing ExternalDatagramHandler send functions: connection data and quit commands");
      // building receiving bufer and datagram packet
      expected = remotehandler.getConnectionData();
      harness.verbose("Expected datagrampacket <"+expected+">");

      inbuffer = new byte[255];
      inpacket = new DatagramPacket(inbuffer, inbuffer.length);

      //sending data and receiving packet
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected);

      //with cropped packet
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendConnectionData();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, expected.substring(0,received.length()));

      // quit-message
      inbuffer = new byte[20];
      inpacket = new DatagramPacket(inbuffer, 20);
      remotehandler.sendQuitMessage();
      onboardsocket.receive(inpacket);
      harness.verbose("Received datagrampacket <"+inpacket+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received data <"+received+"> length : "+received.length());
      harness.check(received, NETDEFS.QUITSTRING );



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler send functions: string to byte");

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);

      String testsingles = "12345ABCED"; //10 chars
      harness.verbose("    =>  sending unicode one-byte string "+testsingles);
      remotehandler.sendMessage(testsingles);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testsingles) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testdoubles = unicodeString("sm/u00f6rebr/u00f6d"); //9 chars
      // 2-byte unicode but representation as wide 255 char ascii possibge
      harness.verbose("    =>  sending unicode two-byte string "+testdoubles);
      remotehandler.sendMessage(testdoubles);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testdoubles) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testdoubles2 = unicodeString("/u0395/u039B/u039B/u0391/u03A3 - hellas"); //14 chars
      harness.verbose("    =>  sending unicode two-byte string "+testdoubles2);
      remotehandler.sendMessage(testdoubles2);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testdoubles2) );

      inbuffer = new byte[100];
      inpacket = new DatagramPacket(inbuffer, 100);
      String testtriples = unicodeString("/u3070-BA /u3093-N /u3056-ZA /u3044-I"); //16 chars
      harness.verbose("    =>  sending unicode three-byte string "+testtriples);
      harness.verbose("    =>  byte representation "+byteString(testtriples.getBytes()));
      remotehandler.sendMessage(testtriples);
      onboardsocket.receive(inpacket);
      harness.verbose(" ***   Received datagrampacket <"+byteString(inpacket)+">, length "+inpacket.getLength());
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose(" ***   Received data <"+received+"> length : "+received.length());
      harness.check(received, remotehandler.buildSendMessage(testtriples) );


      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:delayed send-messages");
      //sending immediately and with delay
      remotehandler.sendMessage("message1: directly");
      remotehandler.sendMessage("message2: delay 1 second" ,1000);
      remotehandler.sendMessage("message3: delay 3 seconds",3000);
      remotehandler.sendMessage("message4: delay 2 seconds",2000);
      remotehandler.sendMessage("message5: directly");
      harness.verbose(" 5 mesages sent with delay up to 3 seconds");

      //message 1
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (1st) : "+received);
      harness.check(received, remotehandler.buildSendMessage("message1: directly") );
      //message5
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (2nd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message5: directly") );
      //message2
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (3rd) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message2: delay 1 second") );
      //message4
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (4th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message4: delay 2 seconds") );
      //message3
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received message (5th) : "+received);
      harness.check(received,remotehandler.buildSendMessage("message3: delay 3 seconds") );



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:mirror-messages");
      //mirror once
      remotehandler.replyMessage(true); //verbose on
      expected = "mirrortest1";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage(expected));

      expected="mirrortest2";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      remotehandler.replyMessagesContinuously(true); //verbose on
      expected="mirrortest3";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));
      expected="mirrortest4";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest2"));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest3"));

      expected="Quit";
      onboardsocket.send(new DatagramPacket(expected.getBytes(), expected.length(), remoteaddress.getTarget(), remoteaddress.getPort()));

      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("mirrortest4"));
      onboardsocket.receive(inpacket = new DatagramPacket(new byte[100], 100) );
      received = new String(inpacket.getData(), 0, inpacket.getLength()  );
      harness.verbose("Received "+received);
      harness.check(received,remotehandler.buildReplyMessage("Quit"));



      /**********************************************/

      harness.checkPoint("testing ExternalDatagramHandler:awaitMessage");
      // wrap host socket into handler, so we can order messages to be sent x seconds into the future
      ExternalMulticastHandler onboardhandler = new ExternalMulticastHandler(onboardsocket,"on-board handler",harness, remoteaddress);
      harness.verbose("constructed onboard socket handler ");
      //test 1: direct call on receiver without timeout
      onboardhandler.sendMessage("receivetest1");
      harness.verbose("waiting for message already sent");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest1"));
      //test 2 : delayed call on receiver without timeout
      onboardhandler.sendMessage("receivetest2", 1000); //wait one second before sending
      harness.verbose("waiting for message to be sent 1 second from now");
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received,onboardhandler.buildSendMessage("receivetest2"));

      harness.checkPoint("testing ExternalSocketHandler:receiveMessage(timeout)");
      //timeout test 3 : calling timeout with imediate reply;
      onboardhandler.sendMessage("receivetest3", 1000); //wait one second before sending
      harness.verbose("waiting for message already sent with timeout listener");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest3"));
     //timeout test 4; calling timeout, letting expire
      harness.verbose("timeout listener waiting for message non sent => will timeout after 3 seconds");//for 1000 millis ");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessage("receivetest5",3000);
      harness.verbose("waiting for message to be sent in future, but after listener timeout");
      received = remotehandler.receiveMessage(2000L,NETDEFS.VERBOSE);
      harness.check(received, null);
      //catching message sent
      received = remotehandler.awaitMessage(NETDEFS.VERBOSE); //verbose on
      harness.check(received, onboardhandler.buildSendMessage("receivetest5"));
      //timeout test 6: timeout listener waiting for mesage to be sent in future, but within timeout
      onboardhandler.sendMessage("receivetest6",500);
      harness.verbose("waiting for message to be sent in future, but within timeout limits");
      received = remotehandler.receiveMessage(3000L,NETDEFS.VERBOSE);
      harness.check(received, onboardhandler.buildSendMessage("receivetest6"));

      /**********************************************/

      //
      //closing down
      harness.verbose("Closing down handler and socket");
      //handler
      remotehandler.close();
      //socket
      onboardsocket.close();

    }
    catch (Exception e)
    {
       harness.fail(e.toString() );
    }
  }

      /**********************************************/
      /**********************************************/
      /**********************************************/
	private void specificMulticastHandlerTest()
	{
    harness.checkPoint("Specific MulticastHandler tests");
    int port_ab = 40004;
    int port_c = 40005;
    int port_d = 40006;
    String multicast_a  = "225.0.0.0";//"224.0.0.1";
    String multicast_bc = "226.0.0.0";//"224.0.0.2";

      /**********************************************/
    harness.verbose("Building Datagram adresses ");
    DatagramAddress group_a = new DatagramAddress(multicast_a,  port_ab, harness);
    DatagramAddress group_b = new DatagramAddress(multicast_bc, port_ab, harness);
    DatagramAddress group_c = new DatagramAddress(multicast_bc, port_c , harness);

    harness.verbose("Building Multicast Handlers ");
    ExternalMulticastHandler handler_ab = new ExternalMulticastHandler(port_ab,"Handler ab",harness);
    ExternalDatagramHandler externalsender = new ExternalDatagramHandler(port_d,"offboard sender", harness);

    String expected;
    String received;
      /**********************************************/
    harness.checkPoint("Adding adresses to MulticastHandlers newsgroups (group ab)");
    harness.check(handler_ab.getMailgroupsSize(),  0);
    harness.check(handler_ab.subscribe(group_a),   1);
    harness.check(handler_ab.subscribe(group_b),   2);
    harness.check(handler_ab.subscribe(group_a),  -1, "already subscribed");
    harness.check(handler_ab.subscribe(group_c),  -1, "wrong port");
    harness.check(handler_ab.unSubscribe(group_b), 1, "unsubscribing");
    harness.check(handler_ab.unSubscribe(group_b),-1, "already unsubscribed");

    harness.check(handler_ab.getLastMailgroup(),group_a,"Last mailgroup of a");
    harness.check(handler_ab.isSubscribed(group_a),"handler a containing group a");
    harness.check(handler_ab.findMailgroup(group_b), -1, "handler a containing group b");

    harness.verbose("handler_ab mailgroup list : "+handler_ab.getMailgroupsList());

      /**********************************************/


      /**********************************************/
      harness.checkPoint("sending to multicast adress");
      // send a simple datagram message to group a: handlers a and ab shold receive it.
      expected = "from onboard to group a(subscribed)";
      externalsender.sendMessage(expected,group_a);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, externalsender.buildSendMessage(expected),"(caught by handler ab)");

      expected = "from onboard to group b(not subscribed)";
      externalsender.sendMessage(expected,group_b);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(by handler ab from unsubscribed group b)");

      //subscribed to group b
      harness.check(handler_ab.subscribe(group_b), 2);
      expected = "from onboard to group b(subscribed)";
      externalsender.sendMessage(expected,group_b);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, externalsender.buildSendMessage(expected),"(by handler ab from subscribed group b)");

      //group c: right address, wrong port
      expected = "from onboard to group c(wrong port)";
      externalsender.sendMessage(expected,group_c);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(by handler ab wrong port)");

      //unsubscribe group a
      harness.check(handler_ab.unSubscribe(group_a), 1);
      expected = "from onboard to group a(unsubscribed)";
      externalsender.sendMessage(expected,group_a);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(by handler ab from unsubscribed group a)");

      handler_ab.unSubscribeAll();
      expected = "from onboard to group b(unsubscribed)";
      externalsender.sendMessage(expected,group_b);
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(by handler ab from unsubscribed group a)");


      /**********************************************/
/*
      // let handler c send a message to group a: handlers a and ab shold receive it.
      expected = "to group b";
      handler_c.sendMessage(expected,group_b);
      received = handler_a.receiveMessage(1000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(caught by handler a)");
      received = handler_b.receiveMessage(1000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, handler_c.buildSendMessage(expected),"(caught by handler b)");
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, handler_c.buildSendMessage(expected),"(caught by handler ab)");
      received = handler_c.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(caught by handler c)");

      /**********************************************/
/*
      // let handler a send a message to group a: handlers a and ab shold receive it.
      expected = "to group c";
      handler_c.sendMessage(expected,group_c);
      received = handler_a.receiveMessage(1000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received,null,"(caught by handler a)");
      received = handler_b.receiveMessage(1000L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(caught by handler b)");
      received = handler_ab.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, null,"(caught by handler ab)");
      received = handler_c.receiveMessage(500L,NETDEFS.VERBOSE); //verbose on
      harness.check(received, handler_c.buildSendMessage(expected),"(caught by handler c)");

      /**********************************************/

      /**********************************************/
    externalsender.close();
    handler_ab.close();
	}
/********************************************************************************************************/
/**
 Special help function: transform a substring "/u1234" of a given string into a /u1234 unicode char. Like this we can
 initialise strings with unicode characters from a western standard keyboard
*/
  private String unicodeString(String source)
  {
    String dest = "";
    int letter = 0;
    String unicodestring;
    char c;
    while(letter <source.length() )
    {
      c=source.charAt(letter);
      if(c != '/')
        letter++;
      else
      {
        unicodestring = source.substring(letter+2,letter+6);
        c= (char)(Integer.parseInt(unicodestring,16));
        letter +=6;
      }

      dest += c;
    }

    return dest;
  }

  private String byteString(DatagramPacket source) { return byteString(source.getData(),source.getLength());}
  private String byteString(byte[] source) { return byteString(source, source.length);}
  private String byteString(byte[] source, int size)
  {
    String bytestring = "";
    for(int i=0; i<size; i++)
    {
      if(source[i]>0 && source[i]<=0x7f )
        bytestring = bytestring  + ", " + (char)source[i];
      else
        bytestring = bytestring +", 0x"+Integer.toHexString(0x7f-source[i]);
    }
    bytestring = bytestring.substring(2) +"]";
    return bytestring;
  }

  public void test (TestHarness newharness)
	{
		harness = newharness;
		harness.setclass("Testing the Net external handler classes");
		
		clientHandlerTest();
		threadServerTest();
		datagramHandlerTest();
		basicMulticastHandlerTest();
		specificMulticastHandlerTest();
	}

}
