###########################################################################
# Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 #
#                                                                         #
# This software is copyrighted by and is the sole property of Acunia N.V. #
# and its licensors, if any. All rights, title, ownership, or other       #
# interests in the software remain the property of Acunia N.V. and its    #
# licensors, if any.                                                      #
#                                                                         #
# This software may only be used in accordance with the corresponding     #
# license agreement. Any unauthorized use, duplication, transmission,     #
#  distribution or disclosure of this software is expressly forbidden.    #
#                                                                         #
# This Copyright notice may not be removed or modified without prior      #
# written consent of Acunia N.V.                                          #
#                                                                         #
# Acunia N.V. reserves the right to modify this software without notice.  #
#                                                                         #
#   Acunia N.V.                                                           #
#   Vanden Tymplestraat 35      info@acunia.com                           #
#   3000 Leuven                 http://www.acunia.com                     #
#   Belgium - EUROPE                                                      #
###########################################################################

#!/usr/bin/awk -f
# $Id: loadlib.awk 2532 2002-08-14 11:09:08Z vandeneede $

# The Wonka kernel is software copyright by ACUNIA NV (1999-2001)
# Please see the file Copyright for information on its legal use.

# This script takes the output of `nm' and converts it into a C function
# which will calls registerJNImethod for every JNI-looking function it finds.
# The function will be called loadlib_<libname>, where <libname> is the name
# of the input file minus its extension and any leading "lib": e.g.
#  nm -s ../bin/x86/lib/libtestjni.a > libtestjni.nm
#  awk -f loadlib.awk libtestjni.nm > ../wonka/vm/load-libraries.c
# will create a function loadlib_testjni(...).
#
# The script generates none such function for every file on the awk command
# line, followed by a function init_libraries() which will set up the
# hashtable mapping library names (e.g. "testjni") onto load_... functions.

# Save names of files in an array
FILENAME!=filename {
  filename = FILENAME
  libname = filename
  while(slash = index(libname,"/")) libname = substr(libname,slash+1)
  if (index(libname,"lib")==1) libname = substr(libname,4)
  if (dot = index(libname,".")) libname = substr(libname,1,dot-1)
  files[filename] = libname
  print "// File " filename " --> library " libname
}

# Skip everything before the line that begins "Archive:'
/^Archive/ {process=1;next}

# The process all lines until the next blank line
NF==0  {if (process) process=0}

# Only functions whose names begin with Java_ are interesting to us
/^Java_/ {
    if (!process) next

# Find the `__' separator if present, and skip over 'Java_'
# Separate class+method (mungedname) from parameters (mungedparam)
    doublewhammy = index($1,"__")
    if (doublewhammy) {
      mungedname = substr($1,6,doublewhammy-6)
      mungedparam = substr($1,doublewhammy+2)
    }
    else {
      mungedname = substr($1,6)
      mungedparam = ""
    }
#    print "// Processing " $1

# demunge (substitute escape codes)
# The last slash in slashedname separates the class from the method
    name = mungedname
    param = mungedparam

    lastunderscore = 0
    underscore = match(name,"_[^0123]")
    slashedname = substr(name,1,underscore-1)
                  while (1) {
                    lastunderscore = underscore
      underscore = underscore + match(substr(name,underscore+1),"_[^0123]")
      if (underscore==lastunderscore) {
        slashedname = slashedname "/" substr(name,lastunderscore+1)
        break;
      }
                    else slashedname = slashedname "/" substr(name,lastunderscore+1,underscore-lastunderscore-1)
    }

    mungedclass = substr(name,1,lastunderscore-1)
    mungedmethod = substr(name,lastunderscore+1)
    classname = substr(slashedname,1,lastunderscore-1)
    methodname = substr(slashedname,lastunderscore+1)
    gsub("_0","\\u",classname)
    gsub("_1","_",classname)
    gsub("_2",";",classname)
    gsub("_3","[",classname)
    gsub("_0","\\u",methodname)
    gsub("_1","_",methodname)
    gsub("_2",";",methodname)
    gsub("_3","[",methodname)

#    print "// Munged classname is " mungedclass
#    print "// Demunged classname is " classname
#    print "// Munged params are " mungedparam

    if (param) {
      mungedmethod = mungedmethod "__" param
      lastunderscore = 0
      underscore = match(param,"_[^0123]")
      if (underscore) {
        slashedparam = substr(param,1,underscore-1)
                      while (1) {
                        lastunderscore = underscore
          underscore += match(substr(param,underscore+1),"_[^0123]")
          if (underscore==lastunderscore) {
            slashedparam = slashedparam "/" substr(param,lastunderscore+1)
            break;
          }
                        else slashedparam = slashedparam "/" substr(param,lastunderscore+1,underscore-lastunderscore-1)
        }
      }
      else slashedparam = param
      gsub("_0","\\u",slashedparam)
      gsub("_1","_",slashedparam)
      gsub("_2",";",slashedparam)
      gsub("_3","[",slashedparam)
      methodname = methodname "(" slashedparam ")"
    }
#    print "// Demunged method+param name is " methodname

# Array `libclasses' is indexed by libname, and holds a comma-separated
# list of (demunged) classnames for each libname.
# Array `classes' and `methods' are both indexed by (demunged) classname.
# Each entry in `classes' is a munged classname.
    if (!classes[classname]) {
      if (libclasses[libname]) {
        libclasses[libname] = libclasses[libname] "," classname
      }
      else {
        libclasses[libname] = classname
      }
      classes[classname] = mungedclass
    }
    sofar = methods[classname]
    if (sofar) {
      methods[classname] = sofar "," methodname
    }
    else {
      methods[classname] = methodname
    }
    sofar = mungedmethods[classname]
    if (sofar) {
      mungedmethods[classname] = sofar "," mungedmethod
    }
    else {
      mungedmethods[classname] = mungedmethod
    }
  }
END {
  print " "
  print "/*"
  print "** This file is generated automatically"
  print "** Do NOT edit this file."
  print "*/"
  print " "
  print "#include \"clazz.h\""
  print "#include \"field.h\""
  print "#include \"hashtable.h\""
  print "#include \"loading.h\""
  print "#include \"method.h\""
  print "#include \"realm.h\""
  print "#include \"strings.h\""
  print "#include \"threads.h\""
  print "#include \"wonka.h\""
  print ""

  for (l in libclasses) {
    split(libclasses[l],classlist,",")
    for(i in classlist) {
      printf "#include \"%s.h\"\n",classes[classlist[i]]
    }
    print ""

    printf "/*\n";
    printf "** Load all the classes referenced by %s and link native methods\n",l;
    printf "**  %s\n",libclasses[l]
    printf "*/\n\n"

    printf "void loadlib_%s(w_thread thread, w_instance classloader, w_status *status) {\n",l;
    printf "  w_string classname;\n"
    printf "  w_instance exception = 0;\n"
    for(i in classlist) {
      c = classlist[i]
      mungedclass = classes[c]
      printf "  w_clazz clazz_%s;\n",mungedclass 
    }
    printf "\n"
    for(i in classlist) {
      c = classlist[i]
      mungedclass = classes[c]
      printf "  classname = cstring2String(thread,\"%s\",%d);\n",c,length(mungedclass)
      printf "  woempa(9,\"Loading class %w\\n\",classname);\n"
      printf "  clazz_%s = namedClassMustBeLoaded(thread,classloader,classname,&exception);\n",mungedclass 
      printf "  if (exception) {\n"
      printf "    woempa(9,\"Could not load class %s: got %%w\\n\",NM(instance2clazz(exception)));\n",c
      printf "    *status = WS_LinkageError;\n"
      printf "    return;\n"
      printf "}\n"
      printf "  woempa(9,\"Preparing class %w\\n\",classname);\n"
      printf "  exception = classMustBePrepared(clazz_%s,thread);\n",mungedclass 
      printf "  if (exception) {\n"
      printf "    woempa(9,\"Could not prepare class %s: got %%w\\n\",NM(instance2clazz(exception)));\n",c
      printf "    *status = WS_LinkageError;\n"
      printf "    return;\n"
      printf "}\n"
      split(methods[c],methodlist,",")
      split(mungedmethods[c],mungedmethodlist,",")
      for (i in methodlist) {
        printf "  woempa(9,\"Linking method %s (%s)\\n\",*status);\n",methodlist[i],mungedmethodlist[i]
        methodname = methodlist[i]
        mungedmethod = mungedmethodlist[i]
        printf "  registerJNImethod(clazz_%s, ",mungedclass
        printf "(w_fun_dec)Java_%s_%s, ", mungedclass, mungedmethod
        printf "\"%s\");\n", methodname
      }
    }
    printf("\n}\n\n");
  }

  print "void init_libraries(w_hashtable library_hashtable) {"
  for(f in files) if (libclasses[files[f]]) {
    printf "        woempa(9,\"Adding {\\\"%s\\\",%%p} to library_hashtable\\n\",loadlib_%s);\n",files[f],files[f]
    printf "        ht_write_no_lock(library_hashtable,(w_word)cstring2String(W_Thread_system,\"%s\",%d),(w_word)loadlib_%s);\n",files[f],length(files[f]),files[f]
  }
  print "}"

}

