package com.acunia.wonka.test.awt.dnd;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.dnd.*;

import java.io.*;

class DropText extends TextField implements
                DropTargetListener {
  DropText(String txt){
    super(txt);
    setDropTarget(new DropTarget(this, this));
  }

  public void dragEnter(DropTargetDragEvent dtde) {
System.out.println("DropText.dragEnter");
    dtde.acceptDrag(DnDConstants.ACTION_COPY);
  }

  public void dragOver(DropTargetDragEvent dtde) {
System.out.println("DropText.dragOver");
    dtde.acceptDrag(DnDConstants.ACTION_COPY);
  }

  public void dragExit(DropTargetEvent dte) {
System.out.println("DropText.dragExit");
    repaint();
  }

  public void drop(DropTargetDropEvent dtde) {
System.out.println("DropText.drop");
    DropTargetContext dtc = dtde.getDropTargetContext();

    boolean outcome = false;
    if ((dtde.getSourceActions() & DnDConstants.ACTION_COPY) != 0)
      dtde.acceptDrop(DnDConstants.ACTION_COPY);
    else {
      dtde.rejectDrop();
      return;
    }

    DataFlavor[] dataFlavors = dtde.getCurrentDataFlavors();
    DataFlavor   currentDataFlavor = null;

    for (int i = 0; i < dataFlavors.length; i++) {
      if (DataFlavor.plainTextFlavor.equals(dataFlavors[i])) {
        currentDataFlavor = dataFlavors[i];
        break;
      }
    }

    if (currentDataFlavor != null) {
      Transferable transf  = dtde.getTransferable();
      InputStream  is = null;

      try {
        is = (InputStream)transf.getTransferData(currentDataFlavor);
      } catch (IOException ioe) {
        ioe.printStackTrace();
        dtc.dropComplete(false);

        return;
      } catch (UnsupportedFlavorException ufe) {
        ufe.printStackTrace();
        dtc.dropComplete(false);

        repaint();
        return;
      }

      if (is != null) {
        String newText = getText();

        try {
          int length = is.available();
          byte[] string = new byte[length];
          is.read(string, 0, length);
          for (int i = 0; i < length; i++){
            if (string[i] == 0) {
              length = i;
              break;
            }
          }
          newText = new String(string, 0, length);
          outcome = true;
        } catch (Exception e) {
          e.printStackTrace();
          dtc.dropComplete(false);

          repaint();
          return;
        } finally {
          setText(newText);
        }
      } else outcome = false;
    }

    repaint();
    validate();
System.out.println("outcome = "+outcome);
    dtc.dropComplete(outcome);
  }

  public void dragScroll(DropTargetDragEvent dtde) {
System.out.println("DropText.dragScroll");
  }

  public void dropActionChanged(DropTargetDragEvent dtde) {
System.out.println("DropText.dropActionChanged");
  }

}
