/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Etc.c 4140 2004-04-05 08:18:48Z dumon $
*/

#include <stdlib.h>
#include <string.h>
#include <sys/utsname.h>
#include "clazz.h"
#include "core-classes.h"
#include "realms.h"
#include "wstrings.h"
#include "threads.h"
#include "wonka.h"
#include "exception.h"

static struct utsname uname_buffer;
static int uname_called = WONKA_FALSE;

w_instance
Etc_getOSName (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;

  if (!uname_called) {
    uname(&uname_buffer);
  }
  string = cstring2String(thread, uname_buffer.sysname, strlen(uname_buffer.sysname));
  result = newStringInstance(string);
  deregisterString(string);

  return result;
}

w_instance
Etc_getOSVersion (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;

  if (!uname_called) {
    uname(&uname_buffer);
  }
  string = cstring2String(thread, uname_buffer.release, strlen(uname_buffer.release));
  result = newStringInstance(string);
  deregisterString(string);

  return result;
}

w_instance
Etc_getOSArch (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;

  if (!uname_called) {
    uname(&uname_buffer);
  }
  string = cstring2String(thread, uname_buffer.machine, strlen(uname_buffer.machine));
  result = newStringInstance(string);
  deregisterString(string);

  return result;
}

w_instance
Etc_getUserName (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;
  w_ubyte   *bytes = getenv("USER");

  if(!bytes){
    bytes = "";
  }
  string = cstring2String(thread, bytes, strlen(bytes));
  result = newStringInstance(string);
  deregisterString(string);
  return result;

}

w_instance
Etc_getUserHome (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;
  w_ubyte   *bytes = getenv("HOME");

  if(!bytes){
    bytes = "";
  }

  string = cstring2String(thread, bytes, strlen(bytes));
  result = newStringInstance(string);
  deregisterString(string);
  return result;
}

w_instance
Etc_getUserDir (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;
  w_ubyte   *bytes = getenv("PWD");

  if(!bytes){
    bytes = "";
  }
  string = cstring2String(thread, bytes, strlen(bytes));
  result = newStringInstance(string);
  deregisterString(string);
  return result;
}

w_instance
Etc_getVersion (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;

  string = cstring2String(thread, VERSION_STRING, strlen(VERSION_STRING));
  result = newStringInstance(string);
  deregisterString(string);

  return result;
}

w_instance
Etc_getInstallationDir (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;
  w_ubyte   *bytes = BOOTCLASSDIR;
  w_size     len = strlen(BOOTCLASSDIR);

  // We report one level up from BOOTCLASSDIR
  woempa(1, "BOOTCLASSDIR is '%s' (%d chars)\n", bytes, len);
  while (len && bytes[--len] != '/') {
    woempa(1, "Byte[%d] is not '/', decrementing length\n", len);
  }

  string = cstring2String(thread, bytes, len);
  woempa(1, "Returning '%w'\n", string);
  result = newStringInstance(string);
  deregisterString(string);

  return result;

}

w_instance
Etc_getExtensionDir (JNIEnv *env, w_instance classEtc) {
  w_thread thread = JNIEnv2w_thread(env);
  w_instance result;
  w_string   string;
  w_ubyte   *bytes = EXTCLASSDIR;
  w_size     len = strlen(EXTCLASSDIR);

  woempa(7, "EXTCLASSDIR is '%s' (%d chars)\n", bytes, len);

  string = cstring2String(thread, bytes, len);
  woempa(7, "Returning '%w'\n", string);
  result = newStringInstance(string);
  deregisterString(string);

  return result;

}

void 
Etc_static_setTriggerLevel
( JNIEnv *env, w_instance classSystem,
  w_instance filenameString, w_int triggerLevel
) {
  woempa(9, "NOT FUNCTIONAL\n");
}

void 
Etc_static_setAllTriggerLevel
( JNIEnv *env, w_instance classSystem,
  w_int triggerLevel
) {
  setAllTriggerLevel(triggerLevel);
}

void
Etc_static_woempa
( JNIEnv *env, w_instance classSystem,
  w_int triggerLevel,
  w_instance theString
) {
  woempa(triggerLevel, "%w\n", String2string ( theString));
}

void Etc_static_heapCheck(JNIEnv *env, w_instance classSystem){
#ifdef DEBUG
  woempa(9, "calling heapCheck for Etc\n");
  heapCheck;
#endif //DEBUG

}

