/**************************************************************************
* Copyright (c) 2001, 2003 by Acunia N.V. All rights reserved.            *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: platform.c 4294 2006-05-10 11:35:19Z chris $
*/

#include "comms.h"
#include "wonka.h"
#include <stdio.h> /* For printf */
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <execinfo.h>
#include <setjmp.h>

void w_dump_info(void);

static struct sigaction term;
static struct sigaction quit;
static struct sigaction segv;
static struct sigaction fpe;
static struct sigaction usr1;
static struct sigaction usr2;

int wonka_killed = 0;
x_thread heartbeat_thread = NULL;
static int in_quit_handler = 0;
static int in_segv_handler = 0;
static jmp_buf jmpbuffer;

static void term_handler(int signum, siginfo_t * sis, void * arg) {
  wonka_killed = 1;
  if(heartbeat_thread) x_thread_priority_set(heartbeat_thread, 1);
}

extern int c1,c2,c3,c4;

void w_stacktraces(void) {
  void *address[1024];
  int count;
 
  w_dump("\nNative stacktrace:\n\n");

  count = backtrace((void **)&address, 1024);
  backtrace_symbols_fd((void **)&address, count, 2);

  if(x_thread_current() && x_thread_current()->xref) {
    w_dump("\nJava stacktrace:\n\n");

    w_dump_trace(x_thread_current()->xref);
  }

  w_dump("\n");
  
}

static void quit_handler(int signum, siginfo_t * sis, void * arg) {
  x_preemption_disable;
  in_quit_handler = 1;

  if(setjmp(jmpbuffer)) {
    w_dump("\nSegfault while dumping threads.. Ignoring..\n");
  }
  else {

  // w_dump("c1 %d c2 %d c3 %d c4 %d preemptive status %d\n", c1, c2, c3, c4, critical_status);
  
    // w_stacktraces();
    w_dump_info();
  }
  in_quit_handler = 0;
  x_preemption_enable;
}

static void segv_handler(int signum, siginfo_t * sis, void * arg) {
#ifdef SPAWN_DEBUGGER
  w_int pid = getpid();
  w_int child = 0;
  w_int result = -1;
#endif

  if(in_segv_handler) {
    w_dump("\nSegmentation Fault while in the segfault handler... This is just too much...\n");
    abort();
  }
  if(in_quit_handler) {
    longjmp(jmpbuffer, 1);
  }
  in_segv_handler = 1;
  w_dump("\nSegmentation Fault!\n");
  w_stacktraces();

#ifdef SPAWN_DEBUGGER

  /*
  ** Spawn the debugger
  */

  signal(SIGALRM, SIG_IGN);
  
  w_dump("Spawning the debugger.\n");
  
  if((child = fork())) {
    char *arg[4];

    w_dump("About to do it !\n");
    
    arg[0] = "/usr/bin/gdb";
    arg[1] = "/usr/local/bin/wonka";
    arg[2] = (char *)malloc(10);
    arg[3] = NULL;
    
    w_dump("%s %s %s\n", arg[0], arg[1], arg[2]);
    w_dump("the child: %d\n", child);

    sprintf(arg[2], "%d", child);

    w_dump("%s %s %s\n", arg[0], arg[1], arg[2]);

    execvp(arg[0], (char **)arg);

    w_dump("Spawning debugger failed...\n");

    _exit(127);
  }

  w_dump("Waiting for the child %d\n", pid);
  
  while(result == -1) {
    result = wait4(pid, NULL, 0, NULL); 
    if(result != pid) result = -1;
  }

  w_dump("Done running the debugger..\n");
  
#endif

  abort();
  
}

static void fpe_handler(int signum, siginfo_t * sis, void * arg) {
  w_dump("\nFPE!\n");
  w_stacktraces();
}

static void usr1_handler(int signum, siginfo_t * sis, void * arg) {
  //reportExternalMemory();
}

static void usr2_handler(int signum, siginfo_t * sis, void * arg) {
  //reportInstanceStat();
}

void install_term_handler(void) {

  w_int result;

  term.sa_sigaction = term_handler;
  sigemptyset(&term.sa_mask);
  term.sa_flags = SA_SIGINFO;
  result = sigaction(SIGTERM, &term, NULL);
  if (result == -1) {
    woempa(9, "Registering the term handler failed.\n");
    abort();
  }

  result = sigaction(SIGINT, &term, NULL);
  if (result == -1) {
    woempa(9, "Registering the int handler failed.\n");
    abort();
  }

  quit.sa_sigaction = quit_handler;
  sigemptyset(&quit.sa_mask);
  quit.sa_flags = SA_SIGINFO;
  result = sigaction(SIGQUIT, &quit, NULL);
  if (result == -1) {
    woempa(9, "Registering the quit handler failed.\n");
    abort();
  }

  segv.sa_sigaction = segv_handler;
  sigfillset(&segv.sa_mask);
  segv.sa_flags = SA_SIGINFO;
  result = sigaction(SIGSEGV, &segv, NULL);
  if (result == -1) {
    woempa(9, "Registering the segv handler failed.\n");
    abort();
  }
  
  fpe.sa_sigaction = fpe_handler;
  sigemptyset(&fpe.sa_mask);
  fpe.sa_flags = SA_SIGINFO;
  result = sigaction(SIGFPE, &fpe, NULL);
  if (result == -1) {
    woempa(9, "Registering the fpe handler failed.\n");
    abort();
  }
  
  usr1.sa_sigaction = usr1_handler;
  sigemptyset(&usr1.sa_mask);
  usr1.sa_flags = SA_SIGINFO;
  result = sigaction(SIGUSR1, &usr1, NULL);
  if (result == -1) {
    woempa(9, "Registering the USR1 handler failed.\n");
    abort();
  }
  
  usr2.sa_sigaction = usr2_handler;
  sigemptyset(&usr2.sa_mask);
  usr2.sa_flags = SA_SIGINFO;
  result = sigaction(SIGUSR2, &usr2, NULL);
  if (result == -1) {
    woempa(9, "Registering the USR2 handler failed.\n");
    abort();
  }
}

