#ifndef _CONSTANT_H
#define _CONSTANT_H

/**************************************************************************
* Copyright  (c) 2001, 2002 by Acunia N.V. All rights reserved.           *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: constant.h 3788 2003-04-02 08:18:45Z dbuytaert $
*/

#include "exception.h"
#include "wonka.h"

/*
** For more info on the structure of the constant pool, refer
** to any book on the classfile structure.
*/

#define CONSTANT_UTF8               0x01
#define CONSTANT_UNICODE            0x02
#define CONSTANT_INTEGER            0x03
#define CONSTANT_FLOAT              0x04
#define CONSTANT_LONG               0x05
#define CONSTANT_DOUBLE             0x06
#define CONSTANT_CLASS              0x07
#define CONSTANT_STRING             0x08
#define CONSTANT_FIELD              0x09
#define CONSTANT_METHOD             0x0A
#define CONSTANT_IMETHOD            0x0B
#define CONSTANT_NAME_AND_TYPE      0x0C
#define NO_VALID_ENTRY              0x0E
#define COULD_NOT_RESOLVE           0x0F

#define RESOLVING_CONSTANT          0x10
#define RESOLVED_CONSTANT           0x20

#define RESOLVING_CLASS             (CONSTANT_CLASS+RESOLVING_CONSTANT)
#define RESOLVING_STRING            (CONSTANT_STRING+RESOLVING_CONSTANT)
#define RESOLVING_FIELD             (CONSTANT_FIELD+RESOLVING_CONSTANT)
#define RESOLVING_METHOD            (CONSTANT_METHOD+RESOLVING_CONSTANT)
#define RESOLVING_IMETHOD           (CONSTANT_IMETHOD+RESOLVING_CONSTANT)

#define RESOLVED_CLASS              (CONSTANT_CLASS+RESOLVED_CONSTANT)
#define RESOLVED_STRING             (CONSTANT_STRING+RESOLVED_CONSTANT)
#define RESOLVED_FIELD              (CONSTANT_FIELD+RESOLVED_CONSTANT)
#define RESOLVED_METHOD             (CONSTANT_METHOD+RESOLVED_CONSTANT)
#define RESOLVED_IMETHOD            (CONSTANT_IMETHOD+RESOLVED_CONSTANT)

#define CONSTANT_DELETED            0xff

/*
** UTF8-encoded Unicode string.
*/
typedef struct w_Utf8 {
  w_string string;
} w_Utf8;

/*
** Format used for Field, Method and IMethod constants.
*/
typedef struct w_Member {
  u2 class_index;
  u2 name_and_type_index;
} w_Member;

/*
** A name-and-type constant contains the indices of two UTF8 constants:
** the name of the member and its descriptor.
*/
typedef struct w_Name_and_Type {
  u2 name_index;
  u2 descriptor_index;
} w_Name_and_Type;

/*
** The w_Cnt structure consists of:
** - a `cnt_tag' which indicates the type of the constant: one of the
**   CONSTANT_xxxx values defined above, or NO_VALID_ENTRY for the index
**   value which is skipped after a CONSTANT_LONG or CONTANT_DOUBLE, 
**   or COULD_NOT_RESOLVE for a constant which we already tried unsuccessfully
**   to resolve (so we should not try again).
**
** The value of cnt_tag determines the type of data stored in the corresponding
** entry of clazz->values:
**     CONSTANT_UTF8                     w_string
**     CONSTANT_UNICODE                  (not used)
**     CONSTANT_INTEGER                  w_int value
**     CONSTANT_FLOAT                    w_float value
**     CONSTANT_LONG                     w_long value
**     CONSTANT_DOUBLE                   w_double value
**     CONSTANT_CLASS                    index of a CONSTANT_UTF8
**     RESOLVED_CLASS                    w_clazz                  
**     CONSTANT_STRING                   index of a CONSTANT_UTF8
**     RESOLVED_STRING                   w_instance of java.lang.String
**     CONSTANT_FIELD                    indices of a CONSTANT_CLASS (16 bits),
**                                       CONSTANT_NAME_AND_TYPE (16 bits).
**     RESOLVED_FIELD                    w_field
**     CONSTANT_METHOD                   indices of a CONSTANT_CLASS (16 bits),
**                                       CONSTANT_NAME_AND_TYPE (16 bits).
**     RESOLVED_METHOD                   w_method
**     CONSTANT_IMETHOD                  indices of a CONSTANT_CLASS (16 bits),
**                                       CONSTANT_NAME_AND_TYPE (16 bits).
**     RESOLVED_IMETHOD                  w_method
**     CONSTANT_NAME_AND_TYPE            indices of two CONSTANT_UTF8's.
**     NO_VALID_ENTRY                    (second word of w_long/w_double)
**     COULD_NOT_RESOLVE                 w_instance of a subclass of Throwable.
**   
**
** All fields are marked as volatile because it is possible that two threads
** will compete to resolve the same constant: therefore the compiler should
** not `optimise away' consecutive accesses to these fields.
*/
typedef struct w_Cnt {
  volatile u1 cnt_tag;
} w_Cnt;

typedef volatile u1 w_ConstantType;

/*
** Function resolveUtf8Constant returns the result of resolving the i'th
** constant of clazz, which must be a UTF* constant.
*/
w_string resolveUtf8Constant(w_clazz, w_int idx);

/*
** Function resolveStringConstant resolves the i'th constant of clazz (which
** must be a String constant).
*/
void resolveStringConstant(w_clazz clazz, w_int i);

/*
** Function resolveClassConstant resolves the i'th constant of clazz (which
** must be a Class constant).
*/
void resolveClassConstant(w_clazz clazz, w_int i);

/*
** Function resolveFieldConstant resolves the i'th constant of clazz (which
** must be a Field constant).
*/
void resolveFieldConstant(w_clazz clazz, w_int i);

/*
** Function resolveMethodConstant resolves the i'th constant of clazz (which
** must be a Method constant).
*/
void resolveMethodConstant(w_clazz clazz, w_int i);

/*
** Function resolveIMethodConstant resolves the i'th constant of clazz (which
** must be a IMethod constant).
*/
void resolveIMethodConstant(w_clazz clazz, w_int i);

/*
** The functions getXXXXConstant return the value of the i'th constant of `clazz'.
*/

static inline w_int getIntegerConstant(w_clazz clazz, w_int i) {
  return (w_int)clazz->values[i];
}

static inline w_float getFloatConstant(w_clazz clazz, w_int i) {
  return (w_float)clazz->values[i];
}

static inline w_u64 getLongConstant(w_clazz clazz, w_int i) {
  w_u64 l;

  ((w_word*)&l)[WORD_MSW] = clazz->values[i];
  ((w_word*)&l)[WORD_LSW] = clazz->values[i + 1];

  return l;
}

static inline w_u64 getDoubleConstant(w_clazz clazz, w_int i) {
  w_u64 d;

  ((w_word*)&d)[WORD_MSW] = clazz->values[i];
  ((w_word*)&d)[WORD_LSW] = clazz->values[i + 1];

  return d;
}

static inline w_instance getStringConstant(w_clazz clazz, w_int i) {
  if (clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {
    resolveStringConstant(clazz, i);
  }

  return (w_instance)clazz->values[i];
}

static inline w_clazz getResolvedClassConstant(w_clazz clazz, w_int i) {
  return (w_clazz)clazz->values[i];
}

static inline w_clazz getClassConstant(w_clazz clazz, w_int i) {
  if (clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {
    resolveClassConstant(clazz, i);
  }

  if (clazz->constants[i].cnt_tag > RESOLVED_CONSTANT) {

    return (w_clazz)clazz->values[i];

  }

  if (clazz->constants[i].cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(currentWonkaThread, (w_instance)clazz->values[i]);
  }

  return NULL;
}

static inline w_field getResolvedFieldConstant(w_clazz clazz, w_int i) {
  return (w_field)clazz->values[i];
}

static inline w_field getFieldConstant(w_clazz clazz, w_int i) {
  if (clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {
    resolveFieldConstant(clazz, i);
  }

  if (clazz->constants[i].cnt_tag > RESOLVED_CONSTANT) {

    return (w_field)clazz->values[i];

  }

  if (clazz->constants[i].cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(currentWonkaThread, (w_instance)clazz->values[i]);
  }

  return NULL;
}

static inline w_method getResolvedMethodConstant(w_clazz clazz, w_int i) {
  return (w_method)clazz->values[i];
}

static inline w_method getMethodConstant(w_clazz clazz, w_int i) {
  if (!clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {
    resolveMethodConstant(clazz, i);
  }

  if (clazz->constants[i].cnt_tag > RESOLVED_CONSTANT) {

    return (w_method)clazz->values[i];

  }

  if (clazz->constants[i].cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(currentWonkaThread, (w_instance)clazz->values[i]);
  }

  return NULL;
}

static inline w_method getResolvedIMethodConstant(w_clazz clazz, w_int i) {
  return (w_method)clazz->values[i];
}

static inline w_method getIMethodConstant(w_clazz clazz, w_int i) {
  if (!clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {
    resolveIMethodConstant(clazz, i);
  }

  if (clazz->constants[i].cnt_tag > RESOLVED_CONSTANT) {

    return (w_method)clazz->values[i];

  }

  if (clazz->constants[i].cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(currentWonkaThread, (w_instance)clazz->values[i]);
  }

  return NULL;
}

void dissolveConstant(w_clazz, int idx);

void dumpPools(int fd, w_clazz clazz);

void checkUTF8References(w_clazz clazz, w_int idx);

w_int addUTF8ConstantToPool(w_clazz, w_string);

w_int addNatConstantToPool(w_clazz, w_string name, w_string type);

#endif /* _CONSTANT_H */
