#ifndef _DEBUG_H
#define _DEBUG_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: debug.h 4173 2004-06-18 12:08:10Z dumon $
*/

/*
** --- IMPORTANT NOTE ---
**
** This file is NOT meant to be included by any other file than "wonka.h".
*/

#define ABORT_INFO                    0
#define ABORT_THREAD                  1
#define ABORT_GROUP                   2
#define ABORT_ROOT_GROUP              3
#define ABORT_WONKA                   4

#ifndef DEBUG_LEVEL
#define DEBUG_LEVEL 1
#endif

#if defined (__LINT__)

#define woempa                      lint_woempa
#define wabort                      lint_wabort
#define wassert(test)               ((test) ? (void)0 : lint_wabort(ABORT_ROOT_GROUP, "Assertion failed: '%s'\n", #test))
#define ThrowLocalException         lint_ThrowLocalException 
#define inline
#define DEBUG_STATEMENT(x)          (x)

void lint_woempa(int level, const char *format, ...);
void lint_wabort(int scope, const char *format, ...);
void lint_ThrowLocalException(w_thread ad, w_clazz exception, const char *fmt, ...);

#elif defined (DEBUG)

#include <stdio.h>

#define D(n) fprintf(stderr, "DEBUG %d\n", n); fflush(stderr);
#define setTriggerLevel(t)          _setTriggerLevel(__FILE__, t)
#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
#define woempa(level, ...) if(level>=DEBUG_LEVEL)_woempa(__FILE__, __FUNCTION__, __LINE__, level, __VA_ARGS__)
#else
#define woempa(level, format, a...) if(level>=DEBUG_LEVEL)_woempa(__FILE__, __FUNCTION__, __LINE__, level, format, ##a)
#endif

#define wassert(test)               ((test) ? (void)0 : _wabort(__FUNCTION__, __LINE__, ABORT_ROOT_GROUP, "Assertion failed: '%s'\n", #test))
#define DEBUG_STATEMENT(x)          (x)
void _wabort(const char *function, int line, int scope, const char *format, ...);

#if defined(__STDC_VERSION__) && (__STDC_VERSION__ >= 199901L)
#define wabort(scope, ...)   _wabort(__FUNCTION__, __LINE__, scope, __VA_ARGS__)
#else
#define wabort(scope, format, a...)   _wabort(__FUNCTION__, __LINE__, scope, format, ##a)
#endif

//void _woempa(const char *file, const char *function, int line, int level, const char *format, ...) __attribute__ ((format (printf, 5, 6)));
void _woempa(const char *file, const char *function, int line, int level, const char *format, ...);
void _setTriggerLevel(const char *file, int trigger);

#else /* DEBUG nor __LINT__ is defined */

#define D(x)
#define setTriggerLevel(t) 
#define woempa(level, format, a...)  
#define wassert(test) 
#define DEBUG_STATEMENT(x)
//void _wabort(const char *function, int line, int scope, const char *format, ...) __attribute__ ((format (printf, 4, 5)));
void _wabort(const char *function, int line, int scope, const char *format, ...);

#define wabort(scope, format, a...)   _wabort(__FUNCTION__, __LINE__, scope, format, ##a)

#endif /* DEBUG */

#ifdef __INSURE__
//#define inline
#endif

/*
** Define the format of a woempa.  If any of the following symbols is not
** #defined (e.g. the definition is comment out) then the corresponding
** information will not be printed.
**   WOEMPA_JTHREAD_FORMAT       prints the address of the current w_thread.
**   WOEMPA_LEVEL_FORMAT         prints the woempa level.
**   WOEMPA_BYTECODECOUNT_FORMAT prints the count of opcodes executed
**                               (summed over all threads).
**   WOEMPA_METHOD_FORMAT        prints the current method's declaring class
**                               and name, and the current PC.
**   WOEMPA_FUNCTION_FORMAT      prints the current C function name and line
**                               number.
*/
// #define WOEMPA_JTHREAD_FORMAT       "%p"
// #define WOEMPA_LEVEL_FORMAT         "(%d)"
// #define WOEMPA_BYTECODECOUNT_FORMAT "%d"
#define WOEMPA_METHOD_FORMAT        "[%k/%w:%d]"
#define WOEMPA_FUNCTION_FORMAT      "[%s:%d]"

void w_dump(const char *fmt, ... );
  
typedef struct w_Trace_Element {
  w_method method;
  int pc;
  int line;
} w_Trace_Element;

typedef w_Trace_Element * w_trace_element;

#endif /* _DEBUG_H */
