#ifndef _FIELD_H
#define _FIELD_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips Site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: fields.h 4140 2004-04-05 08:18:48Z dumon $
*/

#include "threads.h"
#include "wonka.h"

/*
** Enable this in order to have boolean and byte fields packed bytewise
** instead of wordwise (to save heap space).
** ==> NOT WORKING YET <===
*/
//#define PACK_BYTE_FIELDS

/*
** Define F_Field_wotsit. This way we don't need to include the huge 
** core-classes.h file and as a result speed up compilation a lot since 
** almost every .c file includes this file.
*/

extern w_int F_Field_wotsit;

/*
** Fields of w_Field:
**  declaring_clazz  the declaring clazz
**  label           the C string "field"
**  name            the field's name
**  value_clazz     the type of data it holds
**  flags           low 16 bits: ACC_xxx flags from class file
**                  high 16 bits: Wonka FIELD_xxx flags, see below
**  size_and_slot   high 4 bits: if all '1', then this field is a reference
**                  field, otherwise it is a primitive field and these bits
**                  hold log2 of the field's logical size: e.g. 0000 for a
**                  boolean, 0101 for an int.
**                  low 28 bits: offset of the field within an instance.
**                  currently repesented as follows:
**                    boolean, byte: offset in bytes
**                    other primitives: offset in words
**                    reference: negative offset from the end of the instance,
**                    e.g. the last field has -1 (all '1's).
**                  if this negative then the field is a reference field and
**                  `slot' needs to be subtracted from the instanceSize of
**                  the actual clazz ro arrive at an offset in words.
**                  64 bit values live at 'slot' + WORD_LSW and 'slot' + WORD_MSW.
**  desc_index      index into the constant pool of this field's descriptor
**  initval         index in the constant pool of the constant initialiser (0 if none)
*/
typedef struct w_Field {
  w_clazz  declaring_clazz;
  char    *label;
  w_string name;
  w_clazz  value_clazz;
  w_flags  flags;
  w_int    size_and_slot;
  w_ushort desc_index;
  w_ushort initval;
} w_Field;

#ifdef PACK_BYTE_FIELDS
#define FIELD_SIZE_MASK    0xf0000000
#define FIELD_OFFSET_MASK  0x0000ffff

#define FIELD_SIZE_1_BIT   0x00000000
#define FIELD_SIZE_8_BITS  0x30000000
#define FIELD_SIZE_16_BITS 0x40000000
#define FIELD_SIZE_32_BITS 0x50000000
#define FIELD_SIZE_64_BITS 0x60000000

#define FIELD_SIZE(f)   ((((f) & FIELD_SIZE_MASK) >> 28) & 0xf)
#define FIELD_OFFSET(f) ((f) & FIELD_OFFSET_MASK)

#else

#define FIELD_OFFSET(f) (f)
// FIELD_SIZE deliberately left undefined, should never be asked for
#endif

/*
** Wonka flags.  The following combinations are possible:
**  WOTSIT  ARRAY  REFERENCE  LONG  meaning
**     0       0       0       1    primitive field, long or double
**     0       0       0       0    primitive field, other
**     0       0       1       0    reference field, not an array
**     0       1       1       0    array reference field
**     1       0       0       0    wotsit
*/
#define FIELD_IS_LONG       0x10000
#define FIELD_IS_REFERENCE  0x20000
#define FIELD_IS_ARRAY      0x40000
#define FIELD_IS_WOTSIT     0x80000

/*
** fieldSize(f) returns 2 if f->flags has FIELD_IS_LONG set, else 1.
*/
#define fieldSize(f) (1 + !!isSet((f)->flags,  FIELD_IS_LONG))

/*
** Matching criteria
*/

#define MATCH_STATIC_FIELD                         1
#define MATCH_INSTANCE_FIELD                      -1
// #define MATCH_STATIC_OR_INSTANCE                   0

#define MATCH_PRIVATE_FIELD                        1
#define MATCH_PUBLIC_FIELD                        -1
// #define MATCH_PUBLIC_OR_PRIVATE                    0

// #define MATCH_ANY_DESCRIPTOR                       NULL

#define MATCH_ANY                                  0

w_field searchClazzHierarchyForField(w_clazz clazz, w_string name, w_string desc_string, w_int isStatic, w_int isPrivate);
w_field searchClazzAndInterfacesForField(w_clazz clazz, w_string name, w_string desc_string, w_int isStatic, w_int isPrivate);
w_size findFieldOffset(w_clazz clazz, const char * utf8name);

void fieldTableDump(w_clazz clazz);

/*
** Special Wonka flags.
./

#define FLD_64_BITS       0x80
#define FLD_REFERENCE     0x40
#define FLD_ARRAY         0x20
#define FLD_SYNTHETIC     0x10
#define FLD_spare         0x80
#define FLD_FPIMASK       0x07    /. The first 3 bits are for function indexing. ./

/.
** Field pointer index values. The functions indexed by these values return or take
** a void pointer that is aligned to the correct type. 
./

#define FPI_PRIM_8        0       /. Gets a void pointer to a primitive 8 bit (B,Z) instance field. ./
#define FPI_PRIM_16       1       /. Gets a void pointer to a primitive 16 bit (C,S) instance field. ./
#define FPI_PRIM_32       2       /. Gets a void pointer to a primitive 32 bit (F,I) instance field. ./
#define FPI_PRIM_64       3       /. Gets a void pointer to a primitive 64 bit (D,J) instance field. ./
#define FPI_REFERENCE     4       /. Gets a void pointer to a reference instance field. ./
#define FPI_STATIC        5       /. Used for static class data, always in 32 bit slots. ./

/.
** Official access flags.
./

#define FLD_PUBLIC        0x01
#define FLD_PRIVATE       0x02
#define FLD_PROTECTED     0x04
#define FLD_STATIC        0x08
#define FLD_FINAL         0x10
#define FLD_DEPRECATED    0x20
#define FLD_VOLATILE      0x40
#define FLD_TRANSIENT     0x80

/.
** The flags that are allowed in the classfile, any other flags in the classfile are ignored.
** This value will be anded with the value read from the classfile.
./

#define FLD_CLASSFLAGS    (FLD_PUBLIC | FLD_PRIVATE | FLD_PROTECTED | FLD_STATIC | FLD_FINAL | FLD_VOLATILE | FLD_TRANSIENT)
                                                                                                                                                           
typedef struct w_Fld {
  w_ushort class_offset;          /. Offset from w_Fld to the class. ./
  w_ushort data_offset;           /. Offset for data, reference point depends on static or not, reference or not. ./
  w_ushort type_offset;           /. Type offset of field in constant pool. ./
  w_ubyte  oflags;                /. Offical Java flags. ./
  w_ubyte  wflags;                /. Wonka field flags and status information. ./
  w_string name;                  /. Full pointer to name string of field. ./
} w_Fld;

inline static w_type fieldType(w_fld fld) {
  return (w_type) * ((w_word *)fld - fld->type_offset);
}
  
inline static w_class fieldClass(w_fld fld) {
  return (w_class) ((w_word *)fld - fld->class_offset);
}

inline static void * fieldStaticData(w_fld fld) {
  return ((w_word *)fld - fld->data_offset);
}
*/

/*
** Get a pointer to a word-sized field of an instance.
** (Currently also used for half- and double-word fields; this may change).
*/
static inline w_word *wordFieldPointer(w_instance parent, w_int slot) {
#ifdef PACK_BYTE_FIELDS
  return parent + (slot & FIELD_OFFSET_MASK);
#else
  return parent + slot;
#endif
}

#ifdef PACK_BYTE_FIELDS
/*
** Get a pointer to a byte-sized field of an instance.
** (Currently also used for boolean fields; this may change).
*/
static inline char *byteFieldPointer(w_instance parent, w_int slot) {
  return ((char*)parent) + (slot & FIELD_OFFSET_MASK);
}
#else
#define byteFieldPointer wordFieldPointer
#endif

/*
** "Getters" for the various types of primitive field.
*/
static inline w_boolean getBooleanField(w_instance parent, w_int slot) {
  return !!*byteFieldPointer(parent, slot);
}

static inline w_int getCharacterField(w_instance parent, w_int slot) {
  return (w_char)*wordFieldPointer(parent, slot);
}

static inline w_int getIntegerField(w_instance parent, w_int slot) {
  return (w_int)*wordFieldPointer(parent, slot);
}

static inline w_long getLongField(w_instance parent, w_int slot) {
  w_long value;

  value = (wordFieldPointer(parent, slot))[WORD_MSW];
  value <<= 32;
  value |= (wordFieldPointer(parent, slot))[WORD_LSW];

  return value;
}

static inline void *getWotsitField(w_instance parent, w_int slot) {
  return *((void**)wordFieldPointer(parent, slot));
}

/*
** "Setters" for the various types of primitive field.
*/
static inline void setBooleanField(w_instance parent, w_int slot, w_boolean value) {
  *(byteFieldPointer(parent, slot)) = !!value;
}

static inline void setByteField(w_instance parent, w_int slot, w_sbyte value) {
  *(byteFieldPointer(parent, slot)) = value;
}

static inline void setShortField(w_instance parent, w_int slot, w_short value) {
  *(wordFieldPointer(parent, slot)) = value;
}

static inline void setCharacterField(w_instance parent, w_int slot, w_char value) {
  *(wordFieldPointer(parent, slot)) = value;
}

static inline void setIntegerField(w_instance parent, w_int slot, w_int value) {
  *((w_int*)wordFieldPointer(parent, slot)) = value;
}

static inline void setFloatField(w_instance parent, w_int slot, w_int value) {
  *((w_float*)wordFieldPointer(parent, slot)) = value;
}

static inline void setLongField(w_instance parent, w_int slot, w_long value) {
  (wordFieldPointer(parent, slot))[WORD_LSW] = value & 0x00000000ffffffffUL;
  (wordFieldPointer(parent, slot))[WORD_MSW] = value >> 32;
}

static inline void setDoubleField(w_instance parent, w_int slot, w_double value) {
  (wordFieldPointer(parent, slot))[WORD_LSW] = ((jvalue)value).w[0];
  (wordFieldPointer(parent, slot))[WORD_MSW] = ((jvalue)value).w[1];
}

static inline void setWotsitField(w_instance parent, w_int slot, void *value) {
  *((void**)wordFieldPointer(parent, slot)) = value;
}

w_clazz instance2clazz(w_instance ins);
#include "clazz.h"
#include "heap.h"

/*
** To get a reference field of any object, use this function to add it also to the local ref table.
*/

static inline w_instance getReferenceField(w_instance parent, w_int slot) {
  w_instance result;
  w_int offset = instance2clazz(parent)->instanceSize + slot;

  
  if (parent == NULL) {
    return NULL;
  }

  result = (w_instance)parent[offset];

  return result;
}

/*
** To set a reference field of any object, always use this function, which 
** acts as a ``write barrier''. Note that `slot' is a negative value relative
** to the end of the instance.
*/

static inline void setReferenceField(w_instance parent, w_instance child, w_int slot) {
#ifdef GC_SAFE_POINTS
    w_thread  thread = currentWonkaThread;
    w_boolean unsafe = enterUnsafeRegion(thread);
#endif
  w_int offset = instance2clazz(parent)->instanceSize + slot;

  if (child == NULL) {
    parent[offset] = 0;
  }
  else {
    parent[offset] = (w_word)(child);

    newReferenceTo(child);
  }

#ifdef GC_SAFE_POINTS
    if (!unsafe) {
      enterSafeRegion(thread);
    }
#endif
}

/*
** Use this variant if you know that the calling thread is already marked unsafe
** (e.g. from within J-spot or the interpreter).  In case of doubt use setReferenceField above!
*/
static inline void setReferenceFieldUnsafe(w_instance parent, w_instance child, w_int slot) {
  w_int offset = instance2clazz(parent)->instanceSize + slot;

  if (child == NULL) {
    parent[offset] = 0;
  }
  else {
    parent[offset] = (w_word)(child);

    newReferenceTo(child);
  }
}

/*
** Get the w_field pointer corresponding to an instance of java.lang.reflect.Field.
*/
static inline w_field Field2field(w_instance Field) {

  if (Field == NULL) {
    return NULL;
  }

  return getWotsitField(Field, F_Field_wotsit);

}

/*
** Return WONKA_TRUE if field is declared in clazz or one of its superclasses,
** WONKA_FALSE otherwise.
*/
static inline w_boolean isFieldFromClazz(w_clazz clazz, w_field field) {
  return isSuperClass(field->declaring_clazz, clazz);
}

#endif /* _FIELD_H */
