#ifndef _LOADING_H
#define _LOADING_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: loading.h 3554 2003-02-21 10:56:51Z gray $ 
** 
** The Wonka kernel is software copyright by SmartMove NV (1999).
** Please see the file Copyright for information on its legal use.
** 
*/

#include "clazz.h"
#include "core-classes.h"
#include "wstrings.h"
#include "oswald.h"
#include "wonka.h"

/*
** Some special methods: java.lang.Object/finalize() and 
java.lang/ClassLoader/loadClass(). These are set up during startLoading().
*/
w_method finalize_method;
w_method loadClass_method;

/*
** If a class with the given name has already been loaded by `loader',
** return a pointer to its w_Clazz structure; otherwise return a pointer
** to a w_UnloadedClazz which encapsulates the name and loader.
*/
w_clazz identifyClazz(w_string name, w_instance loader);

/*
** seekClazzByName looks for the named class in the loaded_classes hashtable
** of the given classLoader, and returns either a w_clazz (if it was found)
** or null.
*/
w_clazz seekClazzByName(w_string name, w_instance classLoader);

/*
** namedClassMustBeLoaded looks for the named class in the loaded_classes
** hashtable of the given classLoader, and if it is not found then it tries
** to load it (by calling the classLoader's loadClass() method).  If loading
** fails an exception is thrown and namedClassMustBeLoaded returns NULL.
*/
w_clazz namedClassMustBeLoaded(w_instance classLoader, w_string name);

/*
** namedArrayClassMustBeLoaded behaves the same as namedClassMustBeLoaded,
** except that the name must denote an array class (must begine with '[').
*/
w_clazz namedArrayClassMustBeLoaded(w_instance classLoader, w_string name);

/*
** getNextDimension returns the w_clazz which represents a 1-D array of the
** given clazz (so it the given clazz is itself an n-dimensional array the
** result will be (n+1)-dimensional). If the required array clazz does not
** exist then it will be created.
*/
w_clazz getNextDimension(w_clazz);

/*
** createNextDimension creates the w_clazz which represents a 1-D array of the
** given clazz.  The target class must not already exist!
*/
w_clazz createNextDimension(w_clazz);

/*
** Convert the given string from descriptor form (e.g. "Ljava/lang/Class;", "I")
** to Class.getName() form ("java.lang.Class", "int").
** Returns a registered w_string.
*/
w_string undescriptifyClassName(w_string string);


extern w_hashtable fixup_hashtable;

static inline w_hashtable loader2loaded_classes(w_instance loader) {
  if (loader == NULL) {

    return system_loaded_class_hashtable;

  }
  else {

    return getWotsitField(loader, F_ClassLoader_loaded_classes);
  }
}

static inline w_hashtable loader2unloaded_classes(w_instance loader) {
  if (loader == NULL) {

    return system_unloaded_class_hashtable;

  }
  else {

    return getWotsitField(loader, F_ClassLoader_unloaded_classes);
  }
}

/*
** Possible outcomes for mustBeXxxx calls: no action was needed, action was
** successful, action was unsuccessful.
*/
#define CLASS_LOADING_DID_NOTHING 0
#define CLASS_LOADING_SUCCEEDED   1
#define CLASS_LOADING_FAILED     -1

w_int mustBeLoaded(volatile w_clazz *clazzptr);
w_int mustBeSupersLoaded(w_clazz clazz);
w_int mustBeReferenced(w_clazz clazz);
w_int mustBeLinked(w_clazz clazz);

/*
** The following functions go in groups of four: 
** classMustBeInitialized/classIsInitialized/initializeClass/alreadyInitialized, etc..
** Initially the clazz structure is set up so that clazz->initialize points to initializeClass, etc..
** When one the static inlines classMustInitialized etc.
** is executed for the first time on this class, it calls
** initializeClass (etc.) which in turn updates class->initialize to point at
** alreadyInitialized (etc.).
*/

#define classMustBeResolved(c,v) _classMustBeResolved(&(c),(v))

// TODO: get rid of this silly synonym

void initializeClass(w_clazz);
void alreadyInitialized(w_clazz);

static inline w_boolean 
classIsInitialized(w_clazz clazz) {
  return clazz->initialize == alreadyInitialized;
}

/// Class is already FUBAR -- throw a NoClassDefFoundError
void alreadyBroken(w_clazz);

w_clazz loadBootstrapClass(w_string name);
w_clazz loadNonBootstrapClass(w_instance classLoader, w_string name);

/*
** Function to be called during VM startup, to initialise some data structures.
*/
void startLoading(void);

/*
** Name of the zipfile which contains the bootstrap classes, as a C string.
*/
char     *zipname;

/*
** Set the system class loader to 'scl'.  From this point on classes on
** the bootstrap class path will be loaded by scl, not by loadBootstrapClass.
*/
void setSystemClassLoader(w_instance scl);

/*
** Hashtable mapping clazz x imethod -> method
*/
extern w_hashtable2k interface_hashtable;

/*
** The unique instance of SystemClassLoader (or NULL if not yet installed).
** Must only be set once, using setSystemClassLoader()!
*/
extern w_instance systemClassLoader;

/*
** Function to compare two w_[Unloaded}Clazz pointers.
** Returns WONKA_TRUE if they are either both unloaded (state == 0) or both
** loading or loaded (state != 0), and both refer to the same class name
** and loader. Otherwise, returns WONKA_FALSE.
*/
w_boolean sameClazz(w_clazz, w_clazz);

/*
** Ditto but taking w_word parameters, for use as a hashtable comparator.
*/
w_boolean clazzComparator(w_word clazz1_word, w_word clazz2_word);

/*
** Function to compare two pointers to w_[Unloaded}Clazz pointers.
** If the two pointers point to w_[Unloaded}Clazz pointers with different
** names, returns WONKA_FALSE without further checking.
** If both pointers are to w_UnloadedClazz pointers with the same name and
** same (originating) class loader, returns WONKA_TRUE. Also returns
** WONKA_TRUE if both pointers are to (loaded) w_Clazz pointers with the
** same name and (defining) class loader. Otherwise (one is loaded and
** the other is not, or both are unloaded and the initiating loaders are
** different), both classes will be loaded in order to compare them.
**
** Because this function can call mustBeLoaded(), the caller must ensure
** that for each of the two pointers at least one of the following is
** true:
**   - the class is already loaded
**   - the pointer-to-a-pointer is a local variable of a function
**   - the pointer-to-a-pointer is a field of a w_Method or w_Field, and
**     the caller holds the resolution_monitor for the clazz in which the
**     field or method is declared.
** Failure to do so may result in race situations in which two threads load
** the same class and deregister the same w_UnloadedClazz (ouch).
*/
w_boolean sameClassReference(w_clazz *clazzptr1, w_clazz *clazzptr2);

/*
** Hashcode for a w_[Unloaded]Class.
*/
w_word clazzHashCode(w_word clazz_word);

/*
** Register a w_UnloadedClazz. The result returned is a w_UnloadedClazz*
** which may or not be the same as the one passed as input: if it is
** different, then the one passed as input has been released.
*/
w_clazz registerUnloadedClazz(w_clazz clazz);

/*
** Deregister a w_UnloadedClazz. When we see that the registered count drops
** to 0, we release the w_UnloadeClazz structure.
*/

void deregisterUnloadedClazz(w_clazz clazz);

#endif /* _LOADING_H */
