#ifndef _MEMSET_H
#define _MEMSET_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: memset.h 4278 2005-10-28 13:19:40Z chris $
*/

#define min(a, b) (a < b ? a : b)
#define max(a, b) (a > b ? a : b)

inline static void *inline_memset(void *address, unsigned int cbyte, unsigned int size) {

  unsigned int mask = cbyte & 0x000000ff;
  unsigned int prebytecopies   = min(4 - ((unsigned int)address % 4), size) % 4;
  unsigned int postbytecopies  = (size - prebytecopies) % 4;
  unsigned int wordcopies = max((int)size - (int)prebytecopies - (int)postbytecopies, 0) >> 2;
  unsigned int duffs = (wordcopies + 63) / 64;
  unsigned char *bytecursor = address;
  u_int32_t *wordcursor = address;

  cbyte &= 0x000000ff;

  /*
  ** Prepare the mask
  */

  mask <<= 8;
  mask |= cbyte;
  mask <<= 8;
  mask |= cbyte;
  mask <<= 8;
  mask |= cbyte;

  /*
  ** Do byte copying (to get aligned)
  */

  for (; prebytecopies; prebytecopies--) {
    *bytecursor++ = cbyte;
  }
  
  /*
  ** Do word copying
  */

  if(wordcopies) { 
    switch (wordcopies & 0x3f) {
      default:
      case  0: do { *wordcursor++ = mask;
      case 63:      *wordcursor++ = mask;
      case 62:      *wordcursor++ = mask;
      case 61:      *wordcursor++ = mask;
      case 60:      *wordcursor++ = mask;
      case 59:      *wordcursor++ = mask;
      case 58:      *wordcursor++ = mask;
      case 57:      *wordcursor++ = mask;
      case 56:      *wordcursor++ = mask;
      case 55:      *wordcursor++ = mask;
      case 54:      *wordcursor++ = mask;
      case 53:      *wordcursor++ = mask;
      case 52:      *wordcursor++ = mask;
      case 51:      *wordcursor++ = mask;
      case 50:      *wordcursor++ = mask;
      case 49:      *wordcursor++ = mask;
      case 48:      *wordcursor++ = mask;
      case 47:      *wordcursor++ = mask;
      case 46:      *wordcursor++ = mask;
      case 45:      *wordcursor++ = mask;
      case 44:      *wordcursor++ = mask;
      case 43:      *wordcursor++ = mask;
      case 42:      *wordcursor++ = mask;
      case 41:      *wordcursor++ = mask;
      case 40:      *wordcursor++ = mask;
      case 39:      *wordcursor++ = mask;
      case 38:      *wordcursor++ = mask;
      case 37:      *wordcursor++ = mask;
      case 36:      *wordcursor++ = mask;
      case 35:      *wordcursor++ = mask;
      case 34:      *wordcursor++ = mask;
      case 33:      *wordcursor++ = mask;
      case 32:      *wordcursor++ = mask;
      case 31:      *wordcursor++ = mask;
      case 30:      *wordcursor++ = mask;
      case 29:      *wordcursor++ = mask;
      case 28:      *wordcursor++ = mask;
      case 27:      *wordcursor++ = mask;
      case 26:      *wordcursor++ = mask;
      case 25:      *wordcursor++ = mask;
      case 24:      *wordcursor++ = mask;
      case 23:      *wordcursor++ = mask;
      case 22:      *wordcursor++ = mask;
      case 21:      *wordcursor++ = mask;
      case 20:      *wordcursor++ = mask;
      case 19:      *wordcursor++ = mask;
      case 18:      *wordcursor++ = mask;
      case 17:      *wordcursor++ = mask;
      case 16:      *wordcursor++ = mask;
      case 15:      *wordcursor++ = mask;
      case 14:      *wordcursor++ = mask;
      case 13:      *wordcursor++ = mask;
      case 12:      *wordcursor++ = mask;
      case 11:      *wordcursor++ = mask;
      case 10:      *wordcursor++ = mask;
      case  9:      *wordcursor++ = mask;
      case  8:      *wordcursor++ = mask;
      case  7:      *wordcursor++ = mask;
      case  6:      *wordcursor++ = mask;
      case  5:      *wordcursor++ = mask;
      case  4:      *wordcursor++ = mask;
      case  3:      *wordcursor++ = mask;
      case  2:      *wordcursor++ = mask;
      case  1:      *wordcursor++ = mask;
               } while (--duffs > 0);
    }
    bytecursor += wordcopies * 4;
  }
  
  //for (; wordcopies; wordcopies--) {
  //  *((unsigned int *)cursor)++ = mask;
  //}

  /*
  ** Do byte copying
  */

  for (; postbytecopies; postbytecopies--) {
    *bytecursor++ = cbyte;
  }

  return address;
  
}

#endif /* _MEMSET_H */
