#ifndef _METHODS_H
#define _METHODS_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: methods.h 4041 2003-07-30 11:28:28Z dumon $
*/

#include "clazz.h"
#include "fields.h"
#include "hashtable.h"
#include "wonka.h"
#include "dispatcher.h"
#include "heap.h"

/*
** Debugging information (line numbers, local vars) is only included if either
** DEBUG or JDWP is defined.
*/

#if defined(DEBUG) || defined(JDWP) || defined(METHOD_DEBUG)

#define STORE_METHOD_DEBUG_INFO

typedef struct w_LineNum {
  w_short start_pc;
  w_short line_number;
} w_LineNum;

typedef struct w_LocalVar {
  /*
  ** The ranges of the pc where this local variable is active.
  */
  w_pc start_pc;
  w_pc end_pc;
  /* 
  ** The slot into the frame->locals[] array 
  ** where this local variable lives.
  */
  w_short slot; 
  /*
  ** A Name and Type union; after resolution, the name and desc indexes
  ** are converted into a real w_string and a w_desc.
  */
  w_ushort name_index;
  w_ushort desc_index;
} w_LocalVar;

#endif

/*
** Define F_Method_wotsit. This way we don't need to include the huge core-classes.h file 
** and as a result speed up compilation a lot since almost every .c file includes this file.
*/

extern w_int F_Method_wotsit;

/*
** The slot number for a method that still isn't processed fully.
*/

#define SLOT_NOT_ALLOCATED                     -1

/*
** Description of an exception handler.
*/
typedef struct w_Exception {
  w_ushort start_pc;   /* Start of code covered by this handler (inclusive) */
  w_ushort end_pc;     /* End of code covered by this handler (exclusive)   */
  w_ushort handler_pc; /* Entry point of the handler                        */
  w_ushort type_index; /* Type of exeception covered (0 -> all execeptions) */
} w_Exception;

/*
** The information needed to uniquely identify a method.
*/
typedef struct w_MethodSpec {
  w_clazz  declaring_clazz; /* Class in which the method is declared       */
  w_string name;            /* Method name                                 */
  w_clazz *arg_types;       /* Types of arguments, null-terminated         */
  w_clazz  return_type;     /* Return type                                 */
} w_MethodSpec;

typedef struct w_MethodExec {
  w_int dispatcher;
  w_size       arg_i;      /* The number of argument words. !! DONT move !! Assembly depends on it.    */
  w_function   function;   /* if native, the function pointer. !! DONT move !! Assembly depends on it. */
  w_ushort return_i;       /* Number of words returned as result (0, 1, or 2) */
  w_ushort local_i;        /* Number local vars, from class file */
  w_ushort stack_i;        /* Number of temporary locations used on stack */
  w_ushort nargs;          /* Number of arguments (of whatever size)      */
  w_int    code_length;    /* Length of code - can be > 65535 (!)         */
  w_code   code;           /* Bytecode array (if not native method)       */
  w_short numExceptions;   /* Number of exception handlers                */
  w_exception exceptions;  /* Exception handler array                     */
#ifdef STORE_METHOD_DEBUG_INFO
  w_short numLocalVars;    /* Number of local variable records            */
  w_short numLineNums;     /* Number of line number records               */

  w_LocalVar *localVars;   /* Local variable record array                 */
  w_LineNum *lineNums;     /* Line number record array                    */
#endif
#ifdef JAVA_PROFILE
  w_long  bytecodes;
  w_word  runs;
  w_long  totaltime;
  w_long  runtime;
  w_ubyte checked;
  void *callData;
  void *instanceData;
#endif
} w_MethodExec;

/*
** Complete data for a method.
*/
typedef struct w_Method {
  w_MethodSpec spec;         /* Method spec, see above                     */
  w_int        slot;         /* Offset in vmlt (-1 for abstract methods)   */
  w_flags      flags;
  w_ushort     desc_index;   /* index of the descriptor in clazz->values   */
  w_method     parent;       /* method which we override or implement      */
  w_short      numThrows;    /* number of classes in the `throws' clause   */
  w_ushort    *throws;       /* the classes themselves                     */

  w_MethodExec exec;

  #ifdef JSPOT
    /*
    ** The return address of a J-Spot compiled method.  
    **   FIXME: as soon I figured out what new fields are required to 
    **          support J-Spot compiled methods, I'll probably bundle
    **          them in a seperate struct.
    */
  
    w_word return_address;
    int counter;
  #endif
    
} w_Method;

/*
** The lower 16 bits of the "flags" field are the ACC_... flags defined
** in the Java class file definition. The upper 16 bits are used as follows:
*/
#define METHOD_IS_CLINIT       0x00010000 /* static initializer (<clinit>)  */
#define METHOD_IS_CONSTRUCTOR  0x00020000 /* constructor (<init>)           */
#define METHOD_NO_OVERRIDE     0x00040000 /* private/final/constructor      */
                                          /* Note: a constructor has both   */
                                          /* CONSTRUCTOR and OVERRIDE set   */
#define METHOD_IS_INTERFACE    0x00080000 /* method of an interface         */
#define METHOD_IS_COMPILED     0x00200000 /* JITed method                   */
#define METHOD_IS_MIRANDA      0x01000000 /* abstract method inherited from */
                                          /* a superinterface               */
#define METHOD_IS_PROXY        0x02000000 /* Automatically-generated method */
                                          /*  a dynamic Proxy class         */
/*
** Convert a w_methodExec pointer to a w_method, by subtracting the offset
** of the `exec' field within w_Method.
*/
#define exec2method(x) ((w_method)(((char*)(x)) - (size_t)&((w_method)0)->exec))

/*
** registerNative() registers `fp' as the native code for the native method
** with the given name and signature of the given clazz.
*/
void _registerNativeMethod(w_clazz clazz, w_fun_dec fp, const char *name, const char *sig);
#define registerNativeMethod(clazz, function, name, sig) _registerNativeMethod(clazz, (w_fun_dec)function, name, sig) 

/*
** code2pc translates a pointer into the code of a method into a PC (offset
** from the start of the method).  Special case: if the method is native,
** always returns -1.
*/
w_int code2pc(w_method method, w_code current);

#ifdef STORE_METHOD_DEBUG_INFO

/*
** code2line translates a pointer into the code of a method into a line number
** within the source file.  Special cases: 
** - if the method is native, always returns -1.
** - if the code pointer is out of range, returns the last line of the method.
*/
w_int code2line(w_method method, w_code current);
#endif

/*
** Formatter functions to print a method name only, or the full Wonka.
*/

char * print_method_short(char * buffer, int * remain, void * data, int w, int p, unsigned int f);
char * print_method_long(char * buffer, int * remain, void * data, int w, int p, unsigned int f);

#if defined(DEBUG)
/*
** Print a report detailing all methods of a class.
*/
void methodTableDump(w_clazz clazz);
#endif

/*
** Convert an instance of java.lang.reflect.Method to a w_method pointer.
*/
static inline w_method Method2method(w_instance Method) {

  if (Method == NULL) {
    return NULL;
  }

  return getWotsitField(Method, F_Method_wotsit);

}

/*
** Create a w_MethodStack given the declaring class and the descriptor string.
** Returns CLASS_LOADING_SUCCEEDED or CLASS_LOADING_FAILED.
*/
w_int createMethodSpecUsingDescriptor(w_clazz declaring_clazz, w_string name, w_string desc_string, w_MethodSpec **specptr);

/*
** Release a method spec.
*/
void releaseMethodSpec(w_MethodSpec *spec);

/*
** Register a set of native methods (e.g. all those of a particular class).
*/
void registerNatives(w_clazz clazz, const JNINativeMethod *methods, w_int mcount);

/*
** Look up the implementation of a given (non-interface) method in a subclass.
** If clazz does not inherit method, result is undefined(!).  May be called if
** clazz ==  method->declaring_clazz.
*/
static inline w_method virtualLookup(w_method method, w_clazz clazz) {
#ifdef RUNTIME_CHECKS
  if (isSet(method->flags, METHOD_IS_INTERFACE)) {
    wabort(ABORT_WONKA, "%M is an interface method, must use interfaceLookup instead\n", method);
  }
#endif

  if (isSet(method->flags, METHOD_NO_OVERRIDE)) {
    woempa(1, "%M cannot be overridden, so let's not do that\n", method);

    return method;

  }

  woempa(1, "Looking up %M in %K: result is %M\n", method, clazz, clazz->vmlt[method->slot]);

  return clazz->vmlt[method->slot];
}

/*
** Look up the implementation of a given interface method in an implementing class.
** If clazz does not implement this method, will return NULL.  
*/
w_method interfaceLookup(w_method method, w_clazz clazz);

/*
** Test whether a method matches a given MethodSpec. The method must have
** the correct declaring_clazz, name, and arg_types: the return_type must
** also match, unless it is given in the spec as 'null', in which case any
** return_type is acceptable. 
*/
w_boolean methodMatchesSpec(w_method method, w_MethodSpec *spec);

// STEVEN

/*
** Method table information structure.
*/

typedef struct w_Mti {
  w_ushort lnt_offset;            /* Offset to line number table data; i.e. lnt = w_mti + offset. */
  w_ushort nlnt;                  /* Number of entries in line number table. */
  w_ushort lvt_offset;            /* Offset to local variable table data; i.e. lvt = w_mti + offset. */
  w_ushort nlvt;                  /* Number of entries in local variable table. */
} w_Mti;

typedef struct w_Lne {
  w_ushort start_pc;              /* Index into code array where line starts. */
  w_ushort line;                  /* Corresponding line number in source file. */
} w_Lne;

typedef struct w_Lve {
  w_ushort start_pc;              /* Index into code array where given local variable becomes valid. */
  w_ushort length;                /* Number of bytes codes, starting from start_pc, where local is valid. */
  w_ushort name_index;            /* Name index into class constant pool of local name. */
  w_ushort type_index;            /* Type index into class constant pool of type of local. */
  w_ushort slot;                  /* Index slot of local variable in the current frame. */
  w_ushort padding;               /* Unused, for padding to word boundary. */
} w_Lve;

typedef struct w_Handler {
  w_ushort start_pc;              /* Index into code array where catched region starts. */
  w_ushort end_pc;                /* Index into code array where catched region stops. */ 
  w_ushort handler_pc;            /* Index into code array where handler begins. */
  w_ushort catch_index;           /* Index into constant pool to const_class of catched type. */
} w_Handler;

typedef struct w_Mei {
  w_ushort  nthrows;              /* The number of thrown exceptions or the index if MTD_ONE_THROWN set. */
  w_ushort  nhandlers;            /* The number of w_Handlers following this method information header. */
  w_Handler handlers[0];          /* The array of handlers, followed by the thrown indexes MTD_ONE_THROWN is not set. */
} w_Mei;

typedef struct w_Mtd {
  w_ushort    name_offset;        /* Unsigned offset to name string; w_mtd - offset = string slot. */
  w_ubyte     dispatcher;         /* Index into array of dispatchers for this method. */
  w_ubyte     narg_words;         /* Number of arguments words of method. */
  w_ubyte   * code;               /* Pointer to block of code; bytecode, native pointer or JIT code. */
  w_ushort    max_locals;         /* Maximum number of local slots. */
  w_ushort    max_stack;          /* Maximum slots required for stack. */
  w_ushort    type_offset;        /* Unsigned offset to type; w_mtd - offset = type slot. */
  w_ushort    flags;              /* Flags for this method. */
  w_word      id;                 /* 32 bit unique method ID. */
  w_ushort    extra_thrown;       /* Offset to extra information or thrown index. */
  w_ushort    class_offset;       /* Unsigned offset to w_Class; w_mtd - offset = w_Class begin. */
} w_Mtd;

/*
** Official access flags, except MTD_VIRTUAL and MTD_EXTRA_INFO which are Wonka flags.
*/

#define MTD_PUBLIC        0x0001
#define MTD_PRIVATE       0x0002
#define MTD_PROTECTED     0x0004
#define MTD_STATIC        0x0008
#define MTD_FINAL         0x0010
#define MTD_SYNCHRONIZED  0x0020
#define MTD_EXTRA_INFO    0x0040
#define MTD_VIRTUAL       0x0080
#define MTD_NATIVE        0x0100

/*
** Wonka specific flags.
*/

#define MTD_JSPOT         0x0200  /* Method is a J-spot compiled method. */
#define MTD_LOCAL         0x0400  /* Method is only applicable in the declaring class. */
#define MTD_ONE_THROWN    0x0800  /* Method throws only a single exception. */
#define MTD_IFACE         0x1000  /* Method implements an interface method. */
#define MTD_UNIQUE        0x2000  /* When set, the method is potentially virtual, but never overriden. */
#define MTD_STRICT        0x4000
#define MTD_ABSTRACT      0x8000

/*
** The flags we take from the class file, if the classfile sets any other flags, they will
** not upset Wonka. We and this value with the value read from the classfile.
*/

#define MTD_CLASSFLAGS    (MTD_PUBLIC | MTD_PRIVATE | MTD_PROTECTED | MTD_STATIC | MTD_FINAL | MTD_SYNCHRONIZED | MTD_NATIVE | MTD_ABSTRACT | MTD_STRICT)

w_string methodName(w_mtd mtd);

inline static w_type methodType(w_mtd mtd) {
  return (w_type) * ((w_word *)mtd - mtd->type_offset);
}

inline static w_class methodClass(w_mtd mtd) {
  return (w_class) ((w_word *)mtd - mtd->class_offset);
}

inline static w_mei methodMei(w_mtd mtd) {
  return (w_mei) ((w_word *) mtd + mtd->extra_thrown);
}

void addVirtualMethod(w_class class, w_mtd mtd);

void addInterfaceMethod(w_class class, w_mtd mtd);

w_mtd lookupVirtualMethod(w_class class, w_mtd mtd);

w_mtd lookupInterfaceMethod(w_class class, w_mtd mtd);

w_ushort * getMethodThrows(w_mtd mtd);

w_word uniqueVirtualID(void);
w_word uniqueInterfaceID(void);

void startMethods(void);

// END STEVEN

#endif /* _METHODS_H */
