#ifndef _WSTRINGS_H
#define _WSTRINGS_H

/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, bus 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: wstrings.h 4161 2004-05-14 13:44:19Z dumon $
*/

#include "fields.h"
#include "oswald.h"
#include "wonka.h"

/*
** If NEWSTRINGREG is defined, registerString/deregisterString use the
** `refcount' field of the w_String header in conjunction with ht_read/
** ht_write/ht_erase instead of the special ht_register/ht_deregister.
** The string_hashtable then contains keys only (key==value), saving an
** average of two words per string (one word per hashtable slot).
** TODO: measure the performance impact (if any).
*/
#define NEWSTRINGREG

/*
** If BYTEWISELATIN is defined, strings containing only Latin-1 characters
** are stored using one byte per word instead of two.  Under construction!
*/
#define BYTEWISELATIN

/*
** Define F_String_wotsit. This way we don't need to include the huge core-classes.h file 
** and as a result speed up compilation a lot since almost every .c file includes this file.
*/

extern w_int F_String_wotsit; 

/*
** The Wonka string structure.  Incorporates a reference count and a cached
** copy of the hash function, as well as a pointer to the canonical instance
** (if any) of this string.  Use function hashString() to calculate and 
** cache the hashcode.
** The canonical instance is created when getStringInstance() is called,
** and removed by GC when the instance becomes inaccessible.
*/

#ifdef BYTEWISELATIN
typedef struct w_String {
  w_word      refcount;
  w_word      wonka_hash;
  volatile w_instance  instance;
  w_size      length_and_flags;
  union {
  w_ubyte     bytes[8];
  w_char      chars[4];
  } contents;
} w_String;

#define string_length(s) ((s)->length_and_flags & 0x00ffffff)

#define STRING_IS_LATIN1 0x80000000
#define string_is_latin1(s) (!!((s)->length_and_flags & STRING_IS_LATIN1))

#define string_char(s,n) (string_is_latin1(s)?(w_char)((s)->contents.bytes[n]):(s)->contents.chars[n])
#else
typedef struct w_String {
  w_word      refcount;
  w_word      wonka_hash;
  volatile w_instance  instance;
  w_size      length;
  w_char      chars[4]; // variable length, ashley
} w_String;

#define string_length(s) (s)->length

#define string_is_latin1(s) 0

#define string_char(s,n) (s)->chars[n]

#endif

/*
** methods used by the character decoders!
*/
w_char*  utf2chars(w_thread thread, const char *utf8string, w_int* utf8length);

/*
** The compareStrings function does a compare on two unicode Wonka strings
** as documented in the compareTo method for java/lang/String.
*/

w_int compareStrings(w_string string1, w_string string2);

/*
** w_string2chars converts a w_string to an array of unicode characters.
** The caller is responsible for providing a sufficient amount of memory
** for the unpacked characters.  The result returned is the number of
** w_chars written (equal to source->numChars).
** The source w_string need not be registered, but *must* be well-formed.
*/

w_size w_string2chars(w_string source, w_char *dest);

#define WONKA_DUMMY_CHAR 0xFFFF

/*
** putw_char outputs one unicode character as UTF8.  The character is
** stored at *dest, and dest is then incremented by 1, 2 or 3 as appropriate.
*/

void putw_char(w_char ch, w_ubyte **dest);

/*
** string2UTF8 converts the given w_string into a UTF8 string.
** The UTF8 string is returned as a byte array in which the first two bytes
** are a 16-byte character count (most significant byte first). The same count
** is also returned as a w_int in 'count'. A null byte is also appended, but 
** is not included in the count.
** The memory used for the UTF8 string is allocated from the system pool. The
** caller is responsible for returning this memory when it is no longer needed.
*/
w_ubyte *string2UTF8(w_string string, w_int * buflen);

/*
** chars2UTF8 converts the 'length' unicode chars at 'chars' into a UTF8 string.
** The UTF8 string is returned as a byte array in which the first two bytes
** are a 16-byte character count (most significant byte first). The same count
** is also returned as a w_int in 'count'. A null byte is also appended, but 
** is not included in the count.
** The memory used for the UTF8 string is allocated from the system pool. The
** caller is responsible for returning this memory when it is no longer needed.
*/
w_ubyte* chars2UTF8(w_char *chars, w_int length, w_int *count);

/*
** w_charLength calculates the length in characters of a well-formed UTF8 string.
*/

w_size w_charLength(w_ubyte *utf8);

/*
** compareUtf8 compares two UTF8 strings
*/

w_int compareUtf8(w_ubyte* utf8a, w_ubyte* utf8b);

/*
** isPrefixUtf8 determines whether string a is a prefix of string b
*/

w_boolean isPrefixUtf8(w_ubyte* utf8a, w_ubyte* utf8b);

/*
** Dump the string hashtable to stdout
*/

void stringDump(void);


/*
** ht_stringCompare() compares two strings for equality and returns WONKA_TRUE
** iff they are identical.  Canonical equality function for string-based
** hashtables.
*/
w_boolean ht_stringCompare(w_word string1_word, w_word string2_word);

/*
** ht_stringHash() returns the hashcode cached within a w_string.  Use this
** for string-based hashtables.
*/
w_word ht_stringHash(w_word string_word);

/*
** cstring2String converts a C (ASCII) string into a registered w_string.
** The terminating null byte need not be present, and is not included
** in the length.
*/
w_string cstring2String(w_thread thread, const char *cstring, w_size length);

/*
** unicode2String converts a sequence of unicode characters into a registered w_string.
*/
w_string unicode2String(w_thread thread, w_char *chars, w_size length);

/*
** unicode2w_string converts a sequence of unicode characters string into an unregistered w_string.
*/
w_string unicode2w_string(w_char *chars, w_size length);

/*
** Create a Wonka string from a UTF8 string of bytes. The length argument 
**is the number of UTF8 bytes, not the number of characters! No null terminating** byte is expected, so if one is present it should not be included in the
** length argument.
*/
w_string utf2String(w_thread thread, const char *utf8string, w_size length);

/*
** Register a Wonka string. The result returned may be the same as the argument
** passed (if the string is "new"), or it may be different (an already existing
** string). In the latter case, the string passed as argument is automatically
** released. 
*/
w_string registerString(w_string string);

/*
** Deregister a Wonka string. When a string has been deregistered as many
** times as it has been registered, the string will be realeased.
*/
void deregisterString(w_string string);

/*
** Get the "canonical" instance of java.lang.String associated with s.
** If no canonical instance yet exists then it will be created.
*/
w_instance getStringInstance(w_string s);

/*
** Create an instance of java.lang.String which points to s. A new instance
** will always be created, even if a "canonical" instance already exists.
*/
w_instance newStringInstance(w_string s);

/*
** stringAdd will copy over the 'source' string to the 'destination' buffer.
** The caller is again expected to give a suitably sized buffer. It returns the
** number of characters added to the buffer so that the calls can again be daisy chained,
** as in this example:
**
** w_char *buffer = allocMem(...);
** w_int length = 0;
** ...
** length += stringAdd(buffer + length, string1);
** length += stringAdd(buffer + length, string2);
** ...
*/

w_int stringAdd(w_char *destination, w_string source);

/*
** dots2slashes takes a Wonka string and returns a registered Wonka
** string in which the dots are replaced with slashes If the string already
** contains slashes then these are replaced by DEL characters(!). Why? 
** Because the only use for this function is to convert fully-qualified
** class names from Java to "internal" format, and in Java format a slash
** should never occur.
** slashes2dots does the exact opposite.
*/

w_string dots2slashes(w_thread thread, w_string string);
w_string slashes2dots(w_thread thread, w_string string);

/*
** startStrings is called from startWonka: it creates the hashtable
** used by the string registration process, and populates it with all
** those commonly-used strings we were just talking about.
*/

void startStrings(void);

// Special strings `<NULL>', `<BOGUS>', `<EMPTY>'
extern w_string string_NULL;
extern w_string string_BOGUS;
extern w_string string_EMPTY;

// Names of classfile attributes
extern w_string string_ConstantValue;
extern w_string string_Code;
extern w_string string_Deprecated;
extern w_string string_Exceptions;
extern w_string string_LineNumberTable;
extern w_string string_LocalVariableTable;
extern w_string string_Reference;
extern w_string string_SourceFile;
extern w_string string_Synthetic;

// Descriptors, names of primitive types, etc.
extern w_string string_angle_brackets_init;
extern w_string string_angle_brackets_clinit;
extern w_string string_no_params;
extern w_string string_no_params_V;
extern w_string string_boolean;
extern w_string string_byte;
extern w_string string_char;
extern w_string string_double;
extern w_string string_finalize;
extern w_string string_run;
extern w_string string_float;
extern w_string string_int;
extern w_string string_long;
extern w_string string_short;
extern w_string string_void;

// Strings used by unicode.c
extern w_string string_medial;
extern w_string string_square;
extern w_string string_compat;
extern w_string string_narrow;
extern w_string string_small;
extern w_string string_isolated;
extern w_string string_initial;
extern w_string string_font;
extern w_string string_super;
extern w_string string_final;
extern w_string string_circle;
extern w_string string_vertical;
extern w_string string_fraction;
extern w_string string_wide;
extern w_string string_noBreak;
extern w_string string_sub;

// Defined in unicode.c
extern w_string category_name[];

#define w_string_equals(a, b)                  ((a) == (b))

void dumpStrings(void);

extern w_hashtable string_hashtable;

static inline w_string String2string(w_instance String) {

  if (String == NULL) {
    return NULL;
  }

  return getWotsitField(String, F_String_wotsit);

}

static inline void attachString(w_instance strinstance, w_string string) {
  w_string old = String2string(strinstance);
  if (old) {
    deregisterString(old);
  }
  setWotsitField(strinstance, F_String_wotsit, registerString(string));
}

char * print_string(char * buffer, int * remain, void * s, w_int width, w_int prec, w_flags flags);

/*
** Mutex required for mutual exclusion to the string pool.
*/
x_mutex string_mutex;

#endif /* _WSTRINGS_H */

