#ifndef _ZIP_H
#define _ZIP_H

/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: zip.h 2532 2002-08-14 11:09:08Z vandeneede $
*/

#include "wonka.h"
#include "dates.h"

#define WSF_ZIS_STRUCTURED                     0x00010000

#define Z_LOCAL_HEADER                         0x04034b50
#define Z_DIR_HEADER                           0x02014b50
#define Z_DIR_END                              0x06054b50

#define bytes2short(b)                         (w_short)((b[1] << 8) | b[0])
#define bytes2word(b)                          (w_word)((b[3] << 24) | (b[2] << 16) | (b[1] << 8) | b[0])

typedef struct z_ZipEntry *                    z_zipEntry;
typedef struct z_ZipFile *                         z_zipFile;
typedef w_int                                (*z_decoder)(z_zipEntry zipEntry);

/*
** A zip file is built with the following structure:
**
** ---------------------------------------------------------------------------------------------------------------
** [local file header + file data + data_descriptor]    for file 1
** [local file header + file data + data_descriptor]    for file 2
**         ...                                              ...
** [local file header + file data + data_descriptor]    for file n
** [directory file header]                              for file 1
** [directory file header]                              for file 2
**         ...                                              ...
** [directory file header]                              for file n
** Concluding file information
** ---------------------------------------------------------------------------------------------------------------
**
** Most of the information that is in the local file header is repeated in the directory 
** file header; we compile all this information in the structure z_Entry since for Wonka, all
** zipfiles will be one block and not by definition scattered over different floppies (the
** reason for all this redundancy). In the comment, we sometimes refer to the PKWARE Application notes
** that can be found back at http://www.pkware.com. When a comment starts with 'LFH' it is located
** in the 'Local File Header' according to that document, and when it starts with 'DFH' it is to
** be found back in the 'Directory File Header'.
*/

#define ZE_COMPRESSED_IS_STATIC               0x00010000 /* When the z_entry->c_data is pointing to static memory */
#define ZE_UNCOMPRESSED                       0x00020000 /* The entry has been uncompressed allready              */
#define ZE_ERROR                              0x00040000 /* Something is wrong with the data in this entry        */
#define ZE_NO_DECOMPRESSOR                    0x00080000 /* The decompressor is not yet implemented               */
#define ZE_NO_MEMORY                          0x00100000 /* Not enough memory was available for decompressing     */
#define ZE_ZERO_LENGTH                        0x00200000 /* The original file had 0 bytes in it!                  */

/*
** The different general purpose bit flags of PKWARE. Note that the flags usaed or reserved by
** PKWARE are 16 bits long. We use a complete 32 bit word to add our own flags to it. These flags
** start after the 16'th bit.
*/

#define ZE_ENCRYPTED                          0x00000001
#define ZE_BIT_1                              0x00000002
#define ZE_BIT_2                              0x00000004
#define ZE_DESCRIPTOR_AVAILABLE               0x00000008
#define ZE_BIT_4                              0x00000010
#define ZE_PATCHED_DATA                       0x00000020
#define ZE_BIT_6                              0x00000040
#define ZE_BIT_7                              0x00000080
#define ZE_BIT_8                              0x00000100
#define ZE_BIT_9                              0x00000200
#define ZE_BIT_10                             0x00000400
#define ZE_BIT_11                             0x00000800
#define ZE_BIT_12                             0x00001000
#define ZE_BIT_13                             0x00002000
#define ZE_BIT_14                             0x00004000
#define ZE_BIT_15                             0x00008000

typedef struct z_ZipEntry { 
  z_zipEntry next;
  z_zipEntry previous;
  z_zipFile zipFile;
  w_ubyte x_major;                             /* LFH: Version needed to extract (major version number)            */
  w_ubyte x_minor;                             /* LFH: Version needed to extract (minor version number)            */
  w_ubyte c_major;                             /* DFH: version made by (major version number)                      */
  w_ubyte c_minor;                             /* DFH: version made by (minor version number)                      */
  w_flags flags;                              /* LFH: General purpose bit flags, see ZE_ flags above              */
  w_short compression;                        /* LFH: compression method used                                     */
  w_date modified;                            /* LFH: Date and time of last modification                          */
  w_short s_disk;                             /* DFH: disk number start                                           */
  w_flags i_attr;                             /* DFH: internal file attributes                                    */
  w_word e_attr;                              /* DFH: external file attributes                                    */
  w_int offset;                               /* DFH: relative offset of local header                             */
  w_word d_crc;                               /* LFH: CRC 32 as found in the directory header                     */
  w_word c_crc;                               /* The calculated CRC value after decompressing the zipEntry.          */
  w_size c_size;                              /* LFH: compressed size                                             */
  w_size u_size;                              /* LFH: uncompressed size                                           */
  w_size p_size;                              /* The number of bytes we have written in the decoding process.     */
  w_short nameLength;                         /* LFH: filename length                                             */
  w_short extraLength;
  w_short commentLength;                      /* DFH: file comment length                                         */
  w_ubyte *name;
  w_ubyte *extra;
  w_ubyte *comment;                            /* the comment for the file                                         */
  w_ubyte *c_data;                             /* a pointer to the begin of the compressed data                    */
  w_ubyte *u_data;                             /* a pointer to the uncompressed data                               */
} z_ZipEntry;

typedef struct z_ZipFile {
  z_zipEntry entries;
  w_short c_disk;                             /* number of this disk                                              */
  w_short d_disk;                             /* number of disk with the central directory                        */
  w_short numDisk;                            /* total number of entries on this disk                             */
  w_short numDir;                             /* total number of entries in the central directory                 */
  w_int size;                                 /* size of the central directory                                    */
  w_int offset;                               /* offset of start of central directory w.r.t. starting disk number */
  w_short commentLength;                      /* zipfile comment length                                           */
  w_ubyte *comment;
  w_ubyte *stream;
} z_ZipFile;


// must be called before any call to uncompressZipEntry
w_void zipInit(w_void);
// must be called after final use of uncompressZipEntry
w_void zipEnd(w_void);

z_zipFile parseZipFile(w_thread thread, w_ubyte *data);
z_zipEntry uncompressZipEntry(z_zipEntry zipEntry);
w_int zipEntry2RepositoryType(z_zipEntry entry);
w_boolean isZipFile(w_ubyte *data);
void releaseZipFile(z_zipFile zipFile);

#endif /* _ZIP_H */
