/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package com.acunia.wonka.net.http;

import java.io.*;
import java.util.*;
import java.net.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import wonka.decoders.Latin1Decoder;
import wonka.decoders.Latin1Decoder;
import wonka.decoders.Base64Encoder;

public class BasicHttpURLConnection extends HttpURLConnection {

  private static SimpleDateFormat dateFormatter = new SimpleDateFormat("EEE, dd MMM yyyy HH:mm:ss 'GMT'");
  private static SimpleDateFormat rfc850Parser = new SimpleDateFormat("EEEE, dd-MMM-yy HH:mm:ss 'GMT'");
  private static SimpleDateFormat asctimeParser = new SimpleDateFormat("EEE MMM dd HH:mm:ss yyyy");

  private static Latin1Decoder decoder = new Latin1Decoder();

  static {
    try {
      TimeZone tz = TimeZone.getTimeZone("GMT");
      dateFormatter.setTimeZone(tz);
      rfc850Parser.setTimeZone(tz);
      asctimeParser.setTimeZone(tz);
    }
    catch(RuntimeException rt){}
  }

  private Socket socket;
  private InputStream in;
  private HttpOutputStream out;
  private Properties requestHeaders = new Properties();
  private Properties responseHeaders;
  private ArrayList keys;

  public BasicHttpURLConnection(URL url) {
    super(url);
    requestHeaders.put("accept-encoding","gzip");
    requestHeaders.put("host",url.getHost()+(url.getPort()== -1 ? "" : ":"+String.valueOf(url.getPort())));
    requestHeaders.put("user-agent","wonka Http protocol Handler");
  }

  protected int getProxyPort() throws IllegalArgumentException
  {
	String proxyPortStr = System.getProperty("http.proxyPort");

	if((proxyPortStr==null) || (proxyPortStr.trim().equals("")))
	{
		return 80;
	}
	else
	{
		try
		{
			return Integer.parseInt(proxyPortStr);
		}
		catch(NumberFormatException ex)
		{
			throw new IllegalArgumentException("http.proxyPort is not a number");
		}
	}
  }
  
  public synchronized void connect() throws IOException {
    if(!connected){
      responseHeaders = new Properties();
      keys = new ArrayList();
      int port = url.getPort();
      
      if(usingProxy())
      {
		String proxyHost = System.getProperty("http.proxyHost");
		int proxyPort = getProxyPort();

		socket = new Socket(proxyHost, proxyPort);
      }
      else
      {
      socket = new Socket(url.getHost(), port == -1 ? 80 : port);
      }
	  
      in = new BufferedInputStream(socket.getInputStream(),4096);
      connected = true;
      //send request ...
      if("PUT".equals(method) || "POST".equals(method)){
        OutputStream out = socket.getOutputStream();
        out.write(getRequestLine().getBytes());
        sendPartialHeaders(out);
        doOutput = true;
      }
      else if("GET".equals(method)){
        requestGET();
      }
      else if("HEAD".equals(method)){
        requestGET();
      }
      else if("DELETE".equals(method)){
        requestGET();
      }
      else if("OPTIONS".equals(method)){
        requestGET();
      }
      else if("TRACE".equals(method)){
        requestGET();
      }
      else {
        throw new IOException("invalid method '"+method+"'");
      }
    }
  }

  public synchronized void disconnect(){
    if(socket != null){
      try {
        socket.close();
      }
      catch(IOException ioe){}
      connected = false;
      socket = null;
    }
    connected = false;
  }

  public String getHeaderField(int n){
    if (!connected) {
      try {
        connect();
      }
      catch(IOException ioe){
        ioe.printStackTrace();
        return null;
      }
    }
    if(n < 0 || n >= keys.size()){
      return null;
    }
   	return (String)responseHeaders.get(keys.get(n));
  }

  public String getHeaderField(String name){
    if (!connected) {
      try {
        connect();
      }
      catch(IOException ioe){
        ioe.printStackTrace();
        return null;
      }
    }
   	return responseHeaders.getProperty(name);
  }

  public long getHeaderFieldDate(String name, long def){
    String field = getHeaderField(name);
    if(field != null){
      try {
        return dateFormatter.parse(field).getTime();
      }
      catch(ParseException pe1){
        try {
          return rfc850Parser.parse(field).getTime();
        }
        catch(ParseException pe2){
          try {
            return asctimeParser.parse(field).getTime();
          }
          catch(ParseException pe3){}
        }
      }
    }
    return def;
  }

  public String getHeaderFieldKey(int n) {
    if (!connected) {
      try {
        connect();
      }
      catch(IOException ioe){
        ioe.printStackTrace();
        return null;
      }
    }
    if(n < 0 || n >= keys.size()){
      return null;
    }
    Object key = keys.get(n);
    if(key instanceof String){
      return (String)key;
    }
    return null;
  }

  public InputStream getInputStream() throws IOException {
    if (!connected) {
      connect();
    }
    return in;
  }

  public synchronized OutputStream getOutputStream() throws IOException {
    if (!"PUT".equals(method)) {
      method = "POST";
    }
    if (!connected) {
      connect();
    }
    if(!doOutput){
      throw new IOException("output is disabled");
    }
    if (out == null) {
      out = new HttpOutputStream(socket.getOutputStream(),this);
    }
    return out;
  }

  public String getRequestProperty(String key){
    return requestHeaders.getProperty(key);
  }

  public void setIfModifiedSince(long time){
    super.setIfModifiedSince(time);
    requestHeaders.put("If-modified-since", dateFormatter.format(new Date(time)));
  }

  public void setRequestProperty(String key, String value){
    requestHeaders.put(key.toLowerCase(),value);
  }

  public boolean usingProxy(){
  	String proxyHost = System.getProperty("http.proxyHost");
	return ((proxyHost!=null) && (!proxyHost.trim().equals("")));
  }

  private String getRequestLine(){
	StringBuffer requestLine = new StringBuffer(method);

	requestLine.append(' ');
	
	if(usingProxy())
	{
		requestLine.append(url.toString());
	}
	else if(url.getFile().equals(""))
	{
		requestLine.append('/');
	}
	else
	{
		requestLine.append(url.getFile());
	}

    String add;

    /*
    add = url.getQuery();
    if(add != null){
      //System.out.println("adding Query '"+add+"' to request:\n"+method + " "+ file + " HTTP/1.1\r\n");
      file = file + "?" + add;
    }
    */

    add = url.getRef();
    if(add != null){
      //System.out.println("adding fragment '"+add+"' to request:\n"+method + " "+ file + " HTTP/1.1\r\n");
      requestLine.append('#');
      requestLine.append(add);
    }
    //System.out.println("sending request:\n"+method + " "+ file + " HTTP/1.1\r\n");
	
    requestLine.append(" HTTP/1.1\r\n");

    return requestLine.toString();
  }

  private String getRequestHeaders(){
    StringBuffer request = new StringBuffer(1024);
    request.append("Connection: close\r\n"); //connection will be closed after the response
    requestHeaders.remove("connection");
    request.append("Date: ");
    dateFormatter.format(new Date(), request, new java.text.FieldPosition(0));
    request.append("\r\n");
    Iterator it = requestHeaders.entrySet().iterator();
    while(it.hasNext()){
      Map.Entry entry = (Map.Entry)it.next();
      request.append(entry.getKey());
      request.append(": ");
      request.append(entry.getValue());
      request.append("\r\n");
    }
	
	addProxyAuthenticationHeader(request);	
	
    request.append("\r\n");
    //System.out.println("sending requestHeaders:\n"+request);
    return request.toString();
  }

  private void addProxyAuthenticationHeader(StringBuffer buffer)
  {
	if(usingProxy())
	{
		// Proxy Authentication			
		InetAddress addr;
		
		try
		{
			addr = InetAddress.getByName(System.getProperty("http.proxyHost"));
		}
		catch(UnknownHostException ex)
		{
			throw new IllegalStateException("cannot resolve http.proxyHost");
		}		
		
		int port = getProxyPort();
		
		String protocol = new String("http");
		String prompt = new String("Proxy Authentication required");
		String scheme = new String("[not set]");
		
		PasswordAuthentication passwordAuth = Authenticator.requestPasswordAuthentication(addr, port, protocol, prompt, scheme);

		if(passwordAuth!=null)
		{		
			buffer.append("Proxy-authorization: Basic ");
			
			StringBuffer unencoded = new StringBuffer(passwordAuth.getUserName());
			unencoded.append(':');
			unencoded.append(passwordAuth.getPassword());											
			
			String encUserPass = Base64Encoder.encode(unencoded.toString());
			
			buffer.append(encUserPass);
			
			buffer.append("\r\n");
		}
	}	  
  }  

  private void sendPartialHeaders(OutputStream out) throws IOException {
    StringBuffer request = new StringBuffer(1024);
    request.append("Connection: close\r\n"); //connection will be closed after the response
    requestHeaders.remove("connection");
    requestHeaders.remove("content-length");
    request.append("Date: ");
    dateFormatter.format(new Date(), request, new java.text.FieldPosition(0));
    request.append("\r\n");
    Iterator it = requestHeaders.entrySet().iterator();
    while(it.hasNext()){
      Map.Entry entry = (Map.Entry)it.next();
      request.append(entry.getKey());
      request.append(": ");
      request.append(entry.getValue());
      request.append("\r\n");
    }
	
	addProxyAuthenticationHeader(request);	
	
    int length = request.length();
    char[] chars = new char[length];
    request.getChars(0,length,chars,0);

    //System.out.println("sending requestPartialHeaders:\n"+request);
    out.write(decoder.cToB(chars,0,length));
  }

  private void requestGET() throws IOException {
    OutputStream out = socket.getOutputStream();
    out.write(getRequestLine().getBytes());
    out.write(getRequestHeaders().getBytes());
    doOutput = false;
    parseResponse();
  }

  void parseResponse() throws IOException {
    //System.out.println("parsing Response");
    try {
      if(probeStatusLine()){
        String line = readLine(false);
        int index = line.indexOf(' ');
        String version = line.substring(0,index++);
        int end = line.indexOf(' ',index);
        responseCode = Integer.parseInt(line.substring(index,end++));
        responseMessage = line.substring(end);
        while(!"".equals(line)){
          index = line.indexOf(':');
          if(index != -1){
            Object key = line.substring(0,index);
            if(key.equals("")){
              key = new Integer(keys.size());
            }
            responseHeaders.put(key, line.substring(index+1).trim());
            keys.add(key);
          }
          else {
            Object key = new Integer(keys.size());
            responseHeaders.put(key, line.trim());
            keys.add(key);
          }
          line = readLine(true);
        }
      }
    }
    catch(RuntimeException rt){
      rt.printStackTrace();
      throw new IOException();
    }

    if("Chunked".equalsIgnoreCase(responseHeaders.getProperty("Transfer-Encoding"))){
      //System.out.println("CREATING A CHUNCKED INPUTSTREAM !!!");
      in = new ChunkedInputStream(in);
    }
	
    if(responseCode==100) {
        //System.out.println("HTTP/1.1 100 Continue recognized. Parse next header");
        parseResponse();
    }
	else if((responseCode==HTTP_MOVED_PERM) || (responseCode==HTTP_MOVED_TEMP)) {
		String location = getHeaderField("Location");
		
		if(location==null) {
			throw new IllegalStateException("Handling of HTTP 301,302: Header 'Location' is missing.");
		}
		
		this.url = new URL(location);

		disconnect();
		connect();
	}
	else if((responseCode<200) || (responseCode>=300)) {
		StringBuffer buffer = new StringBuffer();		
		
		buffer.append("Server returned HTTP response code: ");
		buffer.append(responseCode);
		buffer.append(" for URL: ");
		buffer.append(url.toString());

		throw new IOException(buffer.toString());
	}		
  }

  /**
  ** check if the first bytes match a status-line, otherwise we get a simple response (= HTTP/0.9).
  */
  private boolean probeStatusLine() throws IOException {
    byte[] bytes = new byte[5];
    in.mark(5);
    int len = in.read(bytes,0,5);
    in.reset();
    if(len == -1){
      //System.out.println("probeStatusLine return false");
      return false;
    }
    //System.out.println("probeStatusLine "+new String(bytes,0,len));
    return "HTTP/".equals(new String(bytes,0,len));
  }

  private String readLine(boolean continued) throws IOException {
    StringBuffer buf = new StringBuffer(64);
    int ch = in.read();
    while(ch != -1){
      boolean stop = false;
      if(ch == '\r'){
        stop = true;
        in.mark(1);
        ch = in.read();
      }
      if(ch == '\n'){
        if(buf.length() == 0){
          break;
        }
        stop = true;
        in.mark(1);
        ch = in.read();
      }
      if(stop){
        if(continued && (ch == ' ' || ch  == '\t')){
          continue;
        }
        else {
          in.reset();
          break;
        }
      }
      buf.append((char)ch);
      ch = in.read();
    }

    //System.out.println("readLine returns '"+buf);
    return buf.toString();
  }
}
