/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package com.acunia.wonka.net.http;

import java.io.*;

class HttpOutputStream extends OutputStream {

  private static final byte[] CHUNKED = "Transfer-Encoding: chunked\r\n\r\n".getBytes();
  private static final byte[] NEWLINE = new byte[]{(byte)'\r',(byte)'\n'};

  private OutputStream out;

  private int count;
  private byte[] buffer;
  private boolean closed;
  private boolean chunked;
  private BasicHttpURLConnection httpCon;

  HttpOutputStream(OutputStream out, BasicHttpURLConnection httpC) throws IOException {
    this.out = out;
    httpCon = httpC;
    buffer = new byte[4096];
  }

  public void close() throws IOException {
    if(!closed){
      closed = true;
      if(chunked){
        if(count > 0){
          flushBuffer(buffer,0);
        }
        out.write('0');
        out.write(NEWLINE,0,2);
        out.write(NEWLINE,0,2);
      }
      else {
        out.write(("content-length: "+count+"\r\n\r\n").getBytes());
        out.write(buffer,0,count);
        buffer = null;
      }
      out.flush();
      httpCon.parseResponse();
    }
  }

  public void flush() throws IOException {
    if(closed){
      throw new IOException("Stream is closed");
    }
    if(chunked && count > 0){
      flushBuffer(buffer, 0);
    }
  }

  public void write(int b) throws IOException {
    if(closed){
      throw new IOException("Stream is closed");
    }
    if(count == buffer.length){
      flushBuffer(buffer, 0);
    }
    buffer[count++] = (byte)b;
  }

  public void write(byte[] bytes, int off, int length) throws IOException {
    if(closed){
      throw new IOException("Stream is closed");
    }
    if(length > (buffer.length - count)){
      if(count > 0){
        flushBuffer(buffer, 0);
      }
      if(length > buffer.length){
        count = length;
        flushBuffer(bytes, off);
        return;
      }
    }
    System.arraycopy(bytes,off,buffer,count,length);
    count += length;
  }

  private void flushBuffer(byte[] buf, int off) throws IOException {
    if(!chunked){
      chunked = true;
      out.write(CHUNKED,0,CHUNKED.length);
    }
    out.write((Integer.toHexString(count)+"\r\n").getBytes());
    out.write(buf,off, count);
    count = 0;
    out.write(NEWLINE,0,2);
    out.flush();
  }
}
