/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


package java.io;

import java.net.URL;
import java.net.MalformedURLException;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.NoSuchElementException;
import java.util.Vector;
import java.util.ArrayList;
import java.util.Random;

public class File implements Comparable, Serializable {

  private static final long serialVersionUID = 301077366599181567L;

  private static Vector toBeDeleted;
  
  public static final String separator;
  public static final char   separatorChar;
  public static final String pathSeparator;
  public static final char   pathSeparatorChar;

  private static String current_working_dir = null;
  private static String current_root_dir = null;

  private transient  String     prefix;
  private transient  String     dirpath;
  private transient  boolean    absolute;
  private transient  String     filename;
  private transient  String     fullname;
  private transient  String     canonicalPath;
  private transient  String     absolutePath;
  private transient  String     absname;  // Absolute filesystem path = ./blah/blah, only for internal use
                               // The . is needed to get a relative path inside fsroot

  private ByteArrayOutputStream baos;
  private byte[] b;

  private static native String get_CWD();
  private static native String get_root();
  private static native void init();
  
  static {
    init();
    int i = 0;
    
    current_working_dir = get_CWD();
    current_root_dir = get_root();
    char working[] = current_working_dir.toCharArray();
    char root[] = current_root_dir.toCharArray();

    /*
    ** Substract the root path from the current path.
    */
    int root_length = root.length;
    int pwd_length = working.length;

    while(i < root_length &&
          i < pwd_length &&
          working[i] == root[i]) i++;
   
    if(i == root_length) {
      
      /*
      **  The current working directory is in the current root path.
      */

      current_working_dir = "/" + current_working_dir.substring(i) + "/";
    
    }
    else {

      /*
      ** The current working directory is outside the current root path.
      ** Change it to /
      */

      current_working_dir = "/";
    }
    
    separator = GetSystemProperty.FILE_SEPARATOR;
    pathSeparator = GetSystemProperty.PATH_SEPARATOR;
    separatorChar = separator.charAt(0);
    pathSeparatorChar = pathSeparator.charAt(0);

    System.getProperties().setProperty("user.dir", current_working_dir);
  }

  public static File[] listRoots() {
    return new File[] { new File("/") };
  }

  public static File createTempFile(String pre, String suf) throws IOException {
    return createTempFile(pre, suf, null);
  }

  public static File createTempFile(String pre, String suf, File dir) throws IOException {
    if(dir == null) {
      dir = new File(System.getProperties().getProperty("java.io.tmpdir", "/tmp"));
    }
    
    if(suf == null) {
      suf = ".tmp";
    }

    if(pre == null) {
      pre = "";
    }

    File result = new File(dir, pre + (new Random()).nextLong() + suf);
    result.createNewFile();

    return result;
  }

  private String pack(String path) {
    String result = "";
    LinkedList list = new LinkedList();
    StringTokenizer st = new StringTokenizer(path, "/");

     try {
      while (true) {
        String token = st.nextToken();
        if(!token.equals(".")) { 
          if(!token.equals("..")) {
            list.add(token);
          } else {
            if(list.size() > 0)
              list.removeLast();
          }
        }
      }
    }
    catch(NoSuchElementException e) {
    }

    Iterator iter = list.iterator();
    while(iter.hasNext()) 
      result += "/" + (String)iter.next();
      
    if(!path.startsWith("/") && (result.length() > 0)) result = result.substring(1);
    if(path.endsWith("/")) result += "/";

    return result; 
  }

  private String stripSlashes(String path) {
    String result = "";
    StringTokenizer st = new StringTokenizer(path, "/");

    try {
      while (true) {
        result += "/" + st.nextToken();
      }
    }
    catch(NoSuchElementException e) {
    }

    if(!path.startsWith("/")) result = result.substring(1);
    if(path.endsWith("/")) result += "/";

    return result; 
  }

  public File(String path) throws NullPointerException {
    String relpath;
    filename = "";

    if(path.length() == 0) path = current_working_dir;

    if(path.charAt(0) == separatorChar) {  // Absolute path
      relpath = path.substring(1);
      absolute = true;
      prefix = "" + separatorChar;
    } else {
      relpath = path;
      absolute = false;
      prefix = "";
    }

    try {

      int dirpathlen = relpath.lastIndexOf(separatorChar);
      if (dirpathlen < 0) {
        dirpath = "";
        filename = relpath;
      } else {
        dirpath = relpath.substring(0, dirpathlen);
        filename = relpath.substring(dirpathlen + 1);
      }
    } catch (IndexOutOfBoundsException e) {
      e.printStackTrace();
    }

    fullname = stripSlashes(path);

    absolutePath = (absolute ? "" : current_working_dir) + fullname;
    absname = "." + absolutePath;

  }

  public File(String dirname, String name) throws NullPointerException {
    this(name == null ? null : (dirname == null ? "" : (dirname.equals("") ? "" : dirname + separatorChar)) + name);
  }

  public File(File dir, String name) throws NullPointerException {
    this((dir == null ? "" : dir.getPath()), name);
  }

  public String toString() {
    return fullname;
  }

  public boolean equals(Object obj) {
    if ((obj != null) && obj instanceof File) {
      return (((File)obj).getPath().equals(getPath()));
    }
    return false;
  }

  public int compareTo(File pathname) {
    return (getPath().compareTo(pathname.getPath()));
  }

  public int compareTo(Object obj) {
    if (obj instanceof File) {
      return (absolutePath.compareTo(((File)obj).getAbsolutePath()));
    } else { throw new ClassCastException(); }
  }

  public int hashCode() {
    return getPath().hashCode() ^ 1234321;
  }

  public String getName() {
    String result = new String(filename);
    int snip = filename.lastIndexOf((int)separatorChar);
    if (snip >= 0) result = result.substring(snip+1);

    return result;
  }

  public String getPath() {
    return fullname;
  }

  public String getAbsolutePath() {
    return absolutePath;
  }

  public File getAbsoluteFile() {
    return new File(absolutePath);
  }

  public String getCanonicalPath() throws IOException {
    if(canonicalPath == null) {
      canonicalPath = pack(absolutePath);
    }
    return canonicalPath;
  }

  public File getCanonicalFile() throws IOException {
    return new File(getCanonicalPath());
  }

  public String getParent() {
    return (absolute ? "/" : "") + dirpath;
  }

  public File getParentFile() {
    return new File(getParent());
  }

  public boolean isAbsolute() {
    return absolute;
  }

  public boolean isHidden() {
    return (filename.charAt(0) == '.');
  }
  
  public boolean createNewFile() throws IOException {
    if(!this.exists()) {
      try {
        FileOutputStream fo = new FileOutputStream(this);
        fo.close();
        return true;
      } catch(FileNotFoundException e) {
        throw new IOException("Unable to create " + this);
      }
    }
    return false;
  }

  public native boolean exists() throws SecurityException;

  public native String[] list() throws SecurityException;

  public String[] list(FilenameFilter filter) throws SecurityException {
    String[] files = this.list();
    if(files == null){
      return null;
    }

    ArrayList alist = new ArrayList(files.length);
    for(int i=0; i < files.length; i++){
      if(filter.accept(this, files[i])){
        alist.add(files[i]);
      }
    }
    String[] result = new String[alist.size()];
    return (String[])alist.toArray(result);
  }

  public File[] listFiles() throws SecurityException {
    String[] filenames = this.list();
    if(filenames == null){
      return null;
    }
    File[] files = new File[filenames.length];
    for(int i=0; i<filenames.length; i++) 
      files[i] = new File(this, filenames[i]);
    return files;
  }

  public File[] listFiles(FilenameFilter filter) throws SecurityException {
    String[] filenames = this.list(filter);
    if(filenames == null){
      return null;
    }
    File[] files = new File[filenames.length];
    for(int i=0; i<filenames.length; i++)
      files[i] = new File(this, filenames[i]);
    return files;
  }

  public File[] listFiles(FileFilter filter) throws SecurityException {
    File[] files = this.listFiles();
    if(files == null){
      return null;
    }
    ArrayList alist = new ArrayList(files.length);
    for(int i=0; i < files.length; i++){
      if(filter.accept(files[i])){
        alist.add(files[i]);
      }
    }
    File[] result = new File[alist.size()];
    return (File[])alist.toArray(result);
  }

  public native boolean canRead() throws SecurityException;

  public native boolean canWrite() throws SecurityException;

  public native boolean isFile() throws SecurityException;

  public native boolean isDirectory() throws SecurityException;

  public native long lastModified() throws SecurityException;

  public boolean setLastModified(long time) throws SecurityException {
    if (time < 0)
      throw new IllegalArgumentException("Negative modification time: " + time);	  	  
	 
	if(canWrite())
	{	
		return setLastModifiedInternal(time);
	}
	else
	{
            return false;
        }
  }
  
  private native boolean setLastModifiedInternal(long time);

  public void deleteOnExit() throws SecurityException {
    if(toBeDeleted == null) {
      createShutdownHook();
    }
    toBeDeleted.add(this);
  }

  private synchronized static void  createShutdownHook(){
    if(toBeDeleted == null) {
      toBeDeleted = new Vector();
      Runtime.getRuntime().addShutdownHook(new Thread("File:deleteOnExit") {
        public void run() {
          Iterator iter = File.toBeDeleted.iterator();
          while(iter.hasNext()) {
            File f = (File)iter.next();
            try {
              if(f != null) f.delete();
            }
            catch(Exception e) {
            }
          }
        }
      });
    }
  }

  public boolean setReadOnly() throws SecurityException {
    return false;
  }

  public native long length() throws SecurityException;

  public native boolean mkdir() throws SecurityException;

  public boolean mkdirs() throws SecurityException {
    boolean result = false;
    if(!absolutePath.equals(separator)) {
       getParentFile().mkdirs();
       result = mkdir();
    }
    return result;
  }

  private native boolean rename(String src, String dest);
  
  public boolean renameTo(File dest) throws SecurityException {
    return rename(absolutePath, dest.getAbsolutePath());
  }

  public native boolean delete() throws SecurityException;

  public URL toURL() throws MalformedURLException {
    if(canonicalPath == null) {
      canonicalPath = pack(absolutePath);
    }
    return new URL("file", "", canonicalPath);
  }

}
