/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Character.java 3208 2002-11-22 16:57:24Z ruelens $
*/

package java.lang;

public final class Character implements java.io.Serializable,Comparable {

  private static final long serialVersionUID = 3786198910865385080L;

  public static final char MIN_VALUE = '\u0000';
  public static final char MAX_VALUE = '\uffff';
  public static final int MIN_RADIX = 2;
  public static final int MAX_RADIX = 36;
  public static final Class TYPE = Character.getWrappedClass();
  public static final byte SPACE_SEPARATOR = 12;
  public static final byte LINE_SEPARATOR = 13;
  public static final byte PARAGRAPH_SEPARATOR = 14;
  public static final byte UPPERCASE_LETTER = 1;
  public static final byte LOWERCASE_LETTER = 2;
  public static final byte TITLECASE_LETTER = 3;
  public static final byte MODIFIER_LETTER = 4;
  public static final byte OTHER_LETTER = 5;
  public static final byte DECIMAL_DIGIT_NUMBER = 9;
  public static final byte LETTER_NUMBER = 10;
  public static final byte OTHER_NUMBER = 11;
  public static final byte NON_SPACING_MARK = 6;
  public static final byte ENCLOSING_MARK = 7;
  public static final byte COMBINING_SPACING_MARK = 8;
  public static final byte DASH_PUNCTUATION = 20;
  public static final byte START_PUNCTUATION = 21;
  public static final byte END_PUNCTUATION = 22;
  public static final byte CONNECTOR_PUNCTUATION = 23;
  public static final byte OTHER_PUNCTUATION = 24;
  public static final byte MATH_SYMBOL = 25;
  public static final byte CURRENCY_SYMBOL = 26;
  public static final byte MODIFIER_SYMBOL = 27;
  public static final byte OTHER_SYMBOL = 28;
  public static final byte CONTROL = 15;
  public static final byte FORMAT = 16;
  public static final byte UNASSIGNED = 0;
  public static final byte PRIVATE_USE = 18;
  public static final byte SURROGATE = 19;

  /**
   ** The char which is wrapped by this Character.
   */
  private final char value;

  /**
   ** Create our native tables.
   */
  private static native void createTables();
  static {
    createTables();
  }

  /**
   ** Construct a Character wrapping a particular char
   */
  public Character(char value) {
    this.value = value;
  }

  /**
   ** The string representattion is simply a one-character string.
   */
  public String toString() {
    char[] onechar = new char[1];

    onechar[0] = this.value;

    return new String(onechar);
  }

  /**
   ** This Character is `equal' to any other Character which wraps the same
   ** char, and to nothing else.
   */
  public boolean equals(Object obj) {
    if (obj == null) {

      return false;

    }

    try {
      Character c = (Character)obj;

      return this.value == c.value;

    }
    catch (ClassCastException cce) {

      return false;

    }
  }

  /**
   ** The hashcode is equal to the wrapped char.
   */
  public int hashCode() {

    return (int) value;

  }

  /**
   ** Get the wrapped char.
   */
  public char charValue() {

    return value;

  }

  /**
   ** Compare the values of the wrapped char's.
   */
  public int compareTo(Character anotherCharacter) {
    char other_value = anotherCharacter.value;

    if (value > other_value) return 1;

    if (value < other_value) return -1;

    return 0;
  }

  /**
   ** Will throw a ClassCastException if object of comparison is not a Character.
   */
  public int compareTo(Object something) throws ClassCastException {
    return compareTo((Character)something);
  }

  /**
   ** Consult native code tables to see if the char is assigned in UnicodeData.txt.
   */
  public native static boolean isDefined(char ch);

  /**
   ** Consult native code tables to see if the char is a lower-case letter
   ** (has no lower-case equivalent in  UnicodeData.txt and either has an
   ** upper-case equivalent or has `SMALL LETTER' or `SMALL LIGATURE' in its name).
   */
  public native static boolean isLowerCase(char ch);

  /**
   ** Consult native code tables to see if the char is an upper-case letter
   ** (has no upper-case equivalent in  UnicodeData.txt and either has an
   ** lower-case equivalent or has `CAPITAL LETTER' or `CAPITAL LIGATURE' 
   ** in its name).
   */
  public native static boolean isUpperCase(char ch);

  /**
   ** Consult native code tables to see if the char is a title-case letter
   ** (has both lower-case and upper-case equivalents in UnicodeData.txt).
   */
  public native static boolean isTitleCase(char ch);

  /**
   ** Consult native code tables to see if the char is a letter (is not
   ** in the range 2000-2FFF, and either is one of the three letter cases 
   ** above, or has a general category code in UnicodeData.txt beginning 
   ** with `L', or is in the range 0100-1FFF and is not a Digit, see below).
   */
  public native static boolean isLetter(char ch);

  /**
   ** Consult native code tables to see if the char is a digit (is not
   ** in the range 2000-2FFF, and either has `DIGIT' in its name or
   ** general category code `Nd' in UnicodeData.txt, depending on which
   ** version of the spec you are reading).
   */
  public native static boolean isDigit(char ch);

  /**
   ** Consult native code tables to see if the char is whitespace (is one
   ** of 0009, 000A, 000B, 000C, 000D, 001C, 001D, 001E, or 001F, or has
   ** a general category code in UnicodeData.txt beginning with `Z' and
   ** does not have a compatibility mapping marked <nobreak>).
   */
  public native static boolean isWhitespace(char ch);

  /**
   ** A char is a letter or a digit if it is a letter or a digit.
   */
  public static boolean isLetterOrDigit(char ch) {
    return isLetter(ch) || isDigit(ch);
  }

  /**
   ** A `Java letter' is a letter or $_.
   */
  public static boolean isJavaLetter(char ch) {

    return (ch=='$') || (ch=='_') || Character.isLetter(ch);

  }

  /**
   ** A `Java letter or digit' is a Java letter or a digit.
   */
  public static boolean isJavaLetterOrDigit(char ch) {

    return (ch=='$') || (ch=='_') || Character.isLetterOrDigit(ch);

  }

  // DEPRECATED - use isWhitespace(char)
  public static boolean isSpace(char ch) {
    return ch=='\n' || ch=='\t' ||ch=='\f' || ch=='\r' || ch==' ';
  }

  /**
   ** Consult native code tables to see if the char is a formatting char
   ** (has general category code `Cf' in UnicodeData.txt).
   */
  private native static boolean isFormat(char ch);

  public native static char toLowerCase(char ch);

  public native static char toUpperCase(char ch);

  public native static char toTitleCase(char ch);

  private static native int digitValue(char ch);

  private static native int numericValue(char ch);

  public static int digit(char ch, int radix) {
    if(radix<MIN_RADIX || radix>MAX_RADIX) {

      return -1;
 
    }

    if(isDigit(ch)) {
      int dv = digitValue(ch);

      return dv < radix ? dv : -1 ;

    }

    if(ch>='A' && ch<='Z' && ch < 'A' + radix - 10) {

      return ch-'A'+10;

    }

    if(ch>='a' && ch<='z' && ch < 'a' + radix - 10) {

      return ch-'a'+10;
    } 

    return -1;

  }


  public static int getNumericValue(char ch) {
    int result = digit(ch, 36);

    if (result == -1) {
      result = numericValue(ch);
    }

    return result;
  }

  public static boolean isIdentifierIgnorable(char ch) {
    return ( 
         (ch < ' ' && !Character.isWhitespace(ch))
      || (ch > '\u007e' && ch < '\u00a0') 
      || isFormat(ch)
    );
  }

  public static boolean isJavaIdentifierPart(char ch) {
    if (isLetterOrDigit(ch)) {

      return true;

    }
       
    int type = getType(ch);

    return
      type == CURRENCY_SYMBOL
      || type == CONNECTOR_PUNCTUATION
      || type == LETTER_NUMBER
      || type == COMBINING_SPACING_MARK
      || type == NON_SPACING_MARK
      || isIdentifierIgnorable(ch);

  }

  public static boolean isJavaIdentifierStart(char ch) {
    if (isLetter(ch)) {

      return true;

    }
       
    int type = getType(ch);

    return
      type == CURRENCY_SYMBOL
      || type == LETTER_NUMBER
      || type == CONNECTOR_PUNCTUATION;

  }

  public static boolean isUnicodeIdentifierPart(char ch) {
    if (isLetterOrDigit(ch)) {

      return true;

    }
       
    int type = getType(ch);

    return
      type == CONNECTOR_PUNCTUATION
      || type == LETTER_NUMBER
      || type == COMBINING_SPACING_MARK
      || type == NON_SPACING_MARK
      || isIdentifierIgnorable(ch);

  }

  public static boolean isUnicodeIdentifierStart(char ch) {
    return isLetter(ch) || getType(ch) == LETTER_NUMBER;
  }

  public static boolean isSpaceChar(char ch) {
    int type = getType(ch);

    return
      type == SPACE_SEPARATOR
      || type == LINE_SEPARATOR
      || type == PARAGRAPH_SEPARATOR;
  }

  public static boolean isISOControl(char ch) {
    return ((int)ch <= 0x001f) || ((int)ch >= 0x007f && (int)ch <= 0x009f);
  }

  public static int getType(char ch) {
    String cat = getCategory(ch);
    char cat0 = cat.charAt(0);
    char cat1 = cat.charAt(1);

    switch (cat0) {
      case 'C':
        switch (cat1) {
          case 'c':
            return CONTROL;

          case 'f':
            return FORMAT;

          case 'n':
            return UNASSIGNED;

          case 'o':
            return PRIVATE_USE;

          case 's':
            return SURROGATE;

        }
      case 'L':
        switch (cat1) {
          case 'l':
            return LOWERCASE_LETTER;

          case 'm':
            return MODIFIER_LETTER;

          case 'o':
            return OTHER_LETTER;

          case 't':
            return TITLECASE_LETTER;

          case 'u':
            return UPPERCASE_LETTER;

        }
      case 'M':
        switch (cat1) {
          case 'c':
            return COMBINING_SPACING_MARK;

          case 'e':
            return ENCLOSING_MARK;

          case 'n':
            return NON_SPACING_MARK;

        }
      case 'N':
        switch (cat1) {
          case 'd':
            return DECIMAL_DIGIT_NUMBER;

          case 'l':
            return LETTER_NUMBER;

/* New in 1.4?
          case 'm':
            return MODIFIER_NUMBER;
*/
          case 'o':
            return OTHER_NUMBER;

        }
      case 'P':
        switch (cat1) {
          case 'c':
            return CONNECTOR_PUNCTUATION;

          case 'd':
            return DASH_PUNCTUATION;

          case 'e':
          case 'f':
            return END_PUNCTUATION;

          case 'o':
            return OTHER_PUNCTUATION;

          case 'i':
          case 's':
            return START_PUNCTUATION;

/* New in 1.4?
          case 'f':
            return FINAL_QUOTE_PUNCTUATION;

          case 'i':
            return INITIAL_QUOTE_PUNCTUATION;
*/
        }
      case 'S':
        switch (cat1) {
          case 'c':
            return CURRENCY_SYMBOL;

          case 'k':
            return MODIFIER_SYMBOL;

          case 'm':
            return MATH_SYMBOL;

          case 'o':
            return OTHER_SYMBOL;

        }
      case 'Z':
        switch (cat1) {
          case 'l':
            return LINE_SEPARATOR;

          case 'p':
            return PARAGRAPH_SEPARATOR;

          case 's':
            return SPACE_SEPARATOR;
          
        }
    }

    return UNASSIGNED;
  }

  public static char forDigit(int digit, int radix) {
    if (radix < MIN_RADIX || radix > MAX_RADIX) {

      return 0;

    }

    if (digit < 0 || digit >= radix) {

      return 0;

    }

    if (digit < 10) {

      return (char)(digit + '0');

    }

    return (char)(digit + 'a' - 10);
  }

  private native static Class getWrappedClass();

  private native static String getCategory(char ch);

  public static class Subset {

    private String name;

    protected Subset(String str){
      //TODO ...
      name = str;
    }

    public final boolean equals(Object o){
      return this == o;
    }

    public final int hashCode(){
      return super.hashCode();
    }

    public final String toString(){
      return super.toString();
    }
  }

  public static final class UnicodeBlock extends Subset {

    public static final UnicodeBlock ALPHABETIC_PRESENTATION_FORMS = new UnicodeBlock("ALPHABETIC_PRESENTATION_FORMS");
    public static final UnicodeBlock ARABIC = new UnicodeBlock("ARABIC");
    public static final UnicodeBlock ARABIC_PRESENTATION_FORMS_A = new UnicodeBlock("ARABIC_PRESENTATION_FORMS_A");
    public static final UnicodeBlock ARABIC_PRESENTATION_FORMS_B = new UnicodeBlock("ARABIC_PRESENTATION_FORMS_B");
    public static final UnicodeBlock ARMENIAN = new UnicodeBlock("ARMENIAN");
    public static final UnicodeBlock ARROWS = new UnicodeBlock("ARROWS");
    public static final UnicodeBlock BASIC_LATIN = new UnicodeBlock("BASIC_LATIN");
    public static final UnicodeBlock BENGALI = new UnicodeBlock("BENGALI");
    public static final UnicodeBlock BLOCK_ELEMENTS = new UnicodeBlock("BLOCK_ELEMENTS");
    public static final UnicodeBlock BOPOMOFO = new UnicodeBlock("BOPOMOFO");
    public static final UnicodeBlock BOX_DRAWING = new UnicodeBlock("BOX_DRAWING");
    public static final UnicodeBlock CJK_COMPATIBILITY = new UnicodeBlock("CJK_COMPATIBILITY");
    public static final UnicodeBlock CJK_COMPATIBILITY_FORMS = new UnicodeBlock("CJK_COMPATIBILITY_FORMS");
    public static final UnicodeBlock CJK_COMPATIBILITY_IDEOGRAPHS = new UnicodeBlock("CJK_COMPATIBILITY_IDEOGRAPHS");
    public static final UnicodeBlock CJK_SYMBOLS_AND_PUNCTUATION = new UnicodeBlock("CJK_SYMBOLS_AND_PUNCTUATION");
    public static final UnicodeBlock CJK_UNIFIED_IDEOGRAPHS = new UnicodeBlock("CJK_UNIFIED_IDEOGRAPHS");
    public static final UnicodeBlock COMBINING_DIACRITICAL_MARKS = new UnicodeBlock("COMBINING_DIACRITICAL_MARKS");
    public static final UnicodeBlock COMBINING_HALF_MARKS = new UnicodeBlock("COMBINING_HALF_MARKS");
    public static final UnicodeBlock COMBINING_MARKS_FOR_SYMBOLS = new UnicodeBlock("COMBINING_MARKS_FOR_SYMBOLS");
    public static final UnicodeBlock CONTROL_PICTURES = new UnicodeBlock("CONTROL_PICTURES");
    public static final UnicodeBlock CURRENCY_SYMBOLS = new UnicodeBlock("CURRENCY_SYMBOLS");
    public static final UnicodeBlock CYRILLIC = new UnicodeBlock("CYRILLIC");
    public static final UnicodeBlock DEVANAGARI = new UnicodeBlock("DEVANAGARI");
    public static final UnicodeBlock DINGBATS = new UnicodeBlock("DINGBATS");
    public static final UnicodeBlock ENCLOSED_ALPHANUMERICS = new UnicodeBlock("ENCLOSED_ALPHANUMERICS");
    public static final UnicodeBlock ENCLOSED_CJK_LETTERS_AND_MONTHS = new UnicodeBlock("ENCLOSED_CJK_LETTERS_AND_MONTHS");
    public static final UnicodeBlock GENERAL_PUNCTUATION = new UnicodeBlock("GENERAL_PUNCTUATION");
    public static final UnicodeBlock GEOMETRIC_SHAPES = new UnicodeBlock("GEOMETRIC_SHAPES");
    public static final UnicodeBlock GEORGIAN = new UnicodeBlock("GEORGIAN");
    public static final UnicodeBlock GREEK = new UnicodeBlock("GREEK");
    public static final UnicodeBlock GREEK_EXTENDED = new UnicodeBlock("GREEK_EXTENDED");
    public static final UnicodeBlock GUJARATI = new UnicodeBlock("GUJARATI");
    public static final UnicodeBlock GURMUKHI = new UnicodeBlock("GURMUKHI");
    public static final UnicodeBlock HALFWIDTH_AND_FULLWIDTH_FORMS = new UnicodeBlock("HALFWIDTH_AND_FULLWIDTH_FORMS");
    public static final UnicodeBlock HANGUL_COMPATIBILITY_JAMO = new UnicodeBlock("HANGUL_COMPATIBILITY_JAMO");
    public static final UnicodeBlock HANGUL_JAMO = new UnicodeBlock("HANGUL_JAMO");
    public static final UnicodeBlock HANGUL_SYLLABLES = new UnicodeBlock("HANGUL_SYLLABLES");
    public static final UnicodeBlock HEBREW = new UnicodeBlock("HEBREW");
    public static final UnicodeBlock HIRAGANA = new UnicodeBlock("HIRAGANA");
    public static final UnicodeBlock IPA_EXTENSIONS = new UnicodeBlock("IPA_EXTENSIONS");
    public static final UnicodeBlock KANBUN = new UnicodeBlock("KANBUN");
    public static final UnicodeBlock KANNADA = new UnicodeBlock("KANNADA");
    public static final UnicodeBlock KATAKANA = new UnicodeBlock("KATAKANA");
    public static final UnicodeBlock LAO = new UnicodeBlock("LAO");
    public static final UnicodeBlock LATIN_1_SUPPLEMENT = new UnicodeBlock("LATIN_1_SUPPLEMENT");
    public static final UnicodeBlock LATIN_EXTENDED_A = new UnicodeBlock("LATIN_EXTENDED_A");
    public static final UnicodeBlock LATIN_EXTENDED_ADDITIONAL = new UnicodeBlock("LATIN_EXTENDED_ADDITIONAL");
    public static final UnicodeBlock LATIN_EXTENDED_B = new UnicodeBlock("LATIN_EXTENDED_B");
    public static final UnicodeBlock LETTERLIKE_SYMBOLS = new UnicodeBlock("LETTERLIKE_SYMBOLS");
    public static final UnicodeBlock MALAYALAM = new UnicodeBlock("MALAYALAM");
    public static final UnicodeBlock MATHEMATICAL_OPERATORS = new UnicodeBlock("MATHEMATICAL_OPERATORS");
    public static final UnicodeBlock MISCELLANEOUS_SYMBOLS = new UnicodeBlock("MISCELLANEOUS_SYMBOLS");
    public static final UnicodeBlock MISCELLANEOUS_TECHNICAL = new UnicodeBlock("MISCELLANEOUS_TECHNICAL");
    public static final UnicodeBlock NUMBER_FORMS = new UnicodeBlock("NUMBER_FORMS");
    public static final UnicodeBlock OPTICAL_CHARACTER_RECOGNITION = new UnicodeBlock("OPTICAL_CHARACTER_RECOGNITION");
    public static final UnicodeBlock ORIYA = new UnicodeBlock("ORIYA");
    public static final UnicodeBlock PRIVATE_USE_AREA = new UnicodeBlock("PRIVATE_USE_AREA");
    public static final UnicodeBlock SMALL_FORM_VARIANTS = new UnicodeBlock("SMALL_FORM_VARIANTS");
    public static final UnicodeBlock SPACING_MODIFIER_LETTERS = new UnicodeBlock("SPACING_MODIFIER_LETTERS");
    public static final UnicodeBlock SPECIALS = new UnicodeBlock("SPECIALS");
    public static final UnicodeBlock SUPERSCRIPTS_AND_SUBSCRIPTS = new UnicodeBlock("SUPERSCRIPTS_AND_SUBSCRIPTS");
    public static final UnicodeBlock SURROGATES_AREA = new UnicodeBlock("SURROGATES_AREA");
    public static final UnicodeBlock TAMIL = new UnicodeBlock("TAMIL");
    public static final UnicodeBlock TELUGU = new UnicodeBlock("TELUGU");
    public static final UnicodeBlock THAI = new UnicodeBlock("THAI");
    public static final UnicodeBlock TIBETAN  = new UnicodeBlock("TIBETAN");

    public static UnicodeBlock of(char c){
      return null;
    }

    private UnicodeBlock(String name){
      super(name);
    }

  }
}
