/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: System.java 3945 2003-05-16 12:00:41Z dumon $
*/

package java.lang;

import wonka.vm.Etc;
import wonka.vm.Wonka;

import java.util.Properties;
import java.util.PropertyPermission;
import java.io.IOException;
import java.io.InputStream;
import java.io.PrintStream;
import java.io.BufferedOutputStream;

public final class System {

  public static InputStream in;
  public static PrintStream out;
  public static PrintStream err;

  private static SecurityManager theSecurityManager;
  private static Runtime theRuntime;
  private static Properties systemProperties;
  private static boolean initialized;

  static {

    theRuntime = Runtime.getRuntime();
    systemProperties = new Properties(createDefaultProperties());

    try {
      InputStream syspropstream = ClassLoader.getSystemResourceAsStream("system.properties");
      systemProperties.load(syspropstream);
    }
    catch (IOException e) {}

    parseCmdLineProperties();

    try {
      in = new wonka.io.StandardInputStream();

      boolean  system_out_autoflush = Boolean.valueOf(Wonka.getProperty("wonka.out.autoflush", "true")).booleanValue();
      out = new PrintStream(new wonka.io.StandardOutputStream(), system_out_autoflush);

      boolean  system_err_autoflush = Boolean.valueOf(Wonka.getProperty("wonka.err.autoflush", "true")).booleanValue();
      err = new PrintStream(new wonka.io.ErrorOutputStream(), system_err_autoflush);

      //FORCE SOME CALLS things that are postponed until the system properties were set up correctly
      ClassLoader.get_defaultProtectionDomain();
      ClassLoader.createApplicationClassLoader();

      //SETUP DEFAULT LOCALE
      java.util.Locale.setDefault(new java.util.Locale(getProperty("user.language", "en"), getProperty("user.country", "")));
      
      //SETUP DEFAULT TIMEZONE
      java.util.TimeZone.setDefault(null);
      
      String theManager = getProperty("java.security.manager");
      if("".equals(theManager) || "default".equals(theManager)) {
        theSecurityManager = new SecurityManager();
      }
      else {
        // TODO: Use the given class..
      }
      
      initialized = true;
    }
    catch(Throwable t){
      t.printStackTrace();
    }

    if (!initialized) {
      exit(1);
    }
  }

  private static native String[] getCmdLineProperties();

  private static void parseCmdLineProperties() {
    String[] props = getCmdLineProperties();
    if(props != null) {
      for(int i=0; i < props.length; i++) {
        String key, value;
        int equals_at = props[i].indexOf('=');
        if (equals_at < 0) {
          key = props[i];
          wonka.vm.Etc.woempa(10,"SystemInit: Setting system property '"+key+"' to \"\"");
          systemProperties.put(key,"");
        }
        else if (equals_at == 0) {
          wonka.vm.Etc.woempa(10,"SystemInit: Ignoring nameless property in "+props[i]);
        }
        else if (equals_at == props[i].length()-1) {
          key = props[i].substring(0,equals_at);
          wonka.vm.Etc.woempa(10,"SystemInit: Setting system property '"+key+"' to \"\"");
          systemProperties.put(key,"");
        }
        else {
          key = props[i].substring(0, equals_at);
          value = props[i].substring(equals_at + 1);
          wonka.vm.Etc.woempa(10,"SystemInit: Setting system property '"+key+"' to '"+value+"'");
          systemProperties.put(key,value);
        }
      }
    }
  }


  private static Properties createDefaultProperties() {
    Properties systemDefaultProperties = new Properties();

/*
** The 1.3 API states that the following should always be defined:
** java.version
** java.vendor
** java.vendor.url
** java.home
** java.vm.specification.version
** java.vm.specification.vendor
** java.vm.specification.name
** java.vm.version
** java.vm.vendor
** java.vm.name
** java.specification.version
** java.specification.vendor
** java.specification.name
** java.class.version
** java.class.path
** java.ext.dirs
** os.name
** os.arch
** os.version
** file.separator
** path.separator
** line.separator
** user.name
** user.home
** user.dir
**
** We support all of these as best we can, except java.vm.specification.*
** and java.specification.* (for legal reasons).
*/
    systemDefaultProperties.put("java.version","1.3");
    systemDefaultProperties.put("java.vendor","ACUNIA");
    systemDefaultProperties.put("java.vendor.url","http://www.acunia.com");
    systemDefaultProperties.put("java.home", Etc.getInstallationDir()+"/");
    systemDefaultProperties.put("java.vm.version", Etc.getVersion());
    systemDefaultProperties.put("java.vm.vendor", "ACUNIA");
    systemDefaultProperties.put("java.vm.name", "Wonka");
    systemDefaultProperties.put("java.class.version","46.0");
// java.class.path will be updated later in the boot process
    systemDefaultProperties.put("java.class.path","");
    systemDefaultProperties.put("java.ext.dirs",Etc.getInstallationDir()+"/ext/");
    systemDefaultProperties.put("os.name",Etc.getOSName());
    systemDefaultProperties.put("os.arch",Etc.getOSArch());
    systemDefaultProperties.put("os.version",Etc.getOSVersion());
    systemDefaultProperties.put("file.separator","/");
  /*
  ** N.B. file.encoding should be one of the six mandatory encodings,
  ** in canonical form.
  */
    systemDefaultProperties.put("file.encoding","8859_1");
    systemDefaultProperties.put("path.separator",":");
    systemDefaultProperties.put("line.separator","\n");
    systemDefaultProperties.put("user.name",Etc.getUserName());
    systemDefaultProperties.put("user.home",Etc.getUserHome());
    systemDefaultProperties.put("user.dir",Etc.getUserDir());
    systemDefaultProperties.put("user.language","en");

    return systemDefaultProperties;
  }

  public static void setIn(InputStream newIn) {
    if (in!=null) {
      try {
        Etc.woempa(9,"setIn: closing "+in);
        in.close();
      } catch (java.io.IOException e) {}
    }

    in = newIn;
  }

  public static void setOut(PrintStream newOut) {
    if (out!=null) {
        Etc.woempa(9,"setOut: closing "+in);
        out.close();
    }
    out = newOut;
  }

  public static void setErr(PrintStream newErr) {
    if (err!=null) {
        Etc.woempa(9,"setErr: closing "+in);
        err.close();
    }

    err = newErr;
  }

  public static SecurityManager getSecurityManager() { 
    return theSecurityManager;
  }

  public static void setSecurityManager(SecurityManager sm)
    throws SecurityException
  { 
    if (theSecurityManager!=null) {
      if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER
       || wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
        java.security.AccessController.checkPermission(new RuntimePermission("setSecurityManager"));
      }
    }
    theSecurityManager = sm;
  }

  public static native long currentTimeMillis();

  private static void propertyCheck(String propname) {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new PropertyPermission(propname, "read"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      if (theSecurityManager != null) {
        theSecurityManager.checkPropertyAccess(propname);
      }
    }
  }

  private static void propertiesCheck() {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new PropertyPermission("*", "read,write"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      if (theSecurityManager != null) {
        theSecurityManager.checkPropertiesAccess();
      }
    }
  }

  public static Properties getProperties() 
    throws SecurityException
  {
    propertiesCheck();

    return systemProperties;
  }

  public static void setProperties(Properties props) 
    throws SecurityException
  {
    propertiesCheck();

    systemProperties = props;
  }


  public static String getProperty(String key) 
    throws SecurityException
  {
    propertyCheck(key);

    try {

      return systemProperties.getProperty(key);

    }
    catch (NullPointerException npe) {

      return null;

    }
  }

    
  public static String getProperty(String key, String defaults)
    throws SecurityException
  {
    propertyCheck(key);

    try {

      return systemProperties.getProperty(key,defaults);

    }
    catch (NullPointerException npe) {

      return defaults;

    }
  }

  public static String setProperty(String key, String defaults)
    throws SecurityException
  {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new PropertyPermission(key, "write"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      if (theSecurityManager != null) {
        theSecurityManager.checkPermission(new PropertyPermission(key, "write"));
      }
    }
    return (String)systemProperties.setProperty(key,defaults);
  }


  public static void exit(int status) 
    throws SecurityException
  {
    theRuntime.exit(status);
  }

  public static void gc() {
    theRuntime.gc();
  }

  public static void runFinalization() {
    theRuntime.runFinalization();
  }

  public static void load(String libname) 
    throws SecurityException, UnsatisfiedLinkError
  {
    throw new UnsatisfiedLinkError("not implemented");
  }

  public static void loadLibrary(String libname) 
    throws SecurityException, UnsatisfiedLinkError
  {
    // throw new UnsatisfiedLinkError("not implemented");
    Runtime.getRuntime().loadLibrary(libname);
  }
  
  public static String mapLibraryName(String libname) {
    return "lib" + libname + ".so";
  }

  native public static void arraycopy(Object src, int srcOffset, Object dst, int dstOffset, int length)
    throws NullPointerException, ArrayStoreException, ArrayIndexOutOfBoundsException;

  public static native int identityHashCode(Object o);

}
