/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: InetAddress.java 4115 2003-12-04 15:37:02Z ruelens $ 
*/

/*
** 
** The Wonka kernel is software copyright by SmartMove NV (1999,2000).
** Please see the file Copyright for information on its legal use.
** 
*/
package java.net;

import java.util.StringTokenizer;
import java.util.Hashtable;
import java.io.*;

public class InetAddress implements Serializable {

  static final int TYPE_LOOPBACK = 1;
  static final int TYPE_IPV4 = 2;
  static final int TYPE_IPV6 = 10;

  private static final long serialVersionUID = 3286316764910316507L;
  private static InetAddress ownAddress;

  static final Hashtable addresses = new Hashtable();

  static final InetAddress loopbackAddress  = new Inet4Address(0x7f000001,"localhost","127.0.0.1");
  static final InetAddress allZeroAddress  = new Inet4Address(0,"0.0.0.0","0.0.0.0");

  static {
    try {
      ownAddress = new Inet4Address(getLocalName());
    }
    catch (UnknownHostException uhe) {
      //just to make sure ownAddress will not be null ...
      ownAddress = loopbackAddress;
    }
    addresses.put("loopback", loopbackAddress);
  }

/**
** InetAddress hosts static permission checks to avoid code duplication.
** Since all Socket classes use InetAddress this does NOT unneeded class loading ...
*/
  static void permissionCheck(String host) {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new SocketPermission(host, "resolve"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        sm.checkConnect(host, -1);
      }
    }
  }

  static void factoryCheck() {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new RuntimePermission("setFactory"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        sm.checkSetFactory();
      }
    }
  }

  static void listenCheck(int port) {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      if (port == 0) {
        java.security.AccessController.checkPermission(new SocketPermission("localhost:1024-","listen"));
      }
      else {
        java.security.AccessController.checkPermission(new SocketPermission("localhost:"+port,"listen"));
      }
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        sm.checkListen(port);
      }
    }
  }

  static void acceptCheck(String host, int port) {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      java.security.AccessController.checkPermission(new SocketPermission(host+":"+port,"accept"));
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        sm.checkAccept(host, port);
      }
    }
  }

  static void connectCheck(String host, int port) {
    if (wonka.vm.SecurityConfiguration.USE_ACCESS_CONTROLLER) {
      if (port == -1) {
        java.security.AccessController.checkPermission(new SocketPermission(host,"resolve"));
      }
      else {
        java.security.AccessController.checkPermission(new SocketPermission(host+":"+port,"connect"));
      }
    }
    else if (wonka.vm.SecurityConfiguration.USE_SECURITY_MANAGER) {
      SecurityManager sm = System.getSecurityManager();
      if (sm != null) {
        connectCheck(host, port);
      }
    }
  }

  //dictated by serialized form ...
  int family;
  int address;
  String hostName;

  //since all InetAddresses get cached and the address is used as a key we might as well cache it.
  transient String addressCache;

  /**
  ** creates an empty InetAddress used by peek
  */
  InetAddress(){}

  InetAddress(int ip, String name, String ipname, int type){
    address = ip;
    family = type;
    hostName = name;
    addressCache = ipname;
    synchronized(addresses){
      addresses.put(name,this);
      addresses.put(ipname,this);
    }
  }

  /**
  ** calling this constructor will add the address to the addressCache !
  */
  InetAddress(String name) throws UnknownHostException {
    int char0 = name.charAt(0);
    if(char0 >= '0' && char0 <= '9'){
      StringTokenizer s = new StringTokenizer(name,".");
      if(s.countTokens() == 4){
        try {
          for(int i = 0 ; i < 4 ; i++){
            int octet = Integer.parseInt(s.nextToken());
            if(octet < 0 || octet > 255){
              throw new UnknownHostException();
            }
            address = octet + (address<<8);
          }
        }
        catch(NumberFormatException nfe){
          throw new UnknownHostException();
        }
      }
      family = TYPE_IPV4;
      synchronized(addresses){
        addresses.put(getHostAddress(), this);
        addresses.put(name, this);
      }
    }
    else {
      createInetAddress(name);
      synchronized(addresses){
        addresses.put(this.getHostName(), this);
        addresses.put(this.getHostAddress(), this);
      }
    }
  }

  /**
   * Compares this object against the specified object
   */
  public boolean equals(Object obj) {
    if(!(obj instanceof InetAddress)){
      return false;
    }
    InetAddress other = (InetAddress)obj;
    return (this.address == other.address);
  }

  /**
   * Returns the raw IP address of this InetAddress object
   */
  public byte[] getAddress() {
   	byte[] octets = new byte[4];
   	octets[3] = (byte)address;
   	octets[2] = (byte)(address>>>8);
   	octets[1] = (byte)(address>>>16);
   	octets[0] = (byte)(address>>>24);
    return octets;
  }

  /**
   * Determines all the hosts IP addresses, given the hostname;
   */
  public static InetAddress[] getAllByName (String host) throws UnknownHostException
  {
    if (host==null || host.length()==0) {
      throw new UnknownHostException();
    }

    //the check is done in getByName
    //permissionCheck(host);

    // FOR NOW, lets do the same thing as for getByName
    InetAddress[] temp = new InetAddress[1];
    temp[0] = getByName(host);
    return temp;
  }

  /**
   * Determine the IP address, given the hostname
   */
  public static InetAddress getByName(String host) throws UnknownHostException {

    if (host==null){
      return loopbackAddress;
    }

    try {
      permissionCheck(host);
    }
    catch (IllegalArgumentException iae) {
      throw new UnknownHostException(host);
    }
    return getByNameImpl(host);
  }

  static InetAddress getByNameImpl(String host) throws UnknownHostException {
    Object cached = addresses.get(host.toLowerCase());
    if(cached != null){
      return (InetAddress)cached;
    }

    if(host.indexOf(':') != -1) {
      return new Inet6Address(host);
    }
    else {
      return new Inet4Address(host);
    }
  }

  /**
   * Returns IP address 'xxx.xxx.xxx.xxx'
   */
  public String getHostAddress() {
    if(addressCache == null){
      addressCache = intToIPString(address);
    }
    return addressCache;
  }

  private static String intToIPString(int address){
      StringBuffer result = new StringBuffer(16);
      result.append((address>>>24 & 0x0ff));
      result.append('.');
      result.append((address>>>16 & 0x0ff));
      result.append('.');
      result.append((address>>>8 & 0x0ff));
      result.append('.');
      result.append((address & 0x0ff));
      return result.toString();
  }

  static InetAddress createInetAddress(int ip){
    String ipname = intToIPString(ip);
    Object address = addresses.get(ipname);
    if(address == null){
      try {
        return new Inet4Address(ipname);
      } catch(UnknownHostException uhe){
        //this exception cannot be thrown on a 'real' IP-string, but to be sure ...
        return new Inet4Address(ip,ipname,ipname);
      }
    }
    return (InetAddress)address;
  }


  /**
   * Returns the hostname for this address
   */
  public String getHostName() {
    if(hostName == null){
      if(getHostAddress() != null && lookupName()) {
        hostName = addressCache;
      }
      else {
        addresses.put(hostName,this);
      }
    }
    permissionCheck(hostName);
    return hostName;
  }

  /**
   * Returns the localhost
   */
  public static InetAddress getLocalHost() 
    throws UnknownHostException 
  {
    try {
      permissionCheck(getLocalName());
      return ownAddress;
    }
    catch (SecurityException se) {
      return loopbackAddress;
    }
  }

  /**
   * returns the hashcode for this IP address
   */
  public int hashCode() {
    return address;
  }

  /**
   * Checks if the InetAddress is a multicast address
   * IP Multicast Addresses (Class D) range from 224.0.0.0 to 239.255.255.255
   */ 
  public boolean isMulticastAddress () {
    byte[] ip = getAddress();
    return (((ip[0]&0xFF) >223) && ((ip[0]&0xFF)<240));
  }

  /**
   * InetAddress -> String conversion
   */
  public String toString () {
    return getHostName() + "/" + getHostAddress();
  }
/*
  public boolean isIPv6 () {
    return family == TYPE_IPV6;
  }
*/

  /**
  ** should only be called when addressCache is not null
  */
  private native boolean lookupName();

  /**
  ** will find an IP address for the name
  */
  native void createInetAddress(String hostname) throws UnknownHostException;

  /**
  ** will find the name of this host ...
  */
  private static native String getLocalName();


  public static InetAddress getByAddress(byte[] addr) {
    return null;
  }

  public static InetAddress getByAddress(String host, byte[] addr) {
    return null;
  }

  public String getCanonicalHostName() {
    return null;
  }

  public boolean isAnyLocalAddress() {
    return false;
  }

  public boolean isLinkLocalAddress() {
    return false;
  }

  public boolean isLoopbackAddress() {
    return false;
  }

  public boolean isMCGlobal() {
    return false;
  }

  public boolean isMCLinkLocal() {
    return false;
  }

  public boolean isMCNodeLocal() {
    return false;
  }

  public boolean isMCOrgLocal() {
    return false;
  }

  public boolean isMCSiteLocal() {
    return false;
  }

  public boolean isSiteLocalAddress() {
    return false;
  }

}

