/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/**
 * $Id: JarFile.java 4024 2003-07-07 08:10:25Z ruelens $
 */
package java.util.jar;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class JarFile extends ZipFile {

  public static final String MANIFEST_NAME = "META-INF/MANIFEST.MF";

  private static final Manifest theEmptyManifest = new Manifest();

  static {
    new JarEntry("");
  }

  private Manifest manifest;

  public JarFile(File file) throws IOException {
   	this(file,false);
  }
  public JarFile(String fname) throws IOException {
  	this(new File(fname),false);
  }
  public JarFile(String fname, boolean verify) throws IOException {
  	this(new File(fname),verify);
  }

/**
** @status Verification of data is not supported !!!
**	
*/
  public JarFile(File file, boolean verify) throws IOException {
    super(file);
    manifest = theEmptyManifest;
    ZipEntry ze = super.getEntry(MANIFEST_NAME);
    if (ze != null){
      manifest = new Manifest(getInputStream(ze));
    }
  }

  public JarFile(File file, boolean verify, int mode) throws IOException {
    super(file, mode);
    manifest = theEmptyManifest;
    ZipEntry ze = super.getEntry(MANIFEST_NAME);
    if (ze != null){
      manifest = new Manifest(getInputStream(ze));
    }
  }

// this method should be implemented by JarFile itself but are inherited now from ZipFile ...
	//  public Enumeration entries() { super.entries(); }


  public ZipEntry getEntry(String name) {
    ZipEntry ze =  super.getEntry(name);
    if(ze != null) {
      JarEntry je = new JarEntry(ze);
     	je.setAttributes(manifest.getAttributes(je.getName()));
      return je;
    }
    return null;
  }


/**
** @remark should add security features ... !
*/
  public InputStream getInputStream (ZipEntry ze) throws IOException {
    return super.getInputStream(ze);
  }

  public JarEntry getJarEntry(String name) {
    ZipEntry ze =  super.getEntry(name);
    if(ze != null) {
      JarEntry je = new JarEntry(ze);
     	je.setAttributes(manifest.getAttributes(je.getName()));
      return je;
    }
    return null;
  }

  public Manifest getManifest() throws IOException {
    return this.manifest;
  }

}
