/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/**
 * $Id: ZipFile.java 4293 2006-02-08 14:28:09Z gerrit $
 */

package java.util.zip;

import java.io.File;
import java.io.RandomAccessFile;
import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.util.Enumeration;

/**
 ** Class for manipulating zip files.
 ** IMPORTANT: if you add any set...() methods to this class, be sure to
 ** override them in com.acunia.wonka.net.jar.ImmutableJarFile!
 */
public class ZipFile implements ZipConstants {

  public static final int OPEN_READ = 1;
  public static final int OPEN_DELETE = 4;

  private final static byte last = endCenDirS[3];

  private RandomAccessFile raf;
  private String name;

  private boolean isJar;

  private int nrEntries;
  String[] strings;
  long[] longs;
  int capacity;

  public ZipFile (String name) throws IOException {
  	this.name = name;
  	this.raf = new RandomAccessFile(name, "r");
  	getEntries();
  	if (strings == null){
 	   	throw new ZipException("Bad or No ZipFile ");
  	}
  }

  public ZipFile (File file) throws ZipException, IOException {
  	this.name = file.getPath();
  	this.raf = new RandomAccessFile(file, "r");
  	getEntries();
  	if (strings == null){
 	   	throw new ZipException("Bad or No ZipFile ");
  	}
  }

  public ZipFile (File file, int mode) throws ZipException, IOException {
     this(file);
     if(mode == OPEN_DELETE){
       throw new IllegalArgumentException("OPEN_DELELTE: Unsupported feature");
     }
  }

  public void close() throws IOException {
   	nrEntries = 0;
   	capacity = 1;
   	strings = new String[1];
   	longs = null;
	raf.close();
  }

  public Enumeration entries(){
  	return new Enum();
  }

  protected void finalize() throws IOException {
    close();
  }

  public ZipEntry getEntry(String zname) {
    int   hash = zname.hashCode() % capacity;
    while(true) {
      if(hash < 0){
        hash += capacity;
      }
      if(strings[hash] == null) {
        return null;
      }
      else if(zname.equals(strings[hash])) {
        break;
      }
      hash--;
    }
	  try {
    	return new ZipEntry(zname,longs[hash],this);
	  }
 	  catch(Exception e){}
	  return null;
  }

  public String getName(){
   	return name;
  }

  public InputStream getInputStream(ZipEntry ze) throws IOException{
    //check if entry belongs to this zipFile!
    if(ze.zipFile != this){
      throw new IOException("entry doesn't not belong to this file");
    }
    if(ze.initPointer != 0) {
      createEntry(ze);
    }
    return new ByteArrayInputStream(checkHeader(ze));
  }

  /**
  ** since jdk 1.2 ...
  */
  public int size() {
   	return nrEntries;
  }	

  /**
  ** called from constructor ... (no synchronization needed on raf)!
  */
  private void getEntries() throws IOException {
   	//our file system uses buffers of at least 1024 bytes
   	long pos = raf.length();
   	int size = (1024 > pos ? (int)pos : 1024);
   	byte [] bytes = new byte[size];
   	while (pos > 0) {
   		pos -= size;
   		raf.seek(pos);
   		raf.readFully(bytes,0,size);
   		size = locateEndCD(bytes, size);
   		if (size == -1) {
   		 	break;
   		}
   		if (size > 0) {
   		 	System.arraycopy(bytes,0,bytes,bytes.length-size,size);
   		}
     	size = (1024 > pos ? (int)pos : 1024)-size;
   	}
  }

  /**
  ** called from within constructor ... (no synchronization needed on raf)!
  */
  private int locateEndCD(byte[] bytes, int size) throws IOException {
    //we start looking for  endCenDirS
    for (int p=size  ; --p > -1 ; ) {
      if (bytes[p] == last) {
      //this might be the last byte of the signature!
        if (p >=3) {
          if (bytes[p-1] == endCenDirS[2] && bytes[p-2] == endCenDirS[1]
           && bytes[p-3] == endCenDirS[0]) {
            //we have found a signature ...
            if(readEntries(bytes, size, p+1)){
              return -1;
            }
          }
        }
        else {
          return p;         		
        } 	
      }
    }
    return 0;
  }

  /**
  ** called from constructor ... (no synchronization needed on raf)!
  */
  private boolean readEntries(byte [] bytes, int size, int pos) throws IOException {
   	int nrE = size - pos;
   	nrE = (16 > nrE ? nrE : 16);
	  byte [] b = new byte [16];
   	System.arraycopy(bytes, pos, b,0, nrE);
   	if (nrE < 16){
   		raf.readFully(b,nrE,16-nrE);
   	}
   	//b should contain 16 bytes of the possible header
   	nrE = (0x0ff&(char)b[6]) + (0x0ff &(char)b[7])*256;
   	long stP = bytesToLong(b, 12);
  	size = (int)bytesToLong(b, 8);
   	if (nrE < 0 || stP+(long)size > raf.length() || size < 0) {
   		// we encountered a fony header
  		return false;
   	}
   	raf.seek(stP);
   	b = new byte[size];
   	raf.readFully(b);
   	pos=0;
   	
   	//creating internal hastable structure.
   	int capacity = ((int)(nrE/0.74f)) + 1;
   	this.capacity = capacity;
    strings = new String[capacity];
    longs = new long[capacity];
    //end of creation.
       	
   	for (int i=0; i < nrE ; i++) {
      if (pos+46 > size) {
   		 	throw new ZipException("to little bytes for header");//bad Header ...   			
      }
   		if (cenFileHeaderS[0] != b[pos] || cenFileHeaderS[1] != b[pos+1]
   			|| cenFileHeaderS[2] != b[pos+2] || cenFileHeaderS[3] != b[pos+3]){
   		 	throw new ZipException("bad header in central file directory");//bad Header ...
   		}
   		long pointer = stP + pos;
   		// we have found a correct header ...
   		// lets skip useless bytes
   		pos+=28;
   		int len = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		pos+=18;
   		if (pos+len > size) {
   		 	throw new ZipException("not enough bytes in header");
   		}

   		String name = new String(b,pos,len);
   		pos-=36;
   		int hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		if (hlp != 0 && hlp != 8) {
   		 	throw new ZipException("unknown store/zip method "+hlp);
   		}
      pos += 20;

   		hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		pos+=2;
   		int com = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
   		if(hlp < 0 || com < 0 || pos+14+len+hlp+com > size) {
   		 	throw new ZipException("invalid header data");
   		}
   		pos+=10;
      //inlined put method.
      int hashcode = name.hashCode() % capacity;
      while(true){
        if(hashcode < 0){
          hashcode += capacity;
        }
        if(strings[hashcode] == null) {
          strings[hashcode] = name;
          longs[hashcode] = pointer;
          break;
        }
        else if(name.equals(strings[hashcode])) {
          throw new IOException("duplicate entry found "+name);
        }
        hashcode--;
      }
   		//end put method.
   		
   		pos+= 4+len;
   		if (hlp > 0) {
   		 	pos += hlp;
   		}
   		if (com > 0) {
   		 	pos += com;
   		}
   	}
   	nrEntries = nrE;
   	return true;
  }

  private byte[] checkHeader(ZipEntry ze) throws IOException {
    synchronized(raf){
      raf.seek(ze.pointer);
     	byte [] header = new byte [30];
      raf.readFully(header,0,30);
    	if ( header[0] != locFileHeaderS[0] || header[1] != locFileHeaderS[1]
    	  || header[2] != locFileHeaderS[2] || header[3] != locFileHeaderS[3]) {
    	 	throw new ZipException("corrupt header found");
    	}

    	// we have 30 bytes --> 4 are already checked  Signature
    	// version needed --> ... (2 bytes)
    	// general purpose bytes (2)
    	int mode = (0x0ff&(char)header[4]) + (0x0ff &(char)header[5])*256;
    	if (mode > 20) {
    		throw new ZipException("higher zip version needed");
    	}
    	mode = (0x08 & header[6]);
    	int fnlen = (0x0ff & header[26]) + (0x0ff & header[27])*256;
    	byte [] fname = new byte[fnlen];

    	raf.readFully(fname,0, fnlen);
    	String name = new String(fname, 0, fnlen);
    	if (!ze.name.equals(name)) {
    	 	throw new ZipException("entry pointed to wrong data '"+name+"' != '"+ze.name+"'");
    	}
    	fnlen = (0x0ff&(char)header[28]) + (0x0ff &(char)header[29])*256;
    	fname = new byte[fnlen];
    	if (fnlen != 0) {
    		raf.readFully(fname,0, fnlen);
    	}

    	mode = (0x0ff&(char)header[8]) + (0x0ff &(char)header[9])*256;

      if (mode != 8 && mode != 0) {
  	 	  throw new ZipException("stream is corrupted");
      }

      if (mode == 8) {
      	header = new byte[(int) ze.compressedSize];
      	raf.readFully(header);
        header = quickInflate(header, (int)ze.size,(int)ze.crc);
      }
      else {
         	header = new byte[(int) ze.size];
         	raf.readFully(header);
      }
      return header;
    	// the rest of the bytes are not verified
    }
  }

  void createEntry(ZipEntry ze) throws IOException {
		synchronized(raf){
  		raf.seek(ze.initPointer);
  		int pos = 0;
  		byte[] b = new byte[46];
  		raf.readFully(b);
  		if (cenFileHeaderS[0] != b[pos] || cenFileHeaderS[1] != b[pos+1]
  			|| cenFileHeaderS[2] != b[pos+2] || cenFileHeaderS[3] != b[pos+3]){

  			throw new ZipException("bad header in central file directory");//bad Header ...
  		}
  		// we have found a correct header ...
  		// lets skip useless bytes
  		pos+=28;
  		int len = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
  		pos+=18;
  		byte[] name = new byte[len];
  		raf.readFully(name);
  		pos-=36;
  		int hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
  		if (hlp != 0 && hlp != 8) {
  		 	throw new ZipException("unknown store/zip method "+hlp);
  		}
  		ze.method = hlp;
  		pos+=2;
  		ze.time = getDate(b,pos);
  		pos+=4;
  		ze.crc = bytesToLong(b,pos);
  		pos+=4;
  		ze.compressedSize = bytesToLong(b,pos);
  		pos+=4;
  		ze.size = bytesToLong(b,pos);
  		pos+=6;
  		hlp = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
  		pos+=2;
  		int com = (0x0ff&(char)b[pos]) + (0x0ff &(char)b[pos+1])*256;
  		if(hlp < 0 || com < 0) {
  		 	throw new ZipException("invalid header data");
  		}
  		pos += 10;
  		ze.pointer = bytesToLong(b,pos);
  		if (hlp > 0) {
    	 	byte [] extra = new byte[hlp];
  		 	raf.readFully(extra);
  		 	ze.extra = extra;
  		}
  		if (com > 0) {
        byte[] comment = new byte[com];
  		 	raf.readFully(comment);
        ze.comment = new String(comment);
  		}
  		ze.initPointer = 0;
    }
  }

  private class Enum implements Enumeration  {

    int index = -1;

    Enum(){
      for(int i=0 ; i < capacity ; i++){
        if(strings[i] != null){
          index = i;
          break;
        }
      }
    }

    public boolean hasMoreElements() {
     	return index != -1;
    }

    public Object nextElement(){
      if(index == -1){
        throw new java.util.NoSuchElementException();
      }
      long l = longs[index];
      String ename = strings[index];
      int i=index+1;
      index = -1;
      for( ; i < capacity ; i++){
        if(strings[i] != null){
          index = i;
          break;
        }
      }
      try {
        return getEntry(ename);
      }
      catch(Exception e){
        return null;
      }
    }
  }

  private native byte[] quickInflate(byte[] cData, int uSize, int CRC) throws ZipException;

/**
**	these methods are static (default acces) so ZipInputStream is able call it
*/	
  native static long bytesToLong(byte [] header, int offset);
  native static long getDate(byte [] header, int offset);

}
