/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Finalizer.java 4293 2006-02-08 14:28:09Z gerrit $
*/

package wonka.vm;

public final class Finalizer implements Runnable {

  private static Finalizer theFinalizer;

  private Object[] empty;
  private java.lang.reflect.Method finalizer_method;
  private java.lang.reflect.Method enqueue_method;

  /**
   ** Our very private constructor.  We look up the methodID's for the
   ** methods Reference/enqueue() and Object/finalize() and start our
   ** thread running.
   */
  private Finalizer() {

    try {
      Class java_lang_Object = Class.forName("java.lang.Object");
      finalizer_method = java_lang_Object.getDeclaredMethod("finalize", new Class[0]);
      finalizer_method.setAccessible(true);

      Class java_lang_ref_Reference = Class.forName("java.lang.ref.Reference");
      enqueue_method = java_lang_ref_Reference.getDeclaredMethod("enqueue", new Class[0]);
      enqueue_method.setAccessible(true);
    }
    catch (Exception e) {
      e.printStackTrace();
    }
    empty = new Object[0];
    Thread thread = new Thread(this, "Confessor");
    thread.setPriority(10);
    thread.setDaemon(true);
    thread.start();

  }

  /**
   ** Return the solitary instance of Finalizer.
   */
  public static synchronized Finalizer getInstance() {
    if (theFinalizer == null) {
      theFinalizer = new Finalizer();
    }

    return theFinalizer;
  }

  /**
   ** The run() method polls the two internal fifo's and calls enqueue()
   ** or finalize() as appropriate.
   */
  public void run() {
    boolean idle;
    Object o = null;

    while(true) {
      idle = false;
      try {
        synchronized(this) {
          o = nextFinalizee();
        }

        if (o == null) {
          synchronized(this) {
            notifyAll();
          }
          idle = true;
        }
        else {
          try {
            finalizer_method.invoke(o, empty);
          }
          catch(Throwable t) {
            // finalize threw something.. We don't care though, down the drain it goes !
          }
          finalized(o);
        }

        synchronized(this) {
          o = nextEnqueueee();
        }

        if (o != null) {
          idle = false;
          enqueue_method.invoke(o, empty);
          enqueued(o);
        }
        if (idle) {
          synchronized(this) {
            wait(1000);
          }
        }
      }
      catch (Throwable t) {
      }
    }
  }

  /**
   ** Poll the enqueue() fifo.
   */
  private native Object nextEnqueueee();

  /**
   ** Poll the finalize() fifo.
   */
  private native Object nextFinalizee();

  /**
   ** Tell GC that object has been finalized.
   */
  private native void finalized(Object o);

  /**
   ** Tell GC that object has been enqueued.
   */
  private native void enqueued(Object o);
}

