/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: pthread_wrapper.c 4191 2004-08-18 07:34:20Z dumon $
*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <errno.h>
#include <setjmp.h>
#include <oswald.h>

#define oempa(format, a...) fprintf(stderr, format, ##a)

#define STACK_SIZE (128 * 1024)
#define TSD_SIZE 100

#ifdef MONITOR_INFO
#define store_symbol(a) { x_word *address[3]; if(backtrace((void **)&address, 2) > 0) { ((x_monitor)(a))->xref = (void *)address[1]; } else { ((x_monitor)(a))->xref = NULL; } }
//#define store_symbol(a)
#endif

//void (*x_longjmp)(jmp_buf env, int val);
//void (*x_siglongjmp)(sigjmp_buf env, int val);

struct sched_param {
};

static volatile int key_count = 0;

static pthread_mutex_t **valid_mutexes = NULL;
static x_int valid_mutexes_count = 0;
static x_thread *pthreads = NULL;
static x_int pthreads_count = 0;

static void register_pthread(x_thread thread) {
  x_int count;
  x_thread *new_pthreads;

  x_preemption_disable;

  if(!pthreads) {
    pthreads = x_mem_calloc(sizeof(x_thread *) * 10);
    pthreads_count = 10;
  }

  for(count = 0; count < pthreads_count; count++) {
    if(!pthreads[count]) { 
      pthreads[count] = thread;
      x_preemption_enable;
      return;
    }
  }

  new_pthreads = x_mem_calloc(sizeof(x_thread *) * (pthreads_count + 10)); 
  memcpy(new_pthreads, pthreads, pthreads_count * sizeof(x_thread *));
  new_pthreads[pthreads_count] = thread;
  pthreads = new_pthreads;
  pthreads_count += 10;
  x_preemption_enable;
}

void pthread_check_dead_threads(void) {
  x_int count;
  x_thread thread;
  
  if(!pthreads) {
    return;
  }

  for(count = 0; count < pthreads_count; count++) {
    thread = pthreads[count];
    if(thread && thread->state == xt_ended) { 
      pthreads[count] = NULL;
      x_mem_free(thread->b_stack);
      x_mem_free(thread);
    }
  }
}

static x_int known_thread(pthread_t thread) {
  x_int count;
  
  if(!pthreads) {
    return false;
  }

  for(count = 0; count < pthreads_count; count++) {
    if(pthreads[count] == (x_thread)thread) { 
      return true;
    }
  }

  return false;
}

extern int pthread_create(pthread_t * thread, pthread_attr_t * attr, void *(*start_routine)(void *), void * arg) {
  x_thread thr;
  x_status status;
  
  //oempa("pthread_create(thread=%p, attr=%p, start_routine=%p, arg=%p)\n", thread, attr, start_routine, arg);

  thr = x_mem_calloc(sizeof(x_Thread));
  register_pthread(thr);
  status = x_thread_create(thr, (x_entry)start_routine, arg, x_mem_alloc(STACK_SIZE), STACK_SIZE, 20, TF_START);
  if (status != xs_success) {
    oempa("%s: status = '%s'\n", __FILE__, x_status2char(status));
    errno = EAGAIN;
    return -1;
  }

  *thread = (pthread_t)thr;
 
  return 0;
}

extern int pthread_cancel(pthread_t thread) {
  //oempa("pthread_cancel(thread=%p)\n", thread);
  if(!known_thread(thread)) return 0;
  x_thread_suspend((x_thread)thread);
  return 0;
}

extern void pthread_exit(void *retval) {
  //oempa("pthread_exit(retval=%p)\n", retval);
  x_thread_exit(retval);
}

extern int pthread_join(pthread_t th, void **thread_return) {
  if(!known_thread(th)) return 0;
  //oempa("pthread_join(th=%p, thread_return=%p)\n", th, thread_return);
  x_thread_join((x_thread)th, thread_return, x_eternal);
  return 0;
}

extern pthread_t pthread_self(void) {
//  oempa("pthread_self() = %p\n", thread_current);
  return (pthread_t)thread_current;
}

extern int pthread_attr_init(pthread_attr_t *attr) {
  //oempa("pthread_attr_init(attr=%p)\n", attr);
  return 0;
}

extern int pthread_attr_destroy(pthread_attr_t *attr) {
  //oempa("pthread_attr_destroy(attr=%p)\n", attr);
  return 0;
}

extern int pthread_attr_setdetachstate(pthread_attr_t *attr, int detachstate) {
  //oempa("pthread_attr_setdetachstate(attr=%p, detachstate=%d\n)", attr, detachstate);
  return 0;
}

extern int pthread_attr_getdetachstate(const pthread_attr_t *attr, int *detachstate) {
  //oempa("pthread_attr_getdetachstate(attr=%p, detachstate=%p\n)", attr, detachstate);
  return 0;
}

extern int pthread_attr_setschedpolicy(pthread_attr_t *attr, int policy) {
  //oempa("pthread_attr_setschedpolicy(attr=%p, policy=%d\n)", attr, policy);
  return 0;
}

extern int pthread_attr_getschedpolicy(const pthread_attr_t *attr, int *policy) {
  //oempa("pthread_attr_getschedpolicy(attr=%p, policy=%p\n)", attr, policy);
  return 0;
}

extern int pthread_attr_setschedparam(pthread_attr_t *attr, const struct sched_param *param) {
  //oempa("pthread_attr_setschedparam(attr=%p, param=%p\n)", attr, param);
  return 0;
}

extern int pthread_attr_getschedparam(const pthread_attr_t *attr, struct sched_param *param) {
  //oempa("pthread_attr_getschedparam(attr=%p, param=%p\n)", attr, param);
  return 0;
}

extern int pthread_attr_setinheritsched(pthread_attr_t *attr, int inherit) {
  //oempa("pthread_attr_setinheritsched(attr=%p, inherit=%d)\n", attr, inherit);
  return 0;
}

extern int pthread_attr_getinheritsched(const pthread_attr_t *attr, int *inherit) {
  //oempa("pthread_attr_getinheritsched(attr=%p, inherit=%p)\n", attr, inherit);
  return 0;
}

extern int pthread_attr_setscope(pthread_attr_t *attr, int scope) {
  //oempa("pthread_attr_setscope(attr=%p, scope=%d)\n", attr, scope);
  return 0;
}

extern int pthread_attr_getscope(const pthread_attr_t *attr, int *scope) {
  //oempa("pthread_attr_getscope(attr=%p, scope=%p)\n", attr, scope);
  return 0;
}

extern int pthread_setcancelstate(int state, int *oldstate) {
  //oempa("pthread_setcancelstate(state=%d, oldstate=%p)\n", state, oldstate);
  return 0;
}

extern int pthread_setcanceltype(int type, int *oldtype) {
//  oempa("pthread_setcanceltype(type=%d, oldtype=%p)\n", type, oldtype);
  return 0;
}

extern void pthread_testcancel(void) {
  //oempa("pthread_testcancel()\n");
}

static void register_mutex(pthread_mutex_t *mutex) {
  x_int count;
  pthread_mutex_t **new_valid;

  x_preemption_disable;

  if(!valid_mutexes) {
    valid_mutexes = x_mem_calloc(sizeof(pthread_mutex_t *) * 10);
    valid_mutexes_count = 10;
  }

  for(count = 0; count < valid_mutexes_count; count++) {
    if(!valid_mutexes[count]) { 
      valid_mutexes[count] = mutex;
      x_preemption_enable;
      return;
    }
  }

  new_valid = x_mem_calloc(sizeof(pthread_mutex_t *) * (valid_mutexes_count + 10)); 
  memcpy(new_valid, valid_mutexes, valid_mutexes_count * sizeof(pthread_mutex_t *));
  new_valid[valid_mutexes_count] = mutex;
  valid_mutexes = new_valid;
  valid_mutexes_count += 10;
  x_preemption_enable;
}

static void deregister_mutex(pthread_mutex_t *mutex) {
  x_int count;
  
  if(!valid_mutexes) {
    return;
  }

  for(count = 0; count < valid_mutexes_count; count++) {
    if(valid_mutexes[count] == mutex) { 
      valid_mutexes[count] = NULL;
    }
  }
}

static x_int known_mutex(pthread_mutex_t *mutex) {
  x_int count;
  
  if(!valid_mutexes) {
    return false;
  }

  for(count = 0; count < valid_mutexes_count; count++) {
    if(valid_mutexes[count] == mutex) { 
      return true;
    }
  }

  return false;
}

extern int pthread_mutex_init(pthread_mutex_t *mutex, const pthread_mutexattr_t *mutexattr) {
//  oempa("pthread_mutex_init(mutex=%p, mutextattr=%p\n", mutex, mutexattr);

  x_monitor mon;
  
  if(!known_mutex(mutex)) {
    mon = x_mem_calloc(sizeof(x_Monitor));
    x_monitor_create(mon);
#ifdef MONITOR_INFO
    mon->xref_type = 2;
#endif
    *(x_monitor *)mutex = mon;

    register_mutex(mutex);
  }

  return 0;
}

void stack(void) {
  void *address[1024];
  int count;
 
  w_dump("\nNative stacktrace:\n\n");

  count = backtrace((void **)&address, 1024);
  backtrace_symbols_fd((void **)&address, count, 2);
  w_dump("\n");
  
}

extern int pthread_mutex_lock(pthread_mutex_t *mutex) {
  //oempa("%p pthread_mutex_lock(mutex=%p)\n", thread_current, mutex);
  x_preemption_disable;
  if(!known_mutex(mutex)) {
    pthread_mutex_init(mutex, NULL);
  }
  //if(!*(x_monitor *)mutex) pthread_mutex_init(mutex, NULL);
  x_preemption_enable;
  x_monitor_enter(*(x_monitor *)mutex, x_eternal);
#ifdef MONITOR_INFO
  store_symbol(*(x_monitor *)mutex);
#endif
  //oempa("mutex entered\n");
  return 0;
}

extern int pthread_mutex_trylock(pthread_mutex_t *mutex) {
  //oempa("%p pthread_mutex_trylock(mutex=%p)\n", thread_current, mutex);
  if((*(x_monitor *)mutex)->owner) {
    return EBUSY;
  }
  x_monitor_enter(*(x_monitor *)mutex, x_eternal);
#ifdef MONITOR_INFO
  store_symbol(*(x_monitor *)mutex);
#endif
  return 0;
}

extern int pthread_mutex_unlock(pthread_mutex_t *mutex) {
  //oempa("%p pthread_mutex_unlock(mutex=%p)\n", thread_current, mutex);
  x_monitor_exit(*(x_monitor *)mutex);
#ifdef MONITOR_INFO
  (*(x_monitor *)mutex)->xref = NULL;
#endif
  return 0;
}

extern int pthread_mutex_destroy(pthread_mutex_t *mutex) {
//  oempa("pthread_mutex_destroy(mutex=%p)\n", mutex);
  deregister_mutex(mutex);
  x_monitor_delete(*(x_monitor *)mutex);
  x_mem_free(*(x_monitor *)mutex);
  return 0;
}

extern int pthread_mutexattr_init(pthread_mutexattr_t *attr) {
//  oempa("pthread_mutexattr_init(attr=%p)\n", attr);
  return 0;
}

extern int pthread_mutexattr_destroy(pthread_mutexattr_t *attr) {
//  oempa("pthread_mutexattr_destroy(attr=%p)\n", attr);
  return 0;
}

extern int pthread_mutexattr_settype(pthread_mutexattr_t *attr, int kind) {
//  oempa("pthread_mutexattr_settype(attr=%p, kind=%d)\n", attr, kind);
  return 0;
}

extern int pthread_cond_init(pthread_cond_t *cond, pthread_condattr_t *cond_attr) {
  //oempa("pthread_cond_init(cond=%p, cond_attr=%p)\n", cond, cond_attr);
  *(int **)cond = x_mem_alloc(sizeof(x_Monitor));
  x_monitor_create((x_monitor)*((int *)cond));
#ifdef MONITOR_INFO
  ((x_monitor)*((int *)cond))->xref_type = 1;
#endif
  return 0;
}

extern int pthread_cond_signal(pthread_cond_t *cond) {
  x_monitor monitor = (x_monitor)*((int *)cond);
  //oempa("pthread_cond_signal(cond=%p)\n", cond);
  x_monitor_enter(monitor, x_eternal);
  x_monitor_notify(monitor);
  x_monitor_exit(monitor);
  return 0;
}

extern int pthread_cond_broadcast(pthread_cond_t *cond) {
  x_monitor monitor = (x_monitor)*((int *)cond);
  //oempa("%p pthread_cond_broadcast(cond=%p)\n", thread_current, cond);
  x_monitor_enter(monitor, x_eternal);
  x_monitor_notify_all(monitor);
  x_monitor_exit(monitor);
  return 0;
}

extern int pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex) {
  x_monitor monitor = (x_monitor)*((int *)cond);
  //oempa("%p pthread_cond_wait(cond=%p, mutex=%p)\n", thread_current, cond, mutex);
  x_monitor_enter(monitor, x_eternal);
  // oempa("entered\n");
  x_monitor_exit(*(x_monitor *)mutex);
#ifdef MONITOR_INFO
  (*(x_monitor *)mutex)->xref = NULL;
#endif
  x_monitor_wait(monitor, x_eternal);
  x_monitor_enter(*(x_monitor *)mutex, x_eternal);
#ifdef MONITOR_INFO
  store_symbol(*(x_monitor *)mutex);
#endif
  // oempa("got notified\n");
  x_monitor_exit(monitor);
  // oempa("exited\n");
  return 0;
}

extern int pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex, const struct timespec *abstime) {
  x_monitor monitor = (x_monitor)*((int *)cond);
  //oempa("pthread_cond_timedwait(cond=%p, mutex=%p, timespec=%p)\n", cond, mutex, abstime);
  x_monitor_enter(monitor, x_eternal);
  x_monitor_wait(monitor, x_eternal);
  x_monitor_exit(monitor);
  return 0;
}

extern int pthread_cond_destroy(pthread_cond_t *cond) {
  x_monitor monitor = (x_monitor)*((int *)cond);
  //oempa("pthread_cond_destroy(cond=%p)\n", cond);
  x_monitor_release(monitor);
  return 0;
}

extern int pthread_once(pthread_once_t *once_control, void (*init_routine)(void)) {
//  oempa("pthread_once(once_control=%p, init_routine=%p)    (%d)\n", once_control, init_routine, (unsigned int)(*once_control));
  if((unsigned int)(*once_control) != 1234567) {
//    oempa("   This is once !\n");
    init_routine();
    *once_control = (pthread_once_t)1234567;
  }
  return 0;
}

extern int pthread_key_create(pthread_key_t *key, void (*destr_function) (void *)) {
  *key = key_count++;
  if(key_count >= TSD_SIZE) {
    errno = EAGAIN;
    return -1;
  }
  //oempa("%p pthread_key_create(key=%p, destr_function=%p) -> %d   %p\n", thread_current, key, destr_function, (int)*key, thread_current->tsd);
  return 0;
}

extern int pthread_key_delete(pthread_key_t key) {
  //oempa("%p pthread_key_delete(key=%d)\n", thread_current, (int)key);
  return 0;
}

extern int pthread_setspecific(pthread_key_t key, const void *pointer) {
  if(thread_current->tsd == NULL) {
    thread_current->tsd = x_mem_calloc(TSD_SIZE * sizeof(int));
  }
  if((int)key >= key_count) {
    errno = EINVAL;
    return -1;
  }
  ((int *)(thread_current->tsd))[(int)key] = (int)pointer;
  return 0;
}

extern void * pthread_getspecific(pthread_key_t key) {
  void *result;
  if(thread_current->tsd == NULL || (int)key >= key_count) {
    errno = EINVAL;
    return NULL;
  }
  result = (void *)((int *)thread_current->tsd)[(int)key];
  //oempa("%p pthread_getspecific(key=%d) = %p    %p\n", thread_current, (int)key, result, thread_current->tsd);
  return result;
}

extern int pthread_sigmask(int how, const sigset_t *newmask, sigset_t *oldmask) {
  //oempa("pthread_sigmask(how=%d, newmask=%p, oldmask=%p)\n", how, newmask, oldmask);
  return 0;
}

extern int pthread_kill(pthread_t thread, int signo) {
  //oempa("pthread_kill(thread=%p, signo=%d)\n", thread, signo);
  return 0;
}

extern unsigned int sleep(unsigned int seconds) {
  //oempa("sleep(seconds=%d)\n", seconds);
  x_thread_sleep(x_usecs2ticks(seconds * 1000000));
  return 1;
}

/*
extern int sigwait(const sigset_t *set, int *sig) {
  //oempa("sigwait(set=%p, sig=%p)\n", set, sig);
  return 0;
}

int (*x_msync)(const void *start, size_t length, int flags);

extern int msync(const void *start, size_t length, int flags) {
  //oempa("msync(start=%p, length=%d, flags=%d)\n", start, length, flags);
  return x_msync(start, length, flags);
}

int (*x_open)();

extern int open(char *file, int oflag, ...) {
  return x_open();
}
*/
/*
extern void longjmp(jmp_buf env, int val) {
  x_longjmp(env, val);
}

extern void siglongjmp(sigjmp_buf env, int val) {
  x_siglongjmp(env, val);
}
*/
void *(* x__res_state)(void);

extern void * __res_state(void) {
  return x__res_state();
}

int *(* x__h_errno_location)(void);

extern int* __h_errno_location(void) {
  return x__h_errno_location();
}

void start_pthread_wrapper(void) {
  void *libc = dlopen("/lib/libc.so.6", RTLD_GLOBAL | RTLD_LAZY);
//  x_open = dlsym(libc, "open");
  x__res_state = dlsym(libc, "__res_state");
  x__h_errno_location = dlsym(libc, "__h_errno_location");
//  x_siglongjmp = dlsym(libc, "siglongjmp");
//  x_longjmp = dlsym(libc, "longjmp");

//  x_msync = dlsym(libc, "msync");
}
