/**************************************************************************
* Copyright  (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.     *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** References of the form [J+JVM ...] are to the book _Java and the Java
** Virtual Machine_ by Robert Staerk, Joachim Schmid, and Egon Boerger.
*/

#include <string.h>

#include "arrays.h"
#include "clazz.h"
#include "constant.h"
#include "exception.h"
#include "interpreter.h"
#include "loading.h"
#include "locks.h"
#include "oswald.h"
#include "hashtable.h"
#include "heap.h"
#include "ts-mem.h"
#include "methods.h"
#include "wstrings.h"
#include "threads.h"
#include "ieee754.h"
#include "dispatcher.h"
#include "list.h"
#include "wonka.h"
#include "bar.h"

extern w_object object_sentinel;

/*
** Each classloader has associated with it a hashtable of loaded classes
** and one of unloaded classes. For the "primordial" class loader these are
** static items.
*/

w_hashtable system_loaded_class_hashtable;
w_hashtable system_unloaded_class_hashtable;

/*
** The clazz we clone arrays from. See also comments in wonka.h
*/

w_clazz clazz_Array;


w_clazz clazz_boolean;
w_clazz clazz_char;
w_clazz clazz_float;
w_clazz clazz_double;
w_clazz clazz_byte;
w_clazz clazz_short;
w_clazz clazz_int;
w_clazz clazz_long;
w_clazz clazz_void;

/*
** Three often used clazzes.
*/

w_clazz clazzArrayOf_Object;
w_clazz clazzArrayOf_Class;
w_clazz clazzArrayOf_String;

static void parseMethodCodeAttributes(w_method, w_bar);

/*
** Function used by x_snprintf to print out the name of a clazz (format %k).
*/

char *print_clazz_short(char *buffer, int *remain, void *c, w_int width, w_int precision, w_flags flags) {

  w_clazz clazz = c;
  w_int    nbytes;
  char    *temp;

  if (*remain < 1) {

    return buffer;

  }

  temp = buffer;

  if (c == NULL) {
    strncpy(temp, (char*)"<NULL>", *remain);
    if (*remain < 6) {
      temp += *remain;
      *remain = 0;
    }
    else {
      temp += 6;
      *remain -= 6;
    }

    return temp;

  }

  nbytes = x_snprintf(temp, *remain, "%w", clazz->dotified);
  *remain -= nbytes;

  return temp + nbytes;
}

static const char * clazzState2name[] = {
  "unloaded",
  "loading",
  "loaded",
  "supers loading",
  "supers loaded",
  "referencing",
  "referenced",
  "preparing",
  "prepared",
  "9",
  "10",
  "initializing",
  "initialized",
  "13",
  "14",
  "broken"
};

/*
** Function used by x_snprintf to print out the name of a clazz (format %K).
*/

char *print_clazz_long(char *buffer, int *remain, void *c, int w, int p, unsigned int f) {

  w_clazz clazz = c;
  w_int    nbytes;
  char    *temp;

  if (*remain < 1) {

    return buffer;

  }

  temp = buffer;

  if (c == NULL) {
    strncpy(temp, (char*)"<NULL>", *remain);
    if (*remain < 6) {
      temp += *remain;
      *remain = 0;
    }
    else {
      temp += 6;
      *remain -= 6;
    }

    return temp;

  }

  nbytes = x_snprintf(temp, *remain, "%s %s ", clazzState2name[getClazzState(clazz)], isSet(clazz->flags, ACC_INTERFACE) ? "interface" : "class");
  temp += nbytes;
  *remain -= nbytes;

  if (clazz->loader  && clazz->loader != systemClassLoader) {
    nbytes = x_snprintf(temp, *remain, "{%j}", clazz->loader);
    temp += nbytes;
    *remain -= nbytes;
  }

  nbytes = x_snprintf(temp, *remain, "%w", clazz->dotified);
  *remain -= nbytes;

  return temp + nbytes;
}

/*
** Functions to get an unsigned integer (respectively 1, 2 and 4 bytes
** long) from the class file
*/

static inline void barread(w_bar bar, w_ubyte *dst, w_int length) {

  w_int duffs = (length + 31) / 32;
  w_ubyte * src = bar->buffer + bar->current;

  switch (length & 0x1f) {
    case  0: do { *dst++ = *src++;
    case 31:      *dst++ = *src++;
    case 30:      *dst++ = *src++;
    case 29:      *dst++ = *src++;
    case 28:      *dst++ = *src++;
    case 27:      *dst++ = *src++;
    case 26:      *dst++ = *src++;
    case 25:      *dst++ = *src++;
    case 24:      *dst++ = *src++;
    case 23:      *dst++ = *src++;
    case 22:      *dst++ = *src++;
    case 21:      *dst++ = *src++;
    case 20:      *dst++ = *src++;
    case 19:      *dst++ = *src++;
    case 18:      *dst++ = *src++;
    case 17:      *dst++ = *src++;
    case 16:      *dst++ = *src++;
    case 15:      *dst++ = *src++;
    case 14:      *dst++ = *src++;
    case 13:      *dst++ = *src++;
    case 12:      *dst++ = *src++;
    case 11:      *dst++ = *src++;
    case 10:      *dst++ = *src++;
    case  9:      *dst++ = *src++;
    case  8:      *dst++ = *src++;
    case  7:      *dst++ = *src++;
    case  6:      *dst++ = *src++;
    case  5:      *dst++ = *src++;
    case  4:      *dst++ = *src++;
    case  3:      *dst++ = *src++;
    case  2:      *dst++ = *src++;
    case  1:      *dst++ = *src++;
            } while (--duffs > 0);
  }

  bar->current += length;

}

inline static u1 get_u1(w_bar bar) {

  return *(bar->buffer + bar->current++);

}

inline static u2 get_u2(w_bar s) {

  w_byte bytes[2];

  barread(s, bytes, 2);
  return ((bytes[0] << 8) | bytes[1]);

}

inline static u4 get_u4(w_bar s) {

  w_byte bytes[4];

  barread(s, bytes, 4);
  return ((bytes[0] << 24) | (bytes[1] << 16) | (bytes[2] << 8) | bytes[3]);

}

inline static void check_length(w_clazz clazz, w_bar s, int min) {
  if (bar_avail(s) < min) {
    woempa(9, "Need to read %d bytes from classfile, only %d bytes left\n", min, bar_avail(s));
    throwException(currentWonkaThread, clazzClassFormatError, "%k: class file truncated", clazz);
  }
}

/*
** Get the "magic number" 0xcafebabe from the class file
*/

inline static void get_cafebabe(w_thread thread, w_clazz clazz, w_bar s) {

  u4 cafebabe;

  check_length(clazz, s, 4);

  if (! exceptionThrown(thread)) {
    cafebabe = get_u4(s);

    if (cafebabe != 0xcafebabe) {
      woempa(9,"Uh oh, cafebabe is called `%08x' today\n", cafebabe);
      throwException(thread, clazzClassFormatError, "%k: bad magic 0x%08x", clazz, cafebabe);
    }
  }
}

/*
** Get the major and minor version number from the class file
*/

inline static void get_version(w_thread thread, w_clazz clazz, w_bar s) {

  check_length(clazz, s, 4);

  if (! exceptionThrown(thread)) {
    clazz->cminor = get_u2(s);
    clazz->cmajor = get_u2(s);
  }
}

static void parseConstant(w_clazz clazz, w_bar s, w_int *idx) {
  /*
  ** We check that at least 3 bytes are available, 1 for the tag and at
  ** least 2 bytes thereafter.  CONSTANT_UTF8, CONSTANT_CLASS, and 
  ** CONSTANT_STRING just need these 2  bytes: the other types need
  ** more, so we insert an extra check when we get there.
  */
  w_cnt c = &clazz->constants[*idx];
  u2         tag;
  u2         length;
  w_thread thread = currentWonkaThread;

  check_length(clazz, s, 3);

  if (exceptionThrown(thread)) {
    return;
  }

  tag = get_u1(s);
  c->cnt_tag = tag;

  switch (tag) {
    case CONSTANT_UTF8: 
      length = get_u2(s);

      if (length > 0) {
        char *buffer = allocMem_system((w_size)length);
        barread(s, buffer, length);
        clazz->values[*idx] = (w_word)utf2String(currentWonkaThread, buffer, length);
        releaseMem(buffer);
      }
      else {
        clazz->values[*idx] = (w_word)utf2String(currentWonkaThread, "", 0);
        woempa(1, "Utf8 constant with length %d!\n", length);
      }
      woempa(1, "Constant[%d] = UTF8 `%w'\n", *idx, clazz->values[*idx]);
      
      *idx += 1;
      break;
      
    case CONSTANT_INTEGER:
    case CONSTANT_FLOAT:
      check_length(clazz, s, 4);
      if (! exceptionThrown(thread)) {
        clazz->values[*idx] = get_u4(s);
        woempa(1, "Constant[%d] = %s 0x%08x\n", *idx, tag == CONSTANT_INTEGER ? "int" : "float", clazz->values[*idx]);
        *idx += 1;
      }
      break;
      
    case CONSTANT_LONG:
      check_length(clazz, s, 8);
      if (!exceptionThrown(thread)) {
        w_word msw = get_u4(s);
        w_word lsw = get_u4(s);

        clazz->values[*idx] = msw;
        clazz->values[*idx + 1] = lsw;
        (c+1)->cnt_tag = NO_VALID_ENTRY;
        woempa(1, "Constant[%d] = long 0x%08x%08x\n", *idx, clazz->values[*idx], clazz->values[*idx + 1]);
        *idx += 2;
      }
      break;
    
    case CONSTANT_DOUBLE:
      check_length(clazz, s, 8);
      if (!exceptionThrown(thread)) {
#if defined(ARM) && defined(HW_DOUBLES)
        w_word lsw = get_u4(s);
        w_word msw = get_u4(s);
#else
        w_word msw = get_u4(s);
        w_word lsw = get_u4(s);
#endif

        clazz->values[*idx] = msw;
        clazz->values[*idx + 1] = lsw;
        (c+1)->cnt_tag = NO_VALID_ENTRY;
        woempa(1, "Constant[%d] = double 0x%08x%08x\n", *idx, clazz->values[*idx], clazz->values[*idx + 1]);
        *idx += 2;
      }
      break;
      
    case CONSTANT_CLASS:
      clazz->values[*idx] = get_u2(s);
      woempa(1, "Constant[%d] = Class[%d]\n", *idx, clazz->values[*idx]);
      *idx += 1;
      break;
      
    case CONSTANT_STRING:
      clazz->values[*idx] = get_u2(s);
      woempa(1, "Constant[%d] = String[%d]\n", *idx, clazz->values[*idx]);
      *idx += 1;
      break;
      
    case CONSTANT_FIELD:
      check_length(clazz, s, 4);
      if (!exceptionThrown(thread)) {
        w_Member member;

        member.class_index = get_u2(s);
        member.name_and_type_index = get_u2(s);
				*(w_Member*)&(clazz->values[*idx]) = member;
        woempa(1, "Constant[%d] = Field with Class[%d], NAT[%d]\n", *idx, member.class_index, member.name_and_type_index);
        *idx += 1;
      }
      break;
      
    case CONSTANT_METHOD:
      check_length(clazz, s, 4);
      if (!exceptionThrown(thread)) {
        w_Member member;

        member.class_index = get_u2(s);
        member.name_and_type_index = get_u2(s);
				*(w_Member*)&(clazz->values[*idx]) = member;
        woempa(1, "Constant[%d] = Method with Class[%d], NAT[%d]\n", *idx, member.class_index, member.name_and_type_index);
        *idx += 1;
      }
      break;
      
    case CONSTANT_IMETHOD:
      check_length(clazz, s, 4);
      if (!exceptionThrown(thread)) {
        w_Member member;

        member.class_index = get_u2(s);
        member.name_and_type_index = get_u2(s);
				*(w_Member*)&(clazz->values[*idx]) = member;
        woempa(1, "Constant[%d] = IMethod with Class[%d], NAT[%d]\n", *idx, member.class_index, member.name_and_type_index);
        *idx += 1;
      }
      break;
      
    case CONSTANT_NAME_AND_TYPE:
      check_length(clazz, s, 4);
      if (!exceptionThrown(thread)) {
        w_Name_and_Type nat;

        nat.name_index = get_u2(s);
        nat.descriptor_index = get_u2(s);
				*(w_Name_and_Type*)&clazz->values[*idx] = nat;
        woempa(1, "Constant[%d] = NAT with Name[%d], Type[%d]\n", *idx, nat.name_index, nat.descriptor_index);
        *idx += 1;
      }
      break;

    default:
      throwException(thread, clazzClassFormatError, "%k: bad constant type %d", clazz, tag);
      break;      
  }

}

/*
** Scan through the constant pool and copy the data to clazz->constants[].
*/

inline static void get_constantpool(w_thread thread, w_clazz clazz, w_bar s) {
  w_int i;
  
  check_length(clazz, s, 2);

  if (!exceptionThrown(thread)) {
    clazz->numConstants = get_u2(s);

    woempa(1, "Number of constant pool entries is %d, using %d bytes each\n", clazz->numConstants - 1, sizeof(w_Cnt));
    clazz->constants = allocMem(thread, (clazz->numConstants + 1) * sizeof(w_Cnt));
    clazz->values = allocMem(thread, (clazz->numConstants + 1) * sizeof(w_word));

    for (i = 1; i < (w_int)clazz->numConstants && ! exceptionThrown(thread); ) {
      woempa(1, "parsing constant pool entry %d of %d\n", i, clazz->numConstants - 1);
      parseConstant(clazz, s, &i);
    }
  }

}

/*
** Parse a field attribute (a name/value pair).
*/

static void parseFieldAttribute(w_field field, w_bar s) {

  u2 attribute_name_index = 0;
  u4 attribute_length = 0;
  w_string attributeName;
  w_thread thread = currentWonkaThread;

  check_length(field->declaring_clazz, s, 6);
  
  if (!exceptionThrown(thread)) {
    attribute_name_index = get_u2(s);
    attribute_length = get_u4(s);
    check_length(field->declaring_clazz, s, (w_int)attribute_length);
  }

  if (exceptionThrown(thread)) {
    return;
  }

  attributeName = resolveUtf8Constant(field->declaring_clazz, attribute_name_index);
  woempa(1, "Parsing attribute %w for field %w of %k.\n", attributeName, NM(field), field->declaring_clazz);

  /*
  ** See which attribute we have here.
  */

  if (attributeName == string_ConstantValue) {
    check_length(field->declaring_clazz, s, 2);
    if (exceptionThrown(thread)) {
      return;
    }
  
    field->initval = get_u2(s);
  }
  else {
    u4 i;

    /*
    ** Unknown attributes are silently ignored.
    */
    woempa(1, "Unknown/ignored attribute %w\n", attributeName);
    for (i = 0; i < attribute_length; i++) {
      get_u1(s);
    }
  }

}

/*
** Parse a field entry in the classfile.  The entry consists of a
** flags set (u2), name index (u2), descriptor index (u2), number of
** attributes (u2), followed by the attributes (see previous function).
**
** We store the field entries sorted as follows:
**   first, all static fields;
**   then all instance fields which store references;
**   lastly, instance fields which store primitives.
** This ordering will simplify later processing (class preparation, etc.).
*/

static void parseField(w_clazz clazz, w_size idx, w_bar s) {

  w_thread thread = currentWonkaThread;
  u2 tmp_u2;
  int attributeNumber;
  w_field  field;
  w_flags  flags;
  w_string name;
  w_string desc_name;
  w_boolean lng = WONKA_FALSE;
  w_boolean ref = WONKA_FALSE;
  w_boolean arr = WONKA_FALSE;

  check_length(clazz, s, 8);

  if (exceptionThrown(thread)) {
    return;
  }

  flags = get_u2(s);
  tmp_u2 = get_u2(s);
  name = resolveUtf8Constant(clazz, tmp_u2);
  
  tmp_u2 = get_u2(s);
  desc_name = resolveUtf8Constant(clazz, tmp_u2);
  switch(string_char(desc_name, 0)) {
    case 'D':
    case 'J':
      woempa(1, "Field %w of %k has descriptor %w, setting FIELD_IS_LONG\n", name, clazz, desc_name);
      lng = WONKA_TRUE;
      break;

    case 'L':
      woempa(1, "Field %w of %k has descriptor %w, setting FIELD_IS_REFERENCE\n", name, clazz, desc_name);
      ref = WONKA_TRUE;
      break;

    case '[':
      woempa(1, "Field %w of %k has descriptor %w, setting FIELD_IS_REFERENCE and FIELD_IS_ARRAY\n", name, clazz, desc_name);
      ref = WONKA_TRUE;
      arr = WONKA_TRUE;
      break;

    default:
      woempa(1, "Field %w of %k has descriptor %w, setting no flags\n", name, clazz, desc_name);
  }

  deregisterString(desc_name);

  field = &clazz->own_fields[idx];
  field->desc_index = tmp_u2;
  field->declaring_clazz = clazz;
  field->label = (char *) "field";
  field->flags = flags;
  if (isSet(field->flags, ACC_STATIC)) {
    ++clazz->numStaticFields;
  }
  field->name  = name;
  field->initval = 0;
  if (lng) {
    setFlag(field->flags, FIELD_IS_LONG);
  }
  else if (ref) {
    setFlag(field->flags, FIELD_IS_REFERENCE);
  }
  if (arr) {
    setFlag(field->flags, FIELD_IS_ARRAY);
  }

  /*
  ** Parse its attributes.
  */

  tmp_u2 = get_u2(s);
  if (tmp_u2) {
    for (attributeNumber = 0; attributeNumber < tmp_u2; attributeNumber++) {
      parseFieldAttribute(field, s);
    }
  }

}

static void parseMethodCodeExceptionTable(w_method method, w_bar s) {

  w_thread thread = currentWonkaThread;
  w_exception exception;
  w_int n;
  w_int i;

  n = get_u2(s);
  method->exec.numExceptions = n;
  if (n) {
    method->exec.exceptions = allocMem(thread, n * sizeof(w_Exception));
    for (i = 0; i < n; i++) {
      exception = &method->exec.exceptions[i];
      exception->start_pc = get_u2(s);
      exception->end_pc = get_u2(s);
      exception->handler_pc = get_u2(s);
      exception->type_index = get_u2(s);
    }
  }
}

static void parseMethodCode(w_method method, w_bar s) {

  unsigned char * code;

  method->exec.stack_i = get_u2(s);
  method->exec.local_i = get_u2(s);
  method->exec.code_length = get_u4(s);
  if (method->exec.code_length > 0) {
    code = allocMem(currentWonkaThread, (w_size)method->exec.code_length + 4);
    method->exec.code = &code[4];
    method->exec.code[-1] = 254; // handle exception
    method->exec.code[-2] = 255; // no handler found, return
    barread(s, method->exec.code, (signed)method->exec.code_length);
    adaptEndianess(method);
  }

  
}

static void parseMethodAttribute(w_method method, w_bar s) {

  w_thread  thread = currentWonkaThread;
  u2 attribute_name_index;
  u4 attribute_length;
  w_int i;
  w_string attributeName;
  u2 attribute_count;
  
  attribute_name_index = get_u2(s);
  attribute_length = get_u4(s);
  woempa(1, "Parsing attribute[%d] of %k\n", attribute_name_index, method->spec.declaring_clazz);
  attributeName = resolveUtf8Constant(method->spec.declaring_clazz, attribute_name_index);

  woempa(1, "Attribute `%w' length %d\n", attributeName, attribute_length);

  /*
  ** See which attribute we have here.
  */

  if (attributeName == string_Code) {
    if (isSet(method->flags, ACC_ABSTRACT | ACC_NATIVE)) {
      woempa(1, "ClassFormatError: %k %w: Code attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      throwException(currentWonkaThread, clazzClassFormatError, "%k %w: Code attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      return;
    }

    parseMethodCode(method, s);
    parseMethodCodeExceptionTable(method, s);

    if (exceptionThrown(thread)) {
      return;
    }

    parseMethodCodeAttributes(method, s);

    if (exceptionThrown(thread)) {
      return;
    }
    woempa(4, "Read Code attribute; %d bytes, %d exception handlers\n", method->exec.code_length, method->exec.numExceptions);
  }
  else if (attributeName == string_Exceptions) {
    method->numThrows = get_u2(s);
    method->throws = allocMem(thread, method->numThrows * sizeof(w_ushort));
    for (i = 0; i < method->numThrows; i++) {
      method->throws[i] = get_u2(s);
    }
    woempa(4, "Read Exceptions attribute; throws %d exceptions\n", method->numThrows);
  }
  else if (attributeName == string_LocalVariableTable) {
    if (isSet(method->flags, ACC_ABSTRACT | ACC_NATIVE)) {
      woempa(1, "ClassFormatError: %k %w: LocalVariableTable attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      throwException(currentWonkaThread, clazzClassFormatError, "%k %w: LocalVariableTable attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      return;
    }

    attribute_count = get_u2(s);
#ifdef STORE_METHOD_DEBUG_INFO
    method->exec.numLocalVars = attribute_count;
    if (attribute_count > 0) {
      method->exec.localVars = allocMem(thread, attribute_count * sizeof(w_LocalVar));
      for (i = 0; i < attribute_count; i++) {
        method->exec.localVars[i].start_pc = get_u2(s);
        method->exec.localVars[i].end_pc = method->exec.localVars[i].start_pc + get_u2(s);
        method->exec.localVars[i].name_index = get_u2(s);
        method->exec.localVars[i].desc_index = get_u2(s);
        method->exec.localVars[i].slot = get_u2(s);
      }
    }
#else
    for (i = 0; i < attribute_count; i++) {
      get_u2(s);
      get_u2(s);
      get_u2(s);
      get_u2(s);
      get_u2(s);
    }
#endif
  }
  else if (attributeName == string_LineNumberTable) {
    if (isSet(method->flags, ACC_ABSTRACT | ACC_NATIVE)) {
      woempa(1, "ClassFormatError: %k %w: LineNumberTable attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      throwException(currentWonkaThread, clazzClassFormatError, "%k %w: LineNumberTable attribute not permitted in abstract or native method", method->spec.declaring_clazz, method->spec.name);
      return;
    }

    attribute_count = get_u2(s);
#ifdef STORE_METHOD_DEBUG_INFO
    method->exec.numLineNums = attribute_count;
    if (attribute_count > 0) {
      method->exec.lineNums = allocMem(thread, attribute_count * sizeof(w_LineNum));
      for (i = 0; i < attribute_count; i++) {
        method->exec.lineNums[i].start_pc = get_u2(s);
        method->exec.lineNums[i].line_number = get_u2(s);
      }
    }
#else
    for (i = 0; i < attribute_count; i++) {
      get_u2(s);
      get_u2(s);
    }
#endif
  }
  else {
    woempa(1, "Unknown/ignored attribute '%w'\n", attributeName);
    for (i=0;(u4)i<attribute_length;++i) {
      get_u1(s);
    }
  }

}

/*
** This function can not be made inline. It's called by parseMethodAttribute and this function calls
** this parseMethodAttribute function recursively...
*/

static void parseMethodCodeAttributes(w_method method, w_bar s) {

  w_int i;
  w_int attr_count;

  attr_count = get_u2(s);
  if (attr_count > 0) {
    for (i = 0; i < attr_count; i++) {
      parseMethodAttribute(method, s);
      if (exceptionThrown(currentWonkaThread)) {
        return;
      }
    }
  }

}

static void parseMethod(w_clazz clazz, w_size idx, w_bar s) {
  u2 access_flags;
  u2 numAttributes;
  w_string name;
  w_size i;
  
  access_flags = get_u2(s);
  name = resolveUtf8Constant(clazz,get_u2(s));
  if (name == string_angle_brackets_clinit) {
    access_flags &= ACC_STRICT;
    access_flags |= ACC_STATIC;
  }
  clazz->own_methods[idx].flags = access_flags;
  clazz->own_methods[idx].spec.declaring_clazz = clazz;
  clazz->own_methods[idx].spec.name = name;
  clazz->own_methods[idx].desc_index = get_u2(s);
  clazz->own_methods[idx].slot = SLOT_NOT_ALLOCATED;
  clazz->own_methods[idx].numThrows = 0;

  clazz->own_methods[idx].exec.code = NULL;
  clazz->own_methods[idx].exec.function.void_fun = NULL;
  clazz->own_methods[idx].exec.numExceptions = 0;
#ifdef STORE_METHOD_DEBUG_INFO
  clazz->own_methods[idx].exec.numLocalVars = 0;
  clazz->own_methods[idx].exec.numLineNums = 0;
#endif

  /*
  ** Parse the attributes of this method.
  */

  numAttributes = get_u2(s);
  if (numAttributes > 0) {
    for (i = 0; i < numAttributes; i++) {
      parseMethodAttribute(&clazz->own_methods[idx], s);
      if (exceptionThrown(currentWonkaThread)) {
        return;
      }
    }
  }
}

/*
** Determine whether a name begins with one of the reserved prefixes "java.",
** "wonka.".  TODO: define and use a function in strings.h.
*/
w_boolean isReservedName(w_string name) {
  w_size l = string_length(name);
  w_size n = 0;

  while (n < l && string_char(name, n) == '[') {
    ++n;
    --l;
  }

  if (n) {
    if (string_char(name, n) == 'L') {
      ++n;
      l -= 2;
    }
    else {
      return WONKA_TRUE;
    }
  }

  if (l > 5 &&
      string_char(name, n + 0) == 'j' && string_char(name, n + 1) == 'a' &&
      string_char(name, n + 2) == 'v' && string_char(name, n + 3) == 'a' &&
      string_char(name, n + 4) == '.')
  {
    return WONKA_TRUE;
  }

  if (l > 6 &&
      string_char(name, n + 0) == 'w' && string_char(name, n + 1) == 'o' &&
      string_char(name, n + 2) == 'n' && string_char(name, n + 3) == 'k' &&
      string_char(name, n + 4) == 'a' && string_char(name, n + 5) == '.')
  {
    return WONKA_TRUE;
  }

  return WONKA_FALSE;
}


/*
** Register a w_Clazz structure in a class hashtable.  
*/

void registerClazz(w_clazz clazz, w_instance loader) {
  w_clazz  existing;
  w_hashtable hashtable = loader2loaded_classes(loader);

  if (getClazzState(clazz) == CLAZZ_STATE_UNLOADED) {
    wabort(ABORT_WONKA, "%K\n", clazz);
  }
  woempa(1, "Registering clazz %k in %s.\n", clazz, hashtable->label);

  ht_lock(hashtable);
  existing = (w_clazz)ht_write_no_lock(hashtable, (w_word)clazz->dotified, (w_word)clazz);

  if (existing) {
    if (existing == clazz) {
      woempa(1, "Class %k (%p) was already present in %s (which is OK).\n", clazz, clazz, hashtable->label);
    }
    else {
      throwException(currentWonkaThread, clazzSecurityException, "Class already defined: %w", clazz->dotified);
      ht_write_no_lock(hashtable, (w_word)clazz->dotified, (w_word)existing);
      wabort(ABORT_WONKA, "Fascinating. The class %k (w_clazz %p) was already registered in %s as w_clazz %p.\n", clazz, clazz, hashtable->label, existing);
    }
  }
  ht_unlock(hashtable);

}

/*
** Deregister a w_Clazz structure from a class hashtable.  
*/

void deregisterClazz(w_clazz clazz, w_instance loader) {
  w_clazz  existing;
  w_hashtable hashtable = loader2loaded_classes(loader);

  woempa(8, "Deregistering clazz %k from %s.\n", clazz, hashtable->label);

  existing = (w_clazz)ht_erase(hashtable, (w_word)clazz->dotified);

  if (!existing) {
    woempa(9, "Fascinating. When we wanted to deregister class %k, we couldn't find it in %s\n", clazz, hashtable->label);
  }

}

inline static void get_superclass(w_thread thread, w_clazz clazz, w_bar s) {
  clazz->temp.super_index = get_u2(s);
}

inline static void get_interfaces(w_thread thread, w_clazz clazz, w_bar s) {

  w_size i;

  clazz->temp.interface_index_count = get_u2(s);
  woempa(1, "%K has %d interfaces\n", clazz, clazz->temp.interface_index_count);
  if (clazz->temp.interface_index_count > 0) {
    clazz->temp.interface_index = allocClearedMem(thread, clazz->temp.interface_index_count * sizeof(w_ushort));
    for (i = 0; i < clazz->temp.interface_index_count; i++) {
      clazz->temp.interface_index[i] = get_u2(s);
      woempa(1, "%K has interface %d index %d\n", clazz, i, clazz->temp.interface_index[i]);
    }
  }
}

static void swapFields(w_clazz clazz, w_int i, w_int j) {
  w_Field f;

  woempa(1, "Swapping fields [%d] (`%w') and [%d] (`%w') of %k\n", i, clazz->own_fields[i].name, j, clazz->own_fields[j].name, clazz);
  w_memcpy(&f, &clazz->own_fields[i], sizeof(w_Field));
  w_memcpy(&clazz->own_fields[i], &clazz->own_fields[j], sizeof(w_Field));
  w_memcpy(&clazz->own_fields[j], &f, sizeof(w_Field));
}

 void sortFields(w_clazz clazz) {
  w_int i;
  w_int j;

  woempa(1, "Sorting fields of %k\n", clazz);
  i = 0;
  j = clazz->numFields - 1;
  while (i < j) {
    if (isSet(clazz->own_fields[i].flags, ACC_STATIC)) {
      woempa(1, "Field [%d] (`%w') is static, leaving it be\n",  i, clazz->own_fields[i].name);
      ++i;
    }
    else if (isNotSet(clazz->own_fields[j].flags, ACC_STATIC)) {
      woempa(1, "Field [%d] (`%w') is non-static, leaving it be\n",  j, clazz->own_fields[j].name);
      --j;
    }
    else {
      swapFields(clazz, i, j);
      ++i;
      --j;
    }
  }

  woempa(1, "Sorting static fields of %k\n", clazz);
  i = 0;
  j = clazz->numStaticFields - 1;
  while (i < j) {
    if (isSet(clazz->own_fields[i].flags, FIELD_IS_REFERENCE)) {
      woempa(1, "Static field [%d] (`%w') is  reference, leaving it be\n",  i, clazz->own_fields[i].name);
      ++i;
    }
    else if (isNotSet(clazz->own_fields[j].flags, FIELD_IS_REFERENCE)) {
      woempa(1, "Static field [%d] (`%w') is non-reference, leaving it be\n",  j, clazz->own_fields[j].name);
      --j;
    }
    else {
      swapFields(clazz, i, j);
    }
  }

  woempa(1, "Sorting non-static fields of %k\n", clazz);
  i = clazz->numStaticFields;
  j = clazz->numFields - 1;
  while (i < j) {
    if (isSet(clazz->own_fields[i].flags, FIELD_IS_REFERENCE)) {
      woempa(1, "Static field [%d] (`%w') is  reference, leaving it be\n",  i, clazz->own_fields[i].name);
      ++i;
    }
    else if (isNotSet(clazz->own_fields[j].flags, FIELD_IS_REFERENCE)) {
      woempa(1, "Static field [%d] (`%w') is non-reference, leaving it be\n",  j, clazz->own_fields[j].name);
      --j;
    }
    else {
      swapFields(clazz, i, j);
    }
  }

}

void get_fields(w_clazz clazz, w_bar s) {

  w_thread thread = currentWonkaThread;
  w_size i;
  w_size n;

  clazz->staticFields = NULL;
  clazz->numStaticFields = 0;
  clazz->numStaticWords = 0;
  n = get_u2(s);
  clazz->numFields = n;
  if (n > 0) {
    woempa(1, "Allocating %d w_Field's of %d bytes each\n", clazz->numFields, sizeof(w_Field));
    clazz->own_fields = allocClearedMem(thread, clazz->numFields * sizeof(w_Field));
    for (i = 0; i < n; i++) {
      check_length(clazz, s, 8);

      if (!exceptionThrown(thread)) {
        parseField(clazz, i, s);
      }

      if (exceptionThrown(thread)) {
        return;
      }
    }
  }

  if (n > 1) {
    sortFields(clazz);
  }

}

void get_methods(w_clazz clazz, w_bar s) {

  w_thread thread = currentWonkaThread;
  w_size i;

  clazz->numDeclaredMethods = get_u2(s);
  clazz->numInheritableMethods = 0;
  if (clazz->numDeclaredMethods > 0) {
    clazz->own_methods = allocClearedMem(thread, clazz->numDeclaredMethods * sizeof(w_Method));
    for (i = 0; i < clazz->numDeclaredMethods; i++) {
      parseMethod(clazz, i, s); 
      if (exceptionThrown(thread)) {
        return;
      }
    }
  }

}

static void parseClassAttributes(w_clazz clazz, w_bar s) {
  u2 attribute_name_index;
  u4 attribute_length;
  w_string attributeName;
  w_int  i;
  
  attribute_name_index = get_u2(s);
  attribute_length = get_u4(s);
  attributeName = resolveUtf8Constant(clazz, attribute_name_index);
  
  if (attributeName == string_SourceFile) {
    clazz->source = resolveUtf8Constant(clazz, get_u2(s));
  }
  else {
    woempa(1, "Unknown attribute '%w'\n", attributeName);
    for (i = 0; (u4)i < attribute_length; i++) {
      get_u1(s);
    }
  }

}

 void get_attributes(w_clazz clazz, w_bar s) {

  u2 numberOfAttributes;

  numberOfAttributes = get_u2(s);
  if (numberOfAttributes > 0) {
    parseClassAttributes(clazz,s);
  }

}

/*
** Create the w_Clazz structure corresponding to a class, reading the class
** file from device dev.  The class will be registered with the given class
** loader as its defining class loader.
*/

w_clazz createClazz(w_thread thread, w_string name, w_bar dev, w_instance loader) {

  w_int thisClassIndex = 0;
  w_int classNameIndex;
  w_clazz clazz;
  w_string slashed;
  w_string dotified;

  clazz = allocClazz(thread);
  // We initialise instanceSize to an impossible value, so any attempt
  // to allocate an instance of an unresolved clazz will fail.
  clazz->instanceSize = 32767;  // mugtrap
  clazz->numReferenceFields = 0;
  clazz->loader = loader;
  clazz->type = VM_TYPE_REF + VM_TYPE_OBJECT;
  clazz->bits = 32;
  clazz->resolution_monitor = allocMem(thread, sizeof(x_Monitor));
  x_monitor_create(clazz->resolution_monitor);
  setClazzState(clazz, CLAZZ_STATE_LOADING);

  get_cafebabe(thread, clazz, dev);
  if (! exceptionThrown(thread)) {
    get_version(thread, clazz, dev);
  }

  if (! exceptionThrown(thread)) {
    get_constantpool(thread, clazz, dev);
  }

  if (! exceptionThrown(thread)) {
    clazz->flags = get_u2(dev) & (ACC_PUBLIC | ACC_FINAL | ACC_SUPER | ACC_INTERFACE | ACC_ABSTRACT);
  }

  if (! exceptionThrown(thread)) {
    if(isSet(clazz->flags, ACC_INTERFACE) && isNotSet(clazz->flags, ACC_ABSTRACT)){
      woempa(9,"How rude, clazz %w is has ACC_INTERFACE set but not ACC_ABSTRACT\n",name);
      woempa(9,"Fixing this ...\n");
      setFlag(clazz->flags, ACC_ABSTRACT);
    } 
    thisClassIndex = get_u2(dev);
  }

  if (! exceptionThrown(thread)) {
    classNameIndex = clazz->values[thisClassIndex];
    woempa(1, "Class %w this_class = %d => UTF8 constant[%d]\n", name, thisClassIndex, classNameIndex);
    slashed = (w_string)clazz->values[classNameIndex];
    woempa(1, "Slashed class name: %w\n", slashed);
    dotified = slashes2dots(W_Thread_system, slashed);
    if (!name || dotified == name) {
      clazz->dotified = dotified;
    }
    else {
      woempa(9,"Impostor! The class which should be known as `%w' is really `%k'.\n",name,clazz);
      throwException(thread, clazzClassFormatError, "%w: this_class = %w", name, slashed);
    }
  }

  if (exceptionThrown(thread)) {
    destroyClazz(clazz);

    return NULL;

  }

  clazz->initialize = initializeClass;
 
  get_superclass(thread, clazz, dev);
  if (! exceptionThrown(thread)) {
    get_interfaces(thread, clazz, dev);
    if (! exceptionThrown(thread)) {
      get_fields(clazz, dev);
      get_methods(clazz, dev);
      if (! exceptionThrown(thread)) {
        get_attributes(clazz, dev);
      }
    }
  }

  if (exceptionThrown(thread)) {
    destroyClazz(clazz);
    return NULL;
  }

  clazz->resolution_thread = NULL;
  setClazzState(clazz, CLAZZ_STATE_LOADED);
  registerClazz(clazz, loader);  

  woempa(1, "%j is the defining class loader of %k\n", loader, clazz);

  return clazz;

}

/*
** Attach an instance of java.lang.Class to this clazz.
*/

w_instance attachClassInstance(w_clazz clazz) {
  w_thread   thread = currentWonkaThread;
  w_instance Class;
  w_int      i;

#ifdef RUNTIME_CHECKS
  if (clazzClass == NULL) {
    woempa(9, "clazzClass not defined\n");
    wabort(ABORT_WONKA, "Cannot create instance of Class for %k, as clazzClass not yet defined\n", clazz);
    return NULL;
  }
#endif

  woempa(1, "clazzClass state is %d\n", getClazzState(clazz));
  if (getClazzState(clazz) < CLAZZ_STATE_LOADED) {
    wabort(ABORT_WONKA, "Cannot create instance of Class for unloaded class: %K\n", clazz);
    return NULL;
  }

  Class = allocInstance(thread, clazzClass);

  if (Class) {
    clazz->Class = Class;
    woempa(1, "Attached instance of Class (%p) to %k.\n", Class, clazz);
    if (clazz->dims) {
      w_instance base_Class;

      base_Class = clazz2Class(clazz->previousDimension);
      for (i = 0; i < (w_int)FIELD_OFFSET(F_Class_wotsit); ++i) {
        woempa(1,"Copying field[%d] (0x%08x) from Class %k to %k\n",i,base_Class[i], clazz->previousDimension, clazz);
        Class[i] = base_Class[i];
      }
    }
    else {
      setReferenceField(Class, clazz->loader, F_Class_loader);
    }
    setWotsitField(Class, F_Class_wotsit, clazz);
  }

  return Class;
}

w_clazz _allocClazz(const char *file, int line, w_thread thread) {

  w_clazz clazz = allocClearedMem(thread, sizeof(w_Clazz));
  clazz->label = (char *) "clazz";
  clazz->file = (char *) file;
  clazz->line = line;
  
  return clazz; 

}

/*
** Destroy the w_Clazz structure corresponding to a class.
*/

void destroyClazz(w_clazz clazz) {
  w_int  i;
  
  woempa(7,"Destroying class %k\n",clazz);
/* TODO: (or rather undo ...)
        get_attributes(clazz, s);
      get_methods(clazz, s, loading_problem);
      get_fields(clazz, s);
*/

  for (i = 1; i < (w_int)clazz->numConstants; i++) {
    dissolveConstant(clazz, i);
  }

  if (clazz->dims) {
  // array classes use a static `supers' and `interfaces', so don't release 'em
  }
  else {
    if (clazz->supers) {
      releaseMem(clazz->supers);
    }
    if (clazz->interfaces) {
      releaseMem(clazz->interfaces);
    }
    if (clazz->references) {
      releaseWordset(&clazz->references);
    }
  }
  if (getClazzState(clazz) >= CLAZZ_STATE_LOADED && getClazzState(clazz) < CLAZZ_STATE_INITIALIZED) {
    if (clazz->temp.interface_index) {
      releaseMem(clazz->temp.interface_index);
    }
  }
  woempa(1,"Deregistering the class (isn't that bass-ackwards?\n");
  deregisterClazz(clazz, clazz->loader);  
  woempa(1,"Deregistering string `%w'\n",clazz->dotified);
  deregisterString(clazz->dotified);

  // TODO: release all strings, methods, fields
  releaseMem(clazz->constants);
  if(clazz->values){
    releaseMem(clazz->values);
  }
  x_monitor_delete(clazz->resolution_monitor);
  releaseMem(clazz->resolution_monitor);
  if (clazz->failure_message) {
    deregisterString(clazz->failure_message);
  }
  woempa(1,"Finally releasing memory of clazz itself\n");
  releaseMem(clazz);
}

/*
** To find a field, we have to search recursively over the superclass
** and all superinterfaces.
*/
w_field getField(w_clazz clazz, w_string name) {
  w_clazz current_clazz = clazz;
  w_field field;
  w_size  i;
  w_size  j;

  woempa(1, "Looking for field `%w' in %k\n", name, clazz);
  while (current_clazz) {
    field = current_clazz->own_fields;
    for (i = 0; i < current_clazz->numFields; ++i) {
      woempa(1, "Trying `%w'\n", field->name);
      if (field->name == name) {
        woempa(1, "Success!  Found field `%w' in %k\n", name, current_clazz);

        return field;

      }
      ++field;
    }
    for (j = 0; j < current_clazz->numInterfaces; ++j) {
      woempa(1, "Looking in superinterface %w of %w\n", current_clazz->interfaces[j], current_clazz);
      field = getField(current_clazz->interfaces[j], name);
      if (field) {

        return field;

      }
    }
    current_clazz = getSuper(current_clazz);
    woempa(1, "Looking in superclass %k\n", current_clazz);
  }

  return NULL;
}

const char * instruction2char[] = {
  "nop", "aconst_null", "iconst_m1", "iconst_0", "iconst_1", "iconst_2", "iconst_3", "iconst_4", "iconst_5",
  "lconst_0", "lconst_1", "fconst_0", "fconst_1", "fconst_2", "dconst_0", "dconst_1", "bipush", "sipush",
  "ldc", "ldc_w", "ldc2_w", "iload", "lload", "fload", "dload", "aload", "iload_0", "iload_1", "iload_2",
  "iload_3", "lload_0", "lload_1", "lload_2", "lload_3", "fload_0", "fload_1", "fload_2", "fload_3", "dload_0",
  "dload_1", "dload_2", "dload_3", "aload_0", "aload_1", "aload_2", "aload_3", "iaload", "laload", "faload",
  "daload", "aaload", "baload", "caload", "saload", "istore", "lstore", "fstore", "dstore", "astore", "istore_0",
  "istore_1", "istore_2", "istore_3", "lstore_0", "lstore_1", "lstore_2", "lstore_3", "fstore_0", "fstore_1",
  "fstore_2", "fstore_3", "dstore_0", "dstore_1", "dstore_2", "dstore_3", "astore_0", "astore_1", "astore_2",
  "astore_3", "iastore", "lastore", "fastore", "dastore", "aastore", "bastore", "castore", "sastore", "pop",
  "pop2", "dup", "dup_x1", "dup_x2", "dup2", "dup2_x1", "dup2_x2", "swap", "iadd", "ladd", "fadd", "dadd",
  "isub", "lsub", "fsub", "dsub", "imul", "lmul", "fmul", "dmul", "idiv", "ldiv", "fdiv", "ddiv", "irem",
  "lrem", "frem", "drem", "ineg", "lneg", "fneg", "dneg", "ishl", "lshl", "ishr", "lshr", "iushr", "lushr",
  "iand", "land", "ior", "lor", "ixor", "lxor", "iinc", "i2l", "i2f", "i2d", "l2i", "l2f", "l2d", "f2i",
  "f2l", "f2d", "d2i", "d2l", "d2f", "i2b", "i2c", "i2s", "lcmp", "fcmpl", "fcmpg", "dcmpl", "dcmpg", "ifeq",
  "ifne", "iflt", "ifge", "ifgt", "ifle", "if_icmpeq", "if_icmpne", "if_icmplt", "if_icmpge", "if_icmpgt",
  "if_icmple", "if_acmpeq", "if_acmpne", "goto", "jsr", "ret", "tableswitch", "lookupswitch", "ireturn",
  "lreturn", "freturn", "dreturn", "areturn", "return", "getstatic", "putstatic", "getfield", "putfield",
  "invokevirtual", "invokespecial", "invokestatic", "invokeinterface", "in_new", "new", "newarray",
  "anewarray", "arraylength", "athrow", "checkcast", "instanceof", "monitorenter", "monitorexit", "wide",
  "multianewarray", "ifnull", "ifnonnull", "goto_w", "jsr_w", "breakpoint", "in_getstatic", "in_getfield",
  "in_putfield", "getfield_quick", "putfield_quick", "getfield2_quick", "putfield2_quick", "getstatic_quick",
  "putstatic_quick", "getstatic2_quick", "putstatic2_quick", "invokevirtual_quick", "invokenonvirtual_quick",
  "invokesuper_quick", "invokestatic_quick", "invokeinterface_quick", "invokevirtualobject_quick",
  "new_quick", "anewarray_quick", "multianewarray_quick", "checkcast_quick", "instanceof_quick", "invokevirtual_quick_w",
  "getfield_quick_w", "putfield_quick_w", "no_code01", "no_code02", "no_code03", "no_code04", "no_code05",
  "no_code06", "no_code07", "no_code08", "no_code09", "no_code10", "no_code11", "no_code12", "no_code13",
  "no_code14", "no_code15", "no_code16", "no_code17", "no_code18", "no_code19", "no_code20", "no_code21",
  "no_code22", "no_code23", "no_code24", "no_code25", "impdep1", "impdep2",
};

