/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: constant.c 4294 2006-05-10 11:35:19Z chris $
*/

#include <string.h>
#include <stdio.h>

#include "checks.h"
#include "clazz.h"
#include "constant.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "hashtable.h"
#include "loading.h"
#include "methods.h"
#include "realms.h"
#include "wstrings.h"
#include "ts-mem.h"
#include "heap.h"
#include "wordset.h"

/*
** Get the Utf8 string from the constant pool array.
*/

w_string resolveUtf8Constant(w_clazz clazz, w_int i) {
  return registerString((w_string)clazz->values[i]);
}

/*
** Check whether any constant still refers to the specified UTF8 constant.
** If not, deregister the string (later we may also compact the values array).
*/
void checkUTF8References(w_clazz clazz, w_int idx) {
  w_size i;

  woempa(1, "We just deleted a reference to constant[%d] of %k, which holds the string `%w'.\n", idx, clazz, clazz->values[idx]);
  for (i = 1; i < clazz->numConstants; ++i) {

    switch (clazz->constants[i].cnt_tag) {
    case CONSTANT_CLASS:
      if ((w_int)clazz->values[i] == idx) {
        woempa(1, "Unresolved Class constant[%d] needs this UTF8 constant, so we keep it\n", i);

        return;
      }
      break;

    case CONSTANT_STRING:
      if ((w_int)clazz->values[i] == idx) {
        woempa(1, "Unresolved String constant[%d] needs this UTF8 constant, so we keep it\n", i);

        return;
      }
      break;

    case CONSTANT_NAME_AND_TYPE:
      {
        w_Name_and_Type nat = *(w_Name_and_Type*)&clazz->values[i];
        if ((w_int)nat.name_index == idx || (w_int)nat.descriptor_index == idx) {
          woempa(1, "Name&type constant[%d] needs this UTF8 constant, so we keep it\n", i);

          return;

        }
      }
      break;

    default:
      break;
    }
  }

  woempa(1, "Nobody needs this UTF8 constant, so we scrap it\n");
  clazz->constants[idx].cnt_tag = CONSTANT_DELETED;
  deregisterString((w_string)clazz->values[idx]);
}

/*
** Check whether any constant still refers to the specified Name&Type constant.
** If not, deregister the constant (later we may also compact the values array).
*/
static void checkNatReferences(w_clazz clazz, w_int idx) {
  w_Name_and_Type nat = *(w_Name_and_Type*)&clazz->values[idx];
  w_size i;

  woempa(1, "We just deleted a reference to constant[%d] of %k, which has name index %d and type index %d.\n", idx, clazz, nat.name_index, nat.descriptor_index);
  for (i = 1; i < clazz->numConstants; ++i) {
    w_Member m = *(w_Member*)&clazz->values[i];

    switch (clazz->constants[i].cnt_tag) {
    case CONSTANT_FIELD:
    case CONSTANT_METHOD:
    case CONSTANT_IMETHOD:
      if (m.name_and_type_index == idx) {
        woempa(1, "Unresolved member constant[%d] needs this Name&Type constant, so we keep it\n", i);

        return;
      }
      break;

    default:
      break;
    }
  }

  woempa(1, "Nobody needs this Name&Type constant, so we scrap it\n");
  clazz->constants[idx].cnt_tag = CONSTANT_DELETED;
  checkUTF8References(clazz, nat.name_index);
  checkUTF8References(clazz, nat.descriptor_index);
}

/*
** Add a new UTF8 constant to the pool, unless it already exists. Returns
** the index of the new or existing constant.
** We assume that the w_string is registered before calling this function,
** so we don't register it again.
*/
w_int addUTF8ConstantToPool(w_clazz clazz, w_string string) {
  w_size i;

  woempa(7, "Adding the string `%w' to constant pool of %k.\n", string, clazz);
  for (i = 1; i < clazz->numConstants; ++i) {
    if (clazz->constants[i].cnt_tag == CONSTANT_UTF8 && clazz->values[i] == (w_word)string) {
      woempa(7, "Already existed as constant[%d]\n", i);
      return i;
    }
  }
    
  for (i = 1; i < clazz->numConstants; ++i) {
    if (clazz->constants[i].cnt_tag == CONSTANT_DELETED) {
      woempa(7, "Slot[%d] is free, recycling it\n", i);
      clazz->constants[i].cnt_tag = CONSTANT_UTF8;
      clazz->values[i] = (w_word)string;
      return i;
    }
  }

  i = clazz->numConstants++;
  clazz->constants = reallocMem(clazz->constants, clazz->numConstants);
  clazz->values = reallocMem(clazz->values, clazz->numConstants * sizeof(w_word));
  woempa(7, "Adding as slot[%d]\n", i);
  clazz->constants[i].cnt_tag = CONSTANT_UTF8;
  clazz->values[i] = (w_word)string;
  return i;
}

/*
** Add a new Name&Type constant to the pool, unless it already exists.
** Returns the index of the new or existing constant.
*/
w_int addNatConstantToPool(w_clazz clazz, w_string name, w_string type) {
  w_int name_index = addUTF8ConstantToPool(clazz, name);
  w_int descriptor_index = addUTF8ConstantToPool(clazz, type);
  w_size i;
  w_Name_and_Type *natptr;

  woempa(7, "Adding the Name & Type `%w %w' to constant pool of %k.\n", name, type, clazz);

  for (i = 1; i < clazz->numConstants; ++i) {
    switch (clazz->constants[i].cnt_tag) {
    case CONSTANT_NAME_AND_TYPE:
      natptr = (w_Name_and_Type*)&clazz->values[i];
      if (natptr->name_index == name_index && natptr->descriptor_index == descriptor_index) {
        woempa(7, "Already existed as constant[%d]\n", i);
        return i;
      }
    }
  }
    
  for (i = 1; i < clazz->numConstants; ++i) {
    switch (clazz->constants[i].cnt_tag) {
    case CONSTANT_DELETED:
      natptr = (w_Name_and_Type*)&clazz->values[i];
      woempa(7, "Slot[%d] is free, recycling it\n", i);
      clazz->constants[i].cnt_tag = CONSTANT_NAME_AND_TYPE;
      natptr->name_index = name_index;
      natptr->descriptor_index = descriptor_index;
      return i;
    }
  }

  i = clazz->numConstants++;
  clazz->constants = reallocMem(clazz->constants, clazz->numConstants);
  clazz->values = reallocMem(clazz->values, clazz->numConstants * sizeof(w_word));
  natptr = (w_Name_and_Type*)&clazz->values[i];
  woempa(7, "Adding as slot[%d]\n", i);
  clazz->constants[i].cnt_tag = CONSTANT_NAME_AND_TYPE;
  natptr->name_index = name_index;
  natptr->descriptor_index = descriptor_index;
  return i;
}

static void addClassReference(w_clazz this_clazz, w_clazz ref_clazz) {
  w_int i;

  if (ref_clazz->loader == NULL || ref_clazz->loader == systemClassLoader) {
    woempa(1, "%K is a system class, doing nowt\n", ref_clazz);

    return;

  }

  if (ref_clazz == this_clazz) {
    woempa(1, "%K is same as %K, doing nowt\n", ref_clazz, this_clazz);

    return;

  }

  for (i = 0; i < this_clazz->numSuperClasses; ++i) {
    if (ref_clazz == this_clazz->supers[i]) {
      woempa(1, "%K is superclass of %K, doing nowt\n", ref_clazz, this_clazz);

      return;

    }
  }

  for (i = 0; i < this_clazz->numInterfaces; ++i) {
    if (ref_clazz == this_clazz->interfaces[i]) {
      woempa(1, "%K is superinterface of %K, doing nowt\n", ref_clazz, this_clazz);

      return;

    }
  }

  woempa(1, "%K references %K\n", ref_clazz, this_clazz);
  addToWordset(&this_clazz->references, (w_word)ref_clazz);
  newReferenceTo(clazz2Class(ref_clazz));
}

/*
** Resolve a String constant.
*/
void resolveStringConstant(w_clazz clazz, w_int i) {
#ifdef GC_SAFE_POINTS
  w_thread  thread = currentWonkaThread;
  w_boolean unsafe = enterSafeRegion(thread);
#endif
  x_monitor_eternal(clazz->resolution_monitor);
  if (clazz->constants[i].cnt_tag == CONSTANT_STRING) {
    w_cnt   c = &clazz->constants[i];
    w_word *v = &clazz->values[i];
    w_int   utf8index = (w_int)*v;
    w_string s = resolveUtf8Constant(clazz, utf8index);

    woempa(1, "Resolving String constant[%d] of %k\n", i, clazz);
    *v = (w_word)getStringInstance(s);
    woempa(1, "  --> `%w'\n", clazz->values[i]);
    c->cnt_tag = RESOLVED_STRING;
    checkUTF8References(clazz, utf8index);
  }
  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif
}

/*
** Resolve a Class constant.
*/

static void reallyResolveClassConstant(w_clazz clazz, w_cnt c, w_word *v) {

  w_thread   thread = currentWonkaThread;
  w_clazz    target_clazz = NULL;
  w_instance loader = clazz2loader(clazz);
  w_int      utf8index = (w_int)*v;
  w_string   slashed = resolveUtf8Constant(clazz, utf8index);
  w_string   dotified = slashes2dots(thread, slashed);

  c->cnt_tag = RESOLVING_CLASS;
  x_monitor_exit(clazz->resolution_monitor);
  woempa(1, "Utf8 index = %d --> class name is `%w'\n", *v, dotified);

  target_clazz = namedClassMustBeLoaded(loader, dotified);

  deregisterString(slashed);

  if (target_clazz) {
    addClassReference(clazz, target_clazz);
    mustBeSupersLoaded(target_clazz);

    x_monitor_eternal(clazz->resolution_monitor);
    *v = (w_word)target_clazz;
    c->cnt_tag = RESOLVED_CLASS;
    checkUTF8References(clazz, utf8index);
    x_monitor_notify_all(clazz->resolution_monitor);

    return;

  }

  if (exceptionThrown(thread) == NULL) {
    woempa(9, "Throwing ClassNotFoundException re: %w\n", dotified);
    throwException(thread, clazzClassNotFoundException, "%w", dotified);
  }
  else {
    woempa(9, "Got a %k\n", instance2clazz(exceptionThrown(thread)));
  }

  x_monitor_eternal(clazz->resolution_monitor);
  *v = (w_word)exceptionThrown(thread);
  c->cnt_tag = COULD_NOT_RESOLVE;
  x_monitor_notify_all(clazz->resolution_monitor);

}

/*
** Wait for another thread to finish resolving a class constant
*/
void waitForClassConstant(w_clazz clazz, w_cnt c, w_word *v) {
  w_thread   thread = currentWonkaThread;

  while (c->cnt_tag == RESOLVING_CLASS) {
    x_monitor_wait(clazz->resolution_monitor, 2);
  }

  if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
}

void resolveClassConstant(w_clazz clazz, w_int i) {

  w_cnt c = &clazz->constants[i];
  w_word *v = &clazz->values[i];
  w_thread   thread = currentWonkaThread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterSafeRegion(thread);
#endif

  x_monitor_eternal(clazz->resolution_monitor);

  if (c->cnt_tag == CONSTANT_CLASS) {
    woempa(1, "Resolving Class constant[%d] of %k\n", i, clazz);
    reallyResolveClassConstant(clazz, c, v);
  }
  else if (c->cnt_tag == RESOLVING_CLASS) {
    waitForClassConstant(clazz, c, v);
  }
  else if (c->cnt_tag == RESOLVED_CLASS) {
    /* do nothing */
  }
  else if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
  else {
    woempa(9, "Wrong tag %d for a Class constant\n", c->cnt_tag);
  }

  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

}

/*
** Resolve a Field constant.
*/
static void reallyResolveFieldConstant(w_clazz clazz, w_cnt c, w_word *v) {

  w_thread   thread = currentWonkaThread;
  w_field    field = NULL;
  w_clazz    search_clazz;
  w_clazz    super;
  w_string   desc_string;
  w_string   name;
  w_Member   member;
  w_Name_and_Type nat;
  w_int      i;
  w_int      start;
  w_int      end;
  w_clazz    value_clazz;

  c->cnt_tag = RESOLVING_FIELD;
  x_monitor_exit(clazz->resolution_monitor);
  member = *(w_Member*)v;
  woempa(1, "Class index = %d name&type = %d\n", member.class_index, member.name_and_type_index);
  search_clazz = getClassConstant(clazz, member.class_index);
  nat = *(w_Name_and_Type*)&clazz->values[member.name_and_type_index];
  name = resolveUtf8Constant(clazz, nat.name_index);
  desc_string = resolveUtf8Constant(clazz, nat.descriptor_index);

  if (search_clazz) {
    mustBeReferenced(search_clazz);
    woempa(1, "Field '%w' (type %w) should be member of %K\n", name, desc_string, search_clazz);
    super = search_clazz;
    while (super && ! exceptionThrown(thread)) {
      woempa(1, "  searching %K.\n", super);
      /*
      ** Look in the class for a field with the correct name
      ** and descriptor.
      */

      start = 0;
      end = string_length(desc_string);
      value_clazz = parseDescriptor(desc_string, &start, end, super->loader);
      if (mustBeLoaded(&value_clazz) == CLASS_LOADING_FAILED) {
        woempa(9, "  failed to load %K!\n", value_clazz);

        break;

      }

      for (i = 0; i < super->numFields; ++i) {
        w_field f = &super->own_fields[i];
        if (f->name == name && (super == search_clazz || isNotSet(f->flags, ACC_PRIVATE))) {
          if (mustBeLoaded(&f->value_clazz) == CLASS_LOADING_FAILED) {
            woempa(9, "  failed to load %K!\n", value_clazz);
          }
          else {
            woempa(1, "  matching field has value_clazz %K (%p), looking for %K (%p)\n", f->value_clazz, f->value_clazz, value_clazz, value_clazz);
            if (f->value_clazz == value_clazz) {
#ifdef PACK_BYTE_FIELDS
              woempa(1, "  found it in %K, size %d slot %d.\n", super, FIELD_SIZE(f->size_and_slot), FIELD_OFFSET(f->size_and_slot));
#else
              woempa(1, "  found it in %K, slot %d.\n", super, f->size_and_slot);
#endif
              field = f;
              break;
            }
          }
        }
      }

      if (field || exceptionThrown(thread)) {

        break;

      }

      woempa(1, "  not found in %K.\n", super);
      super = getSuper(super);
      if (!super) {
        woempa(1, "  --> giving up.\n");
        throwException(thread, clazzLinkageError, "%k: no field '%w' with type %w found in %K or its superclasses", clazz, name, desc_string, search_clazz);
      }
      woempa(1, "  --> trying %K.\n", super);
    }
    deregisterString(desc_string);

    if (! exceptionThrown(thread)) {
      addClassReference(clazz, field->value_clazz);
      x_monitor_eternal(clazz->resolution_monitor);
      *v = (w_word)field;
      c->cnt_tag = RESOLVED_FIELD;
      checkNatReferences(clazz, member.name_and_type_index);
      x_monitor_notify_all(clazz->resolution_monitor);

      return;

    }
  }
  
  x_monitor_eternal(clazz->resolution_monitor);
  *v = (w_word)exceptionThrown(thread);
  c->cnt_tag = COULD_NOT_RESOLVE;
  x_monitor_notify_all(clazz->resolution_monitor);

}

/*
** Wait for another thread to finish resolving a field constant
*/
void waitForFieldConstant(w_clazz clazz, w_cnt c, w_word *v) {
  w_thread   thread = currentWonkaThread;

  while (c->cnt_tag == RESOLVING_FIELD) {
    x_monitor_wait(clazz->resolution_monitor, 2);
  }

  if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
}

void resolveFieldConstant(w_clazz clazz, w_int i) {
  w_cnt c;
  w_word *v;
  w_thread   thread = currentWonkaThread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterSafeRegion(thread);
#endif

  x_monitor_eternal(clazz->resolution_monitor);
  c = &clazz->constants[i];
  v = &clazz->values[i];

  if (c->cnt_tag == CONSTANT_FIELD) {
    woempa(1, "Resolving Field constant[%d] of %k\n", i, clazz);
    reallyResolveFieldConstant(clazz, c, v);
  }
  else if (c->cnt_tag == RESOLVING_FIELD) {
    waitForFieldConstant(clazz, c, v);
  }
  else if (c->cnt_tag == RESOLVED_FIELD) {
    /* do nothing */
  }
  else if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
  else {
    woempa(9, "Wrong tag %d for a Field constant\n", c->cnt_tag);
  }
  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif
}

/*
** Resolve a Method constant.
*/

static void reallyResolveMethodConstant(w_clazz clazz, w_cnt c, w_word *v) {

  w_thread   thread = currentWonkaThread;
  w_method   method = NULL;
  w_clazz    search_clazz;
  w_clazz    super;
  w_cnt nat_cnt;
  w_string name;
  w_string desc_string;
  w_size   i;
  w_size   j;
  w_Member member = *(w_Member*)v;
  w_Name_and_Type nat;
  w_MethodSpec *spec = NULL;
  w_boolean ancestor;
  w_boolean same_package;

  c->cnt_tag += RESOLVING_CONSTANT;
  x_monitor_exit(clazz->resolution_monitor);
  woempa(1, "Resolving constant[%d] of %K\n", c - clazz->constants, clazz);
  woempa(1, "class index = %d name&type = %d\n", member.class_index, member.name_and_type_index);
  search_clazz = getClassConstant(clazz, member.class_index);

  if (search_clazz) {
    woempa(1, "Method was declared in class %k.\n", search_clazz);
    /*
    ** Look in the class for a method with the correct name
    ** and descriptor.
    */
    nat_cnt = &clazz->constants[member.name_and_type_index];
    nat = *(w_Name_and_Type*)&clazz->values[member.name_and_type_index];
    woempa(1,"Name & type at index %d holds {%d,%d}\n",member.name_and_type_index,nat.name_index,nat.descriptor_index);
    name = resolveUtf8Constant(clazz, nat.name_index);
    desc_string = resolveUtf8Constant(clazz, nat.descriptor_index);
    woempa(1,"  -> name is `%w', descriptor is `%w'\n",name,desc_string);

    if ((mustBeReferenced(search_clazz) != CLASS_LOADING_FAILED) && (createMethodSpecUsingDescriptor(search_clazz, name, desc_string, &spec) != CLASS_LOADING_FAILED)) {
      woempa(1,"Seeking method %w%w in %K\n", name, desc_string, search_clazz);
      ancestor = isSuperClass(search_clazz, clazz);
      same_package = sameRuntimePackage(clazz, search_clazz);
 
      for (i = 0; i < search_clazz->numDeclaredMethods; ++i) {
        w_method m = &search_clazz->own_methods[i];

        if (m->spec.name == name && ((w_string)m->spec.declaring_clazz->values[m->desc_index] == desc_string)) {
          woempa(1, "Candidate: %w%w, %ssame package, %sancestor\n", m->spec.name, m->spec.declaring_clazz->values[m->desc_index], same_package ? " " : "not ", ancestor ? " " : "not ");
          if (methodMatchesSpec(m, spec) && (same_package || isSet(m->flags, ACC_PUBLIC) || (ancestor && isSet(m->flags, ACC_PROTECTED)))) {
            method = m;
            break;
          }
        }
      }

      if (!method) {
        for (j = 0; j < search_clazz->numSuperClasses; ++j) {
          super = search_clazz->supers[j];
          ancestor = isSuperClass(super, clazz);
          same_package = sameRuntimePackage(clazz, super);
          woempa(1, "Seek %w in %K\n", name, super);
          for (i = 0; i < super->numDeclaredMethods; ++i) {
            w_method m = &super->own_methods[i];

            if (m->spec.name == name && ((w_string)m->spec.declaring_clazz->values[m->desc_index] == desc_string)) {
              woempa(1, "Candidate: %w%w, %ssame package, %sancestor\n", m->spec.name, m->spec.declaring_clazz->values[m->desc_index], same_package ? " " : "not ", ancestor ? " " : "not ");
              if (methodMatchesSpec(m, spec) && (same_package || isSet(m->flags, ACC_PUBLIC) || (ancestor && isSet(m->flags, ACC_PROTECTED)))) {
                method = m;
                break;
              }
            }
          }
        }
      }

      if (spec) {
        releaseMethodSpec(spec);
      }
    }

    if (!method && !exceptionThrown(thread)) {
        throwException(thread, clazzLinkageError, "%k: no method %w with signature %w found in %K or its superclasses", clazz, name, desc_string, search_clazz);
    }

    if (!exceptionThrown(thread)) {
      woempa(1, "found method %w%w in class %k.\n", name, desc_string, search_clazz);

      if (method->spec.arg_types) {
        for (j = 0; method->spec.arg_types[j]; ++j) {
          woempa(1, "%M argument[%d] type %k must be loaded\n", method, j, method->spec.arg_types[j]);
          addClassReference(clazz, method->spec.arg_types[j]);
        }
      }

      if (!exceptionThrown(thread)) {
        woempa(1, "%M return type %k must be loaded\n", method, method->spec.return_type);
        addClassReference(clazz, method->spec.return_type);
      }

      if (!exceptionThrown(thread)) {
        x_monitor_eternal(clazz->resolution_monitor);
        c->cnt_tag += RESOLVED_CONSTANT - RESOLVING_CONSTANT;
        *v = (w_word)method;
        checkNatReferences(clazz, member.name_and_type_index);
        x_monitor_notify_all(clazz->resolution_monitor);
        return;
      }
    }

  }

  x_monitor_eternal(clazz->resolution_monitor);
  *v = (w_word)exceptionThrown(thread);
  c->cnt_tag = COULD_NOT_RESOLVE;
  x_monitor_notify_all(clazz->resolution_monitor);

}

/*
** Wait for another thread to finish resolving a Method constant
*/
void waitForMethodConstant(w_clazz clazz, w_cnt c, w_word *v) {
  w_thread   thread = currentWonkaThread;

  while (c->cnt_tag == RESOLVING_METHOD) {
    x_monitor_wait(clazz->resolution_monitor, 2);
  }

  if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
}

void resolveMethodConstant(w_clazz clazz, w_int i) {
  w_cnt c;
  w_word *v;
  w_thread   thread = currentWonkaThread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterSafeRegion(thread);
#endif

  x_monitor_eternal(clazz->resolution_monitor);
  c = &clazz->constants[i];
  v = &clazz->values[i];

  if (clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {

    if (c->cnt_tag == CONSTANT_METHOD) {
      woempa(1, "Resolving Method constant [%d] of %k\n", i, clazz);
      reallyResolveMethodConstant(clazz, c, v);
    }
    else if (c->cnt_tag == RESOLVING_METHOD) {
      waitForMethodConstant(clazz, c, v);
    }
    else if (c->cnt_tag == COULD_NOT_RESOLVE) {
      throwExceptionInstance(thread, (w_instance)*v);
    }
    else {
      woempa(9, "Wrong tag %d for a Method constant\n", c->cnt_tag);
    }
  }
  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif
}

/*
** Resolve an IMethod constant.
** We search the interface specified in the IMethod constant and all its
** superinterfaces for a method which matches the name and descriptor;
** if we find a match then we call methodMatchesSpec to check that the
** classes of the parameters and return value really do match. We create
** the spec which we use for comparison using the class loader of the 
** interface specified in the IMethod constant, and force the classes of
** its parameters and return type to be loaded; so if a superinterface was
** loaded by a different class loader then methodMatchesSpec may also need
** to force loading of the parameter/return types. 
*/

static void reallyResolveIMethodConstant(w_clazz clazz, w_cnt c, w_word *v) {

  w_thread   thread = currentWonkaThread;
  w_method   method = NULL;
  w_method   candidate;
  w_clazz    search_clazz;
  w_clazz    current_clazz;
  w_cnt nat_cnt;
  w_string name;
  w_string desc_string;
  w_size   i;
  w_size   j;
  w_Member member = *(w_Member*)v;
  w_Name_and_Type nat;

  c->cnt_tag += RESOLVING_CONSTANT;
  x_monitor_exit(clazz->resolution_monitor);
  woempa(1, "class index = %d name&type = %d\n", member.class_index, member.name_and_type_index);
  search_clazz = getClassConstant(clazz, member.class_index);
  nat_cnt = &clazz->constants[member.name_and_type_index];
  nat = *(w_Name_and_Type*)&clazz->values[member.name_and_type_index];
  woempa(1,"Name & type at index %d holds {%d,%d}\n",member.name_and_type_index,nat.name_index,nat.descriptor_index);
  name = resolveUtf8Constant(clazz, nat.name_index);
  desc_string = resolveUtf8Constant(clazz, nat.descriptor_index);
  woempa(1,"  -> name is `%w', descriptor is `%w'\n",name,desc_string);

  if (search_clazz) {
    w_MethodSpec *spec = NULL;
    woempa(1, "IMethod %w%w should be declared in class %K.\n", name, desc_string, search_clazz);

    if ((mustBeReferenced(search_clazz) != CLASS_LOADING_FAILED) && (createMethodSpecUsingDescriptor(search_clazz, name, desc_string, &spec) != CLASS_LOADING_FAILED)) {
      current_clazz = search_clazz;
      woempa(1, "  searching %K\n", current_clazz);
      for (i = 0; i < current_clazz->numDeclaredMethods; ++i) {
        candidate = &current_clazz->own_methods[i];
        if (candidate->spec.name == name && (w_string)current_clazz->values[candidate->desc_index] == desc_string) {
          method = candidate;
          break;
        }
      }

      if (method && !methodMatchesSpec(method, spec)) {
        woempa(1, "found a method %w%w in class %k, but on closer inspection the parameters / return type do not match.\n", name, desc_string, current_clazz);
        method = NULL;
      }
      else if (!method) {
        woempa(1, "did not find interface method %w%w in class %k.\n", name, desc_string, current_clazz);
        for (j = 0; j < search_clazz->numInterfaces && !method && ! exceptionThrown(thread); ++j) {
          current_clazz = search_clazz->interfaces[j];
          woempa(1, "  searching %K\n", current_clazz);
          spec->declaring_clazz = current_clazz;
          for (i = 0; i < current_clazz->numDeclaredMethods; ++i) {
             candidate = &current_clazz->own_methods[i];
            if (candidate->spec.name == name && (w_string)current_clazz->values[candidate->desc_index] == desc_string) {
              method = candidate;
              break;
            }
          }
          if (!method) {
            woempa(1, "did not find method %w%w in class %k.\n", name, desc_string, current_clazz);
          }
          else if (!methodMatchesSpec(method, spec)) {
            woempa(1, "found a method %w%w in class %k, but on closer inspection the parameters / return type do not match.\n", name, desc_string, current_clazz);
            method = NULL;
            break;
          }
        }
      }

      if (spec) {
        releaseMethodSpec(spec);
      }
    }

    if (!method && !exceptionThrown(thread)) {
        throwException(thread, clazzLinkageError, "%k: no interface method %w with signature %w found in %K or its superinterfaces", clazz, name, desc_string, search_clazz);
    }

    if (!exceptionThrown(thread)) {
      woempa(1, "found interface method %w%w in class %k.\n", name, desc_string, search_clazz);

      if (method->spec.arg_types) {
        for (j = 0; method->spec.arg_types[j]; ++j) {
          addClassReference(clazz, method->spec.arg_types[j]);
        }
      }

      if (!exceptionThrown(thread)) {
        addClassReference(clazz, method->spec.return_type);
      }

      if (!exceptionThrown(thread)) {
        x_monitor_eternal(clazz->resolution_monitor);
        c->cnt_tag += RESOLVED_CONSTANT - RESOLVING_CONSTANT;
        *v = (w_word)method;
        checkNatReferences(clazz, member.name_and_type_index);
        x_monitor_notify_all(clazz->resolution_monitor);
        return;
      }
    }

  }

  x_monitor_eternal(clazz->resolution_monitor);
  *v = (w_word)exceptionThrown(thread);
  c->cnt_tag = COULD_NOT_RESOLVE;
  x_monitor_notify_all(clazz->resolution_monitor);

}

/*
** Wait for another thread to finish resolving an IMethod constant
*/
void waitForIMethodConstant(w_clazz clazz, w_cnt c, w_word *v) {
  w_thread   thread = currentWonkaThread;

  while (c->cnt_tag == RESOLVING_IMETHOD) {
    x_monitor_wait(clazz->resolution_monitor, 2);
  }

  if (c->cnt_tag == COULD_NOT_RESOLVE) {
    throwExceptionInstance(thread, (w_instance)*v);
  }
}

void resolveIMethodConstant(w_clazz clazz, w_int i) {
  w_cnt c;
  w_word *v;
  w_thread   thread = currentWonkaThread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterSafeRegion(thread);
#endif

  x_monitor_eternal(clazz->resolution_monitor);
  c = &clazz->constants[i];
  v = &clazz->values[i];

  if (clazz->constants[i].cnt_tag < RESOLVED_CONSTANT) {

    if (c->cnt_tag == CONSTANT_IMETHOD) {
      woempa(1, "Resolving IMethod constant [%d] of %k\n", i, clazz);
      reallyResolveIMethodConstant(clazz, c, v);
    }
    else if (c->cnt_tag == RESOLVING_IMETHOD) {
      waitForIMethodConstant(clazz, c, v);
    }
    else if (c->cnt_tag == COULD_NOT_RESOLVE) {
      throwExceptionInstance(thread, (w_instance)*v);
    }
    else {
      woempa(9, "Wrong tag %d for an IMethod constant\n", c->cnt_tag);
    }
  }
  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif
}

void dissolveConstant(w_clazz clazz, int idx) {
  w_cnt c = &clazz->constants[idx];

  if (c->cnt_tag == CONSTANT_UTF8) {
    if (clazz->values[idx]) {
      deregisterString((w_string)clazz->values[idx]);
    }
  }
}

#ifdef DEBUG
/*
** Print out all contant pool entries of a class.
*/
void dumpPools(int fd, w_clazz clazz) {

  w_size i;
  w_cnt  c;

  woempa(1,"=== Constant Pool Dump for %k (fd = %d) ===\n", clazz, fd);
  for (i = 1; i < clazz->numConstants; i++) {
    if (clazz->constants) {
      c = &clazz->constants[i];
      switch (c->cnt_tag) {
      case CONSTANT_CLASS:
        woempa(1, "%4d Class:     utf8 index %d\n", i, clazz->values[i]);
        break;

      case RESOLVING_CLASS:
        woempa(1, "%4d Class (resolving)\n", i);
        break;

      case RESOLVED_CLASS:
        woempa(1, "%4d Class (resolved): %k\n", i, clazz->values[i]);
        break;

      case CONSTANT_UTF8:
        woempa(1, "%4d UTF8:      '%w'\n", i, clazz->values[i]);
        break;

      case CONSTANT_INTEGER:
        woempa(1, "%4d Integer    0x%x\n", i, clazz->values[i]);
        break;

      case CONSTANT_FLOAT:
        woempa(1, "%4d Float:     0x%x\n", i, clazz->values[i]);
        break;

      case CONSTANT_LONG:
        woempa(1, "%4d Long:      0x%x%x\n", i, clazz->values[i + WORD_MSW], clazz->values[i + WORD_LSW]);
        break;

      case CONSTANT_DOUBLE:
        woempa(1, "%4d Double:    0x%x%x\n", i, clazz->values[i + WORD_MSW], clazz->values[i + WORD_LSW]);
        break;

      case CONSTANT_STRING:
        woempa(1, "%4d String (unresolved):    index %d\n", i, clazz->values[i]);
        break;

      case RESOLVED_STRING:
        woempa(1, "%4d String (resolved): `%w'\n", i, String2string((w_instance)clazz->values[i]));
        break;

      case CONSTANT_FIELD:
        {
          w_Member member = *(w_Member*)&clazz->values[i];
          woempa(1, "%4d Field (unresolved):     class index %d name&type %d\n", i, member.class_index, member.name_and_type_index);
        }
        break;

      case RESOLVING_FIELD:
        woempa(1, "%4d Field (resolving)\n, i");
        break;

      case RESOLVED_FIELD:
        {
          w_field field = (w_field)clazz->values[i];
          woempa(1, "%4d Field (resolved): %k %w of %k\n", i, field->value_clazz, field->name, field->declaring_clazz);
        }
        break;

      case CONSTANT_METHOD:
        {
          w_Member member = *(w_Member*)&clazz->values[i];
          woempa(1, "%4d Method (unresolved):    class index %d name&type %d\n", i, member.class_index, member.name_and_type_index);
        }
        break;

      case RESOLVING_METHOD:
        woempa(1, "%4d Method (resolving)\n", i);
        break;

      case RESOLVED_METHOD:
        {
          w_method method = (w_method)clazz->values[i];
          woempa(1, "%4d Method (resolved): %M\n", i, method);
        }
        break;

      case CONSTANT_IMETHOD:
        {
          w_Member member = *(w_Member*)&clazz->values[i];
          woempa(1, "%4d IMethod (unresolved):   class index %d name&type %d\n", i, member.class_index, member.name_and_type_index);
        }
        break;

      case RESOLVING_IMETHOD:
        woempa(1, "%4d IMethod (resolving)\n", i);
        break;

      case RESOLVED_IMETHOD:
        {
          w_method method = (w_method)clazz->values[i];
          woempa(1, "%4d IMethod (resolved): %M\n", i, method);
        }
        break;

      case CONSTANT_NAME_AND_TYPE:
        {
          w_Name_and_Type nat = *(w_Name_and_Type*)&clazz->values[i];
        woempa(1, "%4d Name&type: name index %d descriptor index %d\n", i, nat.name_index, nat.descriptor_index); 
        }
        break;

      case CONSTANT_DELETED:
        woempa(1,"%4d Constant was no longer needed, has been cleaned up\n", i);
        break;

      case COULD_NOT_RESOLVE:
        woempa(1,"%4d Failed to resolve constant: threw %k\n", i, instance2clazz((w_instance)clazz->values[i]));
        break;

      case NO_VALID_ENTRY:
        woempa(1,"%4d Skipping second word of double-size constant\n", i);
        break;

      default:
        woempa(9,"clazz %k index %d c->cnt_tag = %d\n", clazz, i, c->cnt_tag);
        wabort(ABORT_WONKA, "Should not get here...\n");
      }
    }
  }
  woempa(1,"=== End Constant Pool Dump for %k ===\n", clazz);

}
#endif


