/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

#ifdef FSENABLE

#include <wonka.h>
#include <ts-mem.h>

#include <stdlib.h>
#include <stdio.h>

#include "driver_blockrandom.h"

extern void _binary_image_end;
extern void _binary_image_start;

typedef struct w_Control_Image {
  w_ubyte  *disk;
  w_long   size;
} w_Control_Image;

typedef w_Control_Image *w_control_image;

/*
** Prototypes 
*/

w_void image_initDevice(w_device device);
w_void image_termDevice(w_device device);
w_device image_open(w_device device);
w_void image_close(w_device device);
w_driver_status image_read(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout);
w_driver_status image_write(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout);
w_driver_status image_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout);
w_driver_status image_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout);


w_void image_initDevice(w_device device) {
  w_control_image  image;

  device->control = allocMem_system(sizeof(w_Control_Image));
  image = device->control;

  image->disk = (w_ubyte *)&_binary_image_start;
  image->size = (w_long)((w_ubyte *)&_binary_image_end - image->disk);
}

w_void image_termDevice(w_device device) {
  //w_control_image image = device->control;
  
  releaseMem(device->control); 
}


w_device image_open(w_device device) {

  device->usage += 1;
  device->driver->usage += 1;
  
  return device;
}


w_void image_close(w_device device) {

  device->usage -= 1;
  device->driver->usage -= 1;
}


w_driver_status image_read(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout) {
  w_control_image image = device->control;
  
  woempa(2, "sector %d   size %d\n", sector, size);
  w_memcpy(bytes, (w_ubyte *)(image->disk + 512 * sector), size);
  
  return wds_success;
}


w_driver_status image_write(w_device device, w_ubyte *bytes, w_int sector, w_int size, x_sleep timeout) {
  w_control_image image = device->control;
  
  woempa(2, "sector %d   size %d\n", sector, size);
  w_memcpy((w_ubyte *)(image->disk + 512 * sector), bytes, size);

  return wds_success;
}


w_driver_status image_set(w_device device, w_driver_ioctl command, w_word param, x_sleep timeout) {
  return wds_not_implemented;
}


w_driver_status image_query(w_device device, w_driver_ioctl query, w_word *reply, x_sleep timeout) {
  return wds_not_implemented;
}


w_Driver_BlockRandom image_driver = {
  "image-driver-linux_v0.1",
  "hdb",
  0,
  NULL,
  image_initDevice,
  image_termDevice,
  image_open,
  image_close,
  image_read,
  image_write,
  image_set,
  image_query
};

#endif

