/**************************************************************************
* Copyright (c) 2001, 2002, 2003, 2004 by Acunia N.V. All rights reserved.*
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: collector.c 4294 2006-05-10 11:35:19Z chris $
*/

#define PRINTRATE 10

#include "arrays.h"
#include "checks.h"
#include "clazz.h"
#include "core-classes.h"
#include "fields.h"
#include "fifo.h"
#include "hashtable.h"
#include "interpreter.h"
#ifdef OS_NONE
#include "comms.h"
#endif
#include "jni.h"
#include "loading.h"
#include "locks.h"
#include "ts-mem.h"
#include "methods.h"
#include "oswald.h"
#include "wstrings.h"
#include "threads.h"
#include "list.h"
#include "constant.h"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

/*
** If DETECT_FLYING_PIGS is defined, Wonka will wabort() if it ever finds that an object
** which was unreachable in pass[n] (O_GARBAGE is set) is reachable in pass[n+1]. You
** probably don't want to set this in production code, because most of the time this
** glitch doesn't lead to a crash: but if there are signs that objects are being GC'd
** while they are still in use (or used after they have been GC'd) then this will help
** to find the guilty party.
*/
//#define DETECT_FLYING_PIGS

#ifdef DETECT_FLYING_PIGS
#define FLYING_PIG_CHECK(o) _flying_pig_check(__FUNCTION__,__FILE__,__LINE__,o)

static void _flying_pig_check(char *func, char *file, int line, w_object object) {
  if (isSet(object->flags, O_GARBAGE)
#ifndef GC_SAFE_POINTS
    && isNotSet(object->flags, O_GRAY)
#endif
    && isNotSet(object->flags, O_GREEN)
  ) {
    w_instance instance = object->fields;

    if (object->clazz == clazzString) {
      wabort(ABORT_WONKA, "Flying pig %j (`%w')! in %s at %s:%d\n", instance, instance[F_String_wotsit], func, file, line);
    }
    else {
      wabort(ABORT_WONKA, "Flying pig %j! in %s at %s:%d\n", instance, func, file, line);
    }
  }
}
#else
#define FLYING_PIG_CHECK(o)
#endif

/*
** Enable this if you really want classes and class loaders to be subject
** to garbage collection. Otherwise they will always be treated as reachable,
** even when the mark phase fails to mark them ...
#define COLLECT_CLASSES_AND_LOADERS
*/

void persistentGarbageCollector(void);

/*
** After freeing bytes_per_meal bytes, the garbage collector will have a
** `nap attack', i.e. will sleep for a short time to give other threads
** a chance. If set to 0, the GC will not take naps. The millis_per_nap
** constant defines the duration of a nap when memory_load_factor is 0;
** higher memory loads lead to shorter naps.
*/

static const w_size bytes_per_meal = 4 * 1024 * 1024;
static const w_size millis_per_nap = 350;

/*
** If DISTRIBUTED_GC is defined, the reclaim callback mechanism is activated.
*/

#define DISTRIBUTED_GC

/*
** If USE_DISCARD_COLLECT is defined, discardMem and collectMem
** will be used instead of releaseMem.
*/

#define USE_DISCARD_COLLECT

/*
** If CONTRACT_FIFOS is defined, the various fifos will be shrunk down to one
** leaf at the start of each GC cycle.  This is useful when checking for memory
** leaks, because the total memory used by fifos depends only on the number of
** instances currently in use, not the high-water mark. Otherwise it represents
** unnecessary overhead and actually increases the probability of a catastrophic
** out-of-memory condition (not enough speace to allocate the fifos required in
** order to perform GC).
**
** REDUCE_FIFOS has a similar effect, but in this case at most one leaf will be
** released from each fifo, so they will still shrink down but more slowly.  
** Kind of a Belgian compromise. ;>
*/
//#define CONTRACT_FIFOS
#define REDUCE_FIFOS

#if (defined(CONTRACT_FIFOS) && defined(REDUCE_FIFOS))
#warning Both CONTRACT_FIFOS and REDUCE_FIFOS are defined, which is silly.
#endif

/*
** FIFO which holds all instances in the current GC window: the GC window
** is the set of instances which existed at the moment the current GC cycle
** began.  We give this one quite a large leaf size, as it is normal to have
** 1000's of objects.
*/

static w_fifo window_fifo;
#define WINDOW_FIFO_LEAF_SIZE   4095

/*
** FIFO used to mark all strongly reachable instances.  We give this one
** quite a large leaf size too.
*/

static w_fifo strongly_reachable_fifo;
#define STRONG_FIFO_LEAF_SIZE 4095

/*
** FIFOs used to mark instances which are reachable only via reference types. 
** We give these a smaller leaf size.
*/

static w_fifo softly_reachable_fifo;
static w_fifo weakly_reachable_fifo;
static w_fifo phantom_reachable_fifo;
static w_fifo finalize_reachable_fifo;
#define OTHER_FIFO_LEAF_SIZE 63

/*
** FIFO used to hold instances of Reference on which enqueue() should be called.. 
*/

w_fifo enqueue_fifo;
#define ENQUEUE_FIFO_LEAF_SIZE         OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/

static x_Mutex enqueue_fifo_Mutex;
x_mutex enqueue_fifo_mutex;


/*
** List of instances whose state has changed since the current GC cycle began,
** or which have been flagged as immortal.  These are added to the normal
** roots (the normal roots are the static fields of all classes and the
** stack frames of all threads). Not required if GC Safe Points are used.
*/

#ifndef GC_SAFE_POINTS
w_fifo gray_fifo;
#define GRAY_FIFO_LEAF_SIZE       OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/

static x_Monitor gray_fifo_Monitor;
static x_monitor gray_fifo_monitor;
#endif

/*
** List of instances waiting to be finalized.
*/

w_fifo finalizer_fifo;
#define FINALIZER_FIFO_LEAF_SIZE   OTHER_FIFO_LEAF_SIZE

/*
** Mutex used to protect the fifo from conflicting accesses.
*/

static x_Mutex finalizer_fifo_Mutex;
x_mutex finalizer_fifo_mutex;

/*
** The Wonka thread in which GC is performed.
*/

w_thread gc_thread;

/*
** The unique instance of GarbageCollector corresponding to gc_thread.
*/

w_instance gc_instance;

/*
** A lock (monitor) associated with the GC thread.
*/

x_monitor gc_monitor;

/*
** The number of times the monitor has been notify'd.
*/

static w_int *gc_kicks_pointer;

/*
** The `total' memory is the amount returned by x_mem_total, minus 
** min_heap_free.  Calculated during GC startup.
*/

static w_int memory_total;

/*
** The memory load factor is the integer part of (rho / (1-rho)), where
** rho = (memory_total - x_mem_avail()) / memory_total .
** It is calculated at the start of each GC cycle.
*/

static w_int memory_load_factor;

/*
** The name of the `enqueue' method.
*/
//static w_string string_enqueue;

/*
** List of strings for which string->instance should always be considered reachable.
*/
static w_string *static_strings[] = {
  &string_NULL,
  &string_BOGUS,
  &string_EMPTY,
  &string_ConstantValue,
  &string_Code,
  &string_Deprecated,
  &string_Exceptions,
  &string_LineNumberTable,
  &string_LocalVariableTable,
  &string_Reference,
  &string_SourceFile,
  &string_Synthetic,
  &string_angle_brackets_init,
  &string_angle_brackets_clinit,
  &string_no_params,
  &string_no_params_V,
  &string_boolean,
  &string_byte,
  &string_char,
  &string_double,
  &string_finalize,
  &string_run,
  &string_float,
  &string_int,
  &string_long,
  &string_short,
  &string_void,
  &string_medial,
  &string_square,
  &string_compat,
  &string_narrow,
  &string_small,
  &string_isolated,
  &string_initial,
  &string_font,
  &string_super,
  &string_final,
  &string_circle,
  &string_vertical,
  &string_fraction,
  &string_wide,
  &string_noBreak,
  &string_sub,
  NULL
};

/*
** For testing ...
*/
w_int sting_count = 0;

/*
** A macro to sting the collector where it hurts the most. For normal
** behaviour, undef sting_collector.
*/

#define sting_collector()  {                    \
  unsigned int rs = (unsigned int)(rand() % 2); \
  if (rs) {                                     \
    sting_count++;                              \
    x_thread_sleep(1);                          \
  }                                             \
}

#undef sting_collector
#define sting_collector()

/*
** Stuff to do with debugging output ...
*/
#define BUF_SIZE 33

char *printFlags(w_flags code) {

  static char buffer[BUF_SIZE];
  w_word mask = (unsigned)(1 << 31);
  w_int i = 0;
  
  memset(buffer, 0x00, BUF_SIZE);
  while (mask) {
    if ((w_word)code & mask) {
      buffer[i++] = '1';
    }
    else {
      buffer[i++] = '0';
    }
    mask >>= 1;
  }

  return buffer;
  
}

extern void String_destructor(w_instance);
extern void Class_destructor(w_instance);
extern void ClassLoader_destructor(w_instance);
extern void Thread_destructor(w_instance);
extern void ThreadGroup_destructor(w_instance);
extern void Throwable_destructor(w_instance);
 
/*
** Reclaim an instance by returning its memory to the global object heap.
** The value returned is the size of the instance in bytes (the amount of 
** memory freed is generally more than this).
*/
static w_int releaseInstance(w_object object) {
  w_clazz clazz;
  w_size  bytes = 0;

  if (object == NULL) {
    wabort(ABORT_WONKA, "Releasing NULL object !!\n");
  }
  
  clazz = object->clazz;
//  woempa(1, "(GC) Releasing %p (object %p) = %k flags %s\n", (char *)object->fields, object, clazz, printFlags(object->flags));

  if (object->clazz == clazzString) {
    String_destructor(object->fields);
  }
  else if (object->clazz == clazzClass) {
#ifdef COLLECT_CLASSES_AND_LOADERS
    Class_destructor(object->fields);
#else
    return 0;
#endif
  }
  else if (isSet(object->clazz->flags, CLAZZ_IS_CLASSLOADER)) {
#ifdef COLLECT_CLASSES_AND_LOADERS
    ClassLoader_destructor(object->fields);
#else
    return 0;
#endif
  }
  else if (isSet(clazz->flags, CLAZZ_IS_THREAD)) {
    woempa(1, "Releasing Thread %t.\n", object->fields);
    Thread_destructor(object->fields);
  }
  else if (isSet(object->clazz->flags, CLAZZ_IS_THREADGROUP)) {
    ThreadGroup_destructor(object->fields);
  }
  else if (isSet(object->clazz->flags, CLAZZ_IS_THROWABLE)) {
    Throwable_destructor(object->fields);
  }

  if (clazz->previousDimension) {
    bytes = clazz->previousDimension->bits * instance2Array_length(object->fields);
    bytes = bytes + sizeof(w_int) + sizeof(w_Object);
    bytes = (bytes + 7) & ~7;
  }
  else {
    bytes = clazz->bytes_needed;
  }
//    woempa(1,"(GC) -> freeing %d bytes\n", bytes);

//    woempa(9, "RELEASING %k %p\n", object->clazz, object->fields);
#ifdef USE_DISCARD_COLLECT
  discardMem(object);
#else
  releaseMem(object);
#endif

#ifdef JAVA_PROFILE
  clazz->instances--;
#endif 

  return bytes;
}

/*
* Function impliesMark returns TRUE iff the highest-order 1 bit in flag1
** is at least as high-order as the single 1 bit in flag2.
** N.B. flag2 must be a power of 2!
*/
static inline w_boolean impliesMark(w_word flag1, w_word flag2) {
  return flag1 > (flag2 - 1);
}

/*
** Function isMarked returns TRUE iff the object in question is already
** marked either with the given flag or by one which `dominates' it, i.e.
** precedes it in the series 
**   O_BLACK > O_SOFT_BLACK > O_WEAK_BLACK > O_PHANTOM_BLACK > O_FINALIZE_BLACK.
** Note that we rely on the fact that the flags in question appear in
** the object flags word contiguously and in the order shown above.
*/
static inline w_boolean isMarked(w_object o, w_word flag) {
  return impliesMark(o->flags & (O_BLACK | O_NEAR_BLACK), flag);
}

#define isNotMarked(o,flag) !isMarked(o,flag)

/*
** Try to add a given instance to a given fifo. Return 1 if successful,
** -1 if failed (because fifo is already full and could not be extended).
*/
static inline w_int tryPutFifo(w_instance instance, w_fifo fifo) {
  if (!instance) {
    wabort(ABORT_WONKA, "Attempt to enqueue null instance!\n");
  }

  if (!instance2clazz(instance)) {
    wabort(ABORT_WONKA, "Instance at %p has no class!\n", instance);
  }

  if (putFifo(instance, fifo) < 0) {
    woempa(9, "Shiver my timbers! Couldn't push instance %j onto fifo %p\n", instance, fifo);

    return -1;

  };

  return 1;
}

/*
** If the given instance is not already marked with the given flag (or a
** "stronger" one), mark it and put it on the given queue. Returns 0 if
** nothing was done, 1 if the instance was queued successfully, -1 if
** enqueueing failed.
*/
#ifdef DETECT_FLYING_PIGS
#define markInstance(instance,fifo,flag) _markInstance(instance,fifo,flag,__FUNCTION__,__FILE__,__LINE__)
 
w_int _markInstance(w_instance instance, w_fifo fifo, w_word flag, char *func, char *file, int line) {
  w_object object = instance2object(instance);
  w_int retcode;

  _flying_pig_check(func, file, line, object);

#else
w_int markInstance(w_instance instance, w_fifo fifo, w_word flag) {
  w_object object = instance2object(instance);
  w_int retcode;
#endif

  if (isMarked(object, flag)) {

    return 0;

  }

  woempa(1, "Pushing %j onto fifo @ %p, setting flag %d\n", instance, fifo, flag);
  setFlag(object->flags, flag);
  retcode = tryPutFifo(instance, fifo);
  if (retcode < 0) {

    return retcode;

  }

  return 1;
}

/*
** Mark all strings in the static_strings list.
*/
w_int markStaticStrings(void) {
  w_instance child_instance;
  w_int      queued = 0;
  w_int      retcode;
  w_string **ptrptr;
  w_string  *ptr;
  w_string   s;

  woempa(1, "Marking static strings\n");
  ptrptr = static_strings;
  while ((ptr = *ptrptr++)) {
    s = *ptr;
    woempa(1, "  %w\n", s);
    if (s) {
      child_instance = getStringInstance(s);
      retcode = markInstance(child_instance, strongly_reachable_fifo, O_BLACK);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }
  }

  ptr = category_name;
  while ((s = *ptr++)) {
    woempa(1, "  %w\n", s);
    child_instance = getStringInstance(s);
    retcode = markInstance(child_instance, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }

  return queued;
}

/*
** If a Class is reachable (because instances exist, or because it is
** known to a ClassLoader), and it has been initialised, we queue all
** of its (unmarked) static fields for marking.
**
** JD: Skip it if the clazz is marked 'BROKEN'
*/

w_int markClazzReachable(w_clazz clazz, w_fifo fifo, w_word flag) {
  w_instance child_instance;
  w_int      i;
  w_int      queued = 0;
  w_int      retcode;
  w_int      state = getClazzState(clazz);

  woempa(1, "(GC) Marking %K\n", clazz);
  child_instance = clazz->loader;
  if (!child_instance) {
    child_instance = systemClassLoader;
  }

  if (child_instance) {
    retcode = markInstance(child_instance, fifo, flag);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }

  woempa(1, "clazz->dotified = %w\n", clazz->dotified);
  child_instance = getStringInstance(clazz->dotified);
  retcode = markInstance(child_instance, fifo, flag);
  if (retcode < 0) {

    return retcode;

  }
  queued += retcode;

  if (clazz->failure_message) {
    woempa(1, "clazz->failure_message = %w\n", clazz->failure_message);
    child_instance = getStringInstance(clazz->failure_message);
    retcode = markInstance(child_instance, fifo, flag);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }

  if (clazz->source) {
    woempa(1, "clazz->source = %w\n", clazz->source);
    child_instance = getStringInstance(clazz->source);
    retcode = markInstance(child_instance, fifo, flag);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }

  child_instance = clazz2Class(clazz);
  if (child_instance) {
    retcode = markInstance(child_instance, fifo, flag);
    if (retcode < 0) {

      return retcode;

    }
    queued += retcode;
  }
#ifdef RUNTIME_CHECKS
  else if (!clazz->previousDimension) {
    wabort(ABORT_WONKA, "No Class instance for non-array class %k\n", clazz);
  }
#endif

  if (state == CLAZZ_STATE_BROKEN) {

    return 0;

  }

  if (clazz->references) {
    w_int n = sizeOfWordset(&clazz->references);

    for (i = 0; i < n; ++i) {
      woempa(1, "(GC) %K references %K\n", clazz, elementOfWordset(&clazz->references, i));
      child_instance = clazz2Class((w_clazz)elementOfWordset(&clazz->references, i));
      if (child_instance) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }
  }

  woempa(1, "(GC) Marking fields of class %k\n",clazz);
  for (i = 0; i < (w_int)clazz->numFields; ++i) {
    child_instance = getStringInstance(clazz->own_fields[i].name);
    if (child_instance) {
      retcode = markInstance(child_instance, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }
  }

  woempa(1, "(GC) Marking methods of class %k\n",clazz);
  for (i = 0; i < (w_int)clazz->numDeclaredMethods; ++i) {
    child_instance = getStringInstance(clazz->own_methods[i].spec.name);
    if (child_instance) {
      retcode = markInstance(child_instance, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }
  }

  woempa(1, "(GC) Marking constant pool of class %k\n",clazz);
  for (i = 1; i < (w_int)clazz->numConstants; ++i) {
    if (clazz->constants[i].cnt_tag == COULD_NOT_RESOLVE) {
      child_instance = (w_instance)clazz->values[i];
      woempa(1, "(GC)    --> constant[%d] of %k contains instance %j.\n", i, clazz, child_instance);
      if (child_instance) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }
    else if (clazz->constants[i].cnt_tag == CONSTANT_UTF8) {
      child_instance = getStringInstance((w_string)clazz->values[i]);
      woempa(1, "(GC)    --> constant[%d] of %k is CONSTANT_UTF8, contains string '%w' in instance %p.\n", i, clazz, clazz->values[i], child_instance);
      if (child_instance) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }
    else if (clazz->constants[i].cnt_tag == RESOLVED_STRING) {
      child_instance = (w_instance)clazz->values[i];
      woempa(1, "(GC)    --> constant[%d] of %k is RESOLVED_STRING, contains string '%w' in instance %p.\n", i, clazz, child_instance[F_String_wotsit], child_instance);
      if (child_instance) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }
  }

  if (state >= CLAZZ_STATE_SUPERS_LOADED) {
    if (clazz->supers) {
      woempa(1, "(GC) Marking Class instance of superclass (%K) of %K\n", clazz->supers[0], clazz);
      child_instance = clazz2Class(clazz->supers[0]);
      if (child_instance) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }

    if (clazz->interfaces) {
      for (i = 0; i < clazz->numInterfaces; ++i) {
        child_instance = clazz2Class(clazz->interfaces[i]);
        if (child_instance) {
          retcode = markInstance(child_instance, fifo, flag);
          if (retcode < 0) {

            return retcode;

          }
          queued += retcode;
        }
      }
    }
  }

  if (clazz->staticFields) {
    woempa(1,"(GC) Marking static fields of class %k\n",clazz);
    for (i = 0; i < (w_int)clazz->numStaticFields; ++i) {
      if (isSet(clazz->own_fields[i].flags, FIELD_IS_REFERENCE)) {
        child_instance = (w_instance)clazz->staticFields[clazz->own_fields[i].size_and_slot];
        woempa(1, "(GC)    --> field %w of %k contains instance %p.\n", clazz->own_fields[i].name, clazz, child_instance);
        if (child_instance) {
          retcode = markInstance(child_instance, fifo, flag);
          if (retcode < 0) {

            return retcode;

          }
          queued += retcode;
        }
      }
      else {
        woempa(1, "(GC)    --> field %w of %k is primitive, getting out.\n",  clazz->own_fields[i].name, clazz);
        break;
      }
    }
  }

  return queued;
}

/*
** Mark one frame of a Java stack: we push onto the fifo every local variable
** and every stack item which looks like a reference and hasn't already been
** marked.
*/

w_int markFrameReachable(x_frame frame, w_fifo fifo, w_word flag) {

  w_int queued = 0;
  volatile x_slot item;
  w_instance child_instance;
  w_instance thisThread = frame->thread->Thread;
  w_int      retcode;

#ifdef JSPOT
  w_instance * map;
#endif

  item = (volatile x_slot) frame->Slots;
  while (item < (volatile x_slot) frame->top) {
    if (item->s == stack_trace && item->c.w) {
      child_instance = (w_instance) item->c.w;
      if (child_instance != thisThread) {
        retcode = markInstance(child_instance, fifo, flag);
        if (retcode < 0) {

          return retcode;

        }
        queued += retcode;
      }
    }
    item += 1;
  }

#ifdef JSPOT
  #ifdef DEBUG
  /*
  ** Ensure that we are in a safe state or bad things will
  ** happen.
  */

  if (isSet(frame->thread->flags, WT_THREAD_NOT_GC_SAFE)) {
    wabort(ABORT_WONKA, "attempt to mark frame %p while it is not in a safe state\n", frame);
  }

  if (thread->top && thread->top->method && (thread->top->method->flags & METHOD_IS_COMPILED)) {
    wabort(ABORT_WONKA, "attempt to mark thread %p while the top-most frame %p is controlled by a JITed method\n", thread, thread->top);
  }


  woempa(8,"%p: scanning frame %p of method %M emitted by %M\n", frame->thread, frame, frame->method, (frame->previous ? frame->previous->method : NULL));

  #endif


  /*
  ** Scan the stack maps emitted by j-spotted methods.
  */
  if (frame->flags & FRAME_STACKMAP) {
  
    woempa(8,"%p: scanning frame %p of method %M emitted by %M\n", frame->thread, frame, frame->method, (frame->previous ? frame->previous->method : NULL));

    map = frame->map;
    while (*map != (w_instance)42) {
      if (*map != NULL) {
        /*
        ** Note that the woempa below helps to detect faulty stack map entries:
        ** that is, Wonka will crash when it tries to print a non-reference.
        */
        
        woempa(8, "reference: %p at location %p of type %j\n", *map, map, *map);
        
        retcode = markInstance(*map, fifo, flag);
        if (retcode < 0) {
          return retcode;
        }
        queued += retcode;
      }
      map++;
    }
  }
#endif
  
  return queued;

}

/*
** If a Thread is reachable (e.g. is reachable from the system thread group)
** and it has a valid Java stack then we mark every frame.
*/

w_int markThreadReachable(w_object object, w_fifo fifo, w_word flag) {

  w_thread thread = getWotsitField(object->fields, F_Thread_wotsit);
  w_int numFrames = 0;
  w_int queued = 0;
  w_int retcode;
  volatile x_frame frame;
  volatile x_slot auxs;
  w_instance i;

  if (thread == NULL || thread->state == wt_dead) {
    return 0;
  }

#ifdef GC_SAFE_POINTS
  if ((thread->flags & WT_THREAD_NOT_GC_SAFE) != 0) {
    kill(getpid(), 3);
    wabort(ABORT_WONKA, "attempt to mark thread %p while it is not in a safe state\n", thread);
  }

  if (thread->top && thread->top->method && (thread->top->method->flags & METHOD_IS_COMPILED)) {
    wabort(ABORT_WONKA, "attempt to mark thread %p while the top-most frame %p is controlled by a JITed method\n", thread, thread->top);
  }
#else
  x_preemption_disable;
#endif

  woempa(1, "Scanning aux stack of %t (%d items, top frame is %M)\n", thread,thread->top->auxs - thread->auxs, thread->top->method);
  for (auxs = (volatile x_slot) thread->auxs; auxs < (volatile x_slot) thread->top->auxs; auxs += 1) {
    if (auxs->c.w && (auxs->s == stack_trace || isMonitoredSlot(auxs))) {
      i = (w_instance) auxs->c.w;
      retcode = markInstance(i, fifo, flag);
      if (retcode < 0) {
#ifndef GC_SAFE_POINTS
        x_preemption_enable;
#endif

        return retcode;

      }
      queued += retcode;
    }
  }
  
  if (thread->exception) {
    retcode = markInstance(thread->exception, fifo, flag);
    if (retcode < 0) {
#ifndef GC_SAFE_POINTS
      x_preemption_enable;
#endif

      return retcode;

    }
    queued += retcode;
  }

  for (frame = (volatile x_frame) thread->top; frame; frame = (volatile x_frame) frame->previous) {
    numFrames += 1;
    retcode = markFrameReachable(frame, fifo, flag);
    if (retcode < 0) {
#ifndef GC_SAFE_POINTS
      x_preemption_enable;
#endif

      return retcode;
    }
    queued += retcode;
  }

#ifndef GC_SAFE_POINTS
  x_preemption_enable;
#endif

  woempa(1, "(GC) Enfifoed %d objects in %d frames of thread %w.\n", queued, numFrames, NM(thread));

  woempa(1, "(GC) Marked stack of thread %w: %d frames, %d queued.\n", NM(thread), numFrames, queued);

  return queued;

}

void markLoadedClass(w_word key, w_word value, void *pointer, void*dummy) {
  w_clazz clazz = (w_clazz)value;
  w_int  *counter = pointer;
  w_int retcode;

  if (*counter >= 0) {
    woempa(1, "--> marking %K\n", clazz);
    retcode = markClazzReachable(clazz, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {
      *counter = retcode;
    }
    else {
      *counter += retcode;
    }
    woempa(1, "markLoadedClass(%K) returned %d\n", clazz, retcode);
  }
}

static w_int markClassLoaderReachable(w_instance loader, w_fifo fifo, w_word flag) {
  w_int  queued = 0;

  w_hashtable ht = loader2loaded_classes(loader);

  if (ht) {
    ht_iterate(ht, markLoadedClass, &queued, NULL);
  }

  return queued;
}

/*
** If a Reference is reachable, and it actually refers to something (the
** `referent', and the referent is `condemned', we zap the reference.
*/
w_int markReferenceReachable(w_instance referent, w_word flag) {
  w_int  queued = 0;
  w_int  retcode = 0;
  w_object ref_obj;
  w_fifo fifo;
 
  if (referent == NULL) {

    return 0;

  }

  ref_obj = instance2object(referent);
  retcode = 0;

  FLYING_PIG_CHECK(ref_obj);

if (flag == O_SOFT_BLACK) {
    fifo = softly_reachable_fifo;
  }
  else if (flag == O_WEAK_BLACK) {
    fifo = weakly_reachable_fifo;
  }
  else if (flag == O_PHANTOM_BLACK) {
    fifo = phantom_reachable_fifo;
  }
  else {
    woempa(9, "flag 0x%08x is not one of O_SOFT_BLACK, O_WEAK_BLACK, O_PHANTOM_BLACK\n", flag);

    return 0;

  }

  if (isNotMarked(ref_obj, flag)) {
    setFlag(ref_obj->flags, flag);
    retcode = tryPutFifo(referent, fifo);
  }

  if (retcode < 0) {

    return retcode;

  }
  queued += retcode;

  return queued;
}

w_int markChildren(w_object o, w_fifo fifo, w_word flag);

static void enqueueReference(w_instance instance) {

  x_mutex_lock(enqueue_fifo_mutex, x_eternal);
  if (tryPutFifo(instance, enqueue_fifo) >= 0) {
    setFlag(instance2flags(instance), O_ENQUEUEING);
    unsetFlag(instance2flags(instance), O_ENQUEUEABLE);
  }
  x_mutex_unlock(enqueue_fifo_mutex);

}

static void finalizeReference(w_instance instance) {

  x_mutex_lock(finalizer_fifo_mutex, x_eternal);
  if (tryPutFifo(instance, finalizer_fifo) >= 0) {
    setFlag(instance2flags(instance), O_FINALIZING);
    unsetFlag(instance2flags(instance), O_FINALIZABLE);
  }
  x_mutex_unlock(finalizer_fifo_mutex);

}

/*
** Mark all instances in the fifo with the given flag, if they're not
** already marked with the same flag or one which `dominates' it the
** sense explained in the definition of isMarked() above.
** If we mark an instance then we also mark its Class (if not already
** marked) and append all its children to the fifo.  (`Children' means
** elements of a reference array or reference fields of a non-array
** instance). If it is an instance of Thread, Class, ClassLoader, or
** Reference (or a subclass) then there is also some type-specific
** scanning to be done.
*/

w_int markFifo(w_fifo fifo, w_word flag) {

  w_instance parent_instance;
  w_object   parent_object;
  w_clazz    clazz;
  w_instance child_instance;
  w_int      i;
  w_int      retcode;
  w_int      queued = 0;

  woempa(1, "Marking FIFO %p, flag is 0x%02x, contains %d elements\n", fifo, flag, fifo->numElements);

  while ((parent_instance = getFifo(fifo))) {
    parent_object = instance2object(parent_instance);
    woempa(1, "(GC) Tracing instance %j\n", parent_instance);

    /*
    ** If this object is already marked at this or a stronger level, skip it.
    ** Otherwise, mark it now and carry on.

    if (isMarked(parent_object, flag)) {
      continue;
    }
    */

    FLYING_PIG_CHECK(parent_object);

    setFlag(parent_object->flags, flag);

    /*
    ** Mark the Class of this instance.
    */
    child_instance = parent_object->clazz->Class;
    if (child_instance) {
      retcode = markInstance(child_instance, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }

    /*
    ** If this is an instance of java.lang.Class, we have extra work to do.
    */
    if (parent_object->clazz == clazzClass && (clazz = getWotsitField(parent_instance, F_Class_wotsit))) {
      woempa(1,"(GC) Object %p is instance of %k\n",parent_object,parent_object->clazz);
      retcode = markClazzReachable(clazz, strongly_reachable_fifo, O_BLACK);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }
    /*
    ** If this is a thread, we scan the stack.
    */
    else if (isSet(parent_object->clazz->flags, CLAZZ_IS_THREAD) && getWotsitField(parent_instance, F_Thread_wotsit)) {
      woempa(1, "(GC) %p is instance of %k, a subclass of Thread.\n", parent_instance, parent_object->clazz);
      retcode = markThreadReachable(parent_object, fifo, flag);
      if (retcode < 0) {
        return retcode;
      }
      queued += retcode;
    }
    /*
    ** For a classloader, we scan all the classes it has loaded.
    */
    else if (isSet(parent_object->clazz->flags, CLAZZ_IS_CLASSLOADER)) {
      woempa(1, "(GC) %p is instance of %k, a subclass of ClassLoader.\n", parent_instance, parent_object->clazz);
      retcode = markClassLoaderReachable(parent_instance, fifo, flag);
      if (retcode < 0) {
        return retcode;
      }
      queued += retcode;
    }
    /*
    ** For a reference type, we scan the referent using either the flag which
    ** brought us here or the flag corresponding to the reference type, 
    ** whichever is weaker.
    */
    else if (isSet(parent_object->clazz->flags, CLAZZ_IS_REFERENCE) && getWotsitField(parent_instance, F_Reference_referent)) {
      w_object ref_obj = instance2object((w_instance)getWotsitField(parent_instance, F_Reference_referent));
      woempa(1, "(GC) Object %p is instance of %k, a subclass of Reference\n", parent_object, parent_object->clazz);
      woempa(1, "(GC) It refers to instance %p of %k\n", ref_obj, ref_obj->clazz);
      if (isSuperClass(clazzSoftReference,parent_object->clazz)) {
        woempa(1, "(GC) Marking soft reference to %j\n", getWotsitField(parent_instance, F_Reference_referent));
        if (isSet(ref_obj->flags, O_CONDEMNED)) {
          setWotsitField(parent_instance, F_Reference_referent, NULL);
          if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
            enqueueReference(parent_instance);      
          }
        }
        else {
          retcode = markReferenceReachable(ref_obj->fields, impliesMark(flag, O_SOFT_BLACK) ? O_SOFT_BLACK : flag);
          if (retcode < 0) {
            return retcode;
          }
          queued += retcode;
        }
      }
      else if (isSuperClass(clazzWeakReference,parent_object->clazz)) {
        woempa(1, "(GC) Marking weak reference from %j to %j\n", parent_instance, getWotsitField(parent_instance, F_Reference_referent));
        if (isSet(ref_obj->flags, O_CONDEMNED)) {
          woempa(1, "%j is only weakly reachable, clearing and enqueueing %j\n", ref_obj->fields, parent_instance);
          setWotsitField(parent_instance, F_Reference_referent, NULL);
          if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
            enqueueReference(parent_instance);      
          }
        }
        else {
          retcode = markReferenceReachable(ref_obj->fields, impliesMark(flag, O_WEAK_BLACK) ? O_WEAK_BLACK : flag);
          if (retcode < 0) {
            return retcode;
          }
          queued += retcode;
        }
      }
      else if (isSuperClass(clazzPhantomReference,parent_object->clazz)) {
        woempa(1, "(GC) Marking phantom reference to %j\n", getWotsitField(parent_instance, F_Reference_referent));
        if (isSet(ref_obj->flags, O_CONDEMNED)) {
          if (isSet(parent_object->flags, O_ENQUEUEABLE)) {
            enqueueReference(parent_instance);      
          }
        }
        retcode = markReferenceReachable(ref_obj->fields, impliesMark(flag, O_PHANTOM_BLACK) ? O_PHANTOM_BLACK : flag);
        if (retcode < 0) {
          return retcode;
        }
        queued += retcode;
      }
      else {
        woempa(9, "Eh??? %k is not a subclass of SoftReference, WeakReference, or PhantomReference!\n", parent_object->clazz);
      }
    }

    /*
    ** Now call all the children...
    */
    
    if (parent_object->clazz->dims) {
      /*
      ** It's an array clazz; if the array clazz stores non primitives 
      ** (other arrays or references), put them all in the FIFO.
      */
      if (parent_object->clazz->dims > 1 || isNotSet(parent_object->clazz->previousDimension->flags, CLAZZ_IS_PRIMITIVE)) {
        woempa(1,"(GC) Tracing references from %k@%p\n",parent_object->clazz,parent_instance);
        for (i = 0; i < instance2Array_length(parent_instance); i++) {
          if ((child_instance = (w_instance)parent_instance[F_Array_data + i])) {
            retcode = markInstance(child_instance, fifo, flag);
            if (retcode < 0) {

              return retcode;

            }
            queued += retcode;
          } 
        } 
      }
    }
    else {
      /*
      ** It's a normal clazz, just go over all the fields that could contain 
      ** references to other instances.
      */
      retcode = markChildren(parent_object, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;


    }
  }

  woempa(1,"Finished marking FIFO %p\n", fifo);

  return queued;

}

/*
** Push an instance onto a fifo and mark the fifo.
*/

w_int markReachable(w_instance ref, w_fifo fifo, w_word flag) {

  w_int    retcode;
  
  retcode = markInstance(ref, fifo, flag);

  if (retcode <= 0) {

    return retcode;

  }

  retcode = markFifo(fifo, flag);
  
  return retcode < 0 ? retcode : retcode + 1;

}

#define GC_MAX_PRIORITY 10

/*
** Function to set the priority of the GC thread to the maximum allowed.
*/

static inline void maximizePriority(void) {

  woempa(1, "(GC) New priority = %d\n", GC_MAX_PRIORITY);
  gc_thread->jpriority = GC_MAX_PRIORITY;
  gc_thread->kpriority = priority_j2k(GC_MAX_PRIORITY, 0);
  x_thread_priority_set(gc_thread->kthread, gc_thread->kpriority);

}

/*
** Function kickGC() sets the priority of the GC thread to maximum and
** interrupts the thread.  Use this to `forcibly persuade' GC to wake
** up and do something.

void kickGC() {

  if (gc_thread) {
    woempa(9,"(GC) Ouch! OK, I'll try to speed things up a bit ...\n");
    *gc_kicks_pointer += 3;
    woempa(9, "(GC) kicks = %d\n", *gc_kicks_pointer);
    maximizePriority();
    setFlag(gc_thread->flags, WT_THREAD_INTERRUPTED);
    x_thread_stop_waiting(gc_thread->kthread);
  }
  else {
    woempa(9, "(GC) kickGC called before GC thread was started?\n");
  }

}
*/

#ifndef GC_SAFE_POINTS
/*
** The number of objects added to the gray_fifo since the last time
** we started to scan the fifo.  Marked volatile because it is always
** incremented by a mutator thread, and we want the GC thread to notice
** that it has changed.
*/

volatile w_int grayed;

/*
** Function _newReferenceTo should be called whenever a new instance is 
** created or a new reference is created to an existing instance.
** It adds the instance to the gray_fifo and increments `grayed'.
*/

void _newReferenceTo(w_instance instance) {

  w_object object = instance2object(instance);
  w_flags *flagsptr = &object->flags;

  if (gray_fifo && gc_phase <= GC_PHASE_MARK && isSet(*flagsptr, O_WINDOW) && isNotSet(*flagsptr, O_GRAY | O_BLACK | O_NEAR_BLACK | O_GREEN)) {
    woempa(1, "Pushing %j onto gray_fifo\n", instance);
    ++grayed;
    setFlag(*flagsptr, O_GRAY);
    x_monitor_eternal(gray_fifo_monitor);
    // Retcode not checked => living dangerously!
    putFifo(instance, gray_fifo);
    x_monitor_exit(gray_fifo_monitor);
  }

}
#endif

w_int markChildren(w_object o, w_fifo fifo, w_word flag) {

  w_instance child_instance;
  w_int      queued;
  w_clazz    clazz;
  w_int      i;
  w_int      retcode;

  queued = 0;
  clazz = o->clazz;
  woempa(1, "Tracing instance fields of %k@%p\n", clazz, o->fields);
  for (i = clazz->instanceSize - clazz->numReferenceFields; i < clazz->instanceSize; ++i) {
    if ((child_instance = (w_instance)o->fields[i])) {
      retcode = markInstance(child_instance, fifo, flag);
      if (retcode < 0) {

        return retcode;

      }
      queued += retcode;
    }
  }

  return queued;

}

w_int gc_phase = GC_PHASE_UNREADY;

/*
** The thread (if any) which is currently running a PREPARE/MARK sequence
** on the heap.
*/
static w_thread marking_thread;

/*
 * In the Preparation phase, we take a snapshot of the instance heap
 *  and build two fifo's: the `window' fifo, which contains all mortal
 *  items in the snapshot, and the `gray-green' fifo, which initially
 *  contains all the immortal (GREEN) items.
 */

#ifdef DEBUG
typedef struct w_Chs {
  w_int count_1;
  w_int count_2;
  w_int count_3;
} w_Chs;

typedef struct w_Chs * w_chs;
#endif

x_boolean preparation_iteration_1(void * mem, void * arg) {
  w_int retcode;
#ifdef DEBUG
  w_chs chs = (w_chs) arg;
#endif
  w_object object = chunk2object(mem);

#ifdef DEBUG
  chs->count_1 += 1;
#endif

  if (object->clazz) {
    unsetFlag(object->flags, O_BLACK | O_NEAR_BLACK);
    if (isSet(object->flags, (O_FINALIZING | O_FINALIZABLE | O_ENQUEUEABLE | O_ENQUEUEING))) {
      w_size i;
      w_instance child_instance;
      w_object child_object;
      for (i = object->clazz->instanceSize - object->clazz->numReferenceFields; i < object->clazz->instanceSize; ++i) {
        child_instance = (w_instance)object->fields[i];
        if (child_instance) {
          child_object = instance2object(child_instance);
          FLYING_PIG_CHECK(child_object);

          setFlag(child_object->flags, O_FINALIZE_BLACK);
          retcode = tryPutFifo(child_instance, finalize_reachable_fifo);
          if (retcode < 0) {
            wabort(ABORT_WONKA, "Couldn't add to finalize_reachable_fifo, PANIC\n");
          }
        }
      }
    }
#ifdef GC_SAFE_POINTS
    woempa(1, "Pushing %j onto window_fifo, setting flag O_WINDOW\n", object->fields);
    setFlag(object->flags, O_WINDOW);
    retcode = tryPutFifo(object->fields, window_fifo);
    if (retcode < 0) {
      wabort(ABORT_WONKA, "Couldn't add to window_fifo, PANIC\n");
    }
#else
    if (isSet(object->flags, O_GREEN)) {
// [CG 20040510]      unsetFlag(object->flags, O_GREEN);
#ifdef DEBUG
      chs->count_2 += 1;
      woempa(1, "Pushing %j onto strongly_reachable_fifo, setting flag O_BLACK\n", object->fields);
#endif
      setFlag(object->flags, O_BLACK);
      retcode = tryPutFifo(object->fields, strongly_reachable_fifo);
      if (retcode < 0) {
        wabort(ABORT_WONKA, "Couldn't add to strongly_reachable_fifo, PANIC\n");
      }
    }
    else {
#ifdef DEBUG
      chs->count_3 += 1;
      woempa(1, "Pushing %j onto window_fifo, setting flag O_WINDOW\n", object->fields);
#endif
      setFlag(object->flags, O_WINDOW);
      retcode = tryPutFifo(object->fields, window_fifo);
      if (retcode < 0) {
        wabort(ABORT_WONKA, "Couldn't add to window_fifo, PANIC\n");
      }
    }
#endif
  }
  else {
    woempa(9, "((w_object)%p)->clazz == NULL\n", object);
  }
  
  return TRUE;

}

w_fifo thread_fifo1;
w_fifo thread_fifo2;

#ifdef GC_SAFE_POINTS
static void prepreparation(void) {
  w_thread other_thread;

  thread_fifo1 = ht_list_values(thread_hashtable);
  if (!thread_fifo1) {
    woempa(9, "Couldn't create thread_fifo1!\n");

    return;

  }
  thread_fifo2 = allocFifo(W_Realm_system, 511);
  if (!thread_fifo2) {
    woempa(9, "Couldn't create thread_fifo2!\n");

    return;

  }
  while ((other_thread = getFifo(thread_fifo1))) {
    if (other_thread != currentWonkaThread) {
      putFifo(other_thread, thread_fifo2);
      woempa(7, "%t: locking %t\n", currentWonkaThread, other_thread);
      setFlag(other_thread->flags,  WT_THREAD_SAFE_POINT_REQUEST);
      lockGCStatus(other_thread, WT_THREAD_BLOCKED_BY_GC);
      woempa(7, "%t: locked %t\n", currentWonkaThread, other_thread);
    }
  }
  releaseFifo(thread_fifo1);
}

static void postmark(void) {
  w_thread other_thread;

  if (!thread_fifo2) {

    return;

  }
  while ((other_thread = getFifo(thread_fifo2))) {
    unsetFlag(other_thread->flags,  WT_THREAD_SAFE_POINT_REQUEST);
    unlockGCStatus(other_thread, WT_THREAD_BLOCKED_BY_GC);
    woempa(7, "%t: unlocked %t\n", currentWonkaThread, other_thread);
  }
  releaseFifo(thread_fifo2);
}
#endif

w_int preparationPhase(void) {
#ifdef DEBUG
  w_int marked1;
  w_int marked2;
  w_Chs Counts;
#endif
  w_int retcode;
  w_int memory_busy = memory_total - x_mem_avail();
//  w_instance instance;

#ifndef GC_SAFE_POINTS
  while (getFifo(gray_fifo));
#endif
  while (getFifo(window_fifo));
  while (getFifo(strongly_reachable_fifo));
  while (getFifo(softly_reachable_fifo));
  while (getFifo(weakly_reachable_fifo));
  while (getFifo(phantom_reachable_fifo));
  while (getFifo(finalize_reachable_fifo));

#ifdef CONTRACT_FIFOS
  retcode = contractFifo(window_fifo);
  woempa(1, "Contracted window_fifo, freed %d bytes\n", retcode);
  retcode = contractFifo(strongly_reachable_fifo);
  woempa(1, "Contracted strongly_reachable_fifo, freed %d bytes\n", retcode);
  retcode = contractFifo(softly_reachable_fifo);
  woempa(1, "Contracted softly_reachable_fifo, freed %d bytes\n", retcode);
  retcode = contractFifo(weakly_reachable_fifo);
  woempa(1, "Contracted reachable_reachable_fifo, freed %d bytes\n", retcode);
  retcode = contractFifo(phantom_reachable_fifo);
  woempa(1, "Contracted phantom_reachable_fifo, freed %d bytes\n", retcode);
  retcode = contractFifo(finalize_reachable_fifo);
  woempa(1, "Contracted finalize_reachable_fifo, freed %d bytes\n", retcode);
#endif

#ifdef REDUCE_FIFOS
  retcode = reduceFifo(window_fifo);
  woempa(1, "Reduced window_fifo, freed %d bytes\n", retcode);
  retcode = reduceFifo(strongly_reachable_fifo);
  woempa(1, "Reduced strongly_reachable_fifo, freed %d bytes\n", retcode);
  retcode = reduceFifo(softly_reachable_fifo);
  woempa(1, "Reduced softly_reachable_fifo, freed %d bytes\n", retcode);
  retcode = reduceFifo(weakly_reachable_fifo);
  woempa(1, "Reduced reachable_reachable_fifo, freed %d bytes\n", retcode);
  retcode = reduceFifo(phantom_reachable_fifo);
  woempa(1, "Reduced phantom_reachable_fifo, freed %d bytes\n", retcode);
  retcode = reduceFifo(finalize_reachable_fifo);
  woempa(1, "Reduced finalize_reachable_fifo, freed %d bytes\n", retcode);
#endif

  memory_load_factor = memory_busy / (memory_total - memory_busy);
  if (memory_load_factor < 0) {
    memory_load_factor = 0;
  }

  x_mem_lock(x_eternal);
#ifdef DEBUG
  Counts.count_1 = 0;
  Counts.count_2 = 0;
  Counts.count_3 = 0;
  x_mem_scan(x_eternal, OBJECT_TAG, preparation_iteration_1, &Counts);
  marked1 = Counts.count_1;
  woempa(9, "COUNTS found %d instances\n", marked1);
  Counts.count_1 = 0;
#else
  x_mem_scan(x_eternal, OBJECT_TAG, preparation_iteration_1, NULL);
#endif

#ifdef DEBUG
  woempa(1, "Snapshot taken: %d instances, %d gray/greens, %d candidates.\n", marked2, Counts.count_2, Counts.count_3);
#endif

  x_mem_unlock();

#ifdef DEBUG
  woempa(1 , "(GC) Found %d objects: %d GRAY/GREENs, %d candidates for collection.\n", marked1, Counts.count_2, Counts.count_3);

  return marked1;
#else
  return 0;
#endif
}

#ifndef GC_SAFE_POINTS
w_int markGrayGreens(void);
#endif

/*
 * In the Mark phase, we first mark all the (transient) `roots':
 *  - system thread group (and hence all threads)
 *  - system class hashtable (static fields of system classes)
 *  - global references hashtable (includes current local references)
 * For every item we mark, we append all references it contains to
 * the appropriate fifo (strong, soft, weak, or phantom).  Therefore
 * by the end of this phase we have marked all items in the window
 * which were reachable at the time the snapshot was taken.
 *
 * During this time new objects and new references to existing objects
 * can be created: all of these are appended to the gray-green fifo.
 * We now scan the gray-green fifo until it is exhausted, treating the
 * items found in it as strongly reachable (which they may or not be -
 * we are playing safe).  Even during this scan new references can be 
 * added to the gray-green fifo, but when it becomes empty we can be sure 
 * that we have achieved closure.  We now make one final pass over the
 * strong, soft, weak and phantom fifo's (in that order) before proceeding
 * to the Sweep phase.
 *
 * Returns the number of objects marked if successful (useless, and
 * probably inaccurate, but at least nonnegative), or a negative error
 * code.  In the latter case the mark phase did not complete, and it
 * would be folly to sweep.
 */
w_int markPhase(void) {

  w_fifo     temp_fifo;
  w_instance instance;
  w_int      retcode;
  w_int      marked = 0;

  retcode = markFifo(finalize_reachable_fifo, O_FINALIZE_BLACK);
  woempa(1, "(GC) Marked %d objects FINALIZE_BLACK\n", retcode);
  if (retcode < 0) {
    woempa(7, "markFifo(finalize_reachable_fifo, O_FINALIZE_BLACK) returned %d, quitting markPhase\n", retcode);

    return -1;

  }

  retcode = markStaticStrings();
  woempa(1, "(GC) Marked %d static strings\n", retcode);
  if (retcode < 0) {
    woempa(7, "markStaticStrings() returned %d, quitting markPhase\n", retcode);

    return -1;

  }

  woempa(1, "(GC) Marking system ThreadGroup.\n");
  retcode = markReachable(W_ThreadGroup_system->ThreadGroup, strongly_reachable_fifo, O_BLACK);
  if (retcode < 0) {
    woempa(7, "markReachable(W_ThreadGroup_system->ThreadGroup, strongly_reachable_fifo, O_BLACK) returned %d, quitting markPhase\n", retcode);

    return -1;

  }

  marked += retcode;
  woempa(1, "(GC) Marking system class hashtable.\n");
  retcode = markClassLoaderReachable(NULL, strongly_reachable_fifo, O_BLACK);
  if (retcode < 0) {
    woempa(7, "markClassLoaderReachable(NULL, strongly_reachable_fifo, O_BLACK) returned %d, quitting markPhase\n", retcode);

    return retcode;

  }
  marked += retcode;

  woempa(1, "(GC) Marking globals/locals hashtable.\n");
  temp_fifo = ht_list_keys(globals_hashtable);
  if (!temp_fifo) {
    woempa(7, "ht_list_keys(globals_hashtable) returned %d, quitting markPhase\n", retcode);

    return -1;

  }
  while ((instance = getFifo(temp_fifo))) {
    retcode = markInstance(instance, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {

      return retcode;

    }
  }
  releaseFifo(temp_fifo);

  do {
    retcode = markFifo(strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {
      woempa(7, "markFifo(strongly_reachable_fifo, O_BLACK) returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;

#ifndef GC_SAFE_POINTS
    woempa(1, "(GC) Marking Grays and Greens.\n");
    retcode = markGrayGreens();

    if (retcode < 0) {
    woempa(7, "markGrayGreens() returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;
#endif

    woempa(1, "(GC) Marking strongly_reachable_fifo.\n");
    retcode = markFifo(strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {
      woempa(7, "markFifo(strongly_reachable_fifo, O_BLACK) returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;

    woempa(1, "(GC) Marking softly_reachable_fifo.\n");
    retcode = markFifo(softly_reachable_fifo, O_SOFT_BLACK);
    if (retcode < 0) {
      woempa(7, "markFifo(softly_reachable_fifo, O_SOFT_BLACK) returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;

    woempa(1, "(GC) Marking weakly_reachable_fifo.\n");
    retcode = markFifo(weakly_reachable_fifo, O_WEAK_BLACK);
    if (retcode < 0) {
      woempa(7, "markFifo(weakly_reachable_fifo, O_WEAK_BLACK) returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;

    woempa(1, "(GC) Marking phantom_reachable_fifo.\n");
    retcode = markFifo(phantom_reachable_fifo, O_PHANTOM_BLACK);
    if (retcode < 0) {
      woempa(7, "markFifo(phantom_reachable_fifo, O_PHANTOM_BLACK) returned %d, quitting markPhase\n", retcode);

      return retcode;

    }
    marked += retcode;

    woempa(1, "(GC) Re-Marking system ThreadGroup.\n");
    retcode = markReachable(W_ThreadGroup_system->ThreadGroup, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {
      woempa(7, "markReachable(W_ThreadGroup_system->ThreadGroup, strongly_reachable_fifo, O_BLACK) returned %d, quitting markPhase\n", retcode);

      return -1;

    }

    marked += retcode;

    woempa(1, "(GC) Re-Marking globals/locals hashtable.\n");
    temp_fifo = ht_list_keys(globals_hashtable);
    if (!temp_fifo) {
      woempa(7, "ht_list_keys(globals_hashtable) returned %d, quitting markPhase\n", retcode);

      return -1;

    }
    while ((instance = getFifo(temp_fifo))) {
      retcode = markInstance(instance, strongly_reachable_fifo, O_BLACK);
      if (retcode < 0) {

        return retcode;

      }
    }
    releaseFifo(temp_fifo);

  } while (
#ifndef GC_SAFE_POINTS
           gray_fifo->numElements ||
#endif
           strongly_reachable_fifo->numElements ||
           softly_reachable_fifo->numElements ||
           weakly_reachable_fifo->numElements ||
           phantom_reachable_fifo->numElements);


  return marked;

}

static w_boolean checkExternalRefs(w_object object) {
  return WONKA_TRUE;
}

static w_boolean checkStrongRefs(w_object object) {

#ifdef GC_SAFE_POINTS
  if (isSet(object->flags, O_BLACK)) {
    woempa(1, "Not collecting %k@%p because it is strongly reachable (BLACK)\n", object->clazz, object->fields);
#else
  if (isSet(object->flags, O_BLACK | O_GRAY)) {
    woempa(1, "Not collecting %k@%p because it is strongly reachable (%s %s)\n", object->clazz, object->fields, isSet(object->flags, O_BLACK)?"BLACK":"", isSet(object->flags, O_GRAY)?"GRAY":"");
#endif

    return WONKA_FALSE;

  }

  return WONKA_TRUE;

}

static w_boolean checkSoftRefs(w_object object) {

  if (isSet(object->flags, O_SOFT_BLACK)) {
    woempa(1, "Not collecting %k@%p because it is softly reachable\n", object->clazz, object->fields);
    if ((memory_load_factor > 0) || (*gc_kicks_pointer > 0)) {
      woempa(1, "Condemning %k@%p because it is only softly reachable, memory load factor is %d, gc_kicks is %d\n", object->clazz, object->fields, memory_load_factor, *gc_kicks_pointer);
      FLYING_PIG_CHECK(object);

      setFlag(object->flags, O_CONDEMNED);
    }

    return WONKA_FALSE;
  }

  return WONKA_TRUE;

}

static w_boolean checkWeakRefs(w_object object) {

  if (isSet(object->flags, O_WEAK_BLACK)) {
    woempa(1, "Not collecting %k@%p because it is weakly reachable\n", object->clazz, object->fields);
    woempa(7, "Condemning %k@%p because it is only weakly reachable\n", object->clazz, object->fields);
     FLYING_PIG_CHECK(object);

    setFlag(object->flags, O_CONDEMNED);

    return WONKA_FALSE;
  }

  return WONKA_TRUE;

}

static w_boolean checkFinalization(w_object object) {

  if (isSet(object->flags, O_ENQUEUEABLE)) {
    if(getWotsitField(object->fields, F_Reference_referent)) {
    woempa(1, "Not collecting %j because it may later be enqueued\n", object->fields);

      return WONKA_FALSE;

    }
  }

  if (isSet(object->flags, (O_FINALIZING | O_ENQUEUEING))) {
    woempa(1, "Not collecting %j because it is currently being enqueued or finalized\n", object->fields);

    return WONKA_FALSE;

  }

  if (isNotSet(object->flags, O_FINALIZABLE)) {

    return WONKA_TRUE;

  }

  woempa(1, "Not collecting %j because it is finalizable.\n", object->fields);
  woempa(1, "Putting %j onto finalization queue.\n", object->fields);
  finalizeReference(object->fields);

  return WONKA_FALSE;

}

static w_boolean checkPhantomRefs(w_object object) {

  if (isSet(object->flags, O_PHANTOM_BLACK)) {
    woempa(1, "Not collecting %k@%p)because it is reachable via phantom reference(s)\n", object->clazz, object->fields);
    FLYING_PIG_CHECK(object);
    setFlag(object->flags, O_CONDEMNED);

    return WONKA_FALSE;

  }
  if (isSet(object->flags, O_FINALIZE_BLACK)) {
    woempa(1, "Not collecting %k@%p)because it is reachable via yet-to-be-finalized object(s)\n", object->clazz, object->fields);

    return WONKA_FALSE;

  }

  return WONKA_TRUE;

}

/*
** Sweep the window_fifo until either at least `target' bytes are freed
** or the fifo is exhausted.  Caller must own gc_monitor.
*/

w_size sweep(w_int target) {

  w_instance instance;
  w_object   object;
  w_int      object_size;
  w_int      objects_examined = 0;
  w_int      objects_freed = 0;
  w_int      bytes_freed = 0;
  w_int      do_collect;

  if (gc_phase < GC_PHASE_SWEEP) {
    woempa(9, "Not in SWEEP phase.\n");

    return 0;

  }

  while(window_fifo) {
    instance = getFifo(window_fifo);
    if (!instance) {
    woempa(1, "Exhausted window_fifo after collecting %d bytes in %d objects.\n", bytes_freed, objects_freed);

      break;

    }
    object = instance2object(instance);
    if (object->clazz) {
      woempa(1, "Examining %k@%p\n", object->clazz, instance);
    }
    else {
      woempa(9, "((w_object)%p)->clazz == NULL\n", object);
      continue;
    }
    ++objects_examined;
#ifdef GC_SAFE_POINTS
    unsetFlag(object->flags, O_WINDOW);
#else
    unsetFlag(object->flags, O_WINDOW | O_GRAY);
#endif
    if ( checkExternalRefs(object)
      && checkStrongRefs(object)
      && checkSoftRefs(object)
      && checkWeakRefs(object)
      && checkFinalization(object)
      && checkPhantomRefs(object)
       ) {

      if (isSet(object->flags, O_GARBAGE)) {
        do_collect = WONKA_TRUE;
        if (isSet(object->clazz->flags, CLAZZ_IS_THREAD)) {
          w_thread thread = getWotsitField(object->fields, F_Thread_wotsit);

          if (thread && thread->state != wt_dead && thread->state != wt_unstarted) {
            woempa(9, "Hold on a moment - thread '%w' is still running...\n", NM(thread));
            do_collect = WONKA_FALSE;
          }
        }

        if (do_collect) {
          if (isSet(object->flags, O_HAS_LOCK)) {
            releaseMonitor(object->fields);
          }
          object_size = releaseInstance(object);
          bytes_freed += object_size;
          instance_returned += 1;
          instance_use -= 1;
        }
        objects_freed += do_collect;

        if (bytes_freed >= target) {
          woempa(1, "Target was %d, freed %d, job done.\n", target, bytes_freed);

          break;

        }
      }
      else {
        setFlag(object->flags, O_GARBAGE);
        if (object->clazz == clazzString) {
          w_string s;
          x_mutex_lock(string_mutex, x_eternal);
          s = getWotsitField(object->fields, F_String_wotsit);
          if (s && (s->instance == object->fields)) {
            s->instance = NULL;
          }
          x_mutex_unlock(string_mutex);
        }
      }
    }
  }

  woempa(7, "(GC) Collected %i bytes from %d objects out of %d.\n", bytes_freed, objects_freed, objects_examined);

#ifdef USE_DISCARD_COLLECT
  x_mem_collect(&bytes_freed, &objects_freed);
  woempa(1, "(GC) Oswald sez: Collected %i bytes from %d objects.\n", bytes_freed, objects_freed);
#endif

  return bytes_freed;

}

void sweepPhase(void) {

  w_size collected = 0;
  x_sleep winks;
  x_status status; 

  if (bytes_per_meal) {
    while(1) {
      winks = x_millis2ticks(millis_per_nap / (memory_load_factor + 1));
      if (collected && *gc_kicks_pointer == 0) {
        woempa(1, "(GC) Collected %d bytes so far, taking %d winks.\n", collected, winks);
        status = x_monitor_wait(gc_monitor, winks);
        if (status == xs_interrupted) {
          x_monitor_eternal(gc_monitor);
        }
      }
      collected = sweep(bytes_per_meal);
      if (collected < bytes_per_meal) {
        break;
      }
    } 
  }
  else {
    sweep(2000000000);
  }

}

#ifndef GC_SAFE_POINTS
w_int markGrayGreens() {

  w_instance instance;
  w_int      marked;
  w_int      retcode;

  marked = 0;
  x_monitor_eternal(gray_fifo_monitor);
  while ((instance = getFifo(gray_fifo))) {
//    x_monitor_exit(gray_fifo_monitor);
    retcode = markInstance(instance, strongly_reachable_fifo, O_BLACK);
    if (retcode < 0) {

      return retcode;

    }
    marked += retcode;
    grayed -= retcode;
//    x_monitor_eternal(gray_fifo_monitor);
  }
  x_monitor_exit(gray_fifo_monitor);

  woempa(1, "Marked %d GREENs\n", marked);

  return marked;

}
#endif

/*
** List of w_reclaim_listener's, and a monitor to protect access thereto.
*/

w_wordset reclaim_listener_list = NULL;
x_Monitor reclaim_listener_Monitor;
x_monitor reclaim_listener_monitor = NULL;

/*
** We only start making callbacks after accumulating RECLAIM_THRESHOLD
** bytes' worth of requests.  The requests are accumulated in reclaim_accumulator.
*/

#define MIN_RECLAIM_THRESHOLD (256 * 1024)
#define MAX_RECLAIM_THRESHOLD (8 * 1024 * 1024)
static w_size reclaim_threshold = MIN_RECLAIM_THRESHOLD;
w_size  reclaim_accumulator = 0;

void registerReclaimCallback(w_reclaim_callback callback) {

  x_status status;

  if (!reclaim_listener_monitor) {
    woempa(1, "Creating reclaim_listener_monitor\n");
    reclaim_listener_monitor = &reclaim_listener_Monitor;
    x_monitor_create(reclaim_listener_monitor);
  }

  status = x_monitor_eternal(reclaim_listener_monitor);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
  addToWordset(&reclaim_listener_list, (w_word)callback);
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);

}

void deregisterReclaimCallback(w_reclaim_callback callback) {

  x_status status;

  if (!reclaim_listener_monitor) {
    wabort(ABORT_WONKA, "No reclaim_listener_monitor!\n");
  }

  status = x_monitor_eternal(reclaim_listener_monitor);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
  removeFromWordset(&reclaim_listener_list, (w_word)callback);
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);

}

/*
** Function gc_reclaim() is called when we would like to reclaim `requested'
** bytes of memory.  Note: after calling the callbacks we reset the accumulator
** to zero even if the amount reclaimed fell short of that requested.  We
** assume that the inability (or unwillingness) to release memory is likely
** to last a little while, and setting reclaim_accumulator = remaining would
** tend to trigger another reclaim cycle rather soon.  For further study.
*/

w_size gc_reclaim(w_int requested, w_instance caller) {

#ifdef DISTRIBUTED_GC
  w_size   i;
  w_size   n;
  w_int   remaining = 0;
  if (!reclaim_listener_monitor) {
    woempa(1, "Call to gc_reclaim() before any listener added\n");

    return 0;

  }

/*
  status = x_monitor_eternal(reclaim_listener_monitor);
  woempa(1, "Entered reclaim_listener_monitor, status = %d\n", status);
*/
  reclaim_accumulator += requested * (memory_load_factor + 1);
  if (reclaim_accumulator > reclaim_threshold) {
    remaining = reclaim_accumulator;
    n = (reclaim_listener_list->occupancy) / sizeof(w_word);
    for (i = 0; remaining > 0 && i < n; ++i) {
      if(xs_success == x_monitor_enter(gc_monitor, 5)){
        remaining -= ((w_reclaim_callback)((w_word*)reclaim_listener_list->contents)[i])(remaining, caller);
        x_monitor_exit(gc_monitor);
      }
      else {
        woempa(7, "Could not acquire gc_monitor for internal_reclaim_callback.\n");
      }
    }
    reclaim_accumulator = 0;
    if (remaining > 0) {
      woempa(9, "Still need  %d bytes\n", remaining);
    }
    if (MAX_RECLAIM_THRESHOLD / (memory_load_factor + 1) > MIN_RECLAIM_THRESHOLD) {
      reclaim_threshold = MAX_RECLAIM_THRESHOLD / (memory_load_factor + 1);
    }
  }
/*
  status = x_monitor_exit(reclaim_listener_monitor);
  woempa(1, "Left reclaim_listener_monitor, status = %d\n", status);
*/

  return requested - remaining;
#else
  return 0;
#endif

}

/*
** Function internal_reclaim_callback calls `sweep' to try to reclaim memory.
*/

w_size internal_reclaim_callback(w_int requested, w_instance instance) {

  w_size reclaimed = 0;
  x_status status;

  status = x_monitor_enter(gc_monitor, 0);
  if (status == xs_success) {
    reclaimed += gc_request(requested);
    woempa(1, "Called with requested = %d, instance = %p : reclaimed %d\n", requested, instance, reclaimed);
    x_monitor_exit(gc_monitor);
  }
  else {
    woempa(7, "Could not acquire gc_monitor for internal_reclaim_callback.\n");
  }

  return reclaimed;

}

/*
** Function gc_create() is called by the constructor of GarbageCollector
** (which is a singleton, so this code is only executed once).
*/

void gc_create(JNIEnv *env, w_instance theGarbageCollector) {

//  string_enqueue = cstring2String(W_Thread_system, "enqueue", 7);
  gc_instance = theGarbageCollector;
  gc_thread = JNIEnv2w_thread(env);
  gc_monitor = allocMonitor(theGarbageCollector);
  gc_kicks_pointer = (w_int *)wordFieldPointer(theGarbageCollector, F_GarbageCollector_kicks);
  instance_returned = 0;
  marking_thread = NULL;
  memory_total = x_mem_total() - min_heap_free;
  memory_load_factor = 1;
#ifndef GC_SAFE_POINTS
  gray_fifo_monitor = &gray_fifo_Monitor;
#endif
  finalizer_fifo_mutex = &finalizer_fifo_Mutex;
  enqueue_fifo_mutex = &enqueue_fifo_Mutex;
#ifndef GC_SAFE_POINTS
  x_monitor_create(gray_fifo_monitor);
#endif
  x_mutex_create(finalizer_fifo_mutex);
  x_mutex_create(enqueue_fifo_mutex);
  window_fifo = allocFifo(W_Realm_system, WINDOW_FIFO_LEAF_SIZE);
  strongly_reachable_fifo = allocFifo(W_Realm_system, STRONG_FIFO_LEAF_SIZE);
  softly_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  weakly_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  phantom_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  finalize_reachable_fifo = allocFifo(W_Realm_system, OTHER_FIFO_LEAF_SIZE);
  enqueue_fifo = allocFifo(W_Realm_system, ENQUEUE_FIFO_LEAF_SIZE);
  finalizer_fifo = allocFifo(W_Realm_system, FINALIZER_FIFO_LEAF_SIZE);
#ifndef GC_SAFE_POINTS
  gray_fifo = allocFifo(W_Realm_system, GRAY_FIFO_LEAF_SIZE);
#endif
  classMustBeInitialized(clazzReference);
  classMustBeInitialized(clazzFinalizer);
  woempa(7,"         window_fifo at %p\n", window_fifo);
  woempa(7,"      finalizer_fifo at %p\n", finalizer_fifo);
  woempa(7,"        enqueue_fifo at %p\n", enqueue_fifo);
#ifndef GC_SAFE_POINTS
  woempa(7,"           gray_fifo at %p\n", gray_fifo);
#endif
  woempa(7, "W_ThreadGroup_system = %p.\n", W_ThreadGroup_system);
  woempa(7, "     W_Thread_system = %p.\n", W_Thread_system);
  woempa(7, "      W_Realm_system = %p.\n", W_Realm_system);
  woempa(7, "           gc_thread = %p.\n", gc_thread);
  woempa(7, "   Current instances = %d.\n", instance_use);

  woempa(7, "Registering internal_reclaim_callback\n");
  registerReclaimCallback(internal_reclaim_callback);

}


w_int gc_collect(w_instance theGarbageCollector) {

  w_int gc_pass_count = getIntegerField(theGarbageCollector, F_GarbageCollector_passes);
  w_int done = 0;
  w_int retcode;

  x_status status;

  woempa(7,"(GC) Begin pass %d\n",gc_pass_count);
  if (gc_pass_count % PRINTRATE == 0) woempa(7, "(GC) %d Collector: %d instances %d freed (remaining %d bytes out of %d) ticks = %ld, sting = %d, priority = %d, wait time = %d.\n", gc_pass_count, instance_use, instance_returned, x_mem_avail(), x_mem_total(), x_time_get(), sting_count, gc_thread->jpriority, getIntegerField(gc_instance, F_GarbageCollector_sleep_millis));
#ifdef DEBUG
  if (gc_pass_count % (PRINTRATE*PRINTRATE) == 0) reportMemStat(1);
#endif
  sting_count = 0;

  while (done < 2) {
    switch (gc_phase) {
      case GC_PHASE_UNREADY:
      case GC_PHASE_COMPLETE:
        woempa(7, "Thread %w: Phase = COMPLETE, doing a PREPARE and MARK\n", currentWonkaThread->name);
        marking_thread = currentWonkaThread;
        gc_phase = GC_PHASE_PREPARE;
#ifdef GC_SAFE_POINTS
        prepreparation();
#endif
        retcode = preparationPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");
          done = 99;
          gc_phase = GC_PHASE_COMPLETE;
          x_monitor_notify_all(gc_monitor);
#ifdef GC_SAFE_POINTS
          postmark();
#endif

          break;

        }
        gc_phase = GC_PHASE_MARK;
        retcode = markPhase();
#ifdef GC_SAFE_POINTS
        postmark();
#endif
        if (retcode < 0) {
          woempa(9, "Aborting MARK\n");
          done = 99;
          gc_phase = GC_PHASE_COMPLETE;
          marking_thread = NULL;
          x_monitor_notify_all(gc_monitor);

          break;

        }
        woempa(7,"(GC) Thread %w: Entering SWEEP phase.\n", currentWonkaThread->name);
        gc_phase = GC_PHASE_SWEEP;
        marking_thread = NULL;
        x_monitor_notify_all(gc_monitor);
        done += 1;
        /* fall through */

      case GC_PHASE_SWEEP:
        woempa(7, "Thread %w: Phase = SWEEP, joining in the fun\n", currentWonkaThread->name);
        sweepPhase();
        gc_phase = GC_PHASE_COMPLETE;
        x_monitor_notify_all(gc_monitor);
        done *= 2;
        woempa(7, "Thread %w: Have %sperformed a complete cycle\n", currentWonkaThread->name, done?"":"not yet ");
        break;

      default: // GC_PHASE_PREPARE/MARK
        woempa(7, "Thread %w: Phase = PREPARE/MARK, waiting\n", currentWonkaThread->name);
        status = x_monitor_wait(gc_monitor, 3); // micronap
        if (status == xs_interrupted) {
          x_monitor_eternal(gc_monitor);
        }
    }
  }
  woempa(7,"(GC) End pass %d : %d instances returned so far\n",gc_pass_count, instance_returned);

  return instance_returned;

}

w_int gc_request(w_int requested) {

  w_int released = 0;
  w_int remaining = requested;
  w_int swept;
  w_int tries = memory_load_factor + 3;
  w_int retcode;

  x_status status;

  while (tries > 0 && remaining > 0) {
    switch (gc_phase) {
      case GC_PHASE_PREPARE:
      case GC_PHASE_MARK:
        if (currentWonkaThread == marking_thread) {
          woempa(9, "Recursive gc_request????\n");
          tries = 0;
        }
        else {
          woempa(7, "Thread %w: Phase = PREPARE/MARK, waiting\n", currentWonkaThread->name);
          status = x_monitor_wait(gc_monitor, 3);
          if (status == xs_interrupted) {
            x_monitor_eternal(gc_monitor);
          }
        }
        break;

      case GC_PHASE_SWEEP:
        woempa(7, "Thread %w: Phase = SWEEP, joining in the fun\n", currentWonkaThread->name);
        swept = sweep(remaining);
        released += swept;
        remaining -= swept;
        woempa(7, "Thread %w: Reclaimed %d bytes, %d to go\n", currentWonkaThread->name, swept, remaining);
        if (swept == 0) {
          gc_phase = GC_PHASE_COMPLETE;
          x_monitor_notify_all(gc_monitor);
        }
        tries -= 1;

        break;

      case GC_PHASE_COMPLETE:
        woempa(7, "Thread %w: Phase = COMPLETE, doing a PREPARE and MARK\n", currentWonkaThread->name);
        marking_thread = currentWonkaThread;
        gc_phase = GC_PHASE_PREPARE;
#ifdef GC_SAFE_POINTS
        prepreparation();
#endif
        retcode = preparationPhase();
        if (retcode < 0) {
          woempa(9, "Aborting PREPARE\n");
          gc_phase = GC_PHASE_COMPLETE;
          x_monitor_notify_all(gc_monitor);
#ifdef GC_SAFE_POINTS
          postmark();
#endif
          tries -= 1;

          break;

        }
        gc_phase = GC_PHASE_MARK;
        retcode = markPhase();
#ifdef GC_SAFE_POINTS
        postmark();
#endif
        if (retcode < 0) {
          woempa(9, "Aborting MARK\n");
          gc_phase = GC_PHASE_COMPLETE;
          marking_thread = NULL;
          x_monitor_notify_all(gc_monitor);
          tries -= 1;

          break;

        }
        gc_phase = GC_PHASE_SWEEP;
        marking_thread = NULL;
        x_monitor_notify_all(gc_monitor);
        break;

      default: 
        /*  GC_PHASE_UNREADY - don't try to reclaim anything yet */
        tries = 0;
    }
  }
  woempa(7, "Released: %d bytes\n", released);

  return released;

}

void gc_housekeeping(void) {
}

