/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: instance.c 4294 2006-05-10 11:35:19Z chris $
*/

//#define DEBUG

#include "string.h"
#include "arrays.h"
#include "checks.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "hashtable.h"
#include "loading.h"
#include "list.h"
#include "ts-mem.h"
#include "wstrings.h"
#include "threads.h"
#include "wordset.h"
#include "profile.h"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

extern w_hashtable lock_hashtable;

char * print_instance_short(char * buffer, int * remain, void * data, int w, int p, unsigned int f) {
  w_instance instance = data;
  w_int    nbytes;
  char    *temp;
  
  if (instance == NULL) {
    strncpy(buffer, (char *)"<NULL>", *remain);
    nbytes = *remain < 6 ? *remain : 6;
    *remain -= nbytes;

    return buffer + nbytes;

  }

  temp = buffer;
  nbytes = x_snprintf(temp, *remain, "%k@%p", instance2clazz(instance), instance);
  *remain -= nbytes;

  return temp + nbytes;
}

// TODO: make this call toString() if not the default one of java.lang.Object ...
char * print_instance_long(char * buffer, int * remain, void * data, int w, int p, unsigned int f) {
  w_instance instance = data;
  w_int    nbytes;
  char    *temp;
  
  if (instance == NULL) {
    strncpy(buffer, (char *)"<NULL>", *remain);
    nbytes = *remain < 6 ? *remain : 6;
    *remain -= nbytes;

    return buffer + nbytes;

  }

  temp = buffer;
  nbytes = x_snprintf(temp, *remain, "%K@%p", instance2clazz(instance), instance);
  *remain -= nbytes;

  return temp + nbytes;
}

w_clazz instance2clazz(w_instance ref) {

  if (ref == NULL) {
    return clazzObject;
  }

  return instance2object(makeLocal(ref))->clazz;

}

w_size instance_use = 0;
w_size instance_allocated = 0;
w_size instance_returned = 0;
w_object instance_first = NULL;

inline static void registerObject(w_object object, w_thread thread) {
  x_frame frame = thread->top;

  woempa(6, "Registering %j\n", object->fields);
  
  pushLocalReference(frame, object->fields);
  
  instance_use += 1;
  instance_allocated += 1;
  setFlag(object->flags, O_IS_JAVA_INSTANCE);
#ifndef GC_SAFE_POINTS
  unsetFlag(object->flags, O_GREEN);
#endif
  x_mem_tag_set(block2chunk(object), (x_mem_tag_get(block2chunk(object)) | OBJECT_TAG));
}

inline static void checkClazz(w_clazz clazz) {

  if (! clazz) {
    wabort(ABORT_WONKA, "Clazz is NULL\n");
  }

  /*
  ** Check if bytes needed is negative or larger than 8 Mbytes.
  */
    
  if (clazz->bytes_needed & 0xff800000) {
    wabort(ABORT_WONKA, "Clazz %k impossible size words = 0x%x bytes = 0x%x\n", clazz, clazz->instanceSize, clazz->bytes_needed);
  }

}

/*
** Time in milliseconds that we will wait for the garbage collector to free
** up the number of requested bytes. When debugging we set it larger since
** the memory release routines and the runtime checks add a lot of overhead.
*/

void reportInstanceStat(void);

#ifdef DEBUG
static const w_size sleep_time = 5000;
#else
static const w_size sleep_time = 500;
#endif

static w_boolean heap_request(w_thread thread, w_int bytes) {

  w_realm realm = NULL;
  w_int   i;

#ifdef GC_SAFE_POINTS
  return TRUE;
#endif

  if (thread == gc_thread) {
    woempa(1, "Called by gc thread.\n");
    return TRUE;
  }

  if (gc_instance == NULL) {
    woempa(1, "Called before gc exists: gc_instance = %p.\n", gc_instance);
    return TRUE;
  }

  // return TRUE;

  if (thread->group) {
    realm = thread2realm(thread);
  }

  if(thread->jpriority > 10) {
    return checkRealmMemoryQuota(realm, bytes) && x_mem_avail() - bytes > min_heap_free;
  }

  for(i=0; i < 3; i++) {
    if (i) {
      x_thread_sleep(x_millis2ticks(sleep_time));    // Don't sleep the first time.
    }

    gc_reclaim(bytes, NULL);

    if (checkRealmMemoryQuota(realm, bytes) && x_mem_avail() - bytes > min_heap_free) {
      return TRUE;
    }
  }

  return FALSE;
}

w_instance allocInstance(w_thread thread, w_clazz clazz) {

  w_object object = NULL;
  w_realm realm = NULL;

  if (thread->group) {
    realm = thread2realm(thread);
  }

  /*
  ** The class must be initialized.
  */

  classMustBeInitialized(clazz);

  if(clazz->initialize != alreadyInitialized){
    return NULL;
  }

  checkClazz(clazz);

  woempa(1, "clazz is %k at %p, requested size is %d words, instance needs %d bytes.\n", clazz, clazz, clazz->instanceSize, clazz->bytes_needed);

  /*
  ** The test for CLAZZ_IS_THROWABLE is really a proxy for `ignore quotas'.
  ** Maybe we should have a separate allocator for priority cases?
  */

  if (heap_request(thread, (w_int)clazz->bytes_needed) || isSet(clazz->flags, CLAZZ_IS_THROWABLE)) {
    object = allocClearedMem_realm(realm, clazz->bytes_needed);
  }

  if (! object) {
    woempa(9, "Could not allocate a %k for thread %w, throwing OutOfMemoryError\n", clazz, thread->name);
    throwOutOfMemoryError(thread);
    return NULL;
  }
  woempa(1, "Allocated a %k at %p, need %d bytes\n", clazz, object->fields, clazz->bytes_needed);

  object->clazz = clazz;
#ifndef GC_SAFE_POINTS
  object->flags = O_GREEN;
#endif

  if (isSet(clazz->flags, CLAZZ_HAS_FINALIZER)) {
    woempa(1, "Clazz %k has a finalizer, setting instance at %p FINALIZABLE\n", clazz, object->fields);
    setFlag(object->flags, O_FINALIZABLE);
  }

  if (isSet(clazz->flags, CLAZZ_IS_REFERENCE)) {
    woempa(1, "Clazz %k is reference, setting instance at %p ENQUEUEABLE\n", clazz, object->fields);
    setFlag(object->flags, O_ENQUEUEABLE);
  }

  if (isSet(clazz->flags, CLAZZ_IS_THROWABLE)) {
    woempa(5, "Clazz %k is a Throwable, filling in stack trace\n", clazz);
    fillThrowable(thread, object->fields);
  }

#ifdef JAVA_PROFILE
  profileAllocInstance(thread, clazz);
#endif 
  
  registerObject(object, thread);

  //woempa(1, "Allocated instance %p (object %p) of %k, %d words, realm %p.\n", object->fields, object, clazz, clazz->instanceSize, realm);

  return object->fields;

}

w_instance internalAllocArrayInstance(w_thread thread, w_clazz clazz, w_size size) {

  w_object object = NULL;
  w_size   bytes;
  w_realm realm = NULL;

  if (thread->group) {
    realm = thread2realm(thread);
  }

  checkClazz(clazz);  

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return NULL;

  }

  /*
  ** Calculate the number of bytes we need. Note that checking the requirements with the quota has been
  ** done allready by the calling function.
  */
  
  bytes = sizeof(w_Object) + (size * sizeof(w_word));

  if (heap_request(thread, bytes)) {
    object = allocClearedMem_realm(realm, bytes);
    woempa(1, "Allocated a %k at %p, need %d bytes\n", clazz, object->fields, bytes);
  }

  if (object == NULL) {
    woempa(9, "Unable to allocate memory for instance of %k (size %d)!\n", clazz, size);
    throwOutOfMemoryError(thread);
    return NULL;
  }

  object->clazz = clazz;
#ifndef GC_SAFE_POINTS
  object->flags = O_GREEN;
#endif

  registerObject(object, thread);

  return object->fields;
  
}

inline static w_size roundBitsToWords(w_int bits) {
  return ((bits + 31) & ~31) >> 5;
}

/*
** A helper structure for allocating arrays.
*/

typedef struct w_Aas * w_aas;

typedef struct w_Aas {
  w_instance Array;               // The array instance of this dimension
  w_clazz    clazz;               // Clazz of this dimension
  w_int      size;                // The size in words of this dimension
  w_int      length;              // The length of this array
  w_aas      next;                // The link to the next dimension, is NULL for the last dimension.
} w_Aas;

/*
** This function to allocate and fill subarrays is called recursively. So keep it very
** short and sweet. On an X86 with GCC 3.x compiled, it requires about 48 stack bytes per
** invocation, so if there is a nutcase that wants a 255 dimensional array, he needs somewhat
** more than 12K stack. Don't forget that the w_Aas structures also take 255 * 20 bytes of
** stack, totaling a bit more than 18K stack use, which is reasonable. In the allocator function
** we verify that the number of dimensions is smaller than 256.
*/

static void fillParentArray(w_thread thread, w_aas parent) {

  w_int x;

  if (parent->next) {
    for (x = 0; x < parent->length && ! exceptionThrown(thread); x++) {
      parent->next->Array = internalAllocArrayInstance(thread, parent->next->clazz, parent->next->size);
      if (exceptionThrown(thread)) {
        return;
      }
      parent->next->Array[F_Array_length] = parent->next->length;
      
      setArrayReferenceField(parent->Array, parent->next->Array, x);
      // [JD] thread->top->auxs -= 1; // Do a manual pop of the local reference; saves stack...
      popLocalReference(thread->top);
      fillParentArray(thread, parent->next);
    }
  }

}

w_instance allocArrayInstance(w_thread thread, w_clazz clazz, w_int dimensions, w_word lengths[]) {

  w_int i;
  w_Aas * Aas;
  w_clazz current;
  w_size total = 0;

  if (dimensions > 255) {
    return NULL;
  }

  Aas = alloca(sizeof(w_Aas) * dimensions);

  current = clazz;
  woempa(1, "Allocating an instance of %k (%d dimensions)\n", clazz, dimensions);
  for (i = 0; i < dimensions; i++) {
    Aas[i].next = (i == dimensions - 1) ? NULL : Aas + i + 1;
    Aas[i].length = lengths[i];
    Aas[i].size = 1 + roundBitsToWords(current->previousDimension->bits * lengths[i]);
    woempa(1, "Dimension %d has length %d, bits/element is %d, need %d words\n", i, lengths[i], current->previousDimension->bits, Aas[i].size);
    Aas[i].clazz = current;
    total += Aas[i].size;
    current = current->previousDimension;
  }

  if (total > (1024 * 1024 * 7)) {
    return NULL;
  }

  Aas[0].Array = internalAllocArrayInstance(thread, Aas[0].clazz, Aas[0].size);
  if (Aas[0].Array) {
    Aas[0].Array[F_Array_length] = Aas[0].length;
    fillParentArray(thread, Aas);
  }

  if (exceptionThrown(thread)) {
    return NULL;
  }
  
  return Aas[0].Array;
}

void newGlobalReference(w_instance instance) {

  w_instance found;

  found = (w_instance) ht_register(globals_hashtable, (w_word)instance);
  newReferenceTo(instance);
  woempa(1, "%s global reference to instance %p.\n", found ? "further" : "new", instance);

}

void startHeap() {

  lock_hashtable = ht_create(W_Realm_system, (char*)"hashtable:locks", 131, NULL, NULL, 0, 0);

  min_heap_free = x_mem_total() / 32;
  if (min_heap_free > (2 * 1024 * 1024)) {
    min_heap_free = 2 * 1024 * 1024;
  }

}

//#ifdef DEBUG 

typedef struct w_IEntry * w_ientry;

typedef struct w_IEntry {
  w_ientry next;
  w_word hash;
  w_string name;
  w_int hits;
} w_IEntry;

static w_ientry * ht;
static const w_size num_buckets = 5000;
static w_int num_ientry;

static w_ientry find(w_ientry new) {

  w_size i = new->hash % num_buckets;
  w_ientry ex;
  
  for (ex = ht[i]; ex; ex = ex->next) {
    if (ex->name == new->name) {
      return ex;
    }
  }

  return NULL;

}

static void add(w_ientry new) {

  w_size i = new->hash % num_buckets;
  w_ientry ex;

  for (ex = ht[i]; ex; ex = ex->next) {
    if (ex->next == NULL) {
      ex->next = new;
      break;
    }
  }

  if (ex == NULL) {
    ht[i] = new;
  }

  num_ientry += 1;
}

static w_int ientry_sort_hits(const void * v_ientry1, const void * v_ientry2) {

  w_ientry ientry1 = *(w_ientry *)v_ientry1;
  w_ientry ientry2 = *(w_ientry *)v_ientry2;

  if(!ientry1) return -1;
  if(!ientry2) return 1;
  
  return (ientry1->hits == ientry2->hits ? 0 : (ientry1->hits < ientry2->hits ? 1 : -1));
}

static x_boolean dumpInstances_callback(void * mem, void * arg) {

  w_instance instance;
  w_object   object;
  w_ientry   ientry;
  w_ientry   ex;

  object = chunk2object(mem);
  instance = object->fields;
  ientry = x_mem_alloc(sizeof(w_IEntry));
  if (! ientry) {
    return FALSE;
  }
  ientry->name = object->clazz->dotified;
  ientry->next = NULL;
  ientry->hits = 1;
  ientry->hash = (w_word)object->clazz->dotified->wonka_hash;

  ex = find(ientry);
  if (ex) {
    ex->hits += 1;
    x_mem_free(ientry);
  }
  else {
    add(ientry);
  }
  
  return TRUE;
  
}

void reportInstanceStat(void) {

  w_size i, j = 0;
  w_ientry entry;
  w_ientry victim;
  static w_ientry * table;

  num_ientry = 0;

  x_mem_lock(x_eternal);
  
  ht = x_mem_calloc(num_buckets * sizeof(w_ientry));
  
  x_mem_scan(x_eternal, OBJECT_TAG, dumpInstances_callback, NULL);

  table = x_mem_calloc(num_ientry * sizeof(w_ientry));
  for (i = 0; i < num_buckets; i++) {
    entry = ht[i];
    while (entry) {
      table[j++] = entry;
      entry = entry->next;
    }
  }
  
  qsort(table, num_ientry, sizeof(w_ientry), ientry_sort_hits);

  w_dump("--- Start Instance Stats --- \n");
  for (i = 0; i < num_ientry; i++) {
    entry = table[i];
    w_dump("%5d   %w \n", entry->hits, entry->name);
  }

  w_dump("--- Stop Instance Stats --- \n");

  x_mem_unlock();

  for (i = 0; i < num_buckets; i++) {
    entry = ht[i];
    while (entry) {
      victim = entry;
      entry = entry->next;
      x_mem_free(victim);
    }
  }

  x_mem_free(ht);  
  x_mem_free(table);  

}

//#endif
