/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: initialize.c 3644 2003-03-13 09:27:57Z gray $
*/

#include <string.h>

#include "checks.h"
#include "clazz.h"
#include "constant.h"
#include "exception.h"
#include "fields.h"
#include "interpreter.h"
#include "loading.h"
#include "methods.h"
#include "wonka.h"

void initializeStaticFields(w_thread thread, w_clazz clazz) {

  w_size  i;
  w_field field;
  w_cnt   c;

  woempa(1, "Setting the static fields of '%k' to default values.\n", clazz);

  for (i = 0; i < clazz->numStaticFields; i++) {
    field = &clazz->own_fields[i];
    if (field->initval) {
      c = &clazz->constants[field->initval];
      switch (c->cnt_tag) {
        case CONSTANT_STRING:
        case RESOLVED_STRING:
          {
            w_instance instance = getStringConstant(clazz, field->initval);

            clazz->staticFields[field->size_and_slot] = (w_word) instance;
            removeLocalReference(thread, instance);
          }
          woempa(1, "Initialized static field '%w' (type %k) of '%k' to 0x%08x.\n", NM(field), field->value_clazz, clazz, clazz->staticFields[field->size_and_slot]);
          break;
      
        case CONSTANT_INTEGER:
          clazz->staticFields[field->size_and_slot] = getIntegerConstant(clazz, field->initval);
          woempa(1, "Initialized static field '%w' (type %k) of '%k' to 0x%08x.\n", NM(field), field->value_clazz, clazz, clazz->staticFields[field->size_and_slot]);
          break;

        case CONSTANT_FLOAT:
          clazz->staticFields[field->size_and_slot] = getFloatConstant(clazz, field->initval);
          woempa(1, "Initialized static field '%w' (type %k) of '%k' to 0x%08x.\n", NM(field), field->value_clazz, clazz, clazz->staticFields[field->size_and_slot]);
          break;

        case CONSTANT_LONG:
          {
            w_u64 u64 = getLongConstant(clazz, field->initval);

            clazz->staticFields[field->size_and_slot] = u64.words[0];
            clazz->staticFields [ field->size_and_slot + 1] = u64.words[1];
            woempa(1, "Initialized static field '%w' (type %k) of '%k' to 0x%08x%08x.\n", NM(field), field->value_clazz, clazz,clazz->staticFields[field->size_and_slot+1],clazz->staticFields[field->size_and_slot]);
          }
          break;

        case CONSTANT_DOUBLE:
          {
            w_u64 u64 = getDoubleConstant(clazz, field->initval);

            clazz->staticFields[field->size_and_slot] = u64.words[0];
            clazz->staticFields [ field->size_and_slot + 1] = u64.words[1];
            woempa(1, "Initialized static field '%w' (type %k) of '%k' to 0x%08x%08x.\n", NM(field), field->value_clazz, clazz, clazz->staticFields[field->size_and_slot+1], clazz->staticFields[field->size_and_slot]);
          }
          break;

        default:
          wabort(ABORT_WONKA, "Static field '%w' of %k has initializer with cnt_tag %d.\n", field->name, clazz, c->cnt_tag);
      }
    }
    else {
      clazz->staticFields[field->size_and_slot] = 0;
      if (isSet(field->flags, FIELD_IS_LONG)) {
        clazz->staticFields[field->size_and_slot + 1] = 0;
      }
    }
  }
}

static void doSuperConstructorHack(w_clazz clazz, w_method m) {
  w_size j;
  w_size n;
  w_method super;

  if (isSet(m->flags, METHOD_IS_CONSTRUCTOR) && m->exec.code && (m->exec.code_length > 4) && (m->exec.code[0] == 0x2a) && (m->exec.code[1] == 0xb7)) {
    woempa(1, "First byte of %M is 0x2a (aload_0), second is 0xb7\n", m);
    j = (unsigned char)m->exec.code[2];
    j = (j << 8) | (unsigned char)m->exec.code[3];
    super = getMethodConstant(clazz, j);
    if (!super) {
      woempa(9, "Couldn't resolve method called using invokespecial at start of %M !\n", m);

      return;

    }
    woempa(1, "Method being called is %M with code length %d\n", super, super->exec.code_length);
    if (isSet(m->flags, METHOD_IS_CONSTRUCTOR) && super->exec.code_length && (super->exec.code[0] == 0xb1)) {
      woempa(7, "Removing call from %M to trivial constructor %M :)\n", m, super);
      m->exec.code_length -= 4;
      memmove(m->exec.code, m->exec.code + 4, m->exec.code_length);

#ifdef STORE_METHOD_DEBUG_INFO
      n = m->exec.numLineNums;
      while (n && (m->exec.lineNums[0].start_pc < 4)) {
        woempa(1, "Eliminating lineNums[0] entry with start_pc %d\n", m->exec.lineNums[0].start_pc);
        memcpy(m->exec.lineNums, m->exec.lineNums + 1, (--n) * sizeof(w_LineNum));
      }
      m->exec.numLineNums = n;

      for (j = 0; j < n; ++j) {
        if (m->exec.lineNums[j].start_pc > 3) {
          woempa(1, "Reducing lineNums[%d].start_pc from %d to %d\n", j, m->exec.lineNums[j].start_pc, m->exec.lineNums[j].start_pc - 4);
          m->exec.lineNums[j].start_pc -= 4;
        }
      }
      n = m->exec.numLocalVars;
      for (j = 0; j < n; ++j) {
        if (m->exec.localVars[j].start_pc > 3) {
          woempa(1, "Reducing localVars[%d].start_pc from %d to %d\n", j, m->exec.localVars[j].start_pc, m->exec.localVars[j].start_pc - 4);
          m->exec.localVars[j].start_pc -= 4;
        }
        else {
          m->exec.localVars[j].start_pc = 0;
        }
        if (m->exec.localVars[j].end_pc > 3) {
          woempa(1, "Reducing localVars[%d].end_pc from %d to %d\n", j, m->exec.localVars[j].end_pc, m->exec.localVars[j].end_pc - 4);
          m->exec.localVars[j].end_pc -= 4;
        }
        else {
          m->exec.localVars[j].end_pc = 0;
        }
      }
#endif
      n = m->exec.numExceptions;
      for (j = 0; j < n; ++j) {
        if (m->exec.exceptions[j].start_pc > 3) {
          woempa(1, "Reducing exceptions[%d].start_pc from %d to %d\n", j, m->exec.exceptions[j].start_pc, m->exec.exceptions[j].start_pc - 4);
          m->exec.exceptions[j].start_pc -= 4;
        }
        else {
          m->exec.exceptions[j].start_pc = 0;
        }
        if (m->exec.exceptions[j].end_pc > 3) {
          woempa(1, "Reducing exceptions[%d].end_pc from %d to %d\n", j, m->exec.exceptions[j].end_pc, m->exec.exceptions[j].end_pc - 4);
          m->exec.exceptions[j].end_pc -= 4;
        }
        else {
          woempa(8, "Setting exceptions[%d].end_pc to 0 -- handler may be unreachable?\n", j);
          m->exec.exceptions[j].end_pc = 0;
        }
        if (m->exec.exceptions[j].handler_pc > 3) {
          woempa(1, "Reducing exceptions[%d].handler_pc from %d to %d\n", j, m->exec.exceptions[j].handler_pc, m->exec.exceptions[j].handler_pc - 4);
          m->exec.exceptions[j].handler_pc -= 4;
        }
        else {
          woempa(8, "Setting exceptions[%d].handler_pc to 0 -- odd, eh?\n", j);
          m->exec.exceptions[j].handler_pc = 0;
        }
      }
    }
  }
}

/*
**   - if the class is not abstract, it is not allowed to contain any
**     abstract methods.
**   - identify the static initializer, default initializer <init>()V, and
**     finalizer() (if any) and record their addresses.
*/

static void scanMethods(w_clazz clazz) {
  w_method m;
  w_size   i;

  for (i = 0; i < clazz->numDeclaredMethods; i++) {
    m = &clazz->own_methods[i];

    if (m->spec.name == string_angle_brackets_clinit && m->spec.arg_types == NULL) {
      woempa(1, "Class %k has static initializer %m\n", clazz, m);
      clazz->clinit = m;
    }
    else if (m->spec.name == string_angle_brackets_init && m->spec.arg_types == NULL) {
      woempa(1, "Class %k has default constructor %m\n", clazz, m);
      clazz->defaultInit = m;
    }
    else if (m->spec.name == string_run && m->spec.arg_types == NULL) {
      woempa(1, "Class %k has run method %m\n", clazz, m);
      clazz->runner = m;
    }
    doSuperConstructorHack(clazz, m);
  }

  if (isNotSet(clazz->flags, ACC_INTERFACE)) {
    m = virtualLookup(finalize_method, clazz);
    // We only record a finalizer if it is not the one declared in
    // java.lang.Object and is not a bytecode method consisting of "vreturn".
    if (m->spec.declaring_clazz != clazzObject && !(m->exec.code && m->exec.code_length == 1)) {
      woempa(7, "Class %k has finalizer %m\n", clazz, m);
      setFlag(clazz->flags, CLAZZ_HAS_FINALIZER);
    }
  }

  if (pedantic) {
    if (isNotSet(clazz->flags, ACC_ABSTRACT)) {
      for (i = 0; i < clazz->numInheritableMethods; i++) {
        m = clazz->vmlt[i];

        if (isSet(m->flags, ACC_ABSTRACT)) {
          woempa(1,"AbstractMethodError detected: non-abstract class %k fails to implement %M\n",clazz, m);
          throwException(currentWonkaThread, clazzAbstractMethodError, "%w", clazz->dotified);

          return;

        }
      }
    }
  }

  if (!clazz->runner && clazz->supers) {
    clazz->runner = clazz->supers[0]->runner;
  }
}

/*
** Initialize a class.
** The result returned is either NULL_or an exception.
**     clazz     the class to be initialised
**     thread    the calling thread
**     frame     the current stack frame
*/

void initializeClass(w_clazz clazz) {

  w_thread   thread = currentWonkaThread;
  x_frame    frame;
  w_size     i;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  if (classIsInitialized(clazz)) {
    return;
  }

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  x_monitor_eternal(clazz->resolution_monitor);
  if (classIsInitialized(clazz)) {
    x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif
    return;
  }

  mustBeLinked(clazz);
  if (exceptionThrown(thread)) {
    woempa(1, "Failed to resolve class %k prior to initializing it: got %k.\n", clazz, instance2clazz(exceptionThrown(thread)));
    x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif
    return;
  }

  if (getClazzState(clazz) == CLAZZ_STATE_BROKEN) {
    x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif
    return;
  }

  woempa(1, "Thread %w: initializing class %k.\n", NM(thread), clazz);
  setClazzState(clazz, CLAZZ_STATE_INITIALIZING);

  /*
  ** If this is a non-interface class and the super needs initializing,
  ** do it (recursively).  Stop one short of the ultimate superclass,
  ** namely Object.  If superclass initialisation fails then mark
  ** current class as broken,  wake up any other threads that wanted it
  ** initialized, and propagate the exception.
  */

  if ((isNotSet(clazz->flags, ACC_INTERFACE)) && (clazz->numSuperClasses > 1)) {
    for (i = clazz->numSuperClasses - 1; i > 0; --i) {
      woempa(1, "Initializing superclass[%d] (%k) of %k\n", i - 1, clazz->supers[i - 1], clazz);
      classMustBeInitialized(clazz->supers[i - 1]);
    }
  }

  if (!exceptionThrown(thread)) {
    /*
    ** Scan the method pool to check that all abstract classes are implemented
    ** and to identify special methods such as <init>()V, run()V, finalize()V.
    */
    scanMethods(clazz);
  }

  if (!exceptionThrown(thread)) {
    /*
    ** [8] If this is a local copy of a persistent class, read the static
    **     variables from persistent storage: otherwise, Set the static fields
    **     to default values and execute the <clinit> (if any).
    */

    if (clazz->numStaticWords > 0) {
      woempa(1, "Initializing static fields of %k : %d words\n", clazz, clazz->numStaticWords);
      clazz->staticFields = allocClearedMem(thread, clazz->numStaticWords * sizeof(w_word));
    }
  // We optimistically set the class as initialized, as we want the
  // Garbage Collector to trace static variables as soon as they are
  // filled in ...
    if (!clazz->dims) {
      initializeStaticFields(thread, clazz);
      clazz->initialize = alreadyInitialized;
      if (clazz->clinit) {
        /*
        ** Before we run <clinit>, we make sure that the exception we could be throwing ourselves afterwards
        ** is allready initialized. If we would need to initialize this exception clazz when an exception is
        ** pending, havoc could happen...
        */
        classMustBeInitialized(clazzExceptionInInitializerError);
        if (!exceptionThrown(thread)) {
          classMustBeInitialized(clazzAbstractMethodError);
        }
        if (!exceptionThrown(thread)) {
        woempa(1, "Class %k has <clinit> method %m, so let's run it.\n", clazz, clazz->clinit);
          frame = activateFrame(thread, clazz->clinit, 0, 0);
          deactivateFrame(frame, NULL);
        }
      }
    }
    else {
      clazz->initialize = alreadyInitialized;
    }
  }

  /*
  ** If <clinit> terminated normally, mark class initialized
  */

  if (! exceptionThrown(thread)) {
    setClazzState(clazz, CLAZZ_STATE_INITIALIZED);
    woempa(1, "Initialisation of %k terminated normally. Shouting it out...\n",  clazz);
  }
  else {
    /*
    ** [10] If <clinit> failed with something other than Error,  treat it as ExceptionInInitializerError.
    */

    woempa(9, "<clinit> failed with %k thrown.\n", instance2clazz(exceptionThrown(thread)));

    if(!isSuperClass(clazzError, instance2clazz(exceptionThrown(thread)))) {
      throwException(thread, clazzExceptionInInitializerError, "%w", instance2clazz(exceptionThrown(thread))->dotified);
    }

    /*
    ** [11] Mark the class as erroneous and propagate the exception
    */

    woempa(9, "<clinit> for %k did NOT terminate normally. Spreading the rumour...\n",  clazz);
    setClazzState(clazz, CLAZZ_STATE_BROKEN);
    clazz->initialize = alreadyBroken;
  }

  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif

}

void alreadyInitialized(w_clazz clazz) {
  woempa(1, "Class %k is already initialized.\n",  clazz);
}


