/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: link.c 3593 2003-03-06 10:39:33Z gray $
*/

/*
#include <string.h>

#include "checks.h"
#include "descriptor.h"
#include "fields.h"
#include "hashtable.h"
#include "methods.h"
#include "reflection.h"
#include "threads.h"
*/
#include "clazz.h"
#include "constant.h"
#include "exception.h"
#include "loading.h"
#include "wonka.h"

/*
** The ``linked'' state is the last state before class initialisation.
** Currently not much work is done here: however this is probably the
** appropriate moment to invoke a bytecode verifier.
*/

static w_int linkClazz(w_clazz clazz) {
  w_int    result = CLASS_LOADING_DID_NOTHING;

  // nothing to do ...
  result = CLASS_LOADING_SUCCEEDED;

  return result;
}

w_int mustBeLinked(w_clazz clazz) {
  w_thread thread = currentWonkaThread;
  w_int    state = getClazzState(clazz);
  w_int    result = CLASS_LOADING_DID_NOTHING;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

#ifdef RUNTIME_CHECKS
  if (state < CLAZZ_STATE_LOADED) {
    wabort(ABORT_WONKA, "%K must be loaded before it can be Linked\n", clazz);
  }

  if (exceptionThrown(thread)) {
    woempa(9, "Eh? Exception '%e' already pending in mustBeLinked(%K)\n", exceptionThrown(thread), clazz);
  }
#endif

  if (state == CLAZZ_STATE_BROKEN) {
  // TODO - is the right thing to throw?
    throwException(thread, clazzNoClassDefFoundError, "%k", clazz);

    return CLASS_LOADING_FAILED;

  }

  if (state >= CLAZZ_STATE_LINKED) {

    return CLASS_LOADING_DID_NOTHING;

  }

  mustBeReferenced(clazz);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  x_monitor_eternal(clazz->resolution_monitor);
  clazz->resolution_thread = thread;
  state = getClazzState(clazz);

  while(state == CLAZZ_STATE_LINKING) {
    x_monitor_wait(clazz->resolution_monitor, x_millis2ticks(1000));
  }

  if (state == CLAZZ_STATE_REFERENCED) {
    woempa(1, "Linking %K\n", clazz);
    setClazzState(clazz, CLAZZ_STATE_LINKING);
    x_monitor_exit(clazz->resolution_monitor);

/* Do we need this?
    n = clazz->numSuperClasses;
    woempa(1, "%K has %d superclasses\n", clazz, n);
    for (i = n - 1; (result != CLASS_LOADING_FAILED) && (i >= 0); --i) {
      result |= mustBeLinked(clazz->supers[i]);
    }

    n = clazz->numInterfaces;
    woempa(1, "%K has %d superinterfaces\n", clazz, n);
    for (i = n - 1; (result != CLASS_LOADING_FAILED) && (i >= 0); --i) {
      result |= mustBeLinked(clazz->interfaces[i]);
    }

    if (result != CLASS_LOADING_FAILED) {
      result |= linkClazz(clazz);
    }
*/
    result = linkClazz(clazz);

    if (result == CLASS_LOADING_FAILED) {
      x_monitor_eternal(clazz->resolution_monitor);
      setClazzState(clazz, CLAZZ_STATE_BROKEN);
      x_monitor_notify_all(clazz->resolution_monitor);
      x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
      if (unsafe) {
        enterUnsafeRegion(thread);
      }
#endif

      return result;

    }

    x_monitor_eternal(clazz->resolution_monitor);
    if(exceptionThrown(thread)) {
      setClazzState(clazz, CLAZZ_STATE_BROKEN);
      result = CLASS_LOADING_FAILED;
    }
    else {
      setClazzState(clazz, CLAZZ_STATE_LINKED);
    }
    x_monitor_notify_all(clazz->resolution_monitor);
  }
  else if (state == CLAZZ_STATE_BROKEN) {
    x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif

    return CLASS_LOADING_FAILED;

  }

  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  return result;
}

