/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: supers.c 4270 2005-10-28 09:59:10Z chris $
*/

#include "clazz.h"
#include "checks.h"
#include "constant.h"
#include "exception.h"
#include "loading.h"
#include "threads.h"

/*
** The maximum depth of a class in the hierarchy. Deliberately exaggerated,
** so that the reallocMem call will normally free up a useful amount of
** memory.
*/
#define MAX_SUPER_CLASSES 256

w_int loadSuperClasses(w_clazz clazz, w_thread thread) {
  w_clazz super;
  w_int   i;
  w_int   n;
  w_int   retcode;

  clazz->supers = allocMem_system(MAX_SUPER_CLASSES * sizeof(w_clazz));
  super = clazz;
  n = MAX_SUPER_CLASSES;
  for (i = 0; i < MAX_SUPER_CLASSES; ++i) {
    if (!super->temp.super_index) {
      woempa(1, "Reached top of hierarchy after %d class(es): %k\n", i, super);
      n = i;
      break;
    }

    super = getClassConstant(super, super->temp.super_index);
    if (!super) {

      return CLASS_LOADING_FAILED;

    }

    if (super == clazz) {
      throwException(currentWonkaThread, clazzClassCircularityError, "Class %k is its own superclass", clazz);

      return CLASS_LOADING_FAILED;

    }

    clazz->supers[i] = super;
    woempa(1, "Class %k supers[%d] = %k\n", clazz, i, super);
    if (getClazzState(super) >= CLAZZ_STATE_SUPERS_LOADED) {
      woempa(1, "Class %k is already supersLoaded, has %d superclasses => depth of %k is %d\n", super, super->numSuperClasses, clazz, i + super->numSuperClasses + 1);
      n = i + super->numSuperClasses + 1;
      break;
    }
  }

  if (n == MAX_SUPER_CLASSES) {
      wabort(ABORT_WONKA, "Class %k has too many superclasses", clazz);
  }

  for (i= i + 1; i < n; ++i) {
    woempa(1, "Copying %k (superclass[%d] of %k) as superclass[%d] of %k\n", super->supers[i - n + super->numSuperClasses], i - n + super->numSuperClasses, super, i, clazz);
    clazz->supers[i] = super->supers[i - n + super->numSuperClasses];
  }

  woempa(1, "Class %k has total of %d superclasses\n", clazz, n);
  clazz->supers = reallocMem(clazz->supers, n * sizeof(w_clazz));
  clazz->numSuperClasses = n;
  super = clazz->supers[0];
  if (isSet(super->flags, ACC_INTERFACE)) {
    woempa(9, "Violation of J+JVM Constraint 4.1.1, item 1 / 4.1.2, item 1\n");
    throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Superclass %k of %k is an interface", super, clazz);

    return CLASS_LOADING_FAILED;

  }
  if (isSet(super->flags, ACC_FINAL)) {
    woempa(9, "Violation of J+JVM Constraint 4.1.1, item 2\n");
    throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Superclass %k of %k is final", super, clazz);

    return CLASS_LOADING_FAILED;

  }
  if (isNotSet(super->flags, ACC_PUBLIC) && !sameRuntimePackage(clazz, super)) {
  woempa(9, "Violation of J+JVM Constraint 4.1.4\n");
    throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Superclass %k of %k is not accessible", super, clazz);

    return CLASS_LOADING_FAILED;

  }

  for (i = n - 1; i >= 0; --i) {
    retcode = mustBeSupersLoaded(clazz->supers[i]);
    if (exceptionThrown(thread)) {

      return CLASS_LOADING_FAILED;

    }
  }

  clazz->flags |= super->flags & CLAZZ_HERITABLE_FLAGS;

  return CLASS_LOADING_SUCCEEDED;
}

#define MAX_INTERFACES 1024

static w_boolean addInterface(w_clazz new, w_clazz *if_array, w_int *l) {
  w_int i;

  for (i = 0; i < *l; ++i) {
    if (if_array[i] == new) {
      return WONKA_FALSE;
    }
  }

  if_array[(*l)++] = new;

  return WONKA_TRUE;
}

w_int loadSuperInterfaces(w_clazz clazz, w_thread thread) {
  w_int   i;
  w_int   j;
  w_int   n;
  w_clazz interfaze;
  w_clazz super;
  w_int   retcode;

  n = 0;
  clazz->numDirectInterfaces = 0;
  clazz->interfaces = allocMem_system(MAX_INTERFACES * sizeof(w_clazz));

  /*
  ** We have to do this in two passes, in order to do the Right Thing in
  ** Class/getInterfaces(). First we put the directly inherited interfaces
  ** into clazz->interfaces[], counting them in clazz->numDirectInterfaces.
  ** In this pass we also call loadSuperInterfaces() on the direct super-
  ** interfaces, and perform a number of checks; and we discard duplicates.
  */
  for (i = 0; i < clazz->temp.interface_index_count && !exceptionThrown(thread); ++i) {
    interfaze = getClassConstant(clazz, clazz->temp.interface_index[i]);

    if(interfaze == NULL){

      return CLASS_LOADING_FAILED;

    }

    if (interfaze == clazz) {
      throwException(currentWonkaThread, clazzClassCircularityError, "Class %k is its own superinterface", clazz);

      return CLASS_LOADING_FAILED;

    }

    if (isNotSet(interfaze->flags, ACC_INTERFACE)) {
      woempa(9, "Violation of J+JVM Constraint 4.1.?, item ? / 4.1.?, item ?\n");
      throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Superinterface %k of %k is not an interface", interfaze, clazz);

      return CLASS_LOADING_FAILED;

    }

    if (isNotSet(interfaze->flags, ACC_PUBLIC) && !sameRuntimePackage(clazz, interfaze)) {
      woempa(9, "Violation of J+JVM Constraint 4.1.?\n");
      throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Superinterface %k of %k is not accessible", interfaze, clazz);

      return CLASS_LOADING_FAILED;

    }

    if (n == MAX_INTERFACES) {
      wabort(ABORT_WONKA, "Class %k has too many superinterfaces", clazz);
    }

    mustBeSupersLoaded(interfaze);
    if (exceptionThrown(thread)) {

      return CLASS_LOADING_FAILED;

    }

    if (addInterface(interfaze, clazz->interfaces, &n)) {
      ++clazz->numDirectInterfaces;
      woempa(1, "Added superinterface %k to %k\n", interfaze, clazz);
    }
    else {
      woempa(1, "Ignored duplicate superinterface %k of %k\n", interfaze, clazz);
    }
  }

  /*
  ** In the second pass, we append all non-duplicate interfaces inherited from
  ** the direct superinterfaces and from the superclass.
  */
  for (i = 0; i < clazz->numDirectInterfaces && !exceptionThrown(thread); ++i) {
    interfaze = clazz->interfaces[i];

    for (j = 0; j < interfaze->numInterfaces; ++j) {
      if (addInterface(interfaze->interfaces[j], clazz->interfaces, &n)) {
        woempa(1, "Added supersuperinterface %k to %k\n", interfaze->interfaces[j], clazz);
      }
      else {
        woempa(1, "Ignored duplicate supersuperinterface %k of %k\n", interfaze->interfaces[j], clazz);
      }
    }
  }

  super = getSuper(clazz);
  if (super) {
    for (j = 0; j < super->numInterfaces; ++j) {
      if (addInterface(super->interfaces[j], clazz->interfaces, &n)) {
        woempa(1, "Added supersuperinterface %k to %k\n", super->interfaces[j], clazz);
      }
      else {
        woempa(1, "Ignored duplicate supersuperinterface %k of %k\n", super->interfaces[j], clazz);
      }
    }
  }

  clazz->numInterfaces = n;
  woempa(1, "Class %k has total of %d superinterfaces, of which %d direct\n", clazz, n, clazz->numDirectInterfaces);
  releaseMem(clazz->temp.interface_index);
  clazz->temp.interface_index = NULL;
  clazz->interfaces = reallocMem(clazz->interfaces, n * sizeof(w_clazz));

  for (i = n - 1; i >= 0; --i) {
    retcode = mustBeSupersLoaded(clazz->interfaces[i]);
  }

  return CLASS_LOADING_SUCCEEDED;
}

w_int mustBeSupersLoaded(w_clazz clazz) {
  w_thread thread = currentWonkaThread;
  w_int    state = getClazzState(clazz);
  w_int    result = CLASS_LOADING_DID_NOTHING;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

#ifdef RUNTIME_CHECKS
  if (state < CLAZZ_STATE_LOADED) {
    wabort(ABORT_WONKA, "%K must be loaded before it can be SupersLoaded\n", clazz);
  }
#endif

  if (state == CLAZZ_STATE_BROKEN) {
  // TODO - is the right thing to throw?
    throwException(thread, clazzNoClassDefFoundError, "%k", clazz);

    return CLASS_LOADING_FAILED;

  }

  if (state >= CLAZZ_STATE_SUPERS_LOADED) {

    return CLASS_LOADING_DID_NOTHING;

  }

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  x_monitor_eternal(clazz->resolution_monitor);
  clazz->resolution_thread = thread;
  state = getClazzState(clazz);

  while(state == CLAZZ_STATE_LOADING) {
    x_monitor_wait(clazz->resolution_monitor, x_millis2ticks(1000));
  }

  if (state == CLAZZ_STATE_LOADED) {
    if (clazz->loader) {
      woempa(1, "Need to load supers of class %k using loader %j\n", clazz, clazz->loader);
    }
    else {
      woempa(1, "Need to load supers of class %k using bootstrap class loader\n", clazz);
    }

    if (isSet(clazz->flags, ACC_FINAL) && isSet(clazz->flags, ACC_ABSTRACT)) {
      woempa(9, "Violation of J+JVM Constraint 4.1.1, item 3\n");
      throwException(currentWonkaThread, clazzIncompatibleClassChangeError, "Class %k is both FINAL and ABSTRACT", clazz);
      setClazzState(clazz, CLAZZ_STATE_BROKEN);
      x_monitor_notify_all(clazz->resolution_monitor);
      x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
      if (unsafe) {
        enterUnsafeRegion(thread);
      }
#endif

      return CLASS_LOADING_FAILED;

    }

    setClazzState(clazz, CLAZZ_STATE_SUPERS_LOADING);
    x_monitor_exit(clazz->resolution_monitor);

    woempa(1, "Class %k super_index is %d\n", clazz, clazz->temp.super_index);
    if (clazz->temp.super_index) {
      result = loadSuperClasses(clazz, thread);
    }
    else {
      woempa(1, "Class %k super_index is 0, has no superclasses\n", clazz);
    }

    if (result != CLASS_LOADING_FAILED /*&& clazz->temp.interface_index_count*/) {
      result = loadSuperInterfaces(clazz, thread);
    }

    if (result == CLASS_LOADING_FAILED) {
      x_monitor_eternal(clazz->resolution_monitor);
      setClazzState(clazz, CLAZZ_STATE_BROKEN);
      x_monitor_notify_all(clazz->resolution_monitor);
      x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
      if (unsafe) {
        enterUnsafeRegion(thread);
      }
#endif

      return result;

    }

    x_monitor_eternal(clazz->resolution_monitor);
    setClazzState(clazz, CLAZZ_STATE_SUPERS_LOADED);
    x_monitor_notify_all(clazz->resolution_monitor);
  }
  else if (state == CLAZZ_STATE_BROKEN) {
    x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
      if (unsafe) {
        enterUnsafeRegion(thread);
      }
#endif

    return CLASS_LOADING_FAILED;

  }

  x_monitor_exit(clazz->resolution_monitor);
#ifdef GC_SAFE_POINTS
      if (unsafe) {
        enterUnsafeRegion(thread);
      }
#endif

  return result;
}

