/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: inits.c 4298 2006-05-19 09:41:44Z chris $
*/

#include <string.h>
#ifdef O4P
#include <time.h>
#endif

#include "argument.h"
#include "clazz.h"
#include "descriptor.h"
#include "device.h"
#include "exception.h"
#include "methods.h"
#include "driver_virtual.h"
#include "file_driver.h"
#include "deflate_driver.h"
#include "locks.h"
#include "loopback.h"
#include "sio.h"
#include "wstrings.h"
#include "wonka.h"
#include "vfs.h"
#include "network.h"

#ifdef MODULES
extern void  x_symtab_kernel(void);
#endif
//TODO: get rid of these extern's
extern w_Driver_ByteSerial sio_driver;
extern void mouse_set_path(char *s);

char *bootclasspath = BOOTCLASSDIR "/:" BOOTCLASSDIR "/" BOOTCLASSFILE;

extern x_Mutex woempaMutex;

w_ubyte *initWonka(w_ubyte *start_heap) {
  (void)x_mutex_create(&woempaMutex);

  x_formatter('w', print_string);
  x_formatter('k', print_clazz_short);
  x_formatter('K', print_clazz_long);
  x_formatter('j', print_instance_short);
  x_formatter('J', print_instance_long);
  x_formatter('m', print_method_short);
  x_formatter('M', print_method_long);
  x_formatter('e', print_exception);
  x_formatter('t', print_thread_short);
  x_formatter('T', print_thread_long);
  x_formatter('y', print_descriptor);

  initRealms();
  initLocks();

#ifdef MODULES
  x_symtab_kernel();
#endif

  start_heap = initKernel(start_heap);

  install_term_handler();

  return start_heap;

}

#ifdef FSENABLE
#ifdef ECOS
  extern w_driver image_driver;
#else
  extern w_driver disk_driver;
#endif //ECOS
#endif
  
#ifdef JDWP  
  w_void jdwp_init(w_void);
#endif

#ifdef JSPOT
  w_void jspot_init(w_void);
#endif
  
w_int testWonka(void);

void startVFS(void) {
  init_vfs();
#ifdef FSENABLE
#ifdef ECOS
    vfs_mount((char *)"hdb", (char *)"/", (char *)"e2fs", VFS_MOUNT_RW);
#elif defined(LINUX)
    vfs_mount((char *)"hda4", (char *)"/", (char *)"e2fs", VFS_MOUNT_RW);
#elif defined(NETBSD)
#error "Bzzzt. Not implemented."
#else
#error "Bzzzt. Not implemented."
#endif 
#else
  vfs_mount(0, (char *)"/", (char *)"wromfs", VFS_MOUNT_RO); 
#endif
}

extern char *get_default_classpath(void);

char            *awt_args = NULL;
char            *awt_splash = NULL;

#ifdef JDWP
int      jdwp_enabled = 0;
char     *jdwp_args = NULL;
void     print_jdwp_help(void);
#endif

#ifdef JSPOT
  char *jspot_args = NULL;
  void   jspot_help(void);
#endif

void args_read(void) {
  Wonka_InitArgs  *args = &system_InitArgs;
  char            **properties;
  int             prop_count = 0;
  char            *classpath;
  int             jarfileCommand = 0;

  memset(args, 0, sizeof(Wonka_InitArgs));
  properties = allocClearedMem_system(100 * sizeof(char *));


  classpath = get_default_classpath();

  woempa(7, "Found %d command line arguments\n", command_line_argument_count);
  while (command_line_argument_count) {
    w_int extra_args = 0;
    woempa(7, "Analysing command line argument `%s'\n", command_line_arguments[0]);

    if((strcmp(command_line_arguments[0], "-cp") == 0) || 
       (strcmp(command_line_arguments[0], "-classpath") == 0)) {
      extra_args = 1;
      woempa(7, "Found a classpath argument `%s'\n", command_line_arguments[1]);
      if(classpath) {
        woempa(7, "Overrides the default classpath `%s'\n", classpath);
      } 
      classpath = command_line_arguments[1];
    }
    else if(strncmp(command_line_arguments[0], "-D", 2) == 0) {
      woempa(7, "Found a property argument `%s'\n", command_line_arguments[0] + 2);
      properties[prop_count++] = (char *)&command_line_arguments[0][2];
    }
    else if(strncmp(command_line_arguments[0], "-Xbootclasspath:", 16) == 0) {
      woempa(7, "Found an Xbootclasspath argument `%s'\n", command_line_arguments[0] + 16);
      bootclasspath = (char *)&command_line_arguments[0][16];
    }
#ifdef JDWP 

    else if(strncmp(command_line_arguments[0], "-Xjdwp:", 7) == 0) {
      woempa(7, "Found an Xjdwp argument `%s'\n", command_line_arguments[0] + 7);
      jdwp_args = (char *)&command_line_arguments[0][7];
      if(strcmp(jdwp_args, "help") == 0) {
        print_jdwp_help();
      }
    }
    else if(strncmp(command_line_arguments[0], "-Xdebug", 7) == 0) {
      woempa(7, "Found an Xdebug argument\n");
      jdwp_enabled = 1;
    }

#endif  /* JDWP */

#ifdef JSPOT
    
    else if(strncmp(command_line_arguments[0], "-Wcompiler:", 11) == 0) {
      woempa(7, "Found an Wcompiler argument `%s'\n", command_line_arguments[0] + 11);
      jspot_args = (char *)&command_line_arguments[0][11];
      if(strcmp(jspot_args, "help") == 0) {
        jspot_help();
      }
    } 
    
#endif /* JSPOT */
    
    else if(strncmp(command_line_arguments[0], "-Wrudolph:", 10) == 0) { 
      woempa(7, "Found a Wrudolph argument `%s'\n", command_line_arguments[0] + 10);
      awt_args = (char *)&command_line_arguments[0][10];
    }

#if defined(AWT_FDEV)

    else if(strncmp(command_line_arguments[0], "-Wsplash:", 9) == 0) { 
      woempa(7, "Found a Wsplash argument `%s'\n", command_line_arguments[0] + 9);
      awt_splash = (char *)&command_line_arguments[0][9];
    }
    
#endif  /* AWT_XSIM */
    else if(strcmp(command_line_arguments[0], "-jar") == 0) {
      woempa(7, "Found a -jar argument\n");
      jarfileCommand = 1;
    }

    else if(strcmp(command_line_arguments[0], "-Wpedantic") == 0) {
      woempa(7, "Found a -Wpedantic argument\n");
      pedantic = WONKA_TRUE;
    }


    else if(strncmp(command_line_arguments[0], "-", 1) == 0) {
      woempa(7, " Unrecognized option: %s\n", command_line_arguments[0]);

      exit(1);

    } else {
      woempa(7, " Not an option: %s, passing remaining %d arguments on to Init/main()\n", command_line_arguments[0], command_line_argument_count);

      /*
      ** It's not an option, so it has to be the main class and 
      ** it's arguments. Since command_line_argument_count and 
      ** command_line_arguments were updated each time we went
      ** through this loop, they're now pointing right at the
      ** requested startclass.
      */

      break;

    }
    
      command_line_argument_count = command_line_argument_count - 1 - extra_args;
      command_line_arguments += extra_args + 1;
  }

  if(jarfileCommand){
    command_line_argument_count += 1;
    command_line_arguments -= 1;
    woempa(7, "Found a -jar argument overwriting '%s' with '-jar'\n",command_line_arguments[0]);
    command_line_arguments[0] = "-jar";
  }

  args->properties = reallocMem(properties, (prop_count + 1) * sizeof(void *));;
  args->classpath  = classpath;



#ifdef DEBUG
  {
    w_int i;
    for (i = 0; i < prop_count; ++i) {
      woempa(7, "Property: %s\n", properties[i]);
    }
    woempa(7, "Classpath: %s\n", args->classpath);
    woempa(7, "Xbootclasspath: %s\n", bootclasspath);
    for (i = 0; i < command_line_argument_count; ++i) {
      woempa(7, "Java argument: %s\n", command_line_arguments[i]);
    }
  }
#endif
}


void startWonka(void* data) {

  JavaVM *vm;
  JNIEnv *env;

#ifdef O4P
  struct timespec ts;
#endif

  make_ISO3309_CRC_table();

  /*
  ** Here we start routines that require a valid heap (for malloc) to
  ** be set up...
  */

  args_read();

  startStrings();

  startDeviceRegistry();
  startDriverRegistry();
  
#ifdef FSENABLE
#ifdef ECOS
  registerDevice("hdb", "hdb", 0, wdt_block_random);
  registerDriver((w_driver)&image_driver);
#else
  registerDevice("hda", "hda", 0, wdt_block_random);
  registerDriver((w_driver)&disk_driver);
#endif // ECOS
#endif

  /* Initialize known filesystems */
  
#if (defined(FSENABLE)) 
  init_e2fs(); 
#endif
  startVFS();

  registerDevice("unzip_", "zip", 0, wdt_byte_serial);
  registerDevice("zip_", "zip", 20, wdt_byte_serial);
// Enable the following if you want to play with loopback ports
//  registerDevice("loop_", "loop", 0, wdt_byte_serial);
 
#ifndef ECOS
  registerDriver((w_driver)&sio_driver);
#endif // ECOS
  registerDriver((w_driver)&deflate_driver);
  registerDriver((w_driver)&loopback_driver);
  registerDriver((w_driver)&file_driver);

#ifdef JSPOT
  jspot_init();
#endif
#ifdef DEBUG
  PutString((char*)"Calling JNI_CreateJavaVM() ...\n");
#endif
  JNI_CreateJavaVM(&vm, &env, &system_InitArgs);

#ifdef JDWP
  jdwp_init();
#endif
  
  groupManagerEntry(W_ThreadGroup_system);

#ifdef O4P
  // Don't return from here, 'coz the vm gets popped off the stack! (D'oh)
  ts.tv_sec = 10;
  ts.tv_nsec = 0;

  while(1) {
    nanosleep(&ts, NULL);
  }
#endif
}

