/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


#include "oswald.h"
#include "wonka.h"
#include "ts-mem.h"
#include "realms.h"

#include <stdio.h>  /* for sprintf */
#include <string.h> /* for strlen */
#include <unistd.h>

#ifndef SHARED_OBJECTS

#ifdef OSWALD

#include "modules.h"
#include "vfs.h"

void *loadModule(char * name, char * path) {
  
  x_status         status;
  x_ubyte          *elfdata;
  int              fd;
  struct vfs_STAT  statbuf;
  w_word           elfsize;
  x_Module         Module;
  x_module         module = &Module;
  char             *filename;
  
  filename = allocMem_system(strlen(name) + 12);
  sprintf(filename, "./system/%s.o", name);
  
  if(vfs_stat(filename, &statbuf) != -1) { 
    elfsize = statbuf.st_size;
    woempa(9, "%s is %d bytes\n", filename, elfsize);
    elfdata = allocMem_system(elfsize);
    fd = vfs_open(filename, VFS_O_RDONLY, 0);
    vfs_read(fd, elfdata, elfsize);
    vfs_close(fd);
    releaseMem(filename);
  
    status = x_module_load(module, elfdata, x_mem_alloc, x_mem_free);
    if(status != xs_success) {
      woempa(9, "Loading of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }

    status = x_module_resolve(module);
    if(status != xs_success) {
      woempa(9, "Resolving of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
    status = x_module_relocate(module);
    if(status != xs_success) {
      woempa(9, "Relocating of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
    status = x_module_init(module);
    if(status != xs_success) {
      woempa(9, "Initialization of lib %s failed... (%d)\n", name, status);
      releaseMem(elfdata);
      return NULL;
    }
  
  } else {
    releaseMem(filename);
    woempa(9, "Could not find lib %s.\n", name);
    return NULL;
  }

  return (void *)module;
}

x_boolean searchForSymbol(x_module module, void * argument) {
  return 1;
}

void *lookupModuleSymbol(w_ubyte *name) {
 /* Iterate through all the loaded modules to find the symbol */
  x_module_iterate(searchForSymbol, name);
  return NULL; 
}

#else /* !OSWALD */

void *loadModule(w_ubyte *name, w_ubyte *path) { return NULL; }
void *lookupModuleSymbol(w_ubyte *name) { return NULL; }

#endif /* OSWALD */

#else /* SHARED_OBJECTS */

#include "vfs.h"
#include <dlfcn.h>

static void **handles = NULL;
static void **current = NULL;

void *loadModule(w_ubyte *name, w_ubyte *path) {
  char *filename;
  char *error;
  void *handle = NULL;
  int  offset = (int)current - (int)handles;
  char *orig_ld = NULL;
  char *new_ld = NULL;
  char *chptr;
  char *libPath = NULL;
  char *pathItem;

  if((offset % (10 * sizeof(void *))) == 0) {
    handles = x_mem_realloc(handles, offset + 10 * sizeof(void *));
    current = (void **)((int)handles + offset);
  }

  if(name) {
    filename = allocMem_system(strlen(name) + 10);
    sprintf(filename, "lib%s.so", name);

    woempa(9, "%s in %s\n", filename, path);

    if(path) {
      orig_ld = getenv("LD_LIBRARY_PATH");
      if(orig_ld) {
        woempa(9, "old LD_LIBRARY_PATH: %s\n", orig_ld);
        new_ld = x_mem_calloc(strlen(orig_ld) + strlen(path) + 2);
        sprintf(new_ld, "%s:%s", orig_ld, path);
      }
      else {
	// if path is absolute, use it. otherwise use current_working_dir + path
	if (path[0] == '/') {
	    new_ld = x_mem_calloc(strlen(path) + 2);
            sprintf(new_ld, "%s", path);
	}
	else {
	    new_ld = x_mem_calloc(strlen(current_working_dir) + strlen(path) + 2);
            sprintf(new_ld, "%s/%s", current_working_dir, path);
	}
      }
      woempa(9, "new LD_LIBRARY_PATH: %s\n", new_ld);
      
      // try each element in the new_ld path
      chptr = strchr(new_ld, ':');
      if (chptr == NULL) {
	 // only one item in path
	 libPath = x_mem_calloc(strlen(new_ld) + strlen(filename) + 1);
	 // check if pathItem ends in /
	 if (new_ld[strlen(new_ld)] == '/') {
	    sprintf(libPath, "%s%s", new_ld, filename);
	 }
	 else {
            sprintf(libPath, "%s/%s", new_ld, filename); 
	 }
	 woempa(9, "Calling dlopen on %s\n", libPath);
	 handle = dlopen(libPath, RTLD_LAZY | RTLD_GLOBAL);
	 x_mem_free(libPath);
      }
      else {
	 // try each item in path in turn
	 do {
            pathItem = x_mem_calloc(chptr - new_ld + 1);
	    strncpy(pathItem, new_ld, chptr - new_ld);
	    // setup libPath as appropraite
	    // pathItem is absolute and ends eith /
	    if (pathItem[0] == '/' && pathItem[strlen(pathItem)-1] == '/') {
	       libPath = x_mem_calloc(strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s%s", pathItem, filename);
	    }
	    // pathItem is absolute and does not end with /
	    else if (pathItem[0] == '/' && !(pathItem[strlen(pathItem)-1] == '/')) {
	       libPath = x_mem_calloc(strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s/%s", pathItem, filename);
	    }
	    // pathItem is relative and ends eith /
	    else if (!(pathItem[0] == '/') && (pathItem[strlen(pathItem)-1] == '/')) {
	       libPath = x_mem_calloc(strlen(current_working_dir) + strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s/%s%s", current_working_dir, pathItem, filename);
	    }
	    // pathItem is relative and does not end with /
	    else {
	       libPath = x_mem_calloc(strlen(current_working_dir) + strlen(pathItem) +  strlen(filename) + 3);
	       sprintf(libPath, "%s/%s/%s", current_working_dir, pathItem, filename);
	    }
	    woempa(9, "Calling dlopen on %s\n", libPath);
	    handle = dlopen(libPath, RTLD_LAZY | RTLD_GLOBAL);
	    x_mem_free(pathItem);
	    x_mem_free(libPath);
	    // get next item in path
	    new_ld = ++chptr;
	    chptr = strchr(new_ld, ':');
	 } while (handle == NULL && chptr != NULL);
	 // deal with last item
	 if (handle == NULL) {
	    pathItem = x_mem_calloc(strlen(new_ld) + 1);
	    strcpy(pathItem, new_ld);
	    // setup libPath as appropraite - see comments above
	    if (pathItem[0] == '/' && pathItem[strlen(pathItem)-1] == '/') {
	       libPath = x_mem_calloc(strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s%s", pathItem, filename);
	    }
	    else if (pathItem[0] == '/' && !(pathItem[strlen(pathItem)-1] == '/')) {
	       libPath = x_mem_calloc(strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s/%s", pathItem, filename);
	    }
	    else if (!(pathItem[0] == '/') && (pathItem[strlen(pathItem)-1] == '/')) {
	       libPath = x_mem_calloc(strlen(current_working_dir) + strlen(pathItem) + strlen(filename) + 2);
	       sprintf(libPath, "%s/%s%s", current_working_dir, pathItem, filename);
	    }
	    else {
	       libPath = x_mem_calloc(strlen(current_working_dir) + strlen(pathItem) +  strlen(filename) + 3);
	       sprintf(libPath, "%s/%s/%s", current_working_dir, pathItem, filename);
	    }
	    woempa(9, "Calling dlopen on %s\n", libPath);
	    handle = dlopen(libPath, RTLD_LAZY | RTLD_GLOBAL);
	    x_mem_free(libPath);
	    x_mem_free(pathItem);
	 }
      }
    }
  // deal with using LD_LIBRARY_PATH as normal
  else {
    handle = dlopen(filename, RTLD_LAZY | RTLD_GLOBAL);
  }
	  
  }
  else if(path) {
    char * p;
    filename = allocMem_system(strlen(path) + 255);
    filename = getcwd(filename, 255);
    p = filename + strlen(filename);
    sprintf(p, "%s", path);
    woempa(9, "PATH: %s\n", filename);
    handle = dlopen(filename, RTLD_LAZY | RTLD_GLOBAL);
  }
  else {
    return NULL;
  }
  
  if(handle) {
    *current++ = handle;
  }
  else {
    error = dlerror();
    woempa(9, "filename: %s, dlerror() : %s\n", filename, error);
  }

  if(name) {
    releaseMem(filename);

  }
  else if(path) {
    releaseMem(filename);
  }

  return handle;
}

void *lookupModuleSymbol(w_ubyte *name) {
  void *symbol = NULL;
  void **check = handles;

  woempa(9, "%s\n", name);

  while(symbol == NULL && check != current) {
    symbol = dlsym(*check++, name);
    if (!symbol) {
      woempa(9, "symbol: %s, dlerror() : %s\n", name, dlerror());
    }
  }
  
  return symbol;
}

#endif /* SHARED_OBJECTS */ 

