/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Runtime.c 4294 2006-05-10 11:35:19Z chris $
*/

#include "core-classes.h"
#include "hashtable.h"
#include "realms.h"
#include "ts-mem.h"
#include "wstrings.h"
#include "exception.h"
#include "clazz.h"
#include "misc.h"
#include "methods.h"
#include "wonka.h"
#include "arrays.h"
#include "exec.h"
#include "loading.h"
#include "checks.h"
#include "profile.h"

#ifdef JSPOT
  #include "node.h"
  #include "statistics.h"
#endif

void histogram(void);
void w_dump(const char *fmt, ... );

#ifdef JAVA_PROFILE

static w_word cl = 0;

static x_boolean statistics_timings_callback(void * mem, void * arg) {
  w_fifo       fifo;
  w_clazz      clazz = NULL;
  w_method     method = NULL;
  w_word       i;
  w_object     object = chunk2object(mem);
  w_long       external_time;
  w_long       instances;

  if(isAssignmentCompatible(object->clazz, clazzClassLoader)) {

    fifo = ht_list_values(loader2loaded_classes((w_instance)object->fields));

    w_dump("TL%03d: %w\n", cl, object->clazz->dotified);

    while((clazz = getFifo(fifo)) != NULL) {

      for(i = 0; i < clazz->numDeclaredMethods; i++) {

        method = &clazz->own_methods[i];

        external_time = 0;
        instances = 0;

        if(method->exec.callData) {
          w_int tableEntry = 0;
          w_profileMethodCallData data;
          while(tableEntry < METHOD_CALL_TABLE_SIZE) {
            data = ((w_profileMethodCallData *)method->exec.callData)[tableEntry];
            while(data) {
              external_time += (data->child->exec.runtime * data->count / data->child->exec.runs);
              data = data->next;
            }
            tableEntry++;
          }
        }
        if(method->exec.instanceData) {
          w_int tableEntry = 0;
          w_profileMethodInstanceData data;
          while(tableEntry < METHOD_INSTANCE_TABLE_SIZE) {
            data = ((w_profileMethodInstanceData *)method->exec.instanceData)[tableEntry];
            while(data) {
              instances += data->count;
              data = data->next;
            }
            tableEntry++;
          }
        }

        external_time = method->exec.runtime - external_time;
        if(external_time < 0) external_time = 0;

        if (!isSet(method->flags, ACC_ABSTRACT | ACC_NATIVE | METHOD_IS_COMPILED)) {
          if(method->exec.runs && method->exec.bytecodes && !method->exec.checked) {

            w_dump("TD%03d: %8d |%14lld %6lld |%11lld %10lld |%11lld %10lld |%11lld %10lld |%8lld %6lld |  J | %w.%m\n", cl,
                method->exec.runs, method->exec.bytecodes, method->exec.bytecodes / method->exec.runs, 
                method->exec.totaltime, method->exec.totaltime / method->exec.runs, 
                method->exec.runtime, method->exec.runtime / method->exec.runs,
                external_time, external_time / method->exec.runs, instances, instances / method->exec.runs,
                method->spec.declaring_clazz->dotified, method);
            method->exec.checked = 1;
          }
        }
        else if (isSet(method->flags, METHOD_IS_COMPILED)) {
          if(method->exec.runs) {

            w_dump("TD%03d: %8d |%14lld %6lld |%11lld %10lld |%11lld %10lld |%11lld %10lld |%8lld %6lld |  C | %w.%m\n", cl,
                method->exec.runs, 0LL, 0LL, 
                method->exec.totaltime, method->exec.totaltime / method->exec.runs, 
                method->exec.runtime, method->exec.runtime / method->exec.runs, 
                external_time, external_time / method->exec.runs, instances, instances / method->exec.runs,
                method->spec.declaring_clazz->dotified, method);
            method->exec.checked = 1;
          }
        }
        else if (isSet(method->flags, ACC_NATIVE)) {
          if(method->exec.runs) {
            w_dump("TD%03d: %8d |%14lld %6lld |%11lld %10lld |%11lld %10lld |%11lld %10lld |%8lld %6lld |  N | %w.%m\n", cl,
                method->exec.runs, 0LL, 0LL, 
                method->exec.totaltime, method->exec.totaltime / method->exec.runs, 
                method->exec.runtime, method->exec.runtime / method->exec.runs, 
                external_time, external_time / method->exec.runs, instances, instances / method->exec.runs,
                method->spec.declaring_clazz->dotified, method);
            method->exec.checked = 1;
          }
        }
        
        if(method->exec.callData) {
          w_int tableEntry = 0;
          w_profileMethodCallData data;
          while(tableEntry < METHOD_CALL_TABLE_SIZE) {
            data = ((w_profileMethodCallData *)method->exec.callData)[tableEntry];
            while(data) {
              w_dump("TD%03dc: %10d (%8d %11lld) -> %11lld %w.%m\n", cl, data->count, 
                  data->child->exec.runs, data->child->exec.runtime,
                  (data->child->exec.runtime * data->count / data->child->exec.runs),
                  data->child->spec.declaring_clazz->dotified, data->child);
              data = data->next;
            }
            tableEntry++;
          }
        }
        if(method->exec.instanceData) {
          w_int tableEntry = 0;
          w_profileMethodInstanceData data;
          while(tableEntry < METHOD_INSTANCE_TABLE_SIZE) {
            data = ((w_profileMethodInstanceData *)method->exec.instanceData)[tableEntry];
            while(data) {
              w_dump("TD%03di: %10d of %8lld -> %3lld%% %3lld%% %w\n", cl, data->count,
                  data->clazz->total_instances, (data->count * 100) / data->clazz->total_instances, 
                  (data->count * 100) / instances, data->clazz->dotified);
              data = data->next;
            }
            tableEntry++;
          }
        }
        
      }
    }

    cl++;

    releaseFifo(fifo);
  }

  return true;
}

static void statistics_timings(void) {
  w_dump("XX: - Profile data : \nXX:\n");
  w_dump("XX:        Runs | Total bytecds    Avg | Total time        Avg |   Run time        Avg | Non-acc time      Avg | Instances  Avg |Type| Method\n");

  cl = 0;

  x_mem_lock(x_eternal);
  x_mem_scan(x_eternal, OBJECT_TAG, statistics_timings_callback, NULL);
  x_mem_unlock();

  w_dump("XX:\n\n");
}

static x_boolean statistics_instances_callback(void * mem, void * arg) {
  w_fifo       fifo;
  w_clazz      clazz = NULL;
  w_object     object = chunk2object(mem);

  if(isAssignmentCompatible(object->clazz, clazzClassLoader)) {

    fifo = ht_list_values(loader2loaded_classes((w_instance)object->fields));

    w_dump("IL%03d: %w\n", cl, object->clazz->dotified);

    while((clazz = getFifo(fifo)) != NULL) {
      if(clazz->total_instances) {
        w_dump("ID%03d: %8lld |%7lld |%9lld |%5d | %w\n", cl,
            clazz->total_instances, 
            clazz->max_instances, clazz->max_instances * clazz->bytes_needed,
            clazz->bytes_needed, clazz->dotified);
      }
    }

    cl++;

    releaseFifo(fifo);
  } 

  return true;
}

static void statistics_instances(void) {
  w_dump("XX:       Total |    Max | Tot. mem |  Mem | Class\n");

  cl = 0;
  
  x_mem_lock(x_eternal);
  x_mem_scan(x_eternal, OBJECT_TAG, statistics_instances_callback, NULL);
  x_mem_unlock();

  w_dump("XX:\n\n");
}

#endif

void Runtime_static_exit0 (JNIEnv* env, w_instance thisClass, w_int exitcode) {

#ifdef JAVA_PROFILE
  statistics_timings();
  statistics_instances();
#endif

#ifdef JSPOT
  statistics_dump();
#endif
 
//  histogram();
  
#if defined(LINUX) || defined(NETBSD)
  exit(exitcode);
#else
  wabort(ABORT_WONKA, "exit(%d)\n",exitcode);
#endif
}

typedef void (*w_loadlib)(w_thread,w_instance);

extern int woempa_stderr;

void Runtime_loadLibrary0 (JNIEnv* env, w_instance thisRuntime, w_instance libnameString, w_instance libpathString) {

  w_thread   thread = JNIEnv2w_thread(env);
  w_string   libname = NULL;
  w_string   libpath = NULL;
  w_ubyte *  name = NULL;
  w_ubyte *  path = NULL;
  w_int      i;
  w_int      foo = woempa_stderr;

  woempa_stderr = 1;
  
  if(libnameString) {
    libname = String2string(libnameString);
    i = string_length(libname) * 3 + 1;
    name = allocMem_system(i);
    x_snprintf(name, i, "%w", libname);
  }
  
  if(libpathString) {
    libpath = String2string(libpathString);
    i = string_length(libpath) * 3 + 1;
    path = allocMem_system(i);
    x_snprintf(path, i, "%w", libpath);
  }

  if (loadModule(name, path)) {
    woempa(7, "Successfully loaded library %w / %w\n", libname, libpath);
  }
  else {
    woempa(9, "Could not find library %w / %w (%s)\n", libname, libpath, name);
    throwException(thread, clazzLinkageError, "%w / %w (%s)", libname, libpath, name);
  }

  if(name) releaseMem(name);
  if(path) releaseMem(path);

  woempa_stderr = foo;
}

w_long
Runtime_totalMemory (JNIEnv* env, w_instance thisRuntime) {
  return (w_long)(x_mem_total());
}

w_long
Runtime_freeMemory (JNIEnv* env, w_instance thisRuntime) {
  return (w_long)(x_mem_avail());
}


static w_void string2c(w_string string, char *c_string) {
  if (string_length(string) > 0) {
    char *dummy = (char *)c_string;
    int i = 0;
        
    for (i = 0; i < (w_int)string_length(string); ++i) {
       (*dummy) = string_char(string, i);
       dummy++;
    }
    (*dummy) = '\0';
  }
  else {
    *c_string = '\0';
  }
}

w_instance Runtime_exec0 (JNIEnv* jnienv, w_instance thisRuntime, w_instance cmdArray, w_instance envArray, w_instance pathString) {
  w_thread    thread = JNIEnv2w_thread(jnienv);
  w_int       cmdlength = 0; 
  w_int       envlength = 0; 
  w_int       i, len;
  w_ubyte     **cmd = NULL;
  w_ubyte     **env = NULL;
  w_ubyte     *path = NULL;
  w_string    string;
  w_instance  process = nullInstance;
  w_int       pid, fd_in, fd_out, fd_err;

  /*
  ** Check if there's a command to execute.
  */
  
  if(cmdArray == NULL || instance2Array_length(cmdArray) == 0) {
    return process;
  }

  /*
  ** Turn the cmd String array into a plain old char array (Last element should be NULL).
  */

  cmdlength = instance2Array_length(cmdArray);
  cmd = x_mem_calloc((cmdlength + 1) * sizeof(w_word));
  for(i = 0; i < cmdlength; i++) {
    string = String2string(instance2Array_instance(cmdArray)[i]);
    if(string == NULL) {
      throwException(thread, clazzNullPointerException, "");
      for(i--; i >= 0; i--) x_mem_free(cmd[i]);
      return NULL;
    }
    cmd[i] = x_mem_alloc(string_length(string) + 2);
    string2c(string, cmd[i]);
  }

  /*
  ** Turn the env String array into a plain old char array (Last element should be NULL).
  ** If envArray is NULL, there's no environment.
  */
  
  if(envArray != NULL) {
    envlength = instance2Array_length(envArray);
    env = x_mem_calloc((envlength + 1) * sizeof(w_word));
    for(i = 0; i < envlength; i++) {
      string = String2string(instance2Array_instance(envArray)[i]);
      env[i] = x_mem_alloc(string_length(string) + 2);
      string2c(string, env[i]);
    }
  }

  /*
  ** Get the working direcory (if there's one).
  */

  if(pathString != NULL) {
    string = String2string(pathString); 
    path = string2UTF8(string, &len);
    path += 2;
  }

  /*
  ** Instantiate a Process object.
  */

  process = allocInstance(thread, clazzNativeProcess);
  
  /*
  ** Call host_exec.
  */

  pid = host_exec(cmd, env, path, &fd_in, &fd_out, &fd_err);

  if(pid > 0) {
    setIntegerField(process, F_NativeProcess_process_id, pid);
    setIntegerField(process, F_NativeProcess_fd_in, fd_in);
    setIntegerField(process, F_NativeProcess_fd_out, fd_out);
    setIntegerField(process, F_NativeProcess_fd_err, fd_err);
    setIntegerField(process, F_NativeProcess_returnvalue, 10000);
  }
  else {
    throwException(thread, clazzIOException, "%s: %s", cmd[0], strerror(-pid));
  }

  /*
  ** Clean up.
  */
  
  for(i = 0; i < cmdlength; i++) x_mem_free(cmd[i]);
  for(i = 0; i < envlength; i++) x_mem_free(env[i]);
  if(cmd != NULL) x_mem_free(cmd);
  if(env != NULL) x_mem_free(env);
  if(path != NULL) releaseMem((char *)(path - 2));

  /*
  ** Return the Process instance.
  */
  
  return process;
}
