/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, bus 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: String.c 4294 2006-05-10 11:35:19Z chris $
*/

#include <string.h>
#include "arrays.h"
#include "chars.h"
#include "checks.h"
#include "core-classes.h"
#include "clazz.h"
#include "exception.h"
#include "loading.h"
#include "Math.h"
#include "descriptor.h"
#include "fields.h"
#include "heap.h"
#include "ts-mem.h"
#include "methods.h"
#include "realms.h"
#include "wstrings.h"
#include "threads.h"
#include "chars.h"
#include "ieee754.h"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

/*
** Create an empty string
*/

void String_create_empty(JNIEnv *env, w_instance String) {
  
  setWotsitField(String, F_String_wotsit, cstring2String(W_Thread_system,"",0));

  woempa(1, "%p new string at %p, empty\n", String, (char *)(String2string(String)));
    
}

/*
** Create a string instance just like an existing string
*/

void String_create_String(JNIEnv *env, w_instance String, w_instance Value) {

  if (! Value) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
  } 
  else {
    setWotsitField(String, F_String_wotsit, registerString(String2string(Value)));
  }

}

/*
** Create a string instance from a StringBuffer
*/

void String_create_StringBuffer(JNIEnv *env, w_instance String, w_instance value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string string;

  if (isNullInstance(value)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    string = unicode2String(thread, instance2Array_char(getReferenceField(value, F_StringBuffer_value)), getIntegerField(value, F_StringBuffer_count));
    if (! string->instance) {
      string->instance = String;
    }
    setWotsitField(String, F_String_wotsit, string);
  }

}

/*
** Create a string instance from an array of Characters
*/

void String_create_char(JNIEnv *env, w_instance String, w_instance charArray, w_int offset, w_int count) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length;
  w_string string;
  
  if (isNullInstance(charArray)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    length = instance2Array_length(charArray);
    if (offset < 0 || count < 0 || count + offset > length) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, "offset %d, count %d, length %d", offset, count, length);
    } 
    else {
      string = unicode2String(thread, instance2Array_char(charArray) + offset, (w_size)count);
      if (! string->instance) {
        string->instance = String;
      }
      setWotsitField(String, F_String_wotsit, string);
    }
  }

}

/*
** Create a string instance from an array of ASCII bytes. This is used in some constructors of String
** so the string is freshly created and doesn't have a wotsit set yet.
*/

void String_create_byte(JNIEnv *env, w_instance String, w_instance byteArray, w_int hibyte, w_int offset, w_int count) {

  w_thread  thread = JNIEnv2w_thread(env);
  w_string  string;
  w_sbyte * buffer;
  w_char *  charbuff;
  w_int     length;
  w_byte *  from;
  w_char *  to;
  w_int     i;

  if (isNullInstance(byteArray)) {
    throwException(thread, clazzNullPointerException, NULL);
  } 
  else {
    buffer = instance2Array_byte(byteArray);
    length = instance2Array_length(byteArray);

    if (offset < 0 || count < 0 || count + offset > length) {
      throwException(thread, clazzArrayIndexOutOfBoundsException, NULL);
    } 
    else {
      if (hibyte < 0) {
        string = utf2String(thread, buffer + offset, (w_size)count);
        if (! string->instance) {
          string->instance = String;
        }
        setWotsitField(String, F_String_wotsit, string);
      }
      else {
        charbuff = allocMem(thread, count * sizeof(w_char));
        from = buffer + offset;
        to = charbuff;

        if (charbuff) {
          for (i = 0; i < count; ++i) {
            *to++ = (hibyte << 8) | (*from++);
          }
          string = unicode2String(thread, charbuff, (w_size)count);
          if (! string->instance) {
            string->instance = String;
          }
          setWotsitField(String, F_String_wotsit, string);
          releaseMem(charbuff);
        }
      }
    }
  }

}

void String_destructor(w_instance this) {
  w_string string;

  x_mutex_lock(string_mutex, x_eternal);
  string = String2string(this);

  if (string) {
    // Set a tell-tale value to help detect attempts to access a stale wotsit
    setWotsitField(this, F_String_wotsit, 0x12345678);
    if (string->instance == this) {	     
      string->instance = NULL;	 
    }
    deregisterString(string);
  }
  x_mutex_unlock(string_mutex);
  
}

/*
** Vacuous.
*/

w_instance String_toString(JNIEnv *env, w_instance theString) { 

  return theString;

}

w_instance String_toCharArray(JNIEnv *env, w_instance This) {

  w_word length;
  w_string this = String2string(This);
  w_instance result;
  
  length = (w_word)string_length(this);
  result = allocArrayInstance(JNIEnv2w_thread(env), atype2clazz[P_char], 1, &length);
#ifdef BYTEWISELATIN
  if (result) {
    if (string_is_latin1(this)) {
      w_size i;
      for (i = 0; i < string_length(this); ++i) {
        instance2Array_char(result)[i] = (w_char)this->contents.bytes[i];
      }
    }
    else {
      w_memcpy(instance2Array_char(result), this->contents.chars, string_length(this) * sizeof(w_char));
    }
  }
#else
  if (result) {
    w_memcpy(instance2Array_char(result), this->chars, string_length(this) * sizeof(w_char));
  }
#endif

  return result;
  
}

w_int String_compareTo(JNIEnv *env, w_instance This, w_instance String) {

  w_string this = String2string(This);
  w_string string;
  w_int result = 0;
  w_int i;

  if (!String) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
  }
  else {
    string = String2string(String);
    if (string_length(this) == 0) {
      result -= string_length(string);
    }
    else if (string_length(string) == 0) {
      result = string_length(this);
    }
    else if (string != this) {
      for (i = 0; i < (w_int)string_length(this); i++) {

        /*
        ** Calculate possible result based on character compare.
        */
        
        if (string_char(this, i) != string_char(string, i)) {
          result = (w_int)(string_char(this, i) - string_char(string, i));
          break;
        }

        /*
        ** Check that String and This don't differ in length only, otherwise calculate length difference.
        */
        
        if ((i + 1) >= (w_int)string_length(string) || (i + 1) >= (w_int)string_length(this)) {
          result = string_length(this) - string_length(string);
          break;
        }

      }
    }
  }

  return result;
  
}

w_instance String_getBytes(JNIEnv *env, w_instance This, w_boolean Enc) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance result = NULL;
  w_string this = String2string(This);
  w_byte *dst;
  w_int i;

  if (Enc == WONKA_TRUE){
    w_byte * utfbytes = string2UTF8(this, &i);

    if (utfbytes) {
      i -= 2;
      result = allocArrayInstance(thread, atype2clazz[P_byte], 1, (w_word *)&i);
      if (result) {
        w_byte * utfsrc = utfbytes + 2;// skip the two first bytes (the length)
        w_int k=0;
        dst = instance2Array_byte(result);
        for( ; k < i ; k++){
          *dst++ = *utfsrc++;
        }
      }
      releaseMem(utfbytes);
    }
  }
  else {
    w_word   length = (w_word)string_length(this);

    result = allocArrayInstance(thread, atype2clazz[P_byte], 1, &length);
    if (result) {
      dst = instance2Array_byte(result);
      for (i = 0; i < (w_int)string_length(this); i++) {
        *dst++ = (w_byte)string_char(this, i);
      }
    }
  }

  return result;

}

/*
** Compare two strings for equality
*/

w_boolean String_equals(JNIEnv *env, w_instance thisString, w_instance otherString) {

  if (! otherString || instance2clazz(otherString) != clazzString) {
    return WONKA_FALSE;
  }
  else {
    woempa(1, "Comparing %w to %w.\n", String2string(thisString), String2string(otherString));
    return String2string(thisString) == String2string(otherString);
  }

}

/*
** Compare two strings for equality modulo case
*/

w_boolean String_equalsIgnoreCase(JNIEnv *env, w_instance This, w_instance Other) {

  w_boolean result = WONKA_FALSE;
  w_string this;
  w_string other;
  w_int i;

  /*
  ** First check wether our arguments are not null and are of class String or a subclass, 
  ** if that checks out fine, we go further...
  */

  if (! isNullInstance(Other) && isSuperClass(clazzString, instance2clazz(Other))) {
    this = String2string(This);
    other = String2string(Other);
    /*
    ** Do a quick pointer check first.
    */
    if (this == other) {
      result = WONKA_TRUE;
    }
    else if (string_length(this) == string_length(other)) {
      /*
      ** Preset result to TRUE and compare character by character; when we hit an unequal character,
      ** we set result to FALSE and break...
      */
      result = WONKA_TRUE;
      for (i = 0; i < (w_int)string_length(this); i++) {
        w_char this_char = string_char(this, i);
        w_char that_char = string_char(other, i);
        if ( this_char != that_char && (char2lower(this_char) != char2lower(that_char)) && (char2upper(this_char) != char2upper(that_char))) {
          result = WONKA_FALSE;
          break;
        }
      }
    }
  }
  
  return result;

}

/*
** Get the hashcode of a string. 
*/

w_int String_hashCode(JNIEnv *env, w_instance thisString) {
  w_string string = String2string(thisString);
  w_int hash = getIntegerField(thisString, F_String_hashCode);
  w_int i;
  w_int length;
  w_int duffs;

  if(!hash){
    length = string_length(string);
    if(length){
      duffs = (length + 31) / 32;
      i=0;
      switch(length & 0x1f){
          case  0: do {
                     hash = hash*31 + string_char(string, i++);
          case 31:   hash = hash*31 + string_char(string, i++);
          case 30:   hash = hash*31 + string_char(string, i++);
          case 29:   hash = hash*31 + string_char(string, i++);
          case 28:   hash = hash*31 + string_char(string, i++);
          case 27:   hash = hash*31 + string_char(string, i++);
          case 26:   hash = hash*31 + string_char(string, i++);
          case 25:   hash = hash*31 + string_char(string, i++);
          case 24:   hash = hash*31 + string_char(string, i++);
          case 23:   hash = hash*31 + string_char(string, i++);
          case 22:   hash = hash*31 + string_char(string, i++);
          case 21:   hash = hash*31 + string_char(string, i++);
          case 20:   hash = hash*31 + string_char(string, i++);
          case 19:   hash = hash*31 + string_char(string, i++);
          case 18:   hash = hash*31 + string_char(string, i++);
          case 17:   hash = hash*31 + string_char(string, i++);
          case 16:   hash = hash*31 + string_char(string, i++);
          case 15:   hash = hash*31 + string_char(string, i++);
          case 14:   hash = hash*31 + string_char(string, i++);
          case 13:   hash = hash*31 + string_char(string, i++);
          case 12:   hash = hash*31 + string_char(string, i++);
          case 11:   hash = hash*31 + string_char(string, i++);
          case 10:   hash = hash*31 + string_char(string, i++);
          case  9:   hash = hash*31 + string_char(string, i++);
          case  8:   hash = hash*31 + string_char(string, i++);
          case  7:   hash = hash*31 + string_char(string, i++);
          case  6:   hash = hash*31 + string_char(string, i++);
          case  5:   hash = hash*31 + string_char(string, i++);
          case  4:   hash = hash*31 + string_char(string, i++);
          case  3:   hash = hash*31 + string_char(string, i++);
          case  2:   hash = hash*31 + string_char(string, i++);
          case  1:   hash = hash*31 + string_char(string, i++);

        } while(--duffs > 0);
      }
      setIntegerField(thisString, F_String_hashCode, hash);
    }
  }
  return hash;
}

/*
** Length in unicode characters
*/

w_int String_length(JNIEnv *env, w_instance String) {

  return string_length(String2string(String));

}

w_boolean String_regionMatches(JNIEnv *env, w_instance This, w_boolean ic, w_int to, w_instance Other, w_int oo, w_int len) {
  w_thread thread = JNIEnv2w_thread(env);
  w_boolean result = WONKA_TRUE;
  w_string this = String2string(This);
  w_string other;
  w_int i;
  
  if (! isNullInstance(Other)) {
    other = String2string(Other);
    if (to < 0 || oo < 0 || to + len > (w_int)string_length(this) || oo + len > (w_int)string_length(other)) {
      result = WONKA_FALSE;
    }
    else {
      if (ic == WONKA_FALSE) {
        for (i = 0; i < len; i++) {
          if (string_char(this, i + to) != string_char(other, oo + i)) {
            result = WONKA_FALSE;
            break;
          }
        }
      }
      else {
        for (i = 0; i < len; i++) {
          w_char this_char = string_char(this, i + to);
          w_char that_char = string_char(other, oo + i);
          if ((char2upper(this_char) != char2upper(that_char)) && (char2lower(this_char) != char2lower(that_char))) {
            result = WONKA_FALSE;
            break;
          }
        }
      }
    }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }
  
  return result;
  
}

w_boolean String_startsWith(JNIEnv *env, w_instance This, w_instance Prefix, w_int offset) {

  w_string this;
  w_string prefix;
  w_size i;

  if (!Prefix) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
    return FALSE;
  }

  prefix = String2string(Prefix);

  if (string_length(prefix) == 0) {
    return TRUE;
  }

  this = String2string(This);

  if (prefix == this && offset == 0) {
    return TRUE;
  }

  if (offset < (w_int)string_length(this) && string_length(prefix) <= (string_length(this) - offset)) {
    for (i = 0; i < string_length(prefix); ++i) {
      if (string_char(this, offset + i) != string_char(prefix, i)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  return FALSE;

}

w_boolean String_endsWith(JNIEnv *env, w_instance This, w_instance Suffix) {

  w_string this;
  w_string suffix;
  w_int offset;
  w_size i;
  
  if (!Suffix) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, NULL);
    return FALSE;
  }

  this = String2string(This);
  suffix = String2string(Suffix);
  offset = string_length(this) - string_length(suffix);

  if (string_length(suffix) == 0 ) {
    return TRUE;
  }

  if (suffix == this) {
    return TRUE;
  }

  if (offset > 0) {
    for (i = 0; i < string_length(suffix); ++i) {
      if (string_char(this, offset + i) != string_char(suffix, i)) {
        return FALSE;
      }
    }
    return TRUE;
  }

  return FALSE;  

}

/*
** Get one character of a string.  The characters are numbered from 0.
*/

w_char String_charAt(JNIEnv *env, w_instance thisString, w_int Index) {

  w_string string = String2string(thisString);

  if (Index < 0 || Index >= (w_int)string_length(string)) {
    throwException(JNIEnv2w_thread(env), clazzStringIndexOutOfBoundsException, NULL);
    return 0x0000;
  }
  
  return string_char(string, Index); 
 
}

void String_getChars(JNIEnv *env, w_instance This, w_int srcBegin, w_int srcEnd, w_instance Dst, w_int dstBegin) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_char *dst;
  w_int length;

  this = String2string(This);
  if (Dst == NULL) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (srcBegin < 0 || srcBegin > srcEnd || srcEnd > (w_int)string_length(this) || dstBegin < 0 || dstBegin + srcEnd - srcBegin > instance2Array_length(Dst)) {
    throwException(thread, clazzIndexOutOfBoundsException, NULL);
  }
  else {
    if ((srcEnd - srcBegin) > 0) {
      dst = instance2Array_char(Dst) + dstBegin;
      length = srcEnd - srcBegin;
#ifdef BYTEWISELATIN
      if (string_is_latin1(this)) {
        w_int i;
        for (i = srcBegin; i < srcEnd; ++i) {
          dst[i - srcBegin] = (w_char)this->contents.bytes[i];
        }
      }
      else {
        w_memcpy(dst, this->contents.chars + srcBegin, length * sizeof(w_char));
      }
#else
      w_memcpy(dst, this->chars + srcBegin, length * sizeof(w_char));
#endif
    }
  }
  
} 


void String_copyBytes(JNIEnv *env, w_instance Source, w_int srcBegin, w_int srcEnd, w_instance dstByteArray, w_int dstBegin) {
  w_string source = String2string(Source);
  w_sbyte *destination;
  w_int length;
  w_int i;
  
  destination = instance2Array_byte(dstByteArray);
  destination += dstBegin;
  length = srcEnd - srcBegin;
  
#ifdef BYTEWISELATIN
  if (string_is_latin1(source)) {
    w_memcpy(destination, source->contents.bytes + srcBegin, length * sizeof(w_sbyte));
  }
  else {
    for (i = 0; i < length; i++) {
      *destination++ = (w_byte)string_char(source, i);
    }
  }
#else
  for (i = 0; i < length; i++) {
    *destination++ = (w_byte)string_char(source, i);
  }
#endif

}


w_int String_indexOf_char(JNIEnv *env, w_instance thisString, w_int ch, w_int offset) {
  w_string string;
  w_int result = -1;
  w_int length;

  string = String2string(thisString);
  length = string_length(string);
  offset = offset < 0 ? 0 : offset;
  woempa(1, "Searching %slatin1 string '%w' for %d starting at offset %d\n", string_is_latin1(string) ? "" : "non-", string, ch, offset);

  for (  ; offset < length; offset++) {
    w_char cx = string_char(string, offset);
    woempa(1, "Char at offset %d is %d\n", offset, cx);
    if ((w_char)ch == cx) {
      result = offset;
      break;
    }
  }

  return result;
  
}

w_int String_indexOf_String(JNIEnv *env, w_instance thisString, w_instance otherString, w_int fromIndex) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_string other;
  w_int where;
  w_int max;
  w_int result = -1;
  w_size length;
#ifdef BYTEWISELATIN
  w_boolean this_is_latin1;
  w_boolean other_is_latin1;
  w_int s;
  w_int o;
#endif

  if (isNullInstance(otherString)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else {
    string = String2string(thisString);
    other = String2string(otherString);
    length = string_length(other);
#ifdef BYTEWISELATIN
    this_is_latin1 = string_is_latin1(string);
    other_is_latin1 = string_is_latin1(other);
#endif
    woempa(1,"findstring '%w' in '%w' starting at %d\n",other,string,fromIndex);
    
    if (string_length(other) == 0) {
    	return  fromIndex;
    }

    if (fromIndex < (w_int)string_length(string)) {
      where = fromIndex<0?0:fromIndex;
      max = string_length(string) - string_length(other);
      while (where <= max) {
        woempa(1,"trying offset %d\n",where);
#ifdef BYTEWISELATIN
        if (this_is_latin1 && other_is_latin1) {
          if (memcmp(string->contents.bytes + where, other->contents.bytes, length * sizeof(w_ubyte)) == 0) {
            woempa(1,"match at offset %d\n",where);

            return where;

          }
        }
        else if (!this_is_latin1 && !other_is_latin1) {
          if (memcmp(string->contents.chars + where, other->contents.chars, length * sizeof(w_char)) == 0) {
            woempa(1,"match at offset %d\n",where);

            return where;

          }
        }
        else {
          for (s = where, o = 0; o < (w_int)string_length(other); o++, s++) {
            woempa(1, "A = %d, B = %d\n", string_char(string, s), string_char(other, o)); 
            if (string_char(string, s) != string_char(other, o)) {
              woempa(1,"mismatch after %d chars\n",o);
              break;
            }
          }
          if (o == (w_int)string_length(other)) {
            woempa(1,"match at offset %d\n",where);

            return where;

          }
        }
#else
        if (memcmp(string->chars + where, other->chars, length * sizeof(w_char)) == 0) {
          woempa(1,"match at offset %d\n",where);

          return where;

        }
#endif
        where += 1;
      }
    }
  }
  
  woempa(1, "No match found.\n");

  return result;

}

w_int String_lastIndexOf_char(JNIEnv *env, w_instance thisString, w_int ch, w_int offset) {
  w_string string = String2string(thisString);
  w_int i;
  w_int result = -1;

  string = String2string(thisString);

  if (0 <= offset) {
    offset = (offset < (w_int)string_length(string)) ? offset : (w_int)string_length(string) - 1;
    for (i = offset; i >= 0; i--) {
      if (ch == (w_int)string_char(string, i)) {
        result = i;
        break;
      }
    }
  }

  woempa(1, "Looked for last index of '%c', offset %d in %w, result %d.\n", (char)ch, offset, string, result);

  return result;
  
}

w_int String_lastIndexOf_String(JNIEnv *env, w_instance thisString, w_instance otherString, w_int offset) {
  w_thread thread = JNIEnv2w_thread(env);
  w_string string;
  w_string other;
  w_int where;
  w_int max;
  w_int s;
  w_int o;
  w_int result = -1;

  if (isNullInstance(otherString)) {
    throwException(thread, clazzNullPointerException, NULL);
  }
  else if (0 <= offset) {
    string = String2string(thisString);
    other = String2string(otherString);
    woempa(1, "%w <-> %w starting at %d.\n", other, string, offset);
    offset = (offset < (w_int)string_length(string)) ? offset : (w_int)string_length(string);
    max = string_length(string) - string_length(other);
    if (string_length(other) == 0) {
    	return  offset;
    }
    if (max < 0) return -1;
    where = ( max < offset ?  max : offset);
    while (where+1) {
      woempa(1, "trying offset %d.\n", where);
      for (s = where, o = 0; o < (w_int)string_length(other); o++, s++) {
        if (string_char(string, s) != string_char(other, o)) {
          woempa(1, "mismatch after %d chars\n", o);
          break;
        }
      }
      if (o == (w_int)string_length(other)) {
        result = where;
        break;
      }
      where -= 1;
    }

    woempa(1, "Result: %w <-> %w starting at %d, result %d.\n", other, string, offset, result);

  }
  
  return result;

}

w_instance String_toUpperCase(JNIEnv *env, w_instance This, w_instance Locale) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_instance result = This;
  w_int i;
  w_char *buffer;
  w_char *dst;
  w_string string;
  
  buffer = allocMem(thread, string_length(this) * sizeof(w_char));
  if (buffer) {
    dst = buffer;
    for (i = 0; i < (w_int)string_length(this); i++) {
      *dst++ = char2upper(string_char(this, i));
    }
    string = unicode2String(thread, buffer, string_length(this));
    result = newStringInstance(string);
  // string is now registered twice, which is once too many
    deregisterString(string);
    releaseMem(buffer);
  }
    
  return result;

}

w_instance String_toLowerCase(JNIEnv *env, w_instance This, w_instance Locale) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_instance result = This;
  w_int i;
  w_char * buffer;
  w_char * dst;
  w_string string;
  
  buffer = allocMem(thread, string_length(this) * sizeof(w_char));
  if (buffer) {
    dst = buffer;
    for (i = 0; i < (w_int)string_length(this); i++) {
      *dst++ = char2lower(string_char(this, i));
    }
    string = unicode2String(thread, buffer, string_length(this));
    result = newStringInstance(string);
    releaseMem(buffer);
    deregisterString(string);
  }
    
  return result;

}

w_instance String_replace(JNIEnv *env, w_instance This, w_char oldChar, w_char newChar) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_instance Result = This;
  w_string result;
  w_char * buffer;
  w_int i;

  if (isNullInstance(This)) {
    throwException(thread, clazzNullPointerException, NULL);

    return This;

  }

  this = String2string(This);
  buffer = allocMem(thread, string_length(this) * sizeof(w_char));
  if (buffer) {
    for (i = 0; i < (w_int)string_length(this); i++) {
      w_char ch = string_char(this, i);
      if (ch == oldChar) {
        buffer[i] = newChar;
      }
      else {
        buffer[i] = ch;
      }
    }
    result = unicode2String(thread, buffer, string_length(this));
    Result = newStringInstance(result);
    deregisterString(result);
    releaseMem(buffer);
  }

  return Result;

}

w_instance String_concat(JNIEnv *env, w_instance This, w_instance String) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_string string;
  w_instance Result = NULL;
  w_string result;
  w_char * buffer;

  if (isNullInstance(String)) {
    throwException(thread, clazzNullPointerException, NULL);

    return NULL;

  }

  this = String2string(This);
  string = String2string(String);
  if (string_length(string)) {
    buffer = allocMem(thread, (string_length(string) + string_length(this)) * sizeof(w_char));
    if (buffer) {
#ifdef BYTEWISELATIN
      if (string_is_latin1(this)) {
        w_size i;
        for (i = 0; i < string_length(this); ++i) {
          buffer[i] = (w_char)this->contents.bytes[i];
        }
      }
      else {
        w_memcpy(buffer, this->contents.chars, string_length(this) * sizeof(w_char));
      }

      if (string_is_latin1(string)) {
        w_size i;
        for (i = 0; i < string_length(string); ++i) {
          buffer[i + string_length(this)] = (w_char)string->contents.bytes[i];
        }
      }
      else {
        w_memcpy(buffer + string_length(this), string->contents.chars, string_length(string) * sizeof(w_char));
      }
#else
      w_memcpy(buffer + string_length(this), string->chars, string_length(string) * sizeof(w_char));
      w_memcpy(buffer, this->chars, string_length(this) * sizeof(w_char));
#endif
      result = unicode2String(thread, buffer, string_length(this) + string_length(string));
      Result = newStringInstance(result);
      deregisterString(result);
      releaseMem(buffer);
    }
  }
  else {
    Result = This;
  }
  
  return Result;

}

w_instance String_substring(JNIEnv *env, w_instance This, w_int offset, int endIndex) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this;
  w_instance SubString = nullInstance;
  w_string subString = NULL;
  w_char  *buffer;
  w_size   length;

  this = String2string(This);
  length = string_length(this);
  woempa(1, "String '%w' (length %d) offset %d endIndex %d\n", this, length, offset, endIndex);

  if (offset < 0 || offset > (w_int)length || offset > endIndex || endIndex > (w_int)length) {

    throwException(thread, clazzIndexOutOfBoundsException, NULL);

  }
  else {
    buffer = allocMem(thread, (endIndex - offset) * sizeof(w_char));
    if (buffer) {
#ifdef BYTEWISELATIN
      if (string_is_latin1(this)) {
        w_int i;
        for (i = offset; i < endIndex; ++i) {
          buffer[i - offset] = (w_char)this->contents.bytes[i];
        }
      }
      else {
        memcpy(buffer, this->contents.chars + offset, (endIndex - offset) * sizeof(w_char));
      }
#else
      memcpy(buffer, this->chars + offset, (endIndex - offset) * sizeof(w_char));
#endif
      subString = unicode2String(thread, buffer, (w_size)(endIndex - offset));
      releaseMem(buffer);
    }
    SubString = newStringInstance(subString);
    if(SubString) deregisterString(subString);
    woempa(1, "Result is %w\n", subString);
  }

  return SubString;

}

/*
**
*/

w_instance String_trim(JNIEnv *env, w_instance This) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string this = String2string(This);
  w_instance Result = This;
  w_string result;
  w_size i;
  w_int leading;
  w_int trailing;
  w_char * buffer = NULL;

  /*
  ** First check if we have leading and trailing whitespace and a string which is
  ** not empty.
  */

  if (string_length(this) > 0 && (string_char(this, 0) <= 0x0020 || string_char(this, string_length(this) - 1) <= 0x0020)) {
    buffer = allocMem(thread, string_length(this) * sizeof(w_char));
    if (buffer) {
#ifdef BYTEWISELATIN
      if (string_is_latin1(this)) {
        for (i = 0; i < string_length(this); ++i) {
          buffer[i] = (w_char)this->contents.bytes[i];
        }
      }
      else {
        w_memcpy(buffer, this->contents.chars, string_length(this) * sizeof(w_char));
      }
#else
      w_memcpy(buffer, this->chars, string_length(this) * sizeof(w_char));
#endif
      leading = 0;
      trailing = 0;
      for (i = 0; i < string_length(this); i++) {
        if (buffer[i] > 0x0020) {
          break;
        }
        leading += 1;
      }
      for (i = string_length(this) - 1; i >= 0; i--) {
        if (buffer[i] > 0x0020) {
          break;
        }
        trailing += 1;
      }
      if (leading != (w_int)string_length(this)) {
        result = unicode2String(thread, buffer + leading, string_length(this) - trailing - leading);
        Result = newStringInstance(result);
        deregisterString(result);
      }
      else {
        Result = getStringInstance(cstring2String(W_Thread_system,"",0));
      }
      releaseMem(buffer);
    }
  }

  return Result;

}

w_instance String_intern(JNIEnv *env, w_instance thisString) { 

  return getStringInstance(String2string(thisString));

}

w_instance String_static_valueOf_char(JNIEnv *env, w_instance stringClass, w_char c) {
  w_string string;
  w_instance result;

  string = unicode2String(JNIEnv2w_thread(env), &c, 1);
  result = newStringInstance(string);
  // string is now registered twice, which is once too many
  deregisterString(string);
  return result;
}
