/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: StringBuffer.c 3451 2003-02-06 14:25:34Z gray $
*/

#include <string.h>
#include "arrays.h"
#include "chars.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "heap.h"
#include "loading.h"
#include "ts-mem.h"
#include "methods.h"
#include "wstrings.h"
#include "threads.h"
#include "Math.h"

inline static void copyChars(w_char * dst, w_char * src, w_size num) {

  w_size duffs;

  if (num) {
    duffs = (num + 31) / 32;
    switch (num & 0x1f) {
      case  0: do { *dst++ = *src++;
      case 31:      *dst++ = *src++;
      case 30:      *dst++ = *src++;
      case 29:      *dst++ = *src++;
      case 28:      *dst++ = *src++;
      case 27:      *dst++ = *src++;
      case 26:      *dst++ = *src++;
      case 25:      *dst++ = *src++;
      case 24:      *dst++ = *src++;
      case 23:      *dst++ = *src++;
      case 22:      *dst++ = *src++;
      case 21:      *dst++ = *src++;
      case 20:      *dst++ = *src++;
      case 19:      *dst++ = *src++;
      case 18:      *dst++ = *src++;
      case 17:      *dst++ = *src++;
      case 16:      *dst++ = *src++;
      case 15:      *dst++ = *src++;
      case 14:      *dst++ = *src++;
      case 13:      *dst++ = *src++;
      case 12:      *dst++ = *src++;
      case 11:      *dst++ = *src++;
      case 10:      *dst++ = *src++;
      case  9:      *dst++ = *src++;
      case  8:      *dst++ = *src++;
      case  7:      *dst++ = *src++;
      case  6:      *dst++ = *src++;
      case  5:      *dst++ = *src++;
      case  4:      *dst++ = *src++;
      case  3:      *dst++ = *src++;
      case  2:      *dst++ = *src++;
      case  1:      *dst++ = *src++;
              } while (--duffs > 0);
    }
  }

}

/*
** Reallocate a buffer of 'size' and copy over the old contents.
** Can only be used to make a buffer larger, not smaller.
*/

inline static void buffersize(w_thread thread, w_instance StringBuffer, w_int size) {

  w_instance newbuffer;
  w_instance oldbuffer = getReferenceField(StringBuffer, F_StringBuffer_value);
  
  woempa(1, "Currently used %d, requested size is %d.\n", getIntegerField(StringBuffer, F_StringBuffer_count), size);
  wassert(getIntegerField(StringBuffer, F_StringBuffer_count) <= size);

  newbuffer = allocArrayInstance(thread, atype2clazz[P_char], 1, &size);
  if (newbuffer) {
    copyChars(instance2Array_char(newbuffer), instance2Array_char(oldbuffer), getIntegerField(StringBuffer, F_StringBuffer_count));
    setReferenceField(StringBuffer, newbuffer, F_StringBuffer_value);
  }

}

void StringBuffer_ensureCapacity(JNIEnv *env, w_instance StringBuffer, w_int minimum) {

  w_instance buffer = getReferenceField(StringBuffer, F_StringBuffer_value);
  w_int oldsize = instance2Array_length(buffer);

  woempa(1, "%j: minimum capacity is %d, current is %d\n", StringBuffer, minimum, oldsize);

  if (minimum > oldsize) {
    woempa(1,"Current buffer has length %d, new buffer will have length %d\n", oldsize, (minimum > 2 * oldsize) ? minimum : 2 * oldsize + 2);
    buffersize(JNIEnv2w_thread(env), StringBuffer, (minimum > 2 * oldsize) ? minimum : 2 * oldsize + 2);
  }

}

void StringBuffer_setLength(JNIEnv *env, w_instance StringBuffer, w_int newLength) {

  w_thread thread = JNIEnv2w_thread(env);
  w_char *dst;

  if (newLength < 0) {
    woempa(1, "Trying to set length <0 (%d)!\n", newLength);
    throwException(thread, clazzIndexOutOfBoundsException, NULL);
  }
  else {
    if (newLength > getIntegerField(StringBuffer, F_StringBuffer_count)) {
      StringBuffer_ensureCapacity(env, StringBuffer, newLength);
      if (exceptionThrown(thread) == NULL) {
        dst = instance2Array_char(getReferenceField(StringBuffer, F_StringBuffer_value));
        dst += getIntegerField(StringBuffer, F_StringBuffer_count);
        memset(dst, 0x00, (newLength - getIntegerField(StringBuffer, F_StringBuffer_count)) * sizeof(w_char));
      }
    }
    setIntegerField(StringBuffer, F_StringBuffer_count, newLength);
  }
}

void StringBuffer_createFromString(JNIEnv *env, w_instance StringBuffer, w_instance String) {

  w_string string;
  w_size length;
  w_instance buffer;
  w_char  *charptr;

  if (String) {  
    string = String2string(String);
    length = string_length(string) + 16;
    buffer = allocArrayInstance(JNIEnv2w_thread(env), atype2clazz[P_char], 1, &length);
    if (!buffer) {

      return;

    }
    charptr = instance2Array_char(buffer);
    length -= 16;

#ifdef BYTEWISELATIN
    if (string_is_latin1(string)) {
      w_size i;
      for (i = 0; i < length; ++i) {
        *charptr++ =  (w_char)string->contents.bytes[i];
      }
    }
    else {
      w_memcpy(charptr, string->contents.chars, length * sizeof(w_char));
    }
#else
    w_memcpy(charptr, string->chars, length * sizeof(w_char));
#endif
    setIntegerField(StringBuffer, F_StringBuffer_count, length);
  }
  else {
    string = NULL;
    length = 16;
    buffer = allocArrayInstance(JNIEnv2w_thread(env), atype2clazz[P_char], 1, &length);
    if (!buffer) {

      return;

    }
    charptr = instance2Array_char(buffer);
    length = 0;
  }

  setReferenceField(StringBuffer, buffer, F_StringBuffer_value);
  
}

/*
** Internal service routine to insert an array of unicode characters into
** another such array. The parameters are:
**  - the point at which the characters are to be inserted; 
**  - the -remaining- length of the destination array (i.e. the length 
**    after the insertion point); 
**  - the start of the characters to be inserted; and 
**  - the number of characters to be inserted.
**
** This routine performs NO BOUNDARY CHECKS, so make sure you don't pass
** it garbage arguments.  You can safely call it with dest_length==0 though,
** when it acts as an 'append'.
*/

#ifndef BYTEWISELATIN
void insertChars(w_char *dest, w_int dest_length, w_char *srce, w_int srce_length) {

  w_int i;

  /*
  ** First make a 'hole' where the new characters fit into by copying 
  ** the existing characters from the 'hole' to a higher position, 
  ** make sure we do this copying backwards...
  */

  for (i = dest_length - 1; i >= 0; i--) {
    dest[srce_length + i] = dest[i];
  }

  /*
  ** Now copy over the characters from the src array into the 'hole'.
  */

  w_memcpy(dest,srce,srce_length*sizeof(w_char));

}
#endif

w_instance StringBuffer_append_String(JNIEnv *env, w_instance thisStringBuffer, w_instance theString) {

  w_thread thread = JNIEnv2w_thread(env);
  w_char *dst;
  
#ifdef BYTEWISELATIN
  if (theString == NULL) {
    woempa(1, "Appending null string to StringBuffer.\n");

    StringBuffer_ensureCapacity(env, thisStringBuffer, 4 + getIntegerField(thisStringBuffer, F_StringBuffer_count));

    if (exceptionThrown(thread) == NULL) {
      dst = instance2Array_char(getReferenceField(thisStringBuffer, F_StringBuffer_value))
          + getIntegerField(thisStringBuffer, F_StringBuffer_count);
  
      *dst++ = 'n';
      *dst++ = 'u';
      *dst++ = 'l';
      *dst++ = 'l';

      *((w_int*)wordFieldPointer(thisStringBuffer, F_StringBuffer_count)) += 4;
    }
  }
  else {
    w_string string = String2string(theString);
    w_int    src_length = string_length(string);
    w_int    i;
 
    StringBuffer_ensureCapacity(env, thisStringBuffer, src_length + getIntegerField(thisStringBuffer, F_StringBuffer_count));

    if (exceptionThrown(thread) == NULL) {
      dst = instance2Array_char(getReferenceField(thisStringBuffer, F_StringBuffer_value))
          + getIntegerField(thisStringBuffer, F_StringBuffer_count);
      for (i = 0; i < src_length; ++i) {
        *dst++ = string_char(string, i);
      }
      *((w_int*)wordFieldPointer(thisStringBuffer, F_StringBuffer_count)) += src_length;
    }
  }
#else
  w_string string;
  w_char *src;
  w_char  null_chars[4];
  w_int   src_length;
  
  if (theString == NULL) {
    woempa(1, "Appending null string to StringBuffer.\n");
    string = NULL;
    null_chars[0] = 'n';
    null_chars[1] = 'u';
    null_chars[2] = 'l';
    null_chars[3] = 'l';
    src = null_chars;
    src_length = 4;
  }
  else {
    string = String2string(theString);
    src = string->chars;
    src_length = string->length;
  }

  woempa(1, "Appending string '%w' to StringBuffer.\n", string);

  StringBuffer_ensureCapacity(env, thisStringBuffer, src_length + getIntegerField(thisStringBuffer, F_StringBuffer_count));

  if (exceptionThrown(thread) == NULL) {
    dst = instance2Array_char(getReferenceField(thisStringBuffer, F_StringBuffer_value))
          + getIntegerField(thisStringBuffer, F_StringBuffer_count);
    insertChars(dst, 0, src, src_length);
  
    *((w_int*)wordFieldPointer(thisStringBuffer, F_StringBuffer_count)) += src_length;
  }
#endif

  return thisStringBuffer;
   
}

void StringBuffer_getChars(JNIEnv *env, w_instance StringBuffer, w_int srcBegin, w_int srcEnd, w_instance dst, w_int dstBegin) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance buffer = getReferenceField(StringBuffer, F_StringBuffer_value);
  w_char *srcbuffer;
  w_char *dstbuffer;
  w_int i;

  if (! isNullInstance(dst)) {
    if (dstBegin < 0 || srcBegin < 0 || srcBegin > srcEnd || srcEnd > instance2Array_length(buffer) || dstBegin + srcEnd - srcBegin > instance2Array_length(dst)) {
      throwException(thread, clazzIndexOutOfBoundsException, NULL);
    }
    else {
      dstbuffer = instance2Array_char(dst);
      srcbuffer = instance2Array_char(buffer);
      dstbuffer += dstBegin;
      srcbuffer += srcBegin;
      for (i = 0; i < (srcEnd - srcBegin); i++) {
        *dstbuffer++ = *srcbuffer++;
      }
    }
  }
  else {
    throwException(thread, clazzNullPointerException, NULL);
  }

}

w_instance StringBuffer_deleteCharAt(JNIEnv *env, w_instance thisStringBuffer, w_int Index) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = getIntegerField(thisStringBuffer, F_StringBuffer_count);
  w_char *dst;
  w_int i;

  if (Index < 0 || Index >= length) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    dst = instance2Array_char(getReferenceField(thisStringBuffer, F_StringBuffer_value));
    dst += Index;

    for (i = 0; i < length - Index - 1; i++) {
      dst[i] = dst[i + 1];
    }
    *((w_int*)wordFieldPointer(thisStringBuffer, F_StringBuffer_count)) -= 1;
  }

  return thisStringBuffer;

}

w_instance StringBuffer_delete(JNIEnv *env, w_instance thisStringBuffer, w_int start, w_int end) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = getIntegerField(thisStringBuffer, F_StringBuffer_count);
  w_char *dst;
  w_int num;
  w_int i;

  end = end > length ? length : end;

  if (start < 0 || start > length || start > end) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    dst = instance2Array_char(getReferenceField(thisStringBuffer, F_StringBuffer_value));
    dst += start;
    num = end - start;
    for (i = 0; i < length - end; i++) {
      dst[i] = dst[num + i];
    }
    *((w_int*)wordFieldPointer(thisStringBuffer, F_StringBuffer_count)) -= num;
  }

  return thisStringBuffer;

}

w_instance StringBuffer_reverse(JNIEnv *env, w_instance StringBuffer) {

  w_char *a;
  w_char *z;
  w_char sc;
  
  if (1 < getIntegerField(StringBuffer, F_StringBuffer_count)) {
    a = instance2Array_char(getReferenceField(StringBuffer, F_StringBuffer_value));
    z = a;
    z += getIntegerField(StringBuffer, F_StringBuffer_count) - 1;
    while (a < z) {
      sc = *a;
      *a++ = *z;
      *z-- = sc;
    }
  }
  
  return StringBuffer;

}

w_instance StringBuffer_substring(JNIEnv *env, w_instance StringBuffer, w_int start, w_int end) {

  w_thread thread = JNIEnv2w_thread(env);
  w_int length = getIntegerField(StringBuffer, F_StringBuffer_count);
  w_string string;
  w_instance result = nullInstance;
  w_char *src;
  
  if (start < 0 || end < 0 || start > end || start > length || end > length) {
    throwException(thread, clazzStringIndexOutOfBoundsException, NULL);
  }
  else {
    src = instance2Array_char(getReferenceField(StringBuffer, F_StringBuffer_value));
    src += start;
    string = unicode2String(JNIEnv2w_thread(env), src, (w_size)(end - start));
    result = newStringInstance(string);
    // string is now registered twice, when once would be enough
    deregisterString(string);
  }

  return result;

}

w_instance StringBuffer_toString(JNIEnv *env, w_instance StringBuffer) {

  w_string string;
  w_instance String;
  
  string = unicode2String(JNIEnv2w_thread(env), instance2Array_char(getReferenceField(StringBuffer, F_StringBuffer_value)), (w_size)getIntegerField(StringBuffer, F_StringBuffer_count));
  String = newStringInstance(string);
  // string is now registered twice, when once would be enough
  deregisterString(string);

  return String;

}

