/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/


/*
** $Id: Thread.c 4294 2006-05-10 11:35:19Z chris $
*/

#include <stdio.h>
#include <string.h>

#include "clazz.h"
#include "descriptor.h"
#include "exception.h"
#include "hashtable.h"
#include "locks.h"
#include "wstrings.h"
#include "methods.h"
#include "fields.h"
#include "loading.h"
#include "realms.h"
#include "threads.h"
#include "arrays.h"
#include "ts-mem.h"
#include "interpreter.h"
#include "core-classes.h"
#include "heap.h"
#include "jdwp.h"
#include "jdwp_events.h"

extern const char *dumpThread(x_thread);

w_instance Thread_currentThread(JNIEnv *env, w_instance ThreadClass) {

  woempa(1, "Current Thread instance is %p.\n", JNIEnv2Thread(env));
  
  return JNIEnv2Thread(env);

}

static w_int seqnum = 0;

static void threadEntry(void * athread) {

  w_thread thread = athread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  thread->state = wt_ready;
  callMethod(thread->top, thread->run_method);
  thread->top = & thread->Frame;
  removeThreadFromGroup(thread, thread->group);
  if (thread->exception) {
    wabort(ABORT_WONKA, "Uncaught exception in %t: %e\n", thread, thread->exception);
  }
  thread->state = wt_dying;
#ifdef GC_SAFE_POINTS
  unsafe = enterUnsafeRegion(thread);
#endif
  ht_erase(thread_hashtable,(w_word)thread->kthread);
#ifdef GC_SAFE_POINTS
  if (!unsafe) {
    enterSafeRegion(thread);
  }
#endif
  thread->state = wt_dead;
}

void Thread_create(JNIEnv *env, w_instance thisThread, w_instance parentThreadGroup, w_instance nameString, w_instance theRunnable) {

  char *   buffer;
  w_string name;
  w_thread currentthread = JNIEnv2w_thread(env);
  w_thread newthread;
  w_group  parentgroup;
  w_clazz  runnable_clazz;
  w_size   stacksize;
  w_size   locals;

  runnable_clazz = instance2clazz(theRunnable);
  if (runnable_clazz == clazzGarbageCollector) {
    stacksize = gc_stack_size;
    locals = gc_locals;
  }
  else if (runnable_clazz == clazzInit) {
    stacksize = init_stack_size;
    locals = init_locals;
  }
  else {
    stacksize = default_stack_size;
    locals = default_locals;
  }

  woempa(1, "parentThreadGroup is %p.\n", parentThreadGroup);
  woempa(9, "Runnable is a %k, stack size will be %d\n", runnable_clazz, stacksize);

  parentgroup = getWotsitField(parentThreadGroup, F_ThreadGroup_wotsit);

  woempa(1, "w_Group of new thread is %p %w.\n", parentgroup, NM(parentgroup));

  if (isNullInstance(nameString)) {
    buffer = allocMem(parentgroup->manager, THREAD_NAME_BUFFER_SIZE * sizeof(w_byte));
    if (!buffer) {
      woempa(9, "Unable to allocate memory for thread name buffer - aborting constructor\n");

      return;

    }

    x_snprintf(buffer, THREAD_NAME_BUFFER_SIZE, "Thread-%d", ++seqnum);
    name = cstring2String(parentgroup->manager, buffer, strlen(buffer));
    releaseMem(buffer);
  } 
  else {
    name = String2string(nameString);
    registerString(name);
  }

  woempa(1, "Creating thread '%w' for group %p, is%s the system group.\n", name, parentgroup, parentgroup == W_ThreadGroup_system ? "" : " not");
  newthread = createThread(currentthread, thisThread, parentgroup, name, stacksize, locals);
  if (!newthread) {
    woempa(9, "Out of memory when allocating thread %w - aborting constructor\n", name);

    return;

  }

 newthread->state = wt_unstarted;

}

void Thread_destructor(w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);
  w_string name = NULL;

  if (thread) {
    setWotsitField(thisThread, F_Thread_wotsit, NULL);
    if (thread != W_ThreadGroup_system->manager) {
      name = thread->name;
      if (name) {
        deregisterString(name);
      }
    }
    requestThreadTermination(thread);
  }

}

w_instance Thread_getName(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  return newStringInstance(thread->name);

}

void Thread_setName0(JNIEnv *env, w_instance thisThread, w_instance nameString) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);
  w_string oldname = thread->name;
  w_string newname = String2string(nameString);

  registerString(newname);
  deregisterString(oldname);

  thread->name = newname;

}

w_int Thread_start0(JNIEnv *env, w_instance thisThread) {

  w_thread this_thread = getWotsitField(thisThread, F_Thread_wotsit);
  w_thread oldthread;
  x_status status;
  w_int    result = 0;

#ifndef O4P
  this_thread->kstack = allocMem(this_thread, this_thread->ksize);
  if (!this_thread->kstack) {

    return -1;

  }
#endif

  woempa(1, "registering os thread %p as thread instance %p\n", this_thread->kthread, this_thread);
  oldthread = (w_thread)ht_write(thread_hashtable, (w_word)this_thread->kthread, (w_word)this_thread);

  if (oldthread) {
    wabort(ABORT_WONKA, "Sapristi! that os thread %p was already registered!\n", this_thread->kthread);
  }

  x_thread_create(this_thread->kthread, threadEntry, this_thread, this_thread->kstack, this_thread->ksize, this_thread->kpriority, TF_SUSPENDED);
  this_thread->kthread->report = running_thread_report;
/* end */

  woempa(7, "Starting Java Thread %t.\n", this_thread);

  addThreadToGroup(this_thread, this_thread->group);
  status = x_thread_resume(this_thread->kthread);
  if (status == xs_success) {
    if(this_thread->state == wt_unstarted) {
      this_thread->state = wt_ready;
    }
    if(jpda_hooks) {
      jdwp_event_thread_start(this_thread);
    }
  }
  else {
    result = status;
  }

  return result;

}

void Thread_stop0(JNIEnv *env, w_instance thisThread, w_instance Throwable) {

  w_thread thread = (w_thread) thisThread[F_Thread_wotsit];

  thread = getWotsitField(thisThread, F_Thread_wotsit);
  if (!threadIsActive(thread)) {
    woempa(7, "Thread %t is already dying, ignoring %e\n", thread, Throwable);
    return;
  }

  throwExceptionInstance(thread, Throwable);

  if(jpda_hooks) {
    jdwp_event_thread_end(thread);
  }
  
  /*
  ** [CG 20010925] I guess we need to do this?
  */

  if (threadState(thread) == wt_suspended) {
    x_thread_resume(thread->kthread);
  }

}

void Thread_suspend0(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  x_thread_suspend(thread->kthread);

}

void Thread_resume0(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  x_thread_resume(thread->kthread);

}


w_int Thread_getPriority(JNIEnv *env, w_instance thisThread) {

  w_thread thread;

  thread = getWotsitField(thisThread, F_Thread_wotsit);

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);

  return thread->jpriority;

}

void Thread_setPriority0(JNIEnv *env, w_instance thisThread, w_int newPriority) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);
  woempa(1,"requested priority is %d, group max priority is %d\n",newPriority, thread->group->maxPriority);

  thread->jpriority = newPriority;

  thread->kpriority = priority_j2k(thread->jpriority,0);
  if (threadIsActive(thread)) {
   x_thread_priority_set(thread->kthread, thread->kpriority);
  }
}

w_boolean Thread_isDaemon(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  woempa(1,"thread %p group %p instance %p\n",thread,thread->group,thread->group->ThreadGroup);

  return thread->isDaemon;

}

void Thread_setDaemon0(JNIEnv *env, w_instance thisThread, w_boolean on) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  woempa(1, "thread %p group %p instance %p\n", thread, thread->group, thread->group->ThreadGroup);
  thread->isDaemon = on;

}

w_boolean Thread_isInterrupted(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);

  woempa(1,"thread %p group %p\n",thread,thread->group);

  return (thread->flags & WT_THREAD_INTERRUPTED) ? WONKA_TRUE : WONKA_FALSE;

}

void Thread_interrupt(JNIEnv *env, w_instance thisThread) {

  w_thread thread = getWotsitField(thisThread, F_Thread_wotsit);
  x_status status;

  woempa(1, "thread %t is interrupting %t\n", JNIEnv2w_thread(env), thread);

  setFlag(thread->flags, WT_THREAD_INTERRUPTED);
  status = x_thread_wakeup(thread->kthread);
  woempa(7, "x_thread_wakeup status = %d\n", status);
  if (threadState(thread) == wt_waiting) {
    status = x_thread_stop_waiting(thread->kthread);
    woempa(7, "x_thread_stop_waiting status = %d\n", status);
  }

}

w_boolean Thread_static_interrupted(JNIEnv *env, w_instance classThread) {

  w_thread thread = JNIEnv2w_thread(env);
  w_boolean interrupted = (thread->flags & WT_THREAD_INTERRUPTED) ? WONKA_TRUE : WONKA_FALSE;

  woempa(1,"thread %p group %p instance %p: interrupted flag was %d\n",thread,thread->group,thread->group->ThreadGroup,interrupted);

  thread->flags &= ~WT_THREAD_INTERRUPTED;

  return interrupted;

}

void Thread_static_yield(JNIEnv *env, w_instance ThreadClass) {

#ifdef DEBUG
  w_thread thread = JNIEnv2w_thread(env);

  woempa(1, "Yielding on %j (thread %t).\n", thread->Thread, thread);
#endif

  x_thread_yield();

}

/*
** Careful: in the following call, the this instance is called JoiningWith since it
** is the thread with which the CURRENT thread wants to join. The current thread is
** given by the env argument.
*/

void Thread_join0(JNIEnv *env, w_instance JoiningWith, w_long millis, w_int nanos) {

  w_thread joiningWith = getWotsitField(JoiningWith, F_Thread_wotsit);
  w_thread thread = JNIEnv2w_thread(env);
  x_status status;
  void * result;
  w_size snooze = 0;

  if (millis < 0 || nanos < 0 || nanos >= 1000000) {
    throwException(thread, clazzIllegalArgumentException, NULL);
    return;
  }

  if (joiningWith) {
    if (millis) {
      snooze = x_usecs2ticks((w_int)millis * 1000 + nanos / 1000);
    }
    else if (nanos) {
      snooze = x_usecs2ticks(1000);
    }
    else {
      snooze = x_eternal;
    }

    woempa(1, "joining thread %t !!! %d\n", joiningWith, snooze);

#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif

    if (thread->flags & WT_THREAD_INTERRUPTED) {
      throwException(thread, clazzInterruptedException, NULL);
      thread->flags &= ~WT_THREAD_INTERRUPTED;
      woempa(6, "Thread %t was already interrupted throwing exception\n", thread);
    }
    else {
      status = x_thread_join(joiningWith->kthread, &result, snooze);
      woempa(1, "joined thread %t !!! %x\n", joiningWith, status);

      if (thread->flags & WT_THREAD_INTERRUPTED) {
        thread->flags &= ~WT_THREAD_INTERRUPTED;
        woempa(7, "VERIFYING x_thread_join result %d =?= 3\n", *(w_word *)result);
        throwException(thread, clazzInterruptedException, NULL);
      }
    }

#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif

  }
  else {
    woempa(9, "join0 is going wrong !!!\n");
  }

}

void Thread_sleep0(JNIEnv *env, w_instance Thread, w_long millis, w_int nanos) {

  w_thread thread = getWotsitField(Thread, F_Thread_wotsit);
  w_size snooze = 0;

  if (thread) {

    if (millis < 0 || nanos < 0 || nanos >= 1000000) {
      throwException(thread, clazzIllegalArgumentException, NULL);
      return;
    }

    if (millis) {
      snooze = (w_word)((w_int)millis * 1000) + (nanos / 1000) ;
    }
    else if (nanos) {
      snooze = 1000;
    }
    else {
      return;
    }
    woempa(1, "thread will go to sleep!!! %t\n", thread);

#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif

    if (thread->flags & WT_THREAD_INTERRUPTED) {
      throwException(thread, clazzInterruptedException, NULL);
      thread->flags &= ~WT_THREAD_INTERRUPTED;
      woempa(6, "THROWING an InterruptedException\n");
#ifndef GC_SAFE_POINTS
      x_preemption_enable;
#endif
      return;

    }
    else {
      x_thread_sleep(x_usecs2ticks(snooze));
      woempa(6, "thread woke up!!! %t\n", thread);

      if (thread->flags & WT_THREAD_INTERRUPTED) {
        thread->flags &= ~WT_THREAD_INTERRUPTED;
        throwException(thread, clazzInterruptedException, NULL);
      }
    }

#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif

  }
  else {
    woempa(9, "sleep0 is going wrong w_thread == null !!!\n");
  }
}
