/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: InetAddress.c 4294 2006-05-10 11:35:19Z chris $
*/

#include <network.h>
#include <errno.h>

#include "arrays.h"
#include "core-classes.h"
#include "exception.h"
#include "fields.h"
#include "wstrings.h"

#ifndef FALSE
#define FALSE 0
#endif

#ifndef TRUE
#define TRUE 1
#endif

w_boolean InetAddress_lookupName(JNIEnv *env, w_instance InetAddress) {

  w_boolean result = FALSE;
  w_thread thread = JNIEnv2w_thread(env);
  w_string name = String2string(getReferenceField(InetAddress, F_InetAddress_addressCache));
  struct hostent * host = NULL;
  char * hostname = allocMem(W_Thread_system, (string_length(name) + 1) * sizeof(char));

  if (! hostname) {
    throwException(thread, clazzOutOfMemoryError, NULL);
  }
  else {
    int af = 0;
    
    x_snprintf(hostname, string_length(name) + 1, "%w", name);
   
    if(instance2clazz(InetAddress) == clazzInet6Address) {
      af = AF_INET6;
    }
    else {
      af = AF_INET;
    }
      
    host = w_gethostbyname(hostname);
    
    if (host) {
      setReferenceField(InetAddress, (*env)->NewStringUTF(env, host->h_name), F_InetAddress_hostName);
      result = TRUE;
    }
    releaseMem(hostname);
  }

  return result;

}

/**
** hostName will be set to name ...
*/

void InetAddress_createInetAddress (JNIEnv *env, w_instance InetAddress, w_instance Name) {

  w_thread thread = JNIEnv2w_thread(env);
  long ipnumber;
  w_string name = String2string(Name);
  char * hostname;
  struct hostent * host = NULL;

  hostname = allocMem(W_Thread_system, (string_length(name) + 1) * sizeof(char));
  if (! hostname) {
    throwException(thread, clazzOutOfMemoryError, NULL);
  }
  else {
    int af = 0;
    
    x_snprintf(hostname, string_length(name) + 1, "%w", name);
    woempa(6, "Using DNS to get information...%s (%p)\n", hostname, hostname);

    if(instance2clazz(InetAddress) == clazzInet6Address) {
      af = AF_INET6;
    }
    else {
      af = AF_INET;
    }

    host = w_gethostbyname2(hostname, af);

    if (! host) {
      throwException(thread, clazzUnknownHostException, "unable to find DNS name for '%s' due to %s", hostname,strerror(errno));
      releaseMem(hostname);
      return;
    }

    woempa(7, "displaying DNS information... name = %s (%p)\n", host->h_name, hostname);
    woempa(7, "displaying DNS information... length = %d (%p)\n", host->h_length, hostname);

    /*
    ** Creating and copying the hostname to private 'hostName' field of InetAddress Object
    */

    setReferenceField(InetAddress, Name, F_InetAddress_hostName);

    /*
    ** Copying the 'type' into the 'type' private field of InetAddress Object
    ** Loopback	-> type = 1
    ** IPv4 	-> type = 2
    */

    if (host->h_addrtype == AF_LOCAL) {
      setIntegerField(InetAddress, F_InetAddress_family, 1);
    }

    if (host->h_addrtype == AF_INET) {
      setIntegerField(InetAddress, F_InetAddress_family, 2);
    }

    /*
    ** 1.Retrieving the dotted decimal representation of the ip number.
    ** 2.Copying the octet stuff into the 'address' private field of InetAddress Object
    */

    ipnumber = *((unsigned long*)host->h_addr_list[0]);
    releaseMem(hostname);
    setIntegerField(InetAddress, F_InetAddress_address, htonl(ipnumber));
    
  }

}

w_instance InetAddress_getLocalName(JNIEnv *env, w_instance clazz) {

  static const w_size length = 255;
  w_instance Name = NULL;
  char * name = allocMem(W_Thread_system, length * sizeof(char));

  if (name && w_gethostname(name, length) == 0) {
    woempa(7, "w_gethostname => '%s'\n", name);
    Name = (*env)->NewStringUTF(env, name);
  }

  if (name) {
    releaseMem(name);
  }

  return Name;

}
