/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/**
 * $Id: PlainSocketImpl.c 4297 2006-05-10 12:18:44Z chris $
 */

#include <string.h>
#include <errno.h>

#include "arrays.h"
#include "core-classes.h"
#include "exception.h"
#include "fields.h"
#include "network.h"

/**
** SocketImpl use calls to functions defined by network.h. All these calls work with a descriptor
** the descriptor is stored in the wotsit of SocketImpl ! We don't use the FileDescritor Object,
** it will be left null ... . This means other implementions should use this wotsit value to make calls to
** the network.  Note: this is not the way to do it according to java API.  Although using a FileDescriptor
** Almost implies using native code. By leaving that descriptor null we prevent it from being misused ...
*/

w_int PlainSocketImpl_clear(w_instance ThisImpl) {
  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  w_int res;	

  if (getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    res = w_socketclose(sock);

    setBooleanField(ThisImpl, F_PlainSocketImpl_open, WONKA_FALSE);
    setWotsitField(ThisImpl, F_PlainSocketImpl_wotsit, (void*)-1);
  }
  else {
    woempa(6, "socket %p was already closed !\n", ThisImpl);
    res = 0;
  }

  return res;

}

void PlainSocketImpl_close(JNIEnv* env , w_instance ThisImpl) {
  if (PlainSocketImpl_clear(ThisImpl) == -1) {
    throwException(JNIEnv2w_thread(env), clazzIOException, "closing socket failed with %d: '%s'", errno, strerror(errno));
  }
}

void PlainSocketImpl_finalize(JNIEnv* env , w_instance ThisImpl) {
  PlainSocketImpl_clear(ThisImpl);
}

w_int PlainSocketImpl_getSocket(JNIEnv* env , w_instance ThisImpl){

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    throwException(JNIEnv2w_thread(env), clazzSocketException, "socket is not open or uninitialized");
  }

  return (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

}

void PlainSocketImpl_nativeCreate(JNIEnv* env , w_instance ThisImpl) {
  w_int sock;
  w_int pf = (getBooleanField(ThisImpl, F_PlainSocketImpl_ipv6) ? PF_INET6 : PF_INET);

  if(getBooleanField(ThisImpl, F_PlainSocketImpl_open)){
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket is already open");
  }
  else {
    sock = w_socket (pf, SOCK_STREAM, 0);
    setWotsitField(ThisImpl, F_PlainSocketImpl_wotsit, (void*)sock);	

    if (sock == -1) {
      throwException(JNIEnv2w_thread(env), clazzIOException, "socket errno %d '%s'", errno, strerror(errno));
      woempa(9, "Not able to create a socket for %p\n", ThisImpl);
    }
    else {
      setBooleanField(ThisImpl, F_PlainSocketImpl_open, WONKA_TRUE);
    }
  }
}

void PlainSocketImpl_connect(JNIEnv* env , w_instance ThisImpl) {

  w_instance address = getReferenceField(ThisImpl, F_PlainSocketImpl_remoteAddress); 
  w_int port = getIntegerField(ThisImpl, F_PlainSocketImpl_remotePort);
  
  if (!address) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, "address field is NULL");
  }
  else {
    struct sockaddr * sa = NULL;
    int sa_size = 0;
    
    struct sockaddr_in  sa4;
    struct sockaddr_in6 sa6;

    w_int res = getIntegerField(address, F_InetAddress_family);
    w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

    if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
      woempa(9, "socket %i was closed\n", sock);
      throwException(JNIEnv2w_thread(env), clazzIOException, "socket was closed or uninitialized");
      return;
    }  	  	

    if (!(getBooleanField(ThisImpl, F_PlainSocketImpl_ipv6))) { 
      memset(&sa4, 0, sizeof(sa4));
      sa4.sin_addr.s_addr = htonl(getIntegerField(address, F_InetAddress_address));
      sa4.sin_family = AF_INET;
      sa4.sin_port = w_switchPortBytes(port);

      woempa (7,"INET: connect, port %d addr %lx\n", sa4.sin_port, sa4.sin_addr.s_addr);

      sa = (struct sockaddr *)&sa4;
      sa_size = sizeof(struct sockaddr_in);
    }
    else {
      memset(&sa6, 0, sizeof(sa6));
      memcpy(&sa6.sin6_addr, instance2Array_byte(getReferenceField(address, F_Inet6Address_ipaddress)), 16);
      sa6.sin6_family = AF_INET6;
      sa6.sin6_port = w_switchPortBytes(port);

      woempa (7,"INET6: connect, port %d addr %p\n", sa6.sin6_port, sa6.sin6_addr);

      sa = (struct sockaddr *)&sa6;
      sa_size = sizeof(struct sockaddr_in6);
    }

    res = w_connect (sock, sa, sa_size);
    woempa (6,"socketfd = %d, connect result = %d\n",sock, res);
    if (res == -1) {
      woempa(9,"ERROR in connect = %s\n", strerror(errno));
      throwException(JNIEnv2w_thread(env), clazzConnectException, "socket connect errno %d '%s'", errno, strerror(errno));
      return;
    }

    setIntegerField(ThisImpl, F_SocketImpl_port, port);
    setReferenceField(ThisImpl, address, F_SocketImpl_address);
    
    if (!(getBooleanField(ThisImpl, F_PlainSocketImpl_ipv6))) { 
      socklen_t namelen = sizeof(sa4);
      res = w_getsockname(sock , (struct sockaddr *)&sa4 , &namelen);
      if (res == -1) {
        //woempa(9,"ERROR in connect = %s\n", w_strerror((int)w_errno(sock))); 	
        throwException(JNIEnv2w_thread(env), clazzConnectException, "getsockname errno %d '%s'", errno, strerror(errno));
      }
      else {	
        setIntegerField(ThisImpl, F_SocketImpl_localport, w_switchPortBytes(sa4.sin_port));
      }
    }
    else {
      socklen_t namelen = sizeof(sa6);
      res = w_getsockname(sock , (struct sockaddr *)&sa6 , &namelen);
      if (res == -1) {
        //woempa(9,"ERROR in connect = %s\n", w_strerror((int)w_errno(sock))); 	
        throwException(JNIEnv2w_thread(env), clazzConnectException, "getsockname errno %d '%s'", errno, strerror(errno));
      }
      else {	
        setIntegerField(ThisImpl, F_SocketImpl_localport, w_switchPortBytes(sa6.sin6_port));
      }
    }
  }
  
}

w_int PlainSocketImpl_read(JNIEnv* env , w_instance ThisImpl, w_instance byteArray, w_int off, w_int length) {

  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  w_int res;	
  w_int arrayLength;
  w_int timeout;
  
  woempa(6, "reading %i bytes from SocketImpl %p (desp %i)\n", length, ThisImpl, sock);

  if (! byteArray) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, "buffer is NULL");
    return -1;
  }
  else {
    arrayLength = instance2Array_length(byteArray);
    if ((off < 0) || (length < 0) || (off + length > arrayLength)) {
      throwException(JNIEnv2w_thread(env), clazzArrayIndexOutOfBoundsException, "out of bounds; size = %d, offset = %d, length = %d", arrayLength, off, length);
      return -1;
    }
  }

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket closed or uninitialized");
    return -1;  	
  }

  if (length == 0) {
    return 0;
  }

  /*
  ** All seems OK, let's read...
  */

  timeout = getIntegerField(ThisImpl, F_PlainSocketImpl_timeout);
  res = w_recv(ThisImpl, sock, instance2Array_byte(byteArray) + off, (w_word)length, 0, &timeout);

  if (res == -1) {
    if( timeout == -1) {
      woempa(9,"Error in read: timeout expired\n");
      throwException(JNIEnv2w_thread(env), clazzSocketTimeoutException, "timeout %d expired.", getIntegerField(ThisImpl, F_PlainSocketImpl_timeout));
    }
    else {
      //woempa(9,"got error while reading from SocketImpl %p --> error = %s\n", ThisImpl, w_strerror(w_errno(sock)));
      throwException(JNIEnv2w_thread(env), clazzIOException, "recv errno %d '%s'", errno, strerror(errno));
    }
  }

  woempa(6, "reading %i bytes from SocketImpl %p --> got %i\n", length, ThisImpl, res);
  if (res == 0) { 
    res--; 
  }

  return res;

}

void PlainSocketImpl_write(JNIEnv * env, w_instance ThisImpl, w_instance byteArray, w_int off, w_int length) {

  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  w_int res;	
  w_int arrayLength;
  w_int sent;

  woempa(6, "writing %i bytes to SocketImpl %p\n", length, ThisImpl);

  if (! byteArray) {
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, "buffer is NULL");
    return;
  }
  else {
    arrayLength = instance2Array_length(byteArray);
    if ((off < 0) || (length < 0) || (off + length > arrayLength)) {
      throwException(JNIEnv2w_thread(env), clazzArrayIndexOutOfBoundsException, "out of bounds; size = %d, offset = %d, length = %d", arrayLength, off, length);
      return;
    }
  }

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket closed or uninitialized");
    return;  	
  }

  if (length == 0) {
    return;
  }

  res = w_send(sock, instance2Array_byte(byteArray) + off, (w_word)length, MSG_NOSIGNAL);
  sent = 0;

  do {
    if (res == -1) {
      woempa(9,"got error while sending from SocketImpl %p --> error = %i\n", ThisImpl, w_errno(sock));
      throwException(JNIEnv2w_thread(env), clazzIOException, "send errno %d '%s'", errno, strerror(errno));
      break;
    }

    sent += res;
    if(sent >= length) {
      break;
    }

    if(res == 0) {
      throwException(JNIEnv2w_thread(env), clazzIOException, "no data written");
      break;
    }

    res = w_send(sock, instance2Array_byte(byteArray) + off + sent, (w_word)(length - sent), MSG_NOSIGNAL);
  } while (1);
}

void PlainSocketImpl_bind(JNIEnv* env , w_instance ThisImpl) {

  w_instance address = getReferenceField(ThisImpl, F_PlainSocketImpl_localAddress); 
  w_int port = getIntegerField(ThisImpl, F_PlainSocketImpl_localPort);
  w_thread thread = JNIEnv2w_thread(env);
  
  if (! address) {
    throwException(thread, clazzNullPointerException, "address is NULL");
  }
  else {
    struct sockaddr * sa = NULL;
    int sa_size = 0;
    
    struct sockaddr_in  sa4;
    struct sockaddr_in6 sa6;

    w_int res = getIntegerField(address, F_InetAddress_family);
    w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  	
    if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
      woempa(9, "socket %i was closed\n",sock);
      throwException(thread, clazzIOException, "socket closed or uninitialized");
      return;
    }  	  	  	

    if (!(getBooleanField(ThisImpl, F_PlainSocketImpl_ipv6))) { 
      memset(&sa4, 0, sizeof(sa4));
      sa4.sin_family = AF_INET;
      sa4.sin_addr.s_addr = htonl(getIntegerField(address, F_InetAddress_address));
      //w_printIntAsIpString(sa4.sin_addr.s_addr);
      sa4.sin_port = w_switchPortBytes(port);
      //w_comparePorts(sa.sin_port,port);    	
      woempa(7, "INET: connect, port %d addr %x (%x)\n", sa4.sin_port, sa4.sin_addr.s_addr, getIntegerField(address, F_InetAddress_address));

      sa = (struct sockaddr *)&sa4;
      sa_size = sizeof(struct sockaddr_in);
    }
    else {
      memset(&sa6, 0, sizeof(sa6));
      sa6.sin6_family = AF_INET6;
      sa6.sin6_port = w_switchPortBytes(port);
      sa6.sin6_addr = in6addr_any;  /* WOOT -> Should get the address */
      
      sa = (struct sockaddr *)&sa6;
      sa_size = sizeof(struct sockaddr_in6);
    }

    res = w_bind (sock, sa, sa_size);
    woempa(6, "socketfd = %d, bindresult = %d\n", sock, res);

    if (res == -1) {
      //woempa(9,"ERROR = %s\n", w_strerror((int)w_errno(sock)));
      throwException(thread, clazzBindException, "bind errno %d '%s'", errno, strerror(errno));
      return;
    }

    if (port == 0) {
      if (!(getBooleanField(ThisImpl, F_PlainSocketImpl_ipv6))) { 
        socklen_t namelen = sizeof(sa4);
        res = w_getsockname(sock , (struct sockaddr*)&sa4 , &namelen);
        if (res == -1) {
          //woempa(9,"ERROR in bind (getsockname) = %s\n", w_strerror((int)w_errno(sock)));
          throwException(thread, clazzBindException, "getsockname errno %d '%s'", errno, strerror(errno));
        }
        else {
          port = w_switchPortBytes(sa4.sin_port);
          //w_comparePorts(sa4.sin_port,port);    	
        }
      }
      else {
        socklen_t namelen = sizeof(sa6);
        res = w_getsockname(sock , (struct sockaddr*)&sa6 , &namelen);
        if (res == -1) {
          throwException(thread, clazzBindException, "getsockname errno %d '%s'", errno, strerror(errno));
        }
        else {
          port = w_switchPortBytes(sa6.sin6_port);
        }
      }
    }  	
    setIntegerField(ThisImpl, F_SocketImpl_localport, port);
  }

}

void PlainSocketImpl_listen(JNIEnv* env , w_instance ThisImpl, w_int backlog) {

  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    woempa(9, "socket %i is already closed\n",sock);
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket closed or uninitialized");
  }
  else {
    w_int res = w_listen(sock,(int)backlog);
    woempa(6, "listen called on socket %p (desc %i)\n", ThisImpl, sock);
    if (res == -1) {
      //woempa(9,"ERROR occured while listen was called --> %s\n", w_strerror((int)w_errno(sock)));
      throwException(JNIEnv2w_thread(env), clazzIOException, "listen errno %d '%s'", errno, strerror(errno));
    }
  }	

}

int PlainSocketImpl_accept(JNIEnv* env , w_instance ThisImpl, w_instance newImpl) {

  w_thread thread = JNIEnv2w_thread(env);
  
  //newImpl will not be null (this function is a private method)
  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open) || !newImpl) {
    woempa(9, "socket %i is already closed\n",sock);
    throwException(thread, clazzIOException, "socket error; fd = %d, new Socket = %p", sock, newImpl);
  }
  else { // accept is done in two steps step step 1. get a socket from the system
    struct sockaddr_in sa;
    w_int newsocket;
    socklen_t bytelen = sizeof (struct sockaddr_in);
  		
    memset(&sa, 0, sizeof(sa)); //lets play safe
    newsocket = w_accept(ThisImpl, sock, (struct sockaddr *) &sa , &bytelen, getIntegerField(ThisImpl, F_PlainSocketImpl_timeout));
  	
    //set the wotsit value to -1 or the valid socket descriptor
    setWotsitField(newImpl, F_PlainSocketImpl_wotsit, (void*)newsocket);
  	
    if (newsocket == -1) {
      if (bytelen == 0) {
        woempa(9, "ERROR accept timed out\n");
        throwException(thread, clazzInterruptedIOException, "socket timed out");
      }
      else {
        //woempa(9,"ERROR occured while accept was called --> %s\n", w_strerror((int)w_errno(sock)));
        throwException(thread, clazzIOException, "error in accept, errno %d '%s', originating socket %d", errno, strerror(errno), sock);
      }
      return sock;
    }
    else {
      setBooleanField(newImpl, F_PlainSocketImpl_open, WONKA_TRUE);
    }  	

    // step 2. set the fields of newImpl.
    setIntegerField(newImpl, F_SocketImpl_port, w_switchPortBytes(sa.sin_port));
    //w_comparePorts(sa.sin_port,getIntegerField(newImpl, F_SocketImpl_port));    	
    bytelen = sizeof(sa);
    newsocket = w_getsockname(newsocket , (struct sockaddr *) &sa , &bytelen);
    if (newsocket == -1) {
      //woempa(9,"ERROR in accept (getsockname) = %s\n", w_strerror((int)w_errno(sock))); 	
      throwException(thread, clazzIOException, "error in getsockname, errno %d '%s', originating socket %d", errno, strerror(errno), sock);
    }
    else {
      setIntegerField(newImpl, F_SocketImpl_localport, w_switchPortBytes(sa.sin_port));
      //w_comparePorts(sa.sin_port,getIntegerField(newImpl, F_SocketImpl_port));    	
    }
    //w_printIntAsIpString(sa.sin_addr.s_addr);
    return ntohl(sa.sin_addr.s_addr);  	
  }

  return sock;
}

w_int PlainSocketImpl_available(JNIEnv* env , w_instance ThisImpl) {

  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    woempa(9, "socket %i is already closed\n", sock);
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket closed or uninitialized");
  }
  else { 	
    w_int arg = 0;
    w_int res = ioctl(sock, FIONREAD, &arg);

    if (res == -1) {
      //woempa(9, "Error in Available 'ioctl' failed: %s\n", w_strerror((int)w_errno(sock)));
      throwException(JNIEnv2w_thread(env), clazzIOException, "ioctl errno %d '%s'", errno, strerror(errno));
    }

    woempa(6, "Available bytes %x, %x\n", res, arg);
    return arg;  		
  }

  return sock;

}

void PlainSocketImpl_shutdown(JNIEnv* env , w_instance ThisImpl, w_boolean in) {
  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    woempa(9, "socket %i is already closed\n", sock);
    throwException(JNIEnv2w_thread(env), clazzIOException, "socket closed or uninitialized");
  }
  else { 	
    if(shutdown(sock, (in == WONKA_TRUE ? 0 : 1)) == -1){
      throwException(JNIEnv2w_thread(env), clazzIOException, "shutdown failed: errno %d '%s'", errno, strerror(errno));
    }
  }
}

void PlainSocketImpl_setKeepAlive(JNIEnv* env , w_instance ThisImpl, w_instance Boolean) {
  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  int value;

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    woempa(9, "socket %i is already closed\n", sock);
    throwException(JNIEnv2w_thread(env), clazzSocketException, "socket closed or uninitialized");
    return;
  }

  if(Boolean == NULL){
    throwException(JNIEnv2w_thread(env), clazzNullPointerException, "");
    return;
  }

  value = getBooleanField(Boolean, F_Boolean_value);

  if(w_setsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &value, sizeof(int)) == -1){
    throwException(JNIEnv2w_thread(env), clazzSocketException, "setting SO_KEEPALIVE failed: errno %d '%s'", errno, strerror(errno));
  }
}

w_boolean PlainSocketImpl_getKeepAlive(JNIEnv* env , w_instance ThisImpl){
  w_int sock = (w_int)getWotsitField(ThisImpl, F_PlainSocketImpl_wotsit);
  int value;

  if (!getBooleanField(ThisImpl, F_PlainSocketImpl_open)) {
    woempa(9, "socket %i is already closed\n", sock);
    throwException(JNIEnv2w_thread(env), clazzSocketException, "socket closed or uninitialized");
  }
  else {
    socklen_t size = sizeof(int);
    if(w_getsockopt(sock, SOL_SOCKET, SO_KEEPALIVE, &value, &size) == -1){
      throwException(JNIEnv2w_thread(env), clazzSocketException, "getting SO_KEEPALIVE failed: errno %d '%s'", errno, strerror(errno));
    }
    else {
      return (value == 0 ? WONKA_FALSE : WONKA_TRUE);
    }
  }
  return WONKA_FALSE;
}





