/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
*                                                                         *
* Modifications copyright (c) 2005 by Chris Gray, /k/ Embedded Java       *
* Solutions. All rights reserved.                                         *
*                                                                         *
**************************************************************************/

/*
** $Id: NativeProcess.c 4294 2006-05-10 11:35:19Z chris $
*/

#include "wonka.h"
#include "core-classes.h"
#include "exec.h"
#include "fields.h"
#include "threads.h"
#include "exception.h"
#include "arrays.h"
#include <unistd.h>

w_void NativeProcess_destroy(JNIEnv *env, w_instance thisInstance) {
  close(getIntegerField(thisInstance, F_NativeProcess_fd_in));
  close(getIntegerField(thisInstance, F_NativeProcess_fd_out));
  close(getIntegerField(thisInstance, F_NativeProcess_fd_err));
  host_destroy(getIntegerField(thisInstance, F_NativeProcess_process_id));
}

w_int NativeProcess_waitFor(JNIEnv *env, w_instance thisInstance) {
  return host_wait(getIntegerField(thisInstance, F_NativeProcess_process_id));
}

#include <stdio.h>

w_int ProcessInputStream_read(JNIEnv *env, w_instance thisInstance) {
  w_int result;
  result = host_read(getIntegerField(thisInstance, F_ProcessInputStream_process_id), getIntegerField(thisInstance, F_ProcessInputStream_fd));
  if(result == EXECUTION_ERROR){
    w_thread thread = JNIEnv2w_thread(env);
    throwException(thread,clazzIOException,NULL);
  }
  return result;
}

w_int ProcessInputStream_read_Array(JNIEnv *env, w_instance thisInstance, w_instance Array, w_int offset, w_int length) {
  w_thread thread = JNIEnv2w_thread(env);
  w_int result = 0;

  if(Array == NULL){
    throwException(thread,clazzNullPointerException,NULL);
  }
  else if(offset < 0 || length < 0 || (offset + length > instance2Array_length(Array))){
    throwException(thread,clazzArrayIndexOutOfBoundsException,NULL);
  }
  else if(length > 0){
    w_ubyte* buffer = instance2Array_byte(Array) + offset;

    result = host_read_Array(getIntegerField(thisInstance, F_ProcessInputStream_process_id), getIntegerField(thisInstance, F_ProcessInputStream_fd), buffer, length);
    if(result == EXECUTION_ERROR){
      throwException(thread,clazzIOException,NULL);
    }
    else if(result == -1) {
      if(getBooleanField(thisInstance, F_ProcessInputStream_EOFReached)) {
        throwException(thread,clazzIOException,NULL);
      }
      else {
        setBooleanField(thisInstance, F_ProcessInputStream_EOFReached, JNI_TRUE);
      }
    }
  }
  return result;
}

w_void ProcessInputStream_close(JNIEnv *env, w_instance thisInstance) {
  host_close(getIntegerField(thisInstance, F_ProcessInputStream_process_id), getIntegerField(thisInstance, F_ProcessInputStream_fd));
}

w_int ProcessInputStream_available(JNIEnv *env, w_instance thisInstance) {
  return host_available(getIntegerField(thisInstance, F_ProcessInputStream_process_id), getIntegerField(thisInstance, F_ProcessInputStream_fd));
}

w_void ProcessOutputStream_write(JNIEnv *env, w_instance thisInstance, w_int byte) {
  host_write(getIntegerField(thisInstance, F_ProcessOutputStream_process_id), getIntegerField(thisInstance, F_ProcessOutputStream_fd), (w_int)byte);
}

w_void ProcessOutputStream_close(JNIEnv *env, w_instance thisInstance) {
  host_close(getIntegerField(thisInstance, F_ProcessOutputStream_process_id), getIntegerField(thisInstance, F_ProcessOutputStream_fd));
}

