/**************************************************************************
* Copyright  (c) 2001, 2002 by Acunia N.V. All rights reserved.           *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: Wonka.c 3723 2003-03-24 14:44:50Z ruelens $
*/

#include "clazz.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "methods.h"
#include "sha.h"
#include "wstrings.h"
#include "wonka.h"

w_int sort_interfaces(const void * v_i1, const void * v_i2) {

  w_clazz i1 = *(w_clazz *)v_i1;
  w_clazz i2 = *(w_clazz *)v_i2;
 
  return compareStrings(i1->dotified, i2->dotified);
  
}

w_int sort_methods(const void * v_m1, const void * v_m2) {

  w_method m1 = *(w_method *)v_m1;
  w_method m2 = *(w_method *)v_m2;
  w_size   l1;
  w_size   l2;
  w_size   i;
  w_string s1;
  w_string s2;
  w_char *b1;
  w_char *b2;
  w_int result;
  
  l1 = string_length(m1->spec.name) + 2;
  if (m1->spec.arg_types) {
    for (i = 0; m1->spec.arg_types[i]; ++i) {
      s1 = clazz2desc(m1->spec.arg_types[i]);
      l1 += string_length(s1);
      deregisterString(s1);
    }
  }
  s1 = clazz2desc(m1->spec.return_type);
  l1 += string_length(s1);
  deregisterString(s1);

  l2 = string_length(m2->spec.name) + 2;
  if (m2->spec.arg_types) {
    for (i = 0; m2->spec.arg_types[i]; ++i) {
      s2 = clazz2desc(m2->spec.arg_types[i]);
      l2 += string_length(s2);
      deregisterString(s2);
    }
  }
  s2 = clazz2desc(m2->spec.return_type);
  l2 += string_length(s2);
  deregisterString(s2);

  b1 = allocMem_system(l1 * sizeof(w_char));
  stringAdd(b1, m1->spec.name);
  l1 = string_length(m1->spec.name);
  b1[l1++] = '(';
  if (m1->spec.arg_types) {
    for (i = 0; m1->spec.arg_types[i]; ++i) {
      s1 = clazz2desc(m1->spec.arg_types[i]);
      stringAdd(b1 + l1, s1);
      l1 += string_length(s1);
      deregisterString(s1);
    }
  }
  b1[l1++] = ')';
  s1 = clazz2desc(m1->spec.return_type);
  stringAdd(b1 + l1, s1);
  l1 += string_length(s1);
  deregisterString(s1);
  s1 = unicode2String(W_Thread_system, b1, l1);
  releaseMem(b1);

  b2 = allocMem_system(l2 * sizeof(w_char));
  stringAdd(b2, m2->spec.name);
  l2 = string_length(m2->spec.name);
  b2[l2++] = '(';
  if (m2->spec.arg_types) {
    for (i = 0; m2->spec.arg_types[i]; ++i) {
      s2 = clazz2desc(m2->spec.arg_types[i]);
      stringAdd(b2 + l2, s2);
      l2 += string_length(s2);
      deregisterString(s2);
    }
  }
  b2[l2++] = ')';
  s2 = clazz2desc(m2->spec.return_type);
  stringAdd(b2 + l2, s2);
  l2 += string_length(s2);
  deregisterString(s2);
  s2 = unicode2String(W_Thread_system, b2, l2);
  releaseMem(b2);

  woempa(1, "SERIALISE: %w <=> %w\n", s1, s2);
  result = compareStrings(s1, s2);
  
  deregisterString(s1);
  deregisterString(s2);

  return result;
    
}

w_int sort_fields(const void * v_f1, const void * v_f2) {

  w_field f1 = *(w_field *)v_f1;
  w_field f2 = *(w_field *)v_f2;

  return compareStrings(f1->name, f2->name);

}

typedef union  {
  w_word W;
  w_byte B[4];
} word2bytes;

static w_int addModifiers(w_byte *buffer, w_word modifiers) {

  word2bytes u;
  
  u.W = (modifiers & ACC_FLAGS);
#ifdef LITTLE_ENDIAN
  buffer[0] = u.B[3];
  buffer[1] = u.B[2];
  buffer[2] = u.B[1];
  buffer[3] = u.B[0];
#else
  buffer[0] = u.B[0];
  buffer[1] = u.B[1];
  buffer[2] = u.B[2];
  buffer[3] = u.B[3];
#endif  

  return 4;

}    

w_word bytes2word(w_byte *bytes) {

  word2bytes u;
  
#ifdef LITTLE_ENDIAN
  u.B[3] = bytes[0];
  u.B[2] = bytes[1];
  u.B[1] = bytes[2];
  u.B[0] = bytes[3];
#else
  u.B[0] = bytes[0];
  u.B[1] = bytes[1];
  u.B[2] = bytes[2];
  u.B[3] = bytes[3];
#endif  

  return u.W;

}    

#include <ctype.h>
                                        
w_long Wonka_suid(JNIEnv *env, w_instance Wonka, w_instance Class) {
  w_thread thread = currentWonkaThread;
  w_clazz clazz = Class2clazz(Class);
  w_long suid = 0;
  w_clazz *interfaces = NULL;
  w_size numInterfaces = 0;
  w_field *fields = NULL;
  w_size numFields = 0;
  w_method *constructors = NULL;
  w_size numConstructors = 0;
  w_method *methods = NULL;
  w_size numMethods = 0;
  w_size i;
  w_size j;
  w_size k;
  w_size chars;
  w_byte *buffer;
  w_byte *utf8;
  w_size length;
  w_sha sha;
  w_string zig;
  w_string tmp;
  w_size   sigsize;
  w_char  *sig;
  w_flags  flags;
  w_method m;

  woempa(1, "SERIALISE: Calculating suid of clazz %k\n", clazz);

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return 0;
  }

  /*
  ** See if we have a cached version of the SUID in our sleeves...
  */

  if (clazz->suid != 0) {
    woempa(1, "SERIALISE: Returning cached suid 0x%x%x\n", MSW_PART(clazz->suid), LSW_PART(clazz->suid));
    return clazz->suid;
  }


  if (!clazz->dims) {
    /*
    ** First create the arrays so that they can be sorted, some arrays are overallocated
    ** but they are released soon enough so that is not a problem.
    */

  numInterfaces = clazz->numDirectInterfaces;
  if (numInterfaces) {
    interfaces = allocMem_system(numInterfaces * sizeof(w_clazz));
    w_memcpy(interfaces, clazz->interfaces, numInterfaces * sizeof(w_clazz));
    qsort(interfaces, numInterfaces, sizeof(w_clazz), sort_interfaces);
  }

  if (clazz->numFields) {
    fields = allocMem_system(clazz->numFields * sizeof(w_field));
    for (i = 0; i < clazz->numFields; i++) {
      if (clazz->own_fields[i].declaring_clazz == clazz) {
        flags = clazz->own_fields[i].flags;
        if (! (isSet(flags, ACC_PRIVATE) && isSet(flags, ACC_STATIC)) && ! (isSet(flags, ACC_PRIVATE) && isSet(flags, ACC_TRANSIENT)) ) {
          fields[numFields++] = &clazz->own_fields[i];
        }
      }
    }
    qsort(fields, numFields, sizeof(w_field), sort_fields);
  }

  /*
  ** We can not directly rule out that constructors or methods do not apply to this algorithm. So
  ** we have to allocate memory, but still can end up with numConstructors and numMethods being 0...
  */

  constructors = allocMem_system(clazz->numDeclaredMethods * sizeof(w_method));
  for (i = 0; i < clazz->numDeclaredMethods; ++i) {
    m = &clazz->own_methods[i];
    if ((m->spec.name == string_angle_brackets_init) && isNotSet(m->flags, ACC_PRIVATE)) {
      constructors[numConstructors++] = m;
    }
  }
  if (numConstructors) {
    qsort(constructors, numConstructors, sizeof(w_method), sort_methods);
  }

  methods = allocMem_system(clazz->numDeclaredMethods * sizeof(w_method));
  for (i = 0; i < clazz->numDeclaredMethods; ++i) {
    m = &clazz->own_methods[i];
    if (m->spec.name != string_angle_brackets_init && m->spec.name != string_angle_brackets_clinit && isNotSet(m->flags, ACC_PRIVATE)) {
      methods[numMethods++] = m;
    }
  }
  if (numMethods) {
    qsort(methods, numMethods, sizeof(w_method), sort_methods);
  }

  /*
  ** Count the number of chars we need to convert back to UTF8, we need this count so we can allocate a suitably size
  ** buffer to convert the strings to UTF8, add the 32 bit modifier integers and calculate the SHA signature from.
  ** Note that we add 2 to each length to accomodate the 2 bytes for the length in the UTF8 byte sequence, and
  ** we add sizeof(w_word) for accomodating the modifiers.
  */
  
  chars = string_length(clazz->dotified) + 2;
  chars += sizeof(w_word);
  
  for (i = 0; i < numInterfaces; i++) {
    chars += string_length(interfaces[i]->dotified) + 2;
    woempa(1, "  INTERFACE %2d: %w\n", i, interfaces[i]->dotified);
  }

  if (clazz->clinit) {
    chars += string_length(clazz->clinit->spec.name);
    if (clazz->clinit->spec.arg_types) {
      for (j = 0; clazz->clinit->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(clazz->clinit->spec.arg_types[j]);
        chars += string_length(tmp);
        deregisterString(tmp);
      }
    }
    tmp = clazz2desc(clazz->clinit->spec.return_type);
    chars += string_length(tmp);
    deregisterString(tmp);
    chars += sizeof(w_word) + 2 + 2;
  }
  
  for (i = 0; i < numFields; i++) {
    chars += string_length(fields[i]->name);
    tmp = clazz2desc(fields[i]->value_clazz);
    chars += string_length(tmp);
    deregisterString(tmp);
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "      FIELD %2d: %w\n", i, fields[i]->name);
  }

  for (i = 0; i < numConstructors; i++) {
    chars += string_length(constructors[i]->spec.name);
    if (constructors[i]->spec.arg_types) {
      for (j = 0; constructors[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(constructors[i]->spec.arg_types[j]);
        chars += string_length(tmp);
        deregisterString(tmp);
      }
    }
    tmp = clazz2desc(constructors[i]->spec.return_type);
    chars += string_length(tmp);
    deregisterString(tmp);
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "CONSTRUCTOR %2d: %m\n", i, constructors[i]);
  }

  for (i = 0; i < numMethods; i++) {
    chars += string_length(methods[i]->spec.name);
    if (methods[i]->spec.arg_types) {
      for (j = 0; methods[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(methods[i]->spec.arg_types[j]);
        chars += string_length(tmp);
        deregisterString(tmp);
      }
    }
    tmp = clazz2desc(methods[i]->spec.return_type);
    chars += string_length(tmp);
    deregisterString(tmp);
    chars += sizeof(w_word) + 2 + 2;
    woempa(1, "     METHOD %2d: %m\n", i, methods[i]);
  }

  /*
  ** Now allocate a working buffer and convert strings back to UTF8, add the modifiers at the same time, for
  ** each w_char we allocate 3 bytes for the worst case UTF8 encoding.
  */
  
  buffer = allocMem_system(chars * (3 * sizeof(w_byte)));
  chars = 0;

  utf8 = string2UTF8(clazz->dotified, &length);
  woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
  w_memcpy(buffer, utf8, length);
  releaseMem(utf8);
  chars += length;

  /*
  ** It seems that this ACC_SUPER flag, which is always set by the compiler, needs to be unset
  ** before calculating the SUID. Try to find this when it's not in the SUN specs for serialization...
  */
  
  flags = clazz->flags;
  unsetFlag(flags, ACC_SUPER);
  chars += addModifiers(buffer + chars, flags);

  for (i = 0; i < numInterfaces; i++) {
    utf8 = string2UTF8(interfaces[i]->dotified, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }

  for (i = 0; i < numFields; i++) {
    utf8 = string2UTF8(fields[i]->name, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, fields[i]->flags);
    woempa(1, "SERIALISE: buffer length = %d\n", chars);
    tmp = clazz2desc(fields[i]->value_clazz);
    utf8 = string2UTF8(tmp, &length);
    deregisterString(tmp);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  
  if (clazz->clinit) {
    utf8 = string2UTF8(clazz->clinit->spec.name, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, ACC_STATIC);
    woempa(1, "SERIALISE: buffer length = %d\n", chars);
    w_memcpy(buffer + chars, "\000\003()V", 5);
    chars += 5;
  }
  
  for (i = 0; i < numConstructors; i++) {
    utf8 = string2UTF8(constructors[i]->spec.name, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, constructors[i]->flags);
    sigsize = 2;
    if (constructors[i]->spec.arg_types) {
      for (j = 0; constructors[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(constructors[i]->spec.arg_types[j]);
        sigsize += string_length(tmp);
        deregisterString(tmp);
      }
    }
    tmp = clazz2desc(constructors[i]->spec.return_type);
    sigsize += string_length(tmp);
    deregisterString(tmp);
    sig = allocMem_system(sigsize * sizeof(w_char));
    sig[0] = '(';
    sigsize = 1;
    if (constructors[i]->spec.arg_types) {
      for (j = 0; constructors[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(constructors[i]->spec.arg_types[j]);
        for (k = 0; k < string_length(tmp); ++k) {
          sig[k + sigsize] = string_char(tmp, k);
        }
        sigsize += string_length(tmp);
        deregisterString(tmp);
      }
    }
    sig[sigsize++] = ')';
    tmp = clazz2desc(constructors[i]->spec.return_type);
    for (k = 0; k < string_length(tmp); ++k) {
      sig[k + sigsize] = string_char(tmp, k);
    }
    sigsize += string_length(tmp);
    deregisterString(tmp);
    zig = unicode2String(W_Thread_system, sig, sigsize);
    releaseMem(sig);
    tmp = slashes2dots(W_Thread_system, zig);
    woempa(1, "SERIALISE: %w descriptor = '%w', dotified = '%w'\n", constructors[i]->spec.name, zig, tmp);
    deregisterString(zig);
    utf8 = string2UTF8(tmp, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    deregisterString(tmp);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  
  for (i = 0; i < numMethods; i++) {
    utf8 = string2UTF8(methods[i]->spec.name, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    chars += addModifiers(buffer + chars, methods[i]->flags);
    sigsize = 2;
    if (methods[i]->spec.arg_types) {
      for (j = 0; methods[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(methods[i]->spec.arg_types[j]);
        sigsize += string_length(tmp);
        deregisterString(tmp);
      }
    }
    tmp = clazz2desc(methods[i]->spec.return_type);
    sigsize += string_length(tmp);
    deregisterString(tmp);
    sig = allocMem_system(sigsize * sizeof(w_char));
    woempa(1, "sigsize %d\n", sigsize);
    sig[0] = '(';
    sigsize = 1;
    if (methods[i]->spec.arg_types) {
      for (j = 0; methods[i]->spec.arg_types[j]; ++j) {
        tmp = clazz2desc(methods[i]->spec.arg_types[j]);
        for (k = 0; k < string_length(tmp); ++k) {
          sig[k + sigsize] = string_char(tmp, k);
        }
        sigsize += string_length(tmp);
        deregisterString(tmp);
      }
    }
    sig[sigsize++] = ')';
    tmp = clazz2desc(methods[i]->spec.return_type);
    for (k = 0; k < string_length(tmp); ++k) {
      sig[k + sigsize] = string_char(tmp, k);
    }
    sigsize += string_length(tmp);
    deregisterString(tmp);
    woempa(1, "sigsize %d\n", sigsize);
    zig = unicode2String(W_Thread_system, sig, sigsize);
    releaseMem(sig);
    tmp = slashes2dots(W_Thread_system, zig);
    woempa(1, "SERIALISE: %w descriptor = '%w', dotified = '%w'\n", methods[i]->spec.name, zig, tmp);
    deregisterString(zig);
    utf8 = string2UTF8(tmp, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    deregisterString(tmp);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
  }
  }
  else {

    /*
    ** It is an array clazz, just take name and modifiers and process along...
    */

    buffer = allocMem_system(2 + string_length(clazz->dotified) * (3 * sizeof(w_byte)));
    chars = 0;
    utf8 = string2UTF8(clazz->dotified, &length);
    woempa(1, "SERIALISE: utf8 = %d:'%s'\n", length-2, utf8+2);
    w_memcpy(buffer + chars, utf8, length);
    releaseMem(utf8);
    chars += length;
    flags = clazz->flags;
    unsetFlag(flags, ACC_SUPER);
    chars += addModifiers(buffer + chars, flags);

  }

    woempa(1, "SERIALISE: buffer length = %d\n", chars);
  sha = allocSha(JNIEnv2w_thread(env));
  processSha(sha, buffer, chars);
  finishSha(sha);
  
//  suid = WORDS2LONG(sha->signature.W[1], sha->signature.W[0]);
// TODO this is a hack, solve this properly...
  suid = sha->signature.B[4];
  suid <<= 8;  suid |= sha->signature.B[5];
  suid <<= 8;  suid |= sha->signature.B[6];
  suid <<= 8;  suid |= sha->signature.B[7];
  suid <<= 8;  suid |= sha->signature.B[0];
  suid <<= 8;  suid |= sha->signature.B[1];
  suid <<= 8;  suid |= sha->signature.B[2];
  suid <<= 8;  suid |= sha->signature.B[3];

  woempa(1,"SERIALISE: suid = %02x %02x %02x %02x %02x %02x %02x %02x\n",
    sha->signature.B[7], sha->signature.B[6], sha->signature.B[5], sha->signature.B[4],
    sha->signature.B[3], sha->signature.B[2], sha->signature.B[1], sha->signature.B[0]);

#ifdef DEBUGGING_SUID
#ifdef LINUX
  {
  static int counter = 0;
  char name[100];
  FILE* f;
  
  memset(name, 0x00, 100);
  sprintf(name, "suid-%d", counter++);
  f = fopen(name, "w");
  fwrite(buffer, 1, chars, f);
  fclose(f);
/*
  memset(name, 0, 100);
  sprintf(name, "(%02x %02x %02x %02x) (%02x %02x %02x %02x)",
    sha->signature.B[7], sha->signature.B[6], sha->signature.B[5], sha->signature.B[4],
    sha->signature.B[3], sha->signature.B[2], sha->signature.B[1], sha->signature.B[0]);
 woempa(1, "LADIES, the suid bytes are '%s'\n", name);
  sprintf(name, "%lld (0x%016llx)", suid, suid);
  woempa(1, "and the suid is '%s'\n", name);
*/  
}
#endif // LINUX
#endif // DEBUGGING_SUID

  if (interfaces) {
    releaseMem(interfaces);
  }

  if (fields) {
    releaseMem(fields);
  }

  if (constructors) {
    releaseMem(constructors);
  }

  if (methods) {
    releaseMem(methods);
  }

  releaseMem(buffer);

  releaseSha(sha);

  /*
  ** Cache the SUID in the clazz structure for the next time around...
  */
  
  clazz->suid = suid;
  
  return suid;

}

