/**************************************************************************
* Copyright  (c) 2001, 2002 by Acunia N.V. All rights reserved.           *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: dispatcher.c 4183 2004-07-06 08:36:49Z dumon $
*/

#include <string.h>

#include "clazz.h"
#include "dispatcher.h"
#include "heap.h"
#include "threads.h"
#include "methods.h"
#include "exception.h"
#include "locks.h"
#include "hashtable.h"
#include "descriptor.h"
#include "misc.h"
#include "wstrings.h"
#include "calls.h"

extern w_hashtable lock_hashtable;
extern w_clazz clazzLinkageError;
extern w_long _call_instance(JNIEnv *env, x_slot top, w_MethodExec *x);  
extern w_long _call_static(JNIEnv *env, w_instance Class, x_slot top, w_MethodExec *x);

/**
** returning a float or double doesn't happen in the same way as an int or a long on ARM.
** a float/double is returned in special flaoting point registers.  By adding these typedef
** and extra functions the compiler will fetch the results from the corresponding registers.
** We need this to be able to return floats and double from jni to wonka.
*/
typedef jfloat  jfloat_call_instance_proto(JNIEnv *env, x_slot top, w_MethodExec *x);
typedef jdouble jdouble_call_instance_proto(JNIEnv *env, x_slot top, w_MethodExec *x);
typedef jfloat  jfloat_call_static_proto(JNIEnv *env, w_instance Class, x_slot top, w_MethodExec *x);
typedef jdouble jdouble_call_static_proto(JNIEnv *env, w_instance Class, x_slot top, w_MethodExec *x);

static jfloat_call_instance_proto * const _call_instance_jfloat =  (jfloat_call_instance_proto*) &_call_instance;
static jdouble_call_instance_proto* const _call_instance_jdouble = (jdouble_call_instance_proto*) &_call_instance;
static jfloat_call_static_proto   * const _call_static_jfloat =    (jfloat_call_static_proto*) &_call_static;
static jdouble_call_static_proto  * const _call_static_jdouble =   (jdouble_call_static_proto*) &_call_static;
static w_ubyte *mangleName(w_word start, w_word stop, w_string source, w_ubyte *destination) {

  w_ubyte *iter = destination;
  w_word i;
  w_char c;
  w_ubyte hex[5];

  for(i = start; i < stop; i++) {
    c = string_char(source, i);
    if(c & 0xFF80) { /* unicode */
      sprintf((w_ubyte *)&hex, "%x", c);
      *(iter++) = '_';
      *(iter++) = '0';
      strcpy(iter, (w_ubyte *)&hex);
      iter += 4;
    } else {
      switch(c & 0x00FF) {
        case '.' : *(iter++) = '_'; break ;
        case '/' : *(iter++) = '_'; break ;
        case '[' : *(iter++) = '_'; *(iter++) = '3'; break;
        case ';' : *(iter++) = '_'; *(iter++) = '2'; break;
        case '_' : *(iter++) = '_'; *(iter++) = '1'; break ;
        default  : *(iter++) = c & 0x00FF;
      }
    }
  }

  return iter;

}

void searchNativeMethod(w_method method) {
  
  /* 
  ** Allocate memory for the JNI names :
  ** - short : e.g Java_java_lang_String_hashcode is fairly easy, we need 5 (Java_) + 1 
  **           (the _ between class & method) + 1 (\0 at the end) = 7 extra bytes + a few
  **           more for mangling
  ** - long  : Different story because [ is changed in _3 and ; is changed in _2. About 25
  **           extra bytes should cover it, but this is dangerous and should be changed
  */
  
  w_ubyte *jni_short = allocMem_system(7 + string_length(method->spec.declaring_clazz->dotified) + string_length(method->spec.name) + 25);
  w_size   jni_long_length;
  w_ubyte *jni_long;
  w_ubyte *iter = jni_short;
  w_size   i;
  
  strcpy(jni_short, "Java_");  /* First put in the Java_ at the beginning */
  iter += 5;                   /* Skip 5 characters */

  /*
  ** Go over the class (+package) name and add those to the jni name. Also change /'s into _'s 
  */

  iter = mangleName(0, string_length(method->spec.declaring_clazz->dotified), method->spec.declaring_clazz->dotified, iter);
  
  *(iter++) = '_';             /* The _ between class & method */
  
  /*
  ** Go over the method name and add those to the jni name. 
  */
  
  iter = mangleName(0, string_length(method->spec.name), method->spec.name, iter);
  
  *(iter++) = '\0';            /* End of the string */

  woempa(9, "JNI short : '%s'\n", jni_short);

  /*
  ** For the full blown name, we start from the short name and add the parameters
  */
  
  jni_long_length = 7 + string_length(method->spec.declaring_clazz->dotified) + string_length(method->spec.name) + 2;
  if (method->spec.arg_types) {
    for (i = 0; method->spec.arg_types[i]; ++i) {
      jni_long_length += string_length(method->spec.arg_types[i]->dotified) + 2;
    }
  }
  jni_long_length += string_length(method->spec.return_type->dotified) + 27;
  jni_long = allocMem_system(jni_long_length);
  strcpy(jni_long, jni_short); /* Make a copy */
  
  iter = (w_ubyte *)(jni_long + strlen(jni_short));  /* Update our iteration pointer */
  *(iter++) = '_';             /* Add the 2 _'s between method and parameter */
  *(iter++) = '_';

  /*
  ** Go over the parameters and add those to the jni name. Also change /'s in _'s, 
  ** ;'s in _2's and ['s in _3's
  */
  
  if (method->spec.arg_types) {
    for (i = 0; method->spec.arg_types[i]; ++i) {
      w_string desc = clazz2desc(method->spec.arg_types[i]);
      iter = mangleName(0, string_length(desc), desc, iter);
      deregisterString(desc);
    }
  }

  *(iter++) = '\0';            /* End of the string */
  
  woempa(9, "JNI long :  '%s'\n", jni_long);

  method->exec.function.void_fun = lookupModuleSymbol(jni_long);
  if (method->exec.function.void_fun == NULL) {
    method->exec.function.void_fun = lookupModuleSymbol(jni_short);
  }
  releaseMem(jni_short);       /* Be a good boy and return the toys after we */
  releaseMem(jni_long);        /* finnished playing */
}

void initialize_native_dispatcher(x_frame caller, w_method method) {

  w_int i;

  if (method->exec.function.void_fun == NULL) {
    woempa(9, "Oh deary me: method %M has no code -> Trying to look it up\n", method);
    searchNativeMethod(method);
    if (method->exec.function.void_fun == NULL) {
      woempa(9, "Oh deary me: method %M has no code...\n", method);
      throwException(caller->thread, clazzLinkageError, "native method %M has no code...\n", method);
      return;
    }
  }

  if (isSet(method->flags, ACC_STATIC)) {
    i = 17;
  }
  else {
    i = 5;
  }

  if (isNotSet(method->flags, ACC_SYNCHRONIZED)) {
    i += 6;
  }

  if (method->spec.return_type == clazz_void) {
    i += 5;
  }
  else if (method->exec.return_i == 2) {
    if(method->spec.return_type == clazz_double){
      i++;
    }
    i += 3;
  }
  else if (isSet(method->spec.return_type->flags, CLAZZ_IS_PRIMITIVE)) {
    if(method->spec.return_type == clazz_float){
      i++;
    }
    i += 1;
  }

  method->exec.dispatcher = i;

  callMethod(caller, method);

}

inline static void prepareNativeFrame(x_frame frame, w_thread thread, x_frame caller, w_method method) {

  frame->flags = 0;
  frame->previous = caller;
  frame->thread = thread;
  frame->method = method;
  frame->auxs = caller->auxs;
  frame->top = frame->Slots;
  frame->Slots[0].s = stack_notrace;

}

void native_instance_synchronized_reference(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i); 
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;

  frame->top[0].c.w = 0;
  frame->top[0].s = stack_trace;
  frame->top += 1;
  
  caller->top[idx].c.w = (w_word) _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);
  caller->top[idx].s = stack_trace;
  caller->top += (idx + 1);

  x_monitor_exit(m);
  
  thread->top = caller;

}

void native_instance_synchronized_32bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  result.u64 = _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top += 1;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_instance_synchronized_float(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);

#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif
  
  thread->top = frame;

  {
    //we have a float ...
    union {
      w_word u32;
      float   f;
    } result_float;

    result_float.f = _call_instance_jfloat(w_thread2JNIEnv(thread), caller->top, &method->exec);
    result.u64 = result_float.u32;
  }

  caller->top[idx].s = stack_notrace;
  caller->top[idx].c.w = result.words[WORD_LSW];
  caller->top += (idx + 1);

  x_monitor_exit(m);

  thread->top = caller;

}

void native_instance_synchronized_64bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  result.u64 = _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_instance_synchronized_double(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_enter(m, x_eternal);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;

  {
    //we have a double ...
    union {
      w_long u64;
      double   d;
    } result_double;

    result_double.d = _call_instance_jdouble(w_thread2JNIEnv(thread), caller->top, &method->exec);
    result.u64 = result_double.u64;
  }

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  x_monitor_exit(m);

  thread->top = caller;

}
void native_instance_synchronized_void(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = (w_instance) caller->top[idx].c.w;
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top += idx;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_instance_unsynchronized_reference(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;
  
  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  frame->top[0].c.w = 0;
  frame->top[0].s = stack_trace;
  frame->top += 1;
  
  caller->top[idx].c.w = (w_word) _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);
  caller->top[idx].s = stack_trace;
  caller->top += (idx + 1);

  thread->top = caller;
}

void native_instance_unsynchronized_32bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_word  result;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  result = (w_word) _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top -= method->exec.arg_i;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result;
  caller->top += 1;

  thread->top = caller;

}

void native_instance_unsynchronized_float(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_word  result;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;

  {
    //we have a float ...
    union {
      w_word u32;
      float   f;
    } result_float;

    result_float.f = _call_instance_jfloat(w_thread2JNIEnv(thread), caller->top, &method->exec);
    result = result_float.u32;
  }

  caller->top -= method->exec.arg_i;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result;
  caller->top += 1;

  thread->top = caller;

}

void native_instance_unsynchronized_64bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  result.u64 = _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  thread->top = caller;

}

void native_instance_unsynchronized_double(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;

  {
    //we have a double ...
    union {
      w_long u64;
      double   d;
    } result_double;

    result_double.d = _call_instance_jdouble(w_thread2JNIEnv(thread), caller->top, &method->exec);
    result.u64 = result_double.u64;

  }
  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  thread->top = caller;

}

void native_instance_unsynchronized_void(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  _call_instance(w_thread2JNIEnv(thread), caller->top, &method->exec);

  caller->top += idx;

  thread->top = caller;

}

void native_static_synchronized_reference(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  frame->top[0].c.w = 0;
  frame->top[0].s = stack_trace;
  frame->top += 1;
  
  caller->top[idx].c.w = (w_word) _call_static(w_thread2JNIEnv(thread), o, caller->top, &method->exec);
  caller->top[idx].s = stack_trace;
  caller->top += (idx + 1);

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_synchronized_32bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  result.u64 = _call_static(w_thread2JNIEnv(thread), o, caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top += 1;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_synchronized_float(x_frame caller, w_method method) { /* TODO */

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot));
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_enter(m, x_eternal);

  thread->top = frame;

  {
    //we have a float ...
    union {
      w_word u32;
      float   f;
    } result_float;

    result_float.f = _call_static_jfloat(w_thread2JNIEnv(thread), o, caller->top, &method->exec);
    result.u64 = result_float.u32;
  }

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top += 1;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_synchronized_64bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  result.u64 = _call_static(w_thread2JNIEnv(thread), o, caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_synchronized_double(x_frame caller, w_method method) { /* TODO */

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot));
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_enter(m, x_eternal);

  thread->top = frame;

  {
    //we have a double ...
    union {
      w_long u64;
      double   d;
    } result_double;

    result_double.d = _call_static_jdouble(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);
    result.u64 = result_double.u64;
  }

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_synchronized_void(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_instance o;
  x_monitor m;
  w_thread thread = caller->thread;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe;
#endif

  prepareNativeFrame(frame, thread, caller, method);

#ifdef GC_SAFE_POINTS
  unsafe = enterSafeRegion(thread);
#endif
  o = clazz2Class(frame->method->spec.declaring_clazz);
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }
  x_monitor_eternal(m);
#ifdef GC_SAFE_POINTS
  if (unsafe) {
    enterUnsafeRegion(thread);
  }
#endif

  thread->top = frame;
  
  _call_static(w_thread2JNIEnv(thread), o, caller->top, &method->exec);

  caller->top += idx;

  x_monitor_exit(m);

  thread->top = caller;

}

void native_static_unsynchronized_reference(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  frame->top[0].c.w = 0;
  frame->top[0].s = stack_trace;
  frame->top += 1;
  
  caller->top[idx].c.w = (w_word) _call_static(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);
  caller->top[idx].s = stack_trace;
  caller->top += (idx + 1);
  
  thread->top = caller;

}

void native_static_unsynchronized_32bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  result.u64 = _call_static(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top += 1;

  thread->top = caller;

}

void native_static_unsynchronized_float(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;

  {
    //we have a float ...
    union {
      w_word u32;
      float   f;
    } result_float;
    result_float.f = _call_static_jfloat(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);
    result.u64 = result_float.u32;
  }

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top += 1;

  thread->top = caller;

}

void native_static_unsynchronized_64bits(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  result.u64 = _call_static(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  thread->top = caller;

}

void native_static_unsynchronized_double(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_u64 result;
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  {
    //we have a double ...
    union {
      w_long u64;
      double   d;
    } result_double;

    result_double.d = _call_static_jdouble(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);

    result.u64 = result_double.u64;
  }

  caller->top += idx;
  caller->top[0].s = stack_notrace;
  caller->top[0].c.w = result.words[WORD_LSW];
  caller->top[1].s = stack_notrace;
  caller->top[1].c.w = result.words[WORD_MSW];
  caller->top += 2;

  thread->top = caller;

}

void native_static_unsynchronized_void(x_frame caller, w_method method) {

  x_frame frame = alloca(sizeof(x_Frame) + sizeof(x_Slot) + sizeof(x_Slot) * method->exec.local_i);
  w_int idx = - method->exec.arg_i;
  w_thread thread = caller->thread;

  prepareNativeFrame(frame, thread, caller, method);

  thread->top = frame;
  
  _call_static(w_thread2JNIEnv(thread), clazz2Class(frame->method->spec.declaring_clazz), caller->top, &method->exec);

  caller->top += idx;

  thread->top = caller;

}

/*
    0             syn           Bytecode
    1             uns

    2       inst  syn  rf      Native
    3       inst  syn  32
    4       inst  syn  64
    5       inst  syn  vd

    6       inst  uns  rf
    7       inst  uns  32
    8       inst  uns  64
    9       inst  uns  vd
  
   10       stat  syn  rf
   11       stat  syn  32
   12       stat  syn  64
   13       stat  syn  vd

   14       stat  uns  rf
   15       stat  uns  32
   16       stat  uns  64
   17       stat  uns  vd

   18       inst  syn  rf        J-Spot
   19       inst  syn  32
   20       inst  syn  64
   21       inst  syn  vd

   22       inst  uns  rf
   23       inst  uns  32
   24       inst  uns  64
   25       inst  uns  vd
  
   26       stat  syn  rf
   27       stat  syn  32
   28       stat  syn  64
   29       stat  syn  vd

   30       stat  uns  rf
   31       stat  uns  32
   32       stat  uns  64
   33       stat  uns  vd

   34            discard         bytecode that just vreturns

*/
