/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: exception.c 3486 2003-02-12 10:45:31Z gray $
*/

#include <string.h>
#include <stdarg.h>
#include <stdio.h>

#include "clazz.h"
#include "constant.h"
#include "exception.h"
#include "heap.h"
#include "interpreter.h"
#include "loading.h"
#include "realms.h"
#include "oswald.h"
#include "Throwable.h"
#include "wstrings.h"
#include "threads.h"
#include "fields.h"
#include "ts-mem.h"
#include "methods.h"
#include "descriptor.h"
#include "core-classes.h"
#include "checks.h"

#define dumpStackTrace(thrown)

char * print_exception(char * buffer, int * remain, void * data, int w, int p, unsigned int f) {
  w_instance exception = data;
  w_int    nbytes;
  char    *temp;
  
  if (*remain < 1) {

    return buffer;

  }

  temp = buffer;

  if (exception == NULL) {
    strncpy(temp, (char *)"<NULL>", *remain);
    if (*remain < 6) {
      temp += *remain;
      *remain = 0;
    }
    else {
      temp += 6;
      *remain -= 6;
    }

    return temp;

  }

  nbytes = x_snprintf(temp, *remain, "%k : %w", instance2clazz(exception), String2string(getReferenceField(exception, F_Throwable_detailMessage)));

  *remain -= nbytes;

  return temp + nbytes;
}

void throwOutOfMemoryError(w_thread thread) {
  if (!thread->Thread) {
    wabort(ABORT_WONKA, "%t has no Thread in which to throw OutOfMemoryError\n", thread);
  }

  if (instance2clazz(thread->exception) == clazzOutOfMemoryError) {
    setReferenceField(thread->Thread,  (w_instance)thread->exception, F_Thread_thrown);
  }
  else if(!exceptionThrown(thread)) {
    woempa(9,"FREE MEMORY %i TOTAL MEMORY %i\n",x_mem_avail(),  x_mem_total());
    thread->exception = allocInstance(thread, clazzOutOfMemoryError);
  }
  else {
    woempa(9, "OutOfMemoryError thrown when %e already pending - ignoring OutOfMemoryError\n", thread->exception);
  }
}

void fillThrowable(w_thread thread, w_instance Throwable) {

  x_frame frame;
  w_int n;
  w_int i;
  x_Exr * records = getWotsitField(Throwable, F_Throwable_records);

  /*
  ** No need to fill the same throwable twice; it would result in a memory
  ** leak. Been there, done that...
  */
  
  if (records == NULL) {
    for (n = 0, frame = thread->top; frame; frame = frame->previous) {
      if (frame->method) {
        n += 1;
      }
    }

    records = allocClearedMem_system(sizeof(x_Exr) * n);
    if (records) {
      setWotsitField(Throwable, F_Throwable_records,  records);
      for (i = 0, frame = thread->top; frame; frame = frame->previous) {
        if (frame->method) {
          records[i].method = frame->method;
          records[i].pc = frame->current - frame->method->exec.code;
          records[i].position = --n;
          i += 1;
        }
      }
    }
  }

}

w_instance clearException(w_thread thread) {

  w_instance exception = thread->exception;

// We should protect the cleared exception somehow...
//  thread->top->auxs[0].c = exception;
//  thread->top->auxs[0].s = stack_exception;
//  thread->top->auxs += 1;
  thread->exception = NULL; // but leave the thrown field unchanged for protection from GC

  return exception;

}

void throwExceptionInstance(w_thread thread, w_instance Throwable) {

  if (!thread->Thread) {
    woempa(9, "%t has no Thread in which to throw %j - ignoring it\n", thread, Throwable);

    return;

  }

  if (thread->exception == Throwable) {
  // Specially 'coz interpreter.c uses throwExceptionInstance(thread, thread->exception)
    woempa(7, "Throwing %e in %t\n", Throwable, thread);
    setReferenceField(thread->Thread, Throwable, F_Thread_thrown);
  }
  else if(!exceptionThrown(thread)) {
    woempa(7, "Throwing %e in %t\n", Throwable, thread);
    thread->exception = Throwable;
    setReferenceField(thread->Thread, Throwable, F_Thread_thrown);
  }
  else {
    woempa(7, "Exception %e is already pending\n", thread->exception);
    woempa(7, "--> ignoring %e \n", Throwable);
  }
}

static const w_int bufsize = 1024;

void throwException(w_thread thread, w_clazz exception, char * format, ...) {

  va_list ap;
  char * buffer;
  w_int length;
  w_string message = NULL;
  w_instance Message;
  w_instance Throwable;

  if (exceptionThrown(thread)) {

    return;

  }

  if (format) {
    buffer = allocMem_system(bufsize + 1);
    if (buffer) {  
      va_start (ap, format);
      length = x_vsnprintf(buffer, bufsize, format , ap);
      va_end (ap);
      message = cstring2String(W_Thread_system, buffer, length);
      releaseMem(buffer);
    }
  }
  
  woempa(7, "Throwing %k with message `%w' in %t\n", exception, message, thread);

  Throwable = allocInstance(thread, exception);

  if (Throwable) {
    thread->exception = Throwable;

    setReferenceField(thread->Thread, Throwable, F_Thread_thrown);
  
    if (message) {
      Message = newStringInstance(message);
      if (Message) {
        setReferenceField(Throwable, Message, F_Throwable_detailMessage);
        deregisterString(message);
      }
    }

    removeLocalReference(thread, Throwable);
    
  }

}

