/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips-site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: interpreter.c 4293 2006-02-08 14:28:09Z gerrit $
*/

#include <stdio.h>
#include <string.h>
#include <stdarg.h>

#include "arrays.h"
#include "clazz.h"
#include "constant.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "hashtable.h"
#include "ieee754.h"
#include "heap.h"
#include "interpreter.h"
#include "dispatcher.h"
#include "loading.h"
#include "locks.h"
#include "ts-mem.h"
#include "methods.h"
#include "opcodes.h"
#include "wstrings.h"
#include "threads.h"
#include "checks.h"
#include "jdwp.h"
#include "jdwp_events.h"
#include "profile.h"

// #define woempa(level, ...) _woempa(__FILE__, __FUNCTION__, __LINE__, level, __VA_ARGS__)
void _woempa(const char *file, const char *function, int line, int level, const char *format, ...);

#ifdef JSPOT
 #include "hotspot.h"
#endif

extern char * instruction2char[];
extern w_hashtable lock_hashtable;

void trace(x_frame frame);

extern w_int woempa_bytecodecount;
long long bcc = 0;

//#define TRIGGER 4800000 // 4782340
#define TRIGGER 9920900

#ifdef DEBUG
#ifdef bar

static void stackCheck(x_frame frame) {

  x_slot base = frame->Slots + frame->method->exec.local_i;
  x_slot limit = base + frame->method->exec.stack_i;
  x_slot slot;
  w_instance instance;
  w_int i = 0;
  static w_long checks = 0;

  static w_string skip_1 = NULL;
  static w_string skip_2 = NULL;
  static w_string skip_3 = NULL;
  static w_string skip_4 = NULL;
  
  if (skip_1 == NULL) {
    skip_1 = cstring2String(W_Thread_system, "toString_internal", strlen("toString_internal"));
    skip_2 = cstring2String(W_Thread_system, "test_remainder", strlen("test_remainder"));
    skip_3 = cstring2String(W_Thread_system, "check_remainder", strlen("check_remainder"));
    skip_4 = cstring2String(W_Thread_system, "test_random", strlen("test_random"));
  }

  checks += 1;
  
  /*
  ** Check for overflow and underflow...
  */
    
  if (frame->top < base) {
    woempa(9, "Stack underflow...\n");
    abort();
  }
  
  if (frame->top > limit) {
    woempa(9, "!!!! Stack overflow (%d max, %d cur)...\n", frame->method->exec.stack_i, frame->top - base);
    woempa(9, "Method %M\n", frame->method);
    abort();
  }

  /*
  ** Check for stuff on the stack and in locals that is marked traceable and is not an object.
  */

  i = 0;
  for (slot = frame->Slots; slot < frame->Slots + frame->method->exec.local_i; slot += 1) {
    instance = (w_instance) slot->c;
    if (slot->c) {
      if (slot->s == stack_trace && ! (x_mem_tag_get(block2chunk(instance2object(instance))) & OBJECT_TAG)) {
        woempa(9, "A %lld opcodes done...\n", checks);
        woempa(9, "A Method %M, slot %d\n", frame->method, i);
        woempa(9, "A Method takes %d argument slots.\n", frame->method->exec.arg_i);
        abort();
      }
    }
    i += 1;
  }

  i = 0;
  for (slot = frame->Slots + frame->method->exec.local_i; slot < frame->top; slot += 1) {
    instance = (w_instance) slot->c;
    if (slot->c) {
      if (slot->s == stack_trace && ! (x_mem_tag_get(block2chunk(instance2object(instance))) & OBJECT_TAG)) {
        woempa(9, "B %lld opcodes done, reference %p...\n", checks, instance);
        woempa(9, "B Method %M, item %d\n", frame->method, i);
        woempa(9, "B Method takes %d stack slots.\n", frame->method->exec.stack_i);
        abort();
      }
    }
  }

  /*
  ** See if anything unprotected is on the stack or slots...
  */

  if (frame->method->spec.name == skip_3) return;

  i = 0;
  for (slot = frame->Slots; slot < frame->Slots + frame->method->exec.local_i; slot += 1) {
    instance = (w_instance) slot->c;
    if (slot->s == stack_notrace && (unsigned char *) instance > heap_start && (unsigned char *) instance < heap_current) {
      if (x_mem_is_block(block2chunk(instance2object(instance)))) {
        if (x_mem_tag_get(block2chunk(instance2object(instance))) & OBJECT_TAG) {
          woempa(9, "C %lld opcodes done %lld...\n", checks, bcc);
          woempa(9, "C Method %M, slot %d\n", frame->method, i);
          woempa(9, "C Method takes %d argument slots.\n", frame->method->exec.arg_i);
          abort();
        }
      }
    }
    i += 1;
  }

  if (frame->method->spec.name == skip_1) return;
  else if (frame->method->spec.name == skip_2) return;
  else if (frame->method->spec.name == skip_4) return;

  i = 0;
  for (slot = frame->Slots + frame->method->exec.local_i; slot < frame->top; slot += 1) {
    instance = (w_instance) slot->c;
    if (slot->s == stack_notrace && (unsigned char *) instance > heap_start && (unsigned char *) instance < heap_current) {
      if (x_mem_is_block(block2chunk(instance2object(instance)))) {
        if (x_mem_tag_get(block2chunk(instance2object(instance))) & OBJECT_TAG) {
          woempa(9, "D %lld opcodes done, reference %p...\n", checks, instance);
          woempa(9, "D Method %M, item %d\n", frame->method, i);
          woempa(9, "D Method takes %d stack slots.\n", frame->method->exec.stack_i);
          abort();
        }
      }
    }
  }

}

#endif /* bar */
#endif /* DEBUG */

/*
** Function to copy the stack top arguments of the caller to
** the locals argument slots of the callee. Uses loop unrolling
** for the final 16 or lower number of arguments. The 'num' argument
** contains the number of argument slots that this function takes.
*/

inline static void stack2locals(x_Slot locals[], x_Slot args[], w_int num) {

  /*
  ** For a straightforward copy, we first decrement the 'args' argument,
  ** that points to the (empty) top of the stack, with the number of
  ** argument words required.
  */

  args -= num;

  switch (num) {
    default: {
      while (16 < num--) {
        locals[num].c = args[num].c;
        locals[num].s = args[num].s;
      }
    }
    
    case 16: locals[15].c = args[15].c; locals[15].s = args[15].s;
    case 15: locals[14].c = args[14].c; locals[14].s = args[14].s;
    case 14: locals[13].c = args[13].c; locals[13].s = args[13].s;
    case 13: locals[12].c = args[12].c; locals[12].s = args[12].s;
    case 12: locals[11].c = args[11].c; locals[11].s = args[11].s;
    case 11: locals[10].c = args[10].c; locals[10].s = args[10].s;
    case 10: locals[ 9].c = args[ 9].c; locals[ 9].s = args[ 9].s;
    case  9: locals[ 8].c = args[ 8].c; locals[ 8].s = args[ 8].s;
    case  8: locals[ 7].c = args[ 7].c; locals[ 7].s = args[ 7].s;
    case  7: locals[ 6].c = args[ 6].c; locals[ 6].s = args[ 6].s;
    case  6: locals[ 5].c = args[ 5].c; locals[ 5].s = args[ 5].s;
    case  5: locals[ 4].c = args[ 4].c; locals[ 4].s = args[ 4].s;
    case  4: locals[ 3].c = args[ 3].c; locals[ 3].s = args[ 3].s;
    case  3: locals[ 2].c = args[ 2].c; locals[ 2].s = args[ 2].s;
    case  2: locals[ 1].c = args[ 1].c; locals[ 1].s = args[ 1].s;
    case  1: locals[ 0].c = args[ 0].c; locals[ 0].s = args[ 0].s;
    case  0: break;

  }

}

static w_byte truncate_i2b(w_int i) {
  return (w_char) (i & 0x000000ff);
}

static w_int extend_b2i(w_word b) {
  return ((b & 0x80) ? (b | 0xffffff00) : (b & 0x0000007f));
}

static w_int extend_s2i(w_int s) {
  return ((s & 0x8000) ? (w_int) (s | 0xffff0000) : (w_int) (s & 0x00007fff));
}

static void do_xload(x_frame frame, w_word slot) {

  frame->top[0].s = stack_notrace;
  frame->top[0].c.w = frame->Slots[slot].c.w;
  frame->top += 1;

}

inline static void do_xpush(x_frame frame, w_word value) {

  frame->top[0].s = stack_notrace;
  frame->top[0].c.w = value;
  frame->top += 1;

}

inline static void do_xstore(x_frame frame, w_word slot) {

  frame->top -= 1;
  frame->Slots[slot].s = stack_notrace;
  frame->Slots[slot].c.w = frame->top[0].c.w;

}

inline static void do_aload(x_frame frame, w_word slot) {

  frame->top[0].s = stack_trace;
//  frame->top[0].s = frame->Slots[slot].s; // According to the spec, we could set 'stack_trace' explicitely ...
  frame->top[0].c.w = frame->Slots[slot].c.w; // ... but you never know what compilers produce as code.
  frame->top += 1;

}

inline static void do_astore(x_frame frame, w_word slot) {

  frame->Slots[slot].s = stack_notrace;    // Set safe situation for GC if we would be interrupted before assigning value
  frame->Slots[slot].c.w = frame->top[-1].c.w; // The top of stack can also contain return address of 'jsr' ...
  frame->Slots[slot].s = frame->top[-1].s; // ... so use tag of stack, not 'stack_trace' explicitely.
  frame->top -= 1;

}

inline static void do_zpush(x_frame frame, w_word msw, w_word lsw) {

  frame->top[1].s = stack_notrace;
  frame->top[1].c.w = msw;
  frame->top[0].s = stack_notrace;
  frame->top[0].c.w = lsw;
  frame->top += 2;

}

inline static void do_zload(x_frame frame, w_word slot) {

  frame->top[1].s = stack_notrace;
  frame->top[1].c.w = frame->Slots[slot + 1].c.w;
  frame->top[0].s = stack_notrace;
  frame->top[0].c.w = frame->Slots[slot + 0].c.w;
  frame->top += 2;

}

inline static void do_zstore(x_frame frame, w_word slot) {

  frame->top -= 2;
  frame->Slots[slot + 0].s = stack_notrace;
  frame->Slots[slot + 0].c.w = frame->top[0].c.w;
  frame->Slots[slot + 1].s = stack_notrace;
  frame->Slots[slot + 1].c.w = frame->top[1].c.w;

}

static void do_frem(x_frame frame);
static void do_drem(x_frame frame);

#define byte_operand             (* (++current))
#define short_operand            ((signed short)(current[1] << 8 | current[2]))
#define int_operand              ((signed int)((current[1] << 24) | (current[2] << 16) | (current[3] << 8) | current[4]))

#ifdef DEBUG
inline static void updateDebugInfo(x_frame frame, w_code current) { 
  frame->current = current + 1;
  woempa_bytecodecount += 1; 
}
#else
inline static void updateDebugInfo(x_frame frame, w_code current) {}
#endif

#define do_next_opcode           {        \
  updateProfileBytecodes(frame);          \
  updateDebugInfo(frame, current + 1);    \
  goto * jumps[*(++current)];             \
}

#define do_this_opcode           {        \
  updateProfileBytecodes(frame);          \
  updateDebugInfo(frame, current);        \
  goto * jumps[*current];                 \
}

#define add_to_opcode(a)        {         \
  updateProfileBytecodes(frame);          \
  current += (a);                         \
  updateDebugInfo(frame, current);        \
  goto * jumps[*current];                 \
}

#define do_conditional(c)        {        \
  updateProfileBytecodes(frame);          \
  if (!(c)) {                             \
    current += 3;                         \
    updateDebugInfo(frame, current);      \
    goto * jumps[*current];               \
  }                                       \
  else {                                  \
    if (pendingGCRequest(thread)) {       \
      gcSafePoint(thread);                \
    }                                     \
    current += short_operand;             \
    updateDebugInfo(frame, current);      \
    goto * jumps[*current];               \
  }                                       \
}

#define do_the_exception         {        \
  goto c_exception;                       \
}

#define do_throw_clazz(x)        {        \
  c = (x);                                \
  goto c_clazz2exception;                 \
}

#define do_throw_clazz_msg(x, a...) {     \
  throwException(thread, x, ##a);         \
  goto c_exception;                       \
}

w_method do_invokeinterface(x_frame frame, w_int i);

static w_code searchHandler(x_frame frame);

#ifdef bar
typedef struct w_Hist {
  w_int code;
  w_int called;
} w_Hist;

typedef struct w_Hist * w_hist;

w_Hist hist[256];

int histcompare(const void * a, const void * b) {

  w_hist ah = (w_hist) a;
  w_hist bh = (w_hist) b;
  
  if (ah->called > bh->called) return -1;
  else if (ah->called < bh->called) return 1;
  return 0;
  
}

void histogram(void) {
  int i;
  for (i = 0; i < 255; i++) {
    hist[i].code = i;
  }
  qsort(hist, 255, sizeof(w_Hist), histcompare);
  for (i = 0; i < 255; i++) {
    printf("%-25s %8d\n", instruction2char[hist[i].code], hist[i].called);
  }
}
#endif

void interprete(x_frame caller, w_method method);

void void_return_only(x_frame caller, w_method method) {
  caller->top -= method->exec.arg_i;
}

void initialize_interprete_dispatcher(x_frame caller, w_method method) {

  w_int i;
 
#ifdef JSPOT
  i = 21;  // interprete_profiled
#else
  i = 1;   // interprete
#endif
  
  if (isSet(method->flags, ACC_SYNCHRONIZED)) {
    if (isSet(method->flags, ACC_STATIC)) {
      i += 2;
    }
    else {
      i += 1;
    }
  }
  else if (method->exec.code[0] == 177) {
    i = 4;
  }

  method->exec.dispatcher = i;
  callMethod(caller, method);
  
}

void interprete_instance_synchronized(x_frame caller, w_method method) {

  w_instance o;
  x_monitor m;
#ifdef GC_SAFE_POINTS
  w_thread  thread = caller->thread;
#endif
  
  o = (w_instance) caller->top[- method->exec.arg_i].c.w;

#ifdef GC_SAFE_POINTS
  enterSafeRegion(thread);
#endif
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }

  x_monitor_eternal(m);
#ifdef MONITOR_INFO
  m->xref = caller;
#endif
#ifdef GC_SAFE_POINTS
  enterUnsafeRegion(thread);
#endif

  interprete(caller, method);
  
  x_monitor_exit(m);
#ifdef MONITOR_INFO
  m->xref = NULL;
#endif

}

void interprete_static_synchronized(x_frame caller, w_method method) {

  w_instance o;
  x_monitor m;
#ifdef GC_SAFE_POINTS
  w_thread  thread = caller->thread;
#endif

//  woempa(9, "STATIC METHOD %M: length = %d bytes\n", method, method->exec.code_length);
  o = clazz2Class(method->spec.declaring_clazz);

#ifdef GC_SAFE_POINTS
  enterSafeRegion(thread);
#endif
  if (isSet(instance2flags(o), O_HAS_LOCK)) {
    m = (x_monitor) ht_read(lock_hashtable, (w_word) o);
  }
  else {
    m = allocMonitor(o);
  }

  x_monitor_eternal(m);
#ifdef MONITOR_INFO
  m->xref = caller;
#endif
#ifdef GC_SAFE_POINTS
  enterUnsafeRegion(thread);
#endif

  interprete(caller, method);
  
  x_monitor_exit(m);
#ifdef MONITOR_INFO
  m->xref = NULL;
#endif

}

#ifndef JSPOT
static void trap(x_frame caller, w_method method) {
  woempa(9, "Trapped %M\n", method);
  abort();
}
#endif

void interprete_profiled(x_frame caller, w_method method) {

#ifdef JSPOT
  hs_method_invoke(method);
  interprete(caller, method);
  hs_method_return(method);
#else
  trap(caller, method);
#endif

}

void interprete_instance_synchronized_profiled(x_frame caller, w_method method) {

#ifdef JSPOT
  hs_method_invoke(method);
  interprete_instance_synchronized(caller, method);
  hs_method_return(method);
#else
  trap(caller, method);
#endif

}

void interprete_static_synchronized_profiled(x_frame caller, w_method method) {

#ifdef JSPOT
  hs_method_invoke(method);
  interprete_static_synchronized(caller, method);
  hs_method_return(method);
#else
  trap(caller, method);
#endif

}

void initialize_dispatcher(x_frame caller, w_method method) {

  if (isSet(method->flags, ACC_STATIC)) {
    classMustBeInitialized(method->spec.declaring_clazz);
    if (caller->thread->exception) {
      woempa(9, "Initializing %k threw exception %k.\n", method->spec.declaring_clazz, instance2clazz(caller->thread->exception));
      return;
    }
  }

  if (isSet(method->flags, ACC_NATIVE)) {
    initialize_native_dispatcher(caller, method);
  }
  else {
    initialize_interprete_dispatcher(caller, method);
  }

}

w_callfun dispatchers[] = {

  initialize_dispatcher,                       //  0  will select the correct dispatcher the first time

  interprete,                                  //  1  interprete, unsynchronized, either static or instance calls
  interprete_instance_synchronized,            //  2  instance synchronized call
  interprete_static_synchronized,              //  3  static synchronized call
  void_return_only,                            //  4  the dispatcher for Java methods that only do a 'return'

  native_instance_synchronized_reference,      //  5
  native_instance_synchronized_32bits,         //  6
  native_instance_synchronized_float,          //  7
  native_instance_synchronized_64bits,         //  8
  native_instance_synchronized_double,         //  9
  native_instance_synchronized_void,           // 10

  native_instance_unsynchronized_reference,    // 11
  native_instance_unsynchronized_32bits,       // 12
  native_instance_unsynchronized_float,        // 13
  native_instance_unsynchronized_64bits,       // 14
  native_instance_unsynchronized_double,       // 15
  native_instance_unsynchronized_void,         // 16

  native_static_synchronized_reference,        // 17
  native_static_synchronized_32bits,           // 18
  native_static_synchronized_float,            // 19
  native_static_synchronized_64bits,           // 20
  native_static_synchronized_double,           // 21
  native_static_synchronized_void,             // 22

  native_static_unsynchronized_reference,      // 23
  native_static_unsynchronized_32bits,         // 24
  native_static_unsynchronized_float,          // 25
  native_static_unsynchronized_64bits,         // 26
  native_static_unsynchronized_double,         // 27
  native_static_unsynchronized_void,           // 28

  /* dispatchers for J-spot or profiling:  */
  
  interprete_profiled,                         // 29  interprete, unsynchronized, profiled, either static or instance calls
  interprete_instance_synchronized_profiled,   // 30  instance synchronized profiled call
  interprete_static_synchronized_profiled,     // 31  static synchronized profiled call

};

inline static void i_callMethod(x_frame caller, w_method method) {

  w_thread thread = caller->thread;
#ifdef JAVA_PROFILE
  x_long time_start;    
  x_long time_delta;    
#endif

  if (thread->numframes) {
#ifdef GC_SAFE_POINTS
    w_boolean unsafe_to_safe = isSet(thread->flags, WT_THREAD_NOT_GC_SAFE) && isSet(method->flags, ACC_NATIVE);                                               
    w_boolean safe_to_unsafe = isNotSet(thread->flags, WT_THREAD_NOT_GC_SAFE) && isNotSet(method->flags, ACC_NATIVE);
    
    if (unsafe_to_safe) {
      enterSafeRegion(thread);
    }
    else if (safe_to_unsafe) {
      enterUnsafeRegion(thread);
    }

   
#endif
/*    if (isSet(method->flags, ACC_NATIVE) && (unsigned int)method->exec.function.void_fun > 0x40000000) {
      w_dump("[JNI] %p %p %m\n", thread->kthread, method->exec.function.void_fun, method);
    }*/
    thread->numframes--;
#ifdef JAVA_PROFILE
    if(method->exec.dispatcher) {
      
      // profileMethodCallEnter(method);

      updateProfileCalls(caller->method, method);
      time_start = x_systime_get();
      time_delta = caller->thread->kthread->time_delta;
      // w_dump(" --> %M\n", method);
    
      dispatchers[method->exec.dispatcher](caller, method);
      
      // w_dump(" <-- %8lld %M\n", x_systime_get() - time_start - (caller->thread->kthread->time_delta - time_delta), method);
      method->exec.runtime += x_systime_get() - time_start - (caller->thread->kthread->time_delta - time_delta);
      method->exec.totaltime += x_systime_get() - time_start;
      
      // profileMethodCallExit(method);
    }
    else {
      dispatchers[method->exec.dispatcher](caller, method);
    }
#else
    dispatchers[method->exec.dispatcher](caller, method);
#endif

    thread->numframes++;
#ifdef GC_SAFE_POINTS
    if (unsafe_to_safe) {
      enterUnsafeRegion(thread);
    }
    else if (safe_to_unsafe) {
      enterSafeRegion(thread);
    }
#endif
  }
  else {
    throwException(thread, clazzStackOverflowError, "%M: %d frames stacked.", method, thread->maxframes);
  }
  
}

/*
** Non inlined version for functions outside this file...
*/

void callMethod(x_frame caller, w_method method) {
  i_callMethod(caller, method);
}

/*
** Helper structure for match offset pair lookup in lookupswitch; 
** this structure is also used in the method code rewritting routine.
*/

typedef struct w_Mopair {
  w_int m;
  w_int o;
} w_Mopair;

static void ** labels = NULL;

void interprete(x_frame caller, w_method method) {

  static void * codeJumpTable[] = {
    && c_nop, && c_aconst_null, && c_iconst_m1, && c_iconst_0, && c_iconst_1, && c_iconst_2, && c_iconst_3, 
    && c_iconst_4, && c_iconst_5, && c_lconst_0, && c_lconst_1, && c_fconst_0, && c_fconst_1, && c_fconst_2, 
    && c_dconst_0, && c_dconst_1, && c_bipush, && c_sipush, && c_ldc, && c_ldc_w, && c_ldc2_w, && c_iload, 
    && c_lload, && c_fload, && c_dload, && c_aload, && c_iload_0, && c_iload_1, && c_iload_2, && c_iload_3, 
    && c_lload_0, && c_lload_1, && c_lload_2, && c_lload_3, && c_fload_0, && c_fload_1, && c_fload_2, 
    && c_fload_3, && c_dload_0, && c_dload_1, && c_dload_2, && c_dload_3, && c_aload_0, && c_aload_1, 
    && c_aload_2, && c_aload_3, && c_iaload, && c_laload, && c_faload, && c_daload, && c_aaload, && c_baload, 
    && c_caload, && c_saload, && c_istore, && c_lstore, && c_fstore, && c_dstore, && c_astore, && c_istore_0, 
    && c_istore_1, && c_istore_2, && c_istore_3, && c_lstore_0, && c_lstore_1, && c_lstore_2, && c_lstore_3, 
    && c_fstore_0, && c_fstore_1, && c_fstore_2, && c_fstore_3, && c_dstore_0, && c_dstore_1, && c_dstore_2, 
    && c_dstore_3, && c_astore_0, && c_astore_1, && c_astore_2, && c_astore_3, && c_iastore, && c_lastore, 
    && c_fastore, && c_dastore, && c_aastore, && c_bastore, && c_castore, && c_sastore, && c_pop, && c_pop2, 
    && c_dup, && c_dup_x1, && c_dup_x2, && c_dup2, && c_dup2_x1, && c_dup2_x2, && c_swap, && c_iadd, && c_ladd, 
    && c_fadd, && c_dadd, && c_isub, && c_lsub, && c_fsub, && c_dsub, && c_imul, && c_lmul, && c_fmul, && c_dmul, 
    && c_idiv, && c_ldiv, && c_fdiv, && c_ddiv, && c_irem, && c_lrem, && c_frem, && c_drem, && c_ineg, && c_lneg, 
    && c_fneg, && c_dneg, && c_ishl, && c_lshl, && c_ishr, && c_lshr, && c_iushr, && c_lushr, && c_iand, 
    && c_land, && c_ior, && c_lor, && c_ixor, && c_lxor, && c_iinc, && c_i2l, && c_i2f, && c_i2d, && c_l2i, 
    && c_l2f, && c_l2d, && c_f2i, && c_f2l, && c_f2d, && c_d2i, && c_d2l, && c_d2f, && c_i2b, && c_i2c, 
    && c_i2s, && c_lcmp, && c_fcmpl, && c_fcmpg, && c_dcmpl, && c_dcmpg, && c_ifeq, && c_ifne, && c_iflt, 
    && c_ifge, && c_ifgt, && c_ifle, && c_if_icmpeq, && c_if_icmpne, && c_if_icmplt, && c_if_icmpge, 
    && c_if_icmpgt, && c_if_icmple, && c_if_acmpeq, && c_if_acmpne, && c_goto, && c_jsr, && c_ret, 
    && c_tableswitch, && c_lookupswitch, && c_ireturn, && c_lreturn, && c_freturn, && c_dreturn, && c_areturn, 
    && c_vreturn, && c_getstatic, && c_putstatic, && c_getfield, && c_putfield, && c_invokevirtual, 
    && c_invokespecial, && c_invokestatic, && c_invokeinterface, && i_new, && c_new, && c_newarray, 
    && c_anewarray, && c_arraylength, && c_athrow, && c_checkcast, && c_instanceof, && c_monitorenter, 
    && c_monitorexit, && c_wide, && c_multianewarray, && c_ifnull, && c_ifnonnull, && c_goto_w, && c_jsr_w, 
    && c_breakpoint, && i_getstatic, && i_putstatic, && i_getfield, && i_putfield, && i_invokestatic, && i_invokenonvirtual, 
    && i_invokesuper, && i_invokevirtual, && c_nocode_211, && c_nocode_212, && c_nocode_213, && c_nocode_214, && c_nocode_215, 
    && c_nocode_216, && c_nocode_217, && c_nocode_218, && c_nocode_219, && c_nocode_220, && c_nocode_221, 
    && c_nocode_222, && c_nocode_223, && c_nocode_224, && c_nocode_225, && c_nocode_226, && c_nocode_227, 
    && c_nocode_228, && c_nocode_229, && c_nocode_230, && c_nocode_231, && c_nocode_232, && c_nocode_233, 
    && c_nocode_234, && c_nocode_235, && c_nocode_236, && c_nocode_237, && c_nocode_238, && c_nocode_239, 
    && c_nocode_240, && c_nocode_241, && c_nocode_242, && c_nocode_243, && c_nocode_244, && c_nocode_245, 
    && c_nocode_246, && c_nocode_247, && c_nocode_248, && c_nocode_249, && c_nocode_250, && c_nocode_251, 
    && c_nocode_252, && c_nocode_253, && c_find_handler, && c_no_handler,
  };

  x_frame frame;
  w_thread thread;
#if defined(X86)
  register w_code current asm ("%ebx") = method->exec.code;
  register void ** jumps = codeJumpTable;
#elif defined(ARM)
  register w_code current asm ("r8") = method->exec.code;
  register void ** jumps asm ("r9") = codeJumpTable;
#else
  register w_code current = method->exec.code;
  register void ** jumps = codeJumpTable;
#endif
  w_int i;
  w_instance o;
  w_instance a = NULL;
  w_method x;
  w_clazz c;
  w_clazz cclazz;
  x_monitor m;
  w_field field;
  w_cnt constant;
  w_int s;
  w_int * table;
  w_Mopair * mopair;
  w_u64 z1;
  w_u64 z2;
  w_u64 z3;

  labels = codeJumpTable;
  frame = alloca(sizeof(x_Frame) + (sizeof(x_Slot) * (method->exec.local_i + method->exec.stack_i)));
  frame->top = frame->Slots + method->exec.local_i;
  frame->flags = 0;
  frame->previous = caller;
  frame->auxs = caller->auxs;
  thread = caller->thread;
  frame->thread = thread;
  frame->method = method;
  cclazz = method->spec.declaring_clazz;

#ifdef bar
  {
    static int calls = 0;
    static long long unsigned int total = 0;
    int use = (unsigned char *)caller - (unsigned char *)frame;

    if (use > 0 && use < 2048) {
      calls += 1;
      total += use;
    }

    if (calls % 1000 == 0 && calls) {
      _woempa("",__FUNCTION__,0,9, "AVERAGE STACK use = %d bytes.\n", total / calls);
    }
  }
#endif

  stack2locals(frame->Slots, caller->top, method->exec.arg_i);
  for (i = method->exec.arg_i; i < method->exec.local_i; i++) {
    frame->Slots[i].s = stack_notrace;
  }

  /*
  ** It is crucial for GC that the following statement only comes after the frame has been
  ** setup correctly.
  */
  
  thread->top = frame;

  /*
  ** Note that pc is a signed integer and that values -1 and -2, currently, are used to store Wonka specific
  ** opcodes for exception handling. The program counter 'pc' is incremented before we use it, so that
  ** we don't have to increment it after each opcode. If we would increment it after each opcode, GCC
  ** would produce a jump for each opcode, to a code section that increments pc and jumps to the start
  ** of the loop again; doing it this way removes a single jump. The order in which the opcodes appear
  ** is based on histogram information; we want to keep the cache as warm as possible...
  */

  goto * jumps[*current];

  c_aload: {
    do_aload(frame, byte_operand);
    do_next_opcode;
  }

  c_aload_0: {
    do_aload(frame, 0); 
    do_next_opcode;
  }
  
  c_aload_1: {
    do_aload(frame, 1);
    do_next_opcode;
  }
  
  c_aload_2: {
    do_aload(frame, 2);
    do_next_opcode;
  }
  
  c_aload_3: {
    do_aload(frame, 3);
    do_next_opcode;
  }

  i_getfield: {
    field = (w_field) cclazz->values[(unsigned short) short_operand];
    o = (w_instance) frame->top[-1].c.w;

    if (o == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    frame->top[-1].s = stack_notrace;

    if (isSet(field->flags, FIELD_IS_REFERENCE)) {
      frame->top[-1].c.w = o[instance2clazz(o)->instanceSize + field->size_and_slot];
      frame->top[-1].s = stack_trace;
    }
#ifdef PACK_BYTE_FIELDS
    else if ((field->size_and_slot & FIELD_SIZE_MASK) <= FIELD_SIZE_8_BITS) {
      frame->top[-1].c.w = *byteFieldPointer(o, FIELD_OFFSET(field->size_and_slot));
    }
#endif
    else {
      frame->top[-1].c.w = *wordFieldPointer(o, FIELD_OFFSET(field->size_and_slot));
      if (isSet(field->flags, FIELD_IS_LONG)) {
        frame->top[ 0].c.w = o[FIELD_OFFSET(field->size_and_slot) + 1];
        frame->top[ 0].s = stack_notrace;
        frame->top += 1;
      }
    }
    add_to_opcode(3);
  }

  c_ireturn: c_freturn: {
    caller->top -= method->exec.arg_i;
    caller->top[0].s = stack_notrace;
    caller->top[0].c.w = frame->top[-1].c.w;
    caller->top += 1;
    thread->top = caller;
    return;
  }

  c_iload_0: c_fload_0: {
    do_xload(frame, 0);
    do_next_opcode;
  }
  
  c_goto: {
#ifdef GC_SAFE_POINTS
    if (pendingGCRequest(thread)) {
      gcSafePoint(thread);
    }
#endif
    add_to_opcode(short_operand);
  }

  c_ifeq: c_ifnull: {
    frame->top -= 1; 
    do_conditional(frame->top[0].c.w == 0);
  }
  
  c_iload_1: c_fload_1: {
    do_xload(frame, 1);
    do_next_opcode;
  }
  
  c_iload_2: c_fload_2: {
    do_xload(frame, 2);
    do_next_opcode;
  }
  
  c_iload_3: c_fload_3: {
    do_xload(frame, 3);
    do_next_opcode;
  }

  c_iload: c_fload: {
    do_xload(frame, byte_operand);
    do_next_opcode;
  }

  i_getstatic: {
    field  = (w_field) cclazz->values[(unsigned short) short_operand];

    frame->top[0].s = stack_notrace;

    if (isSet(field->flags, FIELD_IS_LONG)) {
      frame->top[0].c.w = field->declaring_clazz->staticFields[field->size_and_slot + 0];
      frame->top[1].c.w = field->declaring_clazz->staticFields[field->size_and_slot + 1];
      frame->top[1].s = stack_notrace;
      frame->top += 2;
    }
    else {
      frame->top[0].c.w = field->declaring_clazz->staticFields[field->size_and_slot];
      if (isSet(field->flags, FIELD_IS_REFERENCE)) {
        frame->top[0].s = stack_trace;
      }
      frame->top += 1;
    }

    add_to_opcode(3);
  }

  i_putstatic: {
    field  = (w_field) cclazz->values[(unsigned short) short_operand];

    if (isSet(field->flags, FIELD_IS_LONG)) {
      field->declaring_clazz->staticFields[field->size_and_slot + 0] = frame->top[-2].c.w;
      field->declaring_clazz->staticFields[field->size_and_slot + 1] = frame->top[-1].c.w;
      frame->top -= 2;
    }
    else {
      if (isNotSet(field->value_clazz->flags, CLAZZ_IS_PRIMITIVE)) {
        setStaticReferenceField(field->declaring_clazz, field->size_and_slot, (w_instance) frame->top[-1].c.w);
      }
      else {
        field->declaring_clazz->staticFields[field->size_and_slot] = frame->top[-1].c.w;
      }
      frame->top -= 1;
    }
    add_to_opcode(3);
  }

  i_invokestatic: {
    frame->current = current;
    x = getMethodConstant(cclazz, (unsigned short) short_operand);
    i_callMethod(frame, x);
    current += 2;
    goto check_async_exception;
  }

  c_ifne: c_ifnonnull: {
    frame->top -= 1; 
    do_conditional(frame->top[0].c.w != 0);
  }

  c_aaload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    frame->top -= 1;
    frame->top[-1].c.w = (w_word) instance2Array_instance(a)[i];

    do_next_opcode;
  }

  c_istore_3: c_fstore_3: {
    do_xstore(frame, 3);
    do_next_opcode;
  }

  c_ifge: {
    frame->top -= 1; 
    do_conditional((w_int) frame->top[0].c.w >= 0);
  }

  c_iconst_0: {
    do_xpush(frame, 0);
    do_next_opcode;
  }

  c_fcmpl: i = -1; goto do_fcmp;
  c_fcmpg: i = +1;

  do_fcmp: {
    frame->top -= 1;
    if (float32_is_NaN(frame->top[-1].c.f) || float32_is_NaN(frame->top[0].c.f) ) {
      frame->top[-1].c.w = i;
    }
    else if (float32_eq(frame->top[-1].c.f, frame->top[0].c.f)) {
      frame->top[-1].c.w = 0;
    }
    else if (float32_lt(frame->top[-1].c.f, frame->top[0].c.f)) {
      frame->top[-1].c.w = -1;
    }
    else {
      frame->top[-1].c.w = 1;
    }
    do_next_opcode;
  }

  c_iinc: {
    i = byte_operand;
    s = (w_int) (signed char) byte_operand;
    frame->Slots[i].c.w += s;
    do_next_opcode;
  }

  c_istore: c_fstore: {
    do_xstore(frame, byte_operand);
    do_next_opcode;
  }

  c_isub: {
    frame->top -= 1;
    frame->top[-1].c.w = (w_int) frame->top[-1].c.w - (w_int) frame->top[0].c.w;
    do_next_opcode;
  }

  c_irem: {
    i = (w_int) frame->top[-1].c.w;

    if (i == 0) {
      do_throw_clazz(clazzArithmeticException);
    }

    s = (w_int) frame->top[-2].c.w;
    frame->top[-2].c.w = 0;

    if (s != (w_int) 0x80000000 || i != -1) {
      frame->top[-2].c.w = s - (w_int) (s / i) * i;
    }
  
    frame->top -= 1;
    do_next_opcode;
  }

  c_lload_0: c_dload_0: {
    do_zload(frame, 0);
    do_next_opcode;
  }

  c_checkcast: {
    c = getClassConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }
    
    o = (w_instance) frame->top[-1].c.w;
    if (o && ! isAssignmentCompatible(instance2object(o)->clazz, c)) {
      woempa(1, "Cannot cast %j to %k\n", o, c);
      //do_throw_clazz(clazzClassCastException);
      throwException(thread, clazzClassCastException, "Cannot cast %j to %k", o, c);
      do_the_exception;
    }
    add_to_opcode(3);
  }

  c_bipush: {
    do_xpush(frame, (signed int) (signed char) byte_operand);
    do_next_opcode;
  }

  c_iconst_1: {
    do_xpush(frame, 1); 
    do_next_opcode;
  }

  c_areturn: {
    caller->top -= method->exec.arg_i;
    caller->top[0].c.w = frame->top[-1].c.w;
    caller->top[0].s = stack_trace;
//    caller->top[0].s = frame->top[-1].s; // Would  be safer...
    caller->top += 1;
    thread->top = caller;
    return;
  }

  c_ldc2_w: {
    w_cnt c = &cclazz->constants[short_operand];
 /*   if(c->cnt_tag == CONSTANT_DOUBLE) {
      frame->top[0].c.w = cclazz->values[(unsigned short) short_operand + 0]; // FIX THIS, LONG CLASS CONSTANTS ARE IN THE BAD ORDER !!!
      frame->top[0].s = stack_notrace;
      frame->top[1].c.w = cclazz->values[(unsigned short) short_operand + 1];  // FIX THIS, LONG CLASS CONSTANTS ARE IN THE BAD ORDER !!!
      frame->top[1].s = stack_notrace;
    }
    else {*/
      frame->top[1].c.w = cclazz->values[(unsigned short) short_operand + 0]; // FIX THIS, LONG CLASS CONSTANTS ARE IN THE BAD ORDER !!!
      frame->top[1].s = stack_notrace;
      frame->top[0].c.w = cclazz->values[(unsigned short) short_operand + 1];  // FIX THIS, LONG CLASS CONSTANTS ARE IN THE BAD ORDER !!!
      frame->top[0].s = stack_notrace;
/*    }*/
    frame->top += 2;
    add_to_opcode(3);
  }

  c_if_icmplt: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w  < (w_int) frame->top[1].c.w);
  }
  
  i_putfield: {
    field = (w_field) cclazz->values[(unsigned short) short_operand];
    o = (w_instance) frame->top[ - fieldSize(field) - 1].c.w;
    if (o == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (isSet(field->flags, FIELD_IS_REFERENCE)) {
      setReferenceFieldUnsafe(o, (w_instance) frame->top[-1].c.w, field->size_and_slot);
      frame->top -= 2;
    }
#ifdef PACK_BYTE_FIELDS
    else if ((field->size_and_slot & FIELD_SIZE_MASK) <= FIELD_SIZE_8_BITS) {
      *byteFieldPointer(o, FIELD_OFFSET(field->size_and_slot)) = (w_sbyte)frame->top[-1].c.w;
      frame->top -= 2;
    }
#endif
    else if (isNotSet(field->flags, FIELD_IS_LONG)) {
      *wordFieldPointer(o, FIELD_OFFSET(field->size_and_slot)) = frame->top[-1].c.w;
      frame->top -= 2;
    }
    else {
      o[FIELD_OFFSET(field->size_and_slot)] = frame->top[-2].c.w;
      o[FIELD_OFFSET(field->size_and_slot) + 1] = frame->top[-1].c.w;
      frame->top -= 3;
    }
    add_to_opcode(3);
  }

  c_if_icmpeq: c_if_acmpeq: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w == (w_int) frame->top[1].c.w);
  }
  
  c_iflt: {
    frame->top -= 1; 
    do_conditional((w_int) frame->top[0].c.w  < 0);
  }

  c_iadd: {
    frame->top -= 1;
    frame->top[-1].c.w = (w_int) frame->top[-1].c.w + (w_int) frame->top[0].c.w;
    do_next_opcode;
  }

  c_dcmpl: i = -1; goto do_dcmp;
  c_dcmpg: i = +1;

  do_dcmp: {
    z2.words[WORD_MSW] = frame->top[-1].c.w;
    z2.words[WORD_LSW] = frame->top[-2].c.w;
    z1.words[WORD_MSW] = frame->top[-3].c.w;
    z1.words[WORD_LSW] = frame->top[-4].c.w;
    frame->top -= 3;
    if (float64_is_NaN(z1.d64) || float64_is_NaN(z2.d64) ) {
      frame->top[-1].c.w = i;
    }
    else if (float64_lt(z1.d64, z2.d64)) {
      frame->top[-1].c.w = -1;
    }
    else if (float64_eq(z1.d64, z2.d64)) {
      frame->top[-1].c.w = 0;
    }
    else {
      frame->top[-1].c.w = 1;
    }
    do_next_opcode;
  }

  c_dup: {
    frame->top[0].c.w = frame->top[-1].c.w;
    frame->top[0].s = frame->top[-1].s;
    frame->top += 1;
    do_next_opcode;
  }

  c_astore: {
    do_astore(frame, byte_operand);
    do_next_opcode;
  }

  c_iand: {
    frame->top -= 1;
    frame->top[-1].c.w &= frame->top[0].c.w;
    do_next_opcode;
  }

  c_baload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    frame->top -= 1;
    frame->top[-1].s = stack_notrace;
    if (instance2object(a)->clazz->previousDimension == clazz_boolean) {
      frame->top[-1].c.w = (instance2Array_byte(a)[i / 8] >> (i % 8)) & 1;
    }
    else {
      frame->top[-1].c.w = instance2Array_byte(a)[i];
    }

    do_next_opcode;
  }

  c_i2f: {
    frame->top[-1].c.f = int32_to_float32(frame->top[-1].c.w);
    do_next_opcode;
  }

  i_new: {
    c = (w_clazz) cclazz->values[(unsigned short) short_operand];

    frame->top[0].s = stack_trace;
    frame->top[0].c.w = (w_word) allocInstance(thread, c);
    if (! frame->top[0].c.w) {
      do_throw_clazz(clazzOutOfMemoryError);
    }
    frame->top += 1;
#ifndef GC_SAFE_POINTS
    popLocalReference(frame);
#endif
    current += 2;
  }

  /*
  ** Check for a pending asynchronous exception. This label is jumped too from several
  ** opcodes and follows the 'new' opcode, since this is the first opcode in the histogrammed
  ** list that needs such a check... !! When you move this label, you need to move the 'i_new'
  ** code as well !!
  */

  check_async_exception: {
    if (thread->exception) {
      current = searchHandler(frame);
    }
    do_next_opcode;
  }

  c_fmul: {
    frame->top -= 1;
    frame->top[-1].c.f = float32_mul(frame->top[-1].c.f, frame->top[0].c.f);
    do_next_opcode;
  }

  c_sipush: {
    do_xpush(frame, (signed int) short_operand);
    add_to_opcode(3);
  }

  c_aastore: {
    a = (w_instance) frame->top[-3].c.w;
    i = (w_int) frame->top[-2].c.w;

    if (! a) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    o = (w_instance) frame->top[-1].c.w;
    if (o && ! isAssignmentCompatible(instance2object(o)->clazz, instance2object(a)->clazz->previousDimension)) {
      do_throw_clazz(clazzArrayStoreException);
    }

    setArrayReferenceField(a, o, i);
    frame->top -= 3;
    do_next_opcode;
  }

  c_astore_2: {
    do_astore(frame, 2);
    do_next_opcode;
  }

  /*
  ** For the l2i opcode, we truncate the long value by only keeping the lowest
  ** 32 bits. Since Wonka works Little Endian on it's stack, we just pop the stack
  ** with 1; which makes the l2i opcode become a simple 'pop'.
  */

  c_pop: c_l2i: {
    frame->top -= 1;
    do_next_opcode;
  }

  c_istore_2: c_fstore_2: {
    do_xstore(frame, 2);
    do_next_opcode;
  }

  c_astore_3: {
    do_astore(frame, 3); 
    do_next_opcode;
  }

  c_arraylength: {
    a = (w_instance) frame->top[-1].c.w;
    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }
    frame->top[-1].s = stack_notrace;
    frame->top[-1].c.w = instance2Array_length(a);
    do_next_opcode;
  }

  c_i2c: {
    frame->top[-1].c.w &= 0x0000ffff;
    do_next_opcode;
  }

  c_monitorenter: {
#ifdef GC_SAFE_POINTS
    w_boolean unsafe;
#endif

    o = (w_instance) frame->top[-1].c.w;
    if (o == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

#ifdef GC_SAFE_POINTS
    unsafe = enterSafeRegion(thread);
#endif
    if (isSet(instance2flags(o), O_HAS_LOCK)) {
      m = (x_monitor) ht_read(lock_hashtable, (w_word)o);
    }
    else {
      m = allocMonitor(o);
    }
    x_monitor_eternal(m);
#ifdef MONITOR_INFO
    m->xref = frame;
#endif
    pushMonitoredReference(frame, o, m);
    frame->top -= 1;
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif
    goto check_async_exception;
  }

  c_monitorexit: {
    x_slot slot;
    x_slot base = caller->auxs;

    o = (w_instance) frame->top[-1].c.w;
    if (o == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

/* [CG 20040504] Bogus : see new code below
    /.
    .. Skip any pushed references that needed protection, from deactivateFrame.
    ./

    do {
      frame->auxs -= 1;
    } while (! isMonitoredSlot(frame->auxs) && frame->auxs >= frame->previous->auxs);

    if (isMonitoredSlot(frame->auxs) && frame->auxs->c.w == (w_word) o) {
      m = (x_monitor) frame->auxs->s;
      frame->auxs->s = stack_notrace;
      if (m && x_monitor_exit(m) == xs_not_owner) {
        do_throw_clazz(clazzIllegalMonitorStateException);
      }
#ifdef MONITOR_INFO
      else if(m) m->xref = NULL;
#endif
    }
    else {
      throwException(thread, clazzInternalError, "Auxillary stack skew with monitorexit.");
      do_the_exception;
    }
*/

    woempa(1, "Have %d auxs, of which %d belong to caller\n", frame->auxs - frame->thread->auxs, base - frame->thread->auxs);
    for (slot = frame->auxs - 1; slot - base >= 0; slot -= 1) {
      if (isMonitoredSlot(slot)) {
        woempa(1, "  aux[%d] is a monitored slot : o = %j\n", slot - frame->thread->auxs, slot->c);
        if (slot->c.w == (w_word) o) {
          m = (x_monitor) slot->s;
          if (!m) { // Probably impossible
            woempa(9, "Monitored slot is null!.\n");
            do_throw_clazz(clazzInternalError);
          }
          if (x_monitor_exit(m) == xs_not_owner) {
            do_throw_clazz(clazzIllegalMonitorStateException);
          }
          slot->s = stack_notrace;
#ifdef MONITOR_INFO
          m->xref = NULL;
#endif
          woempa(1, "Removed monitored object %j from aux[%d] of %t\n", o, slot - frame->thread->auxs, frame->thread);
          break;
        }
        else {
          woempa(9, "Monitored object mismatch - expected %j, found %j\n", o, slot->c);
          do_throw_clazz(clazzInternalError);
        }
      }
      else {
        woempa(1, "  aux[%d] is not a monitored slot\n", slot - frame->thread->auxs);
      }
    }
    while (frame->auxs > base && frame->auxs[-1].s == stack_notrace) {
      // [JD] frame->auxs -= 1;
      popLocalReference(frame);
      woempa(1, "Removed zombie from thread %t, now have %d auxs\n", frame->thread, frame->auxs - frame->thread->auxs);
    }
    if (slot < base) { // Probably impossible
      woempa(9, "Auxiliary stack skew with monitorexit.\n");
      do_throw_clazz(clazzInternalError);
    }

    frame->top -= 1;
    do_next_opcode;
  }

  c_bastore: {
    a = (w_instance) frame->top[-3].c.w;
    i = (w_int) frame->top[-2].c.w;

    if (! a) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    if (instance2object(a)->clazz->previousDimension == clazz_boolean) {
      s = instance2Array_byte(a)[i / 8];
      s &= 0xff ^ (1 << i % 8);
      instance2Array_byte(a)[i / 8] = s | ((frame->top[-1].c.w & 1) << (i % 8));
    }
    else {
      instance2Array_byte(a)[i] = truncate_i2b(frame->top[-1].c.w);
    }
    frame->top -= 3;
    do_next_opcode;
  }

  c_lcmp: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    frame->top -= 3;
    if (z2.s64 > z1.s64) {
      frame->top[-1].c.w = 1;
    }
    else if (z2.s64 == z1.s64) {
      frame->top[-1].c.w = 0;
    }
    else {
      frame->top[-1].c.w = -1;
    }
    do_next_opcode;
  }

  c_nop: {
    do_next_opcode;
  }

  c_aconst_null: {
    frame->top[0].s = stack_trace;
    frame->top[0].c.w = 0;
    frame->top += 1;
    do_next_opcode;
  }

  c_iconst_m1: {
    do_xpush(frame, -1); 
    do_next_opcode;
  }

  c_iconst_2: {
    do_xpush(frame, 2); 
    do_next_opcode;
  }

  c_iconst_3: {
    do_xpush(frame, 3);
    do_next_opcode;
  }

  c_iconst_4: {
    do_xpush(frame, 4);
    do_next_opcode;
  }

  c_iconst_5: {
    do_xpush(frame, 5);
    do_next_opcode;
  }

  c_fconst_0: {
    do_xpush(frame, F_ZERO);
    do_next_opcode;
  }

  c_fconst_1: {
    do_xpush(frame, F_ONE);
    do_next_opcode;
  }

  c_fconst_2: {
    do_xpush(frame, F_TWO);
    do_next_opcode;
  }

  c_lconst_0: c_dconst_0: {
    do_zpush(frame, 0, 0);
    do_next_opcode;
  }

  c_lconst_1: {
    do_zpush(frame, 0, 1);
    do_next_opcode;
  }

  c_dconst_1: {
#if defined(ARM) && defined(HW_DOUBLES)
    do_zpush(frame, D_ONE_LSW, D_ONE_MSW);
#else 
    do_zpush(frame, D_ONE_MSW, D_ONE_LSW);
#endif
    do_next_opcode;
  }

  c_ldc_w: {
    i = (unsigned short) short_operand; 
    current += 2;
    goto load_constant;
  }
  
  c_ldc: {
    i = (unsigned short) byte_operand;
  }

  load_constant: {
    constant = &cclazz->constants[i];
    if (constant->cnt_tag == CONSTANT_INTEGER || constant->cnt_tag == CONSTANT_FLOAT) {
      frame->top[0].s = stack_notrace;
      frame->top[0].c.w = cclazz->values[i];
      frame->top += 1;
    }
    else if (constant->cnt_tag == CONSTANT_STRING || constant->cnt_tag == RESOLVED_STRING) {
      frame->top[0].c.w = (w_word) getStringConstant(cclazz, i);
      frame->top[0].s = stack_trace;
      frame->top += 1;
#ifndef GC_SAFE_POINTS
      removeLocalReference(thread, (w_instance) frame->top[-1].c.w);
#endif
    }
    goto check_async_exception;
  }

  c_lload: c_dload: {
    do_zload(frame, byte_operand);
    do_next_opcode;
  }

  c_lload_1: c_dload_1: {
    do_zload(frame, 1);
    do_next_opcode;
  }
  
  c_lload_2: c_dload_2: {
    do_zload(frame, 2);
    do_next_opcode;
  }
  
  c_lload_3: c_dload_3: {
    do_zload(frame, 3);
    do_next_opcode;
  }

  c_iaload: c_faload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    frame->top -= 1;
    frame->top[-1].s = stack_notrace;
    frame->top[-1].c.w = instance2Array_int(a)[i];

    do_next_opcode;
  }

  c_daload: c_laload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    z1.s64 = instance2Array_long(a)[i];
    frame->top[-2].s = stack_notrace;
    frame->top[-2].c.w = z1.words[WORD_LSW];
    frame->top[-1].c.w = z1.words[WORD_MSW];

    do_next_opcode;
  }

  c_caload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    frame->top -= 1;
    frame->top[-1].s = stack_notrace;
    frame->top[-1].c.w = instance2Array_char(a)[i];
    do_next_opcode;
  }

  c_saload: {
    a = (w_instance) frame->top[-2].c.w;
    i = (w_int) frame->top[-1].c.w;

    if (a == NULL) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    frame->top -= 1;
    frame->top[-1].s = stack_notrace;
    frame->top[-1].c.w = instance2Array_short(a)[i];
    do_next_opcode;
  }

  c_lstore: c_dstore: {
    do_zstore(frame, byte_operand);
    do_next_opcode;
  }

  c_istore_0: c_fstore_0: {
    do_xstore(frame, 0);
    do_next_opcode;
  }
  
  c_istore_1: c_fstore_1: {
    do_xstore(frame, 1);
    do_next_opcode;
  }
  
  c_lstore_0: c_dstore_0: {
    do_zstore(frame, 0);
    do_next_opcode;
  }
  
  c_lstore_1: c_dstore_1: {
    do_zstore(frame, 1);
    do_next_opcode;
  }
  
  c_lstore_2: c_dstore_2: {
    do_zstore(frame, 2);
    do_next_opcode;
  }
  
  c_lstore_3: c_dstore_3: {
    do_zstore(frame, 3);
    do_next_opcode;
  }

  c_astore_0: {
    do_astore(frame, 0); 
    do_next_opcode;
  }

  c_astore_1: {
    do_astore(frame, 1); 
    do_next_opcode;
  }
  
  c_iastore: c_fastore: {
    a = (w_instance) frame->top[-3].c.w;
    i = (w_int) frame->top[-2].c.w;

    if (! a) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    instance2Array_int(a)[i] = frame->top[-1].c.w;
    frame->top -= 3;
    do_next_opcode;
  }

  c_lastore: c_dastore: {
    a = (w_instance) frame->top[-4].c.w;
    i = (w_int) frame->top[-3].c.w;

    if (! a) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    instance2Array_long(a)[i] = z1.u64;
    frame->top -= 4;
    do_next_opcode;
  }


  c_castore: c_sastore: {
    a = (w_instance) frame->top[-3].c.w;
    i = (w_int) frame->top[-2].c.w;

    if (! a) {
      do_throw_clazz(clazzNullPointerException);
    }

    if (i >= instance2Array_length(a) || i < 0) {
      do_throw_clazz_msg(clazzArrayIndexOutOfBoundsException, "index %d, length %d", i, instance2Array_length(a));
    }

    instance2Array_char(a)[i] = (frame->top[-1].c.w & 0x0000ffff);
    frame->top -= 3;
    do_next_opcode;
  }
  

  c_pop2: {
    frame->top -= 2;
    do_next_opcode;
  }

  c_dup_x1: {
#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif
    frame->top[ 0].c.w = frame->top[-1].c.w;
    frame->top[ 0].s = frame->top[-1].s;
    frame->top[-1].c.w = frame->top[-2].c.w;
    frame->top[-1].s = frame->top[-2].s;
    frame->top[-2].c.w = frame->top[ 0].c.w;
    frame->top[-2].s = frame->top[ 0].s;
    frame->top += 1;
#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif
    do_next_opcode;
  }

  c_dup_x2: {
#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif
    frame->top[ 0].c.w = frame->top[-1].c.w;
    frame->top[ 0].s = frame->top[-1].s;
    frame->top[-1].c.w = frame->top[-2].c.w;
    frame->top[-1].s = frame->top[-2].s;
    frame->top[-2].c.w = frame->top[-3].c.w;
    frame->top[-2].s = frame->top[-3].s;
    frame->top[-3].c.w = frame->top[ 0].c.w;
    frame->top[-3].s = frame->top[ 0].s;
    frame->top += 1;
#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif
    do_next_opcode;
  }

  c_dup2: {
    frame->top[1].c.w = frame->top[-1].c.w;
    frame->top[1].s = frame->top[-1].s;
    frame->top[0].c.w = frame->top[-2].c.w;
    frame->top[0].s = frame->top[-2].s;
    frame->top += 2;
    do_next_opcode;
  }

  c_dup2_x1: {
#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif
    frame->top[ 1].c.w = frame->top[-1].c.w;
    frame->top[ 1].s = frame->top[-1].s;
    frame->top[ 0].c.w = frame->top[-2].c.w;
    frame->top[ 0].s = frame->top[-2].s;
    frame->top[-1].c.w = frame->top[-3].c.w;
    frame->top[-1].s = frame->top[-3].s;
    frame->top[-2].c.w = frame->top[ 1].c.w;
    frame->top[-2].s = frame->top[ 1].s;
    frame->top[-3].c.w = frame->top[ 0].c.w;
    frame->top[-3].s = frame->top[ 0].s;
    frame->top += 2;
#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif
    do_next_opcode;
  }

  c_dup2_x2: {
#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif
    frame->top += 2;
    frame->top[-1].c.w = frame->top[-3].c.w;
    frame->top[-1].s = frame->top[-3].s;
    frame->top[-2].c.w = frame->top[-4].c.w;
    frame->top[-2].s = frame->top[-4].s;
    frame->top[-3].c.w = frame->top[-5].c.w;
    frame->top[-3].s = frame->top[-5].s;
    frame->top[-4].c.w = frame->top[-6].c.w;
    frame->top[-4].s = frame->top[-6].s;
    frame->top[-5].c.w = frame->top[-1].c.w;
    frame->top[-5].s = frame->top[-1].s;
    frame->top[-6].c.w = frame->top[-2].c.w;
    frame->top[-6].s = frame->top[-2].s;
#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif
    do_next_opcode;
  }

  c_swap: {
#ifndef GC_SAFE_POINTS
    x_preemption_disable;
#endif
    i = frame->top[-1].c.w;
    s = frame->top[-1].s;
    frame->top[-1].c.w = frame->top[-2].c.w;
    frame->top[-1].s = frame->top[-2].s;
    frame->top[-2].c.w = i;
    frame->top[-2].s = s;
#ifndef GC_SAFE_POINTS
    x_preemption_enable;
#endif
    do_next_opcode;
  }

  c_ladd: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.s64 += z1.s64;  
    frame->top[-4].c.w = z2.words[WORD_LSW];
    frame->top[-3].c.w = z2.words[WORD_MSW];
    frame->top -= 2;
    do_next_opcode;
  }

  c_fadd: {
    frame->top -= 1;
    frame->top[-1].c.f = float32_add(frame->top[-1].c.f, frame->top[0].c.f);
    do_next_opcode;
  }

  c_dadd: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.d64 = float64_add(z2.d64, z1.d64);
    frame->top[-3].c.w = z2.words[WORD_MSW];
    frame->top[-4].c.w = z2.words[WORD_LSW];
    frame->top -= 2;
    do_next_opcode;
  }

  c_lsub: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.s64 -= z1.s64;  
    frame->top -= 2;
    frame->top[-1].c.w = z2.words[WORD_MSW];
    frame->top[-2].c.w = z2.words[WORD_LSW];
    do_next_opcode;
  }

  c_fsub: {
    frame->top -= 1;
    frame->top[-1].c.f = float32_sub(frame->top[-1].c.f, frame->top[0].c.f);
    do_next_opcode;
  }

  c_dsub: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.d64 = float64_sub(z2.d64, z1.d64);
    frame->top[-3].c.w = z2.words[WORD_MSW];
    frame->top[-4].c.w = z2.words[WORD_LSW];
    frame->top -= 2;
    do_next_opcode;
  }

  c_imul: {
    frame->top -= 1;
    frame->top[-1].c.w = (w_int) frame->top[-1].c.w * (w_int) frame->top[0].c.w;
    do_next_opcode;
  }

  c_lmul: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.s64 *= z1.s64;  
    frame->top -= 2;
    frame->top[-1].c.w = z2.words[WORD_MSW];
    frame->top[-2].c.w = z2.words[WORD_LSW];
    do_next_opcode;
  }

  c_dmul: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.d64 = float64_mul(z2.d64, z1.d64);
    frame->top[-3].c.w = z2.words[WORD_MSW];
    frame->top[-4].c.w = z2.words[WORD_LSW];
    frame->top -= 2;
    do_next_opcode;
  }

  c_idiv: {
    frame->top -= 2;

    if ((w_int) frame->top[1].c.w == 0) {
      do_throw_clazz(clazzArithmeticException);
    }
  
    if (((w_int) frame->top[0].c.w >= 0) && ((w_int) frame->top[1].c.w > 0)) {
      frame->top[0].c.w = (w_int) frame->top[0].c.w / (w_int) frame->top[1].c.w;
    }
    else if ((w_int) frame->top[0].c.w == (w_int) 0x80000000) {
      if ((w_int) frame->top[1].c.w == -1) {
        frame->top[0].c.w = 0x80000000;
      }
      else {
        frame->top[0].c.w = (w_int) frame->top[0].c.w / (w_int) frame->top[1].c.w;
      }
    }
    else if (((w_int) frame->top[0].c.w < 0) && ((w_int) frame->top[1].c.w < 0)) {
      frame->top[0].c.w = -(w_int) frame->top[0].c.w;
      frame->top[1].c.w = -(w_int) frame->top[1].c.w;
      frame->top[0].c.w = (w_int) frame->top[0].c.w / (w_int) frame->top[1].c.w;
    }
    else if ((w_int) frame->top[0].c.w < 0) {
      frame->top[0].c.w = -(w_int) frame->top[0].c.w;
      frame->top[0].c.w = -((w_int) frame->top[0].c.w / (w_int) frame->top[1].c.w);
    }
    else {
      frame->top[1].c.w = -(w_int) frame->top[1].c.w;
      frame->top[0].c.w = -((w_int) frame->top[0].c.w / (w_int) frame->top[1].c.w);
    }
  
    frame->top += 1;
    do_next_opcode;
  }

  c_ldiv: {
    z2.words[WORD_MSW] = frame->top[-1].c.w;
    z2.words[WORD_LSW] = frame->top[-2].c.w;
    z1.words[WORD_MSW] = frame->top[-3].c.w;
    z1.words[WORD_LSW] = frame->top[-4].c.w;
    frame->top -= 2;

    if (z2.s64 == 0) {
      do_throw_clazz(clazzArithmeticException);
    }

    if (z1.s64 >= 0 && z2.s64 > 0) {
      z3.s64 = z1.s64 / z2.s64;
    }
    else if (z1.s64 == (w_long) 0x8000000000000000) {
      if (z2.s64 == -1) {
        z3.s64 = 0x8000000000000000;
      }
      else {
        z3.s64 = z1.s64 / z2.s64;
      }
    }
    else if (z1.s64 < 0 && z2.s64 < 0) {
      z1.s64 = -z1.s64;
      z2.s64 = -z2.s64;
      z3.s64 = z1.s64 / z2.s64;
    }
    else if (z1.s64 < 0) { /* z2>0 */
      z1.s64 = -z1.s64;
      z3.s64 = -(z1.s64 / z2.s64);
    }
    else { /* z1 > 0, z2 < 0 */
      z2.s64 = -z2.s64;
      z3.s64 = -(z1.s64 / z2.s64);
    }

    frame->top[-2].c.w = z3.words[WORD_LSW];
    frame->top[-1].c.w = z3.words[WORD_MSW];
    do_next_opcode;
  }

  c_fdiv: {
    frame->top -= 1;
    frame->top[-1].c.f = float32_div(frame->top[-1].c.f, frame->top[0].c.f);
    do_next_opcode;
  }

  c_ddiv: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z2.words[WORD_MSW] = frame->top[-3].c.w;
    z2.words[WORD_LSW] = frame->top[-4].c.w;
    z2.d64 = float64_div(z2.d64, z1.d64);
    frame->top[-3].c.w = z2.words[WORD_MSW];
    frame->top[-4].c.w = z2.words[WORD_LSW];
    frame->top -= 2;
    do_next_opcode;
  }

  c_lrem: {
    z2.words[WORD_MSW] = frame->top[-1].c.w;
    z2.words[WORD_LSW] = frame->top[-2].c.w;
    z1.words[WORD_MSW] = frame->top[-3].c.w;
    z1.words[WORD_LSW] = frame->top[-4].c.w;

    if (z2.s64 == 0) {
      do_throw_clazz(clazzArithmeticException);
    }

    z3.s64 = 0x0000000000000000LL;

    if (z1.s64 != (w_long) 0x8000000000000000 || z2.s64 != -1) {
      z3.s64 = z1.s64 - z2.s64 * (w_long) (z1.s64 / z2.s64);
    }

    frame->top -= 2;
    frame->top[-2].c.w = z3.words[WORD_LSW];
    frame->top[-1].c.w = z3.words[WORD_MSW];
    do_next_opcode;
  }

  c_frem: {
    do_frem(frame);
    do_next_opcode;
  }

  c_drem: {
    do_drem(frame);
    do_next_opcode;
  }

  c_ineg: {
    frame->top[-1].c.w = - (w_int) frame->top[-1].c.w;
    do_next_opcode;
  }

  c_lneg: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z1.s64 = - z1.s64;
    frame->top[-2].c.w = z1.words[WORD_LSW];
    frame->top[-1].c.w = z1.words[WORD_MSW];
    do_next_opcode;
  }

  c_fneg: {
    frame->top[-1].c.f = float32_negate(frame->top[-1].c.f);
    do_next_opcode;
  }

  c_dneg: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z1.d64 = float64_negate(z1.d64);
    frame->top[-2].c.w = z1.words[WORD_LSW];
    frame->top[-1].c.w = z1.words[WORD_MSW];
    do_next_opcode;
  }

  c_ishl: {
    frame->top -= 1;
    frame->top[-1].c.w <<= (frame->top[0].c.w & 0x0000001f);
    do_next_opcode;
  }

  c_lshl: {
    i = frame->top[-1].c.w & 0x0000003f;
    z1.words[WORD_MSW] = frame->top[-2].c.w;
    z1.words[WORD_LSW] = frame->top[-3].c.w;
    z1.s64 <<= i;
    frame->top -= 1;
    frame->top[-1].c.w = z1.words[WORD_MSW];
    frame->top[-2].c.w = z1.words[WORD_LSW];
    do_next_opcode;
  }

  c_ishr: {
    s = frame->top[-1].c.w & 0x0000001f;
    i = frame->top[-2].c.w;
    frame->top -= 1;
    if (i > 0) {
      frame->top[-1].c.w = i >> s;
    }
    else {
      frame->top[-1].c.w = -1 - (( -1 - i) >> s);
    }
    do_next_opcode;
  }

  c_lshr: {
    s = frame->top[-1].c.w & 0x0000003f;
    z1.words[WORD_MSW] = frame->top[-2].c.w;
    z1.words[WORD_LSW] = frame->top[-3].c.w;
    frame->top -= 1;
    if (z1.s64 > 0) {
      z1.s64 >>= s;
    } 
    else {
      z1.s64 = -1 - ((-1 - z1.s64) >> s);
    }
    frame->top[-1].c.w = z1.words[WORD_MSW];
    frame->top[-2].c.w = z1.words[WORD_LSW];
    do_next_opcode;
  }

  c_iushr: {
    frame->top -= 1;
    frame->top[-1].c.w >>= (frame->top[0].c.w & 0x0000001f);
    do_next_opcode;
  }

  c_lushr: {
    s = frame->top[-1].c.w & 0x0000003f;
    z1.words[WORD_MSW] = frame->top[-2].c.w;
    z1.words[WORD_LSW] = frame->top[-3].c.w;
    z1.u64 >>= s;
    frame->top -= 1;
    frame->top[-2].c.w = z1.words[WORD_LSW];
    frame->top[-1].c.w = z1.words[WORD_MSW];
    do_next_opcode;
  }


  c_land: {
    frame->top -= 2; 
    frame->top[-1].c.w &= frame->top[1].c.w;
    frame->top[-2].c.w &= frame->top[0].c.w;
    do_next_opcode;
  }

  c_ior: {
    frame->top -= 1;
    frame->top[-1].c.w |= frame->top[0].c.w;
    do_next_opcode;
  }

  c_lor: {
    frame->top -= 2;
    frame->top[-1].c.w |= frame->top[1].c.w;
    frame->top[-2].c.w |= frame->top[0].c.w;
    do_next_opcode;
  }

  c_ixor: {
    frame->top -= 1;
    frame->top[-1].c.w ^= frame->top[0].c.w;
    do_next_opcode;
  }

  c_lxor: {
    frame->top -= 2;
    frame->top[-1].c.w ^= frame->top[1].c.w;
    frame->top[-2].c.w ^= frame->top[0].c.w;
    do_next_opcode;
  }

  c_i2l: {
    z1.s64 = (signed long long) (w_int) frame->top[-1].c.w;
    frame->top[-1].c.w = z1.words[WORD_LSW];
    frame->top[ 0].s = stack_notrace;
    frame->top[ 0].c.w = z1.words[WORD_MSW];
    frame->top += 1;
    do_next_opcode;
  }

  c_i2d: {
    z1.d64 = int32_to_float64(frame->top[-1].c.w);
    frame->top[-1].c.w = z1.words[WORD_LSW];
    frame->top[ 0].s = stack_notrace;
    frame->top[ 0].c.w = z1.words[WORD_MSW];
    frame->top += 1;
    do_next_opcode;
  }

  c_f2l: {
    if (float32_is_NaN(frame->top[-1].c.f)) {
      z1.s64 = 0x0000000000000000;
    }
    else {
      z1.s64 = float32_to_int64_round_to_zero(frame->top[-1].c.f);
    }
    frame->top[-1].c.w = z1.words[WORD_LSW];
    frame->top[ 0].s = stack_notrace;
    frame->top[ 0].c.w = z1.words[WORD_MSW];
    frame->top += 1;
    do_next_opcode;
  }

  c_f2d: {
    z1.d64 = float32_to_float64(frame->top[-1].c.f);
    frame->top[-1].c.w = z1.words[WORD_LSW];
    frame->top[ 0].s = stack_notrace;
    frame->top[ 0].c.w = z1.words[WORD_MSW];
    frame->top += 1;
    do_next_opcode;
  }

  c_l2f: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    frame->top -= 1;
    frame->top[-1].c.f = int64_to_float32(z1.s64);
    do_next_opcode;
  }

  c_l2d: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    z1.d64 = int64_to_float64(z1.s64);
    frame->top[-1].c.w = z1.words[WORD_MSW];
    frame->top[-2].c.w = z1.words[WORD_LSW];
    do_next_opcode;
  }

  c_f2i: {
    if (float32_is_NaN(frame->top[-1].c.f)) {
      frame->top[-1].c.w = 0;
    }
    else {
      frame->top[-1].c.w = float32_to_int32_round_to_zero(frame->top[-1].c.f);
    }
    do_next_opcode;
  }

  c_d2i: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    if (float64_is_NaN(z1.d64)) {
      frame->top[-2].c.w = 0;
    }
    else {
      frame->top[-2].c.w = (w_int) float64_to_int32_round_to_zero(z1.d64);
    }
    frame->top -= 1;
    do_next_opcode;
  }

  c_d2l: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    if (float64_is_NaN(z1.d64)) {
      z1.s64 = 0x0000000000000000;
    }
    else {
      z1.s64 = float64_to_int64_round_to_zero(z1.d64);
    }
    frame->top[-2].c.w = z1.words[WORD_LSW];
    frame->top[-1].c.w = z1.words[WORD_MSW];
    do_next_opcode;
  }

  c_d2f: {
    z1.words[WORD_MSW] = frame->top[-1].c.w;
    z1.words[WORD_LSW] = frame->top[-2].c.w;
    frame->top[-2].c.f = float64_to_float32(z1.d64);
    frame->top -= 1;
    do_next_opcode;
  }

  c_i2b: {
    frame->top[-1].c.w = extend_b2i(frame->top[-1].c.w & 0x000000ff);
    do_next_opcode;
  }


  c_i2s: {
    frame->top[-1].c.w = extend_s2i(frame->top[-1].c.w & 0x0000ffff);
    do_next_opcode;
  }

  c_ifgt: {
    frame->top -= 1; 
    do_conditional((w_int) frame->top[0].c.w  > 0);
  }

  c_ifle: {
    frame->top -= 1; 
    do_conditional((w_int) frame->top[0].c.w <= 0);
  }

  c_if_icmpne: c_if_acmpne: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w != (w_int) frame->top[1].c.w);
  }
  
  c_if_icmpge: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w >= (w_int) frame->top[1].c.w);
  }
  
  c_if_icmpgt: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w  > (w_int) frame->top[1].c.w);
  }
  
  c_if_icmple: {
    frame->top -= 2; 
    do_conditional((w_int) frame->top[0].c.w <= (w_int) frame->top[1].c.w);
  }

  c_jsr: {
    frame->top[0].s = stack_notrace;
    frame->top[0].c.w = (w_word) (current + 3);
    frame->top += 1;
    add_to_opcode(short_operand);
  }

  c_ret: {
    current = (w_code) frame->Slots[*(current + 1)].c.w;
    do_this_opcode;
  }

  c_tableswitch: {
#ifdef GC_SAFE_POINTS
    if (pendingGCRequest(thread)) {
      gcSafePoint(thread);
    }
#endif
    table = (signed int *) ((unsigned int)((unsigned char *)current + 4) & ~3);
    frame->top -= 1;
    i = (w_int) frame->top[0].c.w;
    if (i >= table[1] && i <= table[2]) {
      add_to_opcode(table[i - table[1] + 3]);
    }
    else {
      add_to_opcode(table[0]);
    }
  }

  c_lookupswitch: {
#ifdef GC_SAFE_POINTS
    if (pendingGCRequest(thread)) {
      gcSafePoint(thread);
    }
#endif
    // todo, make this a binary lookup instead of linear search...
    table = (signed int *) ((unsigned int)((unsigned char *)current + 4) & ~3);
    mopair = (w_Mopair *) (table + 2);

    frame->top -= 1;
    s = (w_int) frame->top[0].c.w;

    for (i = 0; i < table[1]; i++) {
      if (mopair[i].m == s) {
        add_to_opcode(mopair[i].o);
      }
      if (mopair[i].m > s) {
        add_to_opcode(table[0]);
      }
    }
    add_to_opcode(table[0]);
  }

  c_lreturn: c_dreturn: {
    caller->top -= method->exec.arg_i;
    caller->top[0].s = stack_notrace;
    caller->top[0].c.w = frame->top[-2].c.w;
    caller->top[1].s = stack_notrace;
    caller->top[1].c.w = frame->top[-1].c.w;
    caller->top += 2;
    thread->top = caller;
    return;
  }

  c_vreturn: {
    caller->top -= method->exec.arg_i;
    thread->top = caller;
    return;
  }

  c_getstatic: {
    field = getFieldConstant(cclazz, short_operand);

    if (thread->exception) {
      do_the_exception;
    }

    if (field) {
      classMustBeInitialized(field->declaring_clazz);
    }
    else if (thread->exception == NULL) {
      do_throw_clazz(clazzLinkageError);
    }

    if (thread->exception) {
      do_the_exception;
    }

    *current = in_getstatic;
    
    goto i_getstatic;
  }

  c_putstatic: {
    field = getFieldConstant(cclazz, (unsigned short) short_operand);

    if (thread->exception) {
      do_the_exception;
    }

    if (field) {
      classMustBeInitialized(field->declaring_clazz);
    }
    else if (thread->exception == NULL) {
      do_throw_clazz(clazzLinkageError);
    }
    if (thread->exception) {
      do_the_exception;
    }

    *current = in_putstatic;
    
    goto i_putstatic;
  }

  c_getfield: {
    field = getFieldConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }

    if (field) {
      *current = in_getfield;

      goto i_getfield;

    } else if (thread->exception == NULL) {
      do_throw_clazz(clazzLinkageError);
    }
  }

  c_putfield: {
    field = getFieldConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }

    if (field) {
      *current = in_putfield;

      goto i_putfield;

    } else if (thread->exception == NULL) {
      do_throw_clazz(clazzLinkageError);
    }
    
  }

  c_invokevirtual: {
    frame->current = current;
    x = getMethodConstant(frame->method->spec.declaring_clazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }
    if (!x) {
      do_throw_clazz(clazzAbstractMethodError);
    }
    if (isSet(x->flags, ACC_STATIC)) {
      do_throw_clazz(clazzIncompatibleClassChangeError);
    }

    if (isSet(x->flags, METHOD_NO_OVERRIDE)) {
      *current = in_invokenonvirtual;

      goto i_invokenonvirtual;

    }
    else {
      *current = in_invokevirtual;

      goto i_invokevirtual;

    }

  }

  i_invokevirtual: {
    w_instance objectref;

    frame->current = current;
    x = getResolvedMethodConstant(frame->method->spec.declaring_clazz, (unsigned short) short_operand);

    objectref = (w_instance) frame->top[- x->exec.arg_i].c.w;
    if (! objectref) {
      do_throw_clazz(clazzNullPointerException);
    }


    x = virtualLookup(x, instance2object(objectref)->clazz);
    if (!x) {
      do_the_exception;
    }
    if (isSet(x->flags, ACC_ABSTRACT)) {
      do_throw_clazz(clazzAbstractMethodError);
    }

    i_callMethod(frame, x);
    current += 2;
    goto check_async_exception;
  }

  c_invokespecial: {
    frame->current = current;
    x = getMethodConstant(frame->method->spec.declaring_clazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }
    if (!x) {
      do_throw_clazz(clazzAbstractMethodError);
    }
    if (isSet(x->flags, ACC_STATIC)) {
      do_throw_clazz(clazzIncompatibleClassChangeError);
    }

    if ((x->spec.declaring_clazz->flags & (ACC_FINAL | ACC_SUPER)) != ACC_SUPER || isSet(x->flags, ACC_PRIVATE) || isSet(x->flags, METHOD_IS_CONSTRUCTOR) || !isSuperClass(x->spec.declaring_clazz, getSuper(frame->method->spec.declaring_clazz))) {
      *current = in_invokenonvirtual;

      goto i_invokenonvirtual;

    }
    else {
      *current = in_invokesuper;

      goto i_invokesuper;

    }
  }

  i_invokenonvirtual: {
    frame->current = current;
    x = getResolvedMethodConstant(frame->method->spec.declaring_clazz, (unsigned short) short_operand);

    if (! frame->top[- x->exec.arg_i].c.w) {
      do_throw_clazz(clazzNullPointerException);
    }

    i_callMethod(frame, x);
    current += 2;
    goto check_async_exception;
  }

  i_invokesuper: {
    frame->current = current;
    x = getResolvedMethodConstant(frame->method->spec.declaring_clazz, (unsigned short) short_operand);
    x = virtualLookup(x, getSuper(frame->method->spec.declaring_clazz));

    if (!x) {
      do_the_exception;
    }
    if (isSet(x->flags, ACC_ABSTRACT)) {
      do_throw_clazz(clazzAbstractMethodError);
    }

    if (! frame->top[- x->exec.arg_i].c.w) {
      do_throw_clazz(clazzNullPointerException);
    }

    i_callMethod(frame, x);
    current += 2;
    goto check_async_exception;
  }

  c_invokestatic: {
    frame->current = current;
    x = getMethodConstant(cclazz, (unsigned short) short_operand);
    if (! x) {
      do_throw_clazz(clazzAbstractMethodError);
    }

    if (thread->exception) {
      do_the_exception;
    }

    if (!(isSet(x->flags, ACC_STATIC) && isNotSet(x->flags, METHOD_IS_CLINIT))) {
      do_throw_clazz(clazzIncompatibleClassChangeError);
    }

    if (thread->exception) {
      do_the_exception;
    }

    *current = in_invokestatic;
    
    goto i_invokestatic;
  }

  c_invokeinterface: {
    frame->current = current;
    x = do_invokeinterface(frame, (unsigned short) short_operand);
    if (x == NULL) {
      do_the_exception;
    }
    i_callMethod(frame, x);
    current += 4;
    goto check_async_exception;
  }

  c_new: {
    c = getClassConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }

    classMustBeInitialized(c);
    if (thread->exception) {
      do_the_exception;
    }
    
    *current = in_new;
    
    goto i_new;
  }


  c_newarray: {
    s = frame->top[-1].c.w;

    if (s < 0) {
      do_throw_clazz(clazzNegativeArraySizeException);
    }

    a = allocArrayInstance(thread, atype2clazz[*(current + 1)], 1, &s);

    if (a == NULL) {
      do_throw_clazz(clazzOutOfMemoryError);
    }

    frame->top[-1].c.w = (w_word) a;
    frame->top[-1].s = stack_trace;
#ifndef GC_SAFE_POINTS
    popLocalReference(frame);
#endif
    current += 1;
    goto check_async_exception;
  }

  c_anewarray: {
    if ((w_int) frame->top[-1].c.w < 0) {
      do_throw_clazz(clazzNegativeArraySizeException);
    }

    c = getClassConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }

    c = getNextDimension(c);
    if (c == NULL) {
      do_throw_clazz(clazzOutOfMemoryError);
    }

    a = allocArrayInstance(thread, c, 1, &frame->top[-1].c.w);
    if (a == NULL) {
      do_throw_clazz(clazzOutOfMemoryError);
    }

    frame->top[-1].c.w = (w_word) a;
    frame->top[-1].s = stack_trace;
#ifndef GC_SAFE_POINTS
    popLocalReference(frame);
#endif
    current += 2;
    goto check_async_exception;
  }

  c_athrow: {
    if (frame->top[-1].c.w == 0) {
      do_throw_clazz(clazzNullPointerException);
    }
    thread->exception = (w_instance) frame->top[-1].c.w;
    do_the_exception;
  }

  c_instanceof: {
    c = getClassConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      do_the_exception;
    }
    o = (w_instance) frame->top[-1].c.w;
    frame->top[-1].s = stack_notrace;
    frame->top[-1].c.w = (o == NULL) ? 0 : (isAssignmentCompatible(instance2object(o)->clazz, c) ? 1 : 0);
    add_to_opcode(3);
  }


  c_wide: {
    current += 1;
    s = *current;
    i = (w_int) short_operand;
    
    switch (s) {
      case iload:  
      case fload: 
        do_xload(frame, i); 
        add_to_opcode(3);

      case aload: 
        do_aload(frame, i); 
        add_to_opcode(3);

      case lload:  
      case dload: 
        do_zload(frame, i); 
        add_to_opcode(3);

      case istore: 
      case fstore: 
        do_xstore(frame, i); 
        add_to_opcode(3);

      case astore:
        do_astore(frame, i); 
        add_to_opcode(3);

      case lstore: 
      case dstore: 
        do_zstore(frame, i); 
        add_to_opcode(3);

      case ret: 
        current = (w_code) frame->Slots[i].c.w;
        do_this_opcode;

      case iinc:
        current += 2;
        frame->Slots[i].c.w += (w_int) short_operand;
        current += 3;
        do_this_opcode;
    }
  }

  c_multianewarray: {
    w_int * dimensions;

    s = (unsigned int) (unsigned char) *(current + 3);
    dimensions = x_mem_alloc(sizeof(w_int) * s);
    for (i = 0; i < s; i++) {
      dimensions[i] = frame->top[-s + i].c.w;
      if (dimensions[i] < 0) {
        x_mem_free(dimensions);
        do_throw_clazz(clazzNegativeArraySizeException);
      }
    }
    frame->top -= s;

    c = getClassConstant(cclazz, (unsigned short) short_operand);
    if (thread->exception) {
      x_mem_free(dimensions);
      do_the_exception;
    }

    a = allocArrayInstance(thread, c, s, dimensions);
    x_mem_free(dimensions);
    
    if (a == NULL) {
      do_throw_clazz(clazzOutOfMemoryError);
    }

    frame->top[0].c.w = (w_word) a;
    frame->top[0].s = stack_trace;
    frame->top += 1;
#ifndef GC_SAFE_POINTS
    popLocalReference(frame);
#endif
    current += 3;
    do_next_opcode;
  }

  c_goto_w: {
#ifdef GC_SAFE_POINTS
    if (pendingGCRequest(thread)) {
      gcSafePoint(thread);
    }
#endif
    add_to_opcode(int_operand);
  }

  c_jsr_w: {
    frame->top[0].s = stack_notrace;
    frame->top[0].c.w = (w_word) (current + 5);
    frame->top += 1;
    add_to_opcode(int_operand);
  }

  c_breakpoint: {
    do_next_opcode;
  }

  c_nocode_211: c_nocode_212: c_nocode_213: c_nocode_214: c_nocode_215: c_nocode_216:
  c_nocode_217: c_nocode_218: c_nocode_219: c_nocode_220: c_nocode_221: c_nocode_222: c_nocode_223:
  c_nocode_224: c_nocode_225: c_nocode_226: c_nocode_227: c_nocode_228: c_nocode_229: c_nocode_230:
  c_nocode_231: c_nocode_232: c_nocode_233: c_nocode_234: c_nocode_235: c_nocode_236: c_nocode_237:
  c_nocode_238: c_nocode_239: c_nocode_240: c_nocode_241: c_nocode_242: c_nocode_243: c_nocode_244:
  c_nocode_245: c_nocode_246: c_nocode_247: c_nocode_248: c_nocode_249: c_nocode_250: c_nocode_251:
  c_nocode_252: c_nocode_253: abort();

  /*
  ** Exception logic:
  ** c_clazz2exception: allocates the exeception instance given by clazz 'c' and falls through to c_exception
  **       c_exception: (opcode at -1) records the current opcode in the frame, falls through to c_find_handler
  **    c_find_handler: tries to find a handler, handler returns correct code pointer or code pointer for 'c_no_handler'
  **      c_no_handler: (opcode at -2) is called when there is no handler and returns
  */

  c_clazz2exception: {
    if (!thread->exception) {
      thread->exception = allocInstance(thread, c);
#ifndef GC_SAFE_POINTS
      if (thread->exception) {
        popLocalReference(frame);
      }
#endif
    }
  }
  
  c_exception: {
    throwExceptionInstance(thread, thread->exception);
    frame->current = current;
  }
  
  c_find_handler: {
    current = searchHandler(frame);
    do_next_opcode;
  }

  c_no_handler: {
    frame->auxs = frame->previous->auxs;
    caller->thread->top = caller;
    return;
  }


}

void do_frem(x_frame frame) {
  typedef union {
    w_float f;
    w_word w;
  } w32;

  w32 result;
  w32 convert;

  if (float32_is_NaN(frame->top[-2].c.f) || float32_is_NaN(frame->top[-1].c.f) || float32_eq(frame->top[-1].c.f , F_ZERO) || float32_is_Infinite(frame->top[-2].c.f)) {
    result.w = F_NAN;
  }
  else {
    if (float32_eq(frame->top[-2].c.f , F_ZERO) || float32_is_Infinite(frame->top[-1].c.f)) {
      result.f = frame->top[-2].c.f;
    }
    else   {
      convert.w = F_FLOAT_MAX_VALUE;
      result.f = float32_mul(float32_abs(frame->top[-1].c.f), convert.f);
      if (float32_lt(result.f, float32_abs(frame->top[-2].c.f))) {
        woempa(7, "fmod: division would overflow, returning zero as remainder.\n");
        if (float32_is_negative(frame->top[-2].c.f)) {
          result.w = F_MINUS_ZERO;
        }
        else {
          result.w = F_ZERO;
        }
      }
      else {
        result.f = float32_div(frame->top[-2].c.f, frame->top[-1].c.f);
        result.f = int32_to_float32(float32_to_int32_round_to_zero(result.f));
        result.f = float32_mul(frame->top[-1].c.f, result.f);
        result.f= float32_sub(frame->top[-2].c.f, result.f);
      }
    }
  }

  frame->top -= 1;
  frame->top[-1].c.f = result.f;

}

void do_drem(x_frame frame) {

  w_u64 n;
  w_u64 z;
  w_u64 result;
  w_u64 convert;

  z.words[WORD_MSW] = frame->top[-1].c.w;
  z.words[WORD_LSW] = frame->top[-2].c.w;
  n.words[WORD_MSW] = frame->top[-3].c.w;
  n.words[WORD_LSW] = frame->top[-4].c.w;


  if (float64_is_NaN(n.d64) || float64_is_NaN(z.d64) || float64_eq(z.d64, D_ZERO) || float64_is_Infinite(n.d64)) {
    result.u64 = D_NAN;
  }       	
  else {	
    if (float64_eq(n.d64, D_ZERO) || float64_is_Infinite(z.d64)) {
      result.d64 = n.d64;
    }
    else {
      convert.u64 = D_DOUBLE_MAX_VALUE;
      result.d64 = float64_mul(float64_abs(z.d64), convert.d64);
      if (float64_lt(result.d64, float64_abs(n.d64))) {
        woempa(7, "dmod: division would overflow, returning zero as remainder.\n");
        if (float64_is_negative(n.d64)) {
          result.u64 = D_MINUS_ZERO;
        }
        else {
          result.u64 = D_ZERO;
        }
      }
      else {
        result.d64 = float64_div(n.d64, z.d64);
        /*
        ** Rounding is not what we would like since it doesn't round to zero ...
        */

        result.d64 = float64_round_to_int(result.d64);
        result.d64 = float64_mul(z.d64, result.d64);
        result.d64 = float64_sub(n.d64, result.d64);

        /*
        ** The sign of result must be equal to sign of n this is not guaranteed
        ** anymore because of the rounding.
        */

        if (float64_is_negative(n.d64) && (!float64_is_negative(result.d64) && result.d64)) {
          if (float64_is_negative(z.d64)){
            result.d64 = float64_add(z.d64, result.d64);
          }
          else{
            result.d64 = float64_sub(result.d64, z.d64);
          }
        }
        else if ((!float64_is_negative(n.d64)) && float64_is_negative(result.d64) && result.d64) {
          if (float64_is_negative(z.d64)){
            result.d64 = float64_sub(result.d64, z.d64);
          }
          else{
            result.d64 = float64_add(z.d64, result.d64);
          }
        }
      }
    }
  }
  frame->top[-3].c.w = result.words[WORD_MSW];
  frame->top[-4].c.w = result.words[WORD_LSW];
  frame->top -= 2;

}

/*
** In the following function, we rewrite the jump codes in the native
** endianess format of the CPU of lookupswitch and tableswitch. Not other
** indexes since they are not guaranteed to be on properly aligned boundaries.
*/

void adaptEndianess(w_method method) {

  w_int pc = 0;
  w_int i;
  w_int * n;
  w_Mopair * mopair;
  unsigned char * bytecode;

  if (isSet(method->flags, ACC_ABSTRACT | ACC_NATIVE)) {
    return;
  }

  bytecode = method->exec.code;

  while (pc < method->exec.code_length) {
    switch (bytecode[pc]) {
      case aload: case astore: case bipush: case dload: case dstore: case fload: case fstore:
      case iload: case istore: case ldc: case lload: case lstore: case newarray: case ret: {
        pc += 1; 
        break;
      }

      case if_acmpeq: case if_acmpne: case if_icmpeq: case if_icmpne: case if_icmplt: case if_icmpge: 
      case if_icmpgt: case if_icmple: case ifeq: case ifne: case iflt: case ifge: case ifgt: case ifle:
      case ifnonnull: case ifnull: case jsr: case sipush: case anewarray: case instanceof: case j_goto: 
      case checkcast: case ldc_w: case ldc2_w: case new: case getfield: case getstatic: case invokespecial: 
      case invokestatic: case invokevirtual: case putfield: case putstatic: case iinc: {
        pc += 2;
        break;
      }
      
      case multianewarray: {
        pc += 3;
        break;
      }
      
      case invokeinterface: case goto_w: case jsr_w: {
        pc += 4;
        break;
      }
      
      case wide: {
        pc += 1;
        if (bytecode[pc] == iinc) {
          pc += 2;
        }
        pc += 2;
        break;
      }

      case tableswitch: {
        pc = (pc + 4) & ~3;
        n = (w_int *) & bytecode[pc];
        n[0] = (w_int) ((bytecode[pc + 0] << 24) | (bytecode[pc + 1] << 16) | (bytecode[pc +  2] << 8) | bytecode[pc +  3]);
        n[1] = (w_int) ((bytecode[pc + 4] << 24) | (bytecode[pc + 5] << 16) | (bytecode[pc +  6] << 8) | bytecode[pc +  7]);
        n[2] = (w_int) ((bytecode[pc + 8] << 24) | (bytecode[pc + 9] << 16) | (bytecode[pc + 10] << 8) | bytecode[pc + 11]);
        pc += 12;
        for (i = 3; i < n[2] - n[1] + 1 + 3; i++) {
          n[i] = (w_int) ((bytecode[pc + 0] << 24) | (bytecode[pc + 1] << 16) | (bytecode[pc +  2] << 8) | bytecode[pc +  3]);
          pc += 4;
        }
        pc -= 1;
        break;
      }

      case lookupswitch: {
        pc = (pc + 4) & ~3;
        n = (w_int *) & bytecode[pc];
        n[0] = ((bytecode[pc + 0] << 24) | (bytecode[pc + 1] << 16) | (bytecode[pc + 2] << 8) | bytecode[pc + 3]);
        n[1] = ((bytecode[pc + 4] << 24) | (bytecode[pc + 5] << 16) | (bytecode[pc + 6] << 8) | bytecode[pc + 7]);
        pc += 8;
        mopair = (w_Mopair *) (n + 2);
        for (i = 0; i < n[1]; i++) {
          mopair[i].m = ((bytecode[pc + 0] << 24) | (bytecode[pc + 1] << 16) | (bytecode[pc + 2] << 8) | bytecode[pc + 3]);
          mopair[i].o = ((bytecode[pc + 4] << 24) | (bytecode[pc + 5] << 16) | (bytecode[pc + 6] << 8) | bytecode[pc + 7]);
          pc += 8;
        }
        pc -= 1;
        break;
      }

    }
    
    pc += 1;
    
  }
  
}

// HACKS !!!

w_method do_invokeinterface(x_frame frame, w_int i) {

  w_method   interf_method;
  w_method   actual_method;
  w_instance objectref;
  w_clazz    actual_clazz;

  interf_method = getIMethodConstant(frame->method->spec.declaring_clazz, i);
  if (! interf_method) {
    if (! frame->thread->exception) {
      frame->thread->exception = allocInstance(frame->thread, clazzNoSuchMethodError);
    }
    return NULL;
  }

  classMustBeInitialized(interf_method->spec.declaring_clazz);
  if (frame->thread->exception) {
    return NULL;
  }
  
  objectref = (w_instance) frame->top[- interf_method->exec.arg_i].c.w;

  if (! objectref) {
    frame->thread->exception = allocInstance(frame->thread, clazzNullPointerException);
    return NULL;
  }

  woempa(1, "Object %j should implement %m\n", objectref, interf_method);
  actual_clazz = instance2object(objectref)->clazz;
  actual_method = interfaceLookup(interf_method, actual_clazz);
  if (! actual_method) {
    frame->thread->exception = allocInstance(frame->thread, clazzAbstractMethodError);
    return NULL;
  }

  if (isSet(actual_method->flags, ACC_STATIC)) {
    frame->thread->exception = allocInstance(frame->thread, clazzIncompatibleClassChangeError);
    return NULL;
  }
  
  if (isNotSet(actual_method->flags, ACC_PUBLIC)) {
    frame->thread->exception = allocInstance(frame->thread, clazzIllegalAccessError);
    return NULL;
  }
  
  if (isSet(actual_method->flags, ACC_ABSTRACT)) {
    if (isSet(actual_method->flags, METHOD_IS_MIRANDA)) {
      woempa(1, "Miranda! %M\n", actual_method);
    }
    frame->thread->exception = allocInstance(frame->thread, clazzAbstractMethodError);
    return NULL;
  }

  classMustBeInitialized(actual_method->spec.declaring_clazz);
  if (frame->thread->exception) {
    return NULL;
  }

  return actual_method;

}

static w_code searchHandler(x_frame frame) {

  w_thread thread = frame->thread;
  w_exception ex;
  w_clazz cc;
  w_int i;
  x_slot auxs;
  w_int pc = frame->current - frame->method->exec.code;
  w_instance pending;
  w_code nohandler = frame->method->exec.code - 3;

  /*
  ** Store the pending exception locally and clear the thread exception, since
  ** resolving the class constant could result in loading/initializing etc. and
  ** that will check for pending exceptions. When we don't handle the exception
  ** ourselves, we reassign pending to the thread exception field. When resolving
  ** the catched clazz results itself in an exception, we don't handle anything, it will
  ** propagate probably to the final thread handler in the sky.
  **
  ** When we don't handle the exception in this frame, we don't bother releasing the
  ** local pending exception from the locals stack, it will be cleared anyhow when
  ** we return to our caller...
  */
  
  auxs = frame->auxs;
  pending = thread->exception;
  thread->exception = NULL;
  pushLocalReference(frame, pending);

  if (thread->exception == NULL) {
    for (i = 0; i <  frame->method->exec.numExceptions; i++) {
      ex = &frame->method->exec.exceptions[i];
      if (ex->type_index) {
        cc = getClassConstant(frame->method->spec.declaring_clazz, ex->type_index);
        if (cc == NULL || thread->exception) {
          pending = thread->exception ? thread->exception : pending;
          break;
        }
      }
      else {
        cc = NULL;
      }
      if (pc >= ex->start_pc && pc < ex->end_pc) {
        if (cc == NULL || isSuperClass(cc, instance2object(pending)->clazz)) {
//          woempa(9, ">>>> Found a handler for %k exception at pc = %d <<<<\n", instance2object(pending)->clazz, ex->handler_pc);
//          woempa(9, ">>>> in method %M, catchclazz = %p\n", frame->method, cc);
          frame->top = frame->Slots + frame->method->exec.local_i;
          frame->top[0].c.w = (w_word) pending;
          frame->top[0].s = stack_trace;
          frame->top += 1;
          setReferenceFieldUnsafe(thread->Thread, NULL, F_Thread_thrown);
          thread->exception = NULL;
          frame->auxs = auxs;
          return frame->method->exec.code + ex->handler_pc - 1;
        }
      }
    }
  }
  else {
    pending = thread->exception;
  }

  thread->exception = pending;

  setReferenceFieldUnsafe(thread->Thread, pending, F_Thread_thrown);

  return nohandler;
  
}

void dumpcalls(x_frame frame) {

  x_frame current = frame;
  w_int i = 0;
  
  while (current) {
    if (current->method) {
      woempa(9, "%2d method %m clazz %k\n", i, current->method, current->method->spec.declaring_clazz);
    }
    else {
      woempa(9, "%2d ------------------\n", i);
    }
    i += 1;
    current = current->previous;
  }

}

x_frame pushFrame(w_thread thread, w_method method) {

  x_frame frame;

  /*
  ** Add at least 2 slots for an eventual 64 bit return value.
  */

  frame = x_mem_calloc(sizeof(x_Frame) + (sizeof(x_Slot) * (method->exec.arg_i + 2)));

  if (frame) {
    frame->previous = thread->top;
    frame->thread = thread;
   // frame->method = method;
   // frame->clazz = method->spec.declaring_clazz;

    frame->auxs = thread->top->auxs;
    frame->top = frame->Slots;
    thread->top = frame;
  }

  return frame;

}

x_frame activateFrame(w_thread thread, w_method method, w_word flags, w_int nargs, ...) {

  va_list args;
  x_frame frame = pushFrame(thread, method);
  w_int i = 0;

  if (frame) {
    va_start(args, nargs);
    while (i < nargs) {
      frame->top[0].c.w = va_arg(args, w_word);
      frame->top[0].s = va_arg(args, w_word);
      frame->top += 1;
      i += 1;
    }
    va_end(args);

    frame->flags = flags;
    i_callMethod(frame, method);
  }
  
  return frame;

}

void deactivateFrame(x_frame frame, w_instance protect) {

  x_frame newtop = frame->previous;
  w_thread thread = frame->thread;

  if (protect) {
    pushLocalReference(newtop, protect);
  }

  thread->top = newtop;

  x_mem_free(frame);

}
