/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, bus 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: jni.c 4294 2006-05-10 11:35:19Z chris $
*/

#include <string.h>
#include <stdio.h>

#include "jni.h"

#include "arrays.h"
#include "bar.h"
#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "fields.h"
#include "hashtable.h"
#include "heap.h"
#include "ieee754.h"
#include "interpreter.h"
#include "loading.h"
#include "wstrings.h"
#include "threads.h"
#include "checks.h"
#include "locks.h"
#include "exception.h"
#include "ts-mem.h"
#include "methods.h"
#include "misc.h"

w_hashtable globals_hashtable;

#define GLOBALS_HASHTABLE_SIZE   1439

#ifdef RUDOLPH
#ifndef MODULES
extern void init_awt(void);
#endif
#endif

struct JavaVMInitArgs *system_vm_args;

typedef union JNITypes {
  jboolean  z;
  jbyte     b;
  jchar     c;
  jshort    s;
  jint      i;
  jfloat    f;
  jlong     j;
  jdouble   d;
  jobject   o;
  w_word    w[2];
  w_double  wd;
  w_float   wf;
  w_word    ww;
} JNITypes;

/*
** Write a C array of booleans (stored one per byte) into a Java array instance.
*/

void writeBooleansToArray(w_instance Array, w_int start, w_int length, jboolean *elements) {
  w_int    i;
  w_sbyte *  data_8;
  w_byte   bitmask;

  /*
  ** Find the correct offset (start / 8) and the correct bitposition
  ** (1 << (start % 8)) to start with...
  */

  data_8 = instance2Array_byte(Array) + (int)(start / 8);
  bitmask = 1;
  if (start % 8) bitmask <<= (start % 8);
  for (i = 0; i < length; i++) {
    if (elements[i]) {
      (*data_8) |=  bitmask;   /* set bit TRUE */
    }
    else {
      (*data_8) &= ~bitmask;   /* set bit FALSE */
    }
    
    bitmask <<= 1;
    if ((bitmask&0xff) == 0) {
      data_8 += 1;
      bitmask = 1;
    }
  }
  woempa(1, "(JNI) Wrote %d boolean elements from %d of Array %p.\n", length, start, Array);

}

/*
** Write a C array of bytes back into a Java array instance.
*/

void writeBytesToArray(w_instance Array, w_int start, w_int length, jbyte *elements) {
  w_sbyte *  dest = instance2Array_byte(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_byte));
  woempa(1, "(JNI) Wrote %d 8 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of shorts back into a Java array instance.
*/

void writeShortsToArray(w_instance Array, w_int start, w_int length, jshort *elements) {
  w_short * dest = instance2Array_short(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_short));
  woempa(1, "(JNI) Wrote %d 16 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of chars back into a Java array instance.
*/

void writeCharsToArray(w_instance Array, w_int start, w_int length, jchar *elements) {
  w_char* dest = instance2Array_char(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_char));
  woempa(1, "(JNI) Wrote %d 16 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of words back into a Java array instance.
*/

void writeIntsToArray(w_instance Array, w_int start, w_int length, w_int*elements) {
  w_int* dest = instance2Array_int(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_int));
  woempa(1, "(JNI) Written %d ints from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of longs back into a Java array instance.
*/

void writeLongsToArray(w_instance Array, w_int start, w_int length, w_long *elements) {
  w_long* dest = instance2Array_long(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_long));
  woempa(1, "(JNI) Written %d longs from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of floats back into a Java array instance.
*/

void writeFloatsToArray(w_instance Array, w_int start, w_int length, jfloat *elements) {
  w_float* dest = instance2Array_float(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_float));
  woempa(1, "(JNI) Written %d 32 bit elements from %d of Array %p.\n", length, start, Array);
}

/*
** Write a C array of doubles back into a Java array instance.
*/

void writeDoublesToArray(w_instance Array, w_int start, w_int length, jdouble *elements) {
  w_double* dest = instance2Array_double(Array) + start;

  w_memcpy(dest,elements,length*sizeof(w_double));
  woempa(1, "(JNI) Written %d doubles from %d of Array %p.\n", length, start, Array);
}

void readBooleansFromArray(w_instance Array, w_int start, w_int length, jboolean *buffer) {

  w_sbyte*  data_8;
  w_int    i;
  w_byte   bitmask;

  /*
  ** Find the correct offset (start / 8) and the correct bitposition
  ** (1 << (start % 8)) to start with...
  */

  data_8 = instance2Array_byte(Array) + (int)(start / 8);
  bitmask = 1 << (start % 8);
  for (i = 0; i < length; i++) {
    buffer[i] = (w_word)(*data_8 & bitmask);
    bitmask <<= 1;
    if (bitmask == 0) {
      data_8 += 1;
      bitmask = 1;
    }
  }
  woempa(1, "(JNI) Read %d boolean elements from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readBytesFromArray(w_instance Array, w_int start, w_int length, w_byte *buffer) {
  w_sbyte*  source = instance2Array_byte(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_byte));
  woempa(1, "(JNI) Read %d bytes from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readCharsFromArray(w_instance Array, w_int start, w_int length, w_char *buffer) {
  w_char*  source = instance2Array_char(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_char));
  woempa(1, "(JNI) Read %d chars from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readShortsFromArray(w_instance Array, w_int start, w_int length, w_short *buffer) {
  w_short*  source = instance2Array_short(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_short));
  woempa(1, "(JNI) Read %d shorts from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readIntsFromArray(w_instance Array, w_int start, w_int length, w_int*buffer) {
  w_int*  source = instance2Array_int(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_int));
  woempa(1, "(JNI) Read %d ints from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readLongsFromArray(w_instance Array, w_int start, w_int length, w_long *buffer) {
  w_long*  source = instance2Array_long(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_long));
  woempa(1, "(JNI) Read %d longs from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readFloatsFromArray(w_instance Array, w_int start, w_int length, jfloat *buffer) {
  w_float*  source = instance2Array_float(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_float));
  woempa(1, "(JNI) Read %d floats from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

void readDoublesFromArray(w_instance Array, w_int start, w_int length, jdouble *buffer) {
  w_double*  source = instance2Array_double(Array) + start;

  w_memcpy(buffer,source,length*sizeof(w_double));
  woempa(1, "(JNI) Read %d doubles from Array %p offset %d into buffer %p.\n", length, Array, start, buffer);
}

/*
** Convert an array of jvalues into items, starting from a method descriptor.
*/

static void jvalues2items(w_method method, jvalue arguments[], x_frame frame) {

  w_clazz arg_clazz;
  w_int i;

  if (method->spec.arg_types) {
    for (i = 0; method->spec.arg_types[i]; i++) {
      if (mustBeLoaded(&method->spec.arg_types[i]) != CLASS_LOADING_FAILED) {
        arg_clazz = method->spec.arg_types[i];
        if (clazzIsPrimitive(arg_clazz)) {
          switch (arg_clazz->type & 0x0f) {
            case VM_TYPE_BOOLEAN: frame->top[0].c.w = arguments[i].z & 0x01; break;
            case VM_TYPE_CHAR:    frame->top[0].c.w = arguments[i].c; break;
            case VM_TYPE_FLOAT:   frame->top[0].c.w = arguments[i].w[WORD_LSW]; break;
            case VM_TYPE_BYTE:    frame->top[0].c.w = arguments[i].b; break;
            case VM_TYPE_SHORT:   frame->top[0].c.w = arguments[i].s; break;
            case VM_TYPE_INT:     frame->top[0].c.w = arguments[i].i; break;

            case VM_TYPE_LONG: 
            case VM_TYPE_DOUBLE:
              frame->top[0].c.w = arguments[i].w[WORD_LSW];
              frame->top[1].c.w = arguments[i].w[WORD_MSW];
              frame->top += 1;
              break;

            default: 
              wabort(ABORT_WONKA, "Incorrect primitive type 0x%02x.\n", arg_clazz->type);        
          }
        }
        else {
          frame->top[0].c.w = arguments[i].i;
          frame->top[0].s = stack_trace;
        }
        frame->top += 1;
      }
    }
  }

}

/*
** Convert a va_list into items, starting from a method descriptor.
*/

static void va_list2items(w_method method, va_list list, x_frame frame) {

  w_clazz arg_clazz;
  w_int i;
  jvalue large;

  if (method->spec.arg_types) {
    for (i = 0; method->spec.arg_types[i]; i++) {
      if (mustBeLoaded(&method->spec.arg_types[i]) != CLASS_LOADING_FAILED) {
        arg_clazz = method->spec.arg_types[i];
        if (clazzIsPrimitive(arg_clazz)) {
          switch (arg_clazz->type & 0x0f) {
            case VM_TYPE_BOOLEAN:
            case VM_TYPE_CHAR:
            case VM_TYPE_BYTE:
            case VM_TYPE_SHORT:
            case VM_TYPE_INT:
              frame->top[0].c.w = va_arg(list, w_word);
              break;
  
            case VM_TYPE_FLOAT:
            
              /*
              ** Danger, danger, floats are always passed as doubles.
              ** We need to get them as a double and then convert them
              ** to a float.
              */
            
              {
                union { double d; float64 f64; } arg;
                arg.d = va_arg(list, double);
                frame->top[0].c.f = float64_to_float32(arg.f64);
              }
              break;
            case VM_TYPE_LONG:
              large = va_arg(list, jvalue);
              frame->top[0].c.w = large.w[WORD_LSW];
              frame->top[1].c.w = large.w[WORD_MSW];
              frame->top += 1;
              break;
            case VM_TYPE_DOUBLE:
              {
                union { double d; w_word w[2]; } arg;
                arg.d = va_arg(list, double);
                frame->top[0].c.w = arg.w[WORD_LSW];
                frame->top[1].c.w = arg.w[WORD_MSW];
                frame->top += 1;
              }
              break;
            default: wabort(ABORT_WONKA, "Incorrect primitive type 0x%02x.\n", arg_clazz->type);        
          }
        }
        else {
          frame->top[0].c.w = (w_word) va_arg(list, jobject);
          frame->top[0].s = stack_trace;
        }
        frame->top += 1;
      }
    }
  }

}

/*
** Generic calls for the virtual method invocations.
*/

static jvalue run_32_method(w_thread thread, x_frame frame, w_method method) {

  jvalue result;
  w_instance protected = NULL;

  callMethod(frame, method);
  if (! exceptionThrown(thread)) {
    result.i = frame->top[-1].c.w;
    if (isNotSet(method->spec.return_type->flags, CLAZZ_IS_PRIMITIVE)) {
      protected = result.l;
    }
  }

  deactivateFrame(frame, protected);
  
  return result;

}

static jvalue run_64_method(w_thread thread, x_frame frame, w_method method) {

  jvalue result;

  woempa(1, "running 64_method %M for thread '%w', TOS @ %p\n", frame->method, NM(thread), frame->top);
  callMethod(frame, method);
  if (! exceptionThrown(thread)) {
    result.w[WORD_MSW] = frame->top[-1].c.w;
    result.w[WORD_LSW] = frame->top[-2].c.w;
  }
  woempa(1, "finished running 64_method %M for thread '%w', TOS @ %p\n", frame->method, NM(thread), frame->top);

  deactivateFrame(frame, NULL);

  return result;

}

/*
** Get the current group for memory allocation. When we are not running
** from within a JAVA thread, the thread instance will be NULL and
** we assume that we are running from within some kind of system thread.
*/

w_group JNI_Group(JNIEnv *env) {
  w_instance Thread = JNIEnv2w_thread(env)->Thread;
  
  if (Thread) {
    return ((w_thread)getWotsitField(Thread, F_Thread_wotsit))->group;
  }
  else {
    return W_ThreadGroup_system;
  }
}

jint GetVersion(JNIEnv *env) {
  return 0x00010002;
}

jclass DefineClass(JNIEnv *env, const char *name, jobject loader, const jbyte *buf, jsize buflen) {

  w_thread  thread = JNIEnv2w_thread(env);
  w_clazz   clazz;
  w_instance theClass;
  w_BAR bar;

  woempa(1,"installing %s from %p using %j (thread %w)\n",name,buf,loader,NM(thread));

  bar.buffer = (w_byte*)buf;
  bar.length = buflen;
  bar.current = 0;

  enterMonitor(loader);

// [CG 20010322] not passing the name given, as I can't find a consistent
// description of what format it used (slashes or dots, etc.)
  clazz = createClazz(thread, NULL, &bar, loader);

  exitMonitor(loader);

  if (exceptionThrown(thread)) {
    return NULL;
}

  theClass = clazz2Class(clazz);
  setReferenceField(theClass, loader, F_Class_loader);

  return theClass;

}

void ExceptionClear(JNIEnv* env) {

  w_thread thread = JNIEnv2w_thread(env);

  thread->exception = NULL;

  if (thread->Thread) {
    setReferenceField(thread->Thread, NULL, F_Thread_thrown);
  }

}

void ExceptionDescribe(JNIEnv* env) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance throwable;
  w_instance Message;

  if (thread->Thread && (throwable = getReferenceField(thread->Thread, F_Thread_thrown))) {
    Message = getReferenceField(throwable, F_Throwable_detailMessage);
    woempa(9, "Thread '%w' has thrown an instance of '%k'.\n", NM(thread), instance2clazz(throwable));
    if (Message) {
       woempa(9, "Throwable message is '%w'.\n", getWotsitField(Message, F_String_wotsit));
    }
    // Do we need to clear the exception ?
    setReferenceField(thread->Thread, NULL, F_Thread_thrown);
    thread->exception = NULL;
  }

}

jboolean ExceptionCheck(JNIEnv *env) {
  w_thread thread = JNIEnv2w_thread(env);

  return (thread->Thread && getReferenceField(thread->Thread, F_Thread_thrown));
  
}

jthrowable ExceptionOccurred(JNIEnv *env) {

  return JNIEnv2w_thread(env)->exception;
  
}

void FatalError(JNIEnv *env, const char *message) {
  wabort(ABORT_WONKA, "FatalError: %s\n",message);
}

    
jclass FindClass(JNIEnv *env, const char *Name) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string name = cstring2String(thread, Name, strlen(Name));
  w_method caller = thread->top->method;
  w_string dotified;
  w_instance loader;
  w_clazz clazz;
  jclass result;

  if (caller) {
    woempa(1, "(JNI) Searching for class '%s', called from %M\n", Name, caller);
    loader = clazz2loader(caller->spec.declaring_clazz);
    woempa(1, "(JNI) Using class loader %p of %k.\n", loader, instance2clazz(loader));
  }
  else {
    if(clazzApplicationClassLoader && clazzApplicationClassLoader->staticFields) {
      loader = clazzApplicationClassLoader->staticFields[F_ApplicationClassLoader_theApplicationClassLoader];
    }
    else {
      loader = NULL;
    }

    woempa(1, "(JNI) Searching for class '%s', thread %t has no stack frame so using bootstrap class loader\n", Name, thread);
  }

  dotified = undescriptifyClassName(name);
  clazz = namedClassMustBeLoaded(loader, dotified);
  deregisterString(dotified);

  if (! exceptionThrown(thread) && clazz == NULL) {
    throwException(thread, clazzClassNotFoundException, "(JNI) FindClass: %w", name);
  }

  if (exceptionThrown(thread)) {
    woempa(1, "(JNI) exception %j\n", exceptionThrown(thread));
    return NULL;
  }

  mustBeLinked(clazz);

  if (exceptionThrown(thread)) {
    result = NULL;
  }
  else {
    result = clazz2Class(clazz);
  }
  
  woempa(1, "(JNI) result %j\n", result);
  
  deregisterString(name);
  
  return result;

}

jclass GetSuperclass(JNIEnv *env, jclass class) {
  return clazz2Class(getSuper(Class2clazz(class)));
}

jboolean IsAssignableFrom(JNIEnv *env, jclass class1, jclass class2) {
  return isAssignmentCompatible(Class2clazz(class1), Class2clazz(class2));
}

jint Throw(JNIEnv *env, jthrowable Throwable) {

  throwExceptionInstance(JNIEnv2w_thread(env), Throwable);
  
  return 0;

}

jint ThrowNew(JNIEnv *env, jclass class, const char *message) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(class);
  w_string string;
  w_instance Throwable = allocInstance(thread, clazz);
  w_instance Message = NULL;

  if(message) {
    string = cstring2String(thread, message, strlen(message));
    Message = newStringInstance(string);
  }
  
  if (!Throwable) {
    wabort(ABORT_WONKA, "Unable to create Throwable\n");
  }

  setReferenceField(Throwable, Message, F_Throwable_detailMessage);
  throwExceptionInstance(thread, Throwable);
  
  return 0;

}


jobject NewGlobalRef(JNIEnv *env, jobject obj) {
  newGlobalReference(obj);
  return obj;
}

jobject NewLocalRef(JNIEnv *env, jobject obj) {

  woempa(1, "creating new local reference to instance %p of %k\n", obj, instance2clazz((w_instance)obj));
  addLocalReference(JNIEnv2w_thread(env), obj);

  return obj;

}

void DeleteGlobalRef(JNIEnv *env, jobject obj) {

  /*
  ** Decrement the reference count, if the new reference count returned is 0, we erase
  ** the entry from the hashtable.
  */

  woempa(1, "deleting global reference to instance %p\n", obj);
  ht_deregister(globals_hashtable,(w_word)obj);

}

void DeleteLocalRef(JNIEnv *env, jobject instance) {

  woempa(1, "deleting local reference to instance %p of %k\n", instance, instance2clazz((w_instance)instance));
  removeLocalReference(JNIEnv2w_thread(env), instance);

}

jboolean IsSameObject(JNIEnv *env, jobject ref1, jobject ref2) {
  return ref1 == ref2;
}

jobject AllocObject(JNIEnv *env, jclass class) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new = NULL;
  
  classMustBeInitialized(clazz);
  if (! exceptionThrown(thread)) {
    new = allocInstance(thread, clazz);
    if(!new) {
      throwOutOfMemoryError(thread);
    }
  }

  return new;
  
}

jobject NewObject(JNIEnv *env, jclass class, jmethodID methodID, ...) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  x_frame frame;
  va_list list;

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return NULL;
  }
  
  new = allocInstance(thread, clazz);

  if (new) {
    frame = pushFrame(thread, methodID);
    frame->top[0].c.w = (w_word) new;
    frame->top[0].s = stack_trace;
    frame->top += 1;
    va_start(list, methodID);
    va_list2items(methodID, list, frame);
    va_end(list);
    if (exceptionThrown(thread)) {
      new = NULL;
    }
    else {
      callMethod(frame, methodID);
    }
    deactivateFrame(frame, NULL);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;

}

jobject NewObjectV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  x_frame frame;

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return NULL;
  }
  new = allocInstance(thread, clazz);

  if (new) {
    frame = pushFrame(thread, methodID);
    frame->top[0].c.w = (w_word) new;
    frame->top[0].s = stack_trace;
    frame->top += 1;
    va_list2items(methodID, args, frame);
    if (exceptionThrown(thread)) {
      new = NULL;
    }
    else {
      callMethod(frame, methodID);
    }
    deactivateFrame(frame, NULL);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;

}

jobject NewObjectA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance new;
  x_frame frame;

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return NULL;
  }

  new = allocInstance(thread, clazz);

  if (new) {
    frame = pushFrame(thread, methodID);
    frame->top[0].c.w = (w_word) new;
    frame->top[0].s = stack_trace;
    frame->top += 1;
    jvalues2items(methodID, args, frame);
    if (exceptionThrown(thread)) {
      new = NULL;
    }
    else {
      callMethod(frame, methodID);
    }
    deactivateFrame(frame, NULL);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return new;

}

jboolean IsInstanceOf(JNIEnv *env, jobject obj, jclass class) {
///By definition, a null object is instance of every class, and therefore always should return true
  if(!obj) {
    return JNI_TRUE;
  }
  else {
    return isAssignmentCompatible(instance2clazz(obj), Class2clazz(class));
  }
}

jclass GetObjectClass(JNIEnv *env, jobject obj) {

  return clazz2Class(instance2clazz(obj));

}

/*
** Search for a method with a certain template in a certain clazz, based on a clazz.
** The template is in fact a method but we have to resolve the correct method ourselves.
** It may well be the method that was passed as a template.
*/

w_method resolveMethodForCall(w_clazz clazz, w_method template) {
  return isSet(template->spec.declaring_clazz->flags, ACC_INTERFACE) ? interfaceLookup(template, clazz) : virtualLookup(template, clazz);
}

/*
** => Calling java class functions froom within JNI c-code
*/

jvalue CallMethod32(JNIEnv *env, jobject instance, w_method method, va_list args) {

  w_thread thread = JNIEnv2w_thread(env);
  x_frame frame;

  frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  va_list2items(method, args, frame);

  return run_32_method(thread, frame, method);

}

jvalue CallMethod32A(JNIEnv *env, jobject instance, w_method method, jvalue *args) {

  w_thread thread = JNIEnv2w_thread(env);

  x_frame frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  jvalues2items(method, args, frame);
  
  return run_32_method(thread, frame, method);

}

jvalue CallMethod64(JNIEnv *env, jobject instance, w_method method, va_list args) {

  w_thread thread = JNIEnv2w_thread(env);

  x_frame frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  va_list2items(method, args, frame);
  
  return run_64_method(thread, frame, method);

}

jvalue CallMethod64A(JNIEnv *env, jobject instance, w_method method, jvalue *args) {

  w_thread thread = JNIEnv2w_thread(env);
  x_frame frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  jvalues2items(method, args, frame);
  
  return run_64_method(thread, frame, method);

}

w_void CallMethodVoid(JNIEnv *env, jobject instance, w_method method, va_list args) {

  w_thread thread = JNIEnv2w_thread(env);
  x_frame frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  va_list2items(method, args, frame);
  if (!exceptionThrown(thread)) {
    callMethod(frame, method);
  }
  deactivateFrame(frame, NULL);
  
}

w_void CallMethodVoidA(JNIEnv *env, jobject instance, w_method method, jvalue *args) {

  w_thread thread = JNIEnv2w_thread(env);
  x_frame frame = pushFrame(thread, method);

  frame->top[0].c.w = (w_word) instance;
  frame->top[0].s = stack_trace;
  frame->top += 1;

  jvalues2items(method, args, frame);
  if (!exceptionThrown(thread)) {
    callMethod(frame, method);
  }
  deactivateFrame(frame, NULL);

}

/*
** => object function calls
*/

jobject CallObjectMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jobject  result = NULL;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).l;
  va_end(list);
  
  return result;

}

jobject CallObjectMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).l;

}

jobject CallObjectMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).l;

}

/*
** => boolean function calls
*/

jboolean CallBooleanMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jboolean result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).z;
  va_end(list);
  
  return result;

}

jboolean CallBooleanMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).z; 

}

jboolean CallBooleanMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).z; 

}

/*
** => byte function calls
*/

jbyte CallByteMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jbyte    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).b;
  va_end(list);
  
  return result;

}

jbyte CallByteMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).b; 

}

jbyte CallByteMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).b; 

}

/*
** => char function calls
*/

jchar CallCharMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jchar    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).c;
  va_end(list);
  
  return result;

}

jchar CallCharMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).c;

}

jchar CallCharMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).c;

}

/*
** => short function calls
*/

jshort CallShortMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jshort   result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).s;
  va_end(list);
  
  return result;

}

jshort CallShortMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).s; 

}

jshort CallShortMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).s; 

}

/*
** => int function calls
*/

jint CallIntMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jint     result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).i;
  va_end(list);
  
  return result;

}

jint CallIntMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).i; 

}

jint CallIntMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).i; 

}

/*
** => long function calls
*/

jlong CallLongMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jlong    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).j;
  va_end(list);
  
  return result;

}

jlong CallLongMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod64(env, obj, method, args)).j; 

}

jlong CallLongMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod64A(env, obj, method, args)).j; 

}

/*
** => float function calls
*/

jfloat CallFloatMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jfloat   result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).f;
  va_end(list);
  
  return result;

}

jfloat CallFloatMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32(env, obj, method, args)).f; 

}

jfloat CallFloatMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  return (CallMethod32A(env, obj, method, args)).f; 

}

/*
** => double function calls
*/

jdouble CallDoubleMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  jdouble    result = 0;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).d;

  printf("jni: %g\n", result);
  va_end(list);
  
  return result;

}

jdouble CallDoubleMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  jdouble result = (CallMethod64(env, obj, method, args)).d; 
  printf("jni: %g\n", result);
  return result;

}

jdouble CallDoubleMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  jdouble result = (CallMethod64A(env, obj, method, args)).d; 
  printf("jni: %g\n", result);
  return result;

}

/*
** => void function calls
*/

w_void CallVoidMethod(JNIEnv *env, jobject obj, jmethodID methodID, ...) {

  va_list  list;
  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);
  
  va_start(list, methodID);
  CallMethodVoid(env, obj, method, list);
  va_end(list);

}

w_void CallVoidMethodV(JNIEnv *env, jobject obj, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  CallMethodVoid(env, obj, method, args); 

}

w_void CallVoidMethodA(JNIEnv *env, jobject obj, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(instance2clazz((w_instance)obj), methodID);

  CallMethodVoidA(env, obj, method, args); 

}

/*
** =>  nonvirtual object function calls
*/

jobject CallNonvirtualObjectMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jobject  result = NULL;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).l;
  va_end(list);
  
  return result;

}

jobject CallNonvirtualObjectMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).l; 

}

jobject CallNonvirtualObjectMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).l;

}

/*
** =>  nonvirtual boolean function calls
*/

jboolean CallNonvirtualBooleanMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jboolean result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).z;
  va_end(list);
  
  return result;

}

jboolean CallNonvirtualBooleanMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).z; 

}

jboolean CallNonvirtualBooleanMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).z; 

}

/*
** =>  nonvirtual byte function calls
*/

jbyte CallNonvirtualByteMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jbyte    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).b;
  va_end(list);
  
  return result;

}

jbyte CallNonvirtualByteMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).b; 

}

jbyte CallNonvirtualByteMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).b; 

}

/*
** =>  nonvirtual char function calls
*/

jchar CallNonvirtualCharMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jchar    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).c;
  va_end(list);
  
  return result;

}

jchar CallNonvirtualCharMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).c;

}

jchar CallNonvirtualCharMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).c;

}

/*
** =>  nonvirtual short function calls
*/

jshort CallNonvirtualShortMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jshort   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).s;
  va_end(list);
  
  return result;

}

jshort CallNonvirtualShortMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).s; 

}

jshort CallNonvirtualShortMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).s; 

}

/*
** =>  nonvirtual int function calls
*/

jint CallNonvirtualIntMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jint    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).i;
  va_end(list);
  
  return result;

}

jint CallNonvirtualIntMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).i; 

}

jint CallNonvirtualIntMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).i; 

}

/*
** =>  nonvirtual long function calls
*/

jlong CallNonvirtualLongMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).j;
  va_end(list);
  
  return result;

}

jlong CallNonvirtualLongMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod64(env, obj, method, args)).j; 

}

jlong CallNonvirtualLongMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod64A(env, obj, method, args)).j; 

}

/*
** =>  nonvirtual float function calls
*/

jfloat CallNonvirtualFloatMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jfloat   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, obj, method, list)).f;
  va_end(list);
  
  return result;

}

jfloat CallNonvirtualFloatMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32(env, obj, method, args)).f; 

}

jfloat CallNonvirtualFloatMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod32A(env, obj, method, args)).f; 

}

/*
** =>  nonvirtual double function calls
*/

jdouble CallNonvirtualDoubleMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, obj, method, list)).d;
  va_end(list);
  
  return result;

}

jdouble CallNonvirtualDoubleMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod64(env, obj, method, args)).d; 

}

jdouble CallNonvirtualDoubleMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  return (CallMethod64A(env, obj, method, args)).d; 

}

/*
** =>  nonvirtual void function calls
*/

w_void CallNonvirtualVoidMethod(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, ...) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);
  va_list  list;
  
  va_start(list, methodID);
  CallMethodVoid(env, obj, method, list);
  va_end(list);

}

w_void CallNonvirtualVoidMethodV(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, va_list args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  CallMethodVoid(env, obj, method, args); 

}

w_void CallNonvirtualVoidMethodA(JNIEnv *env, jobject obj, jclass class, jmethodID methodID, jvalue *args) {

  w_method method = resolveMethodForCall(Class2clazz(class), methodID);

  CallMethodVoidA(env, obj, method, args); 

}

/*
** =>  member variable access
*/

/*
** =>  retrieve the member variable field ID
*/

jfieldID GetFieldID(JNIEnv *env, jclass class, const char *name, const char *sig) {

  w_string name_string = cstring2String(JNIEnv2w_thread(env), (char *)name, strlen(name));
  w_string desc_string = cstring2String(JNIEnv2w_thread(env), (char *)sig, strlen(sig));
  w_field result;

  woempa(1, "(JNI) name='%s', signature='%s'\n",name,sig);
    
  /*
  ** Search for it, match only instance fields, but match any of the 
  ** "public" or "private" field modifiers.
  */
  
  result = searchClazzHierarchyForField(Class2clazz(class), name_string, desc_string, MATCH_INSTANCE_FIELD, MATCH_ANY);

  deregisterString(desc_string);
  deregisterString(name_string);
  
  return result;
}

/*
** =>  Get the value of a variable, given the variables field ID
*/

JNITypes GetGeneric32Field(JNIEnv *env, jobject obj, jfieldID fieldID) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    JNITypes j;
    return j;
  }
  return (JNITypes)((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot)];
}

JNITypes GetGeneric64Field(JNIEnv *env, jobject obj, jfieldID fieldID) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  w_long result;
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    JNITypes j;

    return j;

  }
  result =  ((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot) + 1];  /* MSW */
  result <<= 32;
  result |= ((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot)];  /* LSW */
  return (JNITypes)result;
}

jobject GetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  w_instance result;
  
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return NULL;
  }
  
  result = getReferenceField((w_instance)obj, fieldID->size_and_slot);

  addLocalReference(thread, result);

  return (jobject)result;
}

jboolean GetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).z;
}

jbyte GetByteField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).b;
}

jchar GetCharField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).c;
}

jshort GetShortField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).s;
}

jint GetIntField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).i;
}

jlong GetLongField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric64Field(env, obj, fieldID)).j;
}

jfloat GetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric32Field(env, obj, fieldID)).f;
}

jdouble GetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID) {
  return (GetGeneric64Field(env, obj, fieldID)).d;
}

/*
** =>  set the a variable to a certain value, given the variables field ID
*/

w_void SetGeneric32Field(JNIEnv *env, jobject obj, jfieldID fieldID, w_word value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return;

  }
  woempa(1,"set field[%d] of instance %p of %k to 0x%08x\n",FIELD_OFFSET(fieldID->size_and_slot),(w_instance)obj,instance2clazz((w_instance)obj),value);
  ((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot)] = value;
}

w_void SetGeneric64Field(JNIEnv *env, jobject obj, jfieldID fieldID, jlong value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = instance2clazz((w_instance)obj);
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return;

  }
  
  woempa(1,"set field[%d-%d] of instance %p of %k to 0x%08x%08x\n", FIELD_OFFSET(fieldID->size_and_slot), FIELD_OFFSET(fieldID->size_and_slot) + 1,
          (w_instance)obj,instance2clazz((w_instance)obj),LSW_PART(value),MSW_PART(value));

  ((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot)] = LSW_PART(value);
  ((w_instance)obj)[FIELD_OFFSET(fieldID->size_and_slot) + 1] = MSW_PART(value);
}

w_void SetObjectField(JNIEnv *env, jobject obj, jfieldID fieldID, jobject value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance instance = obj;
  w_instance wval = value;
  w_clazz clazz = instance2clazz(instance);

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return;

  }
  woempa(1,"set field[%d] of instance %p of %k to %p\n",fieldID->size_and_slot,instance,instance2clazz(instance),wval);

  setReferenceField(instance, wval, fieldID->size_and_slot);

}

w_void SetBooleanField(JNIEnv *env, jobject obj, jfieldID fieldID, jboolean value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetByteField(JNIEnv *env, jobject obj, jfieldID fieldID, jbyte value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetCharField(JNIEnv *env, jobject obj, jfieldID fieldID, jchar value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetShortField(JNIEnv *env, jobject obj, jfieldID fieldID, jshort value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetIntField(JNIEnv *env, jobject obj, jfieldID fieldID, jint value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetLongField(JNIEnv *env, jobject obj, jfieldID fieldID, jlong value) {
  SetGeneric64Field(env, obj, fieldID, (jlong)value);
}

w_void SetFloatField(JNIEnv *env, jobject obj, jfieldID fieldID, jfloat value) {
  SetGeneric32Field(env, obj, fieldID, (w_word)value);
}

w_void SetDoubleField(JNIEnv *env, jobject obj, jfieldID fieldID, jdouble value) {
  SetGeneric64Field(env, obj, fieldID, (jlong)value);
}

/*
** =>  static  function calls
*/

/*
** =>  static function method ID retrieval
*/

jmethodID GetStaticMethodID(JNIEnv *env, jclass class, const char *utf8name, const char *utf8sig) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(class);
  w_clazz  super;
  w_size   i;
  w_size   j;
  w_method method = NULL;
  w_method candidate;
  w_string name_string;
  w_string desc_string;
  w_MethodSpec *spec;

  name_string = utf2String(W_Thread_system, utf8name, strlen(utf8name));
  desc_string = utf2String(W_Thread_system, utf8sig, strlen(utf8sig));
  if (createMethodSpecUsingDescriptor(clazz, name_string, desc_string, &spec) == CLASS_LOADING_FAILED) {
    wabort(ABORT_WONKA,"Uh oh: failed to build method spec using declaring_clazz %k, name %w, desc %w.\n",clazz, name_string, desc_string);
  }

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return NULL;

  }

  woempa(1,"Seeking  method %w%w in %K\n", name_string, desc_string, spec->declaring_clazz);
 
  for (i = 0; i < clazz->numDeclaredMethods; ++i) {
    candidate = &clazz->own_methods[i];

    woempa(1, "  candidate: %w%w\n", candidate->spec.name, candidate->spec.declaring_clazz->values[candidate->desc_index]);
    if ((candidate->spec.name == name_string) && ((w_string)candidate->spec.declaring_clazz->values[candidate->desc_index] == desc_string)) {
      method = candidate;
      break;
    }
  }

  if (!method) {
    for (j = 0; j < clazz->numSuperClasses; ++j) {
      super = clazz->supers[j];
      woempa(1, "Seek %w in %K\n", spec->name, super);
      for (i = 0; i < super->numDeclaredMethods; ++i) {
        candidate = &super->own_methods[i];

        if (candidate->spec.name == name_string && ((w_string)candidate->spec.declaring_clazz->values[candidate->desc_index] == desc_string)) {
          method = candidate;
          break;
        }
      }
    }
  }

  if (method) {
    if (!methodMatchesSpec(method, spec)) {
      woempa(9,"%M matches name %w and descriptor %w, but classes do not resolve correctly.\n", method, name_string, desc_string);
      method = NULL;
    }
    else if (isNotSet(method->flags, ACC_STATIC)) {
      woempa(9,"Was looking for a static method %w%w, but %M is not static\n", name_string, desc_string, method);
      method = NULL;
    }
    else {
      woempa(1,"Found  method %M\n", method);
    }
  }

  releaseMethodSpec(spec);
  deregisterString(name_string);
  deregisterString(desc_string);

  return method;
}

jmethodID GetMethodID(JNIEnv *env, jclass class, const char *utf8name, const char *utf8sig) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz = Class2clazz(class);
  w_clazz  super;
  w_size   i;
  w_size   j;
  w_method method = NULL;
  w_method candidate;
  w_string name_string;
  w_string desc_string;
  w_MethodSpec *spec;

  name_string = utf2String(W_Thread_system, utf8name, strlen(utf8name));
  desc_string = utf2String(W_Thread_system, utf8sig, strlen(utf8sig));
  if (createMethodSpecUsingDescriptor(clazz, name_string, desc_string, &spec) == CLASS_LOADING_FAILED) {
    wabort(ABORT_WONKA,"Uh oh: failed to build method spec using declaring_clazz %k, name %w, desc %w.\n",clazz, name_string, desc_string);
  }

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return NULL;

  }

  woempa(1,"Seeking  method %w%w in %K\n", name_string, desc_string, spec->declaring_clazz);
 
  for (i = 0; i < clazz->numDeclaredMethods; ++i) {
    candidate = &clazz->own_methods[i];

    if ((candidate->spec.name == name_string) && ((w_string)candidate->spec.declaring_clazz->values[candidate->desc_index] == desc_string)) {
      method = candidate;
      break;
    }
  }

  if (!method) {
    for (j = 0; j < clazz->numSuperClasses; ++j) {
      super = clazz->supers[j];
      woempa(1, "Seek %w in %K\n", spec->name, super);
      for (i = 0; i < super->numDeclaredMethods; ++i) {
        candidate = &super->own_methods[i];

        if (candidate->spec.name == name_string && ((w_string)candidate->spec.declaring_clazz->values[candidate->desc_index] == desc_string)) {
          method = candidate;
          break;
        }
      }
    }
  }

  if (!method) {
    for (j = 0; j < clazz->numInterfaces; ++j) {
      super = clazz->interfaces[j];
      woempa(1, "Seek %w in %K\n", spec->name, super);
      for (i = 0; i < super->numDeclaredMethods; ++i) {
        candidate = &super->own_methods[i];

        if (candidate->spec.name == name_string && ((w_string)candidate->spec.declaring_clazz->values[candidate->desc_index] == desc_string)) {
          method = candidate;
          break;
        }
      }
    }
  }

  if (method) {
    if (!methodMatchesSpec(method, spec)) {
      woempa(9,"%M matches name %w and descriptor %w, but classes do not resolve correctly.\n", method, name_string, desc_string);
      method = NULL;
    }
    else if (isSet(method->flags, ACC_STATIC)) {
      woempa(9,"Was looking for an instance method %w%w, but %M is static\n", name_string, desc_string, method);
      method = NULL;
    }
    else {
      woempa(1,"Found  method %M\n", method);
    }
  }

  releaseMethodSpec(spec);
  deregisterString(name_string);
  deregisterString(desc_string);

  return method;
}

/*
** =>  static object function calls
*/

jobject CallStaticObjectMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jobject  result = NULL;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).l;
  va_end(list);
  
  return result;
}

jobject CallStaticObjectMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).l; 
}

jobject CallStaticObjectMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).l; 
}

/*
** =>  static boolean function calls
*/

jboolean CallStaticBooleanMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jboolean result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).z;
  va_end(list);
  
  return result;
}

jboolean CallStaticBooleanMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).z; 
}

jboolean CallStaticBooleanMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).z; 
}

/*
** =>  static byte function calls
*/

jbyte CallStaticByteMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jbyte    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).b;
  va_end(list);
  
  return result;
}

jbyte CallStaticByteMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).b; 
}

jbyte CallStaticByteMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).b; 
}

/*
** =>  static char function calls
*/

jchar CallStaticCharMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jchar    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).c;
  va_end(list);
  
  return result;
}

jchar CallStaticCharMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).c;
}

jchar CallStaticCharMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).c;
}

/*
** =>  static short function calls
*/

jshort CallStaticShortMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jshort   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).s;
  va_end(list);
  
  return result;
}

jshort CallStaticShortMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).s; 
}

jshort CallStaticShortMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).s; 
}

/*
** =>  static int function calls
*/

jint CallStaticIntMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jint     result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).i;
  va_end(list);
  
  return result;
}

jint CallStaticIntMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).i; 
}

jint CallStaticIntMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).i; 
}

/*
** =>  static long function calls
*/

jlong CallStaticLongMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jlong    result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, NULL, methodID, list)).j;
  va_end(list);
  
  return result;
}

jlong CallStaticLongMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod64(env, NULL, methodID, args)).j; 
}

jlong CallStaticLongMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod64A(env, NULL, methodID, args)).j; 
}

/*
** =>  static float function calls
*/

jfloat CallStaticFloatMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jfloat   result = 0;
  
  va_start(list, methodID);
  result = (CallMethod32(env, NULL, methodID, list)).f;
  va_end(list);
  
  return result;
}

jfloat CallStaticFloatMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod32(env, NULL, methodID, args)).f; 
}

jfloat CallStaticFloatMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod32A(env, NULL, methodID, args)).f; 
}

/*
** =>  static double function calls
*/

jdouble CallStaticDoubleMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  jdouble  result = 0;
  
  va_start(list, methodID);
  result = (CallMethod64(env, NULL, methodID, list)).d;
  va_end(list);
 
  return result;
}

jdouble CallStaticDoubleMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  return (CallMethod64(env, NULL, methodID, args)).d; 
}

jdouble CallStaticDoubleMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {
  return (CallMethod64A(env, NULL, methodID, args)).d; 
}

/*
** =>  static void function calls
*/
   
w_void CallStaticVoidMethod(JNIEnv *env, jclass class, jmethodID methodID, ...) {
  va_list  list;
  
  va_start(list, methodID);
  CallMethodVoid(env, NULL, methodID, list);
  va_end(list);
}

w_void CallStaticVoidMethodV(JNIEnv *env, jclass class, jmethodID methodID, va_list args) {
  CallMethodVoid(env, NULL, methodID, args); 
}

w_void CallStaticVoidMethodA(JNIEnv *env, jclass class, jmethodID methodID, jvalue *args) {     
  CallMethodVoidA(env, NULL, methodID, args); 
}

/*
** static member variable field access:
** get the member variable field ID
*/

jfieldID GetStaticFieldID(JNIEnv *env, jclass class, const char *name, const char *sig) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_string name_string = cstring2String(thread, (char *)name, strlen(name));
  w_string desc_string = cstring2String(thread, (char *)sig, strlen(sig));
  w_field result;
  
  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return NULL;

  }
  
  woempa(1, "(JNI) Returning field ID for field '%s', descriptor '%s'.\n", name, sig);
  
  result = searchClazzHierarchyForField(Class2clazz(class), name_string, desc_string, MATCH_STATIC_FIELD, 0);

  deregisterString(desc_string);
  deregisterString(name_string);
  
  return result;

}

/*
** get the static member variable value from field ID
*/

JNITypes GetStatic32Field(JNIEnv *env, jclass class, jfieldID fieldID) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    JNITypes j;

    return j;

  }

  woempa(1, "(JNI) Getting field '%w' at slot %d from class %k.\n", NM(fieldID), fieldID->size_and_slot, clazz);

  return (JNITypes)clazz->staticFields[fieldID->size_and_slot];

}

JNITypes GetStatic64Field(JNIEnv *env, jclass class, jfieldID fieldID) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = Class2clazz(class);
  w_long result;

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    JNITypes j;

    return j;

  }
  woempa(1, "(JNI) Getting field '%w' at slot %d from class %k.\n", NM(fieldID), fieldID->size_and_slot, clazz);
  
  result = clazz->staticFields[fieldID->size_and_slot + 1];
  result <<= 32;
  result |= clazz->staticFields[fieldID->size_and_slot];
  
  return (JNITypes)result;

}

jobject GetStaticObjectField(JNIEnv *env, jclass class, jfieldID fieldID) {
  w_thread thread = JNIEnv2w_thread(env);
  w_clazz  clazz  = Class2clazz(class);
  w_instance result;

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {
    return NULL;
  }

  woempa(1, "(JNI) Getting field '%w' at slot %d from class %k.\n", NM(fieldID), fieldID->size_and_slot, clazz);

  result = clazz->staticFields[fieldID->size_and_slot];

  addLocalReference(thread, result);

  return (jobject)result;
}

jboolean GetStaticBooleanField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).z;
}

jbyte GetStaticByteField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).b;
}

jchar GetStaticCharField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).c;
}

jshort GetStaticShortField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).s;
}

jint GetStaticIntField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).i;
}

jlong GetStaticLongField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic64Field(env, class, fieldID)).j;
}

jfloat GetStaticFloatField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic32Field(env, class, fieldID)).f;
}

jdouble GetStaticDoubleField(JNIEnv *env, jclass class, jfieldID fieldID) {
  return (GetStatic64Field(env, class, fieldID)).d;
}

/*
** set the static member variable in a field ID to given value
*/

w_void SetStatic32Field(JNIEnv *env, jclass class, jfieldID fieldID, w_word value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = Class2clazz(class);

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return;

  }
  clazz->staticFields[fieldID->size_and_slot] = value;

}

w_void SetStatic64Field(JNIEnv *env, jclass class, jfieldID fieldID, jlong value) {

  w_thread thread = JNIEnv2w_thread(env);
  w_clazz clazz = Class2clazz(class);

  classMustBeInitialized(clazz);
  if (exceptionThrown(thread)) {

    return;

  }
  clazz->staticFields[fieldID->size_and_slot   ] = (w_word)LSW_PART(value);
  clazz->staticFields[fieldID->size_and_slot +1] = (w_word)MSW_PART(value);

}

w_void SetStaticObjectField(JNIEnv *env, jclass class, jfieldID fieldID, jobject value) {
  setStaticReferenceField(fieldID->declaring_clazz, fieldID->size_and_slot, value);
}

w_void SetStaticBooleanField(JNIEnv *env, jclass class, jfieldID fieldID, jboolean value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticByteField(JNIEnv *env, jclass class, jfieldID fieldID, jbyte value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticCharField(JNIEnv *env, jclass class, jfieldID fieldID, jchar value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticShortField(JNIEnv *env, jclass class, jfieldID fieldID, jshort value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticIntField(JNIEnv *env, jclass class, jfieldID fieldID, jint value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticLongField(JNIEnv *env, jclass class, jfieldID fieldID, jlong value) {
  SetStatic64Field(env, class, fieldID, (jlong)value);
}

w_void SetStaticFloatField(JNIEnv *env, jclass class, jfieldID fieldID, jfloat value) {
  SetStatic32Field(env, class, fieldID, (w_word)value);
}

w_void SetStaticDoubleField(JNIEnv *env, jclass class, jfieldID fieldID, jdouble value) {
  SetStatic64Field(env, class, fieldID, (jlong)value);
}


jsize GetStringLength(JNIEnv *env, jstring String) {

  w_string string = String2string(String);

  woempa(1, "(JNI) Length of '%w' is %d JAVA chars.\n", string, String, string_length(string));
  
  return string_length(string);
  
}

const jchar *GetStringChars(JNIEnv *env, jstring String, jboolean *isCopy) {

  w_string string = String2string(String);
  w_char  *chars = NULL;

  if (string_length(string)) {
    chars  = allocMem(JNIEnv2w_thread(env), string_length(string) * sizeof(w_char));
    w_string2chars(string, chars);
  }
  woempa(1,"(JNI) created array of %d chars at %p\n",string_length(string),chars);

  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return chars;
  
}

void ReleaseStringChars(JNIEnv *env, jstring string, const jchar *chars) {

  if (chars) {
    woempa(1, "(JNI) release array of chars at %p\n", chars);
    releaseMem((void *)chars);
  }

}
          
jstring NewString(JNIEnv *env, const jchar * uchars, jsize len) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string string = unicode2String(thread, (w_char*) uchars, (w_size) len);
  w_instance String = newStringInstance(string);

  deregisterString(string);
  
  return String;

}

jstring NewStringUTF(JNIEnv *env, const char * bytes) {

  w_thread thread = JNIEnv2w_thread(env);
  w_string string = utf2String(thread, (char *) bytes, strlen(bytes));
  w_instance String = newStringInstance(string);

  deregisterString(string);
    
  return String;
  
}

jsize GetStringUTFLength(JNIEnv *env, jstring String) {
  w_string string = String2string(String);
  w_size   i;
  jsize    result = 0;
 
/*
  if (string_is_latin1(string)) {

    return string_length(string);

  }
*/

  for (i=0; i<string_length(string); ++i) {
    w_char ch = string_char(string, i);
    if (ch == 0) {
      result += 2;
    }
    else if (ch < 0x80 ) {
      result += 1;
    }
    else if (ch < 0x800 ) {
      result += 2;
    }
    else {
      result += 3;
    }
  }

  return result;
}

const jbyte * GetStringUTFChars(JNIEnv *env, jstring String, jboolean *isCopy) {

  w_string string = String2string(String);
  w_size   max_length = string_length(string) * 3;
  jbyte *  result = allocMem(JNIEnv2w_thread(env), (max_length + 1) * sizeof(jbyte));
  jbyte *  utf8ptr = result;
  w_size   i;
  
  for (i = 0; i < string_length(string); i++) {
    w_char ch = string_char(string, i);
    if (ch == 0) {
      *utf8ptr++ = 0xc0;
      *utf8ptr++ = 0x80;
    }
    else if (ch < 0x80 ) {
      *utf8ptr++ = string_char(string, i);
    }
    else if (ch < 0x800 ) {
      *utf8ptr++ = 0xc0 + (string_char(string, i) >> 6);
      *utf8ptr++ = 0x80 + (string_char(string, i) & 0x3f);
    }
    else {
      *utf8ptr++ = 0xe0 + (string_char(string, i) >> 12);
      *utf8ptr++ = 0x80 + ((string_char(string, i) >> 6) & 0x3f);
      *utf8ptr++ = 0x80 + (string_char(string, i) & 0x3f);
    }
  }
  *utf8ptr = '\0'; /* Terminating string */

  if (isCopy) {
    *isCopy = JNI_TRUE;
  }

  return result;

}

void GetStringUTFRegion(JNIEnv *env, jstring String, jsize start, jsize len, char * buf) {

  w_string string = String2string(String);
  jbyte *  utf8ptr = buf;
  jsize    i;
  w_thread thread = JNIEnv2w_thread(env);
  
  if (start > (jsize) string_length(string)) {
    throwException(thread, clazzStringIndexOutOfBoundsException, "start %d > String %d chars.\n", start, string_length(string));
    return;
  }
  
  if (start + len > (jsize) string_length(string)) {
    throwException(thread, clazzStringIndexOutOfBoundsException, "start %d + len %d > String %d chars.\n", start, len, string_length(string));
    return;
  }

  for (i = 0; i < len; i++) {
    w_char ch = string_char(string, start + i);
    if (ch == 0) {
      *utf8ptr++ = 0xc0;
      *utf8ptr++ = 0x80;
    }
    else if (ch < 0x80 ) {
      *utf8ptr++ = ch;
    }
    else if (ch < 0x800 ) {
      *utf8ptr++ = 0xc0 + (ch >> 6);
      *utf8ptr++ = 0x80 + (ch & 0x3f);
    }
    else {
      *utf8ptr++ = 0xe0 + (ch >> 12);
      *utf8ptr++ = 0x80 + ((ch >> 6) & 0x3f);
      *utf8ptr++ = 0x80 + (ch & 0x3f);
    }
  }

}

void ReleaseStringUTFChars(JNIEnv *env, jstring string, const char *utf) {
  releaseMem((void*)utf);
}                  

jsize GetArrayLength(JNIEnv *env, jarray array) {
  return (jsize)instance2Array_length(array);
}
  
jarray NewObjectArray(JNIEnv *env, jsize length, jclass elementType, jobject initialElement) {

  w_instance initval = initialElement;
  w_thread thread = JNIEnv2w_thread(env);
  w_instance Array;
  w_int i;
  w_clazz elementClazz = Class2clazz(elementType);
  w_clazz arrayClazz;
  w_word alength;

  woempa(1, "Asked to construct an array of %d %k's, all set to 0x%08x\n", length, elementClazz, initval);

  arrayClazz = getNextDimension(elementClazz);

  if (exceptionThrown(thread)) {
    return NULL;
  }

  woempa(1, "New array will be of class %k\n", arrayClazz);
  alength = length;
  Array = allocArrayInstance(JNIEnv2w_thread(env), arrayClazz, 1, &alength);
  woempa(1, "New array is %j\n", Array);

  if (Array) {
    for (i = 0; i < length; i++) {
      Array[F_Array_data + i] = (w_word) initval;
    }
    woempa(1, "(JNI) Length %d, type %k, Array = %p.\n", length, clazz_Array, Array);
  }
  else {
    throwOutOfMemoryError(thread);
  }

  return Array;
  
}

jobject GetObjectArrayElement(JNIEnv *env, jobjectArray Array, jsize aindex) {

  w_thread thread = JNIEnv2w_thread(env);
  w_instance *instanceArray = instance2Array_instance(Array);
  w_instance result = instanceArray[aindex];

  woempa(1, "(JNI) (%k)(%p)[%d] = %p.\n", instance2clazz(Array), Array, aindex, instanceArray[aindex]);

  addLocalReference(thread, result);

  return (jobject)result;
  
}

void SetObjectArrayElement(JNIEnv *env, jobjectArray Array, jsize aindex, jobject value) {

  woempa(1, "(JNI) (%k)(%p)[%d] = %p (%k).\n", instance2clazz(Array), Array, aindex, value, instance2clazz(value));
  setArrayReferenceField(Array, value, aindex);
  
}

static w_instance newTypeArray(w_thread thread, w_clazz clazz, w_int length) {

  w_instance Array = allocArrayInstance(thread, clazz, 1, &length);

  woempa(1, "(JNI) Length %d, type %k, Array = %p.\n", length, clazz, Array);
  
  return Array;
  
}

/*
** Create primitive array types. Look at the 'newarray' opcode explanation
** for the meaning of the array indexes.
*/

jbooleanArray NewBooleanArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_boolean], length);
}

jbyteArray NewByteArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_byte], length);
}

jcharArray NewCharArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_char], length);
}

jshortArray NewShortArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_short], length);
}

jintArray NewIntArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_int], length);
}

jlongArray NewLongArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_long], length);
}

jfloatArray NewFloatArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_float], length);
}

jdoubleArray NewDoubleArray(JNIEnv *env, jsize length) {
  return newTypeArray(JNIEnv2w_thread(env), atype2clazz[P_double], length);
}

jboolean *GetBooleanArrayElements(JNIEnv *env, jbooleanArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jboolean *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jboolean));
  jboolean* dest = buffer;
  w_sbyte *arrayAsBytes = instance2Array_byte(array_instance);
  w_size i;
  
  woempa(1, "(JNI) array of %d bits at %p expands to %d jbooleans (%d bytes) at %p\n", length, array_instance, length, (w_size)(length * sizeof(jboolean)), buffer);

  for (i = 0; i < length; i++) {
    *dest++ = !!(arrayAsBytes[i/8] & (1<<(i%8)));
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;

}

jbyte *GetByteArrayElements(JNIEnv *env, jbyteArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jbyte *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jbyte));
  jbyte *dest = buffer;
  w_sbyte *srce = instance2Array_byte(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d bytes from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;

}

jchar *GetCharArrayElements(JNIEnv *env, jcharArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jchar *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jchar));
  jchar *dest = buffer;
  w_char *srce = instance2Array_char(array_instance);
  w_size i;
  
  woempa(1, "(JNI) copy %d char's (%d bytes) from %p to %p\n", length, (w_size)(length * sizeof(w_char)), array_instance + F_Array_data, buffer);

  for (i = 0; i < length; i++) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;

}

jshort *GetShortArrayElements(JNIEnv *env, jshortArray array, jboolean *isCopy) {

  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jshort *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jshort));
  jshort *dest = buffer;
  w_short *srce = instance2Array_short(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d short's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_short)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }

  return buffer;
}

jint *GetIntArrayElements(JNIEnv *env, jintArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jint  *buffer = allocMem(JNIEnv2w_thread(env), length * sizeof(jint));
  jint  *dest = buffer;
  w_int *srce = instance2Array_int(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d int's from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jlong *GetLongArrayElements(JNIEnv *env, jlongArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jlong  *buffer = (jlong*)allocMem(JNIEnv2w_thread(env),length*sizeof(jlong));
  jlong  *dest = buffer;
  w_long *srce = instance2Array_long(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d long's from %p to %p\n",length,array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jfloat *GetFloatArrayElements(JNIEnv *env, jfloatArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jfloat  *buffer = (jfloat*)allocMem(JNIEnv2w_thread(env),length*sizeof(jfloat));
  jfloat  *dest = buffer;
  w_float *srce = instance2Array_float(array_instance);
  w_size i;
  
  woempa(1,"(JNI) copy %d float's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_float)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

jdouble *GetDoubleArrayElements(JNIEnv *env, jdoubleArray array, jboolean *isCopy) {
  w_instance array_instance = array;
  w_size length = instance2Array_length(array_instance);
  jdouble *buffer = (jdouble*)allocMem(JNIEnv2w_thread(env),length*sizeof(jdouble));
  jdouble  *dest = buffer;
  w_double *srce = instance2Array_double(array_instance);
  w_size i;

  woempa(1,"(JNI) copy %d double's (%d bytes) from %p to %p\n",length,(w_size)(length*sizeof(w_double)),array_instance+F_Array_data,buffer);
  for (i=0;i<length;++i) {
    *dest++ = *srce++;
  }
  
  if (isCopy) {
    *isCopy = WONKA_TRUE;
  }
  
  return buffer;
}

void ReleaseBooleanArrayElements(JNIEnv *env, jbooleanArray array, jboolean *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeBooleansToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseByteArrayElements(JNIEnv *env, jbyteArray array, jbyte *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeBytesToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseCharArrayElements(JNIEnv *env, jcharArray array, jchar *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeCharsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseShortArrayElements(JNIEnv *env, jshortArray array, jshort *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeShortsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseIntArrayElements(JNIEnv *env, jintArray array, jint *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeIntsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseLongArrayElements(JNIEnv *env, jlongArray array, jlong *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeLongsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseFloatArrayElements(JNIEnv *env, jfloatArray array, jfloat *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeFloatsToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}

void ReleaseDoubleArrayElements(JNIEnv *env, jdoubleArray array, jdouble *elements, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);

  if (mode!=JNI_ABORT) {
    writeDoublesToArray(array_instance, 0, length, elements);
  }
  
  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",elements);
    releaseMem(elements);
  }
}                                                        

w_void GetBooleanArrayRegion(JNIEnv *env, jbooleanArray array, jsize start, jsize len, jboolean *buf) {
  readBooleansFromArray(array, start, len, buf);
}

w_void GetByteArrayRegion(JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf) {
  readBytesFromArray(array, start, len, buf);
}

w_void GetCharArrayRegion(JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf) {
  readCharsFromArray(array, start, len, buf);
}

w_void GetShortArrayRegion(JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf) {
  readShortsFromArray(array, start, len, buf);
}

w_void GetIntArrayRegion(JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf) {
  readIntsFromArray(array, start, len, buf);
}

w_void GetLongArrayRegion(JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf) {
  readLongsFromArray(array, start, len, buf);
}

w_void GetFloatArrayRegion(JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf) {
  readFloatsFromArray(array, start, len, buf);
}

w_void GetDoubleArrayRegion(JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf) {
  readDoublesFromArray(array, start, len, buf);
}

w_void SetBooleanArrayRegion(JNIEnv *env, jbooleanArray array, jsize start, jsize len, jboolean *buf) {
  writeBooleansToArray(array, start, len, buf);
}

w_void SetByteArrayRegion(JNIEnv *env, jbyteArray array, jsize start, jsize len, jbyte *buf) {
  writeBytesToArray(array, start, len, buf);
}

w_void SetCharArrayRegion(JNIEnv *env, jcharArray array, jsize start, jsize len, jchar *buf) {
  writeShortsToArray(array, start, len, buf);
}

w_void SetShortArrayRegion(JNIEnv *env, jshortArray array, jsize start, jsize len, jshort *buf) {
  writeShortsToArray(array, start, len, buf);
}

w_void SetIntArrayRegion(JNIEnv *env, jintArray array, jsize start, jsize len, jint *buf) {
  writeIntsToArray(array, start, len, buf);
}

w_void SetLongArrayRegion(JNIEnv *env, jlongArray array, jsize start, jsize len, jlong *buf) {
  writeLongsToArray(array, start, len, buf);
}

w_void SetFloatArrayRegion(JNIEnv *env, jfloatArray array, jsize start, jsize len, jfloat *buf) {
  writeFloatsToArray(array, start, len, buf);
}

w_void SetDoubleArrayRegion(JNIEnv *env, jdoubleArray array, jsize start, jsize len, jdouble *buf) {
  writeDoublesToArray(array, start, len, buf);
}
                
jint RegisterNatives(JNIEnv *env, jclass class, const JNINativeMethod *methods, jint nMethods) {
  registerNatives(Class2clazz(class), methods, nMethods);
  return 0;
}

jint UnregisterNatives(JNIEnv *env, jclass class) {
  wabort(ABORT_WONKA, "JNI function not yet implemented.\n");
  return -1;
}
  
jint MonitorEnter(JNIEnv *env, jobject obj) {
  enterMonitor((w_instance)obj);
  return 0;
}

jint MonitorExit(JNIEnv *env, jobject obj) {
  exitMonitor((w_instance)obj);
  return 0;
}
 

/*
** GetPrimitiveArrayCritical / ReleasePrimitiveArrayCritical:
** since all array buffer locks in Wonka are critical, we can just forward these calls
** to the standard Get<type>Arrayelements/Release<type>Arrayelements functions
** according to the right types of array
**
** Version II: merge the codes for the IsInstanceOf(type) and Get/Release<type>Arrayelements into one
** with as few double code and redirections as possible
*/

void* GetPrimitiveArrayCritical(JNIEnv *env, jarray array, jboolean *isCopy)  {
  
  w_instance array_instance = array;
  void     *buffer = NULL;
  w_clazz  array_clazz = instance2clazz(array);
  w_clazz  component_clazz = array_clazz->previousDimension;
  jboolean tmp;

  /*
  ** according to the array type, fill in buffer space and grab array data pointer
  */

  if(component_clazz == clazz_boolean ) {
    buffer = GetBooleanArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_byte) {
    buffer = GetByteArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_char ) {
    buffer = GetCharArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_short ) {
    buffer = GetShortArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_int ) {
    buffer = GetIntArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_long ) {
    buffer = GetLongArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_float ) {
    buffer = GetFloatArrayElements(env, array_instance, &tmp);
  }
  else if(component_clazz == clazz_double ) {
    buffer = GetDoubleArrayElements(env, array_instance, &tmp);
  }
  else {
    buffer = NULL;
  }

  /*
  ** set copy flag to true (this routine ALWAYS copies)
  */

  if (isCopy)
    *isCopy = tmp;

  return buffer;
}


void  ReleasePrimitiveArrayCritical(JNIEnv *env, jarray array, void *carray, jint mode) {
  w_instance array_instance = array;
  w_int length = instance2Array_length(array_instance);
  w_clazz  array_clazz = instance2clazz(array);
  w_clazz  component_clazz = array_clazz->previousDimension;

  if (mode!=JNI_ABORT) {
    if(component_clazz == clazz_boolean ) {
      writeBooleansToArray(array_instance, 0, length, (jboolean*)carray);
    }
    else if(component_clazz == clazz_byte ) {
      writeBytesToArray(array_instance, 0, length, (jbyte*)carray);
    }
    else if(component_clazz == clazz_char ) {
      writeCharsToArray(array_instance, 0, length, (jchar*)carray);
    }
    else if(component_clazz == clazz_short ) {
      writeShortsToArray(array_instance, 0, length, (jshort*)carray);
    }
    else if(component_clazz == clazz_int ) {
      writeIntsToArray(array_instance, 0, length, (jint*)carray);
    }
    else if(component_clazz == clazz_long ) {
      writeLongsToArray(array_instance, 0, length, (jlong*)carray);
    }
    else if(component_clazz == clazz_float ) {
      writeFloatsToArray(array_instance, 0, length, (jfloat*)carray);
    }
    else if(component_clazz == clazz_double ) {
      writeDoublesToArray(array_instance, 0, length, (jdouble*)carray);
    }
  }

  if (mode!=JNI_COMMIT) {
    woempa(1,"(JNI) release copy at %p\n",carray);
    releaseMem(carray);
  }
}

/*
** GetStringCritical / ReleaseStringCritical:
** since all string buffer locks in Wonka are critical, we can just forward these calls
** to the standard GetStringchars/ReleaseStringChars functions
*/

const jchar *GetStringCritical(JNIEnv *env, jstring String, jboolean *isCopy) {
  woempa(1,"redirecting GetStringCritical to GetStringChars(Always critical in Wonka)\n");
  return GetStringChars(env,String,isCopy);
}


void ReleaseStringCritical(JNIEnv *env, jstring string, const jchar *chars) {
  woempa(1,"redirecting ReleaseStringCritical to ReleaseStringChars(Always critical in Wonka)\n");
  ReleaseStringChars(env, string,chars);
}

/*
** There is only one JVM; he wears green trousers and a plum jacket,
** and sports a top hat and a walking-cane.
*/

static JavaVM mrWonka;

/*
** You can ask for any version you want, so long as it's positive.
** Mr. Wonka hates negative attitudes.
*/

jint JNI_GetDefaultJavaVMInitArgs(void *vm_args) {
  return ((Wonka_InitArgs*)vm_args)->version & 0x80000000 ? -1 : 0;
}

/*
** He is unique.
*/

jint JNI_GetCreatedJavaVMs(JavaVM **vmBuf, jsize bufLen, jsize *nVMs) {
  if (bufLen) {
    mrWonka = &w_JNIInvokeInterface;
    *vmBuf = &mrWonka;
    *nVMs = 1;
  }
  return 0;
}

jint JNI_CreateJavaVM(JavaVM **p_VM, JNIEnv **p_env, void *vm_args) {

  globals_hashtable = ht_create(W_Realm_system, (char*)"hashtable:global-refs", GLOBALS_HASHTABLE_SIZE, NULL, NULL, 0, 0);
  woempa(1, "(JNI) created globals_hashtable at %p\n", globals_hashtable);

  system_vm_args = (struct JavaVMInitArgs *)vm_args;

  startHeap();
  startLoading();
  startKernel();
  
#ifdef RUDOLPH  
#ifdef MODULES 
  loadModule("mod_awt");
#else 
  init_awt();
#endif
#endif

  x_thread_resume(W_Thread_sysInit->kthread);

  return 0;

}

jint DestroyJavaVM(JavaVM *vm) {
  woempa(9,"Sorry, you can't do that.\n");
  return -1;
}

jint GetJavaVM(JNIEnv *env, JavaVM **vm) {
  mrWonka = &w_JNIInvokeInterface;
  *vm = &mrWonka;
  return 0;
}
  
jint GetEnv(JavaVM *vm, void **p_env, jint version) {
  *p_env = w_thread2JNIEnv(currentWonkaThread);
  return 0;
}

/*
** In order to visit the Wonka Factory, you must be in possesion of valid
** "pieces of identity".  This function will furnish you with them.
**
** Note: not every field of the w_Thread structure is filled in here.
** Maybe we need to do more?
*/

jint AttachCurrentThread(JavaVM *vm, JNIEnv **p_env, void *thr_args) {
#ifdef O4P
  printf("AttachCurrentThread not implemented!\n");
#else
  w_size     i;
  w_instance Thread;
  w_thread   thread;
  x_frame    frame;
  w_method   method = NULL;

  Wonka_AttachArgs *wonka_args = thr_args;
  
  char *cname = (char*)"C-Thread";
  w_MethodSpec spec;

  if (thread_hashtable == NULL) {
    return -1;
  }

  if (wonka_args && wonka_args->name) {
    cname = wonka_args->name;
  }

  thread  = allocClearedMem_system(sizeof(w_Thread));
  thread->natenv = &w_JNINativeInterface;
  thread->label = (char*)"thread:native";
  thread->name = cstring2String(W_ThreadGroup_system->manager,cname,strlen(cname));
  thread->state = wt_ready;
// [CG 20040507] Removed thread->messages, it was not used for anything
//  thread->messages = (x_queue)allocMem(W_ThreadGroup_system->manager, sizeof(x_Queue));
//  x_queue_create(thread->messages, allocMem_system(4*sizeof(void*)),4);
  thread->group = W_ThreadGroup_system;
  
  /* 
  ** The following is bogus, but means that child threads default to a reasonable priority
  */

  thread->jpriority = W_ThreadGroup_system->maxPriority / 2; 
  thread->isDaemon = WONKA_FALSE;
  thread->flags = 0;
  thread->kthread = (x_thread)x_thread_current();
  thread->kthread->xref = thread;
  thread->kpriority = x_thread_priority_get(thread->kthread);
  woempa(1, "Registering os thread %p as Java thread %p\n", thread->kthread, thread);
  ht_write(thread_hashtable, (w_word)thread->kthread, (w_word)thread);
  classMustBeInitialized(clazzNativeThread);
  if (exceptionThrown(thread)) {

    return -1;

  }

  Thread = allocInstance(W_ThreadGroup_system->manager, clazzNativeThread);
  setReferenceField(Thread, I_ThreadGroup_system, F_Thread_parent);

  thread->Thread = Thread;
  setWotsitField(Thread, F_Thread_wotsit,  thread);
  setReferenceField(Thread, I_ThreadGroup_system, F_Thread_parent);
  thread->Thread = Thread;
  woempa(1, "Native thread %p ('%w') now has instance of %j.\n", thread, NM(thread), Thread);

  spec.declaring_clazz = clazzNativeThread;
  spec.name = cstring2String(W_Thread_system, "_", 1);
  spec.arg_types = NULL;
  spec.return_type = clazz_void;

  woempa(1, "Seek %w in %K\n", spec.name, clazzNativeThread);
  for (i = 0; i < clazzNativeThread->numDeclaredMethods; ++i) {
    w_method m = &clazzNativeThread->own_methods[i];

    woempa(1, "Candidate: %w\n", m->spec.name);
    if (methodMatchesSpec(m, &spec) && isNotSet(m->flags, ACC_STATIC)) {
      method = m;
      break;
    }
  }
  deregisterString(spec.name);

  if (method == NULL) {
    wabort(ABORT_WONKA,"Uh oh: class %k doesn't have a _()V method.  Game over.\n", clazzNativeThread);
  }

  attachFirstFrame(thread);

  frame = pushFrame(thread, method);
  frame->top[0].c.w = (w_word) I_Thread_sysInit;
  frame->top[0].s = stack_trace;
  frame->top += 1;
  thread->run_method = method;
  addThreadToGroup(thread, W_ThreadGroup_system);
  numThreads += 1;

  *p_env = w_thread2JNIEnv(thread);
#endif

  return 0;

}

/*
** Thankyou for visiting the Wonka factory.  Don't forget to give your
** badge to the security squirrel on the way out.
**
** TODO: figure out how to release all monitors held by a thread which
** is dumb enough to call this function from inside a monitor. 8-0
*/

jint DetachCurrentThread(JavaVM *vm) {
#ifdef O4P
  printf("AttachCurrentThread not implemented!\n");
#else
  w_thread thread = currentWonkaThread;
  x_event event;

  deactivateFrame(thread->top, NULL);
  removeThreadFromGroup(thread,W_ThreadGroup_system);
  ht_erase(thread_hashtable,(w_word)thread->kthread);
  numThreads -= 1;

  /*
  ** Release the java monitors.
  */

  
//  for (event = thread->l_owned; event; event = event->l_owned) {
//    if (x_event_type_get(event) == xe_monitor && event->flags_type & 0x10000 != 0) {
      
      /*
      ** Found a locked java monitor !
      */

//      w_dump("Found a locked java monitor\n");
      
//    }

  
#endif
  return 0;
}


const struct JNINativeInterface w_JNINativeInterface = {

  NULL,
  NULL,
  NULL,
  NULL,
  GetVersion,
  
  DefineClass,
  FindClass,

  NULL,
  NULL,
  NULL,
  
  GetSuperclass,                        /* 10 */
  IsAssignableFrom,
  
  NULL,

  Throw,
  ThrowNew,
  ExceptionOccurred,
  ExceptionDescribe,
  ExceptionClear,
  FatalError,
  
  NULL,
  NULL,                                 /* 20 */
  
  NewGlobalRef,
  DeleteGlobalRef,
  DeleteLocalRef,
  IsSameObject,
  
  NewLocalRef,
  NULL,
  
  AllocObject,
  NewObject,
  NewObjectV,
  NewObjectA,                           /* 30 */

  GetObjectClass,
  IsInstanceOf,
  
  GetMethodID,

  CallObjectMethod,
  CallObjectMethodV,
  CallObjectMethodA,
  CallBooleanMethod,
  CallBooleanMethodV,
  CallBooleanMethodA,
  CallByteMethod,                       /* 40 */
  CallByteMethodV,
  CallByteMethodA,
  CallCharMethod,
  CallCharMethodV,
  CallCharMethodA,
  CallShortMethod,
  CallShortMethodV,
  CallShortMethodA,
  CallIntMethod,
  CallIntMethodV,                       /* 50 */
  CallIntMethodA,
  CallLongMethod,
  CallLongMethodV,
  CallLongMethodA,
  CallFloatMethod,
  CallFloatMethodV,
  CallFloatMethodA,
  CallDoubleMethod,
  CallDoubleMethodV,
  CallDoubleMethodA,                    /* 60 */
  CallVoidMethod,
  CallVoidMethodV,
  CallVoidMethodA,

  CallNonvirtualObjectMethod,
  CallNonvirtualObjectMethodV,
  CallNonvirtualObjectMethodA,
  CallNonvirtualBooleanMethod,
  CallNonvirtualBooleanMethodV,
  CallNonvirtualBooleanMethodA,
  CallNonvirtualByteMethod,             /* 70 */
  CallNonvirtualByteMethodV,
  CallNonvirtualByteMethodA,
  CallNonvirtualCharMethod,
  CallNonvirtualCharMethodV,
  CallNonvirtualCharMethodA,
  CallNonvirtualShortMethod,
  CallNonvirtualShortMethodV,
  CallNonvirtualShortMethodA,
  CallNonvirtualIntMethod,
  CallNonvirtualIntMethodV,             /* 80 */
  CallNonvirtualIntMethodA,
  CallNonvirtualLongMethod,
  CallNonvirtualLongMethodV,
  CallNonvirtualLongMethodA,
  CallNonvirtualFloatMethod,
  CallNonvirtualFloatMethodV,
  CallNonvirtualFloatMethodA,
  CallNonvirtualDoubleMethod,
  CallNonvirtualDoubleMethodV,
  CallNonvirtualDoubleMethodA,          /* 90 */
  CallNonvirtualVoidMethod,
  CallNonvirtualVoidMethodV,
  CallNonvirtualVoidMethodA,

  GetFieldID,

  GetObjectField,
  GetBooleanField,
  GetByteField,
  GetCharField,
  GetShortField,
  GetIntField,                          /* 100 */
  GetLongField,
  GetFloatField,
  GetDoubleField,
  SetObjectField,
  SetBooleanField,
  SetByteField,
  SetCharField,
  SetShortField,
  SetIntField,
  SetLongField,                         /* 110 */
  SetFloatField,
  SetDoubleField,

  GetStaticMethodID,

  CallStaticObjectMethod,
  CallStaticObjectMethodV,
  CallStaticObjectMethodA,
  CallStaticBooleanMethod,
  CallStaticBooleanMethodV,
  CallStaticBooleanMethodA,
  CallStaticByteMethod,                 /* 120 */
  CallStaticByteMethodV,
  CallStaticByteMethodA,
  CallStaticCharMethod,
  CallStaticCharMethodV,
  CallStaticCharMethodA,
  CallStaticShortMethod,
  CallStaticShortMethodV,
  CallStaticShortMethodA,
  CallStaticIntMethod,
  CallStaticIntMethodV,                 /* 130 */
  CallStaticIntMethodA,
  CallStaticLongMethod,
  CallStaticLongMethodV,
  CallStaticLongMethodA,
  CallStaticFloatMethod,
  CallStaticFloatMethodV,
  CallStaticFloatMethodA,
  CallStaticDoubleMethod,
  CallStaticDoubleMethodV,
  CallStaticDoubleMethodA,              /* 140 */
  CallStaticVoidMethod,
  CallStaticVoidMethodV,
  CallStaticVoidMethodA,

  GetStaticFieldID,

  GetStaticObjectField,
  GetStaticBooleanField,
  GetStaticByteField,
  GetStaticCharField,
  GetStaticShortField,
  GetStaticIntField,                    /* 150 */
  GetStaticLongField,
  GetStaticFloatField,
  GetStaticDoubleField,
  SetStaticObjectField,
  SetStaticBooleanField,
  SetStaticByteField,
  SetStaticCharField,
  SetStaticShortField,
  SetStaticIntField,
  SetStaticLongField,                   /* 160 */
  SetStaticFloatField,
  SetStaticDoubleField,

  NewString,
  GetStringLength,
  GetStringChars,
  ReleaseStringChars,

  NewStringUTF,
  GetStringUTFLength,
  GetStringUTFChars,
  ReleaseStringUTFChars,                /* 170 */

  GetArrayLength,
  
  NewObjectArray,
  GetObjectArrayElement,
  SetObjectArrayElement,

  NewBooleanArray,
  NewByteArray,
  NewCharArray,
  NewShortArray,
  NewIntArray,
  NewLongArray,                         /* 180 */
  NewFloatArray,
  NewDoubleArray,

  GetBooleanArrayElements,
  GetByteArrayElements,
  GetCharArrayElements,
  GetShortArrayElements,
  GetIntArrayElements,
  GetLongArrayElements,
  GetFloatArrayElements,
  GetDoubleArrayElements,               /* 190 */

  ReleaseBooleanArrayElements,
  ReleaseByteArrayElements,
  ReleaseCharArrayElements,
  ReleaseShortArrayElements,
  ReleaseIntArrayElements,
  ReleaseLongArrayElements,
  ReleaseFloatArrayElements,
  ReleaseDoubleArrayElements,

  GetBooleanArrayRegion,
  GetByteArrayRegion,                   /* 200 */
  GetCharArrayRegion,
  GetShortArrayRegion,
  GetIntArrayRegion,
  GetLongArrayRegion,
  GetFloatArrayRegion,
  GetDoubleArrayRegion,

  SetBooleanArrayRegion,
  SetByteArrayRegion,
  SetCharArrayRegion,
  SetShortArrayRegion,                  /* 210 */
  SetIntArrayRegion,
  SetLongArrayRegion,
  SetFloatArrayRegion,
  SetDoubleArrayRegion,

  RegisterNatives,
  UnregisterNatives,
  
  MonitorEnter,
  MonitorExit,

  GetJavaVM,
  NULL, //GetStringRegion,              /* 220 */
  GetStringUTFRegion,
  GetPrimitiveArrayCritical,
  ReleasePrimitiveArrayCritical,
  GetStringCritical,
  ReleaseStringCritical,
  NULL, //NewWeakGlobalRef,
  NULL, //reserved227,
  ExceptionCheck,

};

const struct JNIInvokeInterface w_JNIInvokeInterface = {
  NULL,
  NULL,
  NULL,
  DestroyJavaVM,
  AttachCurrentThread,
  DetachCurrentThread,
  GetEnv,
};

