/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, bus 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: loading.c 4161 2004-05-14 13:44:19Z dumon $
*/

#include <string.h>

#include "arrays.h"
#include "bar.h"
#include "clazz.h"
#include "checks.h"
#include "constant.h"
#include "core-classes.h"
#include "descriptor.h"
#include "exception.h"
#include "fields.h"
#include "fifo.h"
#include "interpreter.h"
#include "heap.h"
#include "list.h"
#include "loading.h"
#include "locks.h"
#include "ts-mem.h"
#include "methods.h"
#include "wstrings.h"
#include "wonka.h"
#include "threads.h"
#include "hashtable.h"
#include "repository.h"
#include "vfs.h"
#include "argument.h"
#include "jdwp.h"
#include "jdwp_events.h"

w_instance systemClassLoader = NULL;

/*
** A hashtable which maps class names onto ``fixup'' functions.
*/
w_hashtable fixup_hashtable;

/*
** The initial size of \texttt{fixup_hashtable}
*/
#define NATIVE_FUN_HT_SIZE 37

/*
** A hashtable which maps class x imethod -> method
*/
w_hashtable2k interface_hashtable;

/*
** The initial size of \texttt{interface_hashtable}
*/
#define INTERFACE_HT_SIZE 101

char     *zipname;

#ifndef BOOTCLASSDIR
#define BOOTCLASSDIR "."
#endif

#ifndef BOOTCLASSFILE
#define BOOTCLASSFILE "classes.zip"
#endif

/*
** Some well-known strings that are required during loading.
** They are registered in startLoading().
*/
// Names of primitive classes
w_string string_boolean;
w_string string_byte;
w_string string_char;
w_string string_double;
w_string string_float;
w_string string_int;
w_string string_long;
w_string string_short;
w_string string_void;

// Names of classfile attributes
w_string string_ConstantValue;
w_string string_Code;
w_string string_serialVersionUID;
w_string string_InnerClasses;
w_string string_Deprecated;
w_string string_Exceptions;
w_string string_LineNumberTable;
w_string string_LocalVariableTable;
w_string string_Reference;
w_string string_SourceFile;
w_string string_Synthetic;

// Names and signatures of special methods
w_string string_run;
w_string string_finalize;
w_string string_loadClass;
w_string string_angle_brackets_init;
w_string string_angle_brackets_clinit;
w_string string_no_params_V;
w_string string_no_params;
// [CG 20021223] WAS :
// w_string string_params_String_Z_return_Class;
w_string string_params_String_return_Class;

// The "name" of the bootstrap class loader
w_string string_bootstrap;

w_clazz atype2clazz[12];

w_clazz createPrimitiveArrayClazz(w_int pi);

static inline w_string loader2name(w_instance loader) {
  if (loader) {

    return String2string(getReferenceField(loader, F_ClassLoader_ownname));

  }

  return string_bootstrap;
}

/*
** Compare two w_Clazz pointers.
*/
w_boolean sameClazz(w_clazz clazz1, w_clazz clazz2) {
  w_boolean result = WONKA_FALSE;

  /*
  ** Do a quick check first, when the two memory addresses match, the clazzes match.
  ** Otherwise, compare the 'name ' and 'loader' fields.
  */

  woempa(1, "Comparing {%j}%w (%p) with {%j}%w (%p)\n", clazz1->loader, clazz1->dotified, clazz1, clazz2->loader, clazz2->dotified, clazz2);
  if (clazz1 == clazz2) {
    woempa(1, "Identical, return TRUE\n");
    result = WONKA_TRUE;
  }
  else if (!!getClazzState(clazz1) == !!getClazzState(clazz2)
        && clazz1->loader == clazz2->loader
        && clazz1->dotified == clazz2->dotified) {
    woempa(1, "Same loadedness, loader, same name, return TRUE\n");
    result = WONKA_TRUE;
  }

  woempa(1, "Compared {%j}%w (%p) with {%j}%w (%p): they are %s\n", clazz1->loader, clazz1->dotified, clazz1, clazz2->loader, clazz2->dotified, clazz2, result ? "the same" : "different");

  return result;

}

/*
** Compare two pointers to w_Clazz (or w_UnloadedCLazz) pointers.
*/
w_boolean sameClassReference(w_clazz *clazzptr1, w_clazz *clazzptr2) {
  if (*clazzptr1 == *clazzptr2) {

    return WONKA_TRUE;

  }

  if ((*clazzptr1)->dotified != (*clazzptr2)->dotified) {

    return WONKA_FALSE;

  }

  if ((getClazzState(*clazzptr1) == CLAZZ_STATE_UNLOADED) && (getClazzState(*clazzptr2) == CLAZZ_STATE_UNLOADED) && (*clazzptr1)->loader == (*clazzptr2)->loader) {

    return WONKA_TRUE;

  }

  if (mustBeLoaded(clazzptr1) == CLASS_LOADING_FAILED) {
    woempa(7, "Failed to load %K\n", *clazzptr1);

    return WONKA_FALSE;

  }
  else {
    woempa(1, "*clazzptr1 is %K\n", *clazzptr1);
  }

  if (mustBeLoaded(clazzptr2) == CLASS_LOADING_FAILED) {
    woempa(7, "Failed to load %K\n", *clazzptr2);

    return WONKA_FALSE;

  }
  else {
    woempa(1, "*clazzptr2 is %K\n", *clazzptr2);
  }

  return *clazzptr1 == *clazzptr2;

}

/*
** For use in hashtables.
*/
w_boolean clazzComparator(w_word clazz1_word, w_word clazz2_word) {
  return sameClazz((w_clazz)clazz1_word, (w_clazz)clazz2_word);
}

/*
** Hashcode for a w_UnloadedClass.
*/
w_word clazzHashCode(w_word clazz_word) {
  w_clazz clazz = (w_clazz)clazz_word;

  return (w_word)clazz->dotified ^ (w_word)clazz->loader;

}



/*
** Register a w_UnloadedClazz. The result returned is a w_UnloadedClazz*
** which may or not be the same as the one passed as input: if it is
** different, then the one passed as input has been released.
*/

w_clazz registerUnloadedClazz(w_clazz clazz) {
 
  w_clazz result;
  w_hashtable hashtable = loader2unloaded_classes(clazz->loader);

  woempa(1, "Registering %k at %p with %s\n", clazz, clazz, hashtable->label);
  result = (w_clazz)ht_register(hashtable, (w_word)clazz);
  if (result) {
    if (result != clazz) {
      woempa(1, "Found %k at %p, releasing %p\n", result, result, clazz);
      releaseMem(clazz);
    }
    else {
      woempa(1, "Found %k at %p already\n", result,result);
    }
  }
  else {
    woempa(1,"Adding %k at %p\n", clazz, clazz);
    result = clazz;
  }

  return result;
  
}

/*
** Deregister a w_UnloadedClazz. When we see that the registered count drops
** to 0, we release the w_UnloadeClazz structure.
*/

void deregisterUnloadedClazz(w_clazz clazz) {
  w_clazz garbage = NULL;
  w_hashtable hashtable = loader2unloaded_classes(clazz->loader);

  woempa(1, "Deregistering %k\n", clazz);
  garbage = (w_clazz)ht_deregister(hashtable, (w_word)clazz);
  if (garbage) {
    woempa(1,"Releasing %k (%p)\n",garbage,garbage);
    deregisterString(garbage->dotified);
    releaseMem(garbage);
  }
  else {
    woempa(1, "Retaining %k (%p), as still %d reference(s)\n", clazz, clazz, ht_read(hashtable, (w_word)clazz));
  }
}

/*
** If a class with the given name is already known to the classloader,
** return a pointer to the corresponding w_Clazz structure. Othwerwise,
** construct a w_UnloadClazz structure which records the name and 
** initating classloader.
*/
w_clazz identifyClazz(w_string name, w_instance loader) {
  w_clazz result = NULL;

  result = (w_clazz)ht_read_no_lock(loader2loaded_classes(loader), (w_word)name);

  if (!result) {
    result = allocMem_system(sizeof(w_UnloadedClazz));

    result->dotified = registerString(name);
    result->label    = (char *)"clazz(unloaded)";
    result->flags    = 0; 
    result->loader   = loader;

    result = registerUnloadedClazz(result);
  }

  return result;
}

/*
** If *clazzptr is already loaded, do nothing; otherwise try to load the class.
** If nothing is done, returns CLASS_LOADING_DID_NOTHING.
** If class was not loaded and loading succeeds, updates *clazzptr
** and returns CLASS_LOADING_SUCCEEDED.
** If If class was not loaded and loading failed, returns
** CLASS_LOADING_FAILED. An exception will be pending on the current thread.
*/
w_int mustBeLoaded(volatile w_clazz *clazzptr) {
#if defined(DEBUG) || defined(GC_SAFE_POINTS)
  w_thread  thread = currentWonkaThread;
#endif
  w_int     result = CLASS_LOADING_DID_NOTHING;
  w_clazz   current = *clazzptr;
  w_int     state = getClazzState(current);
  x_monitor monitor;

  if (state == CLAZZ_STATE_UNLOADED) {
    w_clazz loaded;

    if (current->loader) {
      woempa(1, "Need to load class %p (%w) using loader %j (%w)\n", current, current->dotified, current->loader, loader2name(current->loader));
    }
    else {
      woempa(1, "Need to load class %p (%w) using bootstrap class loader\n", current, current->dotified);
    }
    loaded = namedClassMustBeLoaded(current->loader, current->dotified);
    if (!loaded) {
      result = CLASS_LOADING_FAILED;
    }
    else {
      result = CLASS_LOADING_SUCCEEDED;
      woempa(1, "Replacing %p with %p\n", current, loaded);
      deregisterUnloadedClazz(current);
      *clazzptr = loaded;
    }
  }
  else if (state < CLAZZ_STATE_LOADED) {
#ifdef GC_SAFE_POINTS
    w_boolean unsafe = enterSafeRegion(thread);
#endif
    monitor = current->resolution_monitor;

    x_monitor_eternal(monitor);
    while(state < CLAZZ_STATE_LOADED) {
      woempa(1, "Another thread (%t) is loading %k (%p), %t waiting\n", current->resolution_thread, current, current, thread);
      x_monitor_wait(monitor, x_millis2ticks(1000));
      state = getClazzState(current);
    }
    x_monitor_exit(monitor);
#ifdef GC_SAFE_POINTS
    if (unsafe) {
      enterUnsafeRegion(thread);
    }
#endif
  }

  if (current && getClazzState(current) == CLAZZ_STATE_BROKEN) {
    result = CLASS_LOADING_FAILED;
  }

  return result;
}

/*
** Set the systemClassLoader global variable and throw away the "repository"
** which was used by the bootstrap class loader. From now on the System
** Class Loader must be used instead.
*/
void setSystemClassLoader(w_instance scl) {
  x_monitor_eternal(&system_loaded_class_hashtable->monitor);
  if (systemClassLoader) {
    woempa(9, "Ahoy there! Someone tried to instal SystemClassLoader twice ...\n");
  }
  else {
    woempa(7, "*** SystemClassLoader created, instance is %p ***\n", scl);
    systemClassLoader = scl;
    destroyRepository(W_Repository_system);
    W_Repository_system = NULL;
  }
  x_monitor_exit(&system_loaded_class_hashtable->monitor);
}

/*
** Create the w_Clazz structure for a given primitive type, given its name
** (e.g. "boolean"), VM_TYPE (e.g. VM_TYPE_BOOLEAN) and the number of bits 
** needed to store it (0, 1, 8, 16, 32, or 64).
*/
static w_clazz createPrimitive(w_string name, w_ubyte type, w_int bits) {
  w_clazz clazz = allocClazz(W_Thread_system);

  clazz->dotified = registerString(name);
  clazz->flags = CLAZZ_IS_PRIMITIVE | (CLAZZ_STATE_LOADED << CLAZZ_STATE_SHIFT);
  clazz->loader = NULL;
  clazz->type = type;
  clazz->bits = bits;
  clazz->resolution_monitor = allocMem(W_Thread_system, sizeof(x_Monitor));
  x_monitor_create(clazz->resolution_monitor);

  woempa(1, "Created clazz_%w @ %p\n", name, clazz);

  return clazz;
}

static void preparePrimitive(w_clazz clazz, w_string string, w_int pi) {
  woempa(1, "Preparing primitive class `%w'\n", string);
  if (pi != P_void) {
    atype2clazz[pi] = createPrimitiveArrayClazz(pi);
  }
  clazz->initialize = alreadyInitialized;
  setClazzState(clazz, CLAZZ_STATE_INITIALIZED);
  clazz->Class = NULL;
  primitive2clazz[pi] = clazz;
  primitive2clazz[pi]->nextDimension = atype2clazz[pi];

  registerClazz(clazz, NULL);
  woempa(1, "Prepared primitive class `%w'\n", string);
}

w_method finalize_method;
w_method loadClass_method;

static void identify_special_methods(void) {
  w_method m;
  w_size   i;

  for (i = 0; i < clazzObject->numDeclaredMethods; i++) {
    m = &clazzObject->own_methods[i];
    if (m->spec.name == string_finalize && (w_string)clazzObject->values[m->desc_index] == string_no_params_V) {
      woempa(1, "Class %k has finalizer %m\n", clazzObject, m);
      finalize_method = m;
    }
  }
  if (!finalize_method) {
    wabort(ABORT_WONKA, "Oops - java.lang.Object doesn't hava a finalize() method\n");
  }

  for (i = 0; i < clazzClassLoader->numDeclaredMethods; i++) {
    m = &clazzClassLoader->own_methods[i];
// [CG 20021223] WAS :
//    if (m->spec.name == string_loadClass && (w_string)clazzClassLoader->values[m->desc_index] == string_params_String_Z_return_Class) {
    if (m->spec.name == string_loadClass && (w_string)clazzClassLoader->values[m->desc_index] == string_params_String_return_Class) {
      woempa(7, "Class %k has loadClass method %m\n", clazzClassLoader, m);
      loadClass_method = m;
    }
  }
  if (!loadClass_method) {
    wabort(ABORT_WONKA, "Oops - java.lang.ClassLoader doesn't hava a loadClass(String,boolean) method\n");
  }
}

void startLoading(void) {

  char            *files_data;
  int              fd;
  struct vfs_STAT  statbuf;
  w_word           files_size;
  w_int            i;
  w_int            j;

  w_string string_java_lang_Object = cstring2String(W_Thread_system, "java.lang.Object", 16);
  w_string string_java_lang_Cloneable = cstring2String(W_Thread_system, "java.lang.Cloneable", 19);
  w_string string_java_io_Serializable = cstring2String(W_Thread_system, "java.io.Serializable", 20);
  w_string string_java_lang_Throwable = cstring2String(W_Thread_system, "java.lang.Throwable", 19);
  w_string string_java_lang_Class = cstring2String(W_Thread_system, "java.lang.Class", 15);
  w_string string_java_lang_ClassLoader = cstring2String(W_Thread_system, "java.lang.ClassLoader", 21);
  w_string string_java_lang_Thread = cstring2String(W_Thread_system, "java.lang.Thread", 16);
  w_string string_java_lang_ThreadGroup = cstring2String(W_Thread_system, "java.lang.ThreadGroup", 21);
  w_string string_java_lang_ref_Reference = cstring2String(W_ThreadGroup_system->manager, "java.lang.ref.Reference", 23);
  w_string string_java_lang_reflect_Constructor = cstring2String(W_Thread_system, "java.lang.reflect.Constructor", 29);

  string_boolean = cstring2String(W_Thread_system, "boolean", 7);
  string_byte = cstring2String(W_Thread_system, "byte", 4);
  string_char = cstring2String(W_Thread_system, "char", 4);
  string_double = cstring2String(W_Thread_system, "double", 6);
  string_float = cstring2String(W_Thread_system, "float", 5);
  string_int = cstring2String(W_Thread_system, "int", 3);
  string_long = cstring2String(W_Thread_system, "long", 4);
  string_short = cstring2String(W_Thread_system, "short", 5);
  string_void = cstring2String(W_Thread_system, "void", 4);

  string_ConstantValue = cstring2String(W_Thread_system, "ConstantValue", 13);
  string_Code = cstring2String(W_Thread_system, "Code", 4);
  string_serialVersionUID = cstring2String(W_Thread_system, "serialVersionUID", 16);
  string_InnerClasses = cstring2String(W_Thread_system, "InnerClasses", 12);
  string_Deprecated = cstring2String(W_Thread_system, "Deprecated", 10);
  string_Exceptions = cstring2String(W_Thread_system, "Exceptions", 10);
  string_LineNumberTable = cstring2String(W_Thread_system, "LineNumberTable", 15);
  string_LocalVariableTable = cstring2String(W_Thread_system, "LocalVariableTable", 18);
  string_Reference = cstring2String(W_Thread_system, "java/lang/ref/Reference", 23);
  string_SourceFile = cstring2String(W_Thread_system, "SourceFile", 10);
  string_Synthetic = cstring2String(W_Thread_system, "Synthetic", 9);


  string_run = cstring2String(W_Thread_system, "run", 3);
  string_finalize = cstring2String(W_Thread_system, "finalize", 8);
  string_angle_brackets_init = cstring2String(W_Thread_system, "<init>", 6);
  string_angle_brackets_clinit = cstring2String(W_Thread_system, "<clinit>", 8);
  string_loadClass = cstring2String(W_Thread_system, "loadClass", 9);
  string_no_params = cstring2String(W_Thread_system, "()", 2);
  string_no_params_V = cstring2String(W_Thread_system, "()V", 3);
// [CG 20021223] WAS :
//  string_params_String_Z_return_Class = cstring2String(W_Thread_system,"(Ljava/lang/String;Z)Ljava/lang/Class;", 38);
  string_params_String_return_Class = cstring2String(W_Thread_system,"(Ljava/lang/String;)Ljava/lang/Class;", 37);

  string_bootstrap = cstring2String(W_Thread_system, "bootstrap class loader", 22);

  clazz_boolean = createPrimitive(string_boolean, VM_TYPE_BOOLEAN, 1);
  clazz_char    = createPrimitive(string_char, VM_TYPE_CHAR, 16);
  clazz_float   = createPrimitive(string_float, VM_TYPE_FLOAT, 32);
  clazz_double  = createPrimitive(string_double, VM_TYPE_DOUBLE + VM_TYPE_TWO_CELL, 64);
  clazz_byte    = createPrimitive(string_byte, VM_TYPE_BYTE, 8);
  clazz_short   = createPrimitive(string_short, VM_TYPE_SHORT, 16);
  clazz_int     = createPrimitive(string_int, VM_TYPE_INT, 32);
  clazz_long    = createPrimitive(string_long, VM_TYPE_LONG + VM_TYPE_TWO_CELL, 64);
  clazz_void    = createPrimitive(string_void, VM_TYPE_VOID, 0);

  /*
  ** Hack the bootclasspath to extract the first element which does not end
  ** in '/': this must be a jar (or zip) file.
  ** Skip over the leading '/' if present.
  */
  zipname = NULL;

  if (bootclasspath) {
    i = 0;
    j = 0;
    woempa(7, "bootclasspath is %s\n", bootclasspath);
    while (*bootclasspath) {
      for (j = i; j < (w_int)strlen(bootclasspath) && bootclasspath[j] != ':'; ++j);
      woempa(1, "Element ends at bootclasspath[%d]\n", j);
      if (j && bootclasspath[j - 1] != '/') {
        woempa(1, "Element does not end in `/', so we use bootclasspath[%d..%d]\n", i, j);

        break;

      }
      woempa(1, "Element ends with `/', start again from bootclasspath[%d]\n", j + 1);
      i = j + 1;
    }

    if (i < (w_int)strlen(bootclasspath)) {
      zipname = allocMem_system(j - i + 1);
      strncpy(zipname, &bootclasspath[i], j - i);
      zipname[j - i] = 0;
      if (zipname[0] == '/') {
        woempa(1, "Path `%s' starts with `/', so skip first char\n", zipname);
        ++zipname;
      }
      woempa(1, "Using path `%s'\n", zipname);
    }
  }

  /*
  ** If that didn't work (or we have no bootclasspath), use the default.
  */
  if (zipname == NULL) {
    zipname = (char*)BOOTCLASSDIR "/" BOOTCLASSFILE;
  }

  /*
  ** Read in the bootstrap classes
  */
  if(vfs_stat(zipname, &statbuf) == -1) { 
    wabort(ABORT_WONKA, "Could not find boot class library`%s'.\n", zipname);
  } 

  files_size = statbuf.st_size;
  woempa(9, "%s is %d bytes\n", zipname, files_size);
  files_data = allocMem(W_Thread_system, files_size);
  fd = vfs_open(zipname, VFS_O_RDONLY, 0);
  vfs_read(fd, files_data, files_size);
  vfs_close(fd);
  W_Repository_system = createRepository(W_Thread_system, (w_byte *)files_data, files_size);
  releaseMem(files_data);

  fixup_hashtable = ht_create(W_Realm_system, (char*)"hashtable:fixups", NATIVE_FUN_HT_SIZE, NULL , NULL, 0, 0);
  interface_hashtable = ht2k_create(W_Realm_system,(char*)"hashtable:interfaces",INTERFACE_HT_SIZE);

  system_loaded_class_hashtable = ht_create(W_Realm_system,(char*)"hashtable:system-loaded-classes", 97, NULL , NULL, 0, 0);
  woempa(7,"created system_loaded_class_hashtable at %p\n",system_loaded_class_hashtable);

  system_unloaded_class_hashtable = ht_create(W_Realm_system,(char*)"hashtable:system-unloaded-classes", 97, clazzHashCode, clazzComparator, 0, 0);
  woempa(7,"created system_unloaded_class_hashtable at %p\n",system_unloaded_class_hashtable);

  collectCoreFixups();

  /*
  ** Initialize base classes, we have to resolve things manually so keep
  ** this procedure intact.
  **  1) load class java/lang/Object.
  **  2) load java/lang/Cloneable, java/lang/Serializable, java/lang/Throwable. 
  **  3) create clazz_Array. clazz_Array acts like clazzObject but has
  **     a modified method table and a modified interfaces table. 
  **     It overrides the clone method of java/lang/Object and adds two 
  **     new interfaces: Cloneable and Serializable.
  **     (We need to do this before any subclasses of Throwable are loaded,
  **     in order to ensure that they are marked CLAZZ_IS_THROWABLE).
  **  4) load java/lang/Class, java/lang/ClassLoader, java/lang/Thread, 
  **     java/lang/ThreadGroup, and java/lang/ref/Reference.
  **  5) load all the classes mentioned in core-classes.in.
  **  6) For each primitive class xxx, create:
  **       -An instance Class_xxx of class java/lang/Class, linked to
  **        a w_Clazz structure (clazz_xxx).
  **        The name associated with the Class instance is "xxx.class".
  **       -Entries in the array atype2clazz[], which is indexed by P_xxx.
  **
  ** From that moment on, it should be possible to load any class without
  ** resolution problems and  to create instances of Object, Class, String
  ** Thread, or ThreadGroup without first checking that the class is prepared.
  */
  
  x_monitor_eternal(&system_loaded_class_hashtable->monitor);

  /****
  * 1 *
  ****/

  woempa(7,"Step 1: load java.lang.Object\n");
  clazzObject = loadBootstrapClass(string_java_lang_Object);

 /****
 * 2 *
 ****/

  woempa(7,"Step 2: load java.lang.Cloneable, java.io.Serializable, java.lang.Throwable.\n");
// load class java/lang/Cloneable
  clazzCloneable = seekClazzByName(string_java_lang_Cloneable, NULL);
  if (!clazzCloneable) {
    clazzCloneable = loadBootstrapClass(string_java_lang_Cloneable);
  }
// load class java/io/Serializable
  clazzSerializable = seekClazzByName(string_java_io_Serializable, NULL);
  if (!clazzSerializable) {
    clazzSerializable = loadBootstrapClass(string_java_io_Serializable);
  }
// load class java/lang/Throwable
  clazzThrowable = seekClazzByName(string_java_lang_Throwable, NULL);
  if (!clazzThrowable) {
    clazzThrowable = loadBootstrapClass(string_java_lang_Throwable);
  }
  clazzThrowable->flags |= CLAZZ_IS_THROWABLE;
 
  /****
  * 3 *
  ****/

  woempa(7,"Step 3: create array pseudo-class\n");
  clazz_Array = createClazzArray();
  mustBeReferenced(clazz_Array);

 /****
 * 4 *
 ****/

  woempa(7,"Step 4: load java.lang.Class, java.lang.ClassLoader, java.lang.Thread, java.lang.ThreadGroup, java.lang.ref.Reference\n");
// load class java/lang/Class
  clazzClass = seekClazzByName(string_java_lang_Class, NULL);
  if (!clazzClass) {
    clazzClass = loadBootstrapClass(string_java_lang_Class);
  }
// load class java/lang/ClassLoader
  clazzClassLoader = seekClazzByName(string_java_lang_ClassLoader, NULL);
  if (!clazzClassLoader) {
    clazzClassLoader = loadBootstrapClass(string_java_lang_ClassLoader);
  }
  clazzClassLoader->flags |= CLAZZ_IS_CLASSLOADER;

  identify_special_methods();

// load class java/lang/Thread
  clazzThread = seekClazzByName(string_java_lang_Thread, NULL);
  if (!clazzThread) {
    clazzThread = loadBootstrapClass(string_java_lang_Thread);
  }
  clazzThread->flags |= CLAZZ_IS_THREAD;
// load class java/lang/ThreadGroup
  clazzThreadGroup = seekClazzByName(string_java_lang_ThreadGroup, NULL);
  if (!clazzThreadGroup) {
    clazzThreadGroup = loadBootstrapClass(string_java_lang_ThreadGroup);
  }
  clazzThreadGroup->flags |= CLAZZ_IS_THREADGROUP;

// load class java/lang/ref/Reference
  clazzReference = seekClazzByName(string_java_lang_ref_Reference, NULL);
  if (!clazzReference) {
    clazzReference = loadBootstrapClass(string_java_lang_ref_Reference);
  }
  clazzReference->flags |= CLAZZ_IS_REFERENCE;

// load class java/lang/reflect/Constructor
  clazzConstructor = seekClazzByName(string_java_lang_reflect_Constructor, NULL);
  if (!clazzConstructor) {
    clazzConstructor = loadBootstrapClass(string_java_lang_reflect_Constructor);
  }

 /****
 * 5 *
 ****/
  woempa(7,"Step 5: loading core classes \n");
  loadCoreClasses();
  x_monitor_exit(&system_loaded_class_hashtable->monitor);


 /****
 * 6 *
 ****/
  woempa(7,"Step 6: create primitive classes and their array classes\n");

  preparePrimitive(clazz_boolean, string_boolean, P_boolean);
  preparePrimitive(clazz_char, string_char, P_char);
  preparePrimitive(clazz_float, string_float, P_float);
  preparePrimitive(clazz_double, string_double, P_double);
  preparePrimitive(clazz_byte, string_byte, P_byte);
  preparePrimitive(clazz_short, string_short, P_short);
  preparePrimitive(clazz_int, string_int, P_int);
  preparePrimitive(clazz_long, string_long, P_long);
  preparePrimitive(clazz_void, string_void, P_void);

  /*
  ** For some strange reason, SUN seems to fix the SUIDs for arrays of primitives. 
  ** So we set the cached SUID for the primitive array clazzes to the one given by 'serialver'.
  */
  
  atype2clazz[P_boolean]->suid = 6309297032502205922LL;
  atype2clazz[P_char]->suid = -5753798564021173076LL;
  atype2clazz[P_float]->suid = 836686056779680834LL;
  atype2clazz[P_double]->suid = 4514449696888150558LL;
  atype2clazz[P_byte]->suid = -5984413125824719648LL;
  atype2clazz[P_short]->suid = -1188055269542874886LL;
  atype2clazz[P_int]->suid = 5600894804908749477LL;
  atype2clazz[P_long]->suid = 8655923659555304851LL;

  /*
  ** Create some other frequently used descriptors and classes.
  ** Again, fix some SUID's of these classes...
  */
  
  clazzArrayOf_Object = getNextDimension(clazzObject);
  clazzArrayOf_Object->suid = -8012369246846506644LL;

  clazzArrayOf_Class = getNextDimension(clazzClass);
  clazzArrayOf_Class->suid = -6118465897992725863LL;

  clazzArrayOf_String = getNextDimension(clazzString);
  clazzArrayOf_String->suid = -5921575005990323385LL;

  /*
  ** Set up the primitive2wrapper array...
  */
  
  primitive2wrapper[P_boolean] = clazzBoolean;
  primitive2wrapperSlot[P_boolean] = F_Boolean_value;

  primitive2wrapper[P_char] = clazzCharacter;
  primitive2wrapperSlot[P_char] = F_Character_value;

  primitive2wrapper[P_float] = clazzFloat;
  primitive2wrapperSlot[P_float] = F_Float_value;

  primitive2wrapper[P_double] = clazzDouble;
  primitive2wrapperSlot[P_double] = F_Double_value;

  primitive2wrapper[P_byte] = clazzByte;
  primitive2wrapperSlot[P_byte] = F_Byte_value;

  primitive2wrapper[P_short] = clazzShort;
  primitive2wrapperSlot[P_short] = F_Short_value;

  primitive2wrapper[P_int] = clazzInteger;
  primitive2wrapperSlot[P_int] = F_Integer_value;

  primitive2wrapper[P_long] = clazzLong;
  primitive2wrapperSlot[P_long] = F_Long_value;


  woempa(7, "Forced class loading complete, loaded %d classes.\n",system_loaded_class_hashtable->occupancy);

}

/*
** Convert the given string from descriptor form (e.g. "Ljava/lang/Class;", "I",** "[Ljava/lang/String;", "[Z") to Class.getName() form ("java.lang.Class", "int",
** but "[Ljava.lang.String;", "[Z" <shrug>).  Returns a registered w_string.
*/

w_string undescriptifyClassName(w_string string) {

  w_string FQName = string;
  w_size   i;
  w_char *charbuf;


  woempa(1, "Starting with %w\n", string);
  if (string_char(string, 0) == 'V') {
    FQName = registerString(string_void);
  }
  else if (string_char(string, 0) == 'I') {
    FQName = registerString(string_int);
  }
  else if (string_char(string, 0) == 'J') {
    FQName = registerString(string_long);
  }
  else if (string_char(string, 0) == 'Z') {
    FQName = registerString(string_boolean);
  }
  else if (string_char(string, 0) == 'F') {
    FQName = registerString(string_float);
  }
  else if (string_char(string, 0) == 'D') {
    FQName = registerString(string_double);
  }
  else if (string_char(string, 0) == 'C') {
    FQName = registerString(string_char);
  }
  else if (string_char(string, 0) == 'B') {
    FQName = registerString(string_byte);
  }
  else if (string_char(string, 0) == 'L') {
    charbuf = allocMem_system(string_length(string) * sizeof(w_char));
    for (i = 1; i < string_length(string) - 1; ++i) {
      charbuf[i - 1] = (string_char(string, i) == '/') ? '.' : string_char(string, i);
    }
    FQName = unicode2String(currentWonkaThread, charbuf, string_length(string) - 2);
    releaseMem(charbuf);
  }
  else {
    FQName = slashes2dots(currentWonkaThread, FQName);
  }

  return FQName;
  
}

/*
** Search for the clazz called 'classname', and return its address or NULL.
** 'classname' is in the format returned by Class.getName().
*/

w_clazz seekClazzByName(w_string classname, w_instance loader) {
  w_hashtable hashtable = loader2loaded_classes(loader);

  woempa(1, "Seeking class '%w' in %s.\n", classname, hashtable->label);
  return (w_clazz)ht_read_no_lock(hashtable, (w_word)classname);
}

/*
** Create the array clazz with one more dimension than the given clazz.
*/
w_clazz createNextDimension(w_clazz base_clazz) {

  w_thread thread = currentWonkaThread;
  w_clazz array_clazz;
  w_char  *name_buffer;
  w_string temp_name;
  w_string desc_name;

  /*
  ** We make a clone of clazz_Array. We don't need to increment
  ** all refcounts that are being used in this clazz.
  */

  array_clazz = allocClazz(thread);
  temp_name = clazz2desc(base_clazz);
  name_buffer = allocMem_system((string_length(temp_name) + 1) * sizeof(w_char));

  if (array_clazz == NULL || temp_name == NULL || name_buffer == NULL) {
    woempa(9, "Could not allocate w_Clazz for [%k\n", base_clazz);
    if (array_clazz) {
      releaseMem(array_clazz);
    }

    if (temp_name) {
      deregisterString(temp_name);
    }

    if (name_buffer) {
      releaseMem(name_buffer);
    }

    return NULL;

  }

  w_memcpy(array_clazz, clazz_Array, sizeof(w_Clazz));
  woempa(1, "Creating next dimension of clazz %k at %p\n", base_clazz, array_clazz);

  array_clazz->dims = base_clazz->dims + 1;
  array_clazz->type = (array_clazz->dims > 1 ? VM_TYPE_MULTI : VM_TYPE_MONO) + VM_TYPE_REF + (base_clazz->type & 0x0f);
  array_clazz->loader = base_clazz->loader;
  array_clazz->bits = 32;

  name_buffer[0] = '[';
  w_string2chars(temp_name, name_buffer + 1);
  desc_name = unicode2String(thread, name_buffer, string_length(temp_name) + 1);
  deregisterString(temp_name);
  releaseMem(name_buffer);
  temp_name = slashes2dots(thread, desc_name);
  deregisterString(desc_name);
  array_clazz->dotified = temp_name;

  woempa(1, "Clazz %k dims is %d, type is 0x%02x, loader is %j (%w)\n", array_clazz, array_clazz->dims, array_clazz->type, array_clazz->loader, loader2name(array_clazz->loader));

  if (base_clazz->dims == 0 && isSet(base_clazz->flags, CLAZZ_IS_PRIMITIVE)) {
    wabort(ABORT_WONKA, "This function cannot be used to create an array of primtives!\n");
  }
  else {
    array_clazz->previousDimension = base_clazz;
    base_clazz->nextDimension = array_clazz;
  }

  setClazzState(array_clazz, CLAZZ_STATE_LINKED);
  registerClazz(array_clazz, base_clazz->loader);

  woempa(1, "Array clazz %k at %p created by %p (%w).\n", array_clazz,array_clazz,array_clazz->loader, loader2name(array_clazz->loader));
  setFlag(array_clazz->flags, ACC_ABSTRACT | ACC_PUBLIC | ACC_FINAL);//setup the clazz flags: Arrays are public final abstract

  return array_clazz;

}

/*
** Get the array clazz with one more dimension than the given clazz.
** If the array clazz does not already exist, create it.
** If base_clazz is NULL then we return NULL.
*/
w_clazz getNextDimension(w_clazz base_clazz) {

  w_instance loader;
  w_clazz array_clazz;

  if (!base_clazz) {

    return NULL;

  }

  loader = base_clazz->loader ? base_clazz->loader : systemClassLoader;

  array_clazz = base_clazz->nextDimension;
  if (!array_clazz) {
    woempa(1, "Clazz %k doesn't yet have a nextDimension, creating it\n", base_clazz);
    array_clazz = createNextDimension(base_clazz);
  }
  else {
    woempa(1, "Clazz %k already has nextDimension %k\n", base_clazz, array_clazz);
  }

  return array_clazz;

}


/*
** Create the 1-D array clazz corresponding to a given primitive type.
*/

w_clazz createPrimitiveArrayClazz(w_int pi) {

  w_thread thread = currentWonkaThread;
  w_clazz element_clazz = NULL;
  w_clazz result = NULL;
  w_string desc_string = NULL;

  woempa(1, "Creating array class for primitive #%d.\n", pi);

  switch(pi) {
    case P_boolean:
      element_clazz = clazz_boolean;
      desc_string = cstring2String(W_Thread_system,"[Z",2);
      break;

    case P_byte:
      element_clazz = clazz_byte;
      desc_string = cstring2String(W_Thread_system,"[B",2);
      break;

    case P_char:
      element_clazz = clazz_char;
      desc_string = cstring2String(W_Thread_system,"[C",2);
      break;

    case P_short:
      element_clazz = clazz_short;
      desc_string = cstring2String(W_Thread_system,"[S",2);
      break;

    case P_int:
      element_clazz = clazz_int;
      desc_string = cstring2String(W_Thread_system,"[I",2);
      break;

    case P_float:
      element_clazz = clazz_float;
      desc_string = cstring2String(W_Thread_system,"[F",2);
      break;

    case P_long:
      element_clazz = clazz_long;
      desc_string = cstring2String(W_Thread_system,"[J",2);
      break;

    case P_double:
      element_clazz = clazz_double;
      desc_string = cstring2String(W_Thread_system,"[D",2);
      break;

    default:
      wabort(ABORT_WONKA, "Eh???\n");
  }

  result = allocClazz(thread);
  if (result == NULL) {
    woempa(9,"Unable to create w_Clazz for %w\n", desc_string);
    return NULL;
  }
  w_memcpy(result, clazz_Array, sizeof(w_Clazz));

  result->loader = NULL;
  result->bits = 32;
  result->Class = NULL;
//  attachClassInstance(result);
  result->dotified = desc_string;

  result->previousDimension = primitive2clazz[pi];
  result->dims = 1;

  setClazzState(result, CLAZZ_STATE_LINKED);
  registerClazz(result, NULL);

  woempa(1, "Created array clazz %k, dimensions %d.\n", result, result->dims);

  if (!result->previousDimension) {
    result->previousDimension = element_clazz;
    result->dims = 1;
    element_clazz->nextDimension = result;
  }

  return result;

}

/*
** If the class with the given name (in "dotted" form) is not loaded,
** then if it is an array class then try to create it, else abort.
** Note that we don't lock the class loader; if another thread were to
** create the array class between our call to seekClazzByName(name,...)
** and namedClassMustBeLoaded(prevname,...) then we will still end up
** using the same array class, so just let it happen.
*/

w_clazz namedArrayClassMustBeLoaded(w_instance classLoader, w_string name) {

  w_thread    thread = currentWonkaThread;
  w_clazz  current;

  current = seekClazzByName(name, classLoader);

  if (current == NULL) {
    w_char * namebuff = allocMem_system(string_length(name) * sizeof(w_char));
    w_string prevname;

    w_string2chars(name, namebuff);
    switch (namebuff[1]) {
    case 'Z':
      prevname = registerString(string_boolean);
      break;

    case 'C':
      prevname = registerString(string_char);
      break;

    case 'F':
      prevname = registerString(string_float);
      break;

    case 'D':
      prevname = registerString(string_double);
      break;

    case 'B':
      prevname = registerString(string_byte);
      break;

    case 'S':
      prevname = registerString(string_short);
      break;

    case 'I':
      prevname = registerString(string_int);
      break;

    case 'J':
      prevname = registerString(string_long);
      break;

    case 'L':
      prevname = unicode2String(thread, namebuff + 2, string_length(name) - 3);
      break;

    default:
      prevname = unicode2String(thread, namebuff + 1, string_length(name) - 1);
    }
    releaseMem(namebuff);
    current = namedClassMustBeLoaded(classLoader, prevname);
    deregisterString(prevname);
    current = getNextDimension(current);
  }

  return current;

}

/*
** Helper function that returns WONKA_TRUE iff the name begins with
** "java." or "wonka.", optionally preceded by one or more "["s,
** and contains only ISO Latin 1 characters.
*/
static w_boolean namedClassIsSystemClass(w_string name) {
  w_ubyte *ch;

  if (!string_is_latin1(name)) {
    woempa(1, "'%w' is not Latin-1 => not a system class\n", name);

    return WONKA_FALSE;

  }

  ch = name->contents.bytes;
  while (*ch == '[') ++ch;

  woempa(1, "Length %d, prefix %c%c%c%c%c%c\n", string_length(name) - (ch - name->contents.bytes), ch[0], ch[1], ch[2], ch[3], ch[4], ch[5]);
  if (((int)string_length(name) > (ch - name->contents.bytes) + 5 && strncmp(ch, "java.", 5) == 0)
    ||
      ((int)string_length(name) > (ch - name->contents.bytes) + 6 && strncmp(ch, "wonka.", 6) == 0)
  ) {
    woempa(1, "'%w' is a system class\n", name);

    return WONKA_TRUE;

  }
  woempa(1, "'%w' is not a system class\n", name);

  return WONKA_FALSE;
}

w_clazz namedClassMustBeLoaded(w_instance classLoader, w_string name) {

  w_thread    thread = currentWonkaThread;
  w_instance  effectiveLoader = classLoader;
  w_clazz     effectiveLoader_clazz;
  w_clazz     current;

  woempa(1, "Class %w must be loaded.\n", name);

  if (string_char(name, 0) == (w_char)'[') {
    return namedArrayClassMustBeLoaded(classLoader, name);
  }

  if (!systemClassLoader) {
    woempa(1, "System class loader not defined, so use bootstrap class loader\n");
  }
  else if (!classLoader) {
    woempa(1, "System class loader exists (%w), so use it instead of bootstrap class loader to load %w\n", loader2name(systemClassLoader), name);
    effectiveLoader = systemClassLoader;
  }
  else if (!classLoader || namedClassIsSystemClass(name)) {
    woempa(1, "Class %w is a system class, so use system class loader (%w) instead of %j (%w)\n", name, loader2name(systemClassLoader), classLoader, loader2name(classLoader));
    effectiveLoader = systemClassLoader;
  }
  else {
    woempa(1, "Class %w is not a system class, so use %j (%w)\n", name, classLoader, loader2name(classLoader));
  }
  effectiveLoader_clazz = instance2clazz(effectiveLoader);

  current = seekClazzByName(name, effectiveLoader);

  if (current == NULL) {
    if (effectiveLoader == NULL) {
      current = loadBootstrapClass(name);
    }
    else {
      current = loadNonBootstrapClass(effectiveLoader, name);
    }
  
    if (current) {
      registerClazz(current, effectiveLoader);
    }
    else {
      woempa(9, "Did not find clazz %w\n", name);
    }
  }

  if (! current && ! exceptionThrown(thread)) {
    throwException(thread, clazzClassNotFoundException, "namedClassMustBeLoaded: %w", name);
  }

  return current;

}

w_clazz loadNonBootstrapClass(w_instance classLoader, w_string name) {

  w_thread thread = currentWonkaThread;
  w_instance theClass = NULL;
  w_clazz    classLoader_clazz = instance2clazz(classLoader);
  w_instance Name;
  x_frame    frame;
  w_clazz    clazz;
  w_method   method;
  w_instance exception;

  method = virtualLookup(loadClass_method, classLoader_clazz);
  woempa(1, "Trying to load class %w using %m of %j (%w)\n", name, method, classLoader, loader2name(classLoader));

  Name = getStringInstance(name);
// [CG 20021223] WAS :
//  frame = activateFrame(thread, method, 0, 3, classLoader, stack_trace, Name, stack_trace, false, stack_notrace);
  frame = activateFrame(thread, method, 0, 2, classLoader, stack_trace, Name, stack_trace);
  exception = exceptionThrown(thread);
  if (! exception) {
    theClass = (w_instance) frame->top[-1].c.w;
  }
  deactivateFrame(frame, theClass);

  if (exception) {
    if (instance2clazz(exceptionThrown(thread)) == clazzClassNotFoundException) {
      woempa(7, "Asked %j to load %w, and it threw %e. Clearing the exception and returning NULL\n", classLoader,name, exceptionThrown(thread));
      clearException(thread);
      exception = NULL;
    }
    else {
      woempa(9, "Oh. I asked %j to load %w, and it threw %e!\n", classLoader,name, exceptionThrown(thread));
    }

    return NULL;
  }

  if (theClass == NULL) {
    woempa(9, "Ah. I asked %j to load %w, and it returned NULL!\n", classLoader,name);
    throwException(thread, clazzClassNotFoundException, "loadNonBootstrapClass: %w", name);
    return NULL;
  }

  clazz = Class2clazz(theClass);

  if (clazz->dotified != name) {
    woempa(9, "Scandal! I asked %j to load %w, and the bounder loaded %k!\n", classLoader, name, clazz);
    throwException(thread, clazzLinkageError, "%w", name);
    return NULL;
  }

  registerClazz(clazz, classLoader);

  woempa(1, "Loaded %k at %p, defining class loader is %j (%w)\n", clazz, clazz, clazz->loader, loader2name(clazz->loader));

  return clazz;

}

w_clazz loadBootstrapClass(w_string name) {

  w_char  *buffer;
  w_size   length;
  w_string filename;
  w_clazz  clazz = NULL;
  w_rentry entry;
  w_BAR    bar;
  w_size   i;
  w_char   ch;

  length = string_length(name);
  buffer = allocMem_system((length + 6) * sizeof(w_char));
  for (i = 0; i < string_length(name); ++i) {
    ch = string_char(name, i);
    buffer[i] = ch == '.' ? '/' : ch;
  }
  buffer[length++] = '.';
  buffer[length++] = 'c';
  buffer[length++] = 'l';
  buffer[length++] = 'a';
  buffer[length++] = 's';
  buffer[length++] = 's';
  filename = unicode2String(W_Thread_system, buffer, length);
  releaseMem(buffer);
  woempa(1, "Need a file called '%w'\n", filename);
  entry = (w_rentry)ht_read_no_lock(W_Repository_system->hashtable, (w_word)filename);
  woempa(1, "Entry = %p\n", entry);

  if (entry) {
    bar.buffer = entry->u_data;
    bar.length = entry->u_size;
    bar.current = 0;

    // We don't pass the name for checking, because we trust the repository ...
    clazz = createClazz(currentWonkaThread, NULL, &bar, NULL);

    woempa(1,"Deleting class %w (%d bytes) from the repository\n",name,entry->u_size);
    deleteRepositoryEntry(W_Repository_system, filename);
  }
  deregisterString(filename);

  if (! clazz && ! clazzClassNotFoundException) {
    wabort(ABORT_WONKA, "Swoggle my horn! Attempt to load system class %w failed, and clazzNoClassDefFoundError is not yet defined ...\n",name);
  }

  return clazz;

}

void alreadyBroken(w_clazz clazz) {

  w_thread thread = currentWonkaThread;
  woempa(9, "Class %k is FUBAR\n", clazz);
  throwException(thread, clazzNoClassDefFoundError, "'%w'", clazz->dotified);

}

