/**************************************************************************
* Copyright (c) 2001, 2002, 2003 by Acunia N.V. All rights reserved.      *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Philips site 5, box 3       info@acunia.com                           *
*   3001 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: stack.c 4155 2004-05-06 13:41:11Z dumon $
*/

#undef debug

#include <string.h>

#include "clazz.h"
#include "core-classes.h"
#include "descriptor.h"
#include "dispatcher.h"
#include "exception.h"
#include "hashtable.h"
#include "realms.h"
#include "wordset.h"
#include "ts-mem.h"
#include "methods.h"
#include "wstrings.h"
#include "locks.h"
#include "heap.h"
#include "threads.h"

#ifdef JSPOT
  #include "hotspot.h"
#endif

#include <assert.h>

x_frame getCurrentFrame(w_thread thread) {

  x_frame frame = thread->top;
  
  if (frame && frame->method == NULL) {
    frame = frame->previous;
  }
    
  if (frame) {
    frame = frame->previous;
  }

  if (frame && frame->method == NULL) {
    frame = frame->previous;
  }
 
  return frame;
}

x_frame getCallingFrame(w_thread thread) {

  x_frame frame = thread->top;
  
  if (frame && frame->method == NULL) {
    frame = frame->previous;
  }
    
  if (frame) {
    frame = frame->previous;
  }

  if (frame && frame->method == NULL) {
    frame = frame->previous;
  }
 
  if (frame) {
    frame = frame->previous;
  }

  if (frame && frame->method == NULL) {
    frame = frame->previous;
  }
 
  return frame;
}

// get current Java method
w_method getCurrentMethod(w_thread thread) {

  x_frame frame = getCurrentFrame(thread);

  if (frame) {
    return frame->method;
  }
  
  return NULL;

}

w_method getCallingMethod(w_thread thread) {

  x_frame frame = getCallingFrame(thread);
  
  if (frame) {
    return frame->method;
  }
  
  return NULL;

}


w_clazz getCurrentClazz(w_thread thread) {

  w_method current = getCurrentMethod(thread);

  if (current) {
    woempa(1, "Current clazz is %k.\n", current->spec.declaring_clazz);
    return current->spec.declaring_clazz;
  }
  else {
    return NULL;
  }

}


w_clazz getCallingClazz(w_thread thread) {

  w_method calling = getCallingMethod(thread);

  if (calling) {
    woempa(1, "Calling clazz is %k.\n", calling->spec.declaring_clazz);
    return calling->spec.declaring_clazz;
  }

  return NULL;

}

w_instance getCurrentInstance(w_thread thread) {

  x_frame frame = getCurrentFrame(thread);

  if (frame && isNotSet(frame->method->flags, ACC_STATIC)) {
    return (w_instance) frame->Slots[0].c.w;
  }

  return NULL;

}

w_instance getCallingInstance(w_thread thread) {

  x_frame frame = getCallingFrame(thread);
  
  if (frame && isNotSet(frame->method->flags, ACC_STATIC)) {
    return (w_instance) frame->Slots[0].c.w;
  }
  
  return NULL;

}

inline static void dumpLocals(x_frame frame) {

  x_slot auxs = frame->auxs - 1;

  woempa(9, "Used %d slots!\n", frame->auxs - frame->thread->auxs);
  while (auxs > (x_slot) frame->thread->auxs) {
    if (auxs->s == stack_trace) {
      woempa(9, "--> %k  auxs = %p\n", instance2object((w_instance) auxs[0].c.w)->clazz, auxs);
    }
    else if (isMonitoredSlot(auxs)) {
      woempa(9, "--> %k  locked instance, auxs = %p\n", instance2object((w_instance) auxs[0].c.w)->clazz, auxs);
    }
    else {
      woempa(9, "--> tag = %d, auxs = %p\n", auxs[0].s, auxs);
    }
    auxs -= 1;
  }

}

static int resize_auxs(w_thread thread, int locals) {
 
  x_frame frame; 
  x_slot maxauxs;
  x_slot slots;
  x_slot auxs;
  int size;
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterUnsafeRegion(thread);
#else
  x_preemption_disable;
#endif
  
  /*
  ** This function grows or shrinks the local reference space.  To
  ** shrink, provide a negative number of "locals" to shrink with.
  */
 
  maxauxs = thread->maxauxs;
  auxs = thread->auxs;
  size = maxauxs - auxs + locals;
  
  woempa(9, "growing/shrinking thread %p's auxs by %d slots to size %d ...\n", thread, locals, size);
  
  slots = allocClearedMem_system(size * sizeof(x_Slot));
  if (slots == NULL) {
#ifdef GC_SAFE_POINTS
    if (!unsafe) {
      enterSafeRegion(thread);
    }
#else
    x_preemption_enable;
#endif
    return 0;
  }
  
  if (slots != auxs) {
    /*
    ** Copy the existing locals and clear the newly allocated part.
    */
    
    w_memcpy(slots, auxs, (maxauxs - auxs) * sizeof(x_Slot));

    /*
    ** Patch each frame in the call stack:
    */

    for (frame = thread->top; frame; frame = frame->previous) {
      frame->auxs = slots + (frame->auxs - auxs);
    }

    /*
    ** Release the old auxs structure and setup the new aux structure.
    */

    if (thread != W_ThreadGroup_system->manager) {
      // NOTE: W_ThreadGroup_system->manager's aux have been allocated using x_alloc_static_mem().
      releaseMem(thread->auxs);
    }

    thread->maxauxs = slots + size;
    thread->auxs = slots;
  }

#ifdef GC_SAFE_POINTS
  if (!unsafe) {
    enterSafeRegion(thread);
  }
#else
  x_preemption_enable;
#endif
  return 1;
}

inline static void i_pushLocalReference(x_frame frame, w_instance instance) {
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterUnsafeRegion(frame->thread);
#endif

  // Grow the locals when we ran out of space:
  if (frame->thread->maxauxs - frame->auxs == 0) {
    if (!resize_auxs(frame->thread, 1024)) {
      return;
    }
  }
  
  if (frame->auxs < frame->thread->auxs) {
    throwException(frame->thread, clazzInternalError, "Local reference stack underflow detected in frame (%M) of %t when pushing local ref : %p < %p.", frame->method, frame->thread, frame->auxs, frame->thread->auxs);
  }
  else {
    frame->auxs[0].c.w = (w_word) instance;
    frame->auxs[0].s = stack_trace;
    frame->auxs += 1;
  }

#ifdef GC_SAFE_POINTS
  if (!unsafe) {
    enterSafeRegion(frame->thread);
  }
#endif
}

void pushMonitoredReference(x_frame frame, w_instance instance, x_monitor monitor) {
#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterUnsafeRegion(frame->thread);
#endif

  // Grow the locals when we ran out of space:
  if (frame->thread->maxauxs - frame->auxs == 0) {
    if (!resize_auxs(frame->thread, 1024)) {
      return;
    }
  }

  if (frame->auxs < frame->thread->auxs) {
    throwException(frame->thread, clazzInternalError, "Local reference stack underflow detected in frame (%M) of %t when pushing monitored ref %j", frame->method, frame->thread, instance);
  }
  else {
    frame->auxs[0].c.w = (w_word) instance;
    frame->auxs[0].s = (w_word) monitor;
    frame->auxs += 1;
  }

#ifdef GC_SAFE_POINTS
  if (!unsafe) {
    enterSafeRegion(frame->thread);
  }
#endif
}

void pushLocalReference(x_frame frame, w_instance instance) {
  i_pushLocalReference(frame, instance);
}

void addLocalReference(w_thread thread, w_instance instance) {
  i_pushLocalReference(thread->top, instance);
}

void removeLocalReference(w_thread thread, w_instance instance) {

  x_frame frame = thread->top;
  x_slot slot;
  x_slot stop;

#ifdef GC_SAFE_POINTS
  w_boolean unsafe = enterUnsafeRegion(thread);
#endif

  if (frame->previous) {
    stop = frame->previous->auxs;
  }
  else {
    stop = thread->auxs;
  }

  for (slot = (x_slot)frame->auxs - 1; slot >= stop; slot -= 1) {
    if (slot->c.w == (w_word) instance && slot->s == stack_trace) {
      woempa(1, "  - is aux[%d]\n", slot - frame->thread->auxs);
      slot->s = stack_notrace;
      break;
    }
  }

  while (frame->auxs > stop && frame->auxs[-1].s == stack_notrace) {
    frame->auxs -= 1;
    woempa(1, "  - skipped a zombie, now have %d auxs\n", frame->auxs - frame->thread->auxs);
  }

/* [CG 20040504] WAS :
  if (frame->auxs[-1].c.w == (w_word) instance && frame->auxs[-1].s == stack_trace) {
    frame->auxs -= 1;
    frame->auxs[0].s = stack_notrace;
    while (frame->auxs[-1].s == stack_notrace && frame->auxs > stop) {
      frame->auxs -= 1;
      frame->auxs[0].s = stack_notrace;
    }
  }
  else {
    for (slot = frame->auxs - 1; slot >= stop; slot -= 1) {
      if (slot->c.w == (w_word) instance && slot->s == stack_trace) {
        slot->s = stack_notrace;
        break;
      }
    }
  }
*/

#ifdef GC_SAFE_POINTS
  if (!unsafe) {
    enterSafeRegion(thread);
  }
#endif
}

void stacktrace(void) {

  w_thread thread = currentWonkaThread;
  x_frame frame;
  w_int i = 0;

  for (frame = thread->top; frame; frame = frame->previous) {
    if (frame->method) {
      woempa(9, "%2d -> %M\n", i, frame->method);
    }
    else {
      woempa(9, "%2d -> arguments frame\n", i);
    }
    i += 1;
  }  
  
}
