/**************************************************************************
* Copyright  (c) 2001 by Acunia N.V. All rights reserved.                 *
*                                                                         *
* This software is copyrighted by and is the sole property of Acunia N.V. *
* and its licensors, if any. All rights, title, ownership, or other       *
* interests in the software remain the property of Acunia N.V. and its    *
* licensors, if any.                                                      *
*                                                                         *
* This software may only be used in accordance with the corresponding     *
* license agreement. Any unauthorized use, duplication, transmission,     *
*  distribution or disclosure of this software is expressly forbidden.    *
*                                                                         *
* This Copyright notice may not be removed or modified without prior      *
* written consent of Acunia N.V.                                          *
*                                                                         *
* Acunia N.V. reserves the right to modify this software without notice.  *
*                                                                         *
*   Acunia N.V.                                                           *
*   Vanden Tymplestraat 35      info@acunia.com                           *
*   3000 Leuven                 http://www.acunia.com                     *
*   Belgium - EUROPE                                                      *
**************************************************************************/

/*
** $Id: zip.c 2809 2002-09-19 14:19:27Z gray $
*/

#include <string.h>

#include "zip.h"
#include "ts-mem.h"
#include "realms.h"
#include "list.h"
#include "repository.h"
#include "wstrings.h"
#include "deflate_driver.h"

/*
** This table maps the z_zipEntry->compression with the appriopriate decoder or NULL
** if this decoder is not yet implemented. We first give the prototypes of the
** functions in the table. The entry 'notyet' signals that the appropriate decompressor
** routine is not yet implemented.
*/

static w_int notyet(z_zipEntry entry);
static w_int zip_inflate(z_zipEntry entry);
static w_int zip_destore(z_zipEntry entry);

static const z_decoder decoders[] = {
  zip_destore,        /*  0 - the file is stored without compression        */
  notyet,             /*  1 - the file is shrunk                            */
  notyet,             /*  2 - the file is reduced with compression factor 1 */
  notyet,             /*  3 - the file is reduced with compression factor 2 */
  notyet,             /*  4 - the file is reduced with compression factor 3 */
  notyet,             /*  5 - the file is reduced with compression factor 4 */
  notyet,             /*  6 - the file is imploded                          */
  notyet,             /*  7 - reserved for tokenizing compression           */
  zip_inflate,        /*  8 - the file is inflated                          */
  notyet,             /*  9 - reserved for enhanced deflating               */
  notyet,             /* 10 - PKWARE Date Compression Library imploding     */
};

/*
** Read an item from a stream of bytes. After reading, set the byte data pointer to the
** next position to parse.
*/

#define readWord(d)                     bytes2word(d); (d) += sizeof(w_word)
#define readShort(d)                    bytes2short(d); (d) += sizeof(w_short)
#define readByte(d)                     (*(d)); (d) += sizeof(w_byte)

#ifdef DEBUG
void dumpEntry(z_zipEntry entry) {

  w_int offset = (w_int)(entry->c_data - entry->zipFile->stream);

  woempa(9, "            Version to extract: %d.%d\n", entry->x_major, entry->x_minor);
  woempa(9, "               Version made by: %d.%d\n", entry->c_major, entry->c_minor);
  woempa(9, "                         Flags: 0x%08x\n", entry->flags);
  woempa(9, "              Compression used: %d\n", entry->compression);
  woempa(9, "               Compressed size: %d\n", entry->c_size);
  woempa(9, "             Uncompressed size: %d\n", entry->u_size);
  woempa(9, "                    Namelength: %d\n", entry->nameLength);
  woempa(9, "                     File name: %s\n", (char *)entry->name);
  woempa(9, "            Extra field length: %d\n", entry->extraLength);
  woempa(9, "                Comment length: %d\n", entry->commentLength);
  if (entry->commentLength) {
    woempa(9, "                       Comment: '%s'\n", (char *)entry->comment);
  }
  woempa(9, "             Disk Number start: %d\n", entry->s_disk);
  woempa(9, "           Internal attributes: 0x%08x\n", entry->i_attr);
  woempa(9, "           External attributes: 0x%08x\n", entry->e_attr);
  woempa(9, "               Relative offset: %d\n", entry->offset);
  woempa(9, "                           CRC: 0x%08x\n", entry->d_crc);
  woempa(9, "     Compressed data at offset: %d\n", offset);

}


void dumpDir(z_zipFile dir) {

  z_zipEntry entry;
  w_int i = 1;
  
  woempa(9, "============================================= ZIP DIR 0x%08x ======================\n", (w_word)dir);
  woempa(9, "           Current disk number: %d\n", dir->c_disk);
  woempa(9, "         Disk with central dir: %d\n", dir->d_disk);
  woempa(9, "Number of entries on this disk: %d\n", dir->numDisk);
  woempa(9, " Number of entries in this dir: %d\n", dir->numDir);
  woempa(9, "              Central dir size: %d\n", dir->size);
  woempa(9, "            Central dir offset: %d\n", dir->offset);
  woempa(9, "                Comment length: %d\n", dir->commentLength);
  if (dir->commentLength) {
    woempa(1, "                       Comment: '%s'\n", (char *)dir->comment);
  }

  entry = dir->entries;
  do {
    woempa(9, "--------------------------------------------- Entry number %3d ------------------------\n", i);
    dumpEntry(entry);
    entry = entry->next;
    i += 1;
  } while (entry != dir->entries);

}
#endif


w_device _inflate_device;

w_void zipInit() {
  if((_inflate_device = deviceBSOpen("unzip_", wdp_none)) == 0) {
    woempa(10, "Unable to open unzip device !!\n");
    exit(0);
  }
}

w_void zipEnd() {
  deviceBSClose(_inflate_device);
}

w_int zipEntry2RepositoryType(z_zipEntry entry) {

  if (entry->name[strlen(entry->name) - 1] == '/') {
    return RT_DIRECTORY;
  }
  else if (strcmp(entry->name + strlen(entry->name) - 4, ".zip") == 0) {
    return RT_ZIP;
  }
  else if (strcmp(entry->name + strlen(entry->name) - 4, ".jar") == 0) {
    return RT_JAR;
  }
  else if (strcmp(entry->name + strlen(entry->name) - 3, ".MF") == 0) {
    return RT_MANIFEST;
  }
  else if (strcmp(entry->name + strlen(entry->name) - 3, ".SF") == 0) {
    return RT_SIGNATURE;
  }
  else {
    return RT_RAW;
  }

}

void readZipEntry(w_boolean local, z_zipEntry entry, w_byte **stream) {

  w_word timestamp;
  w_byte *data = *stream;
  w_word signature;
  w_byte *o_data;
  w_int nameLength;
  w_int extraLength;

  if (! local) {
    entry->c_major = readByte(data);
    entry->c_minor = readByte(data);
  }
  entry->x_major = readByte(data);
  entry->x_minor = readByte(data);
  entry->flags = readShort(data);
  entry->compression = readShort(data);
  timestamp = readWord(data);

  if (! local) {
    entry->d_crc = readWord(data);
    entry->c_size = (w_int)readWord(data);
    entry->u_size = (w_int)readWord(data);
  }
  else {
    /* 
    ** Throw the information away, it could be 0 anyway...
    */
    signature = readWord(data);
    signature = readWord(data);
    signature = readWord(data);
  }

  if (! local) {
    entry->nameLength = readShort(data);
    entry->extraLength = readShort(data);
    entry->commentLength = (w_short)readShort(data);
    entry->s_disk = (w_short)readShort(data);
    entry->i_attr = (w_flags)readShort(data);
    entry->e_attr = readWord(data);
    entry->offset = readWord(data);
    if (entry->nameLength) {
      entry->name = (w_byte *)allocMem(W_Thread_system, (entry->nameLength + 1) * sizeof(w_byte));
      w_memcpy(entry->name, data, (w_size)entry->nameLength);
      entry->name[entry->nameLength] = 0x00;
      data += entry->nameLength;
    }
    if (entry->extraLength) {
      entry->extra = (w_byte *)allocMem(W_Thread_system, (entry->extraLength + 1) * sizeof(w_byte));
      w_memcpy(entry->extra, data, (w_size)entry->extraLength);
      entry->extra[entry->extraLength] = 0x00;
      data += entry->extraLength;
    }
    if (entry->commentLength) {
      entry->comment = allocMem(W_Thread_system, (entry->commentLength + 1) * sizeof(w_byte));
      w_memcpy(entry->comment, data, (w_size)entry->commentLength);
      entry->comment[entry->commentLength] = 0x00;
      data += entry->commentLength;
    }

    /*
    ** For some strange reason, the extra length in the local header doesn't always match
    ** the extra length of the directory entry. So we jump to the local header with 'offset'
    ** and extra the correct lengths ourselfs.
    */

    o_data = entry->zipFile->stream + entry->offset + 26;
    nameLength = (w_int)readShort(o_data);
    o_data = entry->zipFile->stream + entry->offset + 28;
    extraLength = (w_int)readShort(o_data);
    entry->c_data = entry->zipFile->stream + entry->offset + 30 + nameLength + extraLength;
  }

  *stream = data;
  
}

z_zipFile parseZipFile(w_thread thread, w_byte *adata) {

  w_word signature;
  z_zipEntry entry;
  z_zipEntry match;
  z_zipFile zipFile;
  volatile w_ubyte * data = adata; // Fix optimization problem on GCC 3.0.1

  zipFile = allocMem(thread, sizeof(z_ZipFile));

  if (! zipFile) {
    return NULL;
  }

  zipFile->stream = adata;
  
  /*
  ** Read partial information of the first entry.
  */

  signature = bytes2word(data);
  if (signature != Z_LOCAL_HEADER) {
    releaseMem(zipFile);
    return NULL;
  }

  entry = allocClearedMem(thread, sizeof(z_ZipEntry));
  entry->zipFile = zipFile;
  data += 4; /* Skip over the signature */
  readZipEntry(WONKA_TRUE, entry, (w_ubyte **)&data);

  /*
  ** Now skip over the file data until we find a directory header that matches
  ** in name and offset with this first entry.
  */

  match = allocClearedMem(thread, sizeof(z_ZipEntry));
  match->zipFile = zipFile;
  while (1) {
    do {
      signature = bytes2word(data);
      data += 1;
    } while (signature != Z_DIR_HEADER);
    data += 3;
    readZipEntry(WONKA_FALSE, match, (w_ubyte **)&data);
    
    if (match->offset == 0) {
      woempa(9, "Found matching first entry in central directory %d...\n", offsetof(z_ZipEntry, c_data));
      woempa(9, "Match address %p, zipAddress %p (%d)\n", match->c_data, zipFile->stream, (int)(match->c_data - zipFile->stream));
      break;
    }
  }

  list_init(match);

  zipFile->entries = match;
  // MEMORY LEAK, RELEASE entry	!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!1

  while ((signature = bytes2word(data)) == Z_DIR_HEADER) {
    data += sizeof(w_word);
    entry = allocClearedMem(thread, sizeof(z_ZipEntry));
    entry->zipFile = zipFile;
    readZipEntry(WONKA_FALSE, entry, (w_ubyte **)&data);
    list_insert(match, entry);
  }
  
  /*
  ** ... now read the concluding file information.
  */

  woempa(9, "Reading central directory of zipfile 0x%08x\n", (w_word)zipFile);

  data += sizeof(w_word);
  zipFile->c_disk = readShort(data);
  zipFile->d_disk = readShort(data);
  zipFile->numDisk = readShort(data);
  zipFile->numDir = readShort(data);
  zipFile->size = (w_int)readWord(data);
  zipFile->offset = (w_int)readWord(data);
  zipFile->commentLength = readShort(data);
  zipFile->comment = allocMem(thread, (zipFile->commentLength + 1) * sizeof(w_byte));
  w_memcpy(zipFile->comment, (w_ubyte *)data, (w_size)zipFile->commentLength);
  data += zipFile->commentLength;

#ifdef DEBUG
//  dumpDir(zipFile);
#endif

  return zipFile;

}

void releaseEntry(z_zipEntry entry) {

  woempa(1, "Releasing entry '%s'\n", entry->name);
  
  if (entry->u_data) {
    releaseMem(entry->u_data);
  }
  if (entry->name) {
    releaseMem(entry->name);
  }
  if (entry->comment) {
    releaseMem(entry->comment);
  }
  if (entry->extra) {
    releaseMem(entry->extra);
  }
  
  /*
  ** Now release the entry itself...
  */
  
  releaseMem(entry);
  
}

void releaseZipFile(z_zipFile zf) {

  z_zipEntry entry;
  z_zipEntry victim;

  /*
  ** First clean up the stuff that has been allocated in the entries,
  ** then the entries themselves.
  */
  
  entry = zf->entries->next;
  while (entry != zf->entries) {
    victim = entry;
    entry = entry->next;
    list_remove(victim);
    releaseEntry(victim);
  }

  /*
  ** Now release the first entry itself...
  */
  
  releaseEntry(zf->entries);

  /*
  ** Release the file itself...
  */
  
  releaseMem(zf);
    
}

z_zipEntry findZipEntry(z_zipFile dir, w_byte *name) {

  z_zipEntry entry;
  w_boolean found = WONKA_FALSE;
  
  entry = dir->entries;
  if (entry) {
    do {
      if (strcmp((char *)entry->name, (char *)name) == 0) {
        found = WONKA_TRUE;
        break;
      }
      entry = entry->next;
    } while (entry != dir->entries);
  }

  if (! found) {
    return NULL;
  }
  else {
    return entry;
  }
  
}

z_zipEntry uncompressZipEntry(z_zipEntry entry) {

  z_decoder decoder;
  
  /*
  ** .. see if it has been decoded yet, and if yes, return it when it's not
  ** containing an error ...
  */

  if (isSet(entry->flags, ZE_UNCOMPRESSED) && isNotSet(entry->flags, ZE_ERROR)) {
    setFlag(entry->flags, ZE_ZERO_LENGTH);
    return entry;
  }

  if (isSet(entry->flags, ZE_ERROR) || entry->u_size == 0) {
    return NULL;
  }  

  if (entry->compression >= 0 && entry->compression <= 10) {
    decoder = decoders[entry->compression];
    if( (*decoder)(entry) ) {
      return NULL;
    }
    if (isSet(entry->flags, ZE_UNCOMPRESSED) && isNotSet(entry->flags, ZE_ERROR)) {
      return entry;
    }
  }
  else {
    woempa(10, "Unknow compression code %d in zip file.\n", entry->compression);
  }

  /*
  ** When we get here, something is wrong...
  */

  return NULL;
  
}

/*
** Unpack a file that has been "compressed" or better, "stored" with method 0.
*/

static w_int zip_destore(z_zipEntry entry) {

  entry->u_data = allocMem(W_Thread_system, entry->u_size);			// who frees this
  w_memcpy(entry->u_data, entry->c_data, entry->u_size);

  if (entry->d_crc == CCITT_32(entry->u_data, (w_size)entry->u_size)) {
    setFlag(entry->flags, ZE_UNCOMPRESSED);
  }

  return 0;  
}

/*
** Uncompress a zip file entry that has compression method 8 (Inflate).
*/
static w_int zip_inflate(z_zipEntry entry) {
  w_int lread, offs;
  w_driver_status s;

  // check if OK !!!
  deviceBSWrite(_inflate_device, entry->c_data, (signed)entry->c_size, &lread, x_millis2ticks(5000));

  entry->u_data = allocMem(W_Thread_system, entry->u_size);			// who frees this

  s = wds_success;
  offs = 0;
  while (s == wds_success && (signed)entry->u_size - offs > 0) {
    s = deviceBSRead(_inflate_device, entry->u_data + offs, (signed)(entry->u_size - offs), &lread, x_millis2ticks(5000));
    offs += lread;
  }

  if (s == wds_internal_error) {
    woempa(9, "  -UNZIP ERROR !!\n");
    setFlag(entry->flags, ZE_ERROR);
    return 1;
  }

  if (s == wds_data_exhausted) {
    woempa(7, "  -AIAI, no full buffer received, we did not calculate the decompressed size correctly, or device errors\n");
  } 
  else {     
    s = deviceBSRead(_inflate_device, entry->u_data, 1, &lread, x_millis2ticks(500));
    if (s != wds_data_exhausted) {
      woempa(7, "  -AIAI, there is still data left on the device, we did not calculate the decompressed size correctly\n");
      // stream is corrupt here, we could keep reading until we get a wds_data_exhausted
      // then the stream will be in sync, but this file is probably corrupt !
      exit(0);
    }
  }

  if (entry->d_crc == CCITT_32(entry->u_data, (w_size)entry->u_size)) {
    setFlag(entry->flags, ZE_UNCOMPRESSED);
  }
  // if this fails, who is going to free the buffer ??

  return 0;  
}

/*
** The function to handle not yet implemented decompressors.
*/

static w_int notyet(z_zipEntry entry) {
  setFlag(entry->flags, ZE_ERROR);
  setFlag(entry->flags, ZE_NO_DECOMPRESSOR);
  woempa(1, "No decompressor yet for compression type %d.\n", entry->compression);
  
  return 1;  
}


